package ch.ige.adr.client;

import java.awt.Dialog;
import java.awt.Frame;
import java.util.Date;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AdrConstants;
import ch.ige.adr.server.AdresseDelegate;
import ch.ige.adrhelp.HelpUtil;
import ch.ige.auth.IGESubject;
import ch.ige.authz.client.AuthMain;
import ch.ige.main.client.WizardMediator;
import ch.ige.main.client.XxMessage;
import ch.ige.util.vo.VersadrVO;

/**
 * Wizard für das Erfassen und Mutieren der Versandadresse
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseWizard.java,v 1.38 2008/04/28 15:32:34 galli Exp $
 */
public class AdresseWizard extends WizardMediator
{
  // Logging
  private static final Logger LOG = Logger.getLogger( AdresseWizard.class );

  // Applikationsmeldungen
  protected XxMessage xxMessage = null;

  protected boolean finish = false;

  protected boolean edit = false; // true - editAdresse(), false - newAdresse() aufgerufen

  protected int adrNr = 0;

  protected PnlAdresse pnlAdresse = new PnlAdresse();

  protected PnlAdresseSap pnlAdresseSap = new PnlAdresseSap();

  protected String oldLandCd = ""; // alter Landcode, vor dem Mutieren

  /**
   * Konstruktor
   * @param parent aufrufendes Frame
   */
  public AdresseWizard( Frame parent )
  {
    super( parent );
    AdresseWizardInit();
  }

  /**
   * Konstruktor
   * @param parent aufrufender Dialog
   */
  public AdresseWizard( Dialog parent )
  {
    super( parent );
    AdresseWizardInit();
  }

  /**
   * Initialisierung
   */
  protected void AdresseWizardInit()
  {
    try
    {
      xxMessage = XxMessage.getInstance();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      JOptionPane.showMessageDialog( dlgWizard, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    alPanel.add( new WizardMediator.WizardPanel( pnlAdresse, true ) );
    alPanel.add( new WizardMediator.WizardPanel( pnlAdresseSap, true ) );
    maxPanel = alPanel.size();
    title = "Versandadresse erfassen";
  }

  /**
   * Versandadresse erfassen
   * @return Versandadressnummer
   */
  public Integer newVersadr()
  {
    edit = false;
    init();
    return finish ? new Integer( adrNr ) : null;
  }

  /**
   * Versandadresse mutieren
   * @param adrNr Versandadressnummer
   * @return true = fertiggestellt, false = abgebrochen
   */
  public boolean editVersadr( int adrNr )
  {
    edit = true;
    try
    {
      VersadrVO versadrVO = AdresseDelegate.getInstance().getVersadr( adrNr );
      if ( versadrVO.getCustGroupOrEmptyStr().equals( "01" ) )
      {
        // Adresse eines Deponentenkontos darf nicht von jedermann mutiert werden.
        Object[] options = { "Weiter", "Abbrechen" };
        int selectedOption = JOptionPane.showOptionDialog( null, "<html><b><font size=2 face=\"Dialog\">"
            + "Diese Adresse wird von einem Deponentenkonto referenziert.<br>" + "Nehmen Sie bitte Rücksprache mit der Buchhaltung, <br>"
            + "bevor Sie diese Adresse mutieren!</font></b></html>", "Warnung", JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE, null, options,
            options[1] );
        if ( selectedOption != 0 )
        {
          return false;
        }
      }

      // Adresse mutieren
      oldLandCd = versadrVO.getLandCd();
      pnlAdresse.setVersadr( versadrVO );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      String message = xxMessage.getMessage( ex );
      JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    init();
    return finish;
  }

  /**
   * Versandadresse duplizieren<br>
   * alle Daten ausser AdrNr, Änderungsdatum werden kopiert<br>
   * Bei einer Adresse mit Deponentenkonto wird diese Information nicht dupliziert
   * @param adrNr Versandadressnummer
   * @return Versandadressnummer
   */
  public Integer duplicateVersadr( int adrNr )
  {
    edit = false; // neue Adresse erstellen
    try
    {
      VersadrVO versadrVO = AdresseDelegate.getInstance().getVersadr( adrNr );
      versadrVO.setAdrNr( 0 );
      versadrVO.setVersion( 0 );
      versadrVO.setGeaendBen( null );
      versadrVO.setGeaendDat( null );
      // ohne Deponentenkonto
      versadrVO.setCustGroup( "02" );
      pnlAdresse.setVersadr( versadrVO );
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      String message = xxMessage.getMessage( ex );
      JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
    }
    init();
    return finish ? new Integer( this.adrNr ) : null;
  }

  /**
   * Vorwärts
   * @return true = Vorwärts erfolgreich, false = nicht erfolgreich
   */
  public boolean forward()
  {
    if ( !pnlAdresse.verifyInput() )
    {
      return false;
    }
    VersadrVO versadrVO = pnlAdresse.getVersadr();
    versadrVO.setTaxClass( getTaxClass( edit, versadrVO.getLandCd(), oldLandCd, versadrVO.getTaxClass() ) );
    pnlAdresseSap.setVersadr( versadrVO );
    return true;
  }

  /**
   * Fertigstellen
   * @return true = Fertigstellen erfolgreich, false = nicht erfolgreich
   */
  public boolean finish()
  {
    VersadrVO versadrVO;
    if ( pos == 0 )
    {
      // Fertigstellen wurde auf dem Adresse-Panel geklickt
      if ( !pnlAdresse.verifyInput() )
      {
        return false;
      }
      versadrVO = pnlAdresse.getVersadr();
      versadrVO.setTaxClass( getTaxClass( edit, versadrVO.getLandCd(), oldLandCd, versadrVO.getTaxClass() ) );
    }
    else
    {
      // Fertigstellen wurde auf dem AdresseSap-Panel geklickt
      versadrVO = pnlAdresseSap.getVersadr();
    }
    finish = true;
    try
    {
      IGESubject subject = AuthMain.getInstance().getSubject();
      versadrVO.setEmployeeId( subject.getEmployeeID() );
      if ( edit )
      {
        versadrVO.setGeaendBen( subject.getInitials() );
        versadrVO.setGeaendDat( new Date() );
        AdresseDelegate.getInstance().setVersadr( versadrVO );
      }
      else
      {
        versadrVO.setErstBen( subject.getInitials() );
        versadrVO.setErstDat( new Date() );
        adrNr = AdresseDelegate.getInstance().createVersadr( versadrVO );
      }
    }
    catch ( Exception ex )
    {
      LOG.error( ex );
      String message = xxMessage.getMessage( ex );
      int pos = ex.getMessage().indexOf( "Ungültige Email-Adresse" );
      if ( pos != -1 )
      {
        message = "Ungültige Email-Adresse: Die Emailadresse muss ein '@' enthalten\n" + "und vor und nach dem '@' Buchstaben aufweisen.\n"
            + "Eine gültige Emailadresse entspricht z.B. folgendem Muster: 'xxxx@xxxxx.xxx'\n"
            + "Bitte korrigieren Sie im Erfassungsfenster die Emailadresse im Textfeld 'Email'";
      }
      JOptionPane.showMessageDialog( dlgWizard, message, "Fehler", JOptionPane.ERROR_MESSAGE );
      return false;
    }
    return true;
  }

  /**
   * Gibt die neue Mehrwertsteuerklassifikation zurück
   * @param edit true = mutieren, false = erfassen
   * @param landCd Landcode
   * @param oldLandCd alter Landcode beim Mutieren
   * @param oldTaxClass alte Mehrwertsteuerklassifikation beim Mutieren
   * @return Mehrwertsteuerklassifikation
   */
  protected String getTaxClass( boolean edit, String landCd, String oldLandCd, String oldTaxClass )
  {
    if ( edit )
    {
      // Mutieren
      if ( landCd.equals( oldLandCd ) )
      {
        // Wenn das Land nicht mutiert wurde, dann keine Änderung der
        // Mehrwertsteuerklassifikation
        return oldTaxClass;
      }
      else if ( oldTaxClass.equals( "6" ) ) // 6 = Int. Organisation
      {
        // Wenn die Mehrwertsteuerklassifikation "internationale Organisation" ist,
        // dann keine Änderung der Mehrwertsteuerklassifikation
        return oldTaxClass;
      }
      else if ( landCd.equals( "CH" ) || landCd.equals( AdrConstants.LAND_LI ) )
      {
        // Wenn das Land von Ausland nach CH oder FL mutier wurde, dann "steuerpflichtig"
        return "1";
      }
      else
      {
        // Wenn das Land von CH oder FL nach Ausland mutiert wurde, dann "steuerbefreit"
        return "0";
      }
    }
    else
    {
      // Erfassen
      if ( landCd.equals( "CH" ) || landCd.equals( AdrConstants.LAND_LI ) )
      {
        // voller Steuersatz (Inland)
        return "1";
      }
      else
      {
        // steuerbefreit (Ausland)
        return "0";
      }
    }
  }

  /**
   * @return The authorization specification for this wizard
   */
  public static String getAuthzSpecification()
  {
    return "ch.ige.adr.client.PnlAdresseSap = Schreiben / AdresseSAP ";
  }

  /**
   * Initialisiert JavaHelp
   */
  public void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( dlgWizard.getRootPane(), "adrErfassen" );
  }
}
