package ch.ige.adr.client;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AdresseDelegate;
import ch.ige.adr.vo.PostOrtVO;
import ch.ige.adrhelp.HelpUtil;
import ch.ige.util.client.SwingWorker;

/**
 * <p>Beschreibung: Panel für Ortsverzeichnis laden und Ortsabgleich durchführen</p>
 * <p>Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Stefan Nydegger
 * @version $Id: PnlOrtsabgleich.java,v 1.18 2008/04/28 15:32:34 galli Exp $
 */

public class PnlOrtsabgleich extends JPanel
{
  // Logging
  private static final Logger LOG = Logger.getLogger( PnlOrtsabgleich.class );

  // Konstanten
  private static final int ORTS_VERZEICHNIS = 1;

  private static final int ORTS_ABGLEICH = 2;

  // Panels
  private JPanel pnlOrtsverzeichnis = new JPanel();

  private JPanel pnlVerzeichnis = new JPanel();

  private JPanel pnlAbgleich = new JPanel();

  private JPanel pnlSpace = new JPanel();

  // Borders
  private Border border1;

  private Border border2;

  // Buttons
  private JButton btnVerzeichnis = new JButton();

  private JButton btnOrtsverzeichnis = new JButton();

  private JButton btnOrtsabgleich = new JButton();

  // Textfields and Textareas
  private JTextArea txaOrtsabgleich = new JTextArea();

  private JTextArea txaOrtsverzeichnis = new JTextArea();

  private JTextField txtDatei = new JTextField();

  // Layouts
  protected GridBagLayout gridBagLayout1 = new GridBagLayout();

  protected GridBagLayout gridBagLayout2 = new GridBagLayout();

  protected GridBagLayout gridBagLayout3 = new GridBagLayout();

  protected GridBagLayout gridBagLayout4 = new GridBagLayout();

  protected GridBagLayout gridBagLayout5 = new GridBagLayout();

  // Variablen
  private String fileName = "";

  /**
   * Konstruktor
   */
  public PnlOrtsabgleich()
  {
    try
    {
      jbInit();
      enableActions();
      btnOrtsverzeichnis.setEnabled( false );
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.error( ex.getClass() + " " + ex.getMessage() );
      String message = ex.getMessage();
      JOptionPane.showMessageDialog( null, message, "Fehler", JOptionPane.ERROR_MESSAGE );
    }
  }

  /**
   * Komponente initialieren
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    GridBagConstraints gridBagConstraints7 = new GridBagConstraints( 0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0,
        0, 0, 0 ), 0, 0 );
    gridBagConstraints7.anchor = GridBagConstraints.NORTHWEST;
    GridBagConstraints gridBagConstraints6 = new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0,
        0, 0, 0 ), 113, 0 );
    gridBagConstraints6.anchor = GridBagConstraints.NORTHWEST;
    gridBagConstraints6.ipadx = 0;
    GridBagConstraints gridBagConstraints5 = new GridBagConstraints( 0, 2, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0,
        0, 0, 0 ), 176, 0 );
    gridBagConstraints5.anchor = GridBagConstraints.NORTH;
    GridBagConstraints gridBagConstraints4 = new GridBagConstraints( 1, 1, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(
        19, 16, 4, 9 ), 405, 5 );
    gridBagConstraints4.insets = new Insets( 15, 16, 0, 0 );
    gridBagConstraints4.anchor = GridBagConstraints.NORTHWEST;
    GridBagConstraints gridBagConstraints3 = new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0,
        16, 53, 9 ), -62, 0 );
    gridBagConstraints3.ipadx = 0;
    gridBagConstraints3.insets = new Insets( 6, 16, 0, 9 );
    gridBagConstraints3.anchor = GridBagConstraints.NORTH;
    GridBagConstraints gridBagConstraints2 = new GridBagConstraints( 1, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0,
        17, 0, 9 ), -64, -11 );
    gridBagConstraints2.ipadx = 0;
    gridBagConstraints2.insets = new Insets( 0, 16, 0, 0 );
    gridBagConstraints2.anchor = GridBagConstraints.NORTHWEST;
    gridBagConstraints2.ipady = 0;
    GridBagConstraints gridBagConstraints11 = new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 19,
        6, 4, 0 ), 0, 0 );
    gridBagConstraints11.anchor = GridBagConstraints.NORTHWEST;
    gridBagConstraints11.insets = new Insets( 15, 6, 0, 0 );
    pnlAbgleich.setPreferredSize( new Dimension( 770, 149 ) );
    GridBagConstraints gridBagConstraints1 = new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 0, 6,
        95, 0 ), 0, 0 );
    gridBagConstraints1.insets = new Insets( 6, 6, 0, 0 );
    gridBagConstraints1.anchor = GridBagConstraints.NORTHWEST;
    GridBagConstraints gridBagConstraints = new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 0, 6,
        30, 0 ), 0, 0 );
    gridBagConstraints.insets = new Insets( 6, 6, 0, 0 );
    gridBagConstraints.anchor = GridBagConstraints.NORTHWEST;
    this.setLayout( gridBagLayout4 );
    this.setBorder( BorderFactory.createEtchedBorder() );
    this.setPreferredSize( new Dimension( 770, 450 ) );

    border1 = new TitledBorder( BorderFactory.createEtchedBorder( Color.white, new Color( 168, 168, 158 ) ), "Ortsverzeichnis laden" );
    border2 = new TitledBorder( BorderFactory.createEtchedBorder( Color.white, new Color( 168, 168, 158 ) ), "Ortsabgleich starten" );

    pnlOrtsverzeichnis.setLayout( gridBagLayout5 );
    pnlVerzeichnis.setLayout( gridBagLayout1 );
    pnlVerzeichnis.setBorder( border1 );
    pnlVerzeichnis.setMinimumSize( new Dimension( 770, 135 ) );
    pnlVerzeichnis.setPreferredSize( new Dimension( 770, 149 ) );

    txaOrtsverzeichnis.setText( "Start Einlesen des Ortsverzeichnis.\n" + "Bevor das Einlesen gestartet werden kann muss eine Datei ausgewählt werden.\n" );
    txaOrtsverzeichnis.setMinimumSize( new Dimension( 470, 34 ) );
    txaOrtsverzeichnis.setPreferredSize( new Dimension( 470, 68 ) );
    txaOrtsverzeichnis.setRequestFocusEnabled( false );
    txaOrtsverzeichnis.setEditable( false );
    txaOrtsverzeichnis.setLineWrap( true );
    txaOrtsverzeichnis.setWrapStyleWord( true );
    btnVerzeichnis.setMaximumSize( new Dimension( 150, 24 ) );
    btnVerzeichnis.setMinimumSize( new Dimension( 150, 24 ) );
    btnVerzeichnis.setPreferredSize( new Dimension( 150, 26 ) );
    btnVerzeichnis.setText( "Datei auswählen" );
    btnVerzeichnis.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        btnVerzeichnis_actionPerformed( e );
      }
    } );
    btnOrtsverzeichnis.setMaximumSize( new Dimension( 150, 24 ) );
    btnOrtsverzeichnis.setMinimumSize( new Dimension( 150, 24 ) );
    btnOrtsverzeichnis.setPreferredSize( new Dimension( 150, 26 ) );
    btnOrtsverzeichnis.setActionCommand( "Ortsverzeichnis laden" );
    btnOrtsverzeichnis.setText( "Ortsverzeichnis laden" );
    btnOrtsverzeichnis.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        btnOrtsverzeichnis_actionPerformed( e );
      }
    } );

    txtDatei.setRequestFocusEnabled( false );
    txtDatei.setEditable( false );
    txaOrtsabgleich.setText( "Starten des Ortsabgleich\n" + "Bevor der Ortsabgleich ausgeführt werden kann, muss " + "das Laden des Ortsverzeichnis erfolgen" );
    txaOrtsabgleich.setMinimumSize( new Dimension( 470, 43 ) );
    txaOrtsabgleich.setPreferredSize( new Dimension( 470, 68 ) );
    txaOrtsabgleich.setRequestFocusEnabled( false );
    txaOrtsabgleich.setEditable( false );
    txaOrtsabgleich.setLineWrap( true );
    txaOrtsabgleich.setWrapStyleWord( true );
    btnOrtsabgleich.setMaximumSize( new Dimension( 150, 24 ) );
    btnOrtsabgleich.setHorizontalAlignment( SwingConstants.CENTER );
    btnOrtsabgleich.setVerticalAlignment( SwingConstants.TOP );
    btnOrtsabgleich.setMinimumSize( new Dimension( 150, 24 ) );
    btnOrtsabgleich.setPreferredSize( new Dimension( 150, 26 ) );
    btnOrtsabgleich.setActionCommand( "Ortsabgleich" );
    btnOrtsabgleich.setText( "Ortsabgleich" );
    btnOrtsabgleich.addActionListener( new java.awt.event.ActionListener()
    {
      public void actionPerformed( ActionEvent e )
      {
        btnOrtsabgleich_actionPerformed( e );
      }
    } );
    pnlAbgleich.setLayout( gridBagLayout2 );
    pnlAbgleich.setBorder( border2 );
    pnlSpace.setMinimumSize( new Dimension( 770, 135 ) );
    pnlSpace.setPreferredSize( new Dimension( 770, 135 ) );
    pnlSpace.setLayout( gridBagLayout3 );
    pnlOrtsverzeichnis.setPreferredSize( new Dimension( 770, 405 ) );

    pnlOrtsverzeichnis.add( pnlSpace, gridBagConstraints5 );
    this.add( pnlOrtsverzeichnis, new GridBagConstraints( 0, 0, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets( 0, 0, 0, 0 ),
        113, 15 ) );
    pnlVerzeichnis.add( btnOrtsverzeichnis, gridBagConstraints );
    pnlVerzeichnis.add( txaOrtsverzeichnis, gridBagConstraints2 );
    pnlVerzeichnis.add( btnVerzeichnis, gridBagConstraints11 );
    pnlVerzeichnis.add( txtDatei, gridBagConstraints4 );
    pnlAbgleich.add( btnOrtsabgleich, gridBagConstraints1 );
    pnlAbgleich.add( txaOrtsabgleich, gridBagConstraints3 );
    pnlOrtsverzeichnis.add( pnlVerzeichnis, gridBagConstraints6 );
    pnlOrtsverzeichnis.add( pnlAbgleich, gridBagConstraints7 );
  }

  /**
   * aktiviert oder deaktiviert Buttons je nach Status
   */
  private void enableActions()
  {
    btnOrtsverzeichnis.setEnabled( true );
    btnVerzeichnis.setEnabled( true );
    btnOrtsabgleich.setEnabled( true );
  }

  /**
   * Deaktiviert alle Buttons
   * Wird aufgerufen falls kein richtiger Lauf selektiert ist (Knoten)
   */
  private void disableActions()
  {
    btnOrtsverzeichnis.setEnabled( false );
    btnVerzeichnis.setEnabled( false );
    btnOrtsabgleich.setEnabled( false );
  }

  /**
   * Wiederaufnahme der Datenübertragung von Bagis nach SES
   * @param e ActionEvent
   */
  void btnOrtsverzeichnis_actionPerformed( ActionEvent e )
  {
    String question = "Soll das Ortsverzeichnis wirklich geladen werden?";
    startAction( ORTS_VERZEICHNIS, question );
  }

  /**
   * Auswählen der Datei mit den Orten und PLZ
   * @param e ActionEvent
   */
  void btnVerzeichnis_actionPerformed( ActionEvent e )
  {
    JFileChooser chooser = new JFileChooser();
    int state = chooser.showOpenDialog( null );
    File file = chooser.getSelectedFile();
    if ( file != null && state == JFileChooser.APPROVE_OPTION )
    {
      fileName = file.getPath();
      txtDatei.setText( fileName );
      btnOrtsverzeichnis.setEnabled( true );
    }
    else
    {
      // wenn kein File vorgängig ausgewählt wurde, Button Ortsabgleich disablen
      if ( txtDatei.getText().equals( "" ) )
      {
        btnOrtsverzeichnis.setEnabled( false );
      }
    }
  }

  /**
   * Ortsabgleich durchführen
   * @param e ActionEvent
   */
  void btnOrtsabgleich_actionPerformed( ActionEvent e )
  {
    String question = "Soll der Ortsabgleich wirklich ausgeführt werden?";
    startAction( ORTS_ABGLEICH, question );
  }

  /**
   * Startet die gewählte Aktion
   * @param action gewünschte Aktion
   * @param question Abfrage
   */
  private void startAction( int action, String question )
  {
    PostOrtVO[] postOrtVO = null;
    Ortsabgleich ortsabgleich = (Ortsabgleich) this.getRootPane();
    // Rückfrage ob Aktion durchgeführt werden soll
    String title = "Aktion bestätigen";
    String[] options = { "Ja", "Nein" };

    int state = JOptionPane.showOptionDialog( this.getRootPane(), question, title, JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, options,
        options[1] );

    if ( state == JOptionPane.YES_OPTION )
    {
      try
      {
        switch( action )
        {
          case ORTS_VERZEICHNIS:
            postOrtVO = new PostOrt().readFile( fileName );
            processAction( action, postOrtVO );
            break;
          case ORTS_ABGLEICH:
            processAction( action, null );
            break;
        }
      }
      catch ( Exception ex )
      {
        LOG.error( ex.getClass() + " " + ex.getMessage() );
        String message = "Aktion konnte nicht durchgeführt werden\n" + "Fehler: " + ex.getMessage();

        JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      }
    }
    else
    {
      ortsabgleich.setStatusBar( "Aktion abgebrochen" );
    }
  }

  /**
   * startet Ortsabgleich oder das Einlesen des Ortsverzeichnis
   * @param action    gewünschte Aktion
   * @param postOrtVO ValueObject mit allen Orten und PLZ
   */
  private void processAction( final int action, final PostOrtVO[] postOrtVO )
  {
    final Ortsabgleich ortsabgleich = (Ortsabgleich) this.getRootPane();
    ortsabgleich.setStatusBar( "Bitte warten..." );
    ortsabgleich.getGlassPane().setVisible( true );
    final JPanel dialog = this;
    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        try
        {
          switch( action )
          {
            case ORTS_VERZEICHNIS:
              AdresseDelegate.getInstance().ladeOrtsverzeichnis( postOrtVO );
              break;
            case ORTS_ABGLEICH:
              AdresseDelegate.getInstance().ortsabgleich();
              break;
          }
        }
        catch ( Exception ex )
        {
          ortsabgleich.setStatusBar( "Aktion wurde nicht erfolgreich durchgeführt" );
          LOG.error( ex );
          JOptionPane.showMessageDialog( dialog, ex.getMessage(), "Fehler", JOptionPane.ERROR_MESSAGE );
          return null;
        }
        return null;
      }

      public void finished()
      {
        ortsabgleich.getGlassPane().setVisible( false );
        ortsabgleich.setStatusBar( "Aktion erfolgreich durchgeführt" );
      }
    }; // SwingWorker

    worker.start();
  }

  /**
   * initialisiert javahelp
   */
  public void initHelp()
  {
//    HelpUtil.enableHelpKey(this, "ortsabgleich");

    HelpUtil.getInstance().enableHelp( btnOrtsabgleich, "ortsabgleich.ortsabgleich" );
    HelpUtil.getInstance().enableHelp( btnOrtsverzeichnis, "ortsabgleich.ortsverzeichnis" );
    HelpUtil.getInstance().enableHelp( btnVerzeichnis, "ortsabgleich.auswaehlen" );
  }
}
