package ch.ige.adr.server;

import java.util.List;
import java.util.Iterator;

/**
 * field expression parser and sql assembler.
 * supported operators: OR, AND, NOT
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author Thomas Iten
 * @version $Id: FieldParserOrAndNot.java,v 1.1 2007/07/03 15:20:54 nydegger Exp $
 */
public class FieldParserOrAndNot 
{

  /**
   * Default operator used to connect single tokens, if no operator is given
   * between the tokens
   */
  private String defaultOperator;



  /**
   * Construct Field Parser and define default operator
   *
   * @param defaultOperator default operator
   */
  public FieldParserOrAndNot(String defaultOperator)
  {
    this.defaultOperator = defaultOperator;
  }


  /**
   * @see FieldParserInterface
   */
  public void parse (List list, AssemblerOracleText field) throws Exception
  {
    int     step      = 1;
    String  token     = null;
    String  lastToken = "";

    for (Iterator iter = list.iterator(); iter.hasNext(); )
    {
      token = (String)iter.next();

      switch (step)
      {
        //
        // step 1
        //
        case 1:
          if (token.equalsIgnoreCase(AssemblerOracleText.OPERATOR_NOT))
          {
            step = 2;
          }
          else
          {
            if (lastToken.equalsIgnoreCase(AssemblerOracleText.OPERATOR_OR))
            {
              field.assembleOr(token);
            }
            else if (lastToken.equalsIgnoreCase(AssemblerOracleText.OPERATOR_AND))
            {
              field.assembleAnd(token);
            }
            else
            {
              field.assemble(token);
            }
            lastToken = "";
            step = 3;
          }
          break;

        //
        // step 2
        //
        case 2:
          if (lastToken.equalsIgnoreCase(AssemblerOracleText.OPERATOR_OR))
          {
            // is not supported
            throw new Exception("or not operation is not supported");
          }
          else if (lastToken.equalsIgnoreCase(AssemblerOracleText.OPERATOR_AND))
          {
            field.assembleAndNot(token);
          }
          else
          {
            field.assembleNot(token);
          }
          lastToken = "";
          step = 3;
          break;

        //
        // step 3
        //
        case 3:
          if (token.equalsIgnoreCase(AssemblerOracleText.OPERATOR_AND) ||
              token.equalsIgnoreCase(AssemblerOracleText.OPERATOR_OR))
          {
            lastToken = token;
            step = 1;
          }
          else if (token.equalsIgnoreCase(AssemblerOracleText.OPERATOR_NOT))
          {
            lastToken = defaultOperator;
            step = 2;
          }
          else
          {
            if (defaultOperator.equalsIgnoreCase(AssemblerOracleText.OPERATOR_OR))
            {
              field.assembleOr(token);
            }
            else if (defaultOperator.equalsIgnoreCase(AssemblerOracleText.OPERATOR_AND))
            {
              field.assembleAnd(token);
            }
            step = 3;
          }
          break;
      } // end switch

    } // end for

  } // end method


} // end class
