package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

import ch.ige.adr.server.AssemblerOracleText;
import ch.ige.adr.server.Crescendo;
import ch.ige.adr.server.FieldParserOrAndNot;
import ch.ige.adr.vo.AdresseAVO;
import ch.ige.adr.vo.AdresseSelKritVO;
import ch.ige.adr.vo.AdresseVO;
import ch.ige.adr.vo.PostOrtVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Datenbankzugriff auf die Adresse
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: AdresseDAO.java,v 1.33 2008/05/13 07:17:14 nydegger Exp $
 */
public class AdresseDAO
{
  private AssemblerOracleText assembler = null;
  private FieldParserOrAndNot parser = null;
  private static final Logger LOG = Logger.getLogger(AdresseDAO.class);
  
  
  public AdresseDAO()
  {
    assembler = new AssemblerOracleText();
    parser = new FieldParserOrAndNot(AssemblerOracleText.OPERATOR_AND);
  }
  
  /**
   * Selektiert eine Adresse.
   * @param adrNr Adressnummer
   * @return Value Object der Adresse
   * @exception SQLException
   */
  public AdresseVO select(int adrNr) throws SQLException
  {
    assembler.reset();
    AdresseVO adresseVO = new AdresseVO();
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    String query = null;
    try
    {
      query = "SELECT adr_nr, trim(suchbegr) suchbegr, trim(sprach_cd) sprach_cd, trim(anr_cd) anr_cd, " +
            "trim(name) name, trim(zeile_1) zeile_1, trim(zeile_2) zeile_2, trim(zeile_3) zeile_3, " +
            "trim(zeile_4) zeile_4, trim(land_cd) land_cd, trim(pf_plz) pf_plz, trim(plz) plz, "+
            "trim(ort) ort, trim(telefon) telefon, trim(fax) fax, " +
            "erst_dat, trim(erst_ben) erst_ben, geaend_dat, trim(geaend_ben) geaend_ben, " +
            "trim(u_version) u_version, trim(plz_zusatz) plz_zusatz, gesperrt, adr_nr_neu, version " +
            "FROM adresse WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adrNr);
      rs = ps.executeQuery();

      if(!rs.next())
      {
        return null;
      }
      adresseVO.setAdrNr(rs.getInt("adr_nr"));
      adresseVO.setSuchbegr(rs.getString("suchbegr"));
      adresseVO.setSprachCd(rs.getString("sprach_cd"));
      adresseVO.setAnrCd(rs.getString("anr_cd"));
      adresseVO.setName(rs.getString("name"));
      adresseVO.setZeile1(rs.getString("zeile_1"));
      adresseVO.setZeile2(rs.getString("zeile_2"));
      adresseVO.setZeile3(rs.getString("zeile_3"));
      adresseVO.setZeile4(rs.getString("zeile_4"));
      adresseVO.setLandCd(rs.getString("land_cd"));
      adresseVO.setPfPlz(rs.getString("pf_plz"));
      adresseVO.setPlz(rs.getString("plz"));
      adresseVO.setOrt(rs.getString("ort"));
      adresseVO.setTelefon(rs.getString("telefon"));
//      adresseVO.telex = rs.getString("telex");
      adresseVO.setFax(rs.getString("fax"));
      adresseVO.setErstDat(rs.getDate("erst_dat"));
      adresseVO.setErstBen(rs.getString("erst_ben"));
      adresseVO.setGeaendDat(rs.getDate("geaend_dat"));
      adresseVO.setGeaendBen(rs.getString("geaend_ben"));
      adresseVO.setUVersion(rs.getString("u_version"));
      adresseVO.setPlzZusatz(rs.getString("plz_zusatz"));
      adresseVO.setGesperrt(rs.getInt("gesperrt"));
      adresseVO.setAdrNrNeu(rs.getInt("adr_nr_neu"));
      adresseVO.setVersion(rs.getInt("version"));
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + " adr_nr = " + adrNr + " \n");
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return adresseVO;
  }

  /**
   * Liest alle Adr-Nummern aus der Tabelle Adresse
   * @return ArrayList mit Integer-Objekten mit den Adressnummern
   * @param startAdrNr Start-Adressnummer
   * @throws SQLException
   */
  public ArrayList selectAdrNr(int startAdrNr) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    ArrayList adrNrAL = new ArrayList(1000000);
    String query = null;
    try
    {
      query = "SELECT adr_nr FROM adresse WHERE adr_nr >= ? ORDER BY adr_nr";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, startAdrNr);
      rs = ps.executeQuery();
      while(rs.next())
      {
        adrNrAL.add(rs.getObject("adr_nr"));
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + " adr_nr = " + startAdrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    adrNrAL.trimToSize();
    return adrNrAL;
  }

  /**
   * Selekiert alle Schweizer adr_nr, ort und PLZ der Tabelle Adresse welche nicht gesperrt sind<br>
   * Wird für den Ortsabgleich benötigt
   * @return ArrayList mit Integer-Objekten mit den Adressnummern
   * @throws SQLException
   */
  public ArrayList select() throws SQLException
  {
    String query = null;
    query = "SELECT adr_nr, trim(ort) ort, trim(plz) plz FROM adresse " +
            "WHERE land_cd = RPAD('CH', 3, ' ') AND suchbegr <> '**********' " +
            "AND (gesperrt IS NULL OR gesperrt = 0) " +
            "AND erst_dat > to_date('30.06.2003', 'DD.MM.YYYY') " +
            "ORDER BY adr_nr";
    int count = 0;
    ArrayList alAdr = new ArrayList(1000);
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);

      rs = ps.executeQuery();
      while (rs.next())
      {
        AdresseVO adr = new AdresseVO();

        adr.setAdrNr(rs.getInt("adr_nr"));
        adr.setPlz(rs.getString("plz"));
        adr.setOrt(rs.getString("ort"));
        alAdr.add(adr);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    LOG.debug("select() - Anzahl Datensätze= " + count);
    alAdr.trimToSize();
    return alAdr;
  }


  /**
   * Kreiert eine Adresse.
   * @param adresseVO Value Object der Adresse
   * @exception SQLException
   */
  public void insert(AdresseVO adresseVO) throws SQLException
  {
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "INSERT INTO adresse(adr_nr, suchbegr, sprach_cd, anr_cd, name, " +
              "zeile_1, zeile_2, zeile_3, zeile_4, land_cd, pf_plz, plz, ort, " +
              "telefon, fax, " +
//              "telex, " + wird nicht mehr verwendet
              "erst_dat, erst_ben, geaend_dat, geaend_ben, " +
              "u_version, plz_zusatz, gesperrt, adr_nr_neu, version)" +
              "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, " +
              "?, ?, ?, ?, ?, ?, ?)";

        // auf Überlängen prüfen
       String truncMsg = adresseVO.truncateFields();
       if (truncMsg != null)
       {
         LogLevelHelper.log(new Exception(""), AdresseDAO.class, truncMsg + "\n" + query + adresseVO.toString());
       }

      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adresseVO.getAdrNr());
      ps.setString(2, adresseVO.getSuchbegr());
      ps.setString(3, adresseVO.getSprachCd());
      ps.setString(4, adresseVO.getAnrCd());
      ps.setString(5, adresseVO.getName());
      ps.setString(6, adresseVO.getZeile1());
      ps.setString(7, adresseVO.getZeile2());
      ps.setString(8, adresseVO.getZeile3());
      ps.setString(9, adresseVO.getZeile4());
      ps.setString(10, adresseVO.getLandCd());
      ps.setString(11, adresseVO.getPfPlz());
      ps.setString(12, adresseVO.getPlz());
      ps.setString(13, adresseVO.getOrt());
      ps.setString(14, adresseVO.getTelefon());
      ps.setString(15, adresseVO.getFax());
      if (adresseVO.getErstDat() == null)
      {
        ps.setDate(16, null);
      }
      else
      {
        ps.setDate(16, new java.sql.Date(adresseVO.getErstDat().getTime()));
      }
      ps.setString(17, adresseVO.getErstBen());
      if (adresseVO.getGeaendDat() == null)
      {
        ps.setDate(18, null);
      }
      else
      {
        ps.setDate(18, new java.sql.Date(adresseVO.getGeaendDat().getTime()));
      }
      ps.setString(19, adresseVO.getGeaendBen());
      ps.setString(20, adresseVO.getUVersion());
      ps.setString(21, adresseVO.getPlzZusatz());
      ps.setInt(22, adresseVO.getGesperrt());
      ps.setInt(23, adresseVO.getAdrNrNeu());
      ps.setInt(24, adresseVO.getVersion());
      ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + adresseVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Setzt die Adresse auf den Wert des Value Objects.
   * @param adresseVO Value Object der Adresse
   * @return Anzahl der aktualisierten Adressen
   * @exception SQLException
   */
  public int update(AdresseVO adresseVO) throws SQLException
  {
    int rowCount;
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "UPDATE adresse SET " +
              "suchbegr = ?, " +
              "sprach_cd = ?, " +
              "anr_cd = ?, " +
              "name = ?, " +
              "zeile_1 = ?, " +
              "zeile_2 = ?, " +
              "zeile_3 = ?, " +
              "zeile_4 = ?, " +
              "land_cd = ?, " +
              "pf_plz = ?, " +
              "plz = ?, " +
              "ort = ?, " +
              "telefon = ?, " +
              "fax = ?, " +
//              "telex,  " + wird nicht mehr verwendet
              "erst_dat = ?, " +
              "erst_ben = ?, " +
              "geaend_dat = ?, " +
              "geaend_ben = ?, " +
              "u_version = ?, " +
              "plz_zusatz = ?, " +
              "gesperrt = ?, " +
              "adr_nr_neu = ?, " +
              "version = ? " +
              "WHERE adr_nr = ?";

          // auf Überlängen prüfen
       String truncMsg = adresseVO.truncateFields();
       if (truncMsg != null)
       {
         LogLevelHelper.log(new Exception(""), AdresseDAO.class, truncMsg + "\n" + query + adresseVO.toString());
       }

      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, adresseVO.getSuchbegr());
      ps.setString(2, adresseVO.getSprachCd());
      ps.setString(3, adresseVO.getAnrCd());
      ps.setString(4, adresseVO.getName());
      ps.setString(5, adresseVO.getZeile1());
      ps.setString(6, adresseVO.getZeile2());
      ps.setString(7, adresseVO.getZeile3());
      ps.setString(8, adresseVO.getZeile4());
      ps.setString(9, adresseVO.getLandCd());
      ps.setString(10, adresseVO.getPfPlz());
      ps.setString(11, adresseVO.getPlz());
      ps.setString(12, adresseVO.getOrt());
      ps.setString(13, adresseVO.getTelefon());
      ps.setString(14, adresseVO.getFax());
      if (adresseVO.getErstDat() == null)
      {
        ps.setDate(15, null);
      }
      else
      {
        ps.setDate(15, new java.sql.Date(adresseVO.getErstDat().getTime()));
      }
      ps.setString(16, adresseVO.getErstBen());
      if (adresseVO.getGeaendDat() == null)
      {
        ps.setDate(17, null);
      }
      else
      {
        ps.setDate(17, new java.sql.Date(adresseVO.getGeaendDat().getTime()));
      }
      ps.setString(18, adresseVO.getGeaendBen());
      ps.setString(19, adresseVO.getUVersion());
      ps.setString(20, adresseVO.getPlzZusatz());
      ps.setInt(21, adresseVO.getGesperrt());
      ps.setInt(22, adresseVO.getAdrNrNeu());
      ps.setInt(23, adresseVO.getVersion());
      ps.setInt(24, adresseVO.getAdrNr());
      rowCount = ps.executeUpdate();
      ps.close();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + " " + adresseVO.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }

  /**
   * Setzt die Adresse auf
   * @param ort Ort
   * @param plz PLZ
   * @param landCd Landcode
   * @param postOrtVO Ortschaft im Ortsverzeichnis
   * @return Anzahl der aktualisierten Adressen
   * @exception SQLException
   */
  public int update(String ort, String plz, String landCd, PostOrtVO postOrtVO) throws SQLException
  {
    int rowCount = 0;
    String query = "";
    Connection con = null;
    PreparedStatement ps = null;
    PreparedStatement ps2 = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT adr_nr, u_version, version FROM adresse WHERE ort = ? AND plz = ? AND land_cd = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, ort);
      ps.setString(2, plz);
      ps.setString(3, landCd);
      rs = ps.executeQuery();
      int i = 0;
      while(rs.next())
      {
        int adrNr = rs.getInt("adr_nr");
        int version = rs.getInt("version");
        String uVersion = rs.getString("u_version");
        query = "UPDATE adresse SET ort = ?, plz = ?, u_version = ?, version = ?, land_cd = ? WHERE adr_nr = ? ";
        ps2 = con.prepareStatement(query);
        ps2.setString(1, postOrtVO.getOrt());
        ps2.setString(2, postOrtVO.getPlz());
        ps2.setString(3, Crescendo.getNextUVersion(uVersion));
        ps2.setInt(4, ++version );
        ps2.setString(5, postOrtVO.getLandCd());
        ps2.setInt(6, adrNr);
        rowCount += ps2.executeUpdate();
        i++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + " ort = " + ort + " plz = " + plz + " landCd " + landCd);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, ps2, con);
    }
    return rowCount;
  }

  /**
   * Löscht eine Adresse.
   * @param adrNr Adressnummer der zu löschenden Adresse
   * @return Anzahl der gelöschten Adressen
   * @exception SQLException
   */
  public int delete(int adrNr) throws SQLException
  {
    int rowCount;
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    try
    {
      query = "DELETE FROM adresse WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adrNr);
      rowCount = ps.executeUpdate();
    }
    catch (SQLException ex)
    {
      SQLException newEx = null;
      if ( ex.getMessage().indexOf( "ORA-02292" ) > -1 )
      {
       newEx = new SQLException("MNR=5075, delete versandadresse " + ex.getMessage()); 
      }
      else
      {
        newEx = ex;
      }
      LogLevelHelper.log(newEx, AdresseDAO.class, query + " adrNr = " + adrNr);
      throw newEx;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    return rowCount;
  }

  /**
   * Findet alle Adressen, auf welche die Selektionskriterien zutreffen.
   * @param adresseSelKritVO Value Object mit den Selektionskriterien
   * @return ArrayList mit AdresseAVO-Objekten mit den gefundenen Adressen
   * @exception SQLException
   */
  public ArrayList select(AdresseSelKritVO adresseSelKritVO) throws SQLException
  {
    // Assertions
    // Mindestens ein Suchkriterium muss angegeben werden
    if (adresseSelKritVO.adrNr == 0 &&
        adresseSelKritVO.name.equals("") &&
        adresseSelKritVO.strasse.equals("") &&
        adresseSelKritVO.ort.equals("") &&
        adresseSelKritVO.plz.equals("") &&
        adresseSelKritVO.suchbegr.equals("") &&
        !adresseSelKritVO.nurOrtsabgleich)
    {
      if (adresseSelKritVO.landCd.equals(""))
      {
        throw new IllegalArgumentException("Mindestens ein Suchkriterium muss angegeben werden");
      }
      else
      {
        throw new IllegalArgumentException("Land darf nicht das einzige Suchkriterim sein");
      }
    }
    int count = 0;
    ArrayList adresseAL = new ArrayList(1000);
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;

    StringBuffer select = new StringBuffer();
    StringBuffer from = new StringBuffer();
    StringBuffer where = new StringBuffer();

    select.append("SELECT DISTINCT a.adr_nr, a.suchbegr, a.name, a.zeile_1, a.zeile_2, a.zeile_3, a.zeile_4, ");
    select.append("a.land_cd, a.plz, a.ort ");
    from.append("FROM adresse a ");
    where.append("WHERE ");
    String and = "";

    if (adresseSelKritVO.adrNr == 0)
    {
      // keine Adressnummer eingegeben
      // alle anderen Fälle
      if (!adresseSelKritVO.suchbegr.equals(""))
      {
        // Suchbegriff wurde eingegeben
        String umlaut =ch.ige.util.StringUtil.replaceAeOeUe(adresseSelKritVO.suchbegr);
        String suchbegr = ch.ige.util.StringUtil.replaceUmlaut(adresseSelKritVO.suchbegr);
        String likeOrEquals = " = ";
        if (adresseSelKritVO.suchbegr.indexOf('%') > -1)
        {
          likeOrEquals = " LIKE ";
        }
        if (suchbegr.equals(umlaut))
        {
          where.append("a.suchbegr " + likeOrEquals);
          where.append(DBHelper.getSqlString(suchbegr) + " ");
        }
        else
        {
          where.append("(a.suchbegr" + likeOrEquals);
          where.append(DBHelper.getSqlString(suchbegr));
          where.append(" OR a.suchbegr" + likeOrEquals);
          where.append(DBHelper.getSqlString(umlaut) + ") ");
        }
        and = " AND ";
      }
      if (!adresseSelKritVO.name.equals(""))
      {
        // Name wurde eingegeben
        if ( adresseSelKritVO.suchmodusName.equals( AdresseSelKritVO.SUCHMODUS_VOLLTEXT ) )
        {
          // Volltextsuche
          where.append( and );
          String likeOrEquals = " = ";
          if ( adresseSelKritVO.name.indexOf( '%' ) > -1 )
          {
            likeOrEquals = " LIKE ";
          }
          String selKritName = DBHelper.getSqlString( adresseSelKritVO.name );
          where.append( " (a.name" + likeOrEquals );
          where.append( selKritName + " " );
          where.append( "OR a.zeile_1" + likeOrEquals );
          where.append( selKritName + " )" );
          and = " AND ";
        }
        else
        {
          // Index Suche
          assembler.reset();
          where.append( and );
          StringTokenizer st = new StringTokenizer(adresseSelKritVO.name, ".");
          List lst = new ArrayList();
          while(st.hasMoreTokens())
          {
            lst.add(  st.nextToken() );
          }
          try
          {
            parser.parse( lst, assembler );
          }
          catch ( Exception e )
          {
            LOG.error( "Error: " + e.getMessage(), e );
            throw new SQLException(e.getMessage());
          }        
          where.append( "contains (name , '" + assembler.getValue()+"') >0 ");
        }
        and = " AND ";
      }

      if (!adresseSelKritVO.strasse.equals(""))
      {
        where.append( and );
        assembler.reset();
        // Strasse wurde eingegeben
        // Index Suche
        StringTokenizer st = new StringTokenizer(adresseSelKritVO.strasse, ".");
        List lst = new ArrayList();
        while(st.hasMoreTokens())
        {
          lst.add(  st.nextToken() );
        }
        try
        {
          parser.parse( lst, assembler );
        }
        catch ( Exception e )
        {
          LOG.error( "Error: " + e.getMessage(), e );
          throw new SQLException(e.getMessage());
        }        
        where.append( "contains (zeile_2 , '" + assembler.getValue()+"') >0 ");
        and = " AND ";
      }

      if (!adresseSelKritVO.plz.equals(""))
      {
        //PLZ wurde eingegeben
        where.append(and);
        where.append("a.plz ");
        if (adresseSelKritVO.plz.indexOf('%') == -1)
        {
          where.append("= ");
        }
        else
        {
          where.append(" LIKE ");
        }
        where.append("'");
        where.append(adresseSelKritVO.plz);
        where.append("'");
        and = " AND ";
      }

      if (!adresseSelKritVO.ort.equals(""))
      {
        where.append( and );
        assembler.reset();
        //Ort wurde eingegeben
        if(adresseSelKritVO.suchmodusOrt.equals(AdresseSelKritVO.SUCHMODUS_VOLLTEXT))
        {
          String likeOrEquals = " = ";
          if (adresseSelKritVO.ort.indexOf('%') > -1)
          {
            likeOrEquals = " LIKE ";
          }
          String selKritOrt = DBHelper.getSqlString(adresseSelKritVO.ort);
          where.append(" a.ort" + likeOrEquals);
          where.append(selKritOrt + " ");
        }
        else
        {
          assembler.reset();
          // Index Suche
          StringTokenizer st = new StringTokenizer(adresseSelKritVO.ort, ".");
          List lst = new ArrayList();
          while(st.hasMoreTokens())
          {
            lst.add(  st.nextToken() );
          }
          try
          {
            parser.parse( lst, assembler );
          }
          catch ( Exception e )
          {
            LOG.error( "Error: " + e.getMessage(), e );
            throw new SQLException(e.getMessage());
          }        
          where.append( "contains (zeile_4 , '" + assembler.getValue()+"') >0 ");
        }
        and = " AND ";
      }

      if(!adresseSelKritVO.landCd.equals(""))
      {
        //landCd wurde eingegeben
        where.append(and);
        where.append("a.land_cd ");
        if (adresseSelKritVO.landCd.indexOf('%') == -1)
        {
          where.append(" = ");
        }
        else
        {
          where.append(" LIKE ");
        }
        where.append("'");
        where.append(adresseSelKritVO.landCd);
        where.append("'");
        and = " AND ";
      }
      if (adresseSelKritVO.depKto.equals(AdresseSelKritVO.NUR_MIT))
      {
        // nur mit Deponentenkonto
        from.append(", adressesap sap ");
        where.append(and);
        where.append("a.adr_nr = sap.adr_nr AND sap.cust_group = '01' ");
        and = " AND ";
      }
      else if (adresseSelKritVO.depKto.equals(AdresseSelKritVO.NUR_OHNE))
      {
        // nur ohne Deponentenkonto
        from.append(", adressesap sap ");
        where.append(and);
        where.append("a.adr_nr = sap.adr_nr AND sap.cust_group = '02' ");
        and = " AND ";
      }
      if (adresseSelKritVO.gesperrt.equals(AdresseSelKritVO.NUR_GUELTIGE))
      {
        // nur gültige d.h. nicht gesperrte Adressen
        where.append(and);
        where.append("(a.gesperrt = 0 OR a.gesperrt IS null) ");
        and = " AND ";
      }
      else if (adresseSelKritVO.gesperrt.equals(AdresseSelKritVO.NUR_GESPERRTE))
      {
        // nur gültige d.h. nicht gesperrte Adressen
        where.append(and);
        where.append("a.gesperrt = 1 ");
        and = " AND ";
      }
    }
    else
    {
      // Adressnummer wurde eingegeben
      // nur nach Adressnummer suchen
      where.append("a.adr_nr = ");
      where.append(adresseSelKritVO.adrNr);
      and = " AND ";
    }
    if(adresseSelKritVO.nurOrtsabgleich)
    {
      where.append(and);
      from.append(", adresse_ptt_pend app ");
      where.append("a.adr_nr = app.adr_nr ");
    }
    String sql = select.toString() + from.toString() + where.toString();

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());
      int maxCount = 1000;
      if (adresseSelKritVO.nurOrtsabgleich)
      {
        // im Modus Ortsabgleich maximal 5000 Einträge anzeigen
        maxCount = 5000;
      }
      while (rs.next() && count < maxCount)
      {
        AdresseAVO adr = new AdresseAVO();

        adr.suchbegr = rs.getString("suchbegr");
        if (adr.suchbegr != null && adr.suchbegr.equals("**********"))
        {
          // Logisch gelöschte Adressen ignorieren
          continue;
        }
        adr.adrNr = rs.getInt("adr_nr");
        adr.name = rs.getString("name");
        if (adr.name == null || adr.name.trim().length() == 0)
        {
          adr.name = rs.getString("zeile_1");
        }
        adr.strasse = rs.getString("zeile_2");
        if (adr.strasse == null || adr.strasse.trim().length() == 0)
        {
          adr.strasse = rs.getString("zeile_3");
        }
        if (adr.strasse == null || adr.strasse.trim().length() == 0)
        {
          adr.strasse = rs.getString("zeile_4");
        }
        adr.landCd = rs.getString("land_cd");
        adr.plz = rs.getString("plz");
        adr.ort = rs.getString("ort");

        // Da ADRESSE eine Crescendo-Tabelle ist, können die Attribute null sein.
        if (adr.suchbegr == null) adr.suchbegr = "";
        if (adr.name == null) adr.name = "";
        if (adr.strasse == null) adr.strasse = "";
        if (adr.landCd == null) adr.landCd = "";
        if (adr.plz == null) adr.plz = "";
        if (adr.ort == null) adr.ort = "";
        adr.suchbegr = adr.suchbegr.trim();
        adr.name = adr.name.trim();
        adr.strasse = adr.strasse.trim();
        adr.landCd = adr.landCd.trim();
        adr.plz = adr.plz.trim();
        adr.ort = adr.ort.trim();
        adresseAL.add(adr);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, sql.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
    adresseAL.trimToSize();
    return adresseAL;
  }

  /**
   * Findet alle Adressen mit den übergebenen Suchworten.
   * @param suchworte Suchworte
   * @return ArrayList mit AdresseVO-Objekten mit den gefundenen Adressen
   * @exception SQLException
   */
  public ArrayList select(ArrayList suchworte) throws SQLException
  {
    // Assertions
    // Mindestens ein Suchwort muss angegeben werden
    if (suchworte.size() == 0)
    {
      throw new IllegalArgumentException("Mindestens ein Suchwort muss angegeben werden");
    }

    int count = 0;
    ArrayList adresseAL = new ArrayList(1000);
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;

    StringBuffer select = new StringBuffer();
    StringBuffer from = new StringBuffer();
    StringBuffer where = new StringBuffer();

    select.append("SELECT a.adr_nr, trim(a.suchbegr) suchbegr, trim(a.name) name, trim(a.zeile_1) zeile_1, " +
                  "trim(a.zeile_2) zeile_2, trim(a.zeile_3) zeile_3, trim(a.zeile_4) zeile_4, ");
    select.append("trim(a.land_cd) land_cd, trim(a.plz) plz, trim(a.ort) ort ");
    from.append("FROM adresse a ");
    where.append( "WHERE " );
    try
    {
      parser.parse( suchworte, assembler );
    }
    catch ( Exception e )
    {
      LOG.error( "Error: " + e.getMessage(), e );
      throw new SQLException( e.getMessage() );
    }
    where.append( "contains (zeile_4 , '" + assembler.getValue() + "') >0 " );

    String sql = select.toString() + from.toString() + where.toString();
    LOG.debug("sql \n " + sql);

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());
      while (rs.next() && count < 1000)
      {
        AdresseVO adr = new AdresseVO();

        adr.setSuchbegr(rs.getString("suchbegr"));
        if (adr.getSuchbegrOrEmptyStr().equals("**********") || adr.getGesperrt() == 1)
        {
          // Logisch gelöschte Adressen ignorieren
          continue;
        }
        adr.setAdrNr(rs.getInt("adr_nr"));
        adr.setName(rs.getString("name"));
        adr.setZeile1(rs.getString("zeile_1"));
        adr.setZeile2(rs.getString("zeile_2"));
        adr.setZeile3(rs.getString("zeile_3"));
        adr.setZeile4(rs.getString("zeile_4"));
        adr.setLandCd(rs.getString("land_cd"));
        adr.setPlz(rs.getString("plz"));
        adr.setOrt(rs.getString("ort"));

        adresseAL.add(adr);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, sql);
      throw ex;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
    adresseAL.trimToSize();
    return adresseAL;
  }


  /**
   * Findet alle Adressen mit den übergebenen Kriterien.
   * @param name Name
   * @param ort Ort
   * @return ArrayList mit AdresseVO-Objekten mit den gefundenen Adressen
   * @exception SQLException
   */
  public ArrayList select(String name, String ort) throws SQLException
  {
    String query = null;
    // Assertions
    // Name und Ort müssen als Kriterien angegeben werden
    if (name.equals("") || ort.equals(""))
    {
      throw new IllegalArgumentException("Name und Ort müssen als Kriterien angegeben werden");
    }
    query = "SELECT a.adr_nr, trim(a.suchbegr) suchbegr, trim(a.name) name, trim(a.zeile_1) zeile_1, " +
            "trim(a.zeile_2) zeile_2, trim(a.zeile_3) zeile_3, trim(a.zeile_4) zeile_4, " +
            "trim(a.land_cd) land_cd, trim(a.plz) plz, trim(a.ort) ort " +
        "FROM adresse a " +
        "WHERE name = RPAD(?, 50, ' ') AND ort = RPAD(?, 50, ' ')";

    int count = 0;
    ArrayList adresseAL = new ArrayList(1000);
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, name);
      ps.setString(2, ort);
      rs = ps.executeQuery();
      while (rs.next() && count < 1000)
      {
        AdresseVO adr = new AdresseVO();

        adr.setSuchbegr(rs.getString("suchbegr"));
        if (adr.getSuchbegrOrEmptyStr().equals("**********"))
        {
          // Logisch gelöschte Adressen ignorieren
          continue;
        }
        adr.setAdrNr(rs.getInt("adr_nr"));
        adr.setName(rs.getString("name"));
        adr.setZeile1(rs.getString("zeile_1"));
        adr.setZeile2(rs.getString("zeile_2"));
        adr.setZeile3(rs.getString("zeile_3"));
        adr.setZeile4(rs.getString("zeile_4"));
        adr.setLandCd(rs.getString("land_cd"));
        adr.setPlz(rs.getString("plz"));
        adr.setOrt(rs.getString("ort"));
        adresseAL.add(adr);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + " name = " + name + ", ort = " + ort);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
    adresseAL.trimToSize();
    return adresseAL;
  }

  /**
   * Prüft die Existenz einer Adresse.
   * @param adrNr Adressnummer
   * @return true - Adresse existiert
   * @exception SQLException
   */
  public boolean exists(int adrNr) throws SQLException
  {
    String query = null;
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT adr_nr " +
              "FROM adresse WHERE adr_nr = ?";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setInt(1, adrNr);
      rs = ps.executeQuery();
      return rs.next();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, AdresseDAO.class, query + " adr_nr = " + adrNr);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }
}
