package ch.ige.adr.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;

import ch.ige.adr.vo.LandAVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Datenbankzugriff auf das Land
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Ueli Ryser
 * @version $Id: LandDAO.java,v 1.17 2006/12/11 15:17:28 nydegger Exp $
 */

public class LandDAO
{
  /**
   * Findet alle Länder, auf welche die Selektionskriterien zutreffen.
   * @param landCd Ländercode als Selektionskriterium
   * @param landBez Länderbezeichnungs als Selektionskriterium
   * @return ArrayList mit LandAVO-Objekten den gefundenen Länder
   * @throws SQLException
   */
  public ArrayList select(String landCd, String landBez) throws SQLException
  {
    // Assertions
    if (landCd == null) landCd = "";
    if (landBez == null) landBez = "";
    int count = 0;
    ArrayList al = new ArrayList(300);
    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;

    StringBuffer sql = new StringBuffer();

    sql.append("SELECT trim(b.land_cd) land_cd, trim(b.land_bez_d) land_bez_d FROM bland b");

    if (!landCd.equals(""))
    {
      sql.append( " WHERE b.land_cd LIKE '" + landCd + "%' ");
      if (!landBez.equals(""))
      {
        sql.append(" AND SUBSTR(LOWER(b.land_bez_d), 4, length (b.land_bez_d)-3) LIKE '" + landBez.toLowerCase() + "%' ");
      }
    }
    else
    {
      if (!landBez.equals(""))
      {
        sql.append(" WHERE SUBSTR(LOWER(b.land_bez_d), 4, length (b.land_bez_d)-3) LIKE '" + landBez.toLowerCase() + "%' ");
      }
    }

    try
    {
      con = DBHelper.getConnection(DBHelper.BAGIS);
      stmt =con.createStatement();
      rs = stmt.executeQuery(sql.toString());
      while (rs.next() && count < 1000)
      {
        LandAVO land = new LandAVO();
        land.landCd = rs.getString("land_cd");
        land.landBez = rs.getString("land_bez_d");
        if (land.landBez == null) land.landBez = "";
        al.add(land);
        count++;
      }
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, LandDAO.class, sql.toString());
      throw ex;
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
    return al;
  }

  /**
   * Prüft die Existenz eines Ländercodes.
   * @param landCd Ländercode, z.B. CH für Schweiz, F für Frankreich etc.
   * @return true - Ländercode existiert.
   * @throws SQLException
   */
  public boolean exists(String landCd) throws SQLException
  {
    String query = null;
    // Assertion
    if (landCd == null)
    {
      throw new IllegalArgumentException("landCd darf nicht null sein");
    }
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;
    try
    {
      query = "SELECT trim(land_cd) land_cd FROM land WHERE land_cd = RPAD(?, 3, ' ')";
      con = DBHelper.getConnection(DBHelper.BAGIS);
      ps = con.prepareStatement(query);
      ps.setString(1, landCd);
      rs = ps.executeQuery();
      return rs.next();
    }
    catch (SQLException ex)
    {
      LogLevelHelper.log(ex, LandDAO.class, query + " land_cd = " + landCd);
      throw ex;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }
}
