package ch.ige.adr.vo;

import java.io.Serializable;
import java.util.Date;

/**
 * Value Object der Adresse.
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author Ueli Ryser
 * @version $Id: AdresseVO.java,v 1.11 2006/12/11 16:27:00 nydegger Exp $
 */
public class AdresseVO implements Serializable
{

  // ----------------------------------------------------------------------------
  // const
  // ----------------------------------------------------------------------------

  /**
   * Länge Suchbegriff
   */
  public static final int SUCHBEGR_LENGTH = 10;

  /**
   * Länge Sprachcode
   */
  public static final int SPRACH_CD_LENGTH = 3;

  /**
   * Länge Name
   */
  public static final int NAME_LENGTH = 50;

  /**
   * Länge ZEILE_1
   */
  public static final int ZEILE_1_LENGTH = 50;

  /**
   * Länge ZEILE_2
   */
  public static final int ZEILE_2_LENGTH = 50;

  /**
   * Länge ZEILE_3
   */
  public static final int ZEILE_3_LENGTH = 50;

  /**
   * Länge ZEILE_4
   */
  public static final int ZEILE_4_LENGTH = 50;

  /**
   * Länge LAND_CD
   */
  public static final int LAND_CD_LENGTH = 3;

  /**
   * Länge PF_PLZ
   */
  public static final int PF_PLZ_LENGTH = 8;

  /**
   * Länge PLZ
   */
  public static final int PLZ_LENGTH = 8;

  /**
   * Länge ORT
   */
  public static final int ORT_LENGTH = 50;

  // ---------------------------------------------------------------------------
  // fields
  // ---------------------------------------------------------------------------

  private int adrNr;

  private String suchbegr = null;

  private String sprachCd = "D";

  private String anrCd = null;

  private String name = null;

  private String zeile1 = null;

  private String zeile2 = null;

  private String zeile3 = null;

  private String zeile4 = null;

  private String landCd = "CH";

  private String pfPlz = null;

  private String plz = null;

  private String ort = null;

  private String telefon = null;

  // private String telex; werden nicht mehr verwendet
  private String fax = null;

  private Date erstDat = new Date();

  private String erstBen = null;

  private Date geaendDat = null;

  private String geaendBen = null;

  private String uVersion = "!";

  private String plzZusatz = null;

  private int gesperrt;

  private int adrNrNeu;

  private int version;

  private StringBuffer truncateMsg = null;

  // ---------------------------------------------------------------------------
  // methods
  // ---------------------------------------------------------------------------

  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append( "  adrNr     = <" + this.adrNr + "> \n" );
    buf.append( "  suchbegr  = <" + this.suchbegr + "> \n" );
    buf.append( "  sprachCd  = <" + this.sprachCd + "> \n" );
    buf.append( "  anrCd     = <" + this.anrCd + "> \n" );
    buf.append( "  name      = <" + this.name + "> \n" );
    buf.append( "  zeile1    = <" + this.zeile1 + "> \n" );
    buf.append( "  zeile2    = <" + this.zeile2 + "> \n" );
    buf.append( "  zeile3    = <" + this.zeile3 + "> \n" );
    buf.append( "  zeile4    = <" + this.zeile4 + "> \n" );
    buf.append( "  landCd    = <" + this.landCd + "> \n" );
    buf.append( "  pfPlz     = <" + this.pfPlz + "> \n" );
    buf.append( "  plz       = <" + this.plz + "> \n" );
    buf.append( "  ort       = <" + this.ort + "> \n" );
    buf.append( "  telefon   = <" + this.telefon + "> \n" );
    buf.append( "  fax       = <" + this.fax + "> \n" );
    buf.append( "  erstDat   = <" + this.erstDat + "> \n" );
    buf.append( "  erstBen   = <" + this.erstBen + "> \n" );
    buf.append( "  geaendDat = <" + this.geaendDat + "> \n" );
    buf.append( "  geaendBen = <" + this.geaendBen + "> \n" );
    buf.append( "  uVersion  = <" + this.uVersion + "> \n" );
    buf.append( "  plzZusatz = <" + this.plzZusatz + "> \n" );
    buf.append( "  gesperrt  = <" + this.gesperrt + "> \n" );
    buf.append( "  adrNrNeu  = <" + this.adrNrNeu + "> \n" );
    buf.append( "  version   = <" + this.version + "> \n" );
    return buf.toString();
  }

  /**
   * kürzt die Attribute auf die maximal erlaubte Länge
   * 
   * @return Attribute mit korrekter Länge
   */
  public String truncateFields()
  {
    String ret = null;
    if ( truncateMsg == null )
    {
      truncateMsg = new StringBuffer();
    }
    if ( suchbegr != null && suchbegr.length() > SUCHBEGR_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "suchbegr", suchbegr.length(), SUCHBEGR_LENGTH ) );
      suchbegr = suchbegr.substring( 0, SUCHBEGR_LENGTH );
    }
    if ( sprachCd != null && sprachCd.length() > SPRACH_CD_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "sprachCd", sprachCd.length(), SPRACH_CD_LENGTH ) );
      sprachCd = sprachCd.substring( 0, SPRACH_CD_LENGTH );
    }
    if ( name != null && name.length() > NAME_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "name", name.length(), NAME_LENGTH ) );
      name = name.substring( 0, NAME_LENGTH );
    }
    if ( zeile1 != null && zeile1.length() > ZEILE_1_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "zeile1", zeile1.length(), ZEILE_1_LENGTH ) );
      zeile1 = zeile1.substring( 0, ZEILE_1_LENGTH );
    }
    if ( zeile2 != null && zeile2.length() > ZEILE_2_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "zeile2", zeile2.length(), ZEILE_2_LENGTH ) );
      zeile2 = zeile2.substring( 0, ZEILE_2_LENGTH );
    }
    if ( zeile3 != null && zeile3.length() > ZEILE_3_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "zeile3", zeile3.length(), ZEILE_3_LENGTH ) );
      zeile3 = zeile3.substring( 0, ZEILE_3_LENGTH );
    }
    if ( zeile4 != null && zeile4.length() > ZEILE_4_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "zeile4", zeile4.length(), ZEILE_4_LENGTH ) );
      zeile4 = zeile4.substring( 0, ZEILE_4_LENGTH );
    }
    if ( landCd != null && landCd.length() > LAND_CD_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "landCd", landCd.length(), LAND_CD_LENGTH ) );
      landCd = landCd.substring( 0, LAND_CD_LENGTH );
    }
    if ( pfPlz != null && pfPlz.length() > PF_PLZ_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "pfPlz", pfPlz.length(), PF_PLZ_LENGTH ) );
      pfPlz = pfPlz.substring( 0, PF_PLZ_LENGTH );
    }
    if ( plz != null && plz.length() > PLZ_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "plz", plz.length(), PLZ_LENGTH ) );
      plz = plz.substring( 0, PLZ_LENGTH );
    }
    if ( ort != null && ort.length() > ORT_LENGTH )
    {
      truncateMsg.append( composeTruncateMsg( "ort", ort.length(), ORT_LENGTH ) );
      ort = ort.substring( 0, ORT_LENGTH );
    }
    if ( truncateMsg != null && truncateMsg.length() > 0 )
    {
      ret = truncateMsg.toString();
    }
    truncateMsg = null;
    return ret;
  }

  private String composeTruncateMsg( String attrName, int attrSize, int maxSize )
  {
    return attrName + " ist zu gross: " + attrSize + " Zeichen statt " + maxSize + " Zeichen. " + attrName + " wird gekürzt.\n";
  }

  // ---------------------------------------------------------------------------
  // getter/setter methods
  // ---------------------------------------------------------------------------

  /**
   * gets the adrNr
   * 
   * @return adrNr
   */
  public int getAdrNr()
  {
    return adrNr;
  }

  /**
   * @return adrNrNeu
   */
  public int getAdrNrNeu()
  {
    return adrNrNeu;
  }

  /**
   * @return anrCd
   */
  public String getAnrCd()
  {
    return anrCd;
  }

  /**
   * @return anrCd
   */
  public String getAnrCdOrEmptyStr()
  {
    return ( anrCd == null ? "" : anrCd );
  }

  /**
   * @return erstBen
   */
  public String getErstBen()
  {
    return erstBen;
  }

  /**
   * @return getErstBenOrEmptyStr
   */
  public String getErstBenOrEmptyStr()
  {
    return ( erstBen == null ? "" : erstBen );
  }

  /**
   * @return getErstDat
   */
  public Date getErstDat()
  {
    return erstDat;
  }

  /**
   * @return fax
   */
  public String getFax()
  {
    return fax;
  }

  /**
   * @return getFaxOrEmptyStr
   */
  public String getFaxOrEmptyStr()
  {
    return ( fax == null ? "" : fax );
  }

  /**
   * @return geaendBen
   */
  public String getGeaendBen()
  {
    return geaendBen;
  }

  /**
   * @return getGeaendBenOrEmptyStr
   */
  public String getGeaendBenOrEmptyStr()
  {
    return ( geaendBen == null ? "" : geaendBen );
  }

  /**
   * @return geaendDat
   */
  public Date getGeaendDat()
  {
    return geaendDat;
  }

  /**
   * @return gesperrt
   */
  public int getGesperrt()
  {
    return gesperrt;
  }

  /**
   * @return landCd
   */
  public String getLandCd()
  {
    return landCd;
  }

  /**
   * @return getLandCdOrEmptyStr
   */
  public String getLandCdOrEmptyStr()
  {
    return ( landCd == null ? "" : landCd );
  }

  /**
   * @return name
   */
  public String getName()
  {
    return name;
  }

  /**
   * @return getNameOrEmptyStr
   */
  public String getNameOrEmptyStr()
  {
    return ( name == null ? "" : name );
  }

  /**
   * @return ort
   */
  public String getOrt()
  {
    return ort;
  }

  /**
   * @return getOrtOrEmptyStr
   */
  public String getOrtOrEmptyStr()
  {
    return ( ort == null ? "" : ort );
  }

  /**
   * @return pfPlz
   */
  public String getPfPlz()
  {
    return pfPlz;
  }

  /**
   * @return getPfPlzOrEmptyStr
   */
  public String getPfPlzOrEmptyStr()
  {
    return ( pfPlz == null ? "" : pfPlz );
  }

  /**
   * @return plz
   */
  public String getPlz()
  {
    return plz;
  }

  /**
   * @return getPlzOrEmptyStr
   */
  public String getPlzOrEmptyStr()
  {
    return ( plz == null ? "" : plz );
  }

  /**
   * @return plzZusatz
   */
  public String getPlzZusatz()
  {
    return plzZusatz;
  }

  /**
   * @return getPlzZusatzOrEmptyStr
   */
  public String getPlzZusatzOrEmptyStr()
  {
    return ( plzZusatz == null ? "" : plzZusatz );
  }

  /**
   * @return sprachCd
   */
  public String getSprachCd()
  {
    return sprachCd;
  }

  /**
   * @return getSprachCdOrEmptyStr()
   */
  public String getSprachCdOrEmptyStr()
  {
    return ( sprachCd == null ? "" : sprachCd );
  }

  /**
   * @return suchbegr
   */
  public String getSuchbegr()
  {
    return suchbegr;
  }

  /**
   * @return getSuchbegrOrEmptyStr
   */
  public String getSuchbegrOrEmptyStr()
  {
    return ( suchbegr == null ? "" : suchbegr );
  }

  /**
   * @return telefon
   */
  public String getTelefon()
  {
    return telefon;
  }

  /**
   * @return getTelefonOrEmptyStr
   */
  public String getTelefonOrEmptyStr()
  {
    return ( telefon == null ? "" : telefon );
  }

  /**
   * @return uVersion
   */
  public String getUVersion()
  {
    return uVersion;
  }

  /**
   * @return getUVersionOrEmptyStr
   */
  public String getUVersionOrEmptyStr()
  {
    return ( uVersion == null ? "" : uVersion );
  }

  /**
   * @return version
   */
  public int getVersion()
  {
    return version;
  }

  /**
   * @return zeile1
   */
  public String getZeile1()
  {
    return zeile1;
  }

  /**
   * @return getZeile1OrEmptyStr
   */
  public String getZeile1OrEmptyStr()
  {
    return ( zeile1 == null ? "" : zeile1 );
  }

  /**
   * @return zeile2
   */
  public String getZeile2()
  {
    return zeile2;
  }

  /**
   * @return getZeile2OrEmptyStr
   */
  public String getZeile2OrEmptyStr()
  {
    return ( zeile2 == null ? "" : zeile2 );
  }

  /**
   * @return zeile3
   */
  public String getZeile3()
  {
    return zeile3;
  }

  /**
   * @return zeile3
   */
  public String getZeile3OrEmptyStr()
  {
    return ( zeile3 == null ? "" : zeile3 );
  }

  /**
   * @return zeile4
   */
  public String getZeile4()
  {
    return zeile4;
  }

  /**
   * gets Zeile4
   * @return zeile4
   */
  public String getZeile4OrEmptyStr()
  {
    return ( zeile4 == null ? "" : zeile4 );
  }

  /**
   * sets Zeile4
   * @param zeile4
   */
  public void setZeile4( String zeile4 )
  {
    this.zeile4 = ( zeile4 != null && zeile4.equals( "" ) ? null : zeile4 );
  }

  /**
   * sets zeile3
   * @param zeile3
   */
  public void setZeile3( String zeile3 )
  {
    this.zeile3 = ( zeile3 != null && zeile3.equals( "" ) ? null : zeile3 );
  }

  /**
   * sets Zeile2
   * @param zeile2
   */
  public void setZeile2( String zeile2 )
  {
    this.zeile2 = ( zeile2 != null && zeile2.equals( "" ) ? null : zeile2 );
  }

  /**
   * sets Zeile1
   * @param zeile1
   */
  public void setZeile1( String zeile1 )
  {
    this.zeile1 = ( zeile1 != null && zeile1.equals( "" ) ? null : zeile1 );
  }

  /**
   * sets the version
   * @param version
   */
  public void setVersion( int version )
  {
    this.version = version;
  }

  /**
   * sets the uVersion
   * @param uVersion
   */
  public void setUVersion( String uVersion )
  {
    this.uVersion = uVersion;
  }

  /**
   * sets the phone
   * @param telefon
   */
  public void setTelefon( String telefon )
  {
    this.telefon = ( telefon != null && telefon.equals( "" ) ? null : telefon );
  }

  /**
   * sets the suchbegriff
   * @param suchbegr
   */
  public void setSuchbegr( String suchbegr )
  {
    this.suchbegr = ( suchbegr != null && suchbegr.equals( "" ) ? null : suchbegr );
  }

  /**
   * sets the sprackCd
   * @param sprachCd
   */
  public void setSprachCd( String sprachCd )
  {
    this.sprachCd = ( sprachCd != null && sprachCd.equals( "" ) ? null : sprachCd );
  }

  /**
   * sets the plzZusatz
   * @param plzZusatz
   */
  public void setPlzZusatz( String plzZusatz )
  {
    this.plzZusatz = ( plzZusatz != null && plzZusatz.equals( "" ) ? null : plzZusatz );
  }

  /**
   * sets the PfPlz
   * @param pfPlz
   */
  public void setPfPlz( String pfPlz )
  {
    this.pfPlz = ( pfPlz != null && pfPlz.equals( "" ) ? null : pfPlz );
  }

  /**
   * sets the ort
   * @param ort
   */
  public void setOrt( String ort )
  {
    this.ort = ( ort != null && ort.equals( "" ) ? null : ort );
  }

  /**
   * sets the name
   * @param name
   */
  public void setName( String name )
  {
    this.name = ( name != null && name.equals( "" ) ? null : name );
  }

  /**
   * sets the land_cd
   * @param landCd
   */
  public void setLandCd( String landCd )
  {
    this.landCd = ( landCd != null && landCd.equals( "" ) ? null : landCd );
  }

  /**
   * sets gesperrt
   * @param gesperrt
   */
  public void setGesperrt( int gesperrt )
  {
    this.gesperrt = gesperrt;
  }

  /**
   * sets the geaendDat
   * @param geaendDat
   */
  public void setGeaendDat( Date geaendDat )
  {
    this.geaendDat = geaendDat;
  }

  /**
   * sets the geaendBen
   * @param geaendBen
   */
  public void setGeaendBen( String geaendBen )
  {
    this.geaendBen = ( geaendBen != null && geaendBen.equals( "" ) ? null : geaendBen );
  }

  /**
   * sets the fax
   * @param fax
   */
  public void setFax( String fax )
  {
    this.fax = ( fax != null && fax.equals( "" ) ? null : fax );
  }

  /**
   * sets the erstDat
   * @param erstDat
   */
  public void setErstDat( Date erstDat )
  {
    this.erstDat = erstDat;
  }

  /**
   * sets the erstBen
   * @param erstBen
   */
  public void setErstBen( String erstBen )
  {
    this.erstBen = ( erstBen != null && erstBen.equals( "" ) ? null : erstBen );
  }

  /**
   * sets the anrCd
   * @param anrCd
   */
  public void setAnrCd( String anrCd )
  {
    this.anrCd = ( anrCd != null && anrCd.equals( "" ) ? null : anrCd );
  }

  /**
   * sets the adrNrNeu
   * @param adrNrNeu
   */
  public void setAdrNrNeu( int adrNrNeu )
  {
    this.adrNrNeu = adrNrNeu;
  }

  /**
   *  sets the adrNr
   * @param adrNr
   */
  public void setAdrNr( int adrNr )
  {
    this.adrNr = adrNr;
  }

  /**
   * sets the PLZ
   * @param plz
   */
  public void setPlz( String plz )
  {
    this.plz = ( plz != null && plz.equals( "" ) ? null : plz );
  }
}
