package ch.ige.art.client;

import java.security.PrivilegedAction;

import javax.swing.JOptionPane;

import org.apache.log4j.Logger;

import ch.ige.art.client.util.UserMessage;
import ch.ige.art.util.XxDomain;
import ch.ige.authz.client.AuthMain;
import ch.ige.main.client.MainController;
import ch.ige.main.client.MainRootPane;
import ch.ige.main.client.SplashScreen;
import ch.ige.util.client.ClientLogInitalizer;
import ch.ige.util.client.IgeLookAndFeel;

/**
 * <p>Überschrift: Starterklasse</p>
 * <p>Beschreibung: Startpunkt der Appliaktion für die Artikelpflege.<br>
 *  Menubaum der Applikation wird geöffnet.<br>
 *  Alle Domainwerte aus xx_Domain werden eingelesen und zwischengespeichert</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: MainArtikel.java,v 1.24 2009/03/16 11:59:13 heinzer Exp $
 */

public class MainArtikel implements PrivilegedAction<Object>
{
  // Logging
  private static final Logger LOG = Logger.getLogger( "ch.ige.art" );

  private static UserMessage userMessage = new UserMessage();

  // Properties
  private static final String MENU_CONFIG = "ige.menu.configuration";

  private static final String PROPERTY_XML = "/res/ArtMenuBaum.xml";

  // Logging
  static
  {
    ClientLogInitalizer.initalizeLogging();
  }

  /**
   * Dies ist die eigentliche Main method welche nach dem SSO ausgeführt wird.
   * @return don't care
   */
  public Object run()
  {
    // Splash screen anzeigen
    SplashScreen splashScreen = new SplashScreen();

    MainController mc = new MainController();
    MenuPaneArtikel mp = new MenuPaneArtikel( mc, MainRootPane.DETAIL );

    splashScreen.setVisible( false );
    splashScreen.dispose();

    mc.first( mp );
    return null;
  }

  /**
   * Properties werden gesetzt. Alle Domainwerte werden in den Client geladen
   * Aufruf der Main Klasse aus dem Package ch.ige.util.client, wo die Steuerung
   * zum Fenster öffnen und schliessen sowie das Beenden der Applikation implementiert ist.
   * Als Parameter wird das StartPane mitgegeben, das vom Typ MainRootPane sein muss
   *
   * @param args  Startargumente
   */
  public static void main( String[] args )
  {
    MainArtikel mainArtikel = new MainArtikel();

    try
    {
      mainArtikel.loadProperties();
      IgeLookAndFeel.setLF();
      XxDomain.init();
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      System.exit( -1 );
    }

    AuthMain.run( new MainArtikel() );
  }

  /**
   * System-Properties werden gesetzt
   */
  private void loadProperties()
  {
    try
    {
      System.setProperty( MENU_CONFIG, PROPERTY_XML );
    }
    catch ( Exception ex )
    {
      String message = "Properties-Einstellungen konnten nicht geladen werden";
      LOG.warn( message );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
    }
  }

}
