package ch.ige.art.client.art;


import java.awt.*;
import javax.swing.*;

import ch.ige.art.client.util.*;
import ch.ige.art.vo.*;
import ch.ige.main.client.*;



/**
 * <p>Überschrift: Mediator für Artikel-Wizard</p>
 * <p>Beschreibung: Der ArtWizardMediator steuert die Anzeige der einzelnen Panels
 * beim Erfass-/Bearbeitungsprozess eines Artikels
 * Die Attribute der Oberklasse alPanel, maxPanel und title werden gesetzt
 * und die Methoden back(), cancel(), forward und finish() überschrieben.
 * <p>Copyright: Copyright (c) 2002 </p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: ArtWizardAbsolute.java,v 1.4 2007/12/17 16:10:06 ruegsegger Exp $
 */

public class ArtWizardAbsolute extends ArtWizardMediator
{
  /**
   * Initialisiert Panels<br>
   * @param parent Frame
   */
  public ArtWizardAbsolute(Frame parent)
  {
    super(parent);
  }

  /**
   * Initialisiert Panel<br>
   * Prüft, ob Folge und Preispanel dynamisch hinzugefügt werden müssen
   */
  protected void initializePanel()
  {
    alPanel.add( new WizardMediator.WizardPanel( pnlArtGrunddaten, false ) );
    alPanel.add( new WizardMediator.WizardPanel( pnlArtBagis1, false ) );
    alPanel.add( new WizardMediator.WizardPanel( pnlArtBagis2, false ) );
    alPanel.add( new WizardMediator.WizardPanel( pnlArtTexte, true ) );

    checkPreise();
    checkFolgeArtikel();
    maxPanel = alPanel.size();
    setTitle(false);
  }

  /**
   * Setzt Titel des Wizards<br>
   * Wird beim Wechseln der Panels angepasst
   * @param isForward boolean
   * <br>       - true:  forward Button wurde geklickt</br>
   * <br>       - false: back    Button wurde geklickt</br>
   */
  private void setTitle(boolean isForward)
  {
    if(isForward)
    {
      switch (pos)
      {
        case 0:
          title = "Bagisdaten";
          break;
        case 1:
          title = "Bagisdaten";
          break;
        case 2:
          title = "Texte";
          break;
        case 3:
          if(artikelGuiAVO.kzGratis.equals("1"))
          {
            title = "Folgeereignis";
          }
          else
          {
            title = "Preise";
          }
          break;
        case 4:
          if (alPanel.size() == 6)
          {
            title = "Folgeereignis";
          }
          break;
      }
    }
    else
    {
      switch (pos)
      {
        case 0:
          title = "Grunddaten";
          break;
        case 1:
          title = "Grunddaten";
          break;
        case 2:
          title = "Bagisdaten";
          break;
        case 3:
          title = "Bagisdaten";
          break;
        case 4:
          title = "Texte";
          break;
        case 5:
          title = "Preise";
          break;
      }
    }
  }

  /**
   * Artikel Erstellen
   * @return  Artikelnummer wenn Fertigstellen ok
   *          null bei Abbrechen
   */
  public String newArt()
  {
    artikelGuiAVO = new ArtikelGuiAVO();
    artikelGuiAVO.status = "I";

    pnlArtGrunddaten.setData(artikelGuiAVO);
    init();
    return finish ? artNr : null;
  }

  /**
   * Artikel Duplizieren
   * Duplikat Artikelnummer löschen und Stati des Artikels,
   * Texte, Preise und Alternativartikel auf "I" setzen
   * @return  Artikelnummer wenn Fertigstellen ok
   *          null bei Abbrechen
   */
  public String dupliArt(String artNr)
  {
    read(artNr);

    artikelGuiAVO.status = "I";
    artikelGuiAVO.artikelVersion = "!";
    artikelGuiAVO.artBezVersion = "!";
    artikelGuiAVO.artNr = null;

    //Stati artpreisVO auf "I" setzen
    if(artikelGuiAVO.artpreisVO != null)
    {
      for (int i = 0; i < artikelGuiAVO.artpreisVO.length; i++)
      {
        artikelGuiAVO.artpreisVO[i].status = "I";
        artikelGuiAVO.artpreisVO[i].uVersion = "!";
      }
    }
    //Stati artextVO auf "I" setzen
    if(artikelGuiAVO.artextVO != null)
    {
      for (int i = 0; i < artikelGuiAVO.artextVO.length; i++)
      {
        artikelGuiAVO.artextVO[i].status = "I";
        artikelGuiAVO.artextVO[i].uVersion = "!";
      }
    }
    //Stati altartVO auf "I" setzen
    if(artikelGuiAVO.altArtVO != null)
    {
      for (int i = 0; i < artikelGuiAVO.altArtVO.length; i++)
      {
        artikelGuiAVO.altArtVO[i].status = "I";
        artikelGuiAVO.altArtVO[i].uVersion = "!";
      }
    }

    pnlArtGrunddaten.setData(artikelGuiAVO);
    init();
    return finish ? this.artNr : null;
  }

  /**
   * Datensatz bearbeiten
   * @return  true wenn Fertigstellen ok
   *          false bei Abbrechen
   */
  public boolean editArt(String artNr)
  {
    read(artNr);
    artikelGuiAVO.status = "U";

    pnlArtGrunddaten.setData(artikelGuiAVO);
    init();
    return finish;
  }


  /**
   * Speichert Daten des Panels in VO und liest Daten des VO in nächstes Panel ein<br>
   * Ist der Artikel kostenpflichtig (checkbox kzGratis) wird das Panel PnlPreise hinzugefügt<br>
   * Sind Folgeartikel vorhanden wird das Panel PnlFolge hinzugefügt
   * fehler == null --> keine Fehler Artikel kann gespeicher werden
   * fehler.equals("") --> Fehler aufgetreten, und angezeigt Panel nicht wechseln
   * anderenfalls den im fehler enthaltenen String anzeigen
   *
   * @return true  - nächstes Panel anzeigen<br>
   *         false - auf aktuellem Panel bleiben, z.B. wegen Validierungsfehler
   */
  public boolean forward()
  {
    String fehler = null;
    WizardMediator.WizardPanel wizardPanel = null;

    wizardPanel = (WizardMediator.WizardPanel)alPanel.get(pos);

    fehler = ((PnlArtikelWizard)wizardPanel.pnlWizard).validForward(); // Daten aus Panel validieren
    if(fehler != null && ! fehler.equals(""))
    {
      JOptionPane.showMessageDialog(dlgWizard, fehler, "Ungültige Eingabe",
                                    JOptionPane.INFORMATION_MESSAGE);
      return false;
    }
    else if(fehler == null)
    {
      artikelGuiAVO = ((PnlArtikelWizard)wizardPanel.pnlWizard).getData();  //Daten holen und in artikelGuiAVO abspeichern

      //wenn auf erstem Panel prüfen, ob Panels PnlFolge und PnlPreis hinzugefügt werden müssen
      //ebenfalls muss der Finish Button auf dem letzten Panel aktiviert werden
      if(pos == 0)
      {
        checkPreise();
        checkFolgeArtikel();
        activateFinish();
      }

      // Daten für nächsten Panels von artikelGuiAVO in Panel laden
      wizardPanel = (WizardMediator.WizardPanel)alPanel.get(pos+1);
      ((PnlArtikelWizard)wizardPanel.pnlWizard).setData(artikelGuiAVO);
      setTitle(true);
      return true;
    }
    return false; // fehler.equals("")
  }

  /**
   * überschreibt protected Methode der Oberklasse<br>
   * Speichert Daten des Panels in VO und liest Daten des VO in vorheriges Panel ein<br>
   * @return true  - nächstes Panel anzeigen<br>
   *         false - auf aktuellem Panel bleiben, z.B. wegen Validierungsfehler
   */
  public boolean back()
  {
    String fehler = null;
    WizardMediator.WizardPanel wizardPanel = null;

    wizardPanel = (WizardMediator.WizardPanel)alPanel.get(pos);

    fehler = ((PnlArtikelWizard)wizardPanel.pnlWizard).validBackward(); // Daten validieren
    if(fehler != null && ! fehler.equals(""))
    {
      JOptionPane.showMessageDialog(dlgWizard, fehler, "Ungültige Eingabe",
                                    JOptionPane.INFORMATION_MESSAGE);
      return false;
    }
    else if(fehler == null)
    {
      artikelGuiAVO = ((PnlArtikelWizard)wizardPanel.pnlWizard).getData();  //Daten in artikelGuiAVO abspeichern
      wizardPanel = (WizardMediator.WizardPanel)alPanel.get(pos-1);   // Daten in vorangehendes Panels laden
      ((PnlArtikelWizard)wizardPanel.pnlWizard).setData(artikelGuiAVO);
      setTitle(false);
      return true;
    }
    return false; // fehler.equals("")
  }

  /**
   * überschreibt protected Methode der Oberklasse</br>
   * Speichert Daten des Panels in VO und in Datenbank
   * fehler == null --> keine Fehler Artikel kann gespeicher werden
   * fehler.equals("") --> Fehler aufgetreten und angezeigt, Panel nicht wechseln
   * anderenfalls den im fehler enthaltenen String anzeigen
   *
   * @return true  - nächstes Panel anzeigen<br>
   *         false - auf aktuellem Panel bleiben, z.B. wegen Validierungsfehler
   */
  public boolean finish()
  {
    String fehler = null;
    WizardMediator.WizardPanel wizardPanel = null;

    wizardPanel = (WizardMediator.WizardPanel)alPanel.get(pos);
    fehler = ((PnlArtikelWizard)wizardPanel.pnlWizard).validForward();

    if(fehler != null)
    {
      if(!fehler.equals(""))
      {
        JOptionPane.showMessageDialog(dlgWizard, fehler, "Ungültige Eingabe",
                                      JOptionPane.INFORMATION_MESSAGE);
      }
      return finish = false;
    }

    //Daten in artikelGuiAVO abspeichern
    artikelGuiAVO = ((PnlArtikelWizard)wizardPanel.pnlWizard).getData();

    // kostenpflichtiger Artikel muss über Preise verfügen
    if (artikelGuiAVO.kzGratis.equals("0") && artikelGuiAVO.artpreisVO == null)
    {
      fehler =  "Für einen kostenpflichtigen Artikel muss ein Preis erfasst werden!\n" +
                "Wenn es sich um einen Gratisartikel handelt wählen Sie auf dem ersten\n" +
                "Fenster das Feld Gratisartikel an.\n";

      JOptionPane.showMessageDialog(dlgWizard, fehler, "Ungültige Eingabe",
                                    JOptionPane.INFORMATION_MESSAGE);
      return finish = false;
    }

    // Artikel mit Folgeartikel muss über Folgeartikel verfügen
    if ((artikelGuiAVO.altNrCd.equals("1") && artikelGuiAVO.altArtVO == null)
        || (artikelGuiAVO.altNrCd.equals("1") && artikelGuiAVO.altArtVO != null && artikelGuiAVO.altArtVO[0].status.equals("D")))
        {
          fehler =  "Für diesen Artikel muss ein Folgeereignis erfasst werden!\n" +
          "Wenn es sich um einen Artikel ohne Folgeereignis handelt,\n" +
                    "wählen Sie auf dem ersten Fenster das Feld Folgeartikel ab.\n";
          JOptionPane.showMessageDialog(null, fehler, "Ungültige Eingabe",
                                        JOptionPane.INFORMATION_MESSAGE);
          return finish = false;
    }
    this.artNr = artikelGuiAVO.artNr;

    return super.finish();
  }


  /**
   * Aktiviert den Finish Button auf dem letzten Panel<br>
   * Auf allen anderen Panels wird der Finish Button deaktiviert
   */
  private void activateFinish()
  {
    WizardMediator.WizardPanel wizardPanel = null;

    for (int i = 3; i < alPanel.size()-1; i++)
    {
      wizardPanel = (WizardMediator.WizardPanel)alPanel.get(i);
      wizardPanel.finish = false;
    }
    wizardPanel = (WizardMediator.WizardPanel)alPanel.get(alPanel.size()-1);
    wizardPanel.finish = true;
  }

  /**
   * Überprüft, ob das Panel Preise hinzugefügt oder entfernt werden muss<br>
   * ist Artikel kostenpflichtig wird das Panel zur ArrayList hinzugefügt<br>
   * ist Artikel gratis wird das Panel aus der ArrayList entfernt<br>
   * Panel PnlPreise befindet sich immer an Position 4 wenn vorhanden<br>
   */
  private void checkPreise()
  {
    WizardMediator.WizardPanel wizardPanel = null;

    if(alPanel.size() <= 4)
    {
      if(artikelGuiAVO.kzGratis.equals("0"))
      {
        alPanel.add(4, new WizardMediator.WizardPanel(pnlArtPreise, true));
      }
    }
    else //es sind min 5 Panels, also könnte PnlPreise vorhanden sein
    {
      wizardPanel = (WizardMediator.WizardPanel)alPanel.get(4);
      if(artikelGuiAVO.kzGratis.equals("0")) //kostenpflichtig
      {
        if(!((PnlArtikelWizard)wizardPanel.pnlWizard instanceof PnlArtPreise))
        {
          alPanel.add(4, new WizardMediator.WizardPanel(pnlArtPreise, true));
        }
      }
      else //gratis
      {
        if((PnlArtikelWizard)wizardPanel.pnlWizard instanceof PnlArtPreise)
        {
          alPanel.remove(4);
        }
      }
    }
    maxPanel = alPanel.size();
  }

  /**
   * Überprüft, ob das Panel FolgeArtikel hinzugefügt oder entfernt werden muss<br>
   * sind Folgeartikel vorhanden wird das Panel zur ArrayList hinzugefügt<br>
   * sind keine Folgeartikel vorhanden wird das Panel aus der ArrayList entfernt<br>
   * Panel PnlFolge befindet sich entweder an Poition 4 oder 5 wenn vorhanden
   */
  private void checkFolgeArtikel()
  {
    int found = -1;
    WizardMediator.WizardPanel wizardPanel = null;

    if(alPanel.size() <= 4)
    {
      if(artikelGuiAVO.altNrCd.equals("1"))
      {
        alPanel.add(new WizardMediator.WizardPanel(pnlArtFolge, true));
      }
    }
    else //es sind min 5 Panels, also könnte PnlFolge vorhanden sein
    {
      for (int i = 4; i < alPanel.size(); i++)
      {
        wizardPanel = (WizardMediator.WizardPanel)alPanel.get(i);
        if((PnlArtikelWizard)wizardPanel.pnlWizard instanceof PnlArtFolge)
        {
          found = i;
        }
      }
      if(artikelGuiAVO.altNrCd.equals("1"))  //Folgeartikel vorhanden
      {
        if(found == -1)   //noch nicht vorhanden
        {
          alPanel.add(new WizardMediator.WizardPanel(pnlArtFolge, true));
        }
      }
      else // kein Folgeartikel
      {
        if(found > -1)
        {
          alPanel.remove(found);
        }
      }
    }
    maxPanel = alPanel.size();
  }
}

