package ch.ige.art.client.art;

import javax.swing.table.AbstractTableModel;

import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.AltArtVO;

/**
 * <p>Überschrift: Table Model für die Folgeartikel</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Folgeartikel im Anzeigemodus verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: AtmArtFolge.java,v 1.10 2008/07/02 11:00:20 heinzer Exp $
 */

public class AtmArtFolge extends AbstractTableModel
{
  private final String[] columnNames = new String[] { "Folgeereignis", "Priorität" };

  private AltArtVO[] tableData = new AltArtVO[] {};

  public AtmArtFolge()
  {
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return columnNames.length;
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  public String getColumnName( int column )
  {
    return columnNames[column];
  }

  /**
   * Werte der angegebenen Spalte werden mit den standardmässigen
   * Darstellungsobjekten ausgestattet
   *
   * @param column Spalte
   * @return gibt die Klasse der Column zurück
   */
  public Class getColumnClass( int column )
  {
    return getValueAt( 0, column ).getClass();
  }

  /**
   * Gibt den Wert einer Zelle zurück
   * @param row Zeile
   * @param col Spalte
   * @return Wert der entsprechenden Zelle
   */
  public Object getValueAt( int row, int col )
  {
    switch( col )
    {
      case 0:
        return new Integer( tableData[row].altArtNr );
      case 1:
        String bezPriorCd = XxDomain.getBez( tableData[row].priorCd, XxDomainFactory.PRIOR_CD );
        return ( tableData[row].priorCd + ", " + bezPriorCd );
    }
    return null;
  }

  /**
   * Setzt die Daten ins TableModel
   * @param data Alle Folgeereignisse eines Artikels gekapselt in einem AltArtVO
   */
  public void setData( AltArtVO[] data )
  {
    tableData = data;
    fireTableDataChanged();
  }

  /**
   * Daten in TableModel löschen
   */
  public void clear()
  {
    fireTableDataChanged();
  }
}
