package ch.ige.art.client.art;

import javax.swing.table.AbstractTableModel;

import ch.ige.statistikartikel.util.ArticleStatisticInfoVO;

/**
 * <p>Überschrift: Table Model für die Folgeartikel</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Folgeartikel im Anzeigemodus verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: AtmStatistik.java,v 1.1 2008/05/07 13:25:01 ruegsegger Exp $
 */

public class AtmStatistik extends AbstractTableModel
{
  private final String[] columnNames = new String[] { "Statistik", "Beschreibung" };

  private ArticleStatisticInfoVO[] tableData = new ArticleStatisticInfoVO[] {};

  public AtmStatistik() throws Exception
  {
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return columnNames.length;
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  public String getColumnName( int column )
  {
    return columnNames[column];
  }

  /**
   * Werte der angegebenen Spalte werden mit den standardmässigen
   * Darstellungsobjekten ausgestattet
   *
   * @param column Spalte
   * @return gibt die Klasse der Column zurück
   */
  public Class getColumnClass( int column )
  {
    return getValueAt( 0, column ).getClass();
  }

  /**
   * Gibt den Wert einer Zelle zurück
   * @param row Zeile
   * @param col Spalte
   * @return Wert der entsprechenden Zelle
   */
  public Object getValueAt( int row, int col )
  {
    switch( col )
    {
      case 0:
        return tableData[row].getName() == null ? "" : tableData[row].getName();
      case 1:
        return tableData[row].getDescription() == null ? "" : tableData[row].getDescription();
    }
    return null;
  }

  /**
   * Setzt die Daten ins TableModel
   * @param data Alle Folgeereignisse eines Artikels gekapselt in einem AltArtVO
   */
  public void setData( ArticleStatisticInfoVO[] data )
  {
    tableData = data;
    fireTableDataChanged();
  }

  /**
   * Daten in TableModel löschen
   */
  public void clear()
  {
    fireTableDataChanged();
  }
}
