package ch.ige.art.client.art;

import java.util.ArrayList;
import java.util.Vector;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.DtmArtikelWizard;
import ch.ige.art.util.XxDomain;
import ch.ige.art.vo.AltArtVO;

/**
 * <p>Überschrift: DefaultTableModel der Folgeereignisse</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Folgeereignisse verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DtmArtFolge.java,v 1.8 2008/07/02 11:00:20 heinzer Exp $
 */

public class DtmArtFolge extends DtmArtikelWizard
{
  private ArrayList alDelAltArtVO = new ArrayList(); // zu löschende Rows

  /**
   * xxDomainWerte werden geladen und Spaltenüberschriften werden gesetzt
   *
   * @throws Exception falls Domainwerte nicht geladen werden können
   */
  public DtmArtFolge() throws Exception
  {
    super();
    columnIdentifiers.addElement( "Folgeereignis" );
    columnIdentifiers.addElement( "Priorität" );
    columnIdentifiers.addElement( "Status" );
    columnIdentifiers.addElement( "Version" );
  }

  /**
   * Editierbarkeit der persistenten Daten verhindern,
   * die Bestandteil des PrimaryKeys sind
   *
   * @param row selektierte Zeile
   * @param col selektierte Spalte
   * @return true falls Zellen editierbar sind
   */
  public boolean isCellEditable( int row, int col )
  {
    if ( col == 0 && row < getPersistentRows() )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Setzt einen Zeilenstatus nachdem der mutierte Wert in die Zelle geschrieben wurde.<br>
   * Mutierte persistente Zeilen werden in der Spalte Status mit einem
   * "U" = Update gekennzeichnet.
   *
   * @param value Wert der Zelle
   * @param row   aktuelle Zeile
   * @param col   aktuelle Spalte
   */
  public void setValueAt( Object value, int row, int col )
  {
    super.setValueAt( value, row, col );

    if ( row < getPersistentRows() )
    {
      super.setValueAt( "U", row, 2 );
    }
  }

  /**
   * Tabellendaten werden gesetzt.<br>
   * Daten werden in Form eines ValueObjects übergeben.
   * Daten des VOs werden augesplittet und in den Datenhaltungsvector(dataVector)
   * des Tabllenmodels geschrieben.
   *
   * @param data Alle Folgeereignisse eines Artikels
   */
  public void setData( AltArtVO[] data )
  {
    dataVector.clear();
    int persitentRows = 0;

    if ( data != null )
    {
      for ( int i = 0; i < data.length; i++ )
      {
        AltArtVO altArtVO = data[i];

        if ( !altArtVO.status.equals( "D" ) )
        {
          Vector altArt = new Vector();

          altArt.add( altArtVO.altArtNr );
          altArt.add( new ComboBoxObject( altArtVO.priorCd, XxDomain.getBez( "prior_cd", altArtVO.priorCd ) ) );
          altArt.add( altArtVO.status );
          //altArt.add(new Integer(altArtVO.version));
          altArt.add( altArtVO.uVersion );

          dataVector.add( altArt );
        }

        // Anzahl Rows ermitteln die persistent sind
        if ( altArtVO.status.equals( "U" ) || altArtVO.status.equals( "" ) )
        {
          persitentRows++;
        }
      }
      setPersistentRows( persitentRows );
      fireTableDataChanged();
    }
  }

  /**
   * Gibt die Tabellendaten in Form eines Arrays von AltArtVOs zurück
   *
   * @return Tabellendaten gekapselt in einem Array von AltArtVOs
   */
  public AltArtVO[] getData()
  {
    int size = dataVector.size() + alDelAltArtVO.size();

    if ( size > 0 )
    {
      AltArtVO[] arAltArtVO = new AltArtVO[size];
      int i = 0;

      for ( i = 0; i < dataVector.size(); i++ )
      {
        Vector vec = new Vector();

        arAltArtVO[i] = new AltArtVO();
        vec = (Vector) dataVector.get( i );

        arAltArtVO[i].altArtNr = (String) vec.get( 0 );
        ComboBoxObject cbo = (ComboBoxObject) vec.get( 1 );
        arAltArtVO[i].priorCd = cbo.key == null ? "" : cbo.key;
        arAltArtVO[i].status = (String) vec.get( 2 );
        //arAltArtVO[i].version = ((Integer)vec.get(3)).intValue();
        arAltArtVO[i].uVersion = (String) vec.get( 3 );
      }
      // zu löschende Rows dem Array hinzufügen
      for ( int j = 0; j < alDelAltArtVO.size(); j++, i++ )
      {
        arAltArtVO[i] = (AltArtVO) alDelAltArtVO.get( j );
      }
      return arAltArtVO;
    }
    return null;
  }

  /**
   * Für eine neue Zeile werden die Initialwerte pro Zelle gesetzt
   * Spalte Status wird mit I=Insert initialisiert
   *
   * @return gibt eine neue Zeile mit den initialisierten Zellen zurück
   */
  public Vector getNewRow()
  {
    Vector vec = new Vector();
    vec.setSize( 4 );
    vec.add( 0, "" );
    vec.add( 1, new ComboBoxObject( "0", XxDomain.getBez( "prior_cd", "0" ) ) );
    vec.add( 2, "I" );
    //vec.add(3, new Integer(0));
    vec.add( 3, "!" );
    return vec;
  }

  /**
   * Speichert die Zeile die gelöscht werden soll in einem Vector, damit
   * die Daten noch vorhanden sind wenn sie zu einem späteren Zeitpunkt
   * in der Datenbank gelöscht werden wollen
   *
   * @param iRow Index der Zeile die gelöscht werden soll
   */
  public void saveDeletedRow( int iRow )
  {
    AltArtVO altArtVO = new AltArtVO();

    altArtVO.altArtNr = (String) getValueAt( iRow, 0 );
    ComboBoxObject cbo = (ComboBoxObject) getValueAt( iRow, 1 );
    altArtVO.priorCd = cbo.key == null ? "" : cbo.key;
    altArtVO.status = "D";
    //altArtVO.version = ((Integer)getValueAt(iRow, 3)).intValue();
    altArtVO.uVersion = (String) getValueAt( iRow, 3 );

    alDelAltArtVO.add( altArtVO );
  }
}
