package ch.ige.art.client.art;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.SystemColor;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.util.List;

import javax.swing.DefaultCellEditor;
import javax.swing.JCheckBox;
import javax.swing.JComboBox;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.border.TitledBorder;
import javax.swing.table.TableColumn;

import org.apache.log4j.Logger;

import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.PnlArtikelWizard;
import ch.ige.art.client.util.TblArtikelWizard;
import ch.ige.art.client.util.TblNavToolBar;
import ch.ige.art.client.util.UserMessage;
import ch.ige.art.client.util.editoren.BtnEditor;
import ch.ige.art.client.util.renderer.BtnRenderer;
import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.ArtikelGuiAVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.arthelp.HelpUtil;

/**
 * <p>Überschrift: Artikel-Texte</p>
 * <p>Beschreibung: Panel mit Daten der Artikeltexte</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: PnlArtTexte.java,v 1.27 2008/07/02 11:00:20 heinzer Exp $
 */

public class PnlArtTexte extends PnlArtikelWizard
{
  private DtmArtTexte dtmArtTexte;

  private ArtikelGuiAVO artikelGuiAVO;

  private TblArtikelWizard tblTexte;

  private TblNavToolBar tblNavToolBar;

  // Logging
  private static final Logger LOG = Logger.getLogger( PnlArtTexte.class );

  private UserMessage userMessage = new UserMessage();

  private JPanel pnlTexte = new JPanel();

  private JPanel pnlNav = new JPanel();

  private JScrollPane scpTexte = new JScrollPane();

  private BorderLayout borderLayout1 = new BorderLayout();

  private BorderLayout borderLayout2 = new BorderLayout();

  private BorderLayout borderLayout3 = new BorderLayout();

  /**
   * Konstruktor
   * ruft jbInit() auf
   * holt Instanz von xxDomain um Werte aus xx_domain lesen zu können<br>
   */
  public PnlArtTexte()
  {
    try
    {
      jbInit();
      initHelp();
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
      return;
    }
  }

  /**
   * Initialisierung
   * @throws Exception
   */
  private void jbInit() throws Exception
  {
    dtmArtTexte = new DtmArtTexte();
    tblTexte = new TblArtikelWizard( dtmArtTexte );
    tblNavToolBar = new TblNavToolBar( tblTexte );

    this.setLayout( borderLayout1 );
    this.setPreferredSize( new Dimension( 530, 415 ) );
    this.add( pnlTexte, BorderLayout.CENTER );

    pnlTexte.setLayout( borderLayout2 );
    pnlNav.setLayout( borderLayout3 );
    pnlNav.setPreferredSize( new Dimension( 237, 40 ) );

    scpTexte.setBorder( new TitledBorder( "Texte" ) );

    pnlTexte.add( scpTexte, BorderLayout.CENTER );
    pnlTexte.add( pnlNav, BorderLayout.NORTH );
    pnlNav.add( tblNavToolBar, BorderLayout.EAST );
    scpTexte.getViewport().add( tblTexte, null );
    scpTexte.setOpaque( false );

    tblNavToolBar.setEnabled( false );
    tblNavToolBar.setBorderPainted( false );

    // Combobox arttextart_cd
    JComboBox cbxArttextartCd = new JComboBox();
    List<XxDomainVO> listDomainTx = XxDomain.getDomains( XxDomainFactory.ARTTEXTART_CD );
    for ( XxDomainVO xxDomainVO: listDomainTx )
    {
      cbxArttextartCd.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
    }

    // Combobox sprache
    JComboBox cbxSprache = new JComboBox();
    List<XxDomainVO> listDomain = XxDomain.getDomains( XxDomainFactory.SPRACHE );
    for ( XxDomainVO xxDomainVO: listDomain )
    {
      cbxSprache.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
    }

    cbxArttextartCd.setBackground( SystemColor.controlLtHighlight );
    cbxSprache.setBackground( SystemColor.controlLtHighlight );

    // Eigenschaften der Tabelle & Spalten setzen
    TableColumn textartColumn = tblTexte.getColumnModel().getColumn( 0 );
    TableColumn spracheColumn = tblTexte.getColumnModel().getColumn( 1 );
    TableColumn statusColumn = tblTexte.getColumnModel().getColumn( 3 );
    TableColumn versionColumn = tblTexte.getColumnModel().getColumn( 4 );
    TableColumn buttonColumn = tblTexte.getColumnModel().getColumn( 5 );

    tblTexte.removeColumn( statusColumn );
    tblTexte.removeColumn( versionColumn );
    tblTexte.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
    tblTexte.setCellSelectionEnabled( true );
    tblTexte.setRowHeight( 20 );

    textartColumn.setCellEditor( new DefaultCellEditor( cbxArttextartCd ) );
    spracheColumn.setPreferredWidth( 100 );
    spracheColumn.setMaxWidth( 100 );
    spracheColumn.setCellEditor( new DefaultCellEditor( cbxSprache ) );
    buttonColumn.setMaxWidth( 20 );
    buttonColumn.setMinWidth( 20 );
    buttonColumn.setCellRenderer( new BtnRenderer() );
    buttonColumn.setCellEditor( new BtnEditor( new JCheckBox() ) );

    // Table Listener
    tblTexte.addMouseListener( new java.awt.event.MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblNavToolBar.enableButtons();
      }
    } );

    tblTexte.addKeyListener( new java.awt.event.KeyAdapter()
    {
      public void keyReleased( KeyEvent e )
      {
        tblNavToolBar.enableButtons();
      }
    } );

    // Buttons der Toolbar enablen und disablen
    tblNavToolBar.enableButtons();
  }

  /**
   * ArtikelGuiAVO wird mit den geänderten Werten abgefüllt<br>
   * @return ArtikelGuiAVO mit aktuellen Werten
   */
  public ArtikelGuiAVO getData()
  {
    artikelGuiAVO.artextVO = dtmArtTexte.getData();
    return artikelGuiAVO;
  }

  /**
   * Füllt Controls mit den Werten des artikelGuiAVO<br>
   * Setzt den Index der Comboboxen auf die Werte des artikelGuiAVO
   * @param artikelGuiAVO ValueObject mit den Werten
   */
  public void setData( ArtikelGuiAVO artikelGuiAVO )
  {
    this.artikelGuiAVO = artikelGuiAVO;

    // TableModel auffüllen
    dtmArtTexte.setData( artikelGuiAVO.artextVO );
    tblNavToolBar.enableButtons();
  }

  /**
   * Überprüft Eingaben
   * @return  fehler == null --> keine Fehler Artikel kann gespeicher werden
   *          fehler.equals("") --> Fehler aufgetreten und angezeigt, Panel nicht wechseln
   *          anderenfalls den im fehler enthaltenen String anzeigen
   */
  public String validForward()
  {
    return null;
  }

  public String validBackward()
  {
    return null;
  }

  private void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this, "artikelwizard.texte" );
    HelpUtil.getInstance().enableHelp( this.pnlNav, "artikelwizard.texte" );
    HelpUtil.getInstance().enableHelp( this.pnlTexte, "artikelwizard.texte" );
    HelpUtil.getInstance().enableHelp( this.tblTexte, "artikelwizard.texte" );
  }
}
