package ch.ige.art.client.form;

import javax.swing.table.AbstractTableModel;

import ch.ige.art.util.XxDomain;
import ch.ige.art.vo.BformtxtDruckVO;

/**
 * <p>Überschrift: Table Model für die Textinformationen</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Texte im Anzeigemodus verantwortlich</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: AtmFormTexte.java,v 1.11 2008/07/02 11:00:20 heinzer Exp $
 */

public class AtmFormTexte extends AbstractTableModel
{
  private final String[] columnNames = new String[] { "Textart", "Sprache", "Text" };

  private BformtxtDruckVO[] tableData = new BformtxtDruckVO[] {};

  public AtmFormTexte()
  {
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return columnNames.length;
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  public String getColumnName( int column )
  {
    return columnNames[column];
  }

  /**
   * Gibt den Wert einer Zelle zurück
   * @param row Zeile
   * @param col Spalte
   * @return Wert der entsprechenden Zelle
   */
  public Object getValueAt( int row, int col )
  {
    switch( col )
    {
      case 0:
        String textart = XxDomain.getBez( new Integer( tableData[row].textart ).intValue(), "textart" );
        return ( tableData[row].textart + ", " + textart );
      case 1:
        return tableData[row].sprachCd;
      case 2:
        return tableData[row].text;
    }
    return null;
  }

  /**
   * Setzt den Wert einer Zelle
   * @param value Wert der gesetzt wird
   * @param row Zeile
   * @param col Spalte
   */
  public void setValueAt( Object value, int row, int col )
  {
    switch( col )
    {
      case 0:
        int i = value.toString().indexOf( "," );
        tableData[row].textart = value.toString().substring( 0, i );
        break;
      case 1:
        tableData[row].sprachCd = value.toString();
        break;
      case 2:
        tableData[row].text = ( value == null ? "" : value.toString() );
        break;
    }
  }

  /**
   * Setzt die Daten ins TableModel
   * @param data Alle Texte eines Artikels gekapselt in einem ArtextVO
   */
  public void setData( BformtxtDruckVO[] data )
  {
    tableData = data;
    fireTableDataChanged();
  }

  /**
   * Daten im TableModel löschen
   */
  public void clear()
  {
    fireTableDataChanged();
  }
}
