package ch.ige.art.client.form;

import java.awt.BorderLayout;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import org.apache.log4j.Logger;

import ch.ige.art.client.CustomDlgInfo;
import ch.ige.art.client.DlgPrint;
import ch.ige.art.client.util.ComboBoxObject;
import ch.ige.art.client.util.UserMessage;
import ch.ige.art.server.ArtikelDelegate;
import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.BformartVO;
import ch.ige.art.vo.XxDomainVO;
import ch.ige.art.xml.FormListe;
import ch.ige.art.xml.FormStammblatt;
import ch.ige.arthelp.HelpUtil;
import ch.ige.authz.gui.GUIAuthorizer;
import ch.ige.main.client.GlassComponent;
import ch.ige.main.client.MainController;
import ch.ige.main.client.MainRootPane;
import ch.ige.main.client.SelectionPane;
import ch.ige.util.client.IgeTable;
import ch.ige.util.client.LimitedDocument;
import ch.ige.util.client.SwingWorker;
import ch.ige.util.client.TableSorterModel;

/**
 * <p>Überschrift: Formular Auswahl</p>
 * <p>Beschreibung: Panle um nach Formularen zu suchen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author   Anita Rueegsegger
 * @version $Id: FormAwl.java,v 1.29 2008/07/02 11:00:20 heinzer Exp $
 */

public class FormAwl extends SelectionPane
{
  // Logging
  private static final Logger LOG = Logger.getLogger( FormAwl.class );

  private UserMessage userMessage = new UserMessage();

  // Formulardaten
  private BformartVO[] arBformartVO;

  private BformartVO selKritVO;

  // Detail-Pane
  private FormDtl formularDtl = null;

  // Artikel Tabelle
  private TmForm tableModel;

  private TableSorterModel sorter;

  private IgeTable tblSel;

  //Frame konstruieren
  private JPanel mainPanel = new JPanel();

  private JPanel pnlSelKrit = new JPanel();

  private JPanel pnlSelTable = new JPanel();

  private JScrollPane scpSelTable = new JScrollPane();

  private BorderLayout lytSelTable = new BorderLayout();

  private GridBagLayout lytSelKrit = new GridBagLayout();

  private BorderLayout borderLayout1 = new BorderLayout();

  private JLabel lblBez = new JLabel();

  private JLabel lblFormartCd = new JLabel();

  private JTextField txfBez = new JTextField();

  private JTextField txfFormartCd = new JTextField();

  private JLabel lblDruckSystem = new JLabel();

  private JComboBox cbxDruckSystem = new JComboBox();

  //sortColumn als member, da sie zum Sortieren hinzugefügt und entfernt werden muss
  private TableColumn sortColumn = new TableColumn();

  private LimitedDocument ldoFormartCd = new LimitedDocument();

  private LimitedDocument ldoBez = new LimitedDocument();

  private int lastColumn; //für Spaltensortierung der Tabelle

  private int clickCount; //für Spaltensortierung der Tabelle

  /**
   * Parameterloser Konstruktor
   */
  public FormAwl()
  {
    try
    {
      jbInit();
      initHelp();
      GUIAuthorizer.authorizeFieldsIn( this );
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * Konstruktor mit Parameter für Hauptfenster und Suchmodus
   * @param mainController Hauptfenster, zuständig für Panelsteuerung
   * @param mode Suchmodus<br>
   *    0 - Suche in Auswahl<br>
   *    1 - Suche für Lookup
   *
   */
  public FormAwl( MainController mainController, int mode )
  {
    super( mainController, mode );
    focusOwner = txfFormartCd;
    try
    {
      jbInit();
      initHelp();
      GUIAuthorizer.authorizeFieldsIn( this );
    }
    catch ( Exception ex )
    {
      LOG.info( ex.getClass() + " " + ex.getMessage() );
      String message = userMessage.getUserMessage( ex );
      JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * Komponente initialisieren
   * @exception Exception
   */
  void jbInit() throws Exception
  {
    tableModel = new TmForm();
    sorter = new TableSorterModel( tableModel );
    tblSel = new IgeTable( sorter );

    mainPanel.setLayout( borderLayout1 );
    cbxDruckSystem.setMaximumSize( new Dimension( 290, 20 ) );
    cbxDruckSystem.setMinimumSize( new Dimension( 290, 20 ) );
    cbxDruckSystem.setPreferredSize( new Dimension( 400, 20 ) );
    txfFormartCd.setMaximumSize( new Dimension( 140, 20 ) );
    txfFormartCd.setMinimumSize( new Dimension( 140, 20 ) );
    txfFormartCd.setPreferredSize( new Dimension( 400, 20 ) );
    txfBez.setMaximumSize( new Dimension( 290, 20 ) );
    txfBez.setMinimumSize( new Dimension( 290, 20 ) );
    txfBez.setPreferredSize( new Dimension( 400, 20 ) );
    lblFormartCd.setMaximumSize( new Dimension( 140, 16 ) );
    lblFormartCd.setMinimumSize( new Dimension( 140, 16 ) );
    lblFormartCd.setPreferredSize( new Dimension( 200, 16 ) );
    lblBez.setMaximumSize( new Dimension( 140, 16 ) );
    lblBez.setMinimumSize( new Dimension( 140, 16 ) );
    lblBez.setPreferredSize( new Dimension( 140, 16 ) );
    mainPanel.add( pnlSelKrit, BorderLayout.NORTH );
    mainPanel.add( pnlSelTable, BorderLayout.CENTER );
    centerPanel.add( mainPanel, BorderLayout.CENTER );
    cbxDruckSystem.setBackground( SystemColor.controlLtHighlight );

    lblBez.setToolTipText( "bez" );
    lblBez.setHorizontalAlignment( SwingConstants.RIGHT );
    lblBez.setText( "Formular-Bezeichnung:" );
    lblDruckSystem.setText( "Drucksystem:" );
    lblDruckSystem.setHorizontalAlignment( SwingConstants.RIGHT );
    lblDruckSystem.setToolTipText( "druck_system" );
    lblFormartCd.setText( "Formular:" );
    lblFormartCd.setToolTipText( "formart_cd" );
    lblFormartCd.setHorizontalAlignment( SwingConstants.RIGHT );
    txfBez.setDocument( ldoBez );
    txfBez.setToolTipText( "Suche mit Platzhalter % möglich" );
    txfFormartCd.setDocument( ldoFormartCd );

    btnHelp.setEnabled( true );
    mnuHelp.setEnabled( true );

    pnlSelKrit.setMaximumSize( new Dimension( 2147483647, 160 ) );
    pnlSelKrit.setPreferredSize( new Dimension( 300, 160 ) );
    pnlSelKrit.setMinimumSize( new Dimension( 300, 160 ) );
    pnlSelKrit.setLayout( lytSelKrit );
    pnlSelKrit.setBorder( new TitledBorder( "Selektionskriterien" ) );
    pnlSelTable.setLayout( lytSelTable );
    pnlSelTable.setBorder( new TitledBorder( "Formulare" ) );

    tblSel.setPreferredScrollableViewportSize( new Dimension( 450, 50 ) );
    scpSelTable.getViewport().add( tblSel );
    pnlSelTable.add( scpSelTable, BorderLayout.CENTER );
    pnlSelKrit.add( cbxDruckSystem, new GridBagConstraints( 1, 2, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ),
        0, 0 ) );
    pnlSelKrit.add( lblFormartCd, new GridBagConstraints( 0, 0, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0,
        0 ) );
    pnlSelKrit.add( txfFormartCd, new GridBagConstraints( 1, 0, 1, 1, 1.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0,
        0 ) );
    pnlSelKrit.add( lblBez, new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ), 0, 0 ) );
    pnlSelKrit.add( txfBez, new GridBagConstraints( 1, 1, 2, 1, 0.0, 0.0, GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets( 3, 10, 3, 0 ), 0, 0 ) );
    pnlSelKrit.add( lblDruckSystem, new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0, GridBagConstraints.EAST, GridBagConstraints.NONE, new Insets( 0, 0, 0, 0 ),
        0, 0 ) );

    // Eigenschaften der Tabelle & Spalten setzen
    TableColumn formartCdColumn = tblSel.getColumnModel().getColumn( 0 );
    TableColumn bezColumn = tblSel.getColumnModel().getColumn( 1 );
    TableColumn druckSystemColumn = tblSel.getColumnModel().getColumn( 2 );
    sortColumn = tblSel.getColumnModel().getColumn( 3 );

    // Spaltenausrichtung setzen
    DefaultTableCellRenderer rendererRight = new DefaultTableCellRenderer();
    rendererRight.setHorizontalAlignment( SwingConstants.RIGHT );
    formartCdColumn.setCellRenderer( rendererRight );

    tblSel.removeColumn( sortColumn );

    //Spaltenbreite setzen
    formartCdColumn.setPreferredWidth( 60 );
    bezColumn.setPreferredWidth( 400 );
    druckSystemColumn.setPreferredWidth( 180 );

    // MouseListener für Tabelle und Tabellenheader
    tblSel.setColumnSelectionAllowed( false );
    JTableHeader header = tblSel.getTableHeader();
    header.addMouseListener( new MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        header_MouseListener( e );
      }
    } );

    tblSel.addMouseListener( new MouseAdapter()
    {
      public void mouseClicked( MouseEvent e )
      {
        tblSel_MouseListener( e );
      }
    } );
    tblSel.addFocusListener( new java.awt.event.FocusAdapter()
    {
      public void focusGained( FocusEvent e )
      {
        tblSel_focusGained( e );
      }

      public void focusLost( FocusEvent e )
      {
        tblSel_focusLost( e );
      }
    } );
    tblSel.addKeyListener( new java.awt.event.KeyAdapter()
    {
      public void keyPressed( KeyEvent e )
      {
        tblSel_keyPressed( e );
      }
    } );

    // Document Eigenschaften setzen
    ldoFormartCd.setMaxLength( 3 );
    ldoFormartCd.setUpperCase( true );
    ldoBez.setMaxLength( 40 );

    //Combobox druck_system
    List<XxDomainVO> listDomain = XxDomain.getDomains( XxDomainFactory.DRUCK_SYSTEM );
    cbxDruckSystem.addItem( new ComboBoxObject() );
    for ( XxDomainVO xxDomainVO: listDomain )
    {
      cbxDruckSystem.addItem( new ComboBoxObject( xxDomainVO.getKbez(), xxDomainVO.getBez() ) );
    }

    // GlassKomponente setzten
    setGlassPane( new GlassComponent() );
    setStateNotFound();

  } // jbInit

  //********** Öffentliche Standard-Methoden *******************************

  /**
   * Gibt den Titel zurück
   * @return Titel
   */
  public String getTitle()
  {
    return "Formulare Auswahl";
  }

  //********** Ereignisbehandlung zu Menus *******************************

  /**
   * Erstellen
   * @param e ActionEvent
   */
  protected void mnuNew_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( "Datensatz erfassen" );

    FormWizardMediator med = new FormWizardMediator( (Frame) mainController.getWindow() );
    String formartCode = med.newForm();

    if ( formartCode != null ) // null falls insert abgebrochen
    {
      formularDtl = new FormDtl( mainController, MainRootPane.DETAIL );
      formularDtl.setKey( formartCode );
      formularDtl.read();
      formularDtl.setStatusBar( "Datensatz gespeichert." );
      mainController.next( formularDtl );
    }
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Menu Datensatz suchen und anzeigen
   * @param e ActionEvent
   */
  protected void mnuFind_actionPerformed( ActionEvent e )
  {
    tableModel.clear();
    statusBar.setText( "Bitte warten..." );

    // Selektionskriterien abfüllen
    final BformartVO formSelKritVO = fillSelKrit();

    if ( formSelKritVO.formartCd.equals( "" ) && formSelKritVO.bez.equals( "" ) && formSelKritVO.druckSystem.equals( "" ) )
    {
      JOptionPane.showMessageDialog( null, "Es muss mindestens ein Selektionskriterium abgefüllt sein.", "Information", JOptionPane.INFORMATION_MESSAGE );
      statusBar.setText( " " );
      return;
    }

    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        try
        {
          ArtikelDelegate ad = ArtikelDelegate.getInstance();
          ArrayList alBformartVO = ad.findFormular( formSelKritVO.formartCd, formSelKritVO.bez, formSelKritVO.druckSystem );
          arBformartVO = (BformartVO[]) alBformartVO.toArray( new BformartVO[0] );
        }
        catch ( Exception ex )
        {
          LOG.info( ex.getMessage() );
          String message = userMessage.getUserMessage( ex );
          JOptionPane.showMessageDialog( null, message, "Information", JOptionPane.INFORMATION_MESSAGE );
          statusBar.setText( " " );
          return null;
        }

        // TableModel füllen
        tableModel.setData( arBformartVO );
        //Column, nach der sortiert werden soll hinzufügen
        //Es kann nicht der Standardsortiermechanismus (sorter.addMouseListenerToHeaderInTable(tblSel))
        //verwendet werden, da nach einer unsichtbaren Column sortiert wird und nicht nach der angewählten
        tblSel.addColumn( sortColumn );
        sorter.sortByColumn( 3, true );
        tblSel.removeColumn( sortColumn );
        return null;
      }

      public void finished()
      {
        if ( tableModel.getRowCount() > 0 )
        {
          setStateFound();
          tblSel.setRowSelectionInterval( 0, 0 );
          tblSel.requestFocus();
          statusBar.setText( "Es wurden " + tableModel.getRowCount() + " Datensätze gelesen !" );
        }
        else
        {
          setStateNotFound();
          statusBar.setText( "Keine Datensätze gefunden !" );
        }
        getGlassPane().setVisible( false );
      }
    }; // SwingWorker
    worker.start();

  } // mnuFind_actionPerformed

  private BformartVO fillSelKrit()
  {
    selKritVO = new BformartVO();

    selKritVO.formartCd = txfFormartCd.getText();
    selKritVO.bez = txfBez.getText().trim();

    ComboBoxObject cbo = (ComboBoxObject) cbxDruckSystem.getSelectedItem();
    selKritVO.druckSystem = ( cbo.key == null ? "" : cbo.key );

    return selKritVO;
  }

  /**
   * Menu Undo<br>
   * Suchfelder werden gelöscht
   * @param e ActionEvent
   */
  protected void mnuUndo_actionPerformed( ActionEvent e )
  {
    setStateNotFound();
    txfFormartCd.requestFocus();
    txfFormartCd.setText( "" );
    txfBez.setText( "" );
    cbxDruckSystem.setSelectedItem( new String[] { "", "" } );
    cbxDruckSystem.requestFocus();
    txfFormartCd.requestFocus();
    tableModel.clear();
    statusBar.setText( " " );
  }

  /**
   * Menu Verzweigen<br>
   * Formulardetail wird aufgerufen
   * @param e ActionEvent
   */
  protected void mnuDetail_actionPerformed( ActionEvent e )
  {
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.WAIT_CURSOR ) );
    statusBar.setText( " " );
    if ( mode == SEARCH )
    {
      setForwardEnabled( true );
      formularDtl = new FormDtl( mainController, MainRootPane.DETAIL );
    }
    else if ( mode == SEARCH_LOOKUP )
    {
      formularDtl = new FormDtl( mainController, MainRootPane.DETAIL_LOOKUP );
    }

    // Schlüssel abfüllen
    int row = tblSel.getSelectedRows()[0];
    formularDtl.setKey( (String) tableModel.getValueAt( row, 0 ).toString() );
    formularDtl.read();
    mainController.next( formularDtl );
    mainController.getWindow().setCursor( Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR ) );
  }

  /**
   * Menu Info<br>
   * Infofeld der Applikation wird angezeigt
   * @param e ActionEvent
   */
  public void mnuInfo_actionPerformed( ActionEvent e )
  {
    new CustomDlgInfo( (JFrame) this.mainController.getWindow(), true ).setVisible( true );
  }

  /**
   * Menu Druck
   * Dialog mit der Auswahlmöglichkeit Trefferliste oder Stammblatt wird geöffnet
   * @param e ActionEvent
   */
  public void mnuPrint_actionPerformed( ActionEvent e )
  {
    String message = null;

    if ( tableModel.getRowCount() > 0 )
    {

      DlgPrint dlgPrint = new DlgPrint();
      int optionType = dlgPrint.showDialog();

      if ( optionType == DlgPrint.OK_OPTION )
      {
        if ( dlgPrint.isListeSelected() )
        {
          message = "Anzahl auszudruckende Treffer: " + tableModel.getRowCount();
          int option = JOptionPane.showConfirmDialog( this, message, "Druckanzahl", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE );
          if ( option == JOptionPane.OK_OPTION )
          {
            printTrefferliste();
          }
        }
        if ( dlgPrint.isStammblattSelected() )
        {
          message = "Anzahl auszudruckende Stammblätter: " + tableModel.getRowCount();
          int option = JOptionPane.showConfirmDialog( this, message, "Druckanzahl", JOptionPane.OK_CANCEL_OPTION, JOptionPane.INFORMATION_MESSAGE );
          if ( option == JOptionPane.OK_OPTION )
          {
            printStammblatt();
          }
        }
      }
    }
    else
    {
      message = "Keine Daten zum Drucken vorhanden";
      JOptionPane.showMessageDialog( this, message, "Drucken", JOptionPane.INFORMATION_MESSAGE );
    }
  }

  /**
   * Druckt via Jippsclient eine Trefferliste aus,
   * welche die in der Tabelle enthaltenen Daten beinhaltet
   */
  private void printTrefferliste()
  {
    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        try
        {
          FormListe liste = new FormListe();
          Map map = liste.getKriteriumList( selKritVO );

          List listFormular = Arrays.asList( arBformartVO );
          liste.print( listFormular, map );
        }
        catch ( Exception ex )
        {
          LOG.error( ex );
          JOptionPane.showMessageDialog( null, "Die Trefferliste konnte nicht ausgedruckt werden", "Fehlermeldung", JOptionPane.ERROR_MESSAGE );
          statusBar.setText( " " );
          return null;
        }
        return null;
      }

      public void finished()
      {

        statusBar.setText( "Es wurden " + tableModel.getRowCount() + " Treffer gedruckt !" );
        getGlassPane().setVisible( false );
      }
    }; // SwingWorker
    worker.start();
  }

  /**
   * Druckt via Jippsclient Stammblatt / Stammblätter aus
   */
  private void printStammblatt()
  {
    // GlassKomponente wird sichtbar gemacht,
    // damit der Benutzer während der Suche keine Eingaben vornehmen kann
    getGlassPane().setVisible( true );

    // Daten lesen
    final SwingWorker worker = new SwingWorker()
    {
      public Object construct()
      {
        String formartCd = null;
        List listFormartCd = new ArrayList();

        try
        {
          for ( int i = 0; i < tableModel.getRowCount(); i++ )
          {
            // Schlüssel abfüllen
            formartCd = (String) tableModel.getValueAt( i, 0 ).toString();
            listFormartCd.add( formartCd );
          }
          FormStammblatt formStammblatt = new FormStammblatt();
          formStammblatt.print( listFormartCd );

        }
        catch ( Exception ex )
        {
          LOG.error( ex );
          JOptionPane.showMessageDialog( null, "Formular-Stammblatt " + formartCd + " konnte nicht ausgedruckt werden", "Fehlermeldung",
              JOptionPane.ERROR_MESSAGE );
          statusBar.setText( " " );
          return null;
        }
        return null;
      }

      public void finished()
      {
        statusBar.setText( "Es wurden " + tableModel.getRowCount() + " Stammblätter gedruckt !" );
        getGlassPane().setVisible( false );
      }
    }; // SwingWorker
    worker.start();
  }

  //********** Pane-spezifische Methoden ***********************************

  /**
   * Popup-Menu anzeigen nur falls GlassPane nicht sichtbar.
   * @param e Auslösendes Ereignis
   */
  protected void showPopupMenu( MouseEvent e )
  {
    if ( !getGlassPane().isVisible() )
    {
      super.showPopupMenu( e );
    }
  }

  /**
   * Behandelt Mausklicks des Benutzers in Auswahltabelle
   * Bei Doppelclick weiterverzweigen
   * @param e MouseEvent
   */
  protected void tblSel_MouseListener( MouseEvent e )
  {
    setStateFound();

    // bei Doppelclick weiterverzweigen
    if ( e.getClickCount() == 2 )
    {
      if ( mode == SEARCH )
      {
        mnuDetail_actionPerformed( null );
      }
      else if ( mode == SEARCH_LOOKUP )
      {
        mnuOk_actionPerformed( null );
      }
    }
  }

  /**
   * Falls Focus nicht mehr in der Tabelle ist, wird der Suchen-Knopf der Default.
   * @param e FocusEvent
   */
  protected void tblSel_focusLost( FocusEvent e )
  {
    if ( !e.isTemporary() )
    {
      setDefaultButton( btnFind );
    }
  }

  /**
   * Falls der Focus wieder auf der Tabelle ist, wird der Detail-Knopf der Default.
   * @param e FocusEvent
   */
  protected void tblSel_focusGained( FocusEvent e )
  {
    if ( mode == SEARCH )
    {
      setDefaultButton( btnDetail );
    }
    else
    {
      setDefaultButton( btnOk );
    }
  }

  /**
   * Enter-Taste in der Tabelle löst den DefaultButton aus.
   * @param e KeyEvent
   */
  protected void tblSel_keyPressed( KeyEvent e )
  {
    if ( e.getKeyCode() == KeyEvent.VK_ENTER )
    {
      getDefaultButton().doClick();
      e.consume();
    }
  }

  /**
   * Setzt Menus und Buttons in den Initialzustand zurück
   * Sortiert Tabelle nach der angeklickten Column
   * Es kann nicht der Standardsortiermechanismus (sorter.addMouseListenerToHeaderInTable(tblSel))
   * verwendet werden, da nach einer unsichtbaren Column sortiert wird und nicht nach der angewählten
   *
   * @param e Mousevent
   */
  protected void header_MouseListener( MouseEvent e )
  {
    TableColumnModel columnModel = tblSel.getColumnModel();
    int viewColumn = columnModel.getColumnIndexAtX( e.getX() );
    //wird die Column formart_cd angeklickt wird nach der "unsichtbaren" column sortColumn sortiert
    if ( viewColumn == 0 )
    {
      viewColumn = 3;
      tblSel.addColumn( sortColumn );
    }
    int column = tblSel.convertColumnIndexToModel( viewColumn );
    clickCount++;
    if ( column != lastColumn | clickCount == 3 )
    {
      clickCount = 1;
    }

    boolean ascending = ( clickCount == 1 );
    lastColumn = column;
    sorter.sortByColumn( column, ascending );
    tblSel.removeColumn( sortColumn );
    //Zurücksetzen der Buttons und Menues
    setStateNotFound();
  }

  //********** Authorization ***************************************************

  public static String[] getAuthzSpecifications()
  {
    return new String[] { "*New    = Erstellen/Formular" };
  }

  private void initHelp()
  {
    HelpUtil.getInstance().enableHelpKey( this.getRootPane(), "intro" );
    HelpUtil.getInstance().enableHelpOnButton( btnHelp, "formulare" );
    HelpUtil.getInstance().enableHelpOnButton( mnuHelp, "intro" );
    HelpUtil.getInstance().enableHelp( mainPanel, "formulare" );
    HelpUtil.getInstance().enableHelp( pnlSelKrit, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( pnlSelTable, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblFormartCd, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblBez, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblDruckSystem, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( lblFormartCd, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( txfBez, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( txfFormartCd, "formulare.selektionskriterien" );
    HelpUtil.getInstance().enableHelp( cbxDruckSystem, "formulare.selektionskriterien" );
  }
}
