package ch.ige.art.client.form;

import javax.swing.table.AbstractTableModel;

import ch.ige.art.util.XxDomain;
import ch.ige.art.util.XxDomainFactory;
import ch.ige.art.vo.BformartVO;

/**
 * <p>Überschrift: Table Model für Formularauswahl</p>
 * <p>Beschreibung: Ist für die Datenhaltung der Suchergebnisse im Anzeigemodus verantwortlich</p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Anita Rueegsegger
 * @version $Id: TmForm.java,v 1.12 2008/07/02 11:00:20 heinzer Exp $
 */
public class TmForm extends AbstractTableModel
{
  final String[] columnNames = new String[] { "Formular", "Formular-Bezeichnung", "Drucksystem", "Sort" };

  private BformartVO[] tableData = new BformartVO[] {};

  public TmForm()
  {
  }

  /**
   * Gibt Anzahl Spalten zurück
   * @return Anzahl Spalten
   */
  public int getColumnCount()
  {
    return columnNames.length;
  }

  /**
   * Gibt Anzahl Zeilen zurück
   * @return Anzahl Zeilen
   */
  public int getRowCount()
  {
    return tableData.length;
  }

  /**
   * Gibt Spaltenüberschrift der übergebenen Spalte zurück
   * @param column Spalte deren Spaltenüberschrift angefordert wird
   * @return Spaltenüberschrift
   */
  public String getColumnName( int column )
  {
    return columnNames[column];
  }

  /**
   * Werte der angegebenen Spalte werden mit den standardmässigen
   * Darstellungsobjekten ausgestattet
   *
   * @param column Spalte
   * @return gibt die Klasse der Column zurück
   */
  public Class getColumnClass( int column )
  {
    return getValueAt( 0, column ).getClass();
  }

  /**
   * Gibt den Wert einer Zelle zurück
   * @param row Zeile
   * @param col Spalte
   * @return Wert der entsprechenden Zelle
   */
  public Object getValueAt( int row, int col )
  {
    String druckSystem = "";
    switch( col )
    {
      case 0:
        return tableData[row].formartCd;
      case 1:
        return tableData[row].bez == null ? "" : tableData[row].bez;
      case 2:
        if ( tableData[row].druckSystem == null || tableData[row].druckSystem.equals( "" ) )
        {
          druckSystem = "";
        }
        else
        {
          druckSystem = XxDomain.getBez( tableData[row].druckSystem, XxDomainFactory.DRUCK_SYSTEM );
        }
        return ( druckSystem == null || druckSystem.equals( "" ) ) ? "" : tableData[row].druckSystem + ", " + druckSystem;
      case 3:
        return getFormattedValue( tableData[row].formartCd );
    }
    return null;
  }

  /**
   * Setzt den Wert einer Zelle
   * @param value Wert der gesetzt wird
   * @param row Zeile
   * @param col Spalte
   */
  public void setValueAt( Object value, int row, int col )
  {
    switch( col )
    {
      case 0:
        tableData[row].formartCd = value.toString();
        break;
      case 1:
        tableData[row].bez = value == null ? "" : value.toString();
        break;
      case 2:
        if ( value == null || value.equals( "" ) )
        {
          tableData[row].druckSystem = "";
        }
        else
        {
          int i = value.toString().indexOf( "," );
          tableData[row].druckSystem = ( value == null ? "" : value.toString().substring( 0, i ) );
        }
        break;
    }
  }

  /**
   * Setzt die Daten ins TableModel
   * @param data Alle Ergebnisse einer Suche gekapselt in einem BformartVO
   */
  public void setData( BformartVO[] data )
  {
    tableData = data;
    fireTableDataChanged();
  }

  /**
   * Daten in TableModel löschen
   */
  public void clear()
  {
    tableData = new BformartVO[] {};
    fireTableDataChanged();
  }

  /**
   * ergänzt formart_cd mit führenden "0"
   *
   * @param   cellValue Eintrag einer Zelle der Tabelle
   * @return  formatierter formart_cd
   */
  private String getFormattedValue( String cellValue )
  {
    switch( cellValue.length() )
    {
      case 1:
        return "00" + cellValue;
      case 2:
        return "0" + cellValue;
      default:
        return cellValue;
    }
  }
}
