package ch.ige.art.client.resources;

import java.util.ResourceBundle;

/**
 * The resources used in the application, containing common texts.
 *
 * <p>Copyright (c) 2004, Swiss Federal Institute of Intellectual Property</p>
 * @author   hec
 * @version  $Id: Resources.java,v 1.1 2008/07/03 13:19:21 heinzer Exp $
 */
public class Resources
{
  /**
   * Contains all texts for the GUI.
   */
  public static final String TEXT_PROPERTIES = "res.texts";

  /**
   * Contains all the labels and other I18N texts
   */
  private static final ResourceBundle TEXTS = ResourceBundle.getBundle( Resources.TEXT_PROPERTIES );

  /**
   * Gets the text for the given key.
   * @param resourceKey The key to look for
   * @return The text for the key
   */
  public static String getString( String resourceKey )
  {
    return TEXTS.getString( resourceKey );
  }

  /**
   * Gets the texts for the given key.
   * There are two texts: The long label and the name of the db column 
   * @param resourceKey The key to look for
   * @return The DBColumn knowing all texts for the key
   */
  public static DBColumn getDBColumn( String resourceKey )
  {
    String label = TEXTS.getString( resourceKey );
    String column = TEXTS.getString( resourceKey + ".dbcol" );
    DBColumn dbcol = new DBColumn( column, label );
    return dbcol;
  }

  /**
   * Gets the first character for the given key.
   * @param resourceKey The key to look for
   * @return the first character of the text
   */
  public static char getCharacter( String resourceKey )
  {
    return getString( resourceKey ).charAt( 0 );
  }
}
