package ch.ige.art.client.util;

import java.io.File;

import javax.swing.filechooser.FileFilter;

/**
 * FileFilter for CSV Files
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   ruegsegger
 * @version  $Id: CSVFileFilter.java,v 1.1 2008/02/05 20:38:14 ruegsegger Exp $
 */
public class CSVFileFilter extends FileFilter
{
  public final static String CSV = "csv";

  // @see javax.swing.filechooser.FileFilter#accept(java.io.File)
  @Override
  public boolean accept( File f )
  {
    if ( f.isDirectory() )
    {
      return true;
    }

    String extension = getExtension( f );
    if ( extension != null )
    {
      if ( extension.equals( CSV ) )
      {
        return true;
      }
      else
      {
        return false;
      }
    }

    return false;
  }

  /**
   * get extension of a file
   * @param f File to get extension from
   * 
   */
  public String getExtension( File f )
  {
    String ext = null;
    String s = f.getName();
    int i = s.lastIndexOf( '.' );

    if ( i > 0 && i < s.length() - 1 )
    {
      ext = s.substring( i + 1 ).toLowerCase();
    }
    return ext;
  }

  /**
   * add file extension csv if missing
   * @param f file to add extension if missing
   * @return file with csv extension if missing
   * 
   */
  public File addExtension( File f )
  {
    if ( !f.getPath().endsWith( CSV ) )
    {
      f = new File( f.getPath() + "." + CSV );
    }
    return f;
  }

  //The description of this filter
  public String getDescription()
  {
    return "csv";
  }

}
