package ch.ige.art.client.util;

import ch.ige.art.xml.*;


/**
 * <p>Darstellungsobjekt für einen Combobox-Eintrag</p>
 * <p>Wird angewendet, sofern der ComboBox-Eintrag aus zwei Werten zusammengesetzt wird
 * und die Werte einzeln angesprochen werden können müssen</p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author T.Iten, A.Rueegsegger
 * @version $Id: ComboBoxObject.java,v 1.5 2006/12/13 10:08:31 ruegsegger Exp $
 */


public class ComboBoxObject extends Object
{
  public String key = null;
  public String value = null;
  public String tooltip = null;


  public ComboBoxObject()
  {
  }


  /**
   * Konstruktor
   * @param key     Datenbank-Wert
   * @param value   Definiton des Datenbank-Wertes
   */
  public ComboBoxObject(String key, String value)
  {
    this(key, value, null);
  }


  /**
   * Konstruktor
   * @param key     Datenbank-Wert
   * @param value   Definiton des Datenbank-Wertes
   */
  public ComboBoxObject(String key, String value, String tooltip)
  {
    this.key = key;
    this.value = value;
    this.tooltip = tooltip;
  }


  /**
   * Gibt die Werte der Variablen key und value mit Komma abgetrennt zurück
   * @return gibt die Werte der Variablen key und value kommagetrennt zurück
   */
  public String toString()
  {
    if (key == null || key.equals(""))
    {
      if (value == null || value.equals(""))
      {
         return "";
      }
      else
      {
        return value;
      }
    }
    else if (value == null || value.equals(""))
    {
      return key;
    }
    else
    {
      return key + ", " + value;
    }
  }

  /**
   * Vergleicht den Inhalt der Instazvariablen beider Objekte miteinander
   * @param object Objekt das auf Gleichheit mit diesem Objekt überprüft werden soll
   * @return true sofern Werte der Instazvariablen des übergebene Objekt mit diesem übereinstimmen
   */
  public boolean equals(Object object)
  {
    boolean equals = false;

    if(object instanceof ComboBoxObject)
    {
      ComboBoxObject combo = (ComboBoxObject)object;

      if (key != null)
      {
        if (value != null)
        {
          equals = key.equals(combo.key) && value.equals(combo.value);
        }
        else
        {
          equals = key.equals(combo.key) && value == combo.value;
        }
      }
      else
      {
        if (value != null)
        {
          equals = key == combo.key && value.equals(combo.value);
        }
        else
        {
          equals = key == combo.key && value == combo.value;
        }
      }
    }
    return equals;
  }
}


