package ch.ige.art.client.util;

import javax.swing.*;
import ch.ige.art.xml.*;


/**
 * <p>Überschrift: Superklasse für Wizard-Tabellen</p>
 * <p>Beschreibung: Superklasse für Tabellen die über Funktion zum Einfügen,
 * Löschen und Zeilenwechsel verfügen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: TblArtikelWizard.java,v 1.6 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class TblArtikelWizard extends JTable
{
  private DtmArtikelWizard tablemodel;


  /**
   * Zwischenspeichern des TableModels
   * @param model TableModel
   */
  public TblArtikelWizard(DtmArtikelWizard model)
  {
    super(model);
    this.tablemodel = model;
  }

  /**
   * Fügt dem TableModel eine Zeile am Ende der Tabelle hinzu.<br>
   * Die neue Zeile wird aus dem übergebenen TableModel geholt.<br>
   * Neue Zeile wird selektieren
   */
  public void add()
  {
    tablemodel.insertRow(tablemodel.getRowCount(), tablemodel.getNewRow());
    last();
  }

  /**
   * Löscht eine Zeile aus dem TableModel.<br>
   * Gelöschte Daten müssen eventuell zwischengespeichert werden<br>
   * Nach dem Löschen wird immer die letzte Zeile selektiert
   */
  public void remove()
  {
    int iRow = getSelectedRow();
    int persistentRows = tablemodel.getPersistentRows();

    // Werte im Editiermodus werden übernommen
    DefaultCellEditor editor = (DefaultCellEditor)getCellEditor();
    if (editor != null)
    {
      editor.stopCellEditing();
    }

    if (iRow < persistentRows && tablemodel.getDeleteAllowed())
    {
      // zu löschende Zeile zwischenspeichern
      tablemodel.saveDeletedRow(iRow);
      tablemodel.setPersistentRows(--persistentRows);
      tablemodel.removeRow( iRow );
      this.last();
    }
    else if (iRow >= persistentRows)
    {
      // Selektierte Zeile löschen
      tablemodel.removeRow( iRow );
      this.last();
    }
  }

  /**
   * Zur ersten Zeile springen
   */
  public void first()
  {
    changeSelection(0, 0, false, false);
  }

  /**
   * Zur letzten Zeile springen
   */
  public void last()
  {
    int row = tablemodel.getRowCount();
    if (row > 0)
    {
      changeSelection(--row, 0, false, false);
    }
  }

  /**
   * Zur vorhergehenden Zeile springen
   */
  public void prior()
  {
    int selRow = getSelectedRow();

    changeSelection(--selRow, 0, false, false);
  }

  /**
   * Zur nächsten Zeile springen
   */
  public void next()
  {
    int selRow = getSelectedRow();

    changeSelection(++selRow, 0, false, false);
  }
}


