package ch.ige.art.client.util.renderer;

import java.awt.Component;
import javax.swing.*;
import javax.swing.table.TableCellRenderer;
import ch.ige.art.xml.*;


/**
 * <p>Überschrift: Button Darstellungskomponente</p>
 * <p>Beschreibung: Darstellungskomponente um einen Button innerhalb einer Zelle
 * darzustellen</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: BtnRenderer.java,v 1.5 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class BtnRenderer extends JButton implements TableCellRenderer
{

  public BtnRenderer()
  {
    setOpaque(true);
  }

  /**
   * Gibt eine Referenz auf sich selber zurück. Bevor die Referenz zurückgegeben
   * wird, werden Darstellungen in Abhängigkeit davon gesetzt, ob die Tabellenzelle
   * ausgewählt ist<br>
   * Überschreibt die Methode der Superklasse
   * @param table       aktuelle Tabelle
   * @param value       Wert der Zelle
   * @param isSelected  true wenn Zelle selektiert ist
   * @param hasFocus    true wenn Zelle den Focus hat
   * @param row         aktuelle Zeile
   * @param column      aktuelle Spalte
	 * @return            Referenz auf sich selbst
   */
  public Component getTableCellRendererComponent( JTable table,
                                                  Object value,
                                                  boolean isSelected,
                                                  boolean hasFocus,
                                                  int row, int column)
  {
    if (isSelected)
    {
      setForeground(table.getSelectionForeground());
      setBackground(table.getSelectionBackground());
    }
    else
    {
      setForeground(table.getForeground());
      setBackground(UIManager.getColor("Button.background"));
    }
    setText( (value == null) ? "" : value.toString() );
    return this;
  }
}

