package ch.ige.art.client.util.verifier;

import java.math.BigDecimal;
import java.text.*;
import java.util.Locale;
import javax.swing.*;


/**
 * Überschrift: DoubleVerifier
 * <p>Beschreibung: Überprüfung des Inhalte einer Komponenten.<br>
 * Beim Versuch die Komponente zu verlassen wird ein Ereigniss ausgelöst.<br>
 * Input muss ein gültiger Double-Wert sein </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Organisation: Institut für Geistiges Eigentum</p>
 * @author Anita Rueegsegger
 * @version $Id: DoubleVerifier.java,v 1.7 2006/12/13 10:08:31 ruegsegger Exp $
 */

public class DoubleVerifier extends InputVerifier
{
  /**
   *  Methode wird aufgerufen, falls der Benutzer versucht die Komponente zu verlassen<br>
   *  Methode verify(input) wird aufgerufen um den Input zu überprüfen
   *  @param  input   JComponent   Komponente deren Inhalt überprüft werden soll
   *  @return true  - Komponente kann verlassen werden<br>
   *          false - Focus bleibt in der Komponente, an den Benutzer wird eine Meldung ausgegeben
   */
  public boolean shouldYieldFocus( JComponent input )
  {
    boolean valid = super.shouldYieldFocus(input);
    if (!valid)
    {
      String message  = "Wert entspricht nicht dem Eingabeformat 0.00";
      String title    = "Ungültiger Wert";
      input.setInputVerifier(null);
      JOptionPane.showMessageDialog
        (null, message, title, JOptionPane.INFORMATION_MESSAGE);
      input.setInputVerifier(this);
    }
    return valid;
  }

  /**
   *  Überprüft ob der Inhalt der Komponente gültig ist<br>
   *  Zahl muss einen gültigen Wert haben
   *  @param  comp    Komponente deren Inhalt überprüft werden soll
   *  @return true  - Input in Ordnung<br>
   *          false - Inhalt der Komponente nicht gültig
   */
  public boolean verify( JComponent comp )
  {
    boolean returnValue = true;
    double d;

    JTextField textfield = (JTextField)comp;
    String content = textfield.getText();

    // Den Eingabewert nur überprüfen, sofern Eingabe gemacht wurde
    if (content.length() > 0)
    {
      try
      {
        String newContent = content.replace(',', '.');
        // Double auf Gültigkeit überprüfen
        d = Double.parseDouble(newContent);
        //Wert auf 2 Nachkommastellen runden
        BigDecimal bd = new BigDecimal(String.valueOf(d));
        bd = bd.setScale(2, BigDecimal.ROUND_HALF_UP);
        // Wert formatieren
        DecimalFormat df = new DecimalFormat("0.00", new DecimalFormatSymbols(Locale.ENGLISH));
				// formatierten Wert ins Textfield setzen
        textfield.setText(df.format(bd));
      }
      catch (Exception ex)
      {
        returnValue = false;
      }
    }
    return returnValue;
  }
}

