package ch.ige.art.server;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.EJBException;

import org.apache.log4j.Logger;

import ch.ige.art.server.dao.BArtikelDAO;
import ch.ige.art.server.dao.BformartDAO;
import ch.ige.art.server.dao.BformtxtDAO;
import ch.ige.art.server.dao.BformtxtDruckDAO;
import ch.ige.art.vo.BformDruckVO;
import ch.ige.art.vo.BformartVO;
import ch.ige.art.vo.BformtxtDruckVO;
import ch.ige.util.Crescendo;

/**
 * Business Object für Formulare
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: FormBO.java,v 1.20 2008/06/25 13:43:23 ruegsegger Exp $
 */
public class FormBO
{
  private static final Logger LOG = Logger.getLogger( FormBO.class );

  private BformartVO bformartVO = new BformartVO();

  private BformDruckVO bformDruckVO = new BformDruckVO();

  private BformartDAO bformartDAO = new BformartDAO();

  private BformtxtDruckDAO bformtxtDruckDAO = new BformtxtDruckDAO();

  private BArtikelDAO bArtikelDAO = new BArtikelDAO();

  /**
   * Gibt das Value Object des Formular zurück.
   *
   * @param   formartCd          Formular Code
   * @return  BformDruckVO       Value Object des Formulars
   */
  public BformDruckVO getFormular( String formartCd )
  {
    try
    {
      //holen aller Daten eines Formulars
      //Zusammensetzen des BformDruckVO
      bformDruckVO.bformartVO = bformartDAO.select( formartCd );
      List<BformtxtDruckVO> listBformarttxtDruck = bformtxtDruckDAO.select( formartCd );
      bformDruckVO.bformtxtDruckVO = listBformarttxtDruck.toArray( new BformtxtDruckVO[0] );
      bformDruckVO.listArtNr = bArtikelDAO.selectListArtNr( formartCd );

      if ( bformartVO == null )
      {
        throw new EJBException( " | Kein Formular mit Formularcode " + formartCd + " vorhanden |" );
      }

      return bformDruckVO;
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method getFormular error: " + ex.getMessage() );
      throw new EJBException( " | Formular kann nicht gelesen werden | " );
    }
  }

  /**
   * Gibt eine ArrayList der gefundenen Formulare zurück.
   *
   * @param   formartCd        Formartcode
   * @param   bez              Bezeichnung
   * @param   druckSystem      Drucksystem
   * @return  ArrayList        ArrayList mit Value Objects BformartVO
   */
  public ArrayList findFormular( String formartCd, String bez, String druckSystem )
  {
    try
    {
      return new BformartDAO().select( formartCd, bez, druckSystem );
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method findFormular error: " + ex.getMessage() );
      throw new EJBException( " | Suche kann nicht durchgeführt werden | " );
    }
  }

  /**
   * Aktualisiert das Formular auf die Wertes des Value Objects.
   *
   * @param      bformDruckVO   Value Object eines Formulars
   * @exception  EJBException
   */
  public void setFormular( BformDruckVO bformDruckVO ) throws EJBException
  {
    try
    {
      //Prüfen der Integritätsbedingungen
      checkIntegrity( bformDruckVO );

      //Erhöhen der Versionen
      //Datenbankseitig wird nur version überprüft, uVersion ist überflüssig
      bformDruckVO.bformartVO.version++;
      if ( bformDruckVO.bformartVO.uVersion == null || bformDruckVO.bformartVO.uVersion.equals( "" ) )
      {
        bformDruckVO.bformartVO.uVersion = "!";
      }
      else
      {
        bformDruckVO.bformartVO.uVersion = Crescendo.getNextUVersion( bformDruckVO.bformartVO.uVersion );
      }

      new BformartDAO().update( bformDruckVO.bformartVO );

      if ( bformDruckVO.bformtxtDruckVO != null )
      {
        for ( int i = 0; i < bformDruckVO.bformtxtDruckVO.length; i++ )
        {
          bformDruckVO.bformtxtDruckVO[i].version++;
          bformDruckVO.bformtxtDruckVO[i].formartCd = bformDruckVO.bformartVO.formartCd;
          //löschen der bformtxtDruckVO mit Status "D"
          if ( bformDruckVO.bformtxtDruckVO[i].status.equals( "D" ) )
          {
            bformtxtDruckDAO.delete( bformDruckVO.bformtxtDruckVO[i].formartCd, bformDruckVO.bformtxtDruckVO[i].textart,
                bformDruckVO.bformtxtDruckVO[i].sprachCd );
          }
        }
        //einfügen und aktualisieren
        for ( int i = 0; i < bformDruckVO.bformtxtDruckVO.length; i++ )
        {
          if ( bformDruckVO.bformtxtDruckVO[i].status.equals( "U" ) )
          {
            bformtxtDruckDAO.update( bformDruckVO.bformtxtDruckVO[i] );
          }
          if ( bformDruckVO.bformtxtDruckVO[i].status.equals( "I" ) )
          {
            bformtxtDruckDAO.insert( bformDruckVO.bformtxtDruckVO[i] );
          }
        }
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method setFormular error: " + ex.getMessage() );
      throw new EJBException( " | Formular kann nicht gespeichert werden | " );
    }
  }

  /**
   * Kreiert ein Formular
   *
   * @param      bformDruckVO Value Object des Artikels
   * @exception  EJBException
   */
  public void createFormular( BformDruckVO bformDruckVO ) throws EJBException
  {
    try
    {
      //bformDruckVO darf nicht null sein
      if ( bformDruckVO.bformartVO.formartCd.equals( "" ) || bformDruckVO.bformartVO.formartCd == null )
      {
        throw new EJBException( " | Kein FormularCode eingegeben! | " );
      }

      // Prüfen ob Formart code bereits existiert
      if ( bformartDAO.exists( bformDruckVO.bformartVO.formartCd ) )
      {
        throw new EJBException( " | Formularcode " + bformDruckVO.bformartVO.formartCd + " existiert bereits! | " );
      }
      // Checken der Integritätsbedingungen
      checkIntegrity( bformDruckVO );

      // Einfügen der VO in DB
      new BformartDAO().insert( bformDruckVO.bformartVO );

      if ( bformDruckVO.bformtxtDruckVO != null )
      {
        for ( int i = 0; i < bformDruckVO.bformtxtDruckVO.length; i++ )
        {
          bformDruckVO.bformtxtDruckVO[i].formartCd = bformDruckVO.bformartVO.formartCd;
          bformtxtDruckDAO.insert( bformDruckVO.bformtxtDruckVO[i] );
        }
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method createFormular error: " + ex.getMessage() );
      throw new EJBException( " | Formular kann nicht erfasst werden | " + ex );
    }
  }

  /**
   * Prüfen der Integritätsbedingungen
   *
   * @param  bformDruckVO    ValueObject eines Schreibens
   * @throws EJBException
   */
  private void checkIntegrity( BformDruckVO bformDruckVO ) throws EJBException
  {
    try
    {
      // ist Drucksystem auf Crescendo muss in Tabelle bformtxt Datensätze
      // mit diesem formart_cd vorhanden sind.
      if ( bformDruckVO.bformartVO.druckSystem != null && bformDruckVO.bformartVO.druckSystem.equals( "CRESC" ) )
      {
        if ( new BformtxtDAO().exists( bformDruckVO.bformartVO.formartCd ) == false )
        {
          throw new EJBException( " | Mutation nicht erlaubt, da keine Formulartexte\n mit diesem Formularcode auf Crescendo vorhanden sind | " );
        }
      }

      // Kombination Textart und Sprache darf nicht doppelt vorkommen
      if ( bformDruckVO.bformtxtDruckVO != null )
      {
        for ( int i = 0; i < bformDruckVO.bformtxtDruckVO.length; i++ )
        {
          for ( int j = 0; j < bformDruckVO.bformtxtDruckVO.length; j++ )
          {
            if ( ( bformDruckVO.bformtxtDruckVO[i].textart.equals( bformDruckVO.bformtxtDruckVO[j].textart ) )
                && ( bformDruckVO.bformtxtDruckVO[i].sprachCd.equals( bformDruckVO.bformtxtDruckVO[j].sprachCd ) )
                && ( !bformDruckVO.bformtxtDruckVO[i].status.equals( "D" ) ) && ( !bformDruckVO.bformtxtDruckVO[j].status.equals( "D" ) ) && ( i != j ) )
            {
              throw new EJBException( " | Kombination Textart und Sprache darf nicht doppelt existieren | " );
            }
          }
        }
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( new java.util.Date() + " " + this.getClass().toString() + ": " + "method checkIntegrity error: " + ex.getMessage() );
      throw new EJBException( " | Formular kann nicht gespeichert werden | " );
    }
  }
}
