package ch.ige.art.server.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import ch.ige.art.vo.ArtikelVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für artikel
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtikelDAO.java,v 1.12 2008/07/02 07:18:56 heinzer Exp $
 */

public class ArtikelDAO
{
  /**
   * Selektiert einen Datensatz aus der Tabelle artikel
   *
   * @param  artNr         ArtikelNr
   * @return ArtikelVO     Value Object
   * @throws SQLException  Unerwarteter SQL Fehler
   */
  public ArtikelVO select( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    ArtikelVO artikelVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT trim(art_nr) as art_nr, " + "abc_rangnr, " + "trim(abc_cd) as abc_cd, " + "trim(alt_nr_cd) as alt_nr_cd, " + "trim(art_cd) as art_cd, "
          + "trim(artgrp_1) as artgrp_1, " + "trim(artgrp_2) as artgrp_2, " + "trim(artgrp_3) as artgrp_3, " + "trim(artgrp_4) as artgrp_4, "
          + "trim(artgrp_cd) as artgrp_cd, " + "trim(beschaffart) as beschaffart, " + "trim(bonus_cd) as bonus_cd, " + "dat_dschn_ep, " + "dat_standpreis, "
          + "dschn_ep_sfr, " + "trim(ean_cd) as ean_cd, " + "trim(ekeinh) as ekeinh, " + "trim(etiketten_cd) as etiketten_cd, "
          + "trim(fakteinh) as fakteinh, " + "gewicht, " + "gewicht_netto, " + "trim(kennz_charge) as kennz_charge, " + "invpreis, "
          + "trim(kennz_ausl_art) as kennz_ausl_art, " + "trim(kennz_lag_fueh) as kennz_lag_fueh, " + "trim(kennz_serie_nr) as kennz_serie_nr, "
          + "trim(kondart_cd) as kondart_cd, " + "trim(kz_ferart) as kz_ferart, " + "trim(kz_gratis) as kz_gratis, "
          + "trim(kz_hist_design) as kz_hist_design, " + "trim(kz_metnot) as kz_metnot, " + "trim(kz_minverk) as kz_minverk, "
          + "trim(kz_netto_art) as kz_netto_art, " + "trim(kz_skto_ber) as kz_skto_ber, " + "trim(lageinh) as lageinh, " + "lagkost, "
          + "trim(land_cd) as land_cd, " + "letz_ep_fw, " + "letz_ep_sfr, " + "loskosten, " + "marge_min, " + "marge_soll, " + "trim(marken_cd) as marken_cd, "
          + "minverk_menge, " + "opt_best_menge, " + "trim(preisgruppe) as preisgruppe, " + "trim(prov_cd) as prov_cd, " + "trim(provgrp_cd) as provgrp_cd, "
          + "trim(sachb_cd) as sachb_cd, " + "trim(sammelart_cd) as sammelart_cd, " + "trim(sperr_cd) as sperr_cd, " + "spesensatz, " + "standpreis, "
          + "trim(steu_zeil_cd) as steu_zeil_cd, " + "trim(statcd_1) as statcd_1, " + "trim(statcd_2) as statcd_2, " + "trim(statcd_3) as statcd_3, "
          + "trim(stueli_cd) as stueli_cd, " + "trim(suchbegr_art) as suchbegr_art, " + "umr_fakt_ek, " + "umr_fakt_vk," + "trim(waehr_cd) as waehr_cd, "
          + "wbesch_zeit, " + "trim(zolltarif) as zolltarif, " + "trim(erst_ben) as erst_ben, " + "erst_dat, " + "trim(geaend_ben) as geaend_ben, "
          + "geaend_dat, " + "u_version " + "FROM artikel WHERE art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );

      rs = ps.executeQuery();
      if ( rs.next() )
      {
        artikelVO = new ArtikelVO();
        artikelVO.artNr = rs.getString( "art_nr" );
        artikelVO.abcCd = rs.getString( "abc_cd" );
        artikelVO.abcRangnr = new Integer( rs.getInt( "abc_rangnr" ) );
        if ( rs.wasNull() )
        {
          artikelVO.abcRangnr = null;
        }
        artikelVO.altNrCd = rs.getString( "alt_nr_cd" );
        artikelVO.artCd = rs.getString( "art_cd" );
        artikelVO.artgrp1 = rs.getString( "artgrp_1" );
        artikelVO.artgrp2 = rs.getString( "artgrp_2" );
        artikelVO.artgrp3 = rs.getString( "artgrp_3" );
        artikelVO.artgrp4 = rs.getString( "artgrp_4" );
        artikelVO.artgrpCd = rs.getString( "artgrp_cd" );
        artikelVO.beschaffart = rs.getString( "beschaffart" );
        artikelVO.bonusCd = rs.getString( "bonus_cd" );
        artikelVO.datDschnEp = rs.getDate( "dat_dschn_ep" );
        artikelVO.datStandpreis = rs.getDate( "dat_standpreis" );
        artikelVO.dschnEpSfr = rs.getBigDecimal( "dschn_ep_sfr" );
        artikelVO.eanCd = rs.getString( "ean_cd" );
        artikelVO.ekeinh = rs.getString( "ekeinh" );
        artikelVO.etikettenCd = rs.getString( "etiketten_cd" );
        artikelVO.fakteinh = rs.getString( "fakteinh" );
        artikelVO.gewicht = rs.getBigDecimal( "gewicht" );
        artikelVO.gewichtNetto = rs.getBigDecimal( "gewicht_netto" );
        artikelVO.invpreis = rs.getBigDecimal( "invpreis" );
        artikelVO.kennzAuslArt = rs.getString( "kennz_ausl_art" );
        artikelVO.kennzCharge = rs.getString( "kennz_charge" );
        artikelVO.kennzLagFueh = rs.getString( "kennz_lag_fueh" );
        artikelVO.kennzSerieNr = rs.getString( "kennz_serie_nr" );
        artikelVO.kondartCd = rs.getString( "kondart_cd" );
        artikelVO.kzFerart = rs.getString( "kz_ferart" );
        artikelVO.kzGratis = rs.getString( "kz_gratis" );
        artikelVO.kzHistDesign = rs.getString( "kz_hist_design" );
        artikelVO.kzMetnot = rs.getString( "kz_metnot" );
        artikelVO.kzMinverk = rs.getString( "kz_minverk" );
        artikelVO.kzNettoArt = rs.getString( "kz_netto_art" );
        artikelVO.kzSktoBer = rs.getString( "kz_skto_ber" );
        artikelVO.lageinh = rs.getString( "lageinh" );
        artikelVO.lagkost = rs.getBigDecimal( "lagkost" );
        artikelVO.landCd = rs.getString( "land_cd" );
        artikelVO.letzEpFw = rs.getBigDecimal( "letz_ep_fw" );
        artikelVO.letzEpSfr = rs.getBigDecimal( "letz_ep_sfr" );
        artikelVO.loskosten = rs.getBigDecimal( "loskosten" );
        artikelVO.margeMin = new Integer( rs.getInt( "marge_min" ) );
        if ( rs.wasNull() )
        {
          artikelVO.margeMin = null;
        }
        artikelVO.margeSoll = new Integer( rs.getInt( "marge_soll" ) );
        if ( rs.wasNull() )
        {
          artikelVO.margeSoll = null;
        }
        artikelVO.markenCd = rs.getString( "marken_cd" );
        artikelVO.minverkMenge = rs.getBigDecimal( "minverk_menge" );
        artikelVO.optBestMenge = rs.getBigDecimal( "opt_best_menge" );
        artikelVO.preisgruppe = rs.getString( "preisgruppe" );
        artikelVO.provCd = rs.getString( "prov_cd" );
        artikelVO.provgrpCd = rs.getString( "provgrp_cd" );
        artikelVO.sachbCd = rs.getString( "sachb_cd" );
        artikelVO.sammelartCd = rs.getString( "sammelart_cd" );
        artikelVO.sperrCd = rs.getString( "sperr_cd" );
        artikelVO.spesensatz = new Integer( rs.getInt( "spesensatz" ) );
        if ( rs.wasNull() )
        {
          artikelVO.spesensatz = null;
        }
        artikelVO.standpreis = rs.getBigDecimal( "standpreis" );
        artikelVO.statcd1 = rs.getString( "statcd_1" );
        artikelVO.statcd2 = rs.getString( "statcd_2" );
        artikelVO.statcd3 = rs.getString( "statcd_3" );
        artikelVO.steuZeilCd = rs.getString( "steu_zeil_cd" );
        artikelVO.stueliCd = rs.getString( "stueli_cd" );
        artikelVO.suchbegrArt = rs.getString( "suchbegr_art" );
        artikelVO.umrFaktEk = new Integer( rs.getInt( "umr_fakt_ek" ) );
        if ( rs.wasNull() )
        {
          artikelVO.umrFaktEk = null;
        }
        artikelVO.umrFaktVk = new Integer( rs.getInt( "umr_fakt_vk" ) );
        if ( rs.wasNull() )
        {
          artikelVO.umrFaktVk = null;
        }
        artikelVO.waehrCd = rs.getString( "waehr_cd" );
        artikelVO.wbeschZeit = new Integer( rs.getInt( "wbesch_zeit" ) );
        if ( rs.wasNull() )
        {
          artikelVO.wbeschZeit = null;
        }
        artikelVO.zolltarif = rs.getString( "zolltarif" );
        artikelVO.erstDat = rs.getDate( "erst_dat" );
        artikelVO.erstBen = rs.getString( "erst_ben" );
        artikelVO.geaendDat = rs.getDate( "geaend_dat" );
        artikelVO.geaendBen = rs.getString( "geaend_ben" );
        artikelVO.uVersion = rs.getString( "u_version" );
      }
      return artikelVO;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtikelDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle artikel ein
   *
   * @param     artikelVO        ValueObject  ArtikelVO
   * @exception SQLException     Unerwarteter SQL Fehler
   */
  public void insert( ArtikelVO artikelVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "INSERT INTO artikel(" + "art_nr, " + "abc_cd, " + "abc_rangnr, " + "alt_nr_cd, " + "art_cd, " + "artgrp_1, " + "artgrp_2, " + "artgrp_3, "
          + "artgrp_4, " + "artgrp_cd, " + "beschaffart, " + "bonus_cd, " + "dat_dschn_ep, " + "dat_standpreis, " + "dschn_ep_sfr, " + "ean_cd, " + "ekeinh, "
          + "etiketten_cd, " + "fakteinh, " + "gewicht, " + "gewicht_netto, " + "invpreis, " + "kennz_ausl_art, " + "kennz_charge, " + "kennz_lag_fueh, "
          + "kennz_serie_nr, " + "kondart_cd, " + "kz_ferart, " + "kz_gratis, " + "kz_metnot, " + "kz_minverk, " + "kz_netto_art, " + "kz_skto_ber, "
          + "lageinh, " + "lagkost, " + "land_cd, " + "letz_ep_fw, " + "letz_ep_sfr, " + "loskosten, " + "marge_min, " + "marge_soll, " + "marken_cd, "
          + "minverk_menge, " + "opt_best_menge, " + "preisgruppe, " + "prov_cd, " + "provgrp_cd, " + "sachb_cd, " + "sammelart_cd, " + "sperr_cd, "
          + "spesensatz, " + "standpreis, " + "statcd_1, " + "statcd_2, " + "statcd_3, " + "steu_zeil_cd, " + "stueli_cd, " + "suchbegr_art, "
          + "umr_fakt_ek, " + "umr_fakt_vk, " + "waehr_cd, " + "wbesch_zeit, " + "zolltarif, " + "erst_dat, " + "erst_ben, " + "geaend_dat, " + "geaend_ben, "
          + "kz_hist_design, " + "u_version) " + "VALUES( ?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?," + //28
          "?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?," + //56
          "?,?,?,?,?,?,?,?,?,?,?,?,?) "; //69

      ps = con.prepareStatement( query );
      ps.setString( 1, artikelVO.artNr );
      ps.setString( 2, artikelVO.abcCd );
      if ( artikelVO.abcRangnr == null )
      {
        ps.setNull( 3, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 3, artikelVO.abcRangnr.intValue() );
      }
      ps.setString( 4, artikelVO.altNrCd );
      ps.setString( 5, artikelVO.artCd );
      ps.setString( 6, artikelVO.artgrp1 );
      ps.setString( 7, artikelVO.artgrp2 );
      ps.setString( 8, artikelVO.artgrp3 );
      ps.setString( 9, artikelVO.artgrp4 );
      ps.setString( 10, artikelVO.artgrpCd );
      ps.setString( 11, artikelVO.beschaffart );
      ps.setString( 12, artikelVO.bonusCd );
      if ( artikelVO.datDschnEp == null )
      {
        ps.setNull( 13, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 13, new java.sql.Date( artikelVO.datDschnEp.getTime() ) );
      }
      if ( artikelVO.datStandpreis == null )
      {
        ps.setNull( 14, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 14, new java.sql.Date( artikelVO.datStandpreis.getTime() ) );
      }
      ps.setBigDecimal( 15, artikelVO.dschnEpSfr );
      ps.setString( 16, artikelVO.eanCd );
      ps.setString( 17, artikelVO.ekeinh );
      ps.setString( 18, artikelVO.etikettenCd );
      ps.setString( 19, artikelVO.fakteinh );
      if ( artikelVO.gewicht == null )
      {
        ps.setNull( 20, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 20, artikelVO.gewicht.doubleValue() );
      }
      if ( artikelVO.gewichtNetto == null )
      {
        ps.setNull( 21, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 21, artikelVO.gewichtNetto.doubleValue() );
      }
      ps.setBigDecimal( 22, artikelVO.invpreis );
      ps.setString( 23, artikelVO.kennzAuslArt );
      ps.setString( 24, artikelVO.kennzCharge );
      ps.setString( 25, artikelVO.kennzLagFueh );
      ps.setString( 26, artikelVO.kennzSerieNr );
      ps.setString( 27, artikelVO.kondartCd );
      ps.setString( 28, artikelVO.kzFerart );
      ps.setString( 29, artikelVO.kzGratis );
      ps.setString( 30, artikelVO.kzMetnot );
      ps.setString( 31, artikelVO.kzMinverk );
      ps.setString( 32, artikelVO.kzNettoArt );
      ps.setString( 33, artikelVO.kzSktoBer );
      ps.setString( 34, artikelVO.lageinh );
      if ( artikelVO.lagkost == null )
      {
        ps.setNull( 35, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 35, artikelVO.lagkost.doubleValue() );
      }
      ps.setString( 36, artikelVO.landCd );
      ps.setBigDecimal( 37, artikelVO.letzEpFw );
      ps.setBigDecimal( 38, artikelVO.letzEpSfr );
      ps.setBigDecimal( 39, artikelVO.loskosten );
      if ( artikelVO.margeMin == null )
      {
        ps.setNull( 40, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 40, artikelVO.margeMin.intValue() );
      }
      if ( artikelVO.margeSoll == null )
      {
        ps.setNull( 41, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 41, artikelVO.margeSoll.intValue() );
      }
      ps.setString( 42, artikelVO.markenCd );
      if ( artikelVO.minverkMenge == null )
      {
        ps.setNull( 43, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 43, artikelVO.minverkMenge.doubleValue() );
      }
      if ( artikelVO.optBestMenge == null )
      {
        ps.setNull( 44, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 44, artikelVO.optBestMenge.doubleValue() );
      }
      ps.setString( 45, artikelVO.preisgruppe );
      ps.setString( 46, artikelVO.provCd );
      ps.setString( 47, artikelVO.provgrpCd );
      ps.setString( 48, artikelVO.sachbCd );
      ps.setString( 49, artikelVO.sammelartCd );
      ps.setString( 50, artikelVO.sperrCd );
      if ( artikelVO.spesensatz == null )
      {
        ps.setNull( 51, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 51, artikelVO.spesensatz.intValue() );
      }
      ps.setBigDecimal( 52, artikelVO.standpreis );
      ps.setString( 53, artikelVO.statcd1 );
      ps.setString( 54, artikelVO.statcd2 );
      ps.setString( 55, artikelVO.statcd3 );
      ps.setString( 56, artikelVO.steuZeilCd );
      ps.setString( 57, artikelVO.stueliCd );
      ps.setString( 58, artikelVO.suchbegrArt );
      if ( artikelVO.umrFaktEk == null )
      {
        ps.setNull( 59, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 59, artikelVO.umrFaktEk.intValue() );
      }
      if ( artikelVO.umrFaktVk == null )
      {
        ps.setNull( 60, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 60, artikelVO.umrFaktVk.intValue() );
      }
      ps.setString( 61, artikelVO.waehrCd );
      if ( artikelVO.wbeschZeit == null )
      {
        ps.setNull( 62, java.sql.Types.INTEGER );
      }
      else
      {
        ps.setInt( 62, artikelVO.wbeschZeit.intValue() );
      }
      ps.setString( 63, artikelVO.zolltarif );
      if ( artikelVO.erstDat == null )
      {
        ps.setNull( 64, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 64, new java.sql.Date( artikelVO.erstDat.getTime() ) );
      }
      ps.setString( 65, artikelVO.erstBen );
      if ( artikelVO.geaendDat == null )
      {
        ps.setNull( 66, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 66, new java.sql.Date( artikelVO.geaendDat.getTime() ) );
      }
      ps.setString( 67, artikelVO.geaendBen );
      ps.setString( 68, artikelVO.kzHistDesign );
      ps.setString( 69, artikelVO.uVersion );

      ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtikelDAO.class, query + artikelVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Aktualisiert einen Datensatz in der Tabelle artikel
   * Ein Update wird nur auf den Feldern durchgeführt die
   * in Zukunft noch gepflegt werden, alle anderen Felder bleiben unberührt
   *
   * @param     artikelVO     ArtikelVO ValueObject
   * @return    int           Anzahl aktualisierter Datensätze
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public int update( ArtikelVO artikelVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "UPDATE artikel SET " + "alt_nr_cd = ?, " + "artgrp_cd = ?, " + "bonus_cd = ?, " + "kondart_cd = ?, " + "kz_gratis = ?, " + "kz_metnot = ?, "
          + "lageinh = ?, " + "minverk_menge = ?, " + "opt_best_menge = ?, " + "prov_cd = ?, " + "sperr_cd = ?, " + "statcd_1 = ?, " + "statcd_2 = ?, "
          + "statcd_3 = ?, " + "suchbegr_art = ?, " + "geaend_dat = ?, " + "geaend_ben = ?, " + "kz_hist_design = ?, " + "u_version = ? "
          + "WHERE art_nr = RPAD(?, 18,' ') ";

      ps = con.prepareStatement( query );
      ps.setString( 1, artikelVO.altNrCd );
      ps.setString( 2, artikelVO.artgrpCd );
      ps.setString( 3, artikelVO.bonusCd );
      ps.setString( 4, artikelVO.kondartCd );
      ps.setString( 5, artikelVO.kzGratis );
      ps.setString( 6, artikelVO.kzMetnot );
      ps.setString( 7, artikelVO.lageinh );
      if ( artikelVO.minverkMenge == null )
      {
        ps.setNull( 8, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 8, artikelVO.minverkMenge.doubleValue() );
      }
      if ( artikelVO.optBestMenge == null )
      {
        ps.setNull( 9, java.sql.Types.FLOAT );
      }
      else
      {
        ps.setDouble( 9, artikelVO.optBestMenge.doubleValue() );
      }
      ps.setString( 10, artikelVO.provCd );
      ps.setString( 11, artikelVO.sperrCd );
      ps.setString( 12, artikelVO.statcd1 );
      ps.setString( 13, artikelVO.statcd2 );
      ps.setString( 14, artikelVO.statcd3 );
      ps.setString( 15, artikelVO.suchbegrArt );
      if ( artikelVO.geaendDat == null )
      {
        ps.setNull( 16, java.sql.Types.DATE );
      }
      else
      {
        ps.setDate( 16, new java.sql.Date( artikelVO.geaendDat.getTime() ) );
      }
      ps.setString( 17, artikelVO.geaendBen );
      ps.setString( 18, artikelVO.kzHistDesign );
      ps.setString( 19, artikelVO.uVersion );
      ps.setString( 20, artikelVO.artNr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtikelDAO.class, query + artikelVO );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle artikel
   *
   * @param   artNr          Primary Key
   * @return  int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int delete( String artNr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "DELETE FROM artikel WHERE art_nr = RPAD(?, 18,' ') ";
      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtikelDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Prüft die Existenz eines Artikels
   *
   * @param     artNr         zu prüfende ArtikelNr
   * @return    true          Artikel existiert.
   * @exception SQLException  Unerwarteter SQL-Fehler
   */
  public boolean exists( String artNr ) throws SQLException
  {
    if ( artNr == null )
    {
      throw new IllegalArgumentException( "ArtikelNr darf nicht null sein" );
    }
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "SELECT art_nr FROM artikel WHERE art_nr = RPAD(?, 18,' ') ";
      ps = con.prepareStatement( query );
      ps.setString( 1, artNr );
      rs = ps.executeQuery();

      return rs.next();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, ArtikelDAO.class, query + artNr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
