package ch.ige.art.server.dao;

import java.sql.*;
import ch.ige.art.vo.MaVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt zur Tabelle MA
 * Es werden nur die nötigen Methoden implementiert.
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author  A.Rueegsegger
 * @version $Id: MaDAO.java,v 1.9 2007/02/12 13:32:14 ruegsegger Exp $
 */
public class MaDAO
{
  public MaDAO()
  {
  }

  /**
   * Gibt Mitarbeite-Informationen zurück
   * @param vmsUsername User aus Systemtabelle
   * @return MsVO Mitarbeite-Informationen
   * @throws SQLException Unerwarteter SQL-Fehler
   */
  public MaVO select( String vmsUsername ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    MaVO maVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "SELECT ma_nr, trim(ma_kurzz) ma_kurzz, vms_username, ma_name, ma_vname from ma where vms_username = lower( ? ) ";

      ps = con.prepareStatement(query);
      ps.setString( 1, vmsUsername );
      rs = ps.executeQuery();

      if ( rs.next() )
      {
        maVO = new MaVO();
        maVO.maNr = rs.getInt( "ma_nr" );
        maVO.maKurzz = rs.getString( "ma_kurzz" );
        maVO.vmsUsername = rs.getString( "vms_username" );
        maVO.maName = rs.getString( "ma_name" );
        maVO.maVname = rs.getString( "ma_vname" );
      }
      return maVO;
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, MaDAO.class, query );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
