package ch.ige.art.server.dao.test;

import java.util.List;

import ch.ige.art.server.dao.ArtikelSucheDAO;
import ch.ige.art.vo.ArtikelAVO;
import ch.ige.art.vo.ArtikelSelKritVO;

/**
 * Test der Klasse TestArtikelSucheDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: ArtikelSucheDAOTest.java,v 1.13 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class ArtikelSucheDAOTest extends DAOTestCase
{

  /**
   * Konstruktor
   * Treiberklasse laden und DB-Verbindung aufbauen
   * Zugriff auf Oracle-Datenbank via Jdbc-Treiber
   */
  public ArtikelSucheDAOTest( String s )
  {
    super( s );
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle bartikel
   */
  public void testSelect()
  {
    ArtikelSucheDAO artikelSucheDAO = new ArtikelSucheDAO();
    ArtikelAVO artikelAVO;
    ArtikelSelKritVO artikelSelKritVO;
    List<ArtikelAVO> list;

    try
    {
      // Suche nach eingegebener Artikelnummer
      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.artNr = "99620";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      for ( int i = 0; i < list.size(); i++ )
      {
        artikelAVO = list.get( i );
        assertEquals( artikelSelKritVO.artNr, artikelAVO.artNr );
      }

      // Suche nach genauer Artikel Bezeichnung
      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.bezArt = "Lizenz";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      for ( int i = 0; i < list.size(); i++ )
      {
        artikelAVO = list.get( i );
        assertEquals( artikelSelKritVO.bezArt, artikelAVO.bezArt );
      }

      // Suche nach enthaltener Artikel Bezeichnung (%)
      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.bezArt = "%Berichtigung%";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      assertTrue( "Anzahl = ", list.size() > 40 );

      // Suche nach formart_cd
      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.formartCd = "115";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      for ( int i = 0; i < list.size(); i++ )
      {
        artikelAVO = list.get( i );
        assertEquals( artikelSelKritVO.formartCd, artikelAVO.formartCd );
      }

      // Suche nach Art-Nr Range
      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.artNr = "10000-10100";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      assertTrue( "Anzahl = ", list.size() > 10 );

      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.artNr = "10; 18100-18100; 20; 30; 100";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      assertTrue( "Anzahl = ", list.size() == 5 );

      artikelSelKritVO = new ArtikelSelKritVO();
      artikelSelKritVO.formartCd = "150; 151";
      list = artikelSucheDAO.select( artikelSelKritVO );
      if ( list == null || list.size() == 0 )
      {
        fail( "No Hits found" );
      }
      assertTrue( "Anzahl = ", list.size() > 2 );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
