package ch.ige.art.server.dao.test;

import java.util.List;

import ch.ige.art.server.dao.BformartDAO;
import ch.ige.art.vo.BformartVO;
import ch.ige.art.vo.test.Testdata;
import ch.ige.util.Crescendo;

/**
 * Test der Klasse BformartDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BformartDAOTest.java,v 1.9 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class BformartDAOTest extends DAOTestCase

{
  private BformartDAO bformartDAO = new BformartDAO();

  private BformartVO bformartVO = null;

  public BformartDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    bformartVO = Testdata.createBformartVO();
  }

  /**
   *  Fügt einen Datensatz in die Tabelle bformart ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT " );
    BformartVO selectVO = null;

    try
    {
      bformartDAO.insert( bformartVO );

      // Testen ob eingefügt
      selectVO = bformartDAO.select( bformartVO.formartCd );
      if ( selectVO == null )
      {
        fail( "INSERT: Datensatz existiert nicht" );
      }
      assertEquals( bformartVO.formartCd, selectVO.formartCd );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle artikel
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    BformartVO selectVO = null;

    try
    {
      selectVO = bformartDAO.select( bformartVO.formartCd );
      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( bformartVO.formartCd, selectVO.formartCd );
        assertEquals( bformartVO.bez, selectVO.bez );
        assertEquals( bformartVO.druckSystem, selectVO.druckSystem );
        assertEquals( bformartVO.version, selectVO.version );
        assertEquals( bformartVO.uVersion, selectVO.uVersion );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz in der Tabelle artikel
   */
  public void test03Update()
  {
    System.out.println( "Test UPDATE " );
    BformartVO selectVO = null;
    BformartVO updateVO = null;

    try
    {
      updateVO = bformartDAO.select( bformartVO.formartCd );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.bez = "Test after Update";
        updateVO.druckSystem = "SAP";
        updateVO.version++;
        updateVO.uVersion = Crescendo.getNextUVersion( bformartVO.uVersion );

        // Datensatz updaten
        int retVal = bformartDAO.update( updateVO );
        assertEquals( "Anzahl Datensätze ", 1, retVal );

        // Datensatz wieder sektieren und auf Änderungen testen
        selectVO = bformartDAO.select( bformartVO.formartCd );

        assertEquals( updateVO.formartCd, selectVO.formartCd );
        assertEquals( updateVO.bez, selectVO.bez );
        assertEquals( updateVO.druckSystem, selectVO.druckSystem );
        assertEquals( updateVO.version, selectVO.version );
        assertEquals( updateVO.uVersion, selectVO.uVersion );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Prüft die Existenz eines formartCodes aus der Tabelle bformart
   */
  public void test04Exists()
  {
    System.out.println( "Test EXISTS " );

    try
    {
      boolean exist = bformartDAO.exists( bformartVO.formartCd );
      assertEquals( "Datensatz existiert ", true, exist );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Löscht einen Datensatz aus der Tabelle bformart
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE " );

    try
    {
      int retVal = bformartDAO.delete( bformartVO.formartCd );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert Datensätze aus der Tabelle bformart welche
   *  den Selektionskriterien entsprechen
   */
  public void test06SelectSearch()
  {
    List<BformartVO> list;
    BformartVO searchVO = null;

    try
    {
      System.out.println( "TEST SELECT SEARCH" );

      // Suche nach eingegebenem Formualcode
      System.out.println( "Test SELECT formart_cd " );
      searchVO = new BformartVO();
      searchVO.formartCd = "151";
      list = bformartDAO.select( searchVO.formartCd, searchVO.bez, searchVO.druckSystem );
      if ( list == null || list.size() == 0 )
      {
        fail( "SELECT formart_cd: Keine Treffer" );
      }
      for ( int i = 0; i < list.size(); i++ )
      {
        bformartVO = list.get( i );
        assertEquals( "formart_cd: ", searchVO.formartCd, bformartVO.formartCd );
      }

      // Suche nach genauer Bezeichnung
      System.out.println( "Test SELECT genaue Bezeichnung" );
      searchVO = new BformartVO();
      searchVO.bez = "Beanstandungen";
      list = bformartDAO.select( searchVO.formartCd, searchVO.bez, searchVO.druckSystem );
      if ( list == null || list.size() == 0 )
      {
        fail( "SELECT genaue Bezeichnung: Keine Treffer" );
      }
      for ( int i = 0; i < list.size(); i++ )
      {
        bformartVO = list.get( i );
        assertEquals( "bez: ", searchVO.bez, bformartVO.bez );
      }

      // Suche nach enthaltener Artikel Bezeichnung (%)
      System.out.println( "Test SELECT enthaltene Bezeichnung" );
      searchVO = new BformartVO();
      searchVO.bez = "%Beanstandungen%";
      list = bformartDAO.select( searchVO.formartCd, searchVO.bez, searchVO.druckSystem );
      assertEquals( "Anzahl = ", list.size(), 5 );

      // Suche nach Drucksystem
      System.out.println( "Test SELECT Drucksystem" );
      searchVO = new BformartVO();
      searchVO.druckSystem = "JIPPS";
      list = bformartDAO.select( searchVO.formartCd, searchVO.bez, searchVO.druckSystem );
      if ( list == null || list.size() == 0 )
      {
        fail( "SELECT Drucksystem: Keine Treffer" );
      }
      for ( int i = 0; i < list.size(); i++ )
      {
        bformartVO = list.get( i );
        assertEquals( "Drucksystem: ", searchVO.druckSystem, bformartVO.druckSystem );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

}
