package ch.ige.art.server.dao.test;

import java.sql.SQLException;
import java.util.List;

import ch.ige.art.server.dao.BformartDAO;
import ch.ige.art.server.dao.BformtxtDruckDAO;
import ch.ige.art.vo.BformartVO;
import ch.ige.art.vo.BformtxtDruckVO;
import ch.ige.art.vo.test.Testdata;

/**
 * Test der Klasse BformtxtDruckDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BformtxtDruckDAOTest.java,v 1.11 2008/06/25 13:47:29 ruegsegger Exp $
 */

public class BformtxtDruckDAOTest extends DAOTestCase
{
  private BformtxtDruckDAO bformtxtDruckDAO = new BformtxtDruckDAO();

  private BformtxtDruckVO bformtxtDruckVO = null;

  private BformartDAO bformartDAO = new BformartDAO();

  private BformartVO bformartVO = null;

  public BformtxtDruckDAOTest( String s )
  {
    super( s );
  }

  @Override
  protected void setUp()
  {
    bformtxtDruckVO = Testdata.createBformtxtDruckVO();
    bformartVO = Testdata.createBformartVO();

    try
    {
      bformartDAO.insert( bformartVO );
    }
    catch ( SQLException ex )
    {
      fail( ex.getMessage() );
    }
  }

  @Override
  protected void tearDown()
  {
    try
    {
      bformartDAO.delete( bformartVO.formartCd );
    }
    catch ( SQLException ex )
    {
      fail( ex.getMessage() );
    }

  }

  /**
   *  Fügt einen Datensatz in die Tabelle bformart ein
   */
  public void test01Insert()
  {
    System.out.println( "Test INSERT " );
    BformtxtDruckVO selectVO = null;

    try
    {
      bformtxtDruckDAO.insert( bformtxtDruckVO );

      // Testen ob eingefügt
      selectVO = bformtxtDruckDAO.select( bformtxtDruckVO.formartCd, bformtxtDruckVO.textart, bformtxtDruckVO.sprachCd );
      if ( selectVO == null )
      {
        fail( "INSERT: Datensatz existiert nicht" );
      }
      assertEquals( bformtxtDruckVO.formartCd, selectVO.formartCd );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle artikel
   */
  public void test02Select()
  {
    System.out.println( "Test SELECT " );
    BformtxtDruckVO selectVO = null;

    try
    {
      bformtxtDruckDAO.insert( bformtxtDruckVO );

      selectVO = bformtxtDruckDAO.select( bformtxtDruckVO.formartCd, bformtxtDruckVO.textart, bformtxtDruckVO.sprachCd );
      if ( selectVO == null )
      {
        fail( "SELECT: Datensatz existiert nicht" );
      }
      else
      {
        assertEquals( bformtxtDruckVO.formartCd, selectVO.formartCd );
        assertEquals( bformtxtDruckVO.sprachCd, selectVO.sprachCd );
        assertEquals( bformtxtDruckVO.textart, selectVO.textart );
        assertEquals( bformtxtDruckVO.text, selectVO.text );
        assertEquals( bformtxtDruckVO.version, selectVO.version );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Selektiert mehrere Datensätze aus der Tabelle alt_art
   */
  public void test03SelectAll()
  {
    System.out.println( "Test SELECT mehrere VO" );
    List<BformtxtDruckVO> list = Testdata.createListBformtxtDruckVO( bformtxtDruckVO.formartCd );
    BformtxtDruckVO selectVO = null;
    List<BformtxtDruckVO> selectList = null;

    try
    {
      for ( int i = 0; i < list.size(); i++ )
      {
        selectVO = list.get( i );
        bformtxtDruckDAO.insert( selectVO );
      }

      selectList = bformtxtDruckDAO.select( bformtxtDruckVO.formartCd, selectVO.sprachCd );
      selectVO = list.get( 0 );

      assertTrue( "Anzahl = ", selectList.size() == 3 );
      assertEquals( bformtxtDruckVO.formartCd, selectVO.formartCd );

      for ( int i = 0; i < list.size(); i++ )
      {
        selectVO = list.get( i );
        int retVal = bformtxtDruckDAO.delete( selectVO.formartCd, selectVO.textart, selectVO.sprachCd );
        assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
      }

    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Aktualisiert einen Datensatz in der Tabelle artikel
   */
  public void test04Update()
  {
    System.out.println( "Test UPDATE " );
    BformtxtDruckVO selectVO = null;
    BformtxtDruckVO updateVO = null;

    try
    {
      bformtxtDruckDAO.insert( bformtxtDruckVO );

      updateVO = bformtxtDruckDAO.select( bformtxtDruckVO.formartCd, bformtxtDruckVO.textart, bformtxtDruckVO.sprachCd );
      if ( updateVO == null )
      {
        fail( "UPDATE: Datensatz existiert nicht" );
      }
      else
      {
        updateVO.text = "Test after Update";
        updateVO.version++;

        // Datensatz updaten
        int retVal = bformtxtDruckDAO.update( updateVO );
        assertEquals( "Anzahl Datensätze ", 1, retVal );

        // Datensatz wieder sektieren und auf Änderungen testen
        selectVO = bformtxtDruckDAO.select( bformtxtDruckVO.formartCd, bformtxtDruckVO.textart, bformtxtDruckVO.sprachCd );

        assertEquals( updateVO.formartCd, selectVO.formartCd );
        assertEquals( updateVO.sprachCd, selectVO.sprachCd );
        assertEquals( updateVO.textart, selectVO.textart );
        assertEquals( updateVO.text, selectVO.text );
        assertEquals( updateVO.version, selectVO.version );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   *  Löscht einen Datensatz aus der Tabelle bformart
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE " );
    BformtxtDruckVO selectVO = null;
    List<BformtxtDruckVO> selectList = null;

    try
    {
      bformtxtDruckDAO.insert( bformtxtDruckVO );

      selectList = bformtxtDruckDAO.select( bformtxtDruckVO.formartCd );

      for ( int i = 0; i < selectList.size(); i++ )
      {
        selectVO = selectList.get( i );
        int retVal = bformtxtDruckDAO.delete( selectVO.formartCd, selectVO.textart, selectVO.sprachCd );
        assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );
      }
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
