package ch.ige.art.util;

import ch.ige.art.vo.ArtikelGuiAVO;
import org.apache.log4j.Logger;
import ch.ige.art.xml.*;
import java.util.*;


/**
 * Schnittstelle zu Swissreg2
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * @author A. Rueegsegger
 * @version $Id: SwissregController.java,v 1.9 2007/07/09 06:02:03 locher Exp $
 */
public class SwissregController
{
  private static final Logger LOG = Logger.getLogger(SwissregController.class);

  public SwissregController()
  {
  }

  /**
   * Überprüft ob eine Nachführung der Textarten auf Swissreg2 erforderlich ist
   * Dies ist der Fall, wenn sich Texte mit Textart PBL oder REG geändert haben
   * Übermittelt werden immer alle PBL und REG Texte sprich der aktuelle Stand
   * Änderungen werden in Form eines XML welches dem Schmema CH-Domain entspricht übermittelt.
   * Der Inhalt wird validiert
   * Die Informationen werden in Form einer TextMessage via JMS verschickt, dies wird auf dem
   * Application Server durchgeführt.
   * @param artikelGuiAVO ArtikelGuiAVO
   * @throws Exception
   * @return String generiertes XML der nachzuführenden Domains, null falls keine Nachführung erforderlich
   */
  public String checkUpdateDomain( ArtikelGuiAVO artikelGuiAVO) throws Exception
   {
     String message = null;
     DomainDocument document = null;
     List publicationValues = new ArrayList();
     List registerValues = new ArrayList();

     if (artikelGuiAVO != null && artikelGuiAVO.artextVO != null )
     {
        for ( int i = 0; i < artikelGuiAVO.artextVO.length; i++ )
       {
         if ( artikelGuiAVO.artextVO[i].isPublicationTextToUpdate() )
         {
           publicationValues.add( artikelGuiAVO.artextVO[i] );
         }
         else if ( artikelGuiAVO.artextVO[i].isRegisterTextToUpdate() )
         {
           registerValues.add( artikelGuiAVO.artextVO[i] );
         }
       }

       if ( !publicationValues.isEmpty() || !registerValues.isEmpty() )
       {
         document = new DomainDocument( artikelGuiAVO.artNr );
         document.addPublicationEntry( publicationValues );
         document.addRegisterEntry( registerValues );

         try
         {
           message = document.validateAndNormalize();
         }
         catch ( Exception ex )
         {
           String marshaled;
           try {
             marshaled = document.getContent();
           }
           catch (Exception e)
           {
             marshaled = "failed to marshal";
           }
           LOG.error( marshaled, ex );
           throw new Exception( " | Der Artikel kann nicht gespeichert werden\n " +
                                "Bei der Validierung CH-DOMAIN ist ein Problem aufgetreten\n\n" +
                                "Melden Sie den Fehler dem Userdesk | " );
         }
       }
     }

     return message;
   }
}
