package ch.ige.art.vo;

import java.io.Serializable;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * Überschrift:   ArtextVO
 * Beschreibung:  Value Object für die Tabelle Artext
 * Copyright:     Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * Organisation:  IGE
 * @author:       Stefan Nydegger
 * @version $Id: ArtextVO.java,v 1.16 2008/06/25 13:58:04 ruegsegger Exp $
 */

public class ArtextVO implements Serializable
{
  private static Map<String, String> languageMap = new HashMap<String, String>( 10 );

  static
  {
    languageMap.put( "D", "de" );
    languageMap.put( "F", "fr" );
    languageMap.put( "E", "en" );
    languageMap.put( "I", "it" );
  }

  public String artNr = null;

  public String artTextArtCd = null;

  public String sprachCd = null;

  public String text = null;

  public String erstBen = null;

  public Date erstDat = null;

  public String geaendBen = null;

  public Date geaendDat = null;

  public String uVersion = "!";

  public String status = "";

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------

  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    buf.append( "  artNr               = <" + this.artNr + "> \n" );
    buf.append( "  artTextArtCd        = <" + this.artTextArtCd + "> \n" );
    buf.append( "  sprachCd            = <" + this.sprachCd + "> \n" );
    buf.append( "  text                = <" + this.text + "> \n" );
    buf.append( "  erstBen             = <" + this.erstBen + "> \n" );
    buf.append( "  erstDat             = <" + this.erstDat + "> \n" );
    buf.append( "  geaendBen           = <" + this.geaendBen + "> \n" );
    buf.append( "  geaendDat           = <" + this.geaendDat + "> \n" );
    buf.append( "  uVersion            = <" + this.uVersion + "> \n" );
    buf.append( "  status              = <" + this.status + "> \n" );

    return buf.toString();
  }

  /**
   * Textarten die nicht mehr gültig sind, werden nicht mehr angezeigt und nicht mehr angedruckt
   * Auch im Mutiermodus werden diese Daten nicht mehr angezeigt und können somit auch nicht mehr mutiert werden
   * @return boolean
   */
  /** TODO Textarten bei Neuerfassung 
   * abklären weshalb die Textarten welche nicht mehr gültig sind noch erfasst werden können, 
   * Hat wahrscheinlich keinen Grund sondern wurde bei der Anpassung vergessen */
  public boolean isValidTextart()
  {
    if ( artTextArtCd == null || artTextArtCd.equals( "NT" ) || artTextArtCd.equals( "MRT" ) || artTextArtCd.equals( "TNT" ) || artTextArtCd.equals( "BSC" )
        || artTextArtCd.equals( "DEP" ) || artTextArtCd.equals( "OMP" ) )
    {
      return false;
    }
    else
    {
      return true;
    }
  }

  /**
   * Gibt an ob Nachführung auf Swissreg erforderlich für PBL-Text
   * @return true falls Nachführung auf Swissreg erforderlich
   */
  public boolean isPublicationTextToUpdate()
  {
    return ( artTextArtCd.equals( "PBL" ) && !status.equals( "D" ) );
  }

  /**
   * Gibt an ob Nachführung auf Swissreg erforderlich für REG-Text
   * @return true falls Nachführung auf Swissreg erforderlich
   */
  public boolean isRegisterTextToUpdate()
  {
    return ( artTextArtCd.equals( "REG" ) && !status.equals( "D" ) );
  }

  public String getLanguage()
  {
    return languageMap.get( sprachCd.toUpperCase() );
  }
}
