package ch.ige.art.vo;

import java.io.Serializable;

/**
 * Value Object der Tabelle xx_domain
 * <p>
 * Copyright (c) 2002-2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger, Marc Bouquet
 * @version $Id: XxDomainVO.java,v 1.8 2008/06/25 13:58:04 ruegsegger Exp $
 */
public class XxDomainVO implements Serializable
{
  private String separator = ", ";

  public String domain = "";

  public int domainLnr = 0;

  public int sprache = 0;

  private String kbez = null; // vorher "";

  private String bez = null; // vorher "";

  public int orderNr = 0;

  public int version = 0;

  /**
   * Die toString()-Methode gibt alle Attribute als String zurück.
   * @return buf String mit allen Attributen
   */
  @Override
  public String toString()
  {
    StringBuffer buf = new StringBuffer();
    buf.append( " domain      = <" + this.domain + "> \n" );
    buf.append( " domainLnr   = <" + this.domainLnr + "> \n" );
    buf.append( " sprache     = <" + this.sprache + "> \n" );
    buf.append( " kbez        = <" + this.kbez + "> \n" );
    buf.append( " bez         = <" + this.bez + "> \n" );
    buf.append( " orderNr     = <" + this.orderNr + "> \n" );
    buf.append( " version     = <" + this.version + "> \n" );
    return buf.toString();
  }

  /**
   * Gibt den Schlüssel zurück
   * @return String Key
   */
  public String getKey()
  {
    return String.valueOf( domainLnr );
  }

  /**
   * Gibt den Value zurück
   * @return String Value
   */
  public String getText()
  {
    return ( bez == null ? "" : bez );
  }

  /**
   * Gibt Key und Value getrennt durch ein Komma zurück
   * @return String Key und Value getrennt durch ein Komma
   */
  public String getLegend()
  {
    return getLegend( separator );
  }

  /**
   * Gibt Key und Value getrennt durch den separator zurück
   * @param separator String Trennzeichen
   * @return String Key und Value getrennt durch übergebenes Trennzeichen
   */
  public String getLegend( String dividerSymbol )
  {
    return new String( getKey() + dividerSymbol + getText() );
  }

  /**
   * Überprüft ob der übergebenen Wert dem Key entspricht
   * @param value String
   * @return boolean
   */
  public boolean isEqual( String value )
  {
    return getKey().equals( value );
  }

  public String getBez()
  {
    return ( bez == null ? "" : bez );
  }

  public String getKbez()
  {
    return ( kbez == null ? "" : kbez );
  }

  public void setBez( String bez )
  {
    this.bez = bez;
  }

  public void setKbez( String kbez )
  {
    this.kbez = kbez;
  }
}
