package ch.ige.epband.clientneu;

import javax.jms.JMSException;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.util.ServiceLocator;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung: Sendet die XML an Jipps
 * </p>
 * <p>
 * Copyright: Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * <p>
 * Organisation: IGE
 * </p>
 * 
 * @author Jipps / Trachsel
 * @version $Id: JippsXmlSender.java,v 1.9 2007/01/11 14:16:48 ruegsegger Exp $
 */

public class JippsXmlSender
{
  private Context context = null;

  private QueueConnectionFactory queueConnectionFactory = null;

  private QueueConnection queueConnection = null;

  private QueueSession queueSession = null;

  private QueueSender queueSender = null;

  private Queue queue = null;

  private static final String FACTORYNAME = "jms/IGEQueueConnectionFactory";

  private static final String QUEUENAME = "jms/JIPPS.xmlQueue";

  // Mit diesen Konstanten werden die JMS-Meldungen auf dem Server gespeichert, bis diese
  // gelesenen werden.

  // Modus der Nachrichtenzustellung (persistent oder nicht-pesistent)
  private static final int JMS_DELIVERY_MODE = javax.jms.DeliveryMode.PERSISTENT;

  // Nachrichtenpriorität 0-9 wobei 9 die höchste Priorität bedeutet
  private static final int JMS_PRIORITY = javax.jms.Message.DEFAULT_PRIORITY;

  private static final long JMS_TIME_TO_LIVE = javax.jms.Message.DEFAULT_TIME_TO_LIVE;

  private static final Logger LOG = Logger.getLogger( JippsXmlSender.class );

  /**
   * Konstruktor
   *
   */
  public JippsXmlSender()
  {
    try
    {
      // Naming-Kontext ermitteln
      context = new InitialContext( ServiceLocator.getRemoteJNDIEnvironment());
      
      // Lookup auf Factory
      queueConnectionFactory = (QueueConnectionFactory) context.lookup( FACTORYNAME );
      queueConnection = queueConnectionFactory.createQueueConnection();

      // Nachrichten werden nicht als Teil einer Transaktion gesendet (transacted = false)
      queueSession = queueConnection.createQueueSession( false, Session.AUTO_ACKNOWLEDGE );
      queue = (Queue) context.lookup( QUEUENAME );
      queueSender = queueSession.createSender( queue );
    }
    catch ( NamingException ex )
    {
      LOG.error( "Naming JippsXmlSender : " + ex.getMessage() );
    }
    catch ( JMSException ex )
    {
      LOG.error( "JMS JippsXmlSender : " + ex.getMessage() );
    }
  }

  /**
   * Die eigentliche Sender-methode
   * 
   * @param message Textmessage (String), die gesendet werden soll.
   * @param listenTyp Typ der Liste (z.B. eupat)
   * @param ladeDatum Datum für den Create das Dir bei den PDF-Listen (JIPPS)
   * @throws Exception
   */
  public void sendText( String message, String listenTyp, String ladeDatum ) throws Exception
  {
    TextMessage textMessage = queueSession.createTextMessage();
    textMessage.clearBody();
    textMessage.setStringProperty( "Doktyp", listenTyp ); // Property für den Listentyp setzen
    textMessage.setStringProperty( "LadeDatum", ladeDatum ); // Property für das Ladedatum setzen
    textMessage.setText( message ); // XMLStream in die Message einfügen
    queueSender.send( textMessage, JMS_DELIVERY_MODE, JMS_PRIORITY, JMS_TIME_TO_LIVE );
  }

  /**
   * Schliesst die Queue, die Session und die Connection
   * 
   * @throws Exception
   */
  public void close() throws Exception
  {
    if ( queueSender != null )
    {
      queueSender.close();
    }
    if ( queueSession != null )
    {
      queueSession.close();
    }
    if ( queueConnection != null )
    {
      queueConnection.close();
    }
  }
}
