package ch.ige.epband.clientneu;

import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.util.Pruefziffer;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: Pat.java,v 1.21 2008/05/26 09:02:34 ruegsegger Exp $
 */

public class Pat
{
  private EpbandVO epbandVO;

  private XMLDoc xml;

  private EpbandLogFile logFile = EpbandLogFile.getInstance();

  private IpcrClassification ipcrClassification = null;

  /**
   * Initialisierung der Klasse
   * 
   * @throws Exception
   */
  public Pat() throws Exception
  {
    /**
     * Keine Ahnung warum die leer ist...
     */
  }

  /**
   * Aufbereiten von Entität pat
   * 
   * @param xml
   * @param epbandVO
   * @throws ProcessingException
   */
  public void parseSGML( XMLDoc xmlDoc, EpbandVO epVO ) throws ProcessingException
  {

    this.epbandVO = epVO;
    this.xml = xmlDoc;

    epbandVO.patVO.patVerfahr = 6;

    String anum = xml.getValueOf( "B861", "dnum/anum" );
    if ( anum != null && !anum.equals( "" ) )
    {
      epbandVO.patVO.pctAnmNr = prepPctNr( anum );
      if ( epbandVO.patVO.pctAnmNr.equals( "" ) )
      {
        throw new ProcessingException();
      }
    }
    epbandVO.patVO.pctAnmSpra = xml.getValueOf( "B862" );

    String pnum = xml.getValueOf( "B871", "dnum/pnum" );
    if ( pnum != null && !pnum.equals( "" ) )
    {
      epbandVO.patVO.pctPublNr = Util.prepWONr( pnum );
      if ( epbandVO.patVO.pctPublNr == null || epbandVO.patVO.pctPublNr.equals( "" ) )
      {
        throw new ProcessingException();
      }
    }

    /** TODO wenn tag leer??? */
    epbandVO.patVO.pctPublDat = Util.createDate( xml.getValueOf( "B871", "date" ) );
    epbandVO.patVO.epAnmSpra = xml.getValueOf( "B250" );
    epbandVO.patVO.epApublSpra = xml.getValueOf( "B260" );
    epbandVO.patVO.epApublDat = Util.createDate( xml.getValueOf( "B430", "date" ) );

    // Bei Erteilten EP Datum setzen
    if ( ( epbandVO.epStatus.equalsIgnoreCase( "N" ) || epbandVO.epStatus.equalsIgnoreCase( "C" ) ) && epbandVO.epArt.equals( "B1" ) )
    {
      epbandVO.patVO.epBpublDat = Util.createDate( xml.getValueOf( "B450", "date" ) );
      epbandVO.patVO.epBpublSpra = epbandVO.patVO.epApublSpra;

      if ( epbandVO.patVO.epBpublDat == null )
      {
        String line = "ERROR \t Für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr
            + " ist kein Erteilungsdatum vorhanden. Nichts eingesetzt.";
        logFile.write( line );
      }
    }

    epbandVO.patVO.epStammpatNr = xml.getValueOf( "B620", "parent/pdoc/dnum/pnum" );
    if ( !epbandVO.patVO.epStammpatNr.equals( "" ) )
    {
      while ( epbandVO.patVO.epStammpatNr.length() < 8 )
      {
        epbandVO.patVO.epStammpatNr = "0" + epbandVO.patVO.epStammpatNr;
      }
      epbandVO.patVO.epStammpatNr = Pruefziffer.modulo10_ep( epbandVO.patVO.epStammpatNr );
    }
  }

  /**
   * Umbau der PCT Nummer (AnmeldeNummer)
   * 
   * @param iPctNr String mit AnmeldeNummer
   * @return String; aufbereitete Pct-Nummer
   */
  // TODO nummer wird ab 01.2004 korrekt geliefert ein Umbau ist nicht mehr erforderlich die Länge zu überprüfen ist
  public String prepPctNr( String iPctNr )
  {
    String pctNr = iPctNr;
    // ursprüngliche Version XXyy12345
    if ( pctNr.length() == 9 )
    {
      pctNr = pctNr.substring( 0, 4 ) + "0" + pctNr.substring( 4 );
    }
    // Version ab 1.7.2002 xxyy012345
    if ( pctNr.length() == 10 )
    {
      int yy = Util.isnumeric( pctNr.substring( 2, 4 ) );

      if ( yy == -1 )
      {
        String line = "ERROR \t Jahrzahl bei PCT-Nr " + pctNr + " nicht numerisch. In : " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr
            + " Verarbeitung abgebrochen";
        logFile.write( line );
        return "";
      }
      if ( yy < 30 )
      {
        pctNr = pctNr.substring( 0, 2 ) + "20" + pctNr.substring( 2 );
      }
      else
      {
        pctNr = pctNr.substring( 0, 2 ) + "19" + pctNr.substring( 2 );
      }
    }
    // kein Umbau nötig d Format XXyyyy012345 ab 1.1.2004
    return pctNr;
  }

  /**
   * Aufbereiten von Entität patIpc
   */
  public void genpatIpc()
  {
    if ( xml.exists( "B510EP" ) )
    {
      ipcrClassification = new IpcrClassification( xml, epbandVO );
      epbandVO.patIpcAL = ipcrClassification.getPatIpcVOs();
    }
  }
}
