package ch.ige.epband.clientneu;

import java.text.ParseException;
import java.util.Date;
import java.util.List;

import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.PrioritaetVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: Prioritaet.java,v 1.16 2008/05/26 09:02:34 ruegsegger Exp $
 */

public class Prioritaet
{
  private List<String> blandWork;

  private EpbandLogFile logFile = EpbandLogFile.getInstance();

  private PrioritaetVO prioVO = null;

  private XMLDoc xml = null;

  private EpbandVO epbandVO = null;

  /**
   * Initialierung der Klassenvariablen
   * 
   * @param blandWork
   */
  public Prioritaet( List<String> blandWork )
  {
    this.blandWork = blandWork;
  }

  /**
   * Aufbereiten von Entität PRIORITAET B310: Number B320 <DATE>: Datum B330 <CTRY>: Land There is no tag to mark the
   * beginning of the repeatable group for tags <B310>, <B320> and <B330>. However these tags are always presented in
   * the same sequence.
   * 
   * @param xml
   * @param epbandVO
   */
  public void parseSGML( XMLDoc xmlDoc, EpbandVO epVO )
  {
    this.xml = xmlDoc;
    this.epbandVO = epVO;

    int sequences = xml.getNumberOf( "B310" );

    for ( int i = 1; i <= sequences; i++ )
    {
      prioVO = new PrioritaetVO();

      prioVO.prioNr = this.getPrioNr( i );
      prioVO.prioDat = this.getPrioDat( i );
      prioVO.blandCd = this.getBlandCd( i );

      prioVO.prioLnr = i;

      if ( epbandVO.schutitVO.registerKz == 1 )
      {
        prioVO.regStat = 9;
      }
      else
      {
        prioVO.regStat = 8;
      }
      epbandVO.prioAL.add( prioVO );
    }
  }

  /**
   * Gibt die Prioritätsnummer der übergebenen Sequenz zurück Falls keinen Prioritätsnummer exisitert, wird ein Eintrag
   * ins Logfile geschrieben und null zurückgegeben
   * 
   * @param sequence int
   * @return String Priortätsnummer, falls nicht vorhanden null
   */
  private String getPrioNr( int sequence )
  {
    String prioNr = xml.getValueOf( "B310[" + sequence + "]" );

    if ( prioNr == null || prioNr.equals( "" ) )
    {
      String line = "WARN  \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Fehlende Prioritaetsnummer ";

      logFile.write( line );
      return null;
    }
    else
    {
      return prioNr;
    }
  }

  /**
   * Gibt das Prioritätsdatum der übergebenen Sequenz zurück Falls kein Prioritätsdatum exisitert, wird ein Eintrag ins
   * Logfile geschrieben und null zurückgegeben
   * 
   * @param sequence int
   * @return String Prioritätsdatum, falls nicht vorhanden oder ungültig null
   */
  private Date getPrioDat( int sequence )
  {
    Date prioDat = null;

    String prioDatum = xml.getValueOf( "B320[" + sequence + "]", "date" );

    try
    {
      prioDat = Util.stringToDate( prioDatum );
    }
    catch ( ParseException ex )
    {
      String line = "WARN  \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Ungueltiges Prioritaetsdatum <" + prioDatum
          + "> in Prioritaet <" + prioVO.prioNr + "> kein Datum eingesetzt ";

      logFile.write( line );
    }
    return prioDat;
  }

  /**
   * Gibt das Prioritätsland der übergebenen Sequenz zurück Falls kein Prioritätsland exisitert, wird ein Eintrag ins
   * Logfile geschrieben und CH zurückgegeben, da ein Wert von der Datenbank verlangt wird
   * 
   * @param sequence int
   * @return String Prioritätsland, falls nicht vorhanden CH
   */
  private String getBlandCd( int sequence )
  {
    String blandCd = xml.getValueOf( "B330[" + sequence + "]", "ctry" );

    if ( !blandWork.contains( blandCd ) )
    {
      String line = "WARN  \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Ungueltiges Prioritaetsland <" + blandCd
          + "> in Prioritaet <" + prioVO.prioNr + "> CH eingesetzt";

      logFile.write( line );
      blandCd = "CH";
    }

    return blandCd;
  }
}
