package ch.ige.epband.clientneu;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ch.ige.epband.exception.ProcessingException;
import ch.ige.epband.util.MappingUnicode;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.SchutitVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author A.Rueegsegger
 * @version $Id: Schutit.java,v 1.27 2008/05/26 09:02:34 ruegsegger Exp $
 */

public class Schutit
{
  private EpbandLogFile logFile = EpbandLogFile.getInstance();

  private EpbandVO epbandVO = null;

  /**
   * Aufbereiten von Entität schutit
   * @param xml
   * @param epbandVO
   * @throws Exception
   */
  public void parseSGML( XMLDoc xml, EpbandVO epVO ) throws Exception
  {
    this.epbandVO = epVO;

    epbandVO.schutitVO.schutitTyp = SchutitVO.SCHUTIT_TYP_EP;
    epbandVO.schutitVO.sprache = getAnmeldeSprache( xml );

    if ( epbandVO.epStatus.equalsIgnoreCase( "N" ) && epbandVO.epArt.equals( "B1" ) )
    {
      epbandVO.schutitVO.registerKz = SchutitVO.REGISTER; // im Schutztitel-Register eingetragen
    }
    else
    {
      epbandVO.schutitVO.registerKz = SchutitVO.GESUCH; // Gesuchsstadium
    }

    epbandVO.schutitVO.schutzBegin = epbandVO.anmeldeDate;
    // schutzende = 20 jahre minus 1 Tag
    GregorianCalendar cal = new GregorianCalendar();
    cal.setTime( epbandVO.anmeldeDate );
    cal.add( Calendar.YEAR, 20 );
    cal.add( Calendar.DATE, -1 );
    epbandVO.schutitVO.schutzEnde = cal.getTime();

    // Falls bei B1 (Eintragung) Zurückweisung, Zurücknahme oder
    // Zurücknahme Fiktion vorhanden, wird Schutztitel gelöscht.
    // Dies wird beim Erstellen der betr. Ereignisse gemacht.

    String beschreibung = getBeschreibung( xml );

    if ( beschreibung != null && !beschreibung.equals( "" ) )
    {
      epbandVO.schutitVO.beschreibung = beschreibung;

      epbandVO.schutitVO.uppBeschreibung = beschreibung.toUpperCase();
      if ( epbandVO.schutitVO.uppBeschreibung.length() > 340 )
      {
        epbandVO.schutitVO.uppBeschreibung = epbandVO.schutitVO.uppBeschreibung.substring( 0, 340 );
      }
    }

    epbandVO.schutitVO.erfassDat = new Date();
    epbandVO.schutitVO.erstEreigDat = epbandVO.anmeldeDate;

    // Bei Erteilung muss Fälligjahr ab Erteilungsdatum bestimmt werden,
    // ansonsten 2300 einsetzen
    if ( epbandVO.epArt.equals( "B1" ) && epbandVO.epStatus.equalsIgnoreCase( "N" ) )
    {
      String date = xml.getValueOf( "B450", "date" );
      int faelligJahr = 0;

      if ( Util.isnumeric( date ) != -1 && date.length() == 8 )
      {
        faelligJahr = Util.isnumeric( date.substring( 0, 4 ) );

        /** TODO funktion wofür abklären wie das fälligjahr berechnet wird??? */
        // MMDD vergleichen
        if ( epbandVO.anmeldedateString.substring( 4 ).compareTo( date.substring( 4 ) ) < 0 )
        {
          ++faelligJahr;
        }
        if ( Util.isnumeric( epbandVO.anmeldedateString.substring( 0, 4 ) ) <= 1995
            && Util.isnumeric( epbandVO.anmeldedateString.substring( 0, 4 ) ) + 2 > faelligJahr )
        {
          faelligJahr = Util.isnumeric( epbandVO.anmeldedateString.substring( 0, 4 ) ) + 2;
        }
        else if ( Util.isnumeric( epbandVO.anmeldedateString.substring( 0, 4 ) ) > 1995
            && Util.isnumeric( epbandVO.anmeldedateString.substring( 0, 4 ) ) + 4 > faelligJahr )
        {
          faelligJahr = Util.isnumeric( epbandVO.anmeldedateString.substring( 0, 4 ) ) + 4;
        }

        epbandVO.schutitVO.faelligJahr = faelligJahr;
      }
      else
      {
        epbandVO.schutitVO.faelligJahr = 2300;
        String line = "ERROR \t Für " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " kein gültiges Erteilungsdatum gefunden."
            + " fällig_jahr des Schutztitels auf 2300 gesetzt";
        logFile.write( line );
      }
    }
    else
    {
      epbandVO.schutitVO.faelligJahr = 2300;
    }
  }

  /**
   * Falls die Publikations Sprache des Schutztitels "de" oder "fr" wird die Beschreibung in der entsprechenden Sprache
   * gesucht und in schutit.beschreibung abgespeichert Falls die Publikations Sprache "en" wird in schutit.beschreibung
   * die deutsche Beschreibung und im ereigTxtVO.ereigTxt die französiche Beschreibung gespeichert Falls die
   * Publikations Sprache weder "de", "fr" noch "en" ist, wird keine Beschreibung gewählt
   * 
   * @param xml XMLDoc
   * @return String Beschreibung, null falls Übersetzung fehlt oder Publikations Sprache weder "de", "fr" noch "en"
   */
  private String getBeschreibung( XMLDoc xml ) throws Exception
  {
    String beschreibung = null;
    String publSprache = xml.getValueOf( "B260" ); // "" falls nicht vorhanden

    Map<String, String> map = getBeschreibungSprachMapping( xml );
    if ( map.size() > 0 )
    {
      if ( publSprache.equals( "de" ) || publSprache.equals( "en" ) )
      {
        beschreibung = map.get( "de" ); // null falls nicht enthalten
      }
      else if ( publSprache.equals( "fr" ) )
      {
        beschreibung = map.get( "fr" ); // null falls nicht enthalten
      }
    }
    else
    {
      String line = "WARN  \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr
          + " Kein Titel-Element gefunden oder nicht mit Sprache übereinstimmend. Titel muss überpüft werden";
      logFile.write( line );
    }

    if ( beschreibung != null && !beschreibung.equals( "" ) )
    {
      beschreibung = MappingUnicode.getInstance().replaceUnicodeValues( beschreibung );

      if ( beschreibung.length() > 340 )
      {
        String line = "WARN  \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Titel auf 340 Zeichen gekürzt <" + beschreibung
            + ">";
        logFile.write( line );
        beschreibung = beschreibung.substring( 0, 340 );
      }
    }
    else
    {
      if ( epbandVO.isPatent() || ( epbandVO.isGesuch() && epbandVO.schutitVO.schutitNr != null && !epbandVO.schutitVO.schutitNr.equals( "" ) ) )
      {
        String line = "WARN  \t EP " + epbandVO.schutitVO.schutitNr + "/" + epbandVO.schutitVO.gesuchNr + " Uebersetzung fuer Publikationssprache <"
            + publSprache + "> nicht vorhanden. Leerer Titel wird gespeichert";
        logFile.write( line );
      }
      else
      {
        // Auf jedem EP Band hat es EP-Gesuche die keinen Titel und keine Publikationsnummer (schutitnr) haben.
        // Es handelt sich dabei um veröffentlichte PCT mit Benennung EP,
        // die aber nie in die europäische Phase eingetreten sind.
        // Die entsprechenden EP-Gesuche werden nicht mehr in BAGIS geladen.
        throw new ProcessingException();
      }
    }

    return beschreibung;
  }

  /**
   * Gibt die Beschreibung gemappt mit der Sprache zurück <!ELEMENT B540 (B541? , B542, B542EP?)+> Definition im DTD
   * nicht restriktiv sprache und text gehören nicht zwingend zusammen, da kein Gruppentag vorhanden Reihenfolge ist
   * nicht klar, da sprache optional Bemerkung ar 30.05.2006: Elemente im XML gefunden, die nicht dem DTD entsprachen.
   * Wird gemäss rodriguez korrigiert per Mitte Juli <B540> <B541>de</B541> <B541>en</B541> <B542>Re-onyx solution
   * formula</B542> <B541>fr</B541> </B540> <B540> <B542>Re-onyx formula</B542> <B541>en</B541> <B542>Re-onyx
   * solution formula</B542> <B542>onyx solution formula</B542> </B540>
   * 
   * @param xml XMLDoc
   * @return Map leere HashMap falls keine Einträge oder die Anzahl der Einträge nicht mit den Sprachen übereinstimmen
   */
  public Map<String, String> getBeschreibungSprachMapping( XMLDoc xml )
  {
    Map<String, String> map = new HashMap<String, String>();

    List<String> listSprache = xml.getValuesOf( "B541" );
    List<String> listBeschreibung = xml.getValuesOf( "B542" );

    if ( listBeschreibung.size() == listSprache.size() && listBeschreibung.size() > 0 )
    {
      for ( int i = 0; i < listBeschreibung.size(); i++ )
      {
        map.put( listSprache.get( i ), listBeschreibung.get( i ) );
      }
    }

    return map;
  }

  /**
   * Gibt die Anmeldesprache des Schutztitels zurück [B250]
   * 
   * @param xml XMLDoc
   * @return int
   */
  private int getAnmeldeSprache( XMLDoc xml )
  {
    // Sprache abspeichern
    String sprache = xml.getValueOf( "B250" );

    if ( sprache.equalsIgnoreCase( "de" ) )
    {
      return 1;
    }
    else if ( sprache.equalsIgnoreCase( "fr" ) )
    {
      return 2;
    }
    else if ( sprache.equalsIgnoreCase( "it" ) )
    {
      return 3;
    }
    else
    {
      return 4;
    }
  }
}
