package ch.ige.epband.clientneu.test;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.util.ArrayList;
import java.util.List;

import ch.ige.adr.server.AdresseEpFacade;
import ch.ige.adr.server.AdresseEpFacadeHome;
import ch.ige.epband.util.XMLDoc;
import ch.ige.epband.vo.EpbandVO;
import ch.ige.epband.vo.SchutitRegadrVO;
import ch.ige.epband.vo.SchutitVO;
import ch.ige.util.ServiceLocator;
import ch.ige.util.vo.AdrRegadrNrVO;
import ch.ige.util.vo.RegadrVO;

/**
 * <p>
 * Überschrift: Europäische Patente (EP) einlesen
 * </p>
 * <p>
 * Beschreibung:
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * 
 * @author unbekannt
 * @version $id: $
 */

public class MissingAddress
{
  private static MissingAddress missingAddress;

  private AdresseEpFacade adresseEpFacade;

  private static int count = 0;

  // Singleton
  private MissingAddress() throws Exception
  {
    try
    {
      AdresseEpFacadeHome adresseEpFacadeHome = ServiceLocator.getInstance().getHome( "AdresseEpFacade", AdresseEpFacadeHome.class );
      adresseEpFacade = adresseEpFacadeHome.create();
    }
    catch ( Exception ex )
    {
      System.out.println( ex );
    }
  }

  /**
   * Gibt die einzige Instanz zurück (Singleton)
   * 
   * @return EpbandDelegate
   * @throws Exception
   */
  public static MissingAddress getInstance() throws Exception
  {
    if ( missingAddress == null )
    {
      missingAddress = new MissingAddress();
    }
    return missingAddress;
  }

  /**
   * @param epbandVO
   * @param xml
   * @param regadrVO
   * @param adrRolleCd
   */
  public void saveAddress( EpbandVO epbandVO, XMLDoc xml, RegadrVO regadrVO, int adrRolleCd )
  {
    System.out.println( ++count );

    List<String> list = new ArrayList<String>();
    StringBuffer sb = new StringBuffer();
    String delimiter = "|";

    list.add( xml.getValueOf( "B405", "date" ) );
    list.add( xml.getValueOf( "B405", "bnum" ) );
    list.add( xml.getValueOf( "B130" ) );
    list.add( epbandVO.schutitVO.schutitNr );
    list.add( epbandVO.schutitVO.gesuchNr );
    list.add( String.valueOf( adrRolleCd ) );
    list.add( regadrVO.getRegNameOrEmptyString() );
    list.add( regadrVO.getRegStrasseOrEmptyString() );
    list.add( regadrVO.getRegOrtOrEmptyString() );
    list.add( regadrVO.getBlandCdOrEmptyString() );
    list.add( regadrVO.getUpRegOrtOrEmptyString() );
    list.add( regadrVO.getUpVerzNameOrEmptyString() );
    list.add( String.valueOf( regadrVO.getHerkAdrNr() ) );

    try
    {
      AdrRegadrNrVO adrNrVO = getRegadrNr( epbandVO, regadrVO, adrRolleCd );

      list.add( String.valueOf( adrNrVO.regadrNr ) );
      list.add( String.valueOf( adrNrVO.adrNr ) );
    }
    catch ( Exception ex1 )
    {
      ex1.printStackTrace();
    }

    for ( int i = 0; i < list.size(); i++ )
    {
      sb.append( list.get( i ) );
      sb.append( delimiter );
    }

    writeAddress( sb.toString() );
  }

  private void writeAddress( String s )
  {
    try
    {
      File file = new File( "c:\\temp\\fehlende_inhaberadressen.txt" );
      RandomAccessFile randomFile = new RandomAccessFile( file, "rw" );
      if ( randomFile.length() > 0 )
      {
        randomFile.seek( randomFile.length() );
      }
      else
      {
        randomFile.writeBytes( "Publ-Dat|Publ-Week|Kind|Schutit-Nr|Gesuch-Nr|Rolle|"
            + "Reg-Name|Reg-Strasse|Reg-Ort|Land|UpRegOrt|UpVerzName|HerkAdrNr|Regadr-Nr|Adr-Nr|\r\n" );
      }
      randomFile.writeBytes( s + "\r\n" );
      randomFile.close();
    }
    catch ( IOException ex )
    {
      System.out.println( s );
      System.out.println( ex );
    }
  }

  /**
   * holen der Registeradressnummer
   * 
   * @param epbandVO
   * @param regadrVO
   * @param adrRolleCd
   * @return AdrRegadrNrVO
   * @throws Exception
   */
  public AdrRegadrNrVO getRegadrNr( final EpbandVO epbandVO, final RegadrVO regadrVO, final int adrRolleCd ) throws Exception
  {
    AdrRegadrNrVO adrRegadrNrVO = new AdrRegadrNrVO();
    String sprachCd;

    if ( epbandVO.patVO.epApublSpra.equalsIgnoreCase( "DE" ) )
    {
      sprachCd = "D";
    }
    else if ( epbandVO.patVO.epApublSpra.equalsIgnoreCase( "FR" ) )
    {
      sprachCd = "F";
    }
    else
    {
      sprachCd = "USA";
    }

    if ( adrRolleCd == SchutitRegadrVO.ERFINDER || epbandVO.schutitVO.registerKz == SchutitVO.GESUCH )
    {
      // nur Registeradresse kreieren bei Erfinder oder EP im Gesuchstatium
      adrRegadrNrVO.regadrNr = adresseEpFacade.createRegadrEp( regadrVO );
    }
    else
    {
      adrRegadrNrVO = adresseEpFacade.createRegadrVersadrEp( regadrVO, sprachCd );
    }
    return adrRegadrNrVO;
  }
}
