package ch.ige.epband.server.dao;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Types;
import java.util.HashMap;

import oracle.jdbc.OracleConnection;
import oracle.jdbc.OraclePreparedStatement;
import ch.ige.dbutils.ds.OracleConnectionExtractor;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für die Tabelle MappingUnicode
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author A.Rueegsegger
 * @version $Id: MappingUnicodeDAO.java,v 1.9 2008/02/28 14:58:03 wild Exp $
 */

public class MappingUnicodeDAO
{

  /**
   * Selektiert alle Werte der Tabelle mapping_unicode
   * 
   * @throws SQLException
   * @return Map HashMap mit unicode - iso885915 mapping
   */
  public HashMap<String, String> selectAll() throws SQLException
  {
    Connection con = null;
    OracleConnection oracleCon = null;
    OraclePreparedStatement oraclePs = null;
    String query = null;
    ResultSet rs = null;
    HashMap<String, String> map = new HashMap<String, String>();
    String key = null;
    String value = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      // Für den Zugriff auf NCHAR Attribute wird eine OracleConnection benötigt
      // andernfalls wird eine Konvertierung auf das Database Character Set durchgeführt
      // dieser Code funktioniert nur, sofern DBHelper.getConnection auf dem AppServer durchgführt wird
      oracleCon = OracleConnectionExtractor.getOracleConnection( con );

      query = "SELECT unicode_value, iso_value FROM MAPPING_UNICODE";
      oraclePs = (oracle.jdbc.OraclePreparedStatement) oracleCon.prepareStatement( query );
      oraclePs.defineColumnType( 1, Types.VARCHAR, 1, OraclePreparedStatement.FORM_NCHAR );
      rs = oraclePs.executeQuery();

      while ( rs.next() )
      {
        key = rs.getString( "unicode_value" );
        value = rs.getString( "iso_value" );
        map.put( key, value );
      }
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, MappingUnicodeDAO.class, query );
      throw ex;
    }
    finally
    {
      // Die Äussere Connection muss geschlossen werden, ansonsten bekommt Appserver
      // Probleme mit dem Transaction-Handling und versucht ein two phase commit zu machen
      DBHelper.close( oraclePs, con );
    }

    return map;
  }
}
