package ch.ige.epband.server.dao;

import java.sql.*;

import ch.ige.epband.vo.PatVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;


/**
 * DataAccessObject für PAT
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Michael Zaugg
 * @version $Id: PatDAO.java,v 1.10 2006/12/18 14:45:16 wild Exp $
 */

public class PatDAO
{

  /**
   * Fügt neue Daten in die Tabelle PAT ein.
   * @param patVO
   * @return int
   * @throws SQLException
   */
  public int insert( PatVO patVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "INSERT INTO pat(" +
          "schutit_key, " +
          "pat_verfahr, " +
          "pct_anm_nr, " +
          "pct_anm_spra, " +
          "pct_publ_nr, " +
          "pct_publ_spra, " +
          "pct_publ_dat, " +
          "ep_anm_spra, " +
          "ep_apubl_spra, " +
          "ep_apubl_dat, " +
          "ep_bpubl_spra, " +
          "ep_bpubl_dat, " +
          "ep_stammpat_nr) " +
          "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ? ) " ;

      ps = con.prepareStatement( query );
      ps.setInt( 1, patVO.schutitKey );
      ps.setInt( 2, patVO.patVerfahr );
      ps.setString( 3, patVO.pctAnmNr );
      ps.setString( 4, patVO.pctAnmSpra );
      ps.setString( 5, patVO.pctPublNr );
      ps.setString( 6, patVO.pctPublSpra );
      if( patVO.pctPublDat == null )
      {
        ps.setDate( 7, null );
      }
      else
      {
        ps.setDate( 7, new java.sql.Date( patVO.pctPublDat.getTime() ) );
      }
      ps.setString( 8, patVO.epAnmSpra );
      ps.setString( 9, patVO.epApublSpra );
      if( patVO.epApublDat == null )
      {
        ps.setDate( 10, null );
      }
      else
      {
        ps.setDate( 10, new java.sql.Date( patVO.epApublDat.getTime() ) );
      }
      ps.setString( 11, patVO.epBpublSpra );
      if( patVO.epBpublDat == null )
      {
        ps.setDate( 12, null );
      }
      else
      {
        ps.setDate( 12, new java.sql.Date( patVO.epBpublDat.getTime() ) );
      }
      ps.setString( 13, patVO.epStammpatNr );

      return ps.executeUpdate();
    }
    catch( SQLException ex )
    {
      LogLevelHelper.log( ex, PatDAO.class, query + patVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}

