package ch.ige.epband.util;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.jdom.Attribute;
import org.jdom.Document;
import org.jdom.Element;
import org.jdom.JDOMException;
import org.jdom.filter.Filter;
import org.jdom.input.SAXBuilder;
import org.jdom.xpath.XPath;

/**
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class XMLDoc
{
  private Document doc;

  private XPath x;

  private Element e;

  private Attribute att;

  /**
   * Konstruktor mit String
   * @param xmlString
   * @throws Exception
   */
  public XMLDoc( String xmlString ) throws Exception
  {
    // wird kein charset angegeben, wird das default-charset des betriebssystems genommen (hier cp1252)
    // was bei den sonderzeichen zu probleme führen kann
    this( new ByteArrayInputStream( xmlString.getBytes( "UTF-8" ) ) );
  }

  /**
   * Konstruktor mit Stream
   * @param stream
   * @throws Exception
   */
  public XMLDoc( InputStream stream ) throws Exception
  {
    // Build the document with SAX and Xerces, no validation 
    SAXBuilder builder = new SAXBuilder( "org.apache.xerces.parsers.SAXParser" );

    // Pfad DTD
    String systemId = "file:///c:/Programme/ige/epband/config/";

    // Create the document
    doc = builder.build( stream, systemId );
  }

  /**
   * Wert von einem Element zurückgeben
   * @param element
   * @return Wert des Elements
   */
  public String getValueOf( String element )
  {
    return getValueOf( element, null );
  }

  /**
   *
   * @param parent String
   * @param element String
   * @return String falls Element nicht existiert wird "" zurückgegeben
   */
  public String getValueOf( String parent, String element )
  {
    try
    {
      x = XPath.newInstance( getXPath( parent, element ) );
      e = (Element) x.selectSingleNode( doc );
      return ( e == null ? "" : e.getText() );
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return null;
    }
  }

  /**
  * Anhand der Liste der der Elemente wird der Pjad für XPath zusammengestellt. Und der Wert zurückgegeben.
  * Es muss der ganze Pfad angegeben werden. Es ist auch möglich die Angaben von einem Knoten 
  * mit Attributangaben zu ergänzen. ( Bsp: B090[@id="3"]
  * @param elementList Liste von Elementen
  * @return String falls Element nicht existiert wird "" zurückgegeben
  */
  public String getValueOf( List<String> elementList )
  {
    try
    {
      StringBuilder path = new StringBuilder();
      path.append( "./" );

      for ( int i = 0; i < elementList.size(); i++ )
      {
        path = path.append( "/" + elementList.get( i ) );
      }

      x = XPath.newInstance( path.toString() );
      e = (Element) x.selectSingleNode( doc );
      return ( e == null ? "" : e.getText() );
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return null;
    }
  }

  /**
   * Bringt alle Werte zurück, die innerhalb von einem Element mit dem übergebenen Namen ist. 
   * @param element Name des Tags
   * @return Liste von Strings. 
   */
  public List<String> getValuesOf( String element )
  {
    return getValuesOf( element, null );
  }

  /**
   * Bringt alle Werte zurück, die mit übergebenen der Parten/Element Kombination gefunden werden. 
   * 
   * @param parent 
   * @param element
   * @return Liste von Strings
   */
  @SuppressWarnings( "unchecked" )
  public List<String> getValuesOf( String parent, String element )
  {
    List<Element> elementList;
    List<String> valueList = new ArrayList<String>();

    try
    {
      x = XPath.newInstance( getXPath( parent, element ) );
      elementList = x.selectNodes( doc );

      for ( int i = 0; i < elementList.size(); i++ )
      {
        @SuppressWarnings( "hiding" )
        Element e = elementList.get( i );
        valueList.add( e.getText() );
      }
    }
    /** TODO exception handling */
    catch ( JDOMException ex )
    {
      System.out.println( ex );
    }

    return valueList;
  }

  /**
   * Wert des gesuchten Attributes wird zurückgegeben.
   * Die Parameter werden zwar komisch übergeben (element wird im Parent-Parameter übergeben), aber 
   * aufgrund der Logik in der Methode getXPath klappt am Schluss alles.
   * @param element
   * @param attribute
   * @return Wert des Attributes in String-Form
   */
  public String getAttribute( String element, String attribute )
  {
    return getAttribute( element, null, attribute );
  }

  /**
   * Wert des gesuchten Attributes wird zurückgegeben. Es wird mit Parent und Element gesucht.
   * @param parent
   * @param element
   * @param attribute
   * @return Wert des Attributes in String-Form
   */
  public String getAttribute( String parent, String element, String attribute )
  {
    try
    {
      x = XPath.newInstance( getXPath( parent, element, attribute ) );
      att = (Attribute) x.selectSingleNode( doc );
      return ( att == null ? "" : att.getValue() );
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return null;
    }
  }

  /**
   * Liste von Werten des gesuchten Attributes wird zurückgegeben. Es wird nur aufgrund des Elementes gesucht.
   * Es wird mit Parent und Element gesucht.
   * @param element
   * @param attribute
   * @return Liste von String-Werten
   */
  public List<String> getAttributes( String element, String attribute )
  {
    return getAttributes( element, null, attribute );
  }

  /**
   * Liste von Werten des gesuchten Attributes wird zurückgegeben. 
   * Es wird mit Parent und Element gesucht.
   * @param parent
   * @param element
   * @param attribute
   * @return Liste von String-Werten
   */
  @SuppressWarnings( "unchecked" )
  public List<String> getAttributes( String parent, String element, String attribute )
  {
    List<Attribute> attList;
    List<String> valueList = new ArrayList<String>();

    try
    {
      x = XPath.newInstance( getXPath( parent, element, attribute ) );
      attList = x.selectNodes( doc );
      for ( int i = 0; i < attList.size(); i++ )
      {
        att = attList.get( i );
        valueList.add( att.getValue() );
      }
      return valueList;
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return null;
    }
  }

  /**
   * Alle Attribute von einem Element werden als Map zurückgegen.
   * @param element
   * @return MAP mit String Paar; Name, Wert der Attribute
   */
  public Map<String, String> getAttributesOfElement( String element )
  {
    return getAttributesOfElement( element, null );
  }

  /**
   * Alle Attribute von einem Element werden als Map zurückgegen.
   * @param parent 
   * @param element
   * @return MAP mit String Paar; Name, Wert der Attribute
   */
  @SuppressWarnings( "unchecked" )
  public Map<String, String> getAttributesOfElement( String parent, String element )
  {
    Map<String, String> map = new HashMap<String, String>();

    try
    {
      x = XPath.newInstance( getXPath( parent, element ) );
      e = (Element) x.selectSingleNode( doc );

      List<Attribute> attList = e.getAttributes();

      for ( int i = 0; i < attList.size(); i++ )
      {
        att = attList.get( i );
        map.put( att.getName(), att.getValue() );
      }

      return map;
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return null;
    }
  }

  /**
   * Anzahl der gesuchten Elemente
   * @param element
   * @return Anzahl
   */
  public int getNumberOf( String element )
  {
    try
    {
      x = XPath.newInstance( getXPath( element ) );
      return x.selectNodes( doc ).size();
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return 0;
    }
  }

  /**
   * Sagt aus, ob es ein Element mit dem entsprechenden Namen gibt.
   * @param element
   * @return true oder false 
   */
  public boolean exists( String element )
  {
    return exists( element, null );
  }

  /**
   * Sagt aus, ob es ein Parent/Element mit dem entsprechenden Namen gibt.
   * @param parent 
   * @param element 
   * @return true oder false 
  */
  public boolean exists( String parent, String element )
  {
    try
    {
      x = XPath.newInstance( getXPath( parent, element ) );
      e = (Element) x.selectSingleNode( doc );
      return ( e == null ? false : true );
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
      return false;
    }
  }

  /**
   * Alle Texte die innerhalb von einem bestimmten Attribut sind.
   * @param attribute
   * @return Liste mit den Texten
   */
  @SuppressWarnings( { "unchecked", "synthetic-access" } )
  public List<Pair> getText( String attribute )
  {
    List<Pair> list = new ArrayList<Pair>();

    try
    {
      x = XPath.newInstance( ".//*[@id='" + attribute + "']" );
      List<Element> nodes = x.selectNodes( doc );
      for ( int i = 0; i < nodes.size(); i++ )
      {
        @SuppressWarnings( "hiding" )
        Element e = nodes.get( i );

        if ( e != null )
        {
          if ( e.getText() != null && !e.getTextTrim().equals( "" ) )
          {
            list.add( new Pair( e.getName(), e.getText() ) );
          }

          Iterator<Element> it = e.getDescendants( new ListenFilter() ); // gibt alle Nachfolger Text, Element, Comment, ProcessingInstruction, CDATA, and EntityRef.
          while ( it.hasNext() )
          {
            Object o = it.next();

            if ( o instanceof Element )
            {
              Element descendant = (Element) o;
              if ( descendant != null && ( descendant.getName().equals( "B784" ) || ( descendant.getText() != null && !descendant.getTextTrim().equals( "" ) ) ) )
              {
                list.add( new Pair( descendant.getName(), descendant.getText() ) );
              }
            }
          }
        }
      }
    }
    catch ( JDOMException ex )
    {
      System.out.println( ex );
    }
    return list;
  }

  private String getXPath( String element )
  {
    return getXPath( element, null, null );
  }

  private String getXPath( String parent, String element )
  {
    return getXPath( parent, element, null );
  }

  private String getXPath( String parent, String element, String attribute )
  {
    StringBuffer xpath = new StringBuffer();
    xpath.append( ".//" );
    xpath.append( parent );
    if ( element != null && !element.equals( "" ) )
    {
      xpath.append( "/" );
      xpath.append( element );
    }
    if ( attribute != null && !attribute.equals( "" ) )
    {
      xpath.append( "/@" );
      xpath.append( attribute );
    }
    return xpath.toString();
  }

  private class ListenFilter implements Filter
  {
    /**
     *  Defaultwert für die serialVersionUID
     */
    private static final long serialVersionUID = 1L;

    public boolean matches( Object obj )
    {
      if ( obj instanceof Element )
      {
        @SuppressWarnings( "hiding" )
        Element e = (Element) obj;
        if ( !e.getName().equals( "iid" ) && !e.getName().equals( "dnum" ) && !e.getName().equals( "syn" ) && !e.getName().equals( "irf" ) )
        {
          return true;
        }
      }
      return false;
    }
  }
}
