package ch.ige.image;

import java.io.Externalizable;
import java.io.File;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.ByteBuffer;

/**
 * An Image and its representation (encoded with a codec).
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.image.format.Format#encode(Image, boolean)
 * @author Michael Locher
 * @version $Id: EncodedImage.java,v 1.13 2009/03/30 10:35:27 locher Exp $
 */
public interface EncodedImage extends Image, Externalizable
{
  /**
   * Gets the byte representation of the image.
   * 
   * @return the image data.
   * @deprecated since 2009-03-30. use {@link #getDataBuffer()} instead.
   */
  @Deprecated
  public byte[] getData();

  /**
   * Gets the byte representation of the image.
   * @return the image data.
   */
  public ByteBuffer getDataBuffer();

  /**
   * Calculates the SHA-1 Message Digest of the image.
   * 
   * @return the 40 character hex digest
   */
  public String getDigest();

  /**
   * Gets the size of the represenation.
   * 
   * @return number of bytes.
   */
  public int getFilesize();

  /**
   * Releases all resources which are used to speed up processing. 
   * The image can still be used after this operation.
   * 
   * @see Image#dispose()
   */
  public void flushCache();

  /**
   * Writes the image into the given file.
   * 
   * @param destFile the destination
   * @throws IOException if writing to the file fails
   */
  public void write( File destFile ) throws IOException;

  /**
   * Writes the image into the given stream. 
   * The stream will be flushed but not closed.
   * 
   * @param os the destination
   * @throws IOException if writing to the stream fails
   */
  public void write( OutputStream os ) throws IOException;

  /**
   * Wraps the image in a SVG document and writes the result to the given stream. 
   * The stream will be flushed but not closed.
   * 
   * @param os the destination
   * @throws IOException if writing to the stream fails
   */
  public void writeSVG( OutputStream os ) throws IOException;

  /**
   * Encodes the image with BASE64 and writes the result to the given stream. 
   * The stream will be flushed but not closed.
   * 
   * @param os the destination
   * @throws IOException if writing to the stream fails
   */
  public void writeBase64( OutputStream os ) throws IOException;
}
