package ch.ige.image.color;

import java.awt.color.ColorSpace;
import java.awt.color.ICC_ColorSpace;
import java.awt.color.ICC_Profile;

/**
 * Colorspace utils.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   Michael Locher
 * @version  $Id: ColorSpaceUtil.java,v 1.1 2008/05/21 14:28:36 locher Exp $
 */
public class ColorSpaceUtil
{

  /**
   * Checks if two colorspaces are the same.
   * @param a first color space
   * @param b second color space
   * @return true if they are the same, false if unknown or not
   */
  public static boolean isSurelySameColorSpace( ColorSpace a, ColorSpace b )
  {
    if ( a.isCS_sRGB() && b.isCS_sRGB() )
    {
      return true;
    }
    else if ( a instanceof ICC_ColorSpace && b instanceof ICC_ColorSpace )
    {
      ICC_Profile pa = ( (ICC_ColorSpace) a ).getProfile();
      ICC_Profile pb = ( (ICC_ColorSpace) b ).getProfile();
      return pa.equals( pb );
    }
    else
    {
      return a.equals( b );
    }
  }

  /**
   * Determines if the given ColorSpace is one of the well knowns.
   * @param cs the colorspace
   * @return a description
   */
  public static String getHumanReadable( ColorSpace cs )
  {
    String msg = null;
    if ( cs instanceof ICC_ColorSpace )
    {
      ICC_Profile profile = ( (ICC_ColorSpace) cs ).getProfile();
      if ( ICC_Profile.getInstance( ColorSpace.CS_sRGB ).equals( profile ) )
      {
        msg = "sRGB";
      }
      else if ( ICC_Profile.getInstance( ColorSpace.CS_GRAY ).equals( profile ) )
      {
        msg = "Gray";
      }
      else if ( ICC_Profile.getInstance( ColorSpace.CS_LINEAR_RGB ).equals( profile ) )
      {
        msg = "Linear RGB";
      }
      else if ( ICC_Profile.getInstance( ColorSpace.CS_PYCC ).equals( profile ) )
      {
        msg = "Photo YCC";
      }
      else if ( SMPTEColorSpace.getSMPTE_C_ColorSpace().getProfile().equals( profile ) )
      {
        msg = "SMPTE-C";
      }
    }
    if ( msg == null )
    {
      msg = "Unknown: " + cs.toString();
    }
    return msg;
  }

}
