package ch.ige.image.test;

import java.awt.Dimension;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.nio.ByteBuffer;

import ch.ige.image.EncodedImage;
import ch.ige.image.ImageIOImage;
import ch.ige.image.JAIImage;
import ch.ige.image.MetadataBean;
import ch.ige.image.IOUtil.WritableByteBuffer;
import ch.ige.image.format.Format;

/**
 * Test Serialisation of Images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: ImageSerializableTest.java,v 1.10 2009/03/19 13:26:21 locher Exp $
 */
public class ImageSerializableTest extends AbstractImageTester
{

  public ImageSerializableTest( String name )
  {
    super( name );
  }

  private ByteBuffer load( String name ) throws Exception
  {
    assertNotNull( ImageFactoryTest.class.getResource( name ) );
    return toByteBuffer( ImageSerializableTest.class.getResource( name ).openStream() );
  }

  public void testImageIOImage() throws Exception
  {
    EncodedImage img = this.serializeAndDeserialize( ImageIOImage.parse( this.load( "circle.png" ) ) );
    MetadataBean meta = new MetadataBean( img.getMetadata() );
    Format format = meta.getFormat();

    assertTrue( "Format", format.isKindOf( "png" ) );
    assertEquals( "Compression", "deflate", format.getCompressionType() );
    assertTrue( "Lossless information", format.isLossless() );
    assertEquals( "Size", new Dimension( 64, 64 ), meta.getSize() );
    assertEquals( "Resolution", 72, meta.getDPI() );
    assertEquals( "Colorspace", "RGB", meta.getColorspaceType() );
  }

  public void testImageJAIImage() throws Exception
  {
    JAIImage orig = JAIImage.parse( this.load( "circle.jpg" ) );
    EncodedImage again = this.serializeAndDeserialize( orig );
    MetadataBean meta = new MetadataBean( again.getMetadata() );

    assertTrue( "Format", meta.getFormat().isKindOf( "jpeg" ) );
    assertEquals( "Size", new Dimension( 64, 64 ), meta.getSize() );

  }

  private EncodedImage serializeAndDeserialize( EncodedImage img ) throws IOException, ClassNotFoundException
  {
    ByteArrayOutputStream buffer = new ByteArrayOutputStream();
    ObjectOutputStream os = new ObjectOutputStream( buffer );
    os.writeObject( img );
    os.close();
    ObjectInputStream is = new ObjectInputStream( new ByteArrayInputStream( buffer.toByteArray() ) );
    return (EncodedImage) is.readObject();
  }

  /**
   * Convert InputStream to Byte Array.
   * @param in stream to be read
   * @return Byte array which contains the whole stream or null if in was null
   * @throws IOException
   */
  public static ByteBuffer toByteBuffer( InputStream in ) throws IOException
  {
    if ( in == null )
    {
      return null;
    }
    WritableByteBuffer buffer = new WritableByteBuffer( false );
    try
    {
      byte[] transfer = new byte[8192];
      int count = 0;
      while ( count != -1 )
      {
        count = in.read( transfer );
        if ( count > 0 )
        {
          buffer.write( transfer, 0, count );
        }
      }
      return buffer.getBuffer();
    }
    finally
    {
      buffer.close();
    }
  }

}
