package ch.ige.image.test;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.format.JPEGFormat;

/**
 * Test Encoding in JPEG.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: JPEGEncodingTest.java,v 1.6 2009/03/19 13:26:21 locher Exp $
 */
public class JPEGEncodingTest extends AbstractEncodingTester
{

  private static final String TARGET_FORMAT = JPEGFormat.IDENTIFIER;

  public JPEGEncodingTest(String name)
  {
    super(name);
  }

  @Override
  protected String getTargetType()
  {
    return TARGET_FORMAT;
  }

  public void testGIF() throws Exception
  {
    Image img = this.load("circle.gif");
    EncodedImage encoded = encode(img);
    this.save(encoded, "gif2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testPNG() throws Exception
  {
    Image img = this.load("circle.png");
    EncodedImage encoded = encode(img);
    this.save(encoded, "png2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testJPEG() throws Exception
  {
    Image img = this.load("circle.jpg");
    EncodedImage encoded = encode(img);
    this.save(encoded, "jpeg2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testTIFFnoCompress() throws Exception
  {
    Image img = this.load("circle_noCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_no_2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testTIFFdctCompress() throws Exception
  {
    Image img = this.load("circle_dctCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_dct_2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testTIFFlzwCompress() throws Exception
  {
    Image img = this.load("circle_lzwCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_lzw_2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testBMP() throws Exception
  {
    Image img = this.load("circle.bmp");
    EncodedImage encoded = encode(img);
    this.save(encoded, "bmp2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testA6png() throws Exception
  {
    Image img = this.load("a6_300.png");
    EncodedImage encoded = encode(img);
    this.save(encoded, "png_a6_2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testA6tiff() throws Exception
  {
    Image img = this.load("a6_300.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_a6_2jpeg.jpg");
    postEncoding(img, encoded);
  }

  public void testA6gif() throws Exception
  {
    Image img = this.load("a6_300.gif");
    EncodedImage encoded = encode(img);
    this.save(encoded, "gif_a6_2jpeg.jpg");
    postEncoding(img, encoded);
  }

}
