package ch.ige.image.test;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.format.TIFFFormat;

/**
 * Test Encoding in TIFF.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: TIFFEncodingTest.java,v 1.6 2009/03/19 13:26:21 locher Exp $
 */
public class TIFFEncodingTest extends AbstractEncodingTester
{
 
  private static final String TARGET_FORMAT = TIFFFormat.IDENTIFIER;
  
  public TIFFEncodingTest(String name)
  {
    super(name);
  }
  
  @Override
  protected String getTargetType()
  {
    return TARGET_FORMAT;
  }

  public void testGIF() throws Exception
  {
    Image img = this.load("circle.gif");
    EncodedImage encoded = encode(img);
    this.save(encoded, "gif2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testPNG() throws Exception
  {
    Image img = this.load("circle.png");
    EncodedImage encoded = encode(img);
    this.save(encoded, "png2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testJPEG() throws Exception
  {
    Image img = this.load("circle.jpg");
    EncodedImage encoded = encode(img);
    this.save(encoded, "jpeg2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testTIFFnoCompress() throws Exception
  {
    Image img = this.load("circle_noCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_no_2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testTIFFdctCompress() throws Exception
  {
    Image img = this.load("circle_dctCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_dct_2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testTIFFlzwCompress() throws Exception
  {
    Image img = this.load("circle_lzwCompress.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_lzw_2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testBMP() throws Exception
  {
    Image img = this.load("circle.bmp");
    EncodedImage encoded = encode(img);
    this.save(encoded, "bmp2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testA6png() throws Exception
  {
    Image img = this.load("a6_300.png");
    EncodedImage encoded = encode(img);
    this.save(encoded, "png_a6_2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testA6tiff() throws Exception
  {
    Image img = this.load("a6_300.tiff");
    EncodedImage encoded = encode(img);
    this.save(encoded, "tiff_a6_2tiff.tif");
    postEncoding(img, encoded);
  }

  public void testA6gif() throws Exception
  {
    Image img = this.load("a6_300.gif");
    EncodedImage encoded = encode(img);
    this.save(encoded, "gif_a6_2tiff.tif");
    postEncoding(img, encoded);
  }

}
