package ch.ige.image.test;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.util.Arrays;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.apache.commons.jxpath.JXPathContext;
import org.w3c.dom.Document;
import org.xml.sax.EntityResolver;
import org.xml.sax.InputSource;

import sun.misc.BASE64Decoder;
import ch.ige.image.EncodedImage;
import ch.ige.image.ImageFactoryImpl;

/**
 * Test Image IO via SVG images.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author    Michael Locher
 * @version   $Id: XMLImageTest.java,v 1.11 2009/04/07 11:31:18 locher Exp $
 */
public class XMLImageTest extends AbstractImageTester
{

  public XMLImageTest( String name )
  {
    super( name );
  }

  private EncodedImage load( String name ) throws Exception
  {
    assertNotNull( ImageFactoryTest.class.getResource( name ) );
    return new ImageFactoryImpl().loadURL( ImageFactoryTest.class.getResource( name ) );
  }

  protected File getOutputDirectory()
  {
    File dir = new File( AllTests.OUTPUT_DIRECTORY, "imageTest" );
    if ( !dir.exists() && !dir.mkdirs() )
    {
      throw new RuntimeException( "could not create output directory" );
    }
    return dir;
  }

  public void testExportSVG() throws Exception
  {
    EncodedImage img = this.load( "circle.png" );
    OutputStream fos = new FileOutputStream( new File( this.getOutputDirectory(), "circlePNG.svg" ) );
    try
    {
      img.writeSVG( fos );
    }
    finally
    {
      fos.close();
    }
  }

  public void testVerifySVG() throws Exception
  {
    ByteArrayOutputStream buffer = new ByteArrayOutputStream();
    EncodedImage img = this.load( "circle.png" );
    img.writeSVG( buffer );
    buffer.flush();

    ByteArrayOutputStream reference = new ByteArrayOutputStream();
    img.writeBase64( reference );
    reference.flush();

    DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
    DocumentBuilder builder;
    try
    {
      //Xerces
      factory.setAttribute( "http://apache.org/xml/features/nonvalidating/load-external-dtd", Boolean.valueOf( false ) );
      builder = factory.newDocumentBuilder();
    }
    catch ( Exception e )
    {
      //Crimson
      builder = factory.newDocumentBuilder();
      builder.setEntityResolver( new EntityResolver()
      {
        public InputSource resolveEntity( String publicId, String systemId )
        {
          return new InputSource( new StringReader( "" ) );
        }
      } );
    }

    Document reloaded = builder.parse( new InputSource( new ByteArrayInputStream( buffer.toByteArray() ) ) );

    JXPathContext context = JXPathContext.newContext( reloaded );
    String data = (String) context.getValue( "//@xlink:href" );

    String encodedFromReference = reference.toString();

    String[] encodedFromSVG = data.substring( 7 + data.indexOf( "base64," ) ).split( " " );
    StringBuffer encodedFromSVGwithoutSpaces = new StringBuffer();
    for ( String element: encodedFromSVG )
    {
      encodedFromSVGwithoutSpaces.append( element );
    }

    byte[] bytesFromSVG = new BASE64Decoder().decodeBuffer( encodedFromSVGwithoutSpaces.toString() );
    byte[] bytesFromReference = new BASE64Decoder().decodeBuffer( encodedFromReference );

    assertTrue( "data not preserved", Arrays.equals( bytesFromReference, bytesFromSVG ) );
  }
}
