package ch.ige.imagehandler.profile;

import java.util.List;

import ch.ige.imagehandler.rule.RuleException;
import ch.ige.imagehandler.rule.RuleList;

/**
 * Checking and Tranformation Profile.
 * <p>
 * A profile is a collection of named rulesets for image checking and transformation. A profile owns a collection of
 * environment values.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @see ch.ige.imagehandler.profile.ProfileImpl
 * @author Michael Locher
 * @version $Id: Profile.java,v 1.6 2009/04/14 13:28:42 locher Exp $
 */
public interface Profile
{

  public String DEFAULT_DPI_NAME = "defaultResolutionDPI";

  /**
   * Checks the consistency of the rules.
   * 
   * @throws RuleException if invalid or missing parameters are found.
   * @see ch.ige.imagehandler.rule.Rule#checkParameters()
   */
  public void checkProfile() throws RuleException;

  /**
   * Gives all definied rulesets.
   * 
   * @return all rulesets.
   */
  public List<RuleList> getAllRules();

  /**
   * Gives all definied converter rulesets.
   * 
   * @return all converter rulesets.
   */
  public List<RuleList> getAllConverterRules();

  /**
   * Gives the ruleset of the given name.
   * 
   * @param name the requested ruleset.
   * @throws IllegalArgumentException if no such ruleset exists.
   * @return the requested ruleset.
   */
  public RuleList getRules( String name );

  /**
   * Gives the value of the requested environment parameter.
   * 
   * @param name the requested parameter.
   * @return the value.
   * @see #DEFAULT_DPI_NAME
   */
  public String getEnvironmentValue( String name );

}
