package ch.ige.imagehandler.rule;

import java.awt.RenderingHints;
import java.awt.image.renderable.ParameterBlock;

import javax.media.jai.BorderExtender;
import javax.media.jai.InterpolationBilinear;
import javax.media.jai.JAI;
import javax.media.jai.PlanarImage;

import ch.ige.image.EncodedImage;
import ch.ige.image.Image;
import ch.ige.image.JAIImage;

/**
 * Supports trademark related image rules.
 *
 * <p>Copyright (c) 2009, Swiss Federal Institute of Intellectual Property</p>
 * @author   galli
 * @version  $Id: AbstractTrademarkRule.java,v 1.3 2009/04/14 12:10:35 galli Exp $
 */
public abstract class AbstractTrademarkRule<I extends EncodedImage> extends AbstractRule<I> implements Converter<I, Image>
{

  // @see ch.ige.imagehandler.rule.AbstractRule#getRuleName()
  @Override
  protected String getRuleName()
  {
    return "Abstract Trademark Rule";
  }

  //----------------------------------------------------------------------------------------
  //Convert

  private static final RenderingHints SCALE_HINT = new RenderingHints( JAI.KEY_BORDER_EXTENDER, BorderExtender.createInstance( BorderExtender.BORDER_COPY ) );

  protected PlanarImage jaiPipeline( final JAIImage parsed, double xScale, double yScale )
  {
    PlanarImage pipe = scale( parsed.asRenderedImage(), xScale, yScale );
    if ( pipe.getSampleModel().getNumBands() == 4 )
    {
      pipe = cmykToRGB( pipe );
    }
    return pipe;
  }

  private PlanarImage scale( PlanarImage img, double xScale, double yScale )
  {
    if ( ( Math.abs( xScale - 1 ) + Math.abs( yScale - 1 ) ) > 0.01 )
    { //do not touch anything for scale-factor around 1
      ParameterBlock pb = new ParameterBlock();
      pb.addSource( img );
      pb.add( (float) xScale ); // xScale
      pb.add( (float) yScale ); // yScale
      pb.add( null ); // xTrans
      pb.add( null ); // yTrans
      pb.add( new InterpolationBilinear() ); // interpolation, InterpolationNearest is faster but produces worse quality
      return JAI.create( "scale", pb, SCALE_HINT );
    }
    return img;
  }

  /**
   * convert CMYK to RGB
   */
  private PlanarImage cmykToRGB( PlanarImage img )
  {
    final double[][] MATRIX_CMYK_TO_RGB = { { -0.80D, 0.00D, 0.00D, 1.0D, 0.0D }, //R
        { 0.00D, -0.80D, 0.00D, 1.0D, 0.0D }, //G
        { 0.00D, 0.00D, -0.80D, 1.0D, 0.0D } //B
    };

    ParameterBlock pb = new ParameterBlock();
    pb.addSource( img );
    pb.add( MATRIX_CMYK_TO_RGB );
    return JAI.create( "bandcombine", pb );
  }

}
