package ch.ige.imagehandler.rule.parameter;

import java.util.HashMap;

/**
 * Collection of Parameter Bindings.
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * 
 * @author Michael Locher
 * @version $Id: Environment.java,v 1.6 2009/03/19 13:26:23 locher Exp $
 */
public class Environment
{

  private HashMap<String, Parameter> bindings;

  public Environment()
  {
    this.bindings = new HashMap<String, Parameter>();
  }

  /**
   * Checks if bindings are defined in this environment.
   * 
   * @return true if no binding is defined.
   */
  public boolean isEmpty()
  {
    return this.bindings.isEmpty();
  }

  /**
   * Registers a binding in this environment.
   * 
   * @param binding the binding to register.
   * @see Parameter#getName()
   */
  public void addBinding( Parameter binding )
  {
    this.bindings.put( binding.getName(), binding );
  }

  /**
   * Gives the binding which is registered with the given name.
   * 
   * @param name
   * @return the binding
   * @see java.util.Map#get(java.lang.Object)
   */
  public Parameter lookup( String name )
  {
    return this.bindings.get( name );
  }

  @Override
  public String toString()
  {
    StringBuffer result = new StringBuffer();

    for ( Parameter parameter: this.bindings.values() )
    {
      result.append( "  " );
      result.append( parameter.toString() );
      result.append( '\n' );
    }
    return result.toString();
  }

}
