package ch.ige.ipcloadscheme.client.test;

import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import junit.framework.TestCase;

import org.apache.commons.cli.CommandLine;

import ch.ige.ipcloadscheme.client.IPCLoad;
import ch.ige.util.client.batch.BatchException;
import ch.ige.util.client.batch.BatchReport;
import ch.ige.util.client.batch.BatchReturnCode;
import ch.ige.util.client.batch.NewAbstractBatch;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.ipcloadscheme.client.IPCLoad}.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   lewis_lee
 * @version  $Id: IPCLoadTest.java,v 1.7 2008/07/02 09:26:50 kuenzli Exp $
 */
public class IPCLoadTest extends TestCase
{
  /**
   * The unit under test.
   */
  private IPCLoadForTests testIPCLoad;

  /**
   * Creates this.
   * @param name The test name
   */
  public IPCLoadTest( String name )
  {
    super( name );
  }

  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
  }

  /**
   * Tests the command line options.
   * @throws Exception if the test fails 
   */
  public void testCheckCommandLine() throws Exception
  {
    this.testIPCLoad = new IPCLoadForTests( null );
    NewAbstractBatch.start( this.testIPCLoad, new String[] {} );
    assertEquals( "Checking return code", BatchReturnCode.WRONG_CLI_OPTIONS, this.testIPCLoad.getCurrentReturnCode() );

    Map<String, Object> expectedCommandLine = new HashMap<String, Object>();
    expectedCommandLine.put( "mode", IPCLoad.ProgramMode.TRANSFER );
    expectedCommandLine.put( "date", DateConverter.stringToDate( "13.03.2008" ) );
    expectedCommandLine.put( "language", Arrays.asList( Locale.GERMAN, Locale.FRENCH, Locale.ENGLISH ) );
    expectedCommandLine.put( "save", IPCLoad.SaveMode.UPDATE );
    expectedCommandLine.put( "file", IPCLoad.FileType.ALL );

    this.testIPCLoad = new IPCLoadForTests( expectedCommandLine );
    NewAbstractBatch.start( this.testIPCLoad, new String[] { "-m", "transfer", "-d", "13.03.2008" } );
    assertEquals( "Checking return code", BatchReturnCode.OK, this.testIPCLoad.getCurrentReturnCode() );

    expectedCommandLine = new HashMap<String, Object>();
    expectedCommandLine.put( "mode", IPCLoad.ProgramMode.ALL );
    expectedCommandLine.put( "date", DateConverter.stringToDate( "15.05.2008" ) );
    expectedCommandLine.put( "language", Arrays.asList( Locale.FRENCH ) );
    expectedCommandLine.put( "save", IPCLoad.SaveMode.CREATE );
    expectedCommandLine.put( "file", IPCLoad.FileType.TEXT );

    this.testIPCLoad = new IPCLoadForTests( expectedCommandLine );
    NewAbstractBatch.start( this.testIPCLoad, new String[] { "-m", "all", "-d", "15.05.2008", "-l", "fr", "-s", "create", "-f", "text" } );
    assertEquals( "Checking return code", BatchReturnCode.OK, this.testIPCLoad.getCurrentReturnCode() );
  }

  /**
   * Used for getting access to internals.
   */
  private static class IPCLoadForTests extends IPCLoad
  {
    /**
     * Contains the expected command line options.
     */
    private Map<String, Object> expectedCommandLine;

    /**
     * Creates this.
     * @param expectedCommandLine The command line as it is expected
     */
    public IPCLoadForTests( Map<String, Object> expectedCommandLine )
    {
      super( "IPC Load test", OPTIONS, false );
      this.expectedCommandLine = expectedCommandLine;
    }

    @Override
    public BatchReport[] getBatchReports() throws BatchException
    {
      return null;
    }

    @Override
    @SuppressWarnings( "unused" )
    protected void run( CommandLine commandLine, Map<String, Object> parsedCommandLine ) throws BatchException
    {
      assertEquals( "Checking number of options", this.expectedCommandLine.size() * 2, parsedCommandLine.size() );
      for ( String optionKey: this.expectedCommandLine.keySet() )
      {
        Object expectedValue = this.expectedCommandLine.get( optionKey );
        Object currentValue = parsedCommandLine.get( optionKey );
        if ( expectedValue instanceof List<?> )
        {
          List<?> expectedValues = (List<?>) expectedValue;
          List<?> currentValues = (List<?>) currentValue;
          assertEquals( "Checking number of values", expectedValues.size(), currentValues.size() );
          for ( int i = 0; i < expectedValues.size(); i++ )
          {
            assertEquals( "Checking option " + optionKey, expectedValues.get( i ), currentValues.get( i ) );
          }
        }
        else
        {
          assertEquals( "Checking option " + optionKey, expectedValue, currentValue );
        }
      }
    }
  }
}
