package ch.ige.ipcloadscheme.server;

import java.sql.BatchUpdateException;
import java.sql.Date;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.ipcloadscheme.server.dao.IpcBildDAO;
import ch.ige.ipcloadscheme.server.dao.IpcBildKlasseDAO;
import ch.ige.ipcloadscheme.server.dao.IpcDAO;
import ch.ige.ipcloadscheme.server.dao.IpcKlasseDAO;
import ch.ige.ipcloadscheme.vo.CountVO;
import ch.ige.ipcloadscheme.vo.IpcBildKlasseVO;
import ch.ige.ipcloadscheme.vo.IpcBildVO;
import ch.ige.ipcloadscheme.vo.IpcKlasseVO;

/*** <!-- begin-xdoclet-definition --> 
 * @ejb.bean 
 *    name="IpcSchemeFacade"  
 *    description="An EJB named IpcSchemeFacade"
 *    display-name="IpcSchemeFacade"
 *    view-type="remote"
 *    jndi-name="IpcSchemeFacade"
 *    type="Stateless"
 *    transaction-type="Container"
 * @ejb.transaction type="Required"
 * <!-- end-xdoclet-definition --> 
 */
public class IpcSchemeFacadeBean implements SessionBean
{
  private static final Logger LOG = Logger.getLogger( IpcSchemeFacadeBean.class );

  private IpcKlasseDAO ipcKlasseDAO = new IpcKlasseDAO();
  private IpcBildDAO ipcBildDAO = new IpcBildDAO();
  private IpcBildKlasseDAO ipcBildKlasseDAO = new IpcBildKlasseDAO();
  private IpcDAO ipcDAO = new IpcDAO();

  private DecimalFormat dfMain = new DecimalFormat("0000");
  private SimpleDateFormat sdf = new SimpleDateFormat("yyyy.MM");


  SessionContext sessionContext;
  public void ejbCreate() throws CreateException
  {
  }

  public void ejbRemove()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }


  /**
   * *@ejb.interface-method
   */
  public CountVO insertKlassen( Map mapKlassen ) throws Exception
  {
    CountVO countVO = new CountVO();
    try
    {
      getVersionsFromBagis( mapKlassen );

      int anz = ipcBildKlasseDAO.deleteAll();
      LOG.info( "number of deleted rows from ipc_bild_klasse : " + anz );

      anz = ipcKlasseDAO.deleteAll();
      LOG.info( "number of deleted rows from ipc_klasse : " + anz );

      countVO.count = ipcKlasseDAO.executeInsert( mapKlassen );
      LOG.info( "number of tries to insert rows into ipc_klasse: " + mapKlassen.size() );
    }
    catch ( BatchUpdateException bex )
    {
      Object errorVO = findErrorVO( mapKlassen, bex );

      LOG.error( "insertKlassen() failed Message: " +  bex);
      throw new Exception("insertion failed for : \n" + errorVO.toString() + " BatchUpdateException: " + bex.getMessage());
    }
    catch ( SQLException ex )
    {
      LOG.error( "insertKlassen() failed Message: " + ex.getMessage() );
      throw new EJBException( ex );
    }
    return countVO;
  }


  /**
   * *@ejb.interface-method
   */
  public CountVO insertBildKlassen( Map mapBilder ) throws Exception
  {
    CountVO vo = new CountVO();
    try
    {
      int anz = ipcBildKlasseDAO.deleteAll();
      LOG.info( "number of deleted rows from ipc_bild_klasse : " + anz );

      vo.count = ipcBildKlasseDAO.executeInsert( mapBilder );
      LOG.info( "number of tries to insert rows into ipc_bild_klasse: " + mapBilder.size() );
    }
    catch ( BatchUpdateException bex )
    {
      Object errorVO = findErrorVO( mapBilder, bex );

      LOG.error( "insertBildKlasse() failed Message: " +  bex);
      throw new Exception("insertion failed for : \n" + errorVO.toString() + " BatchUpdateException: " + bex.getMessage() );
    }
    catch ( SQLException ex )
    {
      LOG.error( "insertBildKlasse() failed Message: " + ex.getMessage() );
      throw new EJBException( ex );
    }

    return vo;
  }


  /**
   * *@ejb.interface-method
   */
  public CountVO alterKlassen( Map map ) throws Exception
  {
    CountVO vo = new CountVO();
    getVersionsFromBagis( map );
    Collection ipcVOs =  map.values();

    try
    {
      vo.count = ipcKlasseDAO.executeUpdate( ipcVOs );
    }
    catch ( BatchUpdateException bex )
    {
      Object errorVO = findErrorVO( map, bex );

      LOG.error( "alter() failed Message: " +  bex);
      throw new Exception("update failed for : \n" + errorVO.toString() + " BatchUpdateException: " + bex.getMessage() );
    }
    catch ( SQLException ex )
    {
      LOG.error( "alter() failed Message: " + ex.getMessage() );
      throw new EJBException( ex );
    }

    return vo;
  }

  /**
   * *@ejb.interface-method
   */
  public CountVO alterBildKlassen( Map mapBilder ) throws Exception
  {
    CountVO vo = new CountVO();
    IpcBildKlasseVO ipcBildKlasseVO;
    Map mapBilderInserts = new HashMap();

    try
    {
      if (mapBilder != null )
      {
        Iterator iter = mapBilder.entrySet().iterator();
        while ( iter.hasNext() )
        {
          Map.Entry entry = ( Map.Entry ) iter.next();

          ipcBildKlasseVO = ( IpcBildKlasseVO ) entry.getValue();
          if ( ! ipcBildKlasseDAO.exists( ipcBildKlasseVO ) )
          {
            mapBilderInserts.put( ipcBildKlasseVO.getContent(), ipcBildKlasseVO );
          }
        }

        LOG.info( "number of tries to insert rows into ipc_bild_klasse: " + mapBilderInserts.size() );
        vo.count = ipcBildKlasseDAO.executeInsert( mapBilderInserts );
      }
    }
    catch ( BatchUpdateException bex )
    {
      Object errorVO = findErrorVO( mapBilderInserts, bex );

      LOG.error( "insertBildKlasse() failed Message: " +  bex);
      throw new Exception("insertion failed for : \n" + errorVO.toString() + " BatchUpdateException: " + bex.getMessage() );
    }
    catch ( SQLException ex )
    {
      LOG.error( "insertBildKlasse() failed Message: " + ex.getMessage() );
      throw new EJBException( ex );
    }

    return vo;
  }

  /**
   * *@ejb.interface-method
   */
  public void insert( IpcKlasseVO vo )
  {
    try
    {
      LOG.debug(" try to insert into ipc_klasse: " + vo.ipcKlasse);
      vo.version = getVersionFromBagis( vo.ipcKlasse );
      ipcKlasseDAO.insert( vo );
    }
    catch ( SQLException ex )
    {
      LOG.error( "insertion failed for " + vo.ipcKlasse + " Message: " + ex.getMessage() );
      throw new EJBException( ex );
    }
  }

  /**
   * *@ejb.interface-method
   */
  public void insertImage( IpcBildVO ipcBildVO )
  {
    try
    {
      if ( ipcBildDAO.exists( ipcBildVO.bildname ) )
      {
        ipcBildDAO.update( ipcBildVO );
        LOG.debug("update suceed for " + ipcBildVO.bildname);
      }
      else
      {
        ipcBildDAO.insert( ipcBildVO );
        LOG.debug("insert suceed for " + ipcBildVO.bildname);
      }
    }
    catch ( SQLException ex )
    {
      LOG.error( "image insertion or mutation failed for " + ipcBildVO.ipcKlasse + " Message: " +
                 ex.getMessage() );
      throw new EJBException( ex );
    }
  }



  /**
   * Selektiert das gueltig_von Datum ab Bagis für die in der Map enthaltenen IPC-Klassen
   * @param map Map mit IPC-Klassen
   * 
   */
  private void getVersionsFromBagis( Map map )
  {
    IpcKlasseVO ipcKlasseVO;

    Iterator iter = map.entrySet().iterator();
    while ( iter.hasNext() )
    {
      Map.Entry entry = ( Map.Entry ) iter.next();

      ipcKlasseVO = ( IpcKlasseVO ) entry.getValue();
      ipcKlasseVO.version = getVersionFromBagis( ipcKlasseVO.ipcKlasse );
    }
  }


  /**
   * Selektiert das gueltig_von Datum ab Bagis
   * @param ipcKlasse String
   * @return String Version von Bagis in der Form yyyy.MM
   * 
   */
  private String getVersionFromBagis( String ipcKlasse ) 
  {
    String ipcSymb = null;
    try
    {
      ipcSymb = getBagisFormat( ipcKlasse );
      Date gueltigVon = ipcDAO.selectGueltigVon( ipcSymb );

      return ( gueltigVon != null ? sdf.format( gueltigVon ) : null );
    }
    catch ( SQLException ex )
    {
      LOG.error( "getVersion() failed for " + ipcSymb + " Message: " + ex );
      return null;
    }
  }


  /**
   * Formatiert die ID Bsp: C07C5/093 nach C07C005093
   * @param sId IPC-Klasse
   * @return formatierte IPC-Klasse
   * 
   */
  private String getBagisFormat( String ipcKlasse )
  {
    if ( ipcKlasse.length() > 4 )
    {
      try
      {
        String group = ipcKlasse.substring( 4, ipcKlasse.indexOf( "/" ) );

        String maingroup = dfMain.format( new Double( group ) );
        String subgroup = ipcKlasse.substring( ipcKlasse.indexOf( "/" ) + 1, ipcKlasse.length() );

        StringBuffer buffer = new StringBuffer();
        buffer.append( ipcKlasse.substring( 0, 4 ) );
        buffer.append( maingroup );
        buffer.append( subgroup );

        return buffer.toString();
      }
      catch ( Exception ex )
      {
        LOG.error( ipcKlasse + " is not a valid ipc symbol Message: " + ex );
        return null;
      }
    }
    return ipcKlasse;
  }


  /**
    * Methode gibt das Object zurück, welches zu einer BatchUpdateException geführt hat.
    * @param ipcVOs Collection IpcKlasseVO's
    * @param bex BatchUpdateException
    * @return IpcKlasseVO
    * 
    */
   private Object findErrorVO( Map map, BatchUpdateException bex ) throws Exception
   {
     LOG.info( "number of rows tried to execute: " + bex.getUpdateCounts().length + " / " +  map.size() );

     Object errorVO = null;
     int[] updateCounts = bex.getUpdateCounts();
     Collection ipcVOs = map.values();

     if ( updateCounts.length < ipcVOs.size() )
     {
       Iterator it = ipcVOs.iterator();

       for ( int i = 0; i < updateCounts.length; i++ )
       {
         it.next();
       }

       errorVO = it.next();
     }

     return errorVO;
   }

}
