package ch.ige.ipcloadscheme.server.dao;

import java.sql.*;
import java.util.*;
import ch.ige.ipcloadscheme.vo.*;
import ch.ige.util.server.DBHelper;


/**
 * Data Access Object der Tabelle IPC_BILD_KLASSE (SQL-Server)
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: IpcBildKlasseDAO.java,v 1.6 2006/12/19 16:19:52 ruegsegger Exp $
 */
public class IpcBildKlasseDAO
{
  public IpcBildKlasseDAO()
  {
  }

  /**
   * Fügt einen Datensatz in die Tabelle IPC_BILD_KLASSE ein
   * @param vo IpcBildVO einzufügenden Daten
   * @throws SQLException
   */
  public int insert( IpcBildKlasseVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "INSERT INTO ipc_bild_klasse(" +
          "ipc_klasse, " +
          "bildname) " +
          "VALUES(?,?)"
          );

      ps.setString( 1, vo.ipcKlasse );
      ps.setString( 2, vo.bildname );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht Daten aus der Tabelle IPC_BILD_KLASSE
   * @param ipcKlasse String PK des zu löschenden Eintrags
   * @throws SQLException
   * @return int Anzahl gelöschte Datensätze
   */
  public int delete( String bildname ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );

      ps = con.prepareStatement( "DELETE FROM ipc_bild_klasse WHERE bildname = ?" );

      ps.setString( 1, bildname );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Löscht alle Datensätze aus der Tabelle IPC_KLASSE
   * @throws SQLException
   * @return int Anzahl der gelöschten Datensätze
   */
  public int deleteAll() throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );

      ps = con.prepareStatement( "DELETE FROM ipc_bild_klasse" );

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Insert mittels JDBC-Stapelverarbeitung
   * Alle insert-Statements zusammen werden an den SQL-Server übermittelt in einem Schritt
   * @param list Collection einzufügender IpcBildVO
   * @throws SQLException
   * @return int[] insertcount pro Anweisung
   */
  public int[] executeInsert( Map map ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    IpcBildKlasseVO ipcVO;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement(
          "INSERT INTO ipc_bild_klasse(" +
          "ipc_klasse, " +
          "bildname) " +
          "VALUES(?,?)"
          );

      Iterator iter = map.entrySet().iterator();

      while ( iter.hasNext() )
      {
        Map.Entry entry = ( Map.Entry ) iter.next();
        ipcVO = ( IpcBildKlasseVO ) entry.getValue();

        ps.setString( 1, ipcVO.ipcKlasse );
        ps.setString( 2, ipcVO.bildname );

        ps.addBatch();
      }

      return ps.executeBatch();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }


  /**
   * Überprüft ob ein Datensatz in der Tabelle IPC_BILD_KLASSE vorhanden ist
   * @param ipcKlasse String PK
   * @throws SQLException
   * @return boolean true falls Datensatz existiert
   */
  public boolean exists( IpcBildKlasseVO vo ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    ResultSet rs = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.SQLServerIPC );
      ps = con.prepareStatement( "SELECT ipc_klasse FROM ipc_bild_klasse WHERE ipc_klasse = ? AND bildname = ?" );
      ps.setString( 1, vo.ipcKlasse );
      ps.setString( 2, vo.bildname );
      rs = ps.executeQuery();
      return rs.next();
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }
}
