package ch.ige.ipcloadvalidity.client.batch;

import java.util.Date;
import java.util.Properties;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.mail.Message;
import javax.mail.Multipart;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeBodyPart;
import javax.mail.internet.MimeMessage;
import javax.mail.internet.MimeMultipart;

import org.apache.log4j.Logger;


/**
 * Klasse zum Verschicken von Emails
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: MailSender.java,v 1.8 2007/01/09 10:30:32 ruegsegger Exp $
 */
public class MailSender
{
  private static final Logger LOG = Logger.getLogger( MailSender.class );

  public MailSender()
  {
  }

  /**
   * Methode sendMail() schickt ein Email über den angegebenen SMTP-Server<br>
   * Mehrere Empfänger und CC-Empfänger können durch "," getrennt angegeben werden.
   * @param smtpServer  Mailserver
   * @param to          Empfänger
   * @param from        Absender
   * @param subject     Titel
   * @param body        Nachricht
   */
  public static void sendMail( String smtpServer, String to, String from,
                               String subject, String body ) throws Exception
  {
    sendMail( smtpServer, to, null, from, subject, body, null );
  }

  /**
   * Methode sendMail() schickt ein Email über den angegebenen SMTP-Server<br>
   * Mehrere Empfänger und CC-Empfänger können durch "," getrennt angegeben werden.
   * @param smtpServer  Mailserver (Name oder IP-Adresse)
   * @param to          Empfänger
   * @param cc          Carbon Copy (CC) Empfänger
   * @param from        Absender
   * @param subject     Titel
   * @param body        Nachricht
   */
  public static void sendMail( String smtpServer, String to, String cc,
                               String from, String subject, String body,
                               String attachment ) throws Exception
  {
    try
    {
      Properties props = System.getProperties();
      props.put( "mail.smtp.host", smtpServer );
      Session session = Session.getDefaultInstance( props, null );
      session.setDebug( false );

      // Neue Message erstellen
      Message msg = new MimeMessage( session );

      // From und To Felder setzen
      msg.setFrom( new InternetAddress( from ) );
      msg.setRecipients( Message.RecipientType.TO, InternetAddress.parse( to, false ) );
      // allfällige CC Empfänger setzen
      if ( cc != null )
      {
        msg.setRecipients( Message.RecipientType.CC, InternetAddress.parse( cc, false ) );
      }

      // Titel und Nachricht setzen
      msg.setSubject( javax.mail.internet.MimeUtility.encodeText( subject, "ISO-8859-1", null ) );
      msg.setSentDate( new Date() );

      Multipart mp = new MimeMultipart();
      MimeBodyPart textPart = new MimeBodyPart();
      textPart.setContent( body, "text/plain" );
      mp.addBodyPart( textPart );

      // Attachment anhängen
      if ( attachment != null )
      {
        MimeBodyPart attachFilePart = new MimeBodyPart();
        FileDataSource fds = new FileDataSource( attachment );
        attachFilePart.setDataHandler( new DataHandler( fds ) );
        attachFilePart.setFileName( fds.getName() );
        mp.addBodyPart( attachFilePart );
      }

      msg.setContent( mp );

      // Message senden
      Transport.send( msg );
    }
    catch ( Exception ex )
    {
//      ex.printStackTrace();
//      LOG.error(
//          "MailSender Methode sendMail(), Message konnte nicht gesendet werden. " +
//          "Fehler " + ex.getMessage() );

      String message = "MailSender Methode sendMail(), Message konnte nicht gesendet werden. Message: " + ex.getMessage() ;
      throw new Exception( message );
    }
  }
}
