package ch.ige.ipcloadvalidity.server.test;

import java.text.SimpleDateFormat;
import java.util.Date;

import org.apache.log4j.BasicConfigurator;

import junit.framework.TestCase;
import ch.ige.ipcloadvalidity.server.IpcValidityDelegate;
import ch.ige.ipcloadvalidity.server.dao.IpcDAO;
import ch.ige.ipcloadvalidity.vo.IpcVO;
import ch.ige.ipcloadvalidity.vo.TransferVO;
import ch.ige.ipcloadvalidity.vo.test.Testdata;
import ch.ige.util.server.DBHelper;

/**
 * Test des BusinessDelegate
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rüegsegger
 * @version $Id: IpcValidityDelegateTest.java,v 1.16 2007/02/08 17:23:27 ruegsegger Exp $
 */

public class IpcValidityDelegateTest extends TestCase
{
  private IpcVO ipcVO = null;
  private IpcDAO ipcDAO = new IpcDAO();
  private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");

  static 
  {
    BasicConfigurator.configure();
  }
  
  public IpcValidityDelegateTest( String s )
  {
    super( s );    
  }

  protected void setUp()
  {
    ipcVO = Testdata.createIpcVO();
  }

  /**
   * Fügt einen Datensatz in die Tabelle IPC ein
   * Falls schon vorhanden wird in statistik vermerkt
   */
  public void test00SetIpc()
  {
    try
    {
      System.out.println( "TEST SET IPC - INSERT" );

      TransferVO vo = IpcValidityDelegate.getInstance().setIpc( ipcVO, new Date() );
      assertTrue( vo.inserts == 1 );

      // schon vorhanden
      vo = IpcValidityDelegate.getInstance().setIpc( ipcVO, new Date() );
      assertTrue( vo.exists == 1 );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle IPC ein
   * Falls schon vorhanden wird in statistik vermerkt
   */
  public void test01SetIpc()
  {
    try
    {
      System.out.println( "TEST SET IPC - INSERT" );

      ipcVO.gueltigVon = sdf.parse("20091001");
      Date gueltigBis = sdf.parse( "20090930" );

       // vorangehende version nicht abgeschlossen
      TransferVO vo = IpcValidityDelegate.getInstance().setIpc( ipcVO, gueltigBis );
      assertTrue( vo.inserts == 1 );
      assertTrue( vo.terminates == 1);
      assertTrue( vo.terminatedVO != null );
      assertEquals( vo.terminatedVO.gueltigBis, gueltigBis );

      System.out.println(vo.terminatedVO.toString());
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }


  /**
   * Aktualisiert gueltigBis Datum, sofern das Datum null oder gleich dem übergebenen ist
   */
  public void test02SetIpc()
  {
    try
    {
      System.out.println( "TEST SET IPC - UPDATE VALID TO" );

      ipcVO.gueltigBis = sdf.parse("20091030");

      // GueltigBis != null
      Integer anz = IpcValidityDelegate.getInstance().setValidTo( ipcVO );
      assertTrue( anz.intValue() == 0 );


      ipcVO.gueltigBis = sdf.parse("20090930");

      // GueltigBis gleich dem übergebenen
      anz = IpcValidityDelegate.getInstance().setValidTo( ipcVO );
      assertTrue( anz.intValue() == 1 );

      ipcVO.gueltigVon = sdf.parse("20091001");
      ipcVO.gueltigBis = sdf.parse("20091030");
      anz = IpcValidityDelegate.getInstance().setValidTo( ipcVO );
      assertTrue( anz.intValue() == 1 );
    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }



  /**
   *  Löscht einen Datensatz aus der Tabelle ipc
   *  Da Delete über lokales DAO erfolgt, muss VM-Param Djava.naming.factory.initial gesetzt werden
   */
  public void test05Delete()
  {
    System.out.println( "Test DELETE " );

    try
    {
      // init test as standalone client
      DBHelper.setRunOnServer( false );     
      
      int retVal = ipcDAO.delete( ipcVO.ipcSymb, ipcVO.gueltigVon );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );

      retVal = ipcDAO.delete( ipcVO.ipcSymb, sdf.parse("20091001") );
      assertEquals( "Anzahl gelöschte Datensätze ", 1, retVal );

    }
    catch ( Exception e )
    {
      e.printStackTrace();
      fail( e.toString() );
    }
  }
}
