package ch.ige.ipcloadvalidity.vo;

import java.io.Serializable;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * Value Object für die Tabelle IPC
 * <p>
 * Copyright (c) 2005, Institut für Geistiges Eigentum
 * @author A. Rüegsegger
 * @version $Id: IpcVO.java,v 1.11 2007/01/09 10:30:32 ruegsegger Exp $
 */
public class IpcVO implements Serializable
{
  public static final String CLASSIFICATION   = "K";        // for classification purpose
  public static final String INDEXING         = "I";        // for indexing purpose
  public static final String DOUBLE           = "D";        // for both classification and indexing purpose

  public static final String CL_PREFORM       = "P";        // prior to IPC Reform
  public static final String CL_COMMON        = "C";        // valid for Advanced and Core level
  public static final String CL_ADVANCED      = "A";        // valid only for Advanced level
  public static final String CL_OBSOLETE      = "O";        // valid only for Core level
  public static final String CL_SUBCLASS      = "S";        // valid only for Subclasses


  public String   ipcSymb       = null;
  public String   zusatz        = null;
  public Date     gueltigVon    = null;
  public Date     gueltigBis    = null;
  public String   classLevel    = null;
  public String   corePred      = null;
  public int      version       = 0;

  private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");

  /**
   * Gibt den Inhalt des ValueObject IpcVO zurück
   * @return Inhalt von IpcVO
   */
  public String toString()
  {
    SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
    StringBuffer  buf = new StringBuffer();

    buf.append( "  ipcSymb          = <" + this.ipcSymb + ">\n" );
    buf.append( "  zusatz           = <" + this.zusatz + ">\n" );
    buf.append( "  gueltigVon       = <" + ( gueltigVon == null ? null : sdf.format( gueltigVon ) ) + ">\n" );
    buf.append( "  gueltigBis       = <" + ( gueltigBis == null ? null : sdf.format( gueltigBis ) ) + ">\n" );
    buf.append( "  classLevel       = <" + this.classLevel + ">\n" );
    buf.append( "  corePred         = <" + this.corePred + ">\n" );
    buf.append( "  version          = <" + this.version + ">\n" );

    return buf.toString();
  }

  public String getLogInformation( )
  {
    StringBuffer sb = new StringBuffer();
    sb.append( ipcSymb );

    try
    {
      sb.append( "\t" );
      sb.append( sdf.format( gueltigVon ) );
      if ( gueltigBis != null )
      {
        sb.append( "\t" );
        sb.append( sdf.format( gueltigBis ) );
      }
    }
    catch ( Exception ex )
    {
      System.out.println(ex);
    }

    return sb.toString();
  }

}
