package ch.ige.jipps;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.jipps.vo.DruckPeriVO;

/**
 * The main interface to JIPPS.
 *
 * <p>Copyright (c) 2002-2004, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author   Priska, Steiner, Johannes Lang, Marc Bouquet, Elmar Isler, Kurt Jegge
 * @version  $Id: JIPPSFacadeBean.java,v 1.30 2007/12/20 08:26:25 lewis_lee Exp $
 * @ejb.bean name="JIPPSFacade" description="JIPPS Facade" display-name="JIPPSFacade" view-type="remote"
 *           type="Stateless" transaction-type="Container"
 *           remote-business-interface="ch.ige.jipps.JIPPSFacadeServices"
 * @ejb.ejb-ref
 *           ejb-name="JIPPSInfo"
 *           view-type="local"
 * @ejb.ejb-ref
 *           ejb-name="JMSPublisher"
 *           view-type="local"
 * @ejb.transaction type="Required"
 * @ejb.ejb-ref view-type= "local" type="Session" ejb-name="PrintJobStore" ref-name="ejb/PrintJobStoreLocal"
 * @ejb.ejb-ref view-type= "local" type="Session" ejb-name="JIPPSProperties" ref-name="ejb/JIPPSPropertiesLocal"
 *                       
 * @ejb.resource-ref
 *   res-ref-name="mail/JavaMailSession"
 *   res-type="javax.mail.Session"
 *   res-auth="Container"
 *   
 * @jboss.resource-ref
 *   res-ref-name="mail/JavaMailSession"
 *   jndi-name="java:/mail/IGESession"
 */
public class JIPPSFacadeBean extends JIPPSFacadeServicesImpl implements SessionBean
{
  /**
   * The log
   */
  private static final Logger LOG = Logger.getLogger( JIPPSFacadeBean.class );

  /**
   * @ejb.interface-method
   * @ejb.transaction type="RequiresNew"
   * @see ch.ige.jipps.JIPPSFacadeServices#updateVerarbStat(ch.ige.jipps.vo.DruckPeriVO)
   */
  public void updateVerarbStat( DruckPeriVO periVO )
  {
    super.updateVerarbStat( periVO );
  }

  /**
   * @ejb.interface-method
   * @ejb.transaction type="Supports"
   * @see ch.ige.jipps.JIPPSFacadeServicesImpl#printAMVOne(boolean)
   */
  public int printAMVOne( boolean hasCover )
  {
    return super.printAMVOne( hasCover );
  }

  /**
   * @ejb.interface-method
   * @ejb.transaction type="Supports"
   * @see ch.ige.jipps.JIPPSFacadeServicesImpl#printAMVOne(boolean, int, boolean)
   */
  public int printAMVOne( boolean hasCover, int amvLaufId, boolean sendMail )
  {
    return super.printAMVOne( hasCover, amvLaufId, sendMail );
  }

  /**
   * @ejb.interface-method
   * @ejb.transaction type="Supports"
   * @see ch.ige.jipps.JIPPSFacadeServicesImpl#finishAMVOne(boolean, int, boolean)
   */
  public void finishAMVOne( boolean hasCover, int amvLaufId, boolean sendMail )
  {
    super.finishAMVOne( hasCover, amvLaufId, sendMail );
  }

  /**
   * @ejb.interface-method
   * @ejb.transaction type="Supports"
   * @see ch.ige.jipps.JIPPSFacadeServicesImpl#printAMVChanges(boolean)
   */
  public int printAMVChanges( boolean hasCover )
  {
    return super.printAMVChanges( hasCover );
  }

  private SessionContext sessionContext;

  public void ejbCreate()
  {
    try
    {
      PrintJobStoreFactory.getEJBPrintJobStore();
    }
    catch ( NamingException e )
    {
      throw new RuntimeException( "Can't initialize the PrintJobStoreFactory", e );
    }
    catch ( CreateException e )
    {
      throw new RuntimeException( "Can't initialize the PrintJobStoreFactory", e );
    }

    try
    {
      JIPPSPropertiesFactory.getEJBJippsProperties();
    }
    catch ( NamingException e )
    {
      throw new RuntimeException( "Can't initialize the JIPPSPropertiesFactory", e );
    }
    catch ( CreateException e )
    {
      throw new RuntimeException( "Can't initialize the JIPPSPropertiesFactory", e );
    }

    ServicesFactory.getStandaloneServices();
  }

  public void ejbRemove()
  {
  }

  public void ejbActivate()
  {
  }

  public void ejbPassivate()
  {
  }

  public void setSessionContext( SessionContext sessionContext )
  {
    this.sessionContext = sessionContext;
  }
}
