package ch.ige.jipps;

import java.sql.SQLException;

import javax.mail.Session;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;

import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.dao.DruckerDAO;
import ch.ige.jipps.dao.PropertyJippsDAO;
import ch.ige.jipps.vo.DruckerVO;

/**
 * Implements {@link ch.ige.jipps.JIPPSPropertiesServices}.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: JIPPSPropertiesServicesImpl.java,v 1.9 2009/01/12 16:11:33 lewis_lee Exp $
 */
public class JIPPSPropertiesServicesImpl implements JIPPSPropertiesServices
{

  private static final Logger LOG = Logger.getLogger( JIPPSPropertiesBean.class );

  private DruckerDAO druckerDAO = null;

  /**
   * Used to read JIPPS properties
   */
  private PropertyJippsDAO propertyJippsDAO = null;

  private DruckerVO druckerVO = null;

  /**
   * Gets the PropertyJippsDAO - use only this for accessing it!
   * @return The PropertyJippsDAO
   */
  private synchronized PropertyJippsDAO getPropertyJippsDAO()
  {
    if ( this.propertyJippsDAO == null )
    {
      this.propertyJippsDAO = new PropertyJippsDAO();
    }
    return this.propertyJippsDAO;
  }

  // @see ch.ige.jipps.JIPPSPropertiesServices#getDrucker(java.lang.String)
  public int getDrucker( String propertyKey ) throws Exception
  {
    try
    {
      // TODO Use a constant for the properties key
      return new Integer( getPropertyJippsDAO().select( propertyKey + PropertyConstants.PRINTER ) ).intValue();
    }
    catch ( Exception ex )
    {
      throw new Exception( "Drucker kann nicht geladen werden", ex );
    }
  }

  /**
   * Holt ein Drucker mit aus der Tabelle Drucker
   * @param druckerId Drucker-Nr.
   * @return druckerVO Value-Object eines Druckers
   */
  private DruckerVO getDruckerById( int druckerId )
  {
    if ( druckerDAO == null )
    {
      druckerDAO = new DruckerDAO();
    }

    try
    {
      druckerVO = druckerDAO.select( druckerId );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
    }
    return druckerVO;
  }

  /**
   * Holt den Mailempfänger aus Property_Jipps
   * @param propertyKey String Property-Key für den Receiver
   * @return Empängeradresse
   * @throws Exception
   */
  private String getReceiver( String propertyKey ) throws Exception
  {
    try
    {
      return getPropertyJippsDAO().select( propertyKey + PropertyConstants.RECEIVER );
    }
    catch ( Exception ex )
    {
      throw new Exception( "Mail Receiver kann nicht geladen werden", ex );
    }
  }

  /**
   * Holt den zusätzlichen Mailempfänger aus Property_Jipps
   * @param propertyKey String Property-Key für den ReceiverCC
   * @return zusätzliche Empängeradresse
   * @throws Exception
   */
  private String getReceiverCC( String propertyKey ) throws Exception
  {
    try
    {
      return getPropertyJippsDAO().select( propertyKey + PropertyConstants.RECEIVER_CC );
    }
    catch ( Exception ex )
    {
      throw new Exception( "Mail ReceiverCC kann nicht geladen werden", ex );
    }
  }

  // @see ch.ige.jipps.JIPPSPropertiesServices#getDirectoryPath(java.lang.String)
  public String getDirectoryPath( String propertyKey ) throws Exception
  {
    try
    {
      return getPropertyJippsDAO().select( propertyKey + PropertyConstants.PATH );
    }
    catch ( Exception ex )
    {
      throw new Exception( "Directory-Path kann nicht geladen werden", ex );
    }
  }

  // @see ch.ige.jipps.JIPPSPropertiesServices#doMail(java.lang.String, java.lang.String, java.lang.String)
  public void doMail( String betreffText, String meldung, String propertyKey )
  {
    doMail( 0, betreffText, meldung, "", propertyKey, 0, 0, 0, "" );
  }

  // @see ch.ige.jipps.JIPPSPropertiesServices#doMail(int, java.lang.String, java.lang.String, java.lang.String,
  // java.lang.String, int, int, int, java.lang.String)
  public void doMail( int batchId, String betreffText, String meldung, String errorMsg, String propertyKey, int anzahl, int fehler, int drucker, String path )
  {
    try
    {
      String adressTo = getReceiver( propertyKey );
      String adressCC = getReceiverCC( propertyKey );
      String adressFrom = "JIPPSMAILER";

      StringBuffer bufBetreff = new StringBuffer();
      //TODO AMV-Erinnerung 2 kann hier bei Gelegenheit mal entfernt werden, da nicht mehr verwendet.
      //TODO Zudem sollte bei AMV-Erinnerung 1 auf ch.ige.jipps.AMVControl.FORMTYPE_AMV_TEXT referenziert werden.
      if ( betreffText.equalsIgnoreCase( AmvControl.FORMTYPE_AMV_TEXT ) )
      {
        bufBetreff.append( betreffText );
        bufBetreff.append( " JIPPS " );
        if ( fehler > 0 )
        {
          bufBetreff.append( "nok" );
        }
        else
        {
          bufBetreff.append( "ok" );
        }
      }
      else
      {
        bufBetreff.append( "Die Aufbereitung der " );
        bufBetreff.append( betreffText );
        bufBetreff.append( " ist beendet." );
      }

      StringBuffer bufMsg = new StringBuffer();
      if ( meldung == null )
      {
        bufMsg.append( "Statusbericht der Aufbereitung:\nBatch-Nummer:\t\t\t\t\t\t" );
        bufMsg.append( batchId );
        bufMsg.append( "\nDrucker:\t\t\t\t\t\t\t" );
        bufMsg.append( getDruckerById( drucker ).druckerName );
        bufMsg.append( "\nAblage-Verzeichnis:\t\t\t\t\t" );
        bufMsg.append( path );
        bufMsg.append( "\nAnzahl erstellte " );
        bufMsg.append( betreffText );
        bufMsg.append( ":\t" );
        bufMsg.append( anzahl ); // Note: One publication can create up to 7 PDF files!
      }
      else
      {
        bufMsg.append( meldung );
      }

      // Allgemeine Syteminformationen
      bufMsg.append( "\n\n" );
      bufMsg.append( "CVS-Tag:\t\t\t\t\t" );
      bufMsg.append( BuildInfo.CVS_TAG );
      bufMsg.append( "\nBuild-Id:\t\t\t\t\t" );
      bufMsg.append( BuildInfo.BUILD_ID );
      bufMsg.append( "\n\n" );

      if ( fehler > 0 )
      {
        bufMsg.append( "Bei der Aufbereitung ist/sind " );
        bufMsg.append( fehler );
        bufMsg.append( " Fehler aufgetreten.\n" );
        bufMsg.append( errorMsg );
      }

      sendMail( adressTo, adressCC, adressFrom, bufBetreff, bufMsg, new Integer( batchId ) );
      LOG.info( "Statusbericht für Batch-Nummer: " + batchId + " wurde per Mail an: " + adressTo + " gesendet." );
    }
    catch ( Exception ex )
    {
      LOG.warn( "SendMail konnte nicht erzeugt werden", ex );
    }
  }

  // @see ch.ige.jipps.JIPPSPropertiesServices#doMailNoData(java.lang.String, java.lang.String, java.lang.String,
  // java.lang.String, int)
  public void doMailNoData( String betreffText, String meldung, String errorMsg, String propertyKey, int fehler )
  {
    try
    {
      String adressTo = getReceiver( propertyKey );
      String adressCC = getReceiverCC( propertyKey );
      String adressFrom = "JIPPSMAILER";

      StringBuffer bufBetreff = new StringBuffer();
      //TODO AMV-Erinnerung 2 kann hier bei Gelegenheit mal entfernt werden, da nicht mehr verwendet.
      //TODO Zudem sollte bei AMV-Erinnerung 1 auf ch.ige.jipps.AMVControl.FORMTYPE_AMV_TEXT referenziert werden.
      if ( betreffText.equalsIgnoreCase( AmvControl.FORMTYPE_AMV_TEXT ) )
      {
        bufBetreff.append( betreffText );
        bufBetreff.append( " JIPPS " );
        if ( fehler > 0 )
        {
          bufBetreff.append( "nok" );
        }
        else
        {
          bufBetreff.append( "ok" );
        }
      }
      else
      {
        bufBetreff.append( "Die Aufbereitung der " );
        bufBetreff.append( betreffText );
        bufBetreff.append( " ist beendet." );
      }

      StringBuffer bufMsg = new StringBuffer();
      if ( meldung == null )
      {
        bufMsg.append( "Statusbericht der Aufbereitung:\nIn der Datenbank sind keine Daten zur Verarbeitung vorhanden, aus diesem Grund konnten keine " );
        bufMsg.append( betreffText );
        bufMsg.append( " erstellt werden !\n\n" );
      }
      else
      {
        bufMsg.append( meldung );
      }

      // Allgemeine Syteminformationen
      bufMsg.append( "\n\n" );
      bufMsg.append( "CVS-Tag:\t\t\t\t\t" );
      bufMsg.append( BuildInfo.CVS_TAG );
      bufMsg.append( "\nBuild-Id:\t\t\t\t\t" );
      bufMsg.append( BuildInfo.BUILD_ID );
      bufMsg.append( "\n\n" );

      if ( fehler > 0 )
      {
        bufMsg.append( "Bei der Aufbereitung ist/sind " );
        bufMsg.append( fehler );
        bufMsg.append( " Fehler aufgetreten.\n" );
        bufMsg.append( errorMsg );
      }

      sendMail( adressTo, adressCC, adressFrom, bufBetreff, bufMsg, null );
      LOG.info( "Statusbericht (keine Daten vorhanden) wurde per Mail an: " + adressTo + " gesendet." );
    }
    catch ( Exception ex )
    {
      LOG.warn( "SendMail konnte nicht erzeugt werden", ex );
    }
  }

  /**
   * Sends the mail to the recipients - or if the System Property {@link PropertyConstants#BATCH_MAIL_RESULT} is set, the
   * mail message is written into the log.
   * @param addressTo The recipient
   * @param addressCc The addresses that receive a copy
   * @param addressFrom The sender
   * @param subject The mail subject
   * @param message The mail message (body)
   * @param batchId This is the ID of the batch run, which may be <code>null</code>; it's only used for writing the
   * mail to the log anyway
   * @throws Exception if the mail can't be sent
   */
  private void sendMail( String addressTo, String addressCc, String addressFrom, StringBuffer subject, StringBuffer message, Integer batchId ) throws Exception
  {
    String batchMailResultFile = System.getProperty( PropertyConstants.BATCH_MAIL_RESULT );
    if ( StringUtils.isEmpty( batchMailResultFile ) )
    {
      new SendMail( getMailSession() ).sendMail( addressTo, addressCc, addressFrom, subject.toString(), message.toString() );
    }
    else
    {
      writeMailToLog( message, batchId );
    }
  }

  private Session getMailSession() throws NamingException, ClassCastException
  {
    Context ctx = new InitialContext();
    return (javax.mail.Session) ctx.lookup( "java:comp/env/mail/JavaMailSession" );
  }

  /**
   * Writes the mail message to the log, for testing.
   * @param message The mail message to write
   * @param batchId The batch ID of the batch run
   */
  private void writeMailToLog( StringBuffer message, Integer batchId )
  {
    String batchIdString = "";
    if ( batchId != null )
    {
      batchIdString = "(ID " + batchId.toString() + ")";
    }
    LOG.info( "Instead of sending a mail, this batch run " + batchIdString + " writes it into the log, for testing:\n" + message.toString() );
  }

  // @see ch.ige.jipps.JIPPSPropertiesServices#getProperty(java.lang.String)
  public String getProperty( String key )
  {
    try
    {
      return getPropertyJippsDAO().select( key );
    }
    catch ( SQLException ex )
    {
      LOG.error( ex.getMessage(), ex );
      return null;
    }
  }
}
