package ch.ige.jipps;

import java.io.IOException;
import java.sql.Blob;
import java.sql.SQLException;
import java.util.List;
import java.util.Map;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.ipobjects.exceptions.ImageDiscardedException;
import ch.ige.ipobjects.patent.SearchReportImage;
import ch.ige.ipobjects.patent.dao.PatDocSchemaDAO;
import ch.ige.ipobjects.patent.dao.SearchReportDAO;
import ch.ige.ipobjects.patent.vo.PatentDocumentSchemaVO;
import ch.ige.ipobjects.patent.vo.SearchReportVO;

/**
 * Creates a patent search report as a PDF.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: PatentSearchReportControl.java,v 1.9 2009/03/23 09:51:14 locher Exp $
 */
public class PatentSearchReportControl extends PatentSpecificationControl
{
  // @see ch.ige.jipps.PatentSpecificationControl#getReport(java.lang.Integer)
  @Override
  protected AbstractVO getReport( Integer reportId ) throws SQLException
  {
    if ( reportId == null )
    {
      throw new IllegalArgumentException( "Search report ID must not be null" );
    }
    SearchReportVO searchReport = DAOPool.get( SearchReportDAO.class ).select( reportId );
    if ( searchReport == null )
    {
      throw new IllegalArgumentException( "This search report ID " + reportId + " does not exist, no database entry could be found" );
    }
    return searchReport;
  }

  // @see ch.ige.jipps.PatentSpecificationControl#getPatdocSchemaId(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer getPatdocSchemaId( AbstractVO vo )
  {
    SearchReportVO searchReport = (SearchReportVO) vo;
    try
    {
      PatDocSchemaDAO patDocSchemaDAO = DAOPool.get( PatDocSchemaDAO.class );
      Map<Integer, PatentDocumentSchemaVO> schemaMapping = patDocSchemaDAO.selectMappingForSearchreportschemas();
      PatentDocumentSchemaVO patentDocumentSchema = schemaMapping.get( searchReport.getPatdocSchemaMapping_id() );
      if ( patentDocumentSchema == null )
      {
        throw new IllegalArgumentException( "There is no consistent mapping (patdocschema and searchreportschema) for PatDocSchemaMapping-ID "
            + searchReport.getPatdocSchemaMapping_id() );
      }
      return patentDocumentSchema.getPatdocschema_id();
    }
    catch ( SQLException e )
    {
      throw new RuntimeException( "Can't load mapping for search report " + searchReport.getIdentifier() );
    }
  }

  // @see ch.ige.jipps.PatentSpecificationControl#getXmlDocument(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected String getXmlDocument( AbstractVO vo )
  {
    return ( (SearchReportVO) vo ).getXml();
  }

  // @see ch.ige.jipps.PatentSpecificationControl#saveImagesToDisk(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void saveImagesToDisk( AbstractVO vo ) throws IOException, SQLException
  {
    List<? extends SearchReportImage> images = ( (SearchReportVO) vo ).getImages();
    for ( SearchReportImage searchReportImage: images )
    {
      try
      {
        Blob data = searchReportImage.getImage();
        saveImageToDisk( searchReportImage.getFilename(), data );
      }
      catch ( ImageDiscardedException e )
      {
        e.throwAsSystemException();
      }
    }
  }
}
