package ch.ige.jipps.constants;

/**
 * Contains constants for letters.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author Johannes Lang
 * @author Eric Lewis
 * @version  $Id: LetterTypeConstants.java,v 1.6 2009/01/12 09:34:27 lewis_lee Exp $
 */
public class LetterTypeConstants
{
  /**
   * Must be used in a static way.
   */
  private LetterTypeConstants()
  {
    // Do not instantiate this! 
  }

  /**
   * The letter for a patent's application confirmation.
   */
  public static final Integer APPLICATION_CONFIRMATION_105_INTEGER = 105;

  /**
   * The letter for a patent's application confirmation.
   */
  public static final String APPLICATION_CONFIRMATION_105 = APPLICATION_CONFIRMATION_105_INTEGER.toString();

  /**
   * The letter for a patent's application confirmation.
   */
  public static final Integer PUBLICATION_CONFIRMATION_416_INTEGER = 416;

  /**
   * The letter for a publication confirmation.
   */
  public static final String PUBLICATION_CONFIRMATION_416 = PUBLICATION_CONFIRMATION_416_INTEGER.toString();

  /**
   * The letter for a publication confirmation.
   */
  public static final Integer PUBLICATION_CONFIRMATION_417_INTEGER = 417;

  /**
   * The letter for a publication confirmation.
   */
  public static final String PUBLICATION_CONFIRMATION_417 = PUBLICATION_CONFIRMATION_417_INTEGER.toString();

  /**
   * The first letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final Integer APPLICATION_CONFIRMATION_422_INTEGER = 422;

  /**
   * The first letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final String APPLICATION_CONFIRMATION_422 = APPLICATION_CONFIRMATION_422_INTEGER.toString();

  /**
   * The second letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final Integer APPLICATION_CONFIRMATION_462_INTEGER = 462;

  /**
   * The second letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final String APPLICATION_CONFIRMATION_462 = APPLICATION_CONFIRMATION_462_INTEGER.toString();

  /**
   * The third letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final Integer APPLICATION_CONFIRMATION_485_INTEGER = 485;

  /**
   * The third letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final String APPLICATION_CONFIRMATION_485 = APPLICATION_CONFIRMATION_485_INTEGER.toString();

  /**
   * The fourth letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final Integer APPLICATION_CONFIRMATION_441_INTEGER = 441;

  /**
   * The fourth letter type for an application confirmation (Hinterlegungsbescheinigung).
   */
  public static final String APPLICATION_CONFIRMATION_441 = APPLICATION_CONFIRMATION_441_INTEGER.toString();

  /**
   * Shows whether a letter type is an application confirmation for trademarks.
   * In German it's called a <code>Hinterlegungsbescheinigung</code> (or <code>HLB</code>).
   * @param letterType The letter type to check, which is trimmed
   * @return <code>true</code> if the letter type is an application confirmation, otherwise <code>false</code>
   */
  public static boolean isTrademarkApplicationConfirmation( String letterType )
  {
    String trimmed = letterType.trim();
    return trimmed.equals( APPLICATION_CONFIRMATION_422 ) || trimmed.equals( APPLICATION_CONFIRMATION_462 ) || trimmed.equals( APPLICATION_CONFIRMATION_485 )
        || trimmed.equals( APPLICATION_CONFIRMATION_441 );
  }

  /**
   * Shows whether a letter type is a publication confirmation for trademarks.
   * In German it's called a <code>Publikationsbestätigung</code>.
   * @param letterType The letter type to check
   * @return <code>true</code> if it's a publication confirmation, otherwise <code>false</code>
   */
  public static boolean isTrademarkPublicationConfirmation( int letterType )
  {
    return letterType == PUBLICATION_CONFIRMATION_416_INTEGER.intValue() || letterType == PUBLICATION_CONFIRMATION_417_INTEGER.intValue();
  }
}
