package ch.ige.jipps.dao;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;

/**
 * Returns the lower limit of invoice IDs (<code>Bfaktur ID</code> in German).
 * 
 * <p>Copyright (c) 2001, Swiss Federal Institute of Intellectual Property</p>
 * 
 * @author Johannes Lang
 * @version $Id: BfaktIdLimiter.java,v 1.1 2007/04/27 16:24:55 lewis_lee Exp $
 */
public class BfaktIdLimiter extends AbstractBagisDAO
{
  /**
   * The only instance
   */
  private static BfaktIdLimiter me = null;

  /**
   * The default lower limit for invoice IDs
   */
  private static final int INVOICE_ID_DEFAULT_LOWER_LIMIT = 5000000;

  /**
   * The ID for getting the lower limit
   */
  private static final Integer XX_SYSPARAMS_ID = 90;

  /**
   * Selects the lower limit
   */
  private static final SQLStatement<String> SELECT = SQL.select( "SELECT value FROM xx_sysparams WHERE id = ?" );

  /**
   * The lower limit to use
   */
  private int lowerLimit = 0;

  /**
   * Shows whether this has been initialized
   */
  private boolean isInitialized = false;

  /**
   * This must not be used (Singleton).
   *
   */
  private BfaktIdLimiter()
  {
    // Use getInstance()
  }

  /**
   * Gets the only instance of this.
   * @return The only instance of this
   */
  public static synchronized BfaktIdLimiter getInstance()
  {
    if ( me == null )
    {
      me = new BfaktIdLimiter();
    }
    return me;
  }

  /**
   * Gets the lower limit for invoice IDs, defaulting to a value if
   * it can't be read from the database.
   * This is used to limit the number of invoices when selecting:
   * Old invoices are not selected.
   * @return The lower limit for seleting invoices
   */
  public int getBfaktIdLowerLimit()
  {
    if ( this.isInitialized )
    {
      return this.lowerLimit;
    }

    try
    {
      String value = selectString( getQueryRunner(), SELECT, XX_SYSPARAMS_ID );
      if ( value != null )
      {
        this.lowerLimit = Integer.parseInt( value );
        this.isInitialized = true;
        return this.lowerLimit;
      }
      else
      {
        throw new Exception();
      }
    }
    catch ( Exception ex )
    {
      return INVOICE_ID_DEFAULT_LOWER_LIMIT;
    }
  }
}
