package ch.ige.jipps.dao;

import java.sql.SQLException;

import ch.ige.dbutils.dao.AbstractBagisDAO;
import ch.ige.dbutils.sql.SQL;
import ch.ige.dbutils.sql.SQLStatement;
import ch.ige.jipps.vo.FaktparVO;

/**
 * Kapselung der DB-Zugriffe auf die Tabelle FAKTPAR
 * <p>
 * Copyright (c) 2002-2006, Eidgenössisches Institut für Geistiges Eigentum
 * @author    Marc Bouquet
 * @version   $Id: FaktparDAO.java,v 1.18 2007/05/02 13:32:15 locher Exp $
 */
public class FaktparDAO extends AbstractBagisDAO
{
  /**
   * The type of value object to return
   */
  private static final Class<FaktparVO> VO_TYPE = FaktparVO.class;

  /**
   * Selects the parameters
   */
  private static final SQLStatement<FaktparVO> SELECT = SQL
      .select( "SELECT adr_nr, TRIM(druckperi) AS druckperi, TRIM(listenart) AS listenart, fach_nr, TRIM(sort_cd) AS sort_cd "
          + "FROM faktpar WHERE adr_nr = ? " );

  /**
   * Selects the printing parameters for an address's annual invoice.
   * @param addressNumber The address to look for
   * @return The printing parameters for the address invoice
   * @throws SQLException if the query fails
   */
  public FaktparVO select( int addressNumber ) throws SQLException
  {
    return selectVO( VO_TYPE, SELECT, new Integer( addressNumber ) );
  }
}
