package ch.ige.jipps.dao.test;

import java.util.ArrayList;
import java.util.List;

import ch.ige.dbutils.dao.test.BaseDAOTestTemplate;
import ch.ige.jipps.dao.BfakTxtDAO;
import ch.ige.jipps.vo.BfakTxtVO;

/**
 * Tests {@link ch.ige.jipps.dao.BfakTxtDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: BfakTxtDAOTest.java,v 1.1 2007/04/20 13:35:39 lewis_lee Exp $
 */
public class BfakTxtDAOTest extends BaseDAOTestTemplate<BfakTxtDAO, BfakTxtVO>
{
  /**
   * The text numbers to load
   */
  private static final int[] TEXT_NUMBERS = { 12957, 12961, 13600 };

  /**
   * The texts corresponding to {@link #TEXT_NUMBERS}
   */
  private static final String[] TEXTS = { "up in Smoke ((Logo)) (Hinterlegungsdatum: 27.10.1995)", "33: Wein.\nInt. Kl.: 33.",
      "Sony Overseas SA\nWagistrasse 6\n8952 Schlieren" };

  /**
   * The only constructor
   * @param name The test's name
   */
  public BfakTxtDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getDAOClass()
  @Override
  protected Class<BfakTxtDAO> getDAOClass()
  {
    return BfakTxtDAO.class;
  }

  /**
   * Tests {@link ch.ige.jipps.dao.BfakTxtDAO#select(java.util.List)}.
   * @throws Exception if the test fails
   */
  public void testSelect() throws Exception
  {
    try
    {
      getDAO().select( null );
      fail( "Null text numbers must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    List<BfakTxtVO> texts = getDAO().select( new ArrayList<Integer>() );
    assertTrue( "Checking for empty text numbers", texts.isEmpty() );

    List<Integer> searchTextNumbers = createSearchCriteria( new int[] { 999999999 } );
    texts = getDAO().select( searchTextNumbers );
    assertTrue( "Checking for unknown text numbers", texts.isEmpty() );

    searchTextNumbers = createSearchCriteria( TEXT_NUMBERS );
    texts = getDAO().select( searchTextNumbers );
    assertEquals( "Checking for known text numbers", TEXT_NUMBERS.length, texts.size() );
    for ( int i = 0; i < texts.size(); i++ )
    {
      BfakTxtVO text = texts.get( i );
      assertEquals( "Checking for text number", TEXT_NUMBERS[i], text.textNr );
      assertEquals( "Checking text", TEXTS[i], text.text );
    }
  }

  /**
   * Gets the search criteria for the text search.
   * @param textNumbers The text numbers to convert into search criteria
   * @return The search criteria
   */
  private List<Integer> createSearchCriteria( int[] textNumbers )
  {
    List<Integer> criteria = new ArrayList<Integer>( textNumbers.length );
    for ( int textNumber: textNumbers )
    {
      criteria.add( new Integer( textNumber ) );
    }
    return criteria;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#getSerializableVO()
  @Override
  protected BfakTxtVO getSerializableVO()
  {
    BfakTxtVO text = new BfakTxtVO();
    text.textNr = 99;
    text.text = "This is a test";
    return text;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( BfakTxtVO originalText, BfakTxtVO serializedText )
  {
    return originalText.textNr == serializedText.textNr && originalText.text.equals( serializedText.text );
  }
}
