package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FAILURE_CODE;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FAILURE_FOLDER_DOWNLOAD;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FAILURE_FOLDER_UPLOAD;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.SHARE_DESTINATION;

import java.io.File;
import java.io.IOException;
import java.util.List;

import javax.servlet.ServletContext;

import org.apache.commons.io.FileUtils;
import org.apache.commons.io.FilenameUtils;

import ch.ige.ftp.IGEFTPClientAdapter;
import ch.ige.ftp.jakarta.IGEFTPClientAdapterJakarta;
import ch.ige.jipps.ftp.web.util.FTPAccessConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

public abstract class AbstractService implements Service
{


  public static final String TEMPORARY_FOLDER_MARKUP = "ftpservlet";

  protected ServletContext context;

  public AbstractService( ServletContext context )
  {
    this.context = context;
  }

  protected String storeDataOnShareForUpload( String clientfolderPath, String ma, FTPAccessConfigVO ftpParameters, boolean failure )
  {
    return this.storeDataOnShare( clientfolderPath, FTPServletConfiguration.mapTmpFolderPathToRelShareServerPath( clientfolderPath, ma ), ftpParameters,
        failure, FAILURE_FOLDER_UPLOAD );

  }

  protected String storeDataOnShareForDownload( String clientfolderPath, String ma, FTPAccessConfigVO ftpParameters, boolean failure )
  {
    return this.storeDataOnShare( clientfolderPath, FTPServletConfiguration.mapTmpFolderPathToRelShareServerPath( clientfolderPath, ma ), ftpParameters,
        failure, FAILURE_FOLDER_DOWNLOAD );

  }

  private String storeDataOnShare( String clientFolderPath, String relServerFolderPath, FTPAccessConfigVO ftpParameters, boolean failure, String failureFolder )
  {
    if ( ftpParameters == null )
    {
      String msg = "FTP Paramters for destination [" + SHARE_DESTINATION + "] do not exist: Cannot execute ftp request";
      LOG.error( msg );
      return FAILURE_CODE;
    }
    LOG.debug( "Storing data on share from " + clientFolderPath );
    // validate serverFolder as partial path
    StringBuilder serverUrl = new StringBuilder().append( this.context.getAttribute( FTPServletConfiguration.CONTEXT_SHARE_DATA_URL ) );
    serverUrl.append( "\\" );
    if ( failure )
    {
      serverUrl.append( failureFolder );
      serverUrl.append( "\\" );
    }
    serverUrl.append( FilenameUtils.separatorsToWindows( relServerFolderPath ) );

    IGEFTPClientAdapter client = new IGEFTPClientAdapterJakarta( ftpParameters.getFTPHost(), ftpParameters.getFTPUser(), ftpParameters.getFTPPasswort(),
        clientFolderPath, ftpParameters );
    List<String> uploadedFilesPaths = null;
    try
    {
      client.login();
      client.setRelativeServerPath( ftpParameters.getFTPServerPath(), false );
      if ( failure )
      {
        relServerFolderPath = FilenameUtils.concat( failureFolder, relServerFolderPath );
      }
      // ok with path
      client.setRelativeServerPath( FilenameUtils.concat(client.getRelativeServerPath() , relServerFolderPath ), true );
      uploadedFilesPaths = client.uploadData( null, true, true );
      client.logout();
      return serverUrl.toString();
    }
    catch ( Exception e )
    {
      LOG.error( "Error in ftp upload on share", e );
    }
    return "Fehler beim Kopieren der Daten auf " + serverUrl; // kein Fehler Code sodass Client die Daten als gesendet meldet

  }
  
  protected void deleteTmpDirectory(File folder) throws IOException
  {
    String config1 = FTPServletConfiguration.mapRequestFolderPathToTmpDirDownload( "test", "test" );
    config1 = config1.substring( 0, config1.indexOf( "test" ) );
    String config2 = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( "test", "test" );
    config2 = config2.substring( 0, config2.indexOf( "test" ) );

    if ( config1 == null || ! config1.contains( TEMPORARY_FOLDER_MARKUP ) || config2 == null || ! config2.contains( TEMPORARY_FOLDER_MARKUP ))
    {
      LOG.error( "Folder not deleted because configuration not ok " + folder  );
    }
    else if (folder != null && folder.exists() && (folder.getAbsolutePath().contains( config1 ) || folder.getAbsolutePath().contains( config2 )))
    {
      LOG.info( "Deleting the temporary folder " + folder );
      FileUtils.deleteDirectory( folder );
    }
    else
    {
      LOG.error( "Folder not deleted " + folder  );
    }
  }

}
