package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.DATASOURCE_FOLDER_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.EMPLOYEE_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FOLDER_MARK_FOR_SESSION;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.PATENT_NUMMER_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.PUBL_DATE_REQUEST_PARAMETER;

import java.io.File;
import java.rmi.RemoteException;
import java.sql.Date;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.ejb.CreateException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.patent.vo.PatentDocumentEventKeyVO;
import ch.ige.ipobjects.time.BAGISTime;
import ch.ige.jipps.JIPPSFacade;
import ch.ige.jipps.JIPPSFacadeHome;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.patdocutil.receipts.FTPUploadReceipt;
import ch.ige.patdocutil.receipts.PatentDocumentVOWithPDF;
import ch.ige.util.enums.IPType;

/**
 * Extract the PatentDocuments from BAGIS and create the PDF.
 * <p>
 * Required request parameters:
 * <ol>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#PUBL_DATE_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#EMPLOYEE_REQUEST_PARAMETER}</li>
 * </ol>
 * <p>
 * Optional request paramters:
 * <ol>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#PATENT_NUMMER_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#DATASOURCE_FOLDER_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#BATCH_REQUEST_PARAMETER}</li>
 * </ol> 
 * @author fabrizioli
 *
 */
public class ExtractPatentDocumentsService extends AbstractService
{

  private JIPPSFacade jippsFacade;

  public ExtractPatentDocumentsService( ServletContext context )
  {
    super( context );
  }

  public String execute( HttpServletRequest request, ResultBean bean )
  {
    try
    {
      String publDate = FTPServletConfiguration.extractRequestParameter( request, PUBL_DATE_REQUEST_PARAMETER, true );
      String patentNumber = FTPServletConfiguration.extractRequestParameter( request, PATENT_NUMMER_REQUEST_PARAMETER, false );
      String employee = FTPServletConfiguration.extractRequestParameter( request, EMPLOYEE_REQUEST_PARAMETER, true );
      String folderPath = FTPServletConfiguration.extractRequestParameter( request, DATASOURCE_FOLDER_REQUEST_PARAMETER, false );
      String batch = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.BATCH_REQUEST_PARAMETER, false );
      String tmpDirPath = FTPServletConfiguration.mapRequestFolderPathToTmpDirUpload( folderPath, employee );
      String biblio = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.BIBLIOGRAPHIC_DATA_REQUEST_PARAMETER, false );

      HttpSession session = request.getSession( true );
      String sessionMarker = session.getId();
      session.setAttribute( FOLDER_MARK_FOR_SESSION, session.getId() );

      boolean isBatchRequest = ( batch != null && batch.length() > 0 ) ? true : false;
      boolean isBiblioRequest = ( biblio != null && biblio.length() > 0 ) ? true : false;
      // as the session from the user from MA is not known in batch mode, we delete the complete folder in order to clean up
      if ( isBatchRequest )
      {
        LOG.info( "Starting execution in Batch mode" );
        File mafolder = new File( tmpDirPath );
        if ( mafolder.exists() )
        {
          LOG.info( "Folder for writing patent documents is deleted in batch mode before starting " + tmpDirPath );
          deleteTmpDirectory( mafolder );
        }
      }
      else
      {
        LOG.info( "Starting execution in NON-Batch mode" );
      }
      if ( sessionMarker == null )
      {
        LOG.error( "No session entroy for marking folder for session" );
      }
      else
      {
        tmpDirPath += File.separator + sessionMarker;
        LOG.info( "Folder for writing patent documents " + tmpDirPath );
      }
      if ( tmpDirPath == null || tmpDirPath.length() == 0 )
      {
        throw new FTPServletRuntimeException( "tmpDir null canno initialize folder " + folderPath );
      }
      File tmpDir = new File( tmpDirPath );
      if ( ( !isBatchRequest || isBiblioRequest ) && tmpDir.exists() )
      {
        LOG.warn( "The temporary directory already exists, we need to delete it before starting: " + tmpDirPath );
        deleteTmpDirectory( tmpDir );
      }
      LOG.debug( "TempDir " + tmpDir.getAbsolutePath() + " was created " + Boolean.valueOf( tmpDir.mkdirs() ).toString() );
      if ( !tmpDir.exists() )
      {
        throw new FTPServletRuntimeException( FTPServletConfiguration.generateErrorMessage( null,
            "Keine PDFs Generiert: Verzeichnis für Ablage konnte nicht generiert werden " + tmpDirPath + " in Service " + this.getClass().getSimpleName() ) );
      }
      if ( isBiblioRequest )
      {
        LOG.info( "Starting execution for the generation of the bibliographic data" );
      }
      bean.addReceipts( this.getPublishedST36DocumentWithPDFs( BAGISTime.convertISO8601ToDate( publDate ), patentNumber != null ? Integer
          .valueOf( patentNumber ) : -1, employee, tmpDirPath, isBiblioRequest ) );

      LOG.debug( "added receipt to bean " + bean );

      return "extractResult";
    }
    catch ( Exception e )
    {
      throw new FTPServletRuntimeException( FTPServletConfiguration.generateErrorMessage( e, "Keine PDFs Generiert: " + e.getMessage()
          + " in calling the service " + this.getClass().getSimpleName() ), e );
    }

  }

  private List<FTPUploadReceipt> getPublishedST36DocumentWithPDFs( Date publicationDate, Integer patentNumber, String employee, String folderPath,
      boolean biblio ) throws CreateException, RemoteException, NamingException, ServletException, StorageException, ParseException
  {
    List<PatentDocumentEventKeyVO> allPatdocIds = getJippsFacade().getAllPatdocIds( publicationDate, patentNumber );

    int limit = 15;
    List<PatentDocumentEventKeyVO>[] patdocIdViews = splitPatDocIdsIntoViews( allPatdocIds, limit );
    List<FTPUploadReceipt> receipts = new ArrayList<FTPUploadReceipt>( allPatdocIds.size() );

    for ( List<PatentDocumentEventKeyVO> patdocIdPackage: patdocIdViews )
    {
      List<PatentDocumentVOWithPDF> results = null;
      if ( biblio )
      {
        results = getJippsFacade().getBiblioDataWithPDF( createPrintJobs( patdocIdPackage, employee ), true );
      }
      else
      {
        results = getJippsFacade().getPatentDocumentsWithPDF( createPrintJobs( patdocIdPackage, employee ), true );
      }

      // folderPath may be null, a default path is chosen
      storeData( results, folderPath, biblio );
      Iterator<PatentDocumentVOWithPDF> resultIter = results.iterator();
      Iterator<PatentDocumentEventKeyVO> eventIter = patdocIdPackage.iterator();
      while ( resultIter.hasNext() && eventIter.hasNext() )
      {
        PatentDocumentVOWithPDF result = resultIter.next();
        PatentDocumentEventKeyVO patdoIdInfo = eventIter.next();
        receipts.add( FTPUploadReceipt.fromVo( IPKeyVO.create( patdoIdInfo.getIPKey() ), result ) );
      }
    }
    LOG.debug( "receipts generated " + receipts.size() );
    return receipts;

  }

  private List<PatentDocumentEventKeyVO>[] splitPatDocIdsIntoViews( List<PatentDocumentEventKeyVO> patdocIds, int limit )
  {
    int size = patdocIds.size();
    int noPackages = (int) Math.ceil( size / ( limit * 1.0 ) );
    List<PatentDocumentEventKeyVO>[] packages = new List[noPackages];
    int cursor = 0;
    for ( int i = 0; i < noPackages; i++ )
    {
      int packageSize = ( size - cursor ) >= limit ? limit : ( size - cursor );
      packages[i] = patdocIds.subList( cursor, cursor + packageSize );
      LOG.debug( "Generated " + packageSize + " pdf documents starting from " + ( cursor + 1 ) );
      cursor += packageSize;
    }
    return packages;
  }

  private JIPPSFacade getJippsFacade() throws NamingException, RemoteException, CreateException
  {
    if ( jippsFacade == null )
    {
      Context context = new InitialContext();
      JIPPSFacadeHome jippsFacadeHome = (JIPPSFacadeHome) context.lookup( JIPPSFacadeHome.JNDI_NAME );
      jippsFacade = jippsFacadeHome.create();
    }
    return jippsFacade;
  }

  private Map<PatentDocumentEventKeyVO, PrintJobVO> createPrintJobs( List<PatentDocumentEventKeyVO> patdocIds, String employeeUserName )
      throws RemoteException, NamingException, CreateException
  {
    Map<PatentDocumentEventKeyVO, PrintJobVO> result = new HashMap<PatentDocumentEventKeyVO, PrintJobVO>( patdocIds.size() );
    int maNr = getJippsFacade().getMaNr( employeeUserName );
    for ( PatentDocumentEventKeyVO entry: patdocIds )
    {
      result.put( entry, createPrintJobForFTPServlet( entry.getPatentDocuemntId(), maNr ) );
    }
    return result;
  }

  /**
  * Create the print job for FTPServlet
  * @param patdocId the id of the patent document
  * @param maNo the id of the employee
  * @return the print job for the ftp servlet
  */
  private PrintJobVO createPrintJobForFTPServlet( int patdocId, int maNo )
  {

    PrintJobVO printJob = new PrintJobVO();
    printJob.formTyp = PrintJobVO.PATENT_SPECIFICATION;
    printJob.schutitTyp = IPType.PATENT_CH.toDB();
    printJob.reportId = patdocId;
    printJob.maNr = maNo;
    printJob.senderApp = "FTPServlet";
    printJob.senderUser = "FTPServlet";
    printJob.sprache = PrintJobVO.USE_IP_LANGUAGE;
    printJob.druckerId = 4;
    return printJob;
  }

  private void storeData( List<PatentDocumentVOWithPDF> receipt, String folder, boolean biblio ) throws ServletException, StorageException
  {
    if ( folder == null || folder.length() == 0 )
    {
      LOG.debug( "[TRACE] Folder was not set, getting default folder" );
      folder = FTPServletConfiguration.getDefaultFolderForLocalStorage();
    }

    Iterator<PatentDocumentVOWithPDF> patdocVOIter = receipt.iterator();
    while ( patdocVOIter.hasNext() )
    {
      PatentDocumentVOWithPDF patdocWithPdf = patdocVOIter.next();

      try
      {
        File xmlFolder = new File( FTPServletConfiguration.getXMLFolderPath( patdocWithPdf.getVo(), folder, patdocWithPdf.getLanguage(), biblio ) );
        File pdfFolder = new File( FTPServletConfiguration.getPDFFolderPath( patdocWithPdf.getVo(), folder, patdocWithPdf.getLanguage(), biblio ) );
        LOG.debug( "Storing data for " + patdocWithPdf.getRegisterNumber() + " " + patdocWithPdf.getVo().getCompareTime() );
        patdocWithPdf.save( xmlFolder, pdfFolder );

        LOG.info( "Stored data in " + xmlFolder + " and " + pdfFolder );
      }

      catch ( Exception e )
      {
        String msg = " Failed to store data for " + patdocWithPdf.getIdentifier() + " of patent " + patdocWithPdf.getRegisterNumber() + e.getMessage();
        LOG.error( msg, e );
        throw new StorageException( msg, e );
      }
    }

  }

}
