package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.DESTINATION_REQUEST_PARAMETER;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.FAILURE_CODE;
import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;

import ch.ige.ftp.IGEFTPClientAdapter;
import ch.ige.ftp.jakarta.IGEFTPClientAdapterJakarta;
import ch.ige.jipps.ftp.web.util.FTPAccessConfigVO;
import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

/**
 * List the data from a folder on the FTP server.
 * <p>
 * Required request parameters:
 * <ol>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#SERVERPPATH_REQUEST_PARAMETER}</li>
 * <li>{@link ch.ige.jipps.ftp.web.util.FTPServletConfiguration#DESTINATION_REQUEST_PARAMETER}</li>
 * </ol> 
 * @author fabrizioli
 *
 */
public class ListService implements Service
{

  private ServletContext context;

  public ListService( ServletContext context ) throws SQLException
  {
    this.context = context;
  }

  public String execute( HttpServletRequest request, ResultBean bean ) throws FTPServiceException
  {
    String destination = null;
    String serverFolderPath = null;
    try
    {
      List<String> pathList = null;
      destination = FTPServletConfiguration.extractRequestParameter( request, DESTINATION_REQUEST_PARAMETER, true );
      serverFolderPath = FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.SERVERPPATH_REQUEST_PARAMETER, true );

      FTPAccessConfigVO ftpParameters = ( (Map<String, FTPAccessConfigVO>) this.context.getAttribute( FTPServletConfiguration.CONTEXT_FTPLOGIN_PARAMTERER_MAP ) )
          .get( destination );

      pathList = this.list( serverFolderPath, ftpParameters );
      if ( pathList == null || pathList.isEmpty() )
      {
        String errorMsg = FAILURE_CODE + " Keine Daten von " + destination + " Server gelesen";
        bean.addErrorMessage( errorMsg );
      }
      else
      {
        bean.setUploadedFilesPaths( pathList );
      }

    }
    catch ( Exception e )
    {
      String msg = "Error in listing files from " + destination;
      LOG.error( msg );
      throw new FTPServiceException( msg, e );
    }
    return "listResult";
  }

  private List<String> list( String serverFolderPath, FTPAccessConfigVO ftpParameters )
  {

    if ( ftpParameters == null )
    {
      throw new IllegalArgumentException( "Cannot connect: no paramteres" );
    }
    List<String> fileList = new ArrayList<String>( 64 );

    IGEFTPClientAdapter client = new IGEFTPClientAdapterJakarta( ftpParameters.getFTPHost(), ftpParameters.getFTPUser(), ftpParameters.getFTPPasswort(), null,
        ftpParameters );
    try
    {
      client.login();
      client.setRelativeServerPath( ftpParameters.getFTPServerPath(), false );
      client.setRelativeServerPath( serverFolderPath, false );
      fileList = client.listData( null );
      client.logout();
    }
    catch ( Exception e )
    {
      String msg = FAILURE_CODE + " Fehler bei List Operation" + e.getMessage();
      LOG.error( msg, e );
      fileList = client.getLastRequestInfo();
      if ( fileList == null )
      {
        fileList = new ArrayList<String>( 1 );
      }
      fileList.add( msg );
    }
    return fileList;
  }

}
