package ch.ige.jipps.ftp.web;

import static ch.ige.jipps.ftp.web.util.FTPServletConfiguration.LOG;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import ch.ige.jipps.ftp.web.util.FTPServletConfiguration;

public class RequestProcessor
{

  private Map<String, Service> services;

  public RequestProcessor( ServletContext context ) throws Exception
  {
    this.services = new HashMap<String, Service>( 8 );
    this.services.put( FTPServletConfiguration.SERVICE_EXTRACT, new ExtractPatentDocumentsService( context ) );
    this.services.put( FTPServletConfiguration.SERVICE_UPLOAD, new UploadService( context ) );
    this.services.put( FTPServletConfiguration.SERVICE_DOWNLOAD, new DownloadService( context ) );
    this.services.put( FTPServletConfiguration.SERVICE_LIST, new ListService( context ) );
  }

  public String execute( HttpServletRequest request, HttpServletResponse response ) throws IOException, FTPServiceException
  {

    // create an new bean for eachrequest. Since we introduced  session handling on the client, we now could evaluate the results on the server, but all data are required for mail anyhow

    ResultBean bean = new ResultBean();
    request.setAttribute( FTPServletConfiguration.SESSION_PARAMETER_RESULT_BEAN, bean );

    if ( !request.getParameterNames().hasMoreElements() )
    {
      LOG.info( "no request Par, we go to previous page " );
      return getPreviousPage( request );
    }
    Service service = this.getService( FTPServletConfiguration.extractRequestParameter( request, FTPServletConfiguration.SERVICE_REQUEST_PARAMETER, false ) );

    String nextPage = null;
    try
    {
      nextPage = service.execute( request, bean );
    }
    catch ( FTPServiceException e )
    {
      bean.addErrorMessage( e.getMessage() );
      nextPage = "error.jsp";
    }
    if ( nextPage == null )
    {
      LOG.info( "no next page evaluated, we go to previous page " );
      nextPage = getPreviousPage( request );
    }
    LOG.info( "page evaluated: " + nextPage );
    if ( !nextPage.startsWith( "error" ) )
    {
      request.getSession().setAttribute( "previousPage", nextPage );
    }
    return nextPage;
  }

  private String getPreviousPage( HttpServletRequest request )
  {
    HttpSession session = request.getSession( false );
    String previousPage = FTPServletConfiguration.DEFAULT_PREVIOUS_PAGE;
    if ( session != null )
    {
      String sessionPreviousPage = (String) session.getAttribute( "previousPage" );
      if ( sessionPreviousPage != null )
        previousPage = sessionPreviousPage;
    }
    return previousPage;
  }

  private Service getService( String serviceType )
  {
    Service service = services.get( serviceType );
    if ( service == null )
    {
      String msg = "The Service " + serviceType + " ist not initialized ";
      LOG.error( msg );
      throw new IllegalArgumentException( msg );
    }
    LOG.debug( "Found the service " + serviceType + " as " + service.getClass().getName() );
    return service;
  }

}
