package ch.ige.jipps.ftp.web.util;

import org.apache.commons.configuration.Configuration;

import ch.ige.dbutils.vo.VO;
import ch.ige.ftp.FTPServerSystem;
import ch.ige.ftp.fileselector.util.FileType;
import ch.ige.ftp.jakarta.DataTransferConnectionMode;
import ch.ige.ftp.jakarta.FTPClientConfigurationJakarta;

/**
 * VO with the configuration of the FTP Servlet
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   fabrizioli
 * @version  $Id: FTPAccessConfigVO.java,v 1.4 2009/02/19 10:18:19 fabrizioli Exp $
 */
public class FTPAccessConfigVO extends FTPClientConfigurationJakarta implements VO
{

  private String ftp_host;

  private String ftp_user;

  private String ftp_pass;

  private String ftp_path;

  private String ftp_url;

  public FTPAccessConfigVO( String ftp_host, String ftp_user, String ftp_pass, String ftp_path, String ftp_url, FTPServerSystem serverSystem,
      int readTimeoutSec, int conntectionTimeoutSec, boolean useListFiles, FileType file_type_upload, FileType file_type_download )
  {
    super( serverSystem, null, null, conntectionTimeoutSec, readTimeoutSec, useListFiles, file_type_download, file_type_upload, DataTransferConnectionMode
        .getDefault() );
    this.ftp_host = ftp_host;
    this.ftp_user = ftp_user;
    this.ftp_pass = ftp_pass;
    this.ftp_path = ftp_path;
    this.ftp_url = ftp_url;
  }

  public static FTPAccessConfigVO fromConfiguration( Configuration table )
  {
    Configuration tmp = table;
    return new FTPAccessConfigVO( table.getString( "ftp_host" ), table.getString( "ftp_user" ), table.getString( "ftp_pass" ), table.getString( "ftp_path" ),
        table.getString( "ftp_url" ), FTPServerSystem.parse( table.getString( "server_system" ) ), table.getInt( "ftp_connect_timeout" ), table
            .getInt( "ftp_rw_timeout" ), table.getBoolean( "ftp_use_listfiles" ), FileType.parse( table.getString( "file_type_download" ) ), FileType
            .parse( table.getString( "file_type_upload" ) ) );
  }

  /**
   * Gets the host of the intranet FTP Server, where the data sent shall be stored.
   * @return the host of the intranet FTP Server
   */
  public String getFTPHost()
  {
    return ftp_host;
  }

  /**
   * Gets the user for the intranet FTP Server.
   * @return the user for the intranet FTP Server
   */
  public String getFTPUser()
  {
    return ftp_user;
  }

  /**
   * Gets the password for the intranet FTP Server.
   * @return the password for the intranet FTP Server
   */
  public String getFTPPasswort()
  {
    return ftp_pass;
  }

  /**
   * Gets the path on the FTP Server for storing the data sent.
   * @return the path on the FTP Server 
   */
  public String getFTPServerPath()
  {
    return ftp_path;
  }

  /**
   * Gets the intranet_ftp_url.
   * @return the intranet_ftp_url
   */
  public String getFTPUrl()
  {
    return ftp_url;
  }

}
