package ch.ige.jipps.header.test;

import java.util.ArrayList;
import java.util.List;

import org.custommonkey.xmlunit.SimpleXpathEngine;
import org.custommonkey.xmlunit.XMLUnit;
import org.w3c.dom.Document;

import ch.ige.jipps.JIPPSFacadeServicesImpl;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.xml.test.BaseDaoXmlTestTemplate;
import ch.ige.util.constants.IpRegisterStateConstants;
import ch.ige.util.date.DateConverter;
import ch.ige.util.enums.BAGISLanguageDomain;
import ch.ige.util.enums.IPType;
import ch.ige.util.enums.report.ReportType;

/**
 * Checks a special case for the print header IP number.
 * <p>
 * Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum
 * </p>
 * @author Eric Lewis
 * @version $Id: SpecialPrintHeaderTest.java,v 1.4 2007/12/17 17:11:18 lewis_lee Exp $
 */
public class SpecialPrintHeaderTest extends BaseDaoXmlTestTemplate
{
  /**
   * This patent has a normal IP (registration) number
   */
  private static final int PATENT_WITH_IP_NUMBER = 1200084524;

  /**
   * The formatted IP number for {@link #PATENT_WITH_IP_NUMBER}
   */
  private static final String FORMATTED_REGISTRATION_NUMBER = "689680";

  /**
   * This patent has no IP number
   */
  private static final int PATENT_WITHOUT_IP_NUMBER = 1200583557;

  /**
   * The formatted application number for {@link #PATENT_WITHOUT_IP_NUMBER}
   */
  private static final String FORMATTED_APPLICATION_NUMBER = "00001/00";

  /**
   * The unit under test
   */
  private JIPPSFacadeServicesImpl jippsFacade;

  /**
   * The default constructor
   * @param name The test name
   */
  public SpecialPrintHeaderTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();
    this.jippsFacade = new JIPPSFacadeServicesImpl();
  }

  /**
   * Tests the number in the print header. If there's no IP number (registration number), then the appliation number is
   * taken.
   * @throws Exception if the test fails
   */
  public void testNumberInPrintHeaderForPatents() throws Exception
  {
    String xPath = "/auszug/header/eintrNr";
    SimpleXpathEngine xpathEngine = new SimpleXpathEngine();

    String[] xmlDocuments = this.jippsFacade.getXml( createPrintJobs( PATENT_WITH_IP_NUMBER, IpRegisterStateConstants.REGISTRATION_STATE ) );
    Document document = XMLUnit.buildTestDocument( xmlDocuments[0] );

    assertXpathExists( xPath, document );
    String xpathValue = xpathEngine.evaluate( xPath, document );
    assertEquals( "Checking with registration number", FORMATTED_REGISTRATION_NUMBER, xpathValue );

    xmlDocuments = this.jippsFacade.getXml( createPrintJobs( PATENT_WITHOUT_IP_NUMBER, IpRegisterStateConstants.APPLICATION_STATE ) );
    document = XMLUnit.buildTestDocument( xmlDocuments[0] );

    assertXpathExists( xPath, document );
    xpathValue = xpathEngine.evaluate( xPath, document );
    assertEquals( "Checking with application number", FORMATTED_APPLICATION_NUMBER, xpathValue );
  }

  /**
   * Creates the print jobs to create the XML document.
   * @param ipKey The IP's unique key
   * @param registerState The IP's register state
   * @return The print jobs to execute
   * @throws Exception if the date can't be parsed
   */
  private List<PrintJobVO> createPrintJobs( int ipKey, int registerState ) throws Exception
  {
    List<PrintJobVO> printJobs = new ArrayList<PrintJobVO>();

    PrintJobVO printJob = new PrintJobVO();
    printJob.formTyp = PrintJobVO.AUSZUG;
    printJob.schutitTyp = IPType.PATENT_CH.toDB();
    printJob.printIgekopf = false;
    printJob.reportId = ipKey;
    printJob.maNr = 3;
    printJob.regauszTyp = ReportType.SEARCH_REPORT_AFTER_APPLICATION.toDB();
    printJob.sprache = BAGISLanguageDomain.DE.toDB();
    printJob.standDatum = DateConverter.stringToDate( "01.01.2007" );
    printJob.registerKz = registerState;

    printJobs.add( printJob );

    return printJobs;
  }
}
