package ch.ige.jipps.history.description;

import java.util.Locale;
import java.util.Map;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.event.Event;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.util.constants.EventProcessConstants;
import ch.ige.util.enums.report.ReportType;

/**
 * Configures how the history part of the report is built for a PCT.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Esther Bandschapp
 * @version $Id: PatentPCTHistoryDescription.java,v 1.7 2007/12/19 08:56:42 lewis_lee Exp $
 */
public class PatentPCTHistoryDescription extends PatentHistoryDescription
{
  /**
   * Creates this with all required data.
   * @param ip The PCT patent
   * @param currentState The history is built for this date
   * @param language The language of the report
   * @param includeWaitListEvents If this is <code>true</code>, events that are on the wait list are also shown
   * @param startFromApplication If this is <code>true</code>, events starting from the application are shown,
   * otherwise from the registration
   * @param allowedReportGroups The report groups, a <code>Map</code> where the keys show whether an event must be
   * included on a report. The values can be ignored.
   * @param reportType The report type
   */
  public PatentPCTHistoryDescription( GenericIP ip, PointInTime currentState, Locale language, boolean includeWaitListEvents, boolean startFromApplication,
      Map<Integer, String> allowedReportGroups, ReportType reportType )
  {
    super( ip, currentState, language, includeWaitListEvents, startFromApplication, allowedReportGroups, reportType );
  }

  // See interface
  @Override
  public boolean includeInHistory( Event event )
  {
    if ( EventProcessConstants.APPLICATION_INTEGER.equals( event.getErfass_proc() ) )
    {
      return false;
    }
    else
    {
      return super.includeInHistory( event );
    }
  }
}
