package ch.ige.jipps.history.detail.design;

import java.sql.Date;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.design.model.DesignPrioritiesHistory.DesignPrioritiesItem;
import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.BasePrioritiesDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.format.PriorityFormatter;

/**
 * Creates the detail lines for the design's priorities.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DesignPrioritiesDetail.java,v 1.3 2007/12/19 08:57:08 lewis_lee Exp $
 */
public class DesignPrioritiesDetail extends BasePrioritiesDetail<DesignPrioritiesItem>
{
  /**
   * Creates this with all the data.
   * @param priorities All the design priorities
   * @param ipType The IP type
   * @param ip The IP
   */
  public DesignPrioritiesDetail( TimePeriodCollection<DesignPrioritiesItem> priorities, IPType ipType, GenericIP ip )
  {
    super( priorities, ipType, ip );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<DesignPrioritiesItem, Detail> createBuilder()
  {
    return new DesignPrioritiesDetailBuilder();
  }

  // See superclass
  @Override
  public boolean shouldGroupEndDates()
  {
    return true;
  }

  /**
   * The class that actually formats the priorities.
   */
  public class DesignPrioritiesDetailBuilder extends BasePrioritiesDetailBuilder
  {
    /**
     * Creates the builder.
     */
    public DesignPrioritiesDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<DesignPrioritiesItem> detailData )
    {
      if ( !detailData.isEmpty() )
      {
        DesignPrioritiesItem prioritiesItem = detailData.get( 0 );
        return buildDetails( detailData, prioritiesItem.getPriorities(), null );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }

  // See superclass
  @Override
  protected String formatPriority( String countryCode, String countryName, Date priorityDate, String priorityNumber, String priorityRemark )
  {
    return PriorityFormatter.formatDesignPriority( countryCode, priorityNumber, priorityDate );
  }
}
