package ch.ige.jipps.history.detail.patent;

import ch.ige.ipobjects.Types;
import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.common.vo.PrioritaetVO;
import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.BasePrioritiesDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.format.PriorityFormatter;

/**
 * Creates the detail lines for the patent's priorities.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: PatentPrioritiesDetail.java,v 1.4 2007/12/19 08:57:21 lewis_lee Exp $
 */
public class PatentPrioritiesDetail extends BasePrioritiesDetail<PrioritaetVO>
{
  /**
   * The constructor
   * @param priorities The IP's priorities
   * @param ipType The IP's type
   * @param ip The generic IP
   */
  public PatentPrioritiesDetail( TimePeriodCollection<PrioritaetVO> priorities, IPType ipType, GenericIP ip )
  {
    super( priorities, ipType, ip );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<PrioritaetVO, Detail> createBuilder()
  {
    return new PrioritiesDetailBuilder();
  }

  /**
   * The class that actually formats the priorities.
   */
  public class PrioritiesDetailBuilder extends BasePrioritiesDetailBuilder
  {
    /**
     * Creates the builder.
     */
    public PrioritiesDetailBuilder()
    {
      // Nothing to do
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<PrioritaetVO> detailData )
    {
      if ( !detailData.isEmpty() )
      {
        return buildDetails( detailData, detailData, null );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }

  // See superclass
  @Override
  protected String formatPriority( String countryCode, String countryName, java.sql.Date priorityDate, String priorityNumber, String priorityRemark )
  {
    if ( Types.SPC.equals( this.ipType ) )
    {
      return PriorityFormatter.formatSpcRegistration( priorityRemark, priorityNumber, priorityDate );
    }
    else
    {
      return PriorityFormatter.formatPatentPriority( countryCode, priorityNumber, priorityDate );
    }
  }
}
