package ch.ige.jipps.history.detail.trademark;

import java.util.ArrayList;

import ch.ige.ipobjects.common.history.HistoryItem;
import ch.ige.ipobjects.common.history.ValueHistory;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.report.history.impl.AbstractHistoryItemDetailBuilder;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.constants.ReportTextConstants;

/**
 * Creates the detail lines for a trademark's color claim.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: ColorClaimDetail.java,v 1.7 2007/12/19 08:57:21 lewis_lee Exp $
 */
public class ColorClaimDetail extends BaseHistoryItemDetail<HistoryItem<String>>
{
  /**
   * Creates this with its detail data.
   * @param colorClaims The color claims to format
   */
  public ColorClaimDetail( ValueHistory<HistoryItem<String>> colorClaims )
  {
    super( colorClaims, null, null );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<HistoryItem<String>, Detail> createBuilder()
  {
    return new ColorClaimDetailBuilder();
  }

  /**
   * The class that actually formats the color claim.
   */
  public class ColorClaimDetailBuilder extends AbstractHistoryItemDetailBuilder<HistoryItem<String>, Detail>
  {
    /**
     * Creates the builder.
     */
    public ColorClaimDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<HistoryItem<String>> detailData )
    {
      HistoryItem<String> colorClaim = detailData.get( 0 );
      if ( colorClaim.getValue() != null )
      {
        Detail detail = new Detail( colorClaim, ColorClaimDetail.this );
        detail.reportTextNumber = ReportTextConstants.TRADEMARK_COLOR_CLAIM_INTEGER;
        detail.textLines = new ArrayList<String>();
        detail.textLines.add( colorClaim.getValue() );
        return createListSingleDataItemSingleRepresentation( colorClaim, detail );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }
}
