package ch.ige.jipps.history.detail.trademark;

import java.sql.Date;

import ch.ige.ipobjects.common.GenericIP;
import ch.ige.ipobjects.model.IPType;
import ch.ige.ipobjects.report.history.HistoryItemDetailBuilder;
import ch.ige.ipobjects.report.history.HistoryItemDetailList;
import ch.ige.ipobjects.report.history.HistoryItemDetailList.EmptyHistoryItemDetailList;
import ch.ige.ipobjects.time.TimePeriodCollection;
import ch.ige.ipobjects.trademark.model.TrademarkPrioritiesHistory.TrademarkPrioritiesItem;
import ch.ige.jipps.history.BasePrioritiesDetail;
import ch.ige.jipps.history.detail.Detail;
import ch.ige.util.format.PriorityFormatter;

/**
 * Creates the detail lines for the trademark's priorities.
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: TrademarkPrioritiesDetail.java,v 1.4 2007/12/19 08:57:21 lewis_lee Exp $
 */
public class TrademarkPrioritiesDetail extends BasePrioritiesDetail<TrademarkPrioritiesItem>
{
  /**
   * Creates this with all the data.
   * @param priorities The trademark priorities
   * @param ipType The IP type
   * @param ip The IP
   */
  public TrademarkPrioritiesDetail( TimePeriodCollection<TrademarkPrioritiesItem> priorities, IPType ipType, GenericIP ip )
  {
    super( priorities, ipType, ip );
  }

  // See abstract method
  @Override
  protected HistoryItemDetailBuilder<TrademarkPrioritiesItem, Detail> createBuilder()
  {
    return new TrademarkPrioritiesDetailBuilder();
  }

  /**
   * The class that actually formats the priorities.
   */
  public class TrademarkPrioritiesDetailBuilder extends BasePrioritiesDetailBuilder
  {
    /**
     * Creates the builder.
     */
    public TrademarkPrioritiesDetailBuilder()
    {
      super();
    }

    // @see
    // ch.ige.ipobjects.report.history.HistoryItemDetailBuilder#buildDetails(ch.ige.ipobjects.time.TimePeriodCollection)
    public HistoryItemDetailList<Detail> buildDetails( TimePeriodCollection<TrademarkPrioritiesItem> detailData )
    {
      if ( !detailData.isEmpty() )
      {
        TrademarkPrioritiesItem prioritiesItem = detailData.get( 0 );
        return buildDetails( detailData, prioritiesItem.getPriorities(), prioritiesItem.getRemark() );
      }
      else
      {
        return EmptyHistoryItemDetailList.getInstance();
      }
    }
  }

  // See superclass
  @Override
  protected String formatPriority( String countryCode, String countryName, Date priorityDate, String priorityNumber, String priorityRemark )
  {
    /** TODO Check if the priority number is never required */
    return PriorityFormatter.formatTrademarkPriority( countryCode, countryName, null, priorityDate );
  }
}
