package ch.ige.jipps.imageconv.test;

import java.awt.Image;
import java.io.File;

import javax.swing.ImageIcon;

import junit.framework.TestCase;

import org.apache.commons.lang.StringUtils;

import ch.ige.dbutils.dao.AbstractDAO;
import ch.ige.dbutils.dao.DAO;
import ch.ige.jipps.constants.PropertyConstants;
import ch.ige.jipps.imageconv.ImageMagickConverter;
import ch.ige.util.client.ClientLogInitalizer;

/**
 * Tests {@link ch.ige.jipps.imageconv.ImageMagickConverter}. Note that the path to ImageMagick must be specified by
 * setting the System Property {@link ch.ige.jipps.constants.PropertyConstants#IMAGEMAGICK_PATH}. This is normally done
 * manually (or in a launch configuration) and prevents that this is run on servers that don't have ImageMagick
 * installed.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: ImageMagickConverterTest.java,v 1.3 2007/12/19 08:58:00 lewis_lee Exp $
 */
public class ImageMagickConverterTest extends TestCase
{
  static
  {
    ClientLogInitalizer.initializeLoggingForTests( true, true );
    AbstractDAO.setRunningInJ2EEServer( false );
    System.setProperty( DAO.STANDALONE_CONFIG_PROPERTY_NAME, "true" );
  }

  /**
   * The source image which is converted to {@link #TARGET_IMAGE}.
   */
  private static final String SOURCE_IMAGE = "C:\\jp\\projects\\jipps\\src\\res\\test\\schnuefeli.tif";

  /**
   * The target image, which is deleted before starting the test.
   */
  private static final String TARGET_IMAGE = "C:\\jp\\projects\\jipps\\src\\res\\test\\schnuefeli.jpg";

  /**
   * The default constructor
   * @param name The test's name
   */
  public ImageMagickConverterTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    deleteTargetImage();
  }

  /**
   * Test method for {@link ch.ige.jipps.imageconv.ImageMagickConverter#convert(java.lang.String, java.lang.String)}.
   * @throws Exception if the test fails
   */
  public void testConvert() throws Exception
  {
    if ( StringUtils.isEmpty( System.getProperty( PropertyConstants.IMAGEMAGICK_PATH ) ) )
    {
      System.out.println( "\nGot not ImageMagick path, skipping test" );
    }
    else
    {
      ImageMagickConverter.convert( SOURCE_IMAGE, "jpg" );
      File targetImage = new File( TARGET_IMAGE );
      assertTrue( "Checking that the file was written", targetImage.exists() );
      assertTrue( "Checking file size (min. 200KB)", targetImage.length() >= 200000 );

      Image image = new ImageIcon( TARGET_IMAGE ).getImage();
      assertEquals( "Checking width", 603, image.getWidth( null ) );
      assertEquals( "Checking height", 944, image.getHeight( null ) );
    }
  }

  /**
   * Deletes {@link #TARGET_IMAGE} in order to clean up the test environment.
   */
  private void deleteTargetImage()
  {
    File targetFile = new File( TARGET_IMAGE );
    if ( targetFile.exists() )
    {
      boolean deleted = targetFile.delete();
      if ( !deleted )
      {
        System.out.println( "Couldn't delete " + TARGET_IMAGE );
      }
    }
  }

  // @see junit.framework.TestCase#tearDown()
  @Override
  protected void tearDown() throws Exception
  {
    deleteTargetImage();
    super.tearDown();
  }
}