package ch.ige.jipps.test;

import java.io.File;

import org.apache.commons.io.FileUtils;

import ch.ige.jipps.DirectControl;
import ch.ige.jipps.constants.FileConstants;
import ch.ige.jipps.xml.test.BaseDaoXmlTestTemplate;

/**
 * Tests {@link ch.ige.jipps.DirectControl}.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Eric Lewis
 * @version $Id: DirectControlTest.java,v 1.2 2007/12/18 12:36:19 lewis_lee Exp $
 */
public class DirectControlTest extends BaseDaoXmlTestTemplate
{
  // TODO This tests only a small subset

  /**
   * Contains the XML test documents. Note that this is in another project... not very nice, but still better than
   * copying.
   */
  private static final String TEST_XML_DIRECTORY = "C:/jp/projects/jippsbatch/xml_test/";

  /**
   * Contains an EP001 XML document to test with.
   */
  private static final String EP_001_TEST_XML = TEST_XML_DIRECTORY + "ep_001.xml";

  /**
   * Contains an EP002 XML document to test with.
   */
  private static final String EP_002_TEST_XML = TEST_XML_DIRECTORY + "ep_002.xml";

  /**
   * Contains an EP003 XML document to test with.
   */
  private static final String EP_003_TEST_XML = TEST_XML_DIRECTORY + "ep_003.xml";

  /**
   * The only constructor
   * @param name The test's name
   */
  public DirectControlTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.jipps.DirectControl#makePdf(java.lang.String, java.lang.String, java.lang.String)} with the
   * EP_001.
   * @throws Exception if the test fails
   */
  public void testMakePdfEP001() throws Exception
  {
    createAndSavePdf( EP_001_TEST_XML, "ep_001" );
  }

  /**
   * Tests {@link ch.ige.jipps.DirectControl#makePdf(java.lang.String, java.lang.String, java.lang.String)} with the
   * EP_002.
   * @throws Exception if the test fails
   */
  public void testMakePdfEP002() throws Exception
  {
    createAndSavePdf( EP_002_TEST_XML, "ep_002" );
  }

  /**
   * Tests {@link ch.ige.jipps.DirectControl#makePdf(java.lang.String, java.lang.String, java.lang.String)} with the
   * EP_003.
   * @throws Exception if the test fails
   */
  public void testMakePdfEP003() throws Exception
  {
    createAndSavePdf( EP_003_TEST_XML, "ep_003" );
  }

  /**
   * Creates and saves a PDF for the other test methods.
   * @param xmlFilename The XML file to render
   * @param xslFilename The XSL file to use for rendering
   * @throws Exception if the creation fails
   */
  private void createAndSavePdf( String xmlFilename, String xslFilename ) throws Exception
  {
    String xmlDocument = FileUtils.readFileToString( new File( xmlFilename ), "UTF-8" );
    byte[] pdfContent = DirectControl.getInstance().makePdf( xmlDocument, xslFilename, "01.01.2001" );

    assertNotNull( "Checking that a PDF exists", pdfContent );

    File pdfFile = File.createTempFile( "list_" + xslFilename + "_", FileConstants.EXTENSION_PDF );
    FileUtils.writeByteArrayToFile( pdfFile, pdfContent );
    System.out.println( "PDF written to " + pdfFile );
  }
}
