package ch.ige.jipps.test;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.log4j.BasicConfigurator;

import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.ipobjects.patent.vo.PatentDocumentEventKeyVO;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.jipps.JIPPSFacadeServices;
import ch.ige.jipps.JIPPSFacadeServicesImpl;
import ch.ige.jipps.vo.PrintJobVO;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.patdocutil.receipts.PatentDocumentVOWithPDF;
import ch.ige.util.enums.IPType;

/**
 * Test the patent document extraction from BAGIS and generation of PDFs.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author fabrizioli
 * @version $Id: PatentDocumentPDFInterfaceTest.java,v 1.7 2009/02/04 14:45:47 fabrizioli Exp $
 */
public class PatentDocumentPDFInterfaceTest extends AbstractTestFacadeXml
{
  static
  {
    BasicConfigurator.configure();
    ClientInitializer.initStandaloneClient();
  }

  private static final EreigIDat testEreigidat = EreigIDat.from( 2007091415 );

  private static final Integer testRegisterNr = 696655;

  private static final PatentDocumentEventKeyVO key1 = new PatentDocumentEventKeyVO( 239, 1200911712, testEreigidat );

  private static final PatentDocumentEventKeyVO key2 = new PatentDocumentEventKeyVO( 253, 1200903951, testEreigidat );

  private static final PatentDocumentEventKeyVO key3 = new PatentDocumentEventKeyVO( 257, 1200858062, testEreigidat );

  private JIPPSFacadeServices jippsFacade;

  public PatentDocumentPDFInterfaceTest( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  public void setUp() throws Exception
  {
    super.setUp();
    this.jippsFacade = new JIPPSFacadeServicesImpl();
  }

  public void testGetPatdocIds() throws Exception
  {
    assertEquals( 29, this.jippsFacade.getAllPatdocIds( testEreigidat.getDate(), null ).size() );
  }

  public void testGetPatdocIdsWithPatentnummer() throws Exception
  {
    assertEquals( 1, this.jippsFacade.getAllPatdocIds( testEreigidat.getDate(), testRegisterNr ).size() );
  }

  public void testGetPdfs() throws Exception
  {
    Map<PatentDocumentEventKeyVO, PrintJobVO> patdocids = new HashMap<PatentDocumentEventKeyVO, PrintJobVO>( 2 );
    patdocids.put( key1, createPrintJobForFTPServlet( key1.getPatentDocuemntId(), 13 ) );
    patdocids.put( key2, createPrintJobForFTPServlet( key2.getPatentDocuemntId(), 13 ) );
    List<PatentDocumentVOWithPDF> padocsWithPdf = this.jippsFacade.getPatentDocumentsWithPDF( patdocids, true );
    assertTrue( padocsWithPdf.size() == 2 );

    compareResultsWithKeys( padocsWithPdf.get( 0 ), key2 );
    compareResultsWithKeys( padocsWithPdf.get( 1 ), key1 );
  }

  public void testGetBiblio() throws Exception
  {
    Map<PatentDocumentEventKeyVO, PrintJobVO> patdocids = new HashMap<PatentDocumentEventKeyVO, PrintJobVO>( 2 );
    patdocids.put( key1, createPrintJobForFTPServlet( key1.getPatentDocuemntId(), 13 ) );
    patdocids.put( key2, createPrintJobForFTPServlet( key2.getPatentDocuemntId(), 13 ) );
    List<PatentDocumentVOWithPDF> padocsWithPdf = this.jippsFacade.getBiblioDataWithPDF( patdocids, true );
    assertTrue( padocsWithPdf.size() == 2 );

    compareResultsWithKeys( padocsWithPdf.get( 0 ), key2 );
    compareResultsWithKeys( padocsWithPdf.get( 1 ), key1 );
  }

  private void compareResultsWithKeys( PatentDocumentVOWithPDF result, PatentDocumentEventKeyVO key )
  {
    assertEquals( "Time Comparison not ok ", key.getCompareTime(), result.getVo().getCompareTime() );
    assertEquals( "PatdocId not ok ", key.getPatentDocuemntId(), result.getIdentifier() );
  }

  /**
   * Create the print job for FTPServlet
   * @param patdocId the id of the patent document
   * @param maNo the id of the employee
   * @return the print job for the ftp servlet
   */
  private PrintJobVO createPrintJobForFTPServlet( int patdocId, int maNo )
  {

    PrintJobVO printJob = new PrintJobVO();
    printJob.formTyp = PrintJobVO.PATENT_SPECIFICATION;
    printJob.schutitTyp = IPType.PATENT_CH.toDB();
    printJob.reportId = patdocId;
    printJob.maNr = maNo;
    printJob.senderApp = "FTPServlet";
    printJob.senderUser = "FTPServlet";
    printJob.sprache = PrintJobVO.USE_IP_LANGUAGE;
    printJob.druckerId = 4;
    return printJob;
  }

  // @see ch.ige.jipps.test.AbstractTestFacadeXml#compareXml(java.lang.String, java.lang.String, ch.ige.jipps.vo.ReferenceXMLVO)
  @Override
  protected void compareXml( String expectedXml, String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    // TODO Auto-generated method stub

  }

}
