package ch.ige.jipps.test.xslfo;

import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.test.AbstractTestFacadeXml;
import ch.ige.jipps.vo.ReferenceXMLVO;
import ch.ige.util.enums.report.ReportType;
import ch.ige.util.text.Replacer;

/**
 * Gets the reference XSL-FO documents of all reports and compares them to what is generated on the fly.
 *
 * <p>Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author    Eric Lewis
 * @version   $Id: FacadeXslFoTestTemplate.java,v 1.4 2009/01/26 09:33:36 lewis_lee Exp $
 */
public class FacadeXslFoTestTemplate extends AbstractTestFacadeXml
{
  /**
   * The default constructor.
   * @param name The test's name
   */
  public FacadeXslFoTestTemplate( String name )
  {
    super( name );
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  public void setUp() throws Exception
  {
    super.setUp();

    super.setUp();
    this.checkAllIpTypes = true;
    this.reportTypesToIgnore = new ReportType[] {};
    this.useStopwatch = true;
    this.useTestCaseStopwatch = true;

    // Force the images to be written
    System.setProperty( ImageProcessor.PROCESSING_MODE, ImageProcessor.PROCESS_IMAGES );
  }

  /**
   * Compares the expected XSL-FO document with the actual XSL-FO document.
   * @param expectedXml The XML document as it should be
   * @param actualXml The XML document as it was returned
   * @param referenceXmlVo The VO containing the data for the test case
   * @throws Exception if anything goes wrong
   */
  @Override
  protected void compareXml( final String expectedXml, final String actualXml, ReferenceXMLVO referenceXmlVo ) throws Exception
  {
    String correctedXml = expectedXml;

    // Special characters
    if ( referenceXmlVo.getIpKey() == 1200594499 )
    {
      // TODO The JIPPS database has ISO 8859-1, without the character 'œ'. So it should be changed to UTF-8.
      // Only first oe in history is a problem, the rest is not written as a special character
      correctedXml = Replacer.replaceFirst( correctedXml, "d'¿ufs", "d'œufs" );
    }

    // Make this really simple...
    String flatExpectedXml = flatten( correctedXml );
    String flatActualXml = flatten( actualXml );

    if ( !flatExpectedXml.equals( flatActualXml ) )
    {
      String differences = getDifferences( flatExpectedXml, flatActualXml );
      fail( "Checking XSL-FO\n" + differences, correctedXml, actualXml, getInformation( referenceXmlVo ) );
    }
  }
}
