package ch.ige.jipps.url;

import java.net.MalformedURLException;
import java.net.URL;

/**
 * Build URLs from separate parts
 * <p>
 * Copyright (c) 2006, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author bandschapp, lewis_lee
 * @version $Id: URLBuilder.java,v 1.3 2007/12/05 10:13:32 lewis_lee Exp $
 */
public class URLBuilder
{
  /**
   * The question mark used to separate servlet and parameters
   */
  public final static String QUESTION_MARK = "?";

  /**
   * The equals sign to separate parameter and value
   */
  public final static String EQUALS = "=";
  
  /**
   * The ampersand sign to join parameters
   */
  public final static String AMPERSAND = "&";

  /**
   * Builds the url with one parameter.
   * @param servlet The base URL e.g. <code>http://localhost:8080/JippsWebApp/pdfrenderservlet</code> 
   * @param parameter The parameter for the servlet
   * @param value The value of the <code>parameter</code>
   * @return A URL as String
   */
  public static String buildURLString( String servlet, String parameter, String value )
  {
    StringBuffer tmp_url = new StringBuffer( servlet );
    tmp_url.append( QUESTION_MARK );
    tmp_url.append( parameter );
    tmp_url.append( EQUALS );
    tmp_url.append( value );

    return tmp_url.toString();
  }

  /**
   * Builds the url with one parameter.
   * @param servlet The base URL e.g. <code>http://localhost:8080/JippsWebApp/pdfrenderservlet</code> 
   * @param parameter The parameter for the servlet
   * @param value The value of the <code>parameter</code>
   * @return A URL of the built string
   * @throws MalformedURLException
   */
  public static URL buildURL( String servlet, String parameter, String value ) throws MalformedURLException
  {
    return new URL( buildURLString( servlet, parameter, value ) );
  }
  
  /**
   * Builds the url with two parameters
   * @param servlet The base URL e.g. <code>http://localhost:8080/JippsWebApp/pdfrenderservlet</code> 
   * @param parameter1 The first parameter for the servlet
   * @param parameter2 The second parameter for the servlet
   * @param value1 The value of the <code>parameter1</code>
   * @param value2 The value of the <code>parameter2</code>
   * @return A URL as String
   */
  public static String buildURLString( String servlet, String parameter1, String parameter2, String value1, String value2 )
  {
    StringBuffer tmp_url = new StringBuffer( servlet );
    tmp_url.append( QUESTION_MARK );
    tmp_url.append( parameter1 );
    tmp_url.append( EQUALS );
    tmp_url.append( value1 );
    tmp_url.append( AMPERSAND );
    tmp_url.append( parameter2 );
    tmp_url.append( EQUALS );
    tmp_url.append( value2 );

    return tmp_url.toString();
  }
  
  /**
   * Builds the url with two parameters
   * @param servlet The base URL e.g. <code>http://localhost:8080/JippsWebApp/pdfrenderservlet</code> 
   * @param parameter1 The first parameter for the servlet
   * @param parameter2 The second parameter for the servlet
   * @param value1 The value of the <code>parameter1</code>
   * @param value2 The value of the <code>parameter2</code>
   * @return A URL of the built string
   * @throws MalformedURLException
   */
  public static URL buildURL( String servlet, String parameter1, String parameter2, String value1, String value2 ) throws MalformedURLException
  {
    return new URL( buildURLString( servlet, parameter1, parameter2, value1, value2 ) );
  }

}
