package ch.ige.jipps.vo;

import java.sql.Timestamp;

import ch.ige.dbutils.vo.AbstractVO;

/**
 * Represents the batch run of an automatic trademark renewal.
 * 
 * <p>Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Marc Bouquet
 * @author Eric Lewis
 * @version $Id: AmvLaufVO.java,v 1.11 2007/12/14 15:45:07 lang Exp $
 */
public class AmvLaufVO extends AbstractVO
{
  /**
   * The unique ID of the trademark renewal batch run
   */
  public int amvLaufId = 0;

  /**
   * Used for sorting; contains the creation date in the format <code>yyyyMMdd</code> plus
   * a sequence number in the format <code>nn</code>.
   */
  public String laufNr = "";

  /**
   * This is filled if the whole batch run was processed successfully
   */
  public Timestamp verarbEndeDat = null;

  /**
   * Shows that the processing has started.
   */
  public static final int PROCESSING_STARTED = 0;

  /**
   * Shows that the data is ready to be processed.
   */
  public static final int READY_FOR_PROCESSING = 1;

  /**
   * Shows that the data to process is faulty.
   */
  public static final int FAULTY_DATA = 2;

  /**
   * Shows that the data has been processed successfully.
   */
  public static final int PROCESSING_SUCCESSFUL = 3;

  /**
   * Shows that no data was found to process.
   */
  public static final int NO_DATA = 4;

  /**
   * Shows that the processing was done completely, but contained errors.
   */
  public static final int PROCESSED_WITH_ERRORS = 5;

  /**
   * Shows that the processing is running.
   */
  public static final int DATA_IS_BEING_PROCESSED = 6;

  /**
   * The processing status of the batch run.
   * @see #PROCESSING_STARTED
   * @see #READY_FOR_PROCESSING
   * @see #FAULTY_DATA
   * @see #PROCESSING_SUCCESSFUL
   * @see #NO_DATA
   * @see #PROCESSED_WITH_ERRORS
   * @see #DATA_IS_BEING_PROCESSED
   */
  public int verarbStat = PROCESSING_STARTED;

  /**
   * Number of trademark renewals for a batch run
   */
  public int anzAmv = 0;

  /**
   * The first reminder that a trademark will expire.
   */
  public static final String REMINDER_1 = "amv1";

  /**
   * The cancellation of a trademark.
   */
  public static final String CANCELLATION = "amvloesch";

  /**
   * The type of batch run
   * @see #REMINDER_1
   * @see #CANCELLATION
   */
  public String typCd = "";

  /**
   * The unique ID of the JIPPS print batch job corresponding to this.
   */
  public int batchId = 0;

  /**
   * The creation date.
   * Note that this is set automatically when this is saved in the database,
   * so don't set it and try to save it.
   */
  public Timestamp createdDat = null;

  /**
   * The date of the latest update.
   * Note that this is set automatically when this is saved in the database,
   * so don't set it and try to save it.
   */
  public Timestamp changedDat = null;
}
