package ch.ige.jipps.vo;

import java.util.List;
import java.util.ArrayList;

import ch.ige.ipobjects.common.vo.ImageVO;
import ch.ige.jipps.history.HistoryItem;
import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.jipps.text.EventTextContainer;
import ch.ige.jipps.report.ReportElement;
import ch.ige.jipps.history.detail.BaseHistoryItemDetail;

/**
 * One element (text block) on the history of the report.
 * 		<style type="text/css">
 *                        #base
 *                        {
 *                                text-align: left;
 *                                width: 900px;
 *                                height: 500px;
 *                                margin-top: 0px;
 *                                position: relative;
 *                                overflow: visible;
 *                        }
 *                        #content
 *                        {
 *                                width: 450px;
 *                                float: left;
 *                                font-weight: bold;
 *                                border-top: 1px solid grey;
 *                                border-left: 1px solid grey;
 *                                border-right: 1px solid black;
 *                                border-bottom: 1px solid black;
 *                                padding: 10px;
 *                        }
 *                        #info
 *                        {
 *                                width: 200px;
 *                                height: 300px;
 *                                margin-left: 10px;
 *                                float: left;
 *                                overflow: visible;
 *                                padding: 10px;
 *                        }
 *                        .startDate	{background-color: rgb(255, 204, 255);}
 *                        .eventText	{background-color: rgb(51, 255, 51);}
 *                        .publication	{background-color: rgb(153, 255, 255);}
 *                        .reportText	{background-color: rgb(255, 204, 0);}
 *                        .textLines	{background-color: rgb(204, 204, 255);}
 *                        .remarks	{background-color: rgb(255, 0, 0)}
 *                        .endDateLabel	{background-color: rgb(204, 153, 51);}
 *                        .endDate		{background-color: rgb(255, 255, 0);}
 *
 *                        div.date
 *                        {
 *                                width: 70px; position: absolute;
 *                        }
 *                        div.text
 *                        {
 *                                width: 300px;
 *                                margin-left: 80px;
 *                        }
 *                        div.endDateBlock
 *                        {
 *                                text-align: right;
 *                                clear: left;
 *                        }
 *                </style>
 *                <div id="base">
 *                        <div id="content">
 *                                <div>
 *                                        <div class="date"><span class="startDate">15.06.1994</span></div>
 *                                        <div class="text"><span class="eventText">Erteilung</span><br/>
 *                                                                                                <span class="publication">Ver&ouml;ffentlicht im PMMBl Nr. 11 des Jahres 1994</span><br/><br/></div>
 *
 *                                </div>
 *                                <br/>
 *                                <div>
 *                                        <div class="date"><span class="startDate">08.10.1995</span></div>
 *                                        <div class="text"><span class="eventText">Vertreter</span><br/>
 *                                                                                                <span class="reportText">Vertreter/in</span><br/>
 *                                                                                                <span class="textLines">Pantentanwaltsbüro<br/>G. Petschner<br/>Wannenstrasse 16<br/>8800 Thalwil</span><br/></div>
 *
 *                                        <div class="endDateBlock"><span class="endDateLabel">g&uuml;ltig bis:</span>&nbsp;<span class="endDate">05.08.1997</span></div>
 *                                </div>
 *                                <br/>
 *                                <div>
 *                                        <div class="date"><span class="startDate">05.08.1997</span></div>
 *                                        <div class="text"><span class="eventText">Niederlegung der Vertretung</span><br/>
 *                                                                                                <span class="remarks">Publizierter Text</span><br/><br/></div>
 *
 *                                </div>
 *                                <br/>
 *                                <div>
 *                                        <div class="date"><span class="startDate">20.10.1997</span></div>
 *                                        <div class="text"><span class="eventText">Neuer Vertreter</span><br/>
 *                                                                                                <span class="reportText">Vertreter/in</span><br/>
 *                                                                                                <span class="textLines">Isler & Pedrazzini AG<br/>Gotthardstrasse 53<br/>8023 Z&uuml;rich</span><br/>
 *
 *                                                                                                <span class="publication">Ver&ouml;ffentlicht im PMMBl Nr. 21 des Jahres 1997</span><br/><br/></div>
 *                                </div>
 *                                <br/>
 *                                <div>
 *                                        <div class="date"><span class="startDate">31.07.2004</span></div>
 *                                        <div class="text"><span class="eventText">L&ouml;schung wegen Nichtbezahlung einer Jahresgeb&uuml;hr</span><br/>
 *                                                                                                <span class="publication">Ver&ouml;ffentlicht im +pat+ Nr. 5 des Jahres 2005</span><br/><br/></div>
 *                                </div>
 *                        </div>
 *                        <div id="info">
 *                                <span class="startDate">{@link #getStartDate()}</span><br/>
 *                                <span class="eventText">{@link #getEventText()}, {@link #getTitle()}</span><br/>
 *                                <span class="publication">{@link #getPublication()}</span><br/>
 *                                <span class="reportText">{@link #getReportText()}</span><br/>
 *                                <span class="textLines">{@link #getTextLines()}</span><br/>
 *                                <span class="endDateLabel">{@link #getEndDateLabel()}</span><br/>
 *                                <span class="endDate">{@link #getEndDate()}</span><br/>
 *                                <span class="remarks">{@link #getRemarks()}</span>
 *                        </div>
 *                </div>
 * <p>Copyright (c) 2006, Swiss Federal Institute of Intellectual Property</p>
 * @author    Priska Steiner
 * @author    Elmar Isler
 * @author    Eric Lewis
 * @version   $Id: HistoryItemVO.java,v 1.2 2007/04/02 16:06:25 lewis_lee Exp $
 */
public class HistoryItemVO extends AbstractVO implements ReportElement, EventTextContainer, HistoryItem
{
  /**
   * The formatted start date
   */
  private String startDate;
  /**
   * The formatted end date
   */
  private String endDate;
  /**
   * The label for the end date, something like <code>valid to:</code>
   */
  private String endDateLabel;
  /**
   * The number of the event in the history
   */
  private Integer eventNumber;
  /**
   * The event's text
   */
  private String eventText;
  /**
   * The number of the report text for the title
   */
  private Integer reportTextNumber;
  /**
   * The report text according to {@link #reportTextNumber}
   */
  private String reportText;
  /**
   * The text lines containing the history item's details.
   */
  private List<String> textLines;
  /**
   * The remarks for the history item
   */
  private String remarks;
  /**
   * The item's image (for trademarks)
   */
  private ImageVO image;

  /**
   * The object that created this
   */
  private BaseHistoryItemDetail creator;

  /**
   * The publication text, something like
   * <code>Published in xyz no. 5/2001</code>.
   */
  private String publication;

  /**
   * The only constructor
   */
  public HistoryItemVO()
  {
    this.textLines = new ArrayList<String>();
  }

  // See interface
  public Integer getEventNumber()
  {
    return this.eventNumber;
  }

  // See interface
  public void setEventNumber( Integer eventNumber )
  {
    this.eventNumber = eventNumber;
  }

  // See interface
  public String getEventText()
  {
    return this.eventText;
  }

  // See interface
  public void setEventText( String eventText )
  {
    this.eventText = eventText;
  }

  // See interface
  public String getTitle()
  {
    return getEventText();
  }

  // See interface
  public void setTitle( String title )
  {
    setEventText( title );
  }

  // See interface
  public List<String> getTextLines()
  {
    return this.textLines;
  }

  // See interface
  public void setTextLines( List<String> textLines )
  {
    this.textLines = textLines;
  }

  // See interface
  public Integer getReportTextNumber()
  {
    return this.reportTextNumber;
  }

  // See interface
  public void setReportTextNumber( Integer reportTextNumber )
  {
    this.reportTextNumber = reportTextNumber;
  }

  // See interface
  public String getReportText()
  {
    return this.reportText;
  }

  // See interface
  public void setReportText( String reportText )
  {
    this.reportText = reportText;
  }

  // See interface
  public String getStartDate()
  {
    return this.startDate;
  }

  // See interface
  public void setStartDate( String startDate )
  {
    this.startDate = startDate;
  }

  // See interface
  public String getEndDateLabel()
  {
    return this.endDateLabel;
  }

  // See interface
  public void setEndDateLabel( String endDateLabel )
  {
    this.endDateLabel = endDateLabel;
  }

  // See interface
  public String getEndDate()
  {
    return this.endDate;
  }

  // See interface
  public void setEndDate( String endDate )
  {
    this.endDate = endDate;
  }

  // See interface
  public String getPublication()
  {
    return this.publication;
  }

  // See interface
  public void setPublication( String publication )
  {
    this.publication = publication;
  }

  // See interface
  public String getRemarks()
  {
    return this.remarks;
  }

  // See interface
  public void setRemarks( String remarks )
  {
    this.remarks = remarks;
  }

  /**
   * Gets the image.
   * @return The image
   */
  public ImageVO getImage()
  {
    return this.image;
  }

  /**
   * Sets the image.
   * @param image The image to set
   */
  public void setImage( ImageVO image )
  {
    this.image = image;
  }

  /**
   * Gets the detail builder that built this detail.
   * @return the detail builder that built this detail,
   *         which is <code>null</code> if there is no temporal data,
   *         which also means that {@link #getEndDate()} is <code>null</code>
   */
  public BaseHistoryItemDetail getCreator()
  {
    return this.creator;
  }

  /**
   * Sets the detail builder that built this detail.
   * @param creator The detail builder that built this detail
   */
  public void setCreator(BaseHistoryItemDetail creator)
  {
    this.creator = creator;
  }
}
