package ch.ige.jipps.vo.comparator;

import java.util.Comparator;
import java.util.List;

import ch.ige.jipps.text.ReportTextContainer;

/**
 * Compares two {@link ch.ige.jipps.text.ReportTextContainer} objects by
 * report text number ({@link ch.ige.jipps.text.ReportTextContainer#getReportTextNumber()}),
 * based on a predefined sequence which is given when this is created.
 *
 * <p>Copyright (c) 2006, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author    Esther Bandschapp
 * @author    Eric Lewis
 * @version   $Id: ReportTextNumberComparator.java,v 1.9 2007/05/04 08:56:36 lewis_lee Exp $
 */
public class ReportTextNumberComparator implements Comparator<ReportTextContainer>
{
  /**
   * Defines the sequence of report text numbers
   */
  private List<Integer> reportTextNumbers;

  /**
   * Creates this with the predefined sequence of report texts.
   * The sequence defines how events are sorted.
   * @param reportTextNumbers A <code>List</code> of <code>Integer</code> objects
   *        containing report text numbers
   * @throws IllegalArgumentException if <code>reportTextNumbers</code> is empty or contains
   *         anything other than <code>Integer</code> objects
   */
  public ReportTextNumberComparator( List<Integer> reportTextNumbers ) throws IllegalArgumentException
  {
    if ( reportTextNumbers == null || reportTextNumbers.isEmpty() )
    {
      throw new IllegalArgumentException( "Got no report text numbers" );
    }
    this.reportTextNumbers = reportTextNumbers;
  }

  // @see java.util.Comparator#compare(java.lang.Object, java.lang.Object)
  public int compare( ReportTextContainer reportTextContainer1, ReportTextContainer reportTextContainer2 )
  {
    Integer textNumber1 = reportTextContainer1.getReportTextNumber();
    Integer textNumber2 = reportTextContainer2.getReportTextNumber();

    Integer index1 = new Integer( this.reportTextNumbers.indexOf( textNumber1 ) );
    if ( index1.intValue() == -1 )
    {
      throw new IllegalArgumentException( "Report text number " + textNumber1 + " is not defined in the report text number sequence." );
    }
    Integer index2 = new Integer( this.reportTextNumbers.indexOf( textNumber2 ) );
    if ( index2.intValue() == -1 )
    {
      throw new IllegalArgumentException( "Report text number " + textNumber2 + " is not defined in the report text number sequence." );
    }

    return index1.compareTo( index2 );
  }
}
