package ch.ige.jipps.xml.test;

import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;
import ch.ige.jipps.imageconv.ImageProcessor;
import ch.ige.jipps.xml.ImageCropper;

/**
 * Tests {@link ch.ige.jipps.xml.ImageCropper}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: ImageCropperTest.java,v 1.3 2007/05/15 13:09:34 lewis_lee Exp $
 */
public class ImageCropperTest extends TestCase
{
  /**
   * The only constructor
   * @param args Any arguments
   */
  public ImageCropperTest( String args )
  {
    super( args );
  }

  /**
   * Tests {@link ch.ige.jipps.xml.ImageCropper#getCroppedAndFormattedSizes(java.util.Map)}.
   */
  public void testGetCroppedAndFormattedSizes()
  {
    Map<String, Object> imageProperties = new HashMap<String, Object>();
    imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 3 ) );
    imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 4 ) );
    String[] sizes = ImageCropper.getCroppedAndFormattedSizes( imageProperties );
    assertEquals( "Checking width", "3.0cm", sizes[0] );
    assertEquals( "Checking height", "4.0cm", sizes[1] );

    imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 0.1 ) );
    imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 7.9999 ) );
    sizes = ImageCropper.getCroppedAndFormattedSizes( imageProperties );
    assertEquals( "Checking width", "0.1cm", sizes[0] );
    assertEquals( "Checking height", "7.9999cm", sizes[1] );

    // Maximum size
    imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 8 ) );
    imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 8 ) );
    sizes = ImageCropper.getCroppedAndFormattedSizes( imageProperties );
    assertEquals( "Checking width", "8.0cm", sizes[0] );
    assertEquals( "Checking height", "8.0cm", sizes[1] );

    // Image too wide
    imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 8.01 ) );
    imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 3 ) );
    sizes = ImageCropper.getCroppedAndFormattedSizes( imageProperties );
    assertEquals( "Checking width", "8.0cm", sizes[0] );
    assertEquals( "Checking height", "auto", sizes[1] );

    // Image too high
    imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 5 ) );
    imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 9 ) );
    sizes = ImageCropper.getCroppedAndFormattedSizes( imageProperties );
    assertEquals( "Checking width", "auto", sizes[0] );
    assertEquals( "Checking height", "8.0cm", sizes[1] );
  }

  /**
   * Tests {@link ch.ige.jipps.xml.ImageCropper#getCroppedAndFormattedSizes(java.util.Map)} with errors.
   */
  public void testGetCroppedAndFormattedSizesWithErrors()
  {
    try
    {
      ImageCropper.getCroppedAndFormattedSizes( null );
      fail( "null Map must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      ImageCropper.getCroppedAndFormattedSizes( new HashMap<String, Object>() );
      fail( "empty Map must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 3 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Missing width must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 3 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Missing height must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Integer( 3 ) );
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 3 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Wrong width object must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 3 ) );
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Integer( 3 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Wrong height object must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 0 ) );
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 3 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Zero width must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( -0.1 ) );
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 3 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Negative width must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 3 ) );
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( 0 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Zero height must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      Map<String, Object> imageProperties = new HashMap<String, Object>();
      imageProperties.put( ImageProcessor.IMAGE_WIDTH, new Double( 3 ) );
      imageProperties.put( ImageProcessor.IMAGE_HEIGHT, new Double( -0.1 ) );
      ImageCropper.getCroppedAndFormattedSizes( imageProperties );
      fail( "Negative height must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }
  }
}