package ch.ige.pctedi.client.wpa;

import java.text.SimpleDateFormat;
import java.util.Date;

import ch.ige.pctedi.client.XMLDoc;
import ch.ige.pctedi.vo.PatVO;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: Pat.java,v 1.9 2008/03/11 14:55:12 ruegsegger Exp $
 */

public class Pat
{
  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  private WpaLogFile logFile = WpaLogFile.getInstance();

  private XMLDoc xml;

  public Pat()
  {
    sdf.setLenient( false );
  }

  public PatVO extract( XMLDoc xmlDoc ) throws Exception
  {
    this.xml = xmlDoc;
    PatVO patVO = new PatVO();

    patVO.pctAnmSpra = xml.getAttribute( "application-reference", "document-id", "lang" );
    patVO.pctPublSpra = xml.getAttribute( "publication-reference", "document-id", "lang" );
    patVO.pctPublNr = xml.getSchutitNr();
    patVO.pctAnmNr = xml.getGesuchNr();

    patVO.patVerfahr = getPatVerfahr();

    patVO.pctPublDat = getPublicationDate();

    return patVO;
  }

  /**
   * Gibt das Publikations-Datum zurück
   * Falls keines vorhanden wird empty-string in die DB geschrieben
   * @throws Exception
   * @return Date
   */
  private Date getPublicationDate() throws Exception
  {
    String publicationDate = null;
    try
    {
      publicationDate = xml.getValueOf( "publication-reference", "document-id/date" );
      return sdf.parse( publicationDate );
    }
    catch ( Exception ex )
    {
      logFile.appendMessage( WpaLogFile.WARN, "PCT " + xml.getSchutitGesuchNr() + " Kein oder ungueltiges Publlikationsdatum <" + publicationDate
          + "> vorhanden. Publikationsdatum 15.10.1582 eingefügt" );
      return null;
    }
  }

  private int getPatVerfahr() throws Exception
  {
    int patVerfahr = 0;

    boolean chPctFound = xml.exists( "designation-of-states/designation-pct", "regional[country='CH'|country='LI'" );
    boolean euroPctFound = xml.exists( "designation-of-states/designation-pct", "national[country='CH'|country='LI'" );

    if ( chPctFound && euroPctFound )
    {
      patVerfahr = PatVO.CH_EURO_PCT;
    }
    else if ( chPctFound )
    {
      patVerfahr = PatVO.CH_PCT;
    }
    else if ( euroPctFound )
    {
      patVerfahr = PatVO.EURO_PCT;
    }
    return patVerfahr;
  }

}
