package ch.ige.pctedi.client.wpa.file;

import java.io.File;
import java.io.FileFilter;
import java.util.ArrayList;
import java.util.List;

import ch.ige.pctedi.client.PctProperties;

/**
 *
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version Id:
 */
public class WpaFile extends File
{
  /**
   * Default id.
   */
  private static final long serialVersionUID = 1L;
  // statisch da innerhalb rekursiver Funktion Konstruktor aufgerufen wird
  private static List<WpaFile> listWpa = new ArrayList<WpaFile>();

  public WpaFile( String filename )
  {
    super( filename );
  }

  public WpaFile( String parent, String child )
  {
    super( parent, child );
  }

  /**
   * Gibt eine Liste von WPA Files zurück die dem WpaFileFilter entsprechen
   * Filter wird anhand Regular Expression im Properties File ermittelt
   * @return Liste WPA-Files
   */
  public List<WpaFile> listWpaFiles()
  {
    return this.listFiles( new WpaFileFilter() );
  }

  /**
   * Gibt eine Liste von WPA Files zurück die dem Filter entsprechen
   * Die Subdirectories werden rekursiv durchlaufen
   * @param filter WpaFileFilter Filter für WPA-Files
   * @return WpaFile[] Array von WPA-Files
   */
  public List<WpaFile> listFiles( WpaFileFilter filter )
  {
    listWpa = new ArrayList<WpaFile>();

    // Aufruf rekursive Funktion
    listWpaFiles( filter );

    return listWpa;
  }

  /**
   * Fügt die WPA-Files die dem Filter entsprechen einer Liste hinzu
   * @param filter WpaFileFilter Filter für WPA-Files
   */
  private void listWpaFiles( WpaFileFilter filter )
  {
    String ss[] = this.list();
    if ( ss != null )
    {
      for ( int i = 0; i < ss.length; i++ )
      {
        WpaFile file = new WpaFile( this.getPath(), ss[i] );

        if ( file.isDirectory() )
        {
          file.listWpaFiles( filter );
        }
        else if ( ( filter == null ) || filter.accept( file ) )
        {
          listWpa.add( file );
        }
      }
    }
  }

  /**
   * Filter für WPA-File muss dem Regular Expressen im Properties File entsprechen
   * <p>
   * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
   * @author A. Rueegsegger
   * @version Id:
   */
  public class WpaFileFilter implements FileFilter
  {
    public boolean accept( File file )
    {
      return file.getName().matches( PctProperties.getInstance().getProperty( PctProperties.WPA_NAME ) );
    }
  }

}
