package ch.ige.pctedi.server;

import java.sql.SQLException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.ejb.EJBException;

import ch.ige.pctedi.server.dao.BlandDAO;
import ch.ige.pctedi.server.dao.EreignisDAO;
import ch.ige.pctedi.server.dao.IpcDAO;
import ch.ige.pctedi.server.dao.MappingUnicodeDAO;
import ch.ige.pctedi.server.dao.PatDAO;
import ch.ige.pctedi.server.dao.PatIpcDAO;
import ch.ige.pctedi.server.dao.PctAdrDAO;
import ch.ige.pctedi.server.dao.PrioritaetDAO;
import ch.ige.pctedi.server.dao.SchutitDAO;
import ch.ige.pctedi.vo.EreignisVO;
import ch.ige.pctedi.vo.IpcVO;
import ch.ige.pctedi.vo.PatIpcVO;
import ch.ige.pctedi.vo.PctAdrVO;
import ch.ige.pctedi.vo.PctVO;
import ch.ige.pctedi.vo.PrioritaetVO;

/**
 * Business Object für CH-Marke TODO fix comment
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * 
 * @author Martin Trachsel, Daniel Haller Strahm
 * @version $Id: PctBO.java,v 1.22 2008/07/24 11:33:57 wild Exp $
 */

public class PctBO
{
  private static final int SCHUTIT_TYP_PCT = 3;

  private static final String SCHUTIT_KEY = "schutit_key";

  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  /**
   * Kreiert ein PCT.
   * 
   * @param pctVO Value Object von Spidi
   */
  public void createPct( PctVO pctVO )
  {

    // TODO Primärschlüssel in einer eigenen Transaktion lösen
    try
    {
      // Primärschlüssel lösen
      pctVO.schutitVO.schutitKey = NextId.getInstance().getNextId( SCHUTIT_KEY );
    }
    catch ( Exception ex )
    {
      throw new EJBException( ex );
    }

    try
    {
      // Defaultwerte ins PctVO abfüllen und in DB schreiben
      // SchutitVO
      SchutitDAO schutitDAO = new SchutitDAO();
      schutitDAO.insert( pctVO.schutitVO );
      schutitDAO = null;

      // PatVO
      pctVO.patVO.schutitKey = pctVO.schutitVO.schutitKey;

      PatDAO patDAO = new PatDAO();
      patDAO.insert( pctVO.patVO );
      patDAO = null;

      // PatIpcVO
      // Obwohl es nicht nötig ist (Daten werden richtig geliefert), 
      // stelle ich sicher, dass keine doppelten IPC-Symbole eingefügt werden. 
      if ( !pctVO.listPatIpc.isEmpty() )
      {
        Set<String> uniques = new HashSet<String>( 8 );
        PatIpcDAO patIpcDAO = new PatIpcDAO();
        for ( PatIpcVO patIpcVO: pctVO.listPatIpc )
        {
          if ( uniques.add( patIpcVO.createIdentifierForInsertUniqueness() ) )
          {
            patIpcVO.schutitKey = pctVO.schutitVO.schutitKey;
            patIpcDAO.insert( patIpcVO );
          }
        }
      }

      // PctAdrVO
      for ( PctAdrVO pctAdrVO: pctVO.listPctAdr )
      {
        pctAdrVO.schutitKey = pctVO.schutitVO.schutitKey;
        pctAdrVO.schutitTyp = SCHUTIT_TYP_PCT;

        PctAdrDAO pctAdrDAO = new PctAdrDAO();
        pctAdrDAO.insert( pctAdrVO );
        pctAdrDAO = null;
      }
      // Ereignis
      for ( EreignisVO ereignisVO: pctVO.listEreignis )
      {
        ereignisVO.schutitKey = pctVO.schutitVO.schutitKey;

        EreignisDAO ereignisDAO = new EreignisDAO();
        ereignisDAO.insert( ereignisVO );
        ereignisDAO = null;
      }
      // Prioritaet
      for ( PrioritaetVO prioritaetVO: pctVO.listPrio )
      {
        prioritaetVO.schutitKey = pctVO.schutitVO.schutitKey;

        PrioritaetDAO prioritaetDAO = new PrioritaetDAO();
        prioritaetDAO.insert( prioritaetVO );
        prioritaetDAO = null;
      }
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }

  /**
   * MR02/2008 Programm-Anpassung Gazette Section II:
   * 
   * - Falls Ereignis 30010 nicht vorhanden wird nichts gemacht 
   * - Ereignis 30040 errechnet neu Fristablauf von 30010 ab Anmelde-/Priodatum plus 30 Monate
   * - Frist Ereignis 30010 neu setzen, Abschluss Grund auf Frist setzen, und Abschlussdatum falls vorhanden löschen
   * - Abschluss Mut.-Datum setzen, damit ersichtlich ist, dass etwas angepasst wurde.
   * - Information was gemacht wurde in Logfile schreiben
   * - Ereignis 30040 neu erstellen ohne Frist, falls bereits vorhanden vorher löschen
   * - Falls 30062 bereits vorhanden, dieses annulliert
   * 
   * @param schutitKey int
   * @param publDat Date
   * @return String Meldung, was gemacht wurde
   */
  public String createExtension( int schutitKey, Date publDat )
  {
    EreignisDAO ereignisDAO = new EreignisDAO();
    SchutitDAO schutitDAO = new SchutitDAO();

    StringBuffer buf = new StringBuffer( "" );
    String message = "";

    try
    {
      // if 30010 not exists do nothing
      List<EreignisVO> listEreignis = ereignisDAO.selectArtNr( schutitKey, EreignisVO.NATIONALE_PHASE );
      if ( listEreignis != null && listEreignis.size() > 0 )
      {
        EreignisVO ereignis30010 = listEreignis.get( 0 );

        // ereignis 30010 Frist neu berechnen Anmeldedatum plus 30 Monate
        Date minDate = schutitDAO.selectMinimalDate( schutitKey );
        GregorianCalendar cal = new GregorianCalendar();
        cal.setTime( minDate );
        cal.add( Calendar.MONTH, 30 );

        ereignis30010.fristablDat = cal.getTime();
        ereignis30010.ereigStatNr = EreignisVO.EREIG_STAT_FRIST;
        ereignis30010.abschlDat = null;
        ereignis30010.abschlmutDat = new Date();

        int anz = ereignisDAO.update( ereignis30010 );
        if ( anz == 1 )
        {
          buf.append( " Frist 30010 neu berechnet" );

          message = createEreignisPruefung( schutitKey, publDat );
          buf.append( message );

          message = revokeNichteintritt( schutitKey );
          buf.append( message );
        }
        else
        {
          buf.append( " Frist 30010 nicht neu berechnet - Ereignis 30040 nicht erstellt" );
        }
      }
      else
      {
        buf.append( " Ereignis 30010 nicht vorhanden - Ereignis 30040 nicht erstellt" );
      }

      buf.append( " ( Schutitkey: " + schutitKey + ")" );
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }

    return buf.toString();
  }

  /**
   * Ereignis 30040 (Pruefung) wird falls vorhanden gelöscht 
   * Ereignis 30040 wird neu erstellt ohne Frist 
   * 
   * @param schutitKey
   * @param publDat 
   * @return String message fuer Logfile 
   * @throws SQLException    
   */
  private String createEreignisPruefung( int schutitKey, Date publDat ) throws SQLException
  {
    EreignisDAO ereignisDAO = new EreignisDAO();
    StringBuffer message = new StringBuffer( "" );

    // check if 30040 exist 
    boolean exist = ereignisDAO.existArtNr( schutitKey, EreignisVO.PRUEFUNG );

    if ( exist )
    {
      ereignisDAO.delete( schutitKey, EreignisVO.PRUEFUNG );
      message.append( " - Ereignis 30040 bereits vorhanden" );
    }

    // Ereignis 30040 ansetzen
    // Ereig_idat errechnen
    String date = sdf.format( publDat );
    int ereigIdat = new Integer( date ).intValue() * 100;
    EreignisVO ereignis30040 = new EreignisVO();

    // nun nächstes ereigidat bestimmen
    ereignis30040.ereigIdat = ereignisDAO.getNextereigIdat( schutitKey, ereigIdat );
    ereignis30040.schutitKey = schutitKey;
    ereignis30040.artNr = EreignisVO.PRUEFUNG;
    ereignis30040.ereigStatNr = EreignisVO.EREIG_STAT_KEINE_FRIST;

    int anz = ereignisDAO.insert( ereignis30040 );
    if ( anz == 1 )
    {
      message.append( " - Ereignis 30040 neu erstellt" );
    }
    else
    {
      message.append( " - Ereignis 30040 nicht neu erstellt" );
    }

    return message.toString();
  }

  /**
   * Ereignis 30062 (Nichteintritt) annullieren, falls vorhanden und noch auf "keine Frist"
   * 
   * @param schutitKey
   * @return String message für Logfile 
   * @throws SQLException   
   */
  private String revokeNichteintritt( int schutitKey ) throws SQLException
  {
    EreignisDAO ereignisDAO = new EreignisDAO();
    StringBuffer message = new StringBuffer( "" );

    List<EreignisVO> listEreignis = ereignisDAO.selectArtNr( schutitKey, EreignisVO.NICHTEINTRITT );
    for ( EreignisVO ereignis30062: listEreignis )
    {
      if ( ereignis30062.ereigStatNr == EreignisVO.EREIG_STAT_KEINE_FRIST )
      {
        ereignis30062.ereigStatNr = EreignisVO.EREIG_STAT_ANNULLIERT;
        ereignis30062.abschlmutDat = new Date();
        int updates = ereignisDAO.update( ereignis30062 );
        if ( updates == 1 )
        {
          message.append( " - Ereignis 30062 annulliert" );
        }
        else
        {
          message.append( " - Ereignis 30062 vorhanden - nicht annulliert" );
        }
      }
    }

    return message.toString();
  }

  /**
   * Sagt aus, ob der Titel auf der Datenbank gefunden wurde. Die Suche geht über Schutztitelnummer und Gesuch_nummer.
   * Je nachdem, wie der Titel gefunden wurde, gebe ich ein anderes Resultat zurück.
   * 
   * @param pctVO Werte des pct. Mindestens die Felder schutitNr und gesuchNr müssen gefüllt sein
   * @return int 0: kein Schutztitel vorhanden 1: Schutztitel vorhanden; Suche über schutitNr und gesuchNr ergaben das
   *         selbe Resultat. 2: Schutztitel vorhanden; nur Suche über schutitNr ergab ein Resultat. 3: Schutztitel
   *         vorhanden; nur Suche über GesuchNr ergab ein Resultat. 4: Schutztitel vorhanden; Suche über schutitNr und
   *         gesuchNr ergaben unterschiedliche Resultate.
   */
  public int findPct( PctVO pctVO )
  {
    int schutitKeyFromGesuchNr = selectSchutitKeyByGesuchNr( pctVO.schutitVO.gesuchNr );
    int schutitKeyFromSchutitNr = selectSchutitKey( pctVO.schutitVO.schutitNr );

    // Sofern ein Schutztitel gefunden wird..
    if ( schutitKeyFromGesuchNr != 0 || schutitKeyFromSchutitNr != 0 )
    {
      if ( schutitKeyFromGesuchNr != schutitKeyFromSchutitNr )
      {
        if ( schutitKeyFromGesuchNr != 0 & schutitKeyFromSchutitNr != 0 ) // Titel mittels GesuchNr gefunden
        {
          return 4;
        }
        else if ( schutitKeyFromSchutitNr != 0 )
        // Titel mittels SchutitNr gefunden
        {
          return 2;
        }
        else
        // ( schutitKeyFromGesuchNr != 0 )
        {
          return 3;
        }
      }
      else
      {
        pctVO.schutitVO.schutitKey = schutitKeyFromSchutitNr;
        return 1;
      }
    }
    else
    {
      return 0;
    }
  }

  /**
   * Prüft ob die Schutit_Nr vorhanden ist
   * 
   * @param schutitNr zu prüfende PCT-schutitNr
   * @return int Schutit_key
   */
  public int selectSchutitKey( String schutitNr )
  {
    try
    {
      return new SchutitDAO().selectSchutitKeyWithSchutitNr( schutitNr, SCHUTIT_TYP_PCT );
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }

  /**
   * Prüft ob die Gesuch_Nr vorhanden ist.
   * 
   * @param gesuchNr zu prüfende PCT-schutitNr
   * @return boolean
   */
  public int selectSchutitKeyByGesuchNr( String gesuchNr )
  {
    try
    {
      return new SchutitDAO().selectSchutitKeyWithGesuchNr( gesuchNr, SCHUTIT_TYP_PCT );
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert alle BLänder und gibt sie zurück.
   * 
   * @return Value Object aller BLänder
   */
  public List<String> getAllBland()
  {
    try
    {
      return new BlandDAO().getAllBland();
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert alle Unicode Mappings und gibt sie in einer Hashmap zurück.
   * 
   * @return Map HashMap mit unicode iso8859 mapping
   */
  public Map<String, String> getAllMappingUnicode()
  {
    try
    {
      return new MappingUnicodeDAO().selectAll();
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert die Daten einer IPC Klasse
   * 
   * @param ipcSymb ipc Symbol
   * @param gueltigVon Gültig von Datum
   * @return Value Object einer IPC Klasse
   */
  public IpcVO getIpc( String ipcSymb, Date gueltigVon )
  {
    try
    {
      return new IpcDAO().select( ipcSymb, gueltigVon );
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }

  /**
   * Selektiert die Daten einer IPC Klasse
   * 
   * @param ipcSymb ipc Symbol
   * @return Value Object einer IPC Klasse
   */
  public IpcVO getYoungestIpc( String ipcSymb )
  {
    try
    {
      return new IpcDAO().selectYoungest( ipcSymb );
    }
    catch ( SQLException ex )
    {
      throw new EJBException( ex );
    }
  }
}
