package ch.ige.pctedi.server.dao;

import java.sql.*;
import ch.ige.pctedi.vo.PatIpcVO;
import ch.ige.util.log.LogLevelHelper;
import ch.ige.util.server.DBHelper;

/**
 * DataAccessObject für PAT_IPC
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Martin Trachsel
 * @version $Id: PatIpcDAO.java,v 1.8 2006/12/11 14:34:59 ruegsegger Exp $
 */

public class PatIpcDAO
{

  /**
   * Fügt einen Datensatz in die Tabelle PAT_IPC ein
   * @param  patIpcVO      ValueObject
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int insert( PatIpcVO patIpcVO ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query =
          "INSERT INTO pat_ipc ( schutit_key, " +
          "pat_ipc_cd, " +
          "pat_ipc_lnr, " +
          "ipc_symb, " +
          "ipc_gueltig_von, " +
          "class_level, " +
          "class_value, " +
          "geschichte, " +
          "source," +
          "generating_office," +
          "erfass_dat, " +
          "erfass_ma, " +
          "version )" +
          "VALUES (?,?,?,?,?,?,?,?,?,?,?,?,? ) ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, patIpcVO.schutitKey );
      ps.setInt( 2, patIpcVO.patIpcCd );
      ps.setInt( 3, patIpcVO.patIpcLnr );
      ps.setString( 4, patIpcVO.ipcSymb );
      if ( patIpcVO.ipcGueltigVon == null )
      {
        ps.setDate( 5, null );
      }
      else
      {
        ps.setDate( 5, new java.sql.Date( patIpcVO.ipcGueltigVon.getTime() ) );
      }

      ps.setString( 6, patIpcVO.classLevel );
      ps.setString( 7, patIpcVO.classValue );
      ps.setString( 8, patIpcVO.geschichte );
      ps.setString( 9, patIpcVO.source );
      ps.setString( 10, patIpcVO.generatingOffice );
      if ( patIpcVO.erfassDat == null )
      {
        ps.setDate( 11, null );
      }
      else
      {
        ps.setDate( 11, new java.sql.Date( patIpcVO.erfassDat.getTime() ) );
      }
      ps.setInt( 12, patIpcVO.erfassMa );
      ps.setInt( 13, patIpcVO.version );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PatIpcDAO.class, query + patIpcVO.toString() );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

  /**
   * Selektiert einen Datensatz der Tabelle PAT_IPC
   * @return PatIpcVO ValueObject
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public PatIpcVO selectPatIpcVO( int schutitKey, int patIpcCd, int patIpcLnr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;
    ResultSet rs = null;
    PatIpcVO patIpcVO = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );

      query = "SELECT schutit_key, " +
          "pat_ipc_cd, " +
          "pat_ipc_lnr, " +
          "trim(ipc_symb) as ipc_symb, " +
          "ipc_gueltig_von, " +
          "class_level, " +
          "class_value, " +
          "geschichte, " +
          "source, " +
          "generating_office, " +
          "erfass_dat, " +
          "erfass_ma, " +
          "version " +
          "FROM pat_ipc WHERE schutit_key = ? and pat_ipc_cd = ? and pat_ipc_lnr = ? ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      ps.setInt( 2, patIpcCd );
      ps.setInt( 3, patIpcLnr );

      rs = ps.executeQuery();

      if ( rs.next() )
      {
        patIpcVO = new PatIpcVO();
        patIpcVO.schutitKey = rs.getInt( "schutit_key" );
        patIpcVO.patIpcCd = rs.getInt( "pat_ipc_cd" );
        patIpcVO.patIpcLnr = rs.getInt( "pat_ipc_lnr" );
        patIpcVO.ipcSymb = rs.getString( "ipc_symb" );
        patIpcVO.ipcGueltigVon = rs.getDate( "ipc_gueltig_von" );
        patIpcVO.classLevel = rs.getString( "class_level" );
        patIpcVO.classValue = rs.getString( "class_value" );
        patIpcVO.geschichte = rs.getString( "geschichte" );
        patIpcVO.source = rs.getString( "source" );
        patIpcVO.generatingOffice = rs.getString( "generating_office" );
        patIpcVO.erfassDat = rs.getDate( "erfass_dat" );
        patIpcVO.erfassMa = rs.getInt( "erfass_ma" );
        patIpcVO.version = rs.getInt( "version" );
      }
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PatIpcDAO.class,
                          query + " schutitKey: " + schutitKey + " patIpcCd: " + patIpcCd +
                          " patIpcLnr: " + patIpcLnr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }

    return patIpcVO;
  }

  /**
   * Löscht einen Datensatz aus der Tabelle PAT_IPC
   * @param schutitKey int PK
   * @param patIpcCd int PK
   * @param patIpcLnr int PK
   * @throws SQLException
   * @return int Anzahl gelöschte Datensätze
   */
  public int delete( int schutitKey, int patIpcCd, int patIpcLnr ) throws SQLException
  {
    Connection con = null;
    PreparedStatement ps = null;
    String query = null;

    try
    {
      con = DBHelper.getConnection( DBHelper.BAGIS );
      query = "DELETE FROM pat_ipc WHERE schutit_key = ? and pat_ipc_cd = ? and pat_ipc_lnr = ? ";
      ps = con.prepareStatement( query );
      ps.setInt( 1, schutitKey );
      ps.setInt( 2, patIpcCd );
      ps.setInt( 3, patIpcLnr );

      return ps.executeUpdate();
    }
    catch ( SQLException ex )
    {
      LogLevelHelper.log( ex, PatIpcDAO.class,
                          query + " schutitKey: " + schutitKey + " patIpcCd: " + patIpcCd +
                          " patIpcLnr: " + patIpcLnr );
      throw ex;
    }
    finally
    {
      DBHelper.close( ps, con );
    }
  }

}

