package ch.ige.pctedi.server.dao.test;

import java.text.SimpleDateFormat;
import java.util.Date;

import ch.ige.pctedi.server.dao.SchutitDAO;
import ch.ige.pctedi.vo.SchutitVO;
import ch.ige.pctedi.vo.test.Testdata;

/**
 * Junit-Test der Klasse SchutitDAO
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author A. Rueegsegger
 * @version $Id: SchutitDAOTest.java,v 1.9 2008/03/13 15:35:39 ruegsegger Exp $
 */

public class SchutitDAOTest extends DAOTestCase
{
  private SimpleDateFormat sdf = new SimpleDateFormat( "yyyyMMdd" );

  SchutitDAO schutitDAO = new SchutitDAO();

  SchutitVO schutitVO = null;

  /**
   * Konstruktor
   * @param s
   */
  public SchutitDAOTest( String s )
  {
    super( s );
  }

  /**
   * Neuer Titel auf die Datenbank schreiben
   *
   */
  public void test00Insert()
  {
    try
    {
      schutitVO = Testdata.createSchutit();
      int result = schutitDAO.insert( schutitVO );
      assertEquals( 1, result );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Schutit_key von einem PCT anhand einer Schutitnummer finden. 
   */
  public void test02Exist()
  {
    try
    {
      schutitVO = Testdata.createSchutit();
      int schutitKey = schutitDAO.selectSchutitKeyWithSchutitNr( schutitVO.schutitNr, schutitVO.schutitTyp );
      assertEquals( schutitVO.schutitKey, schutitKey );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Schutit_key von einem PCT anhand einer Gesuchsnummer finden. 
   */
  public void test03Exist()
  {
    try
    {
      schutitVO = Testdata.createSchutit();
      int schutitKey = schutitDAO.selectSchutitKeyWithGesuchNr( schutitVO.gesuchNr, schutitVO.schutitTyp );
      assertEquals( schutitVO.schutitKey, schutitKey );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Schutit_key von einem PCT anhand einer Gesuchsnummer finden. 
   * Der Titel ist nicht teil der übrigen Testdaten.
   * 
   */
  public void test04Exist()
  {
    try
    {
      schutitVO = Testdata.createSchutit();
      schutitVO.gesuchNr = "JP2000009366";
      schutitVO.schutitKey = 1200747763;

      int schutitKey = schutitDAO.selectSchutitKeyWithGesuchNr( schutitVO.gesuchNr, schutitVO.schutitTyp );
      assertEquals( schutitVO.schutitKey, schutitKey );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * 
   * unit test select minimal date of filling date or prio dat
   */
  public void test05SelectMinimalDate()
  {
    try
    {
      // test minimal date ist priodat
      int schutitKey = 1200747763;
      Date minDate = schutitDAO.selectMinimalDate( schutitKey );
      assertEquals( "20001227", sdf.format( minDate ) );

      // test more than 50 priodat
      schutitKey = 1200686465;
      minDate = schutitDAO.selectMinimalDate( schutitKey );
      assertEquals( "20000131", sdf.format( minDate ) );

      // test with no prio found
      schutitKey = 1202019399;
      minDate = schutitDAO.selectMinimalDate( schutitKey );
      assertEquals( "20070813", sdf.format( minDate ) );

      // test with no schutit found
      schutitKey = 1300000010;
      minDate = schutitDAO.selectMinimalDate( schutitKey );
      assertEquals( null, minDate );

    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

  /**
   * Titel löschen
   *
   */
  public void test07Delete()
  {
    try
    {
      schutitVO = Testdata.createSchutit();
      int result = schutitDAO.deleteSchutit( schutitVO.schutitKey );
      assertEquals( 1, result );
    }
    catch ( Exception ex )
    {
      ex.printStackTrace();
      fail( ex.getMessage() );
    }
  }

}
