package ch.ige.rbbagisexport.server.xml.test;

import java.sql.Date;
import java.util.ArrayList;
import java.util.GregorianCalendar;

import javax.xml.bind.JAXBException;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

import ch.ige.rbbagisexport.server.vo.RBExportVO;
import ch.ige.rbbagisexport.server.xml.ChSearchReportXMLConverter;

/**
 * Tests for ChSearchReportXMLConverter.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   kuenzli
 * @version  $Id: ChSearchReportXMLConverterTest.java,v 1.5 2008/01/28 13:11:52 locher Exp $
 */
public class ChSearchReportXMLConverterTest extends TestCase
{

  private static final String SOME_GREEK_LETTERS = "\u03A6\u0399\u039B\u0397\u039C\u039F\u039D\u0391";

  private static Logger LOG = null;

  private ChSearchReportXMLConverter converter = null;

  private static RBExportVO getMaximalVO()
  {
    RBExportVO exportVO = new RBExportVO();
    ArrayList<String> applicantNames = new ArrayList<String>();
    applicantNames.add( "Gesuchsteller Nummereins" );
    applicantNames.add( "Gesuchsteller Nummerzwei" );
    applicantNames.add( "Gesuchsteller Nummerdrei" );
    applicantNames.add( "Gesuchsteller Nummervier" );
    applicantNames.add( "Gesuchsteller Nummerfünf" );
    exportVO.setApplicantNames( applicantNames );
    exportVO.setApplicationDate( new Date( new GregorianCalendar( 2007, 3, 1 ).getTimeInMillis() ) );
    exportVO.setAppNr( "02333004" );
    exportVO.setInvestigator( "Inspector Clouseau" );
    ArrayList<String> ipcClasses = new ArrayList<String>();

    ipcClasses.add( "A01B" );
    ipcClasses.add( "A01C" );
    ipcClasses.add( "A01D" );

    exportVO.setIpcClasses( ipcClasses );
    exportVO.setLanguage( 3 );
    exportVO.setPriorityDate( new Date( 12313123 ) );
    exportVO.setTitle( "Wachmittel and Some Greek letters: " + SOME_GREEK_LETTERS );
    return exportVO;
  }

  private static RBExportVO getMinimalVO()
  {

    RBExportVO exportVO = new RBExportVO();
//    applicantNames.add( "Gesuchsteller Nummereins" );
//    applicantNames.add( "Gesuchsteller Nummerzwei" );
//    applicantNames.add( "Gesuchsteller Nummerdrei" );
//    exportVO.setApplicantNames( applicantNames );
    exportVO.setApplicationDate( new Date( new GregorianCalendar( 2007, 3, 1 ).getTimeInMillis() ) );
    exportVO.setAppNr( "02333004" );
//    exportVO.setInvestigator( "Inspector Clouseau" );
    ArrayList<String> ipcClasses = new ArrayList<String>();
    /*
        ipcClasses.add( "A01B" );
        ipcClasses.add( "A01C" );
        ipcClasses.add( "A01D" );
        */
    exportVO.setIpcClasses( ipcClasses );
//    exportVO.setLanguage( 2 );
//    exportVO.setPriorityDate( new Timestamp( 12313123 ) );
//    exportVO.setTitle( "Wachmittel" );
    return exportVO;
  }

  protected void setUp()
  {
    // setup logger with the appropriate configuration
    LOG = Logger.getLogger( ChSearchReportXMLConverterTest.class );
    BasicConfigurator.configure();
    converter = new ChSearchReportXMLConverter();
  }

  public ChSearchReportXMLConverterTest( String s )
  {
    super( s );
  }

  public void testMinimalVO() throws JAXBException
  {
    String xml = converter.convertToXml( getMinimalVO() );
    LOG.info( "*************** Output minimalVO test *******************" );
    LOG.info( xml );
  }

  public void testMaximalVO() throws JAXBException
  {
    String xml = converter.convertToXml( getMaximalVO() );
    assertTrue( "unicode characters corrupted", xml.contains( SOME_GREEK_LETTERS ) );
    LOG.info( "*************** Output maximalVO test *******************" );
    LOG.info( xml );
  }

}
