package ch.ige.rbbagisimport.client.batch;

import java.util.ArrayList;

import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;

/**
 * Generates Statistics about the job
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: StatistikGenerator.java,v 1.9 2008/06/10 13:52:17 wild Exp $
 */
public class StatistikGenerator
{
  // constants
  private static final String mainTitle = "Folgende Daten wurden verarbeitet:";

  private static final String titleOk = "ERFOLGREICH IMPORTIERT";

  private static final String titleNOk = "FEHLER BEIM IMPORT";

  private static final String titleDetailsOK = "DETAILS IMPORTIERTE REPORTS";

  private static final String titleDetailsNOK = "DETAILS FEHLERFÄLLE";

  private static final String subTitleDraft = "vorläufige Rechercheberichte:";

  private static final String subTitleFinal = "endgültige Rechercheberichte:";

  // fields
  private ArrayList<SearchReportFileVO> draftsOk = new ArrayList<SearchReportFileVO>();

  private ArrayList<SearchReportFileVO> draftsNok = new ArrayList<SearchReportFileVO>();

  private ArrayList<SearchReportFileVO> finalsOk = new ArrayList<SearchReportFileVO>();

  private ArrayList<SearchReportFileVO> finalsNok = new ArrayList<SearchReportFileVO>();

  public String generateStatics( ArrayList<SearchReportFileVO> files )
  {
    for ( SearchReportFileVO sr: files )
    {
      sort( sr );
    }
    return getBodyText();
  }

  private void sort( SearchReportFileVO sr )
  {
    if ( isDraft( sr ) )
    {
      if ( isFailure( sr ) )
      {
        draftsNok.add( sr );
      }
      else
      {
        draftsOk.add( sr );
      }
    }
    else
    {
      if ( isFailure( sr ) )
      {
        finalsNok.add( sr );
      }
      else
      {
        finalsOk.add( sr );
      }
    }
  }

  private String getBodyText()
  {
    StringBuilder sb = new StringBuilder();
    sb.append( mainTitle + "\n\n" );
    sb.append( titleOk + "\n" );
    sb.append( subTitleDraft + "\t" );
    sb.append( draftsOk.size() + "\n" );
    sb.append( subTitleFinal + "\t" );
    sb.append( finalsOk.size() + "\n\n\n" );
    sb.append( titleNOk + "\n" );
    sb.append( subTitleDraft + "\t" );
    sb.append( draftsNok.size() + "\n" );
    sb.append( subTitleFinal + "\t" );
    sb.append( finalsNok.size() );

    if ( draftsOk.size() > 0 || finalsOk.size() > 0 )
    {
      sb.append( "\n\n\n" );
      sb.append( titleDetailsOK + "\n" );

      for ( SearchReportFileVO sr: draftsOk )
      {
        sb.append( "\n" );
        sb.append( sr.getFilePathProcessed() );
//      sb.append( sr.getFilePath() );
      }
      for ( SearchReportFileVO sr: finalsOk )
      {
        sb.append( "\n" );
        sb.append( sr.getFilePathProcessed() );
//      sb.append( sr.getFilePath() );
      }
    }

    if ( draftsNok.size() > 0 || finalsNok.size() > 0 ) // There are Failures...
    {
      sb.append( "\n\n\n" );
      sb.append( titleDetailsNOK + "\n" );

      for ( SearchReportFileVO sr: draftsNok )
      {
        sb.append( "\n" );
        sb.append( sr.getFilePathProcessed() + "\t" + sr.getErrorMsg() );
//      sb.append( sr.getFilePath() + "\t" + sr.getErrorMsg() );
      }
      for ( SearchReportFileVO sr: finalsNok )
      {
        sb.append( "\n" );
        sb.append( sr.getFilePathProcessed() + "\t" + sr.getErrorMsg() );
//      sb.append( sr.getFilePath() + "\t" + sr.getErrorMsg() );
      }
    }
    return sb.toString();
  }

  private boolean isDraft( SearchReportFileVO file )
  {
    boolean isDraft = false;
    if ( file.getSrType() == SearchReportFileVO.SR_TYPE_DRAFT )
    {
      isDraft = true;
    }
    return isDraft;
  }

  private boolean isFailure( SearchReportFileVO file )
  {
    boolean isFailure = true;
    if ( file.getErrorMsg().equals( "" ) )
    {
      isFailure = false;
    }
    return isFailure;
  }
} // end class
