package ch.ige.rbbagisimport.server;

import java.sql.SQLException;
import java.util.ArrayList;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import ch.ige.rbbagisimport.common.vo.SearchReportFileVO;
import ch.ige.rbbagisimport.common.vo.XxSysparamsVO;
import ch.ige.rbbagisimport.server.dao.IpcDAO;
import ch.ige.rbbagisimport.server.dao.SchutitDAO;
import ch.ige.rbbagisimport.server.dao.SearchReportDAO;
import ch.ige.rbbagisimport.server.dao.XxSysparamsDAO;
import ch.ige.rbbagisimport.server.vo.IPCClassVO;
import ch.ige.rbbagisimport.server.vo.ReportVO;

/**
 * Business Object for RBWorkBean
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: RBWorkBO.java,v 1.15 2008/05/22 11:42:40 wild Exp $
 */
public class RBWorkBO
{
  // constants
  private static final Logger LOG = Logger.getLogger( RBWorkBO.class );

  // fields
  private XMLHelper helper;

  /**
   * Laden der Konfiguration und Rückgabe in einem einzigen VO
   * @return XxSysparamsVO gefüllt mit den nötigen Werten.
   * @throws RBServerException
   */
  public XxSysparamsVO getConfiguration() throws RBServerException
  {

    XxSysparamsVO sysParams = new XxSysparamsVO();
    XxSysparamsDAO spDAO = new XxSysparamsDAO();

    try
    {
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_SMTP_HOST_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_SMTP_HOST_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_MAIL_TO_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_MAIL_TO_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_MAIL_TO_ISO_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_MAIL_TO_ISO_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_MAIL_CC_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_MAIL_CC_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_MAIL_FROM_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_MAIL_FROM_PARAM_ID ) );

      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_SERVER, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_SERVER ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_PATH, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_PATH ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_FOLDER_PROCESSED_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_FOLDER_PROCESSED_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_FOLDER_ERROR_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_FOLDER_ERROR_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_DRAFT_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_DRAFT_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_IMPORT_FINAL_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_IMPORT_FINAL_PARAM_ID ) );
      sysParams.loadParam( XxSysparamsVO.RB_LOG_FOLDER_PARAM_ID, spDAO.getParameter( XxSysparamsVO.RB_LOG_FOLDER_PARAM_ID ) );
    }
    catch ( SQLException e )
    {
      String errorMessage = "Error on loading Configuration" + e.getMessage();
      LOG.error( errorMessage, e );
      throw new RBServerException( errorMessage );
    }
    return sysParams;

  }

  /**
   * Die gesamte Verarbeitung, die nötig ist um ein XML in die Datenbank zu schreiben
   * - unmarshal der XML-Daten in ein VO
   * - Suchen des Schutit_keys
   * - ergänzen der Daten mit den IPC-Klassen
   * - löschen des alten Datensatzes (sofern vorhanden) und einfügen des neuen Datensatzes.
   * 
   * @param srFile xml-File
   * @throws RBServerException
   */
  public void insertIntoBAGIS( SearchReportFileVO srFile ) throws RBServerException
  {
    // validate by creating the XMLHelper
    helper = new XMLHelper( srFile.getXml() );

    // select the "schutit_key"
    int schutitKey = getSchutitKey();
    if ( schutitKey == 0 )
    {
      throw new RBServerException( SearchReportFileVO.MSG_SCHUTIT_NOT_FOUND + helper.extractApplNr() );
    }

    // add ipc-classes only for Final Reports.
    srFile.setXml( addIpcClasses( schutitKey, srFile.getSrType() ) );

    // insert into bagis (finally!)
    insert( schutitKey, srFile.getSrType(), srFile.getXml() );

  } // end method

  /**
   * Schutit Nummer holen
   * @return int
   * @throws RBServerException
   */
  private int getSchutitKey() throws RBServerException
  {
    try
    {
      SchutitDAO schutitDAO = new SchutitDAO();
      return schutitDAO.selectSchutitKeyByApplNr( helper.extractApplNr() );
    }
    catch ( SQLException e )
    {
      LOG.error( "Error in getting the schutitKey" + "\n" + e.getMessage(), e );
      throw new RBServerException( SearchReportFileVO.MSG_SCHUTIT_NOT_FOUND + helper.extractApplNr() );
    }
  }

  /**
   * IPC Klassen auf der DB holen und ins XML schreiben
   * @param schutitKey
   * @param srType
   * @return String
   * @throws RBServerException
   */
  private String addIpcClasses( int schutitKey, int srType ) throws RBServerException
  {
    IpcDAO ipcDAO;
    try
    {
      ipcDAO = new IpcDAO();
      ArrayList<IPCClassVO> classes = ipcDAO.selectIPCClasses( schutitKey );
      if ( !IPCClassVO.hasMainClass( classes ) && srType == SearchReportFileVO.SR_TYPE_FINAL )
      {
        throw new RBServerException( SearchReportFileVO.MSG_NO_MAIN_CLASS );
      }

      return helper.addIPCClasses( classes );

    }
    catch ( SQLException e )
    {
      LOG.error( SearchReportFileVO.MSG_IPC_CLASSES_NOT_ADDED + "\n" + e.getMessage(), e );
      throw new RBServerException( SearchReportFileVO.MSG_IPC_CLASSES_NOT_ADDED );
    }
    catch ( JAXBException e )
    {
      LOG.error( "Error in addin ipc classes to xml" + "\n" + e.getMessage(), e );
      throw new RBServerException( SearchReportFileVO.MSG_IPC_CLASSES_NOT_ADDED );
    }
  }

  /**
   * Neuer Eintrag in der Tabelle Searchreport erstellen. Sofern bereits ein Eintrag besteht, wird dieser zuerst gelöscht. 
   * 
   * @param type draft oder final
   * @param xml 
   * @throws RBServerException
   */
  private void insert( int schutitKey, int type, String xml ) throws RBServerException
  {
    try
    {
      SearchReportDAO srDAO;
      srDAO = new SearchReportDAO();

      ReportVO report = new ReportVO( schutitKey, type, xml );

      // Prüfen, ob bereits ein Eintrag besteht
      ReportVO checkReport = srDAO.select( report );
      if ( checkReport != null )
      {
        srDAO.delete( checkReport.getSearchReportId() );
      }

      srDAO.insert( report );
    }
    catch ( SQLException e )
    {
      //Fehlermeldungen wurden bereits geloggt.
      throw new RBServerException( SearchReportFileVO.MSG_NO_INSERT );
    }
  }
} // end class
