package ch.ige.rbbagisimport.server.dao;

import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;

import org.apache.commons.dbutils.ResultSetHandler;
import org.apache.log4j.Logger;

import ch.ige.rbbagisimport.server.vo.IPCClassVO;

/**
 * Data-Access on IPC-Data
 * 
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bolzli
 * @version  $Id: IpcDAO.java,v 1.10 2008/04/23 13:28:09 wild Exp $
 */
public class IpcDAO extends BagisDAO
{

  private static final Logger LOG = Logger.getLogger( IpcDAO.class );

  private static final String STMT_SELECT_IPC_CLASSES = "Select p.ipc_symb, i.gueltig_von, p.class_value, p.pat_ipc_cd From ipc i, pat_ipc p Where i.ipc_symb = p.ipc_symb And p.schutit_key = ? And p.pat_ipc_cd <> 1 Order by p.pat_ipc_cd, p.pat_ipc_lnr";

  /**
   * default constructor
   * @throws SQLException
   */
  public IpcDAO() throws SQLException
  {
    super();
  }

  /**
   * Returns a list of ipc Classes for a schutitKey
   * @param schutitKey
   * @return array of ipc Classes
   * @throws SQLException
   */
  @SuppressWarnings( "unchecked" )
  public ArrayList<IPCClassVO> selectIPCClasses( int schutitKey ) throws SQLException
  {
    try
    {
      // Handler Class 
      class RSHandler implements ResultSetHandler
      {
        public Object handle( ResultSet rs ) throws SQLException
        {
          ArrayList<IPCClassVO> ipcClasses = new ArrayList<IPCClassVO>();
          int i = 0;
          while ( rs.next() )
          {
            String symb = rs.getString( "ipc_symb" ).trim();
            Date gueltig_von = rs.getDate( "gueltig_von" );
            String class_value = rs.getString( "class_value" );
            String position = i == 0 ? IPCClassVO.SYMBOL_POSITION_FIRST : IPCClassVO.SYMBOL_POSITION_LATER;
            int ipcCd = rs.getInt( "pat_ipc_cd" );
            if ( class_value.equals( "I" ) )
            {
              class_value = IPCClassVO.CLASSIFICATION_VALUE_INVENTION;
            }
            else
            {
              class_value = IPCClassVO.CLASSIFICATION_VALUE_NON_INVENTION;
            }

            ipcClasses.add( new IPCClassVO( symb, gueltig_von, position, class_value, ipcCd ) );
            i++;
          }
          return ipcClasses;
        }
      }

      String ps; // prepared statement
      Object[] params; // parameters
      ResultSetHandler rsh; // result set handler

      ps = STMT_SELECT_IPC_CLASSES;
      params = new Object[] { schutitKey };
      rsh = new RSHandler();

      return (ArrayList<IPCClassVO>) run.query( ps, params, rsh );
    }
    catch ( SQLException ex )
    {
      String message = "error in selectIPCClasses " + "  " + ex.getMessage();
      LOG.error( message );
      throw new SQLException( message );
    }

  }

} // end method