package ch.ige.st36xmlgenerator;

import java.rmi.RemoteException;
import java.util.List;
import java.util.Map;

import ch.ige.ipobjects.event.vo.EreignisVO;
import ch.ige.xmlutil.doc.XMLDocument;

/**
 * The generator of the ST.36 XML documents
 * @author fabrizioli
 *
 */
public interface ST36BiblGenerator
{

  /**
   * Get the Configuration from the database 
   * @return the configuration stored in the database
   * @throws RemoteException
   */
  public ST36BiblGeneratorProperties getConfiguration() throws RemoteException;

  /**
   * Get the events that shall be publicated together with a patent documentation specification 
   * @param props the properties of the investigation, indicating the time period to investigate,...
   * @return the map of events per schutit, that shall be publicated
   * @throws RemoteException
   */
  public Map<Integer, List<EreignisVO>> getEventsToPublicate( ST36BiblGeneratorProperties props ) throws RemoteException;

  /**
   * Generate the bibliographic data for the events passed as input parameter
   * @param eventsPerSchutitKey the evnets per schutit that require the generation of the bibliographic data for the patent document specification 
   * @param props the properties of the investigation
   * @return the {@link XMLDocument} instances that contain the bibliographic data for the patent specification for each event
   * @throws RemoteException
   */
  public Map<Integer, List<XMLDocumentWithMetadata>> generateBibliographicData( Map<Integer, List<EreignisVO>> eventsPerSchutitKey,
      ST36BiblGeneratorProperties props ) throws RemoteException;

  /**
   * Update the configuration table with the new start ereigidat
   * @param props
   * @throws RemoteException
   */
  public void updateStartEreigidat( ST36BiblGeneratorProperties props ) throws RemoteException;
}
