package ch.ige.st36xmlgenerator;

import java.rmi.RemoteException;
import java.sql.Date;

import javax.ejb.CreateException;

import ch.ige.st36xmlgenerator.ejb.ST36BiblGeneratorFacadeHome;
import ch.ige.util.ServiceLocator;
import ch.ige.util.exception.ServiceLocatorException;

/**
 * Factory for the st36xmlgenerator.
 * <p>
 * The st36xmlgenerator can be created as EJB or as standalone client.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * 
 * @author Patrizia Fabrizioli
 * @version $Id: ST36BiblGeneratorFactory.java,v 1.6 2008/09/25 07:29:27 fabrizioli Exp $
 */
public abstract class ST36BiblGeneratorFactory
{
  private static boolean runsInJ2EEMode = false;

  private static volatile ST36BiblGenerator services = null;

  protected ST36BiblGeneratorFactory()
  {
    super();
  }

  protected static final ST36BiblGenerator getInitializedInstance()
  {
    if ( runsInJ2EEMode )
    {
      if ( services == null )
      {
        throw new IllegalStateException( "factory not initalized" );
      }
      return ST36BiblGeneratorFactory.services;
    }
    else
    {
      return getStandalone();
    }
  }

  /**
   * Get the generator in standalone mode
   */
  public static final ST36BiblGenerator getStandalone()
  {
    if ( ST36BiblGeneratorFactory.services == null )
    {
      runsInJ2EEMode = false;
      ST36BiblGeneratorImpl impl = new ST36BiblGeneratorImpl();
      impl.initStandalone();
      services = impl;
    }
    return ST36BiblGeneratorFactory.services;
  }

  /**
   * Get the EJB stub
   */
  public static final ST36BiblGenerator getEJB() throws RemoteException, CreateException, ServiceLocatorException
  {
    if ( ST36BiblGeneratorFactory.services == null )
    {
      runsInJ2EEMode = true;
      ST36BiblGeneratorFacadeHome home = ServiceLocator.getInstance().getHome( ST36BiblGeneratorFacadeHome.JNDI_NAME, ST36BiblGeneratorFacadeHome.class );
      ST36BiblGeneratorFactory.services = home.create();
    }

    return ST36BiblGeneratorFactory.services;
  }

  /**
   * for tests only
   * @param publDate the date of production that shall be set
   */
  public static ST36BiblGenerator getStandaloneForTest(final Date prodDate)
  {
    if ( ST36BiblGeneratorFactory.services == null )
    {
      runsInJ2EEMode = false;
      ST36BiblGeneratorImpl impl =  new ST36BiblGeneratorImpl(){ @Override protected Date getProductionDate()
      {
        return prodDate;
      }
      };
      impl.initStandalone();
      services = impl;
    }
    return ST36BiblGeneratorFactory.services;
  }

}
