package ch.ige.st36xmlgenerator.builder;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import org.apache.log4j.Logger;

import ch.ige.st36xmlgenerator.exception.ST36GenerationRuntimeException;
import ch.ige.st36xmlgenerator.format.XMLElementConverter;

public abstract class AbstractTreeObjectBuilder implements ObjectBuilder {

	private static final Logger LOG = Logger.getLogger(AbstractTreeObjectBuilder.class);


	public void setObject(RecursionInformation info) 
	{
		Object actualElement = info.getActualObject();
		Object objectToSet = info.getObjectToSet();
		Method setter = getMethod(actualElement
				.getClass(), getSetterMethodName(info.getNextElement()));
			assert (setter != null);
			try {
				setter.invoke(actualElement,
						new Object[] { objectToSet });
				info.setActualElement(info.getNextElement());
				info.setActualObject(objectToSet);
				System.out.println("object set :" + objectToSet);
			} catch (IllegalAccessException e) {
				LOG.error(e);
			} catch (InvocationTargetException e) {
				LOG.error(e);
			}

	}

	protected Class<?> getClass(String xpathElementName)	throws ClassNotFoundException 
	{
		Class<?> nextClass = null;
		try 
		{
			nextClass = Class.forName("ch.ige.chpatdoc."
					+ XMLElementConverter.toClassName(xpathElementName));
		} catch (Exception e) 
		{
			nextClass = Class.forName("ch.ige.chpatdoc."
					+ XMLElementConverter.toRefClassName(xpathElementName));
		}

		return nextClass;
	}
	
	protected boolean multipleOccurrenceIn(String nextElement) {
		if (nextElement.equals("invention-title"))
		{
			return true;
		}
		return false;
	}



	protected Object getNextObject(Object actualObject, String nextElement)
			throws IllegalArgumentException, IllegalAccessException,
			InvocationTargetException {
		Method getter = getMethod(actualObject.getClass(),
				getGetterMethodName(nextElement));

		assert (getter != null);

		Object nextNode = getter.invoke(actualObject, new Object[] {});
		return nextNode;
	}

	public boolean nextObjectIsAlreadySet(RecursionInformation info) {
		Object actualObject = info.getActualObject();
		String nextElement = info.getNextElement();

		try {
			Object nextNode = getNextObject(actualObject,
					nextElement);
			if (nextNode != null) {
				info.setActualElement(nextElement);
				info.setActualObject(nextNode);
				return true;
			}
		} catch (IllegalAccessException e) {
			LOG.error(e);
		} catch (InvocationTargetException e) {
			LOG.error(e);
		}
		return false;
	}

	protected String getGetterMethodName(String xpathElementName) {
		return XMLElementConverter.toGetter(xpathElementName);
	}

	protected String getSetterMethodName(String xpathElementName) {
		return XMLElementConverter.toSetter(xpathElementName);
	}



	public Method getMethod(Class<?> providingClass, String methodName)
			throws ST36GenerationRuntimeException {
		Method[] methods = providingClass.getDeclaredMethods();
		for (int i = 0; i < methods.length; i++) {
			if (methods[i].getName().equals(methodName)) {
				return methods[i];
			}
		}
		// look in methods of superclass
		methods = providingClass.getMethods();
		for (int i = 0; i < methods.length; i++) {
			if (methods[i].getName().equals(methodName)) {
				return methods[i];
			}
		}
		final String errorMsg = "method '" + methodName + "' not found in "
				+ providingClass.getName();
		LOG.fatal(errorMsg);
		throw new ST36GenerationRuntimeException(errorMsg);
	}

}
