package ch.ige.ses.client.sapinbound;

import java.util.*;

import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.SAPAdrStatusVO;
import ch.ige.util.vo.*;
import com.sap.mw.jco.JCO;
import org.apache.log4j.Logger;

/**
 * SAP Remote Function Call's for automatic Mode.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: AutoRFC.java,v 1.31 2006/12/11 12:43:32 iten Exp $
 */
public class AutoRFC extends RFC
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(AutoRFC.class);

  //
  // fields
  //
  private double balance = 0;
  private ArrayList alArtikel = null;

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public AutoRFC()
  {
    super ();
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * create billing request
   *
   * @param  vo       in:  sap billing data
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int createBillingReq(SAPRechnungVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      rfm.addRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("createBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("createBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("createBillingReq - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * create billing request
   *
   * @param  list     in:  list of SAPRechnungVO's
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int createBillingReq(ArrayList list,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;
    Iterator                          it      = null;


    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ(this.fJCoRepository);
      it  = list.iterator();
      rfcOpen();

      while ( it.hasNext() )
      {
        rfm.addRechnung( (SAPRechnungVO)it.next() );
      }

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("createBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("createBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("createBillingReq - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * change billing request
   *
   * @param  vo       in:  sap billing data
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int changeBillingReq(SAPRechnungVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      rfm.addRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("changeBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("changeBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("changeBillingReq - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * commit annual fee run
   *
   * @param jgbLaufNr in: sequence number of run
   * @param retMsg    out: return message (only set in case of a failure)
   * @return          rfc return code (siehe rfc.java)
   */
  public int commitAnnualFee(String jgbLaufNr, StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_COMMIT_ANNUAL_FEE  rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_COMMIT_ANNUAL_FEE(this.fJCoRepository);

      rfcOpen();
      rfm.addSeqNumber(jgbLaufNr);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("commitAnnualFee - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("commitAnnualFee - exception: " + ex.toString() + "\n");
      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * commit tm extension
   *
   * @param amvLaufNr in: sequence number of run
   * @param retMsg    out: return message (only set in case of a failure)
   * @return          rfc return code (siehe rfc.java)
   */
  public int commitTmExtension(String amvLaufNr, StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_COMMIT_TM_EXTENSION  rfm     = null;
    int                                 retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_COMMIT_TM_EXTENSION(this.fJCoRepository);

      rfcOpen();
      rfm.addSeqNumber(amvLaufNr);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("commitTmExtension - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("commitTmExtension - exception: " + ex.toString() + "\n");
      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * reject billing request
   *
   * @param  vo       in:  sap reject data
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code
   */
  public int rejectBillingReq(SAPRechnungAnnullationVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_REJECT_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_REJECT_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      rfm.addAnnullation(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("rejectBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(JCO.Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("rejectBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("rejectBillingReq - hint (next step): " + rfm.fNextStep  + "\n");

      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * delete billing request
   *
   * @param  vo       in:  sap billing data
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code
   */
  public int deleteBillingReq(SAPRechnungVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_DELETE_BILLING_REQ rfm     = null;
    int                               retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_DELETE_BILLING_REQ(this.fJCoRepository);
      rfcOpen();

      rfm.delRechnung(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("deleteBillingReq - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }

    catch(JCO.Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("deleteBillingReq - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("deleteBillingReq - hint (next step): " + rfm.fNextStep  + "\n");

      LOG.error(retMsg.toString());
    }

    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * Customer Create
   *
   * @param  vo       in:  Adresse
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int customerCreate(VersadrVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_CUSTOMER_CREATE  rfm     = null;
    int                         retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_CUSTOMER_CREATE(this.fJCoRepository);
      rfcOpen();

      rfm.addCustomer(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("customerCreate - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }
    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("customerCreate - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("customerCreate - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * Customer Delete
   *
   * @param  vo       in:  adresse status
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int customerDelete(SAPAdrStatusVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_CUSTOMER_DELETE  rfm     = null;
    int                         retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_CUSTOMER_DELETE(this.fJCoRepository);
      rfcOpen();

      rfm.deleteCustomer(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("customerDelete - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }
    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("customerDelete - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("customerDelete - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * Customer Block
   *
   * @param  vo       in:  adresse status
   * @param  retMsg   out: return message (only set in case of a failure)
   * @return rfc return code (siehe rfc.java)
   */
  public int customerBlock(SAPAdrStatusVO vo,StringBuffer retMsg)
  {
    RFM_Z_BAPI_CUSTOMER_BLOCK  rfm     = null;
    int                        retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_CUSTOMER_BLOCK(this.fJCoRepository);
      rfcOpen();

      rfm.blockCustomer(vo);

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("customerBlock - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }
    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("customerBlock - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("customerBlock - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * check if the given customer is available within sap
   *
   * @param   adrNr customer number
   * @return  true if customer is missing,
   *          false if customer exists or in case of an error
   */
  public boolean customerMissing (int adrNr)
  {
    RFM_BAPI_CUSTOMER_CHECKEXISTENCE  rfm     = null;
    boolean                           missing = false;

    try
    {
      rfm = new RFM_BAPI_CUSTOMER_CHECKEXISTENCE(this.fJCoRepository);
      rfcOpen();

      rfm.addCustomer(adrNr);

      rfcExec(rfm.fJCoFunc);

      missing = rfm.missingCustomer();

    }
    catch(Exception ex)
    {
      // write only a log message, do no actions at all
      LOG.error("customerMissing - exception: " + ex.toString() );
      if (rfm != null)
        LOG.error("customerMissing - hint (next step): " + rfm.fNextStep );
    }
    finally
    {
      rfcClose();
    }

    return missing;

  } // end method


  /**
   * Überprüft die Abhängigkeiten einer ganze Liste von VersadrCheckVO's im
   * SAP System. Die Versand Adressen welche im SAP System noch mit einem
   * offenen Auftrag verknüpft sind, dürfen nicht gelöscht oder gesperrt werden.
   *
   * @param   list    in/out: Liste mit zu prüfenden VersadrCheckVO's und Resultat
   * @param   retMsg  out: return message (only set in case of a failure)
   * @return          rfc return code (siehe rfc.java)
   */
  public int customerCheck (ArrayList list, StringBuffer retMsg)
  {
    RFM_Z_BAPI_CUSTOMER_CHECK  rfm     = null;
    int                        retCode = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_CUSTOMER_CHECK(this.fJCoRepository);

      rfcOpen();
      rfm.addVersadrList(list);
      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("customerCheck - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
      else
      {
        // get return list
        rfm.checkVersadrList(list);

        if (LOG.isDebugEnabled())
        {
          for (Iterator iter = list.iterator(); iter.hasNext(); )
          {
            VersadrCheckVO item = (VersadrCheckVO)iter.next();
            LOG.debug ( "adrNr=" + item.adrNr +
                        ", openOrder=" + item.openOrder +
                        ", openOrderMsg=" + item.openOrderMsg );
          }
        } // end if

      }
    }
    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("customerCheck - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("customerCheck - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  public int insertStako(ArrayList list,StringBuffer retMsg)
  {
    RFM_Z_BAPI_STAKO_INSERT rfm     = null;
    int retCode                     = RFC.RET_CODE_SUCCESS;
    Iterator                          it      = null;

    try
    {
      rfm = new RFM_Z_BAPI_STAKO_INSERT(this.fJCoRepository);
      it  = list.iterator();
      rfcOpen();

      while ( it.hasNext() )
      {
        rfm.addStako( (StakoVO)it.next() );
      }

      rfcExec(rfm.fJCoFunc);

      if ( ! rfm.isReturnOk() )
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;

        retMsg.append("insertStako - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }
    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("insertStako - exception: " + ex.toString() + "\n");
      if (rfm != null)
        retMsg.append("insertStako - hint (next step): " + rfm.fNextStep + "\n");

      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * insertStako
   *
   * @param vo      in: ValueObject of Stako - Elements
   * @param retMsg  out: return message (only set in case of failure)
   * @return        rfc return code (see rfc.java)
   * @author        Patrick Bolzli
   */
  public int insertStako(StakoVO vo, StringBuffer retMsg)
  {
    RFM_Z_BAPI_STAKO_INSERT rfm     = null;
    int retCode                     = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_STAKO_INSERT(this.fJCoRepository);
      rfcOpen();
      rfm.addStako(vo);
      rfcExec(rfm.fJCoFunc);

      if (!rfm.isReturnOk())
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;
        retMsg.append("insertStako - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }
    catch (Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("insertStako - exception: " + ex.toString() + "\n");
      if (rfm != null)
      {
        retMsg.append("insertStako - hint (next step): " +rfm.fNextStep + "\n");
      }
      LOG.error(retMsg.toString());

    }
    finally
    {
      rfcClose();
    }

    return retCode;

  } // end method


  /**
   * getArtikel
   * @param retMsg  out: return message (only set in case of failure)
   * @return        rfc return code (see rfc.java)
   * @author        Patrick Bolzli
   */
  public int getArtikel(StringBuffer retMsg)
  {
    RFM_Z_BAPI_STAKO_GET_ARTIKEL rfm  = null;
    int retCode                       = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_STAKO_GET_ARTIKEL(this.fJCoRepository);
      rfcOpen();
      rfcExec(rfm.fJCoFunc);

      if (!rfm.isReturnOk())
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;
        retMsg.append("getArtikel - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
      else
      {
        alArtikel = rfm.getArtikel();
      }
    }
    catch (Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("getArtikel - exception: " + ex.toString() + "\n");
      if (rfm != null)
      {
        retMsg.append("getArtikel - hint (next step): " +rfm.fNextStep + "\n");
      }
      LOG.error(retMsg.toString());

    }
    finally
    {
      rfcClose();
    }

    return retCode;
  }

  /**
   * Gibt eine ArrayList der stakorelevanten Artikel zurück
   * @return alArtikel stakorelevante Artikel
   */
  public ArrayList getArtAL()
  {
    return alArtikel;
  }


  /**
   * existenceCheck
   * @param art_nr    in:   Artikelnummer
   * @param retMsg    out:  return message (only set in case of failure)
   * @return          rfc return code (see rfc.java)
   * @author          Patrick Bolzli
   */
  public int existenceCheck(String art_nr, StringBuffer retMsg)
  {
    RFM_BAPI_MATERIAL_EXISTENCECHECK rfm  = null;
    int retCode                           = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_BAPI_MATERIAL_EXISTENCECHECK(this.fJCoRepository);
      rfcOpen();

      rfm.existenceCheck(art_nr);
      rfcExec(rfm.fJCoFunc);

      if (!rfm.isReturnOk())
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;
        retMsg.append("existenceCheck - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
    }
    catch (Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("existenceCheck - exception: " + ex.toString() + "\n");
      if (rfm != null)
      {
        retMsg.append("existenceCheck - hint (next step): " +rfm.fNextStep + "\n");
      }
      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }
    return retCode;

  }    // end method

  /**
   * CustomerBalance
   * @param ktoNr     in: Kontonummer des Deponentenkontos (entspricht adr_nr)
   * @param retMsg    out: return message (only set in case of failure)
   * @return          rfc return code (see rfc.java)
   * @author          Patrick Bolzli
   */
  public int customerBalance(int ktoNr, StringBuffer retMsg)
  {
    RFM_Z_BAPI_ZVL_GET_CUST_BALANCE rfm = null;
    int retCode                         = RFC.RET_CODE_SUCCESS;

    try
    {
      rfm = new RFM_Z_BAPI_ZVL_GET_CUST_BALANCE(this.fJCoRepository);
      rfcOpen();

      rfm.getBalance(ktoNr);
      rfcExec(rfm.fJCoFunc);

      if (! rfm.isReturnOk())
      {
        retCode = RFC.RET_CODE_RETURN_FAILURE;
        retMsg.append("customerBalance - sap return error: \n");
        retMsg.append(rfm.toStringReturnNOK());

        LOG.error(retMsg.toString());
      }
      else
      {
        balance = rfm.getRetval();
      }
    }
    catch (Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      retMsg.append("customerBalance - exception: " + ex.toString() + "\n");
      if (rfm != null)
      {
        retMsg.append("customerBalance - hint (next step): " +rfm.fNextStep + "\n");
      }
      LOG.error(retMsg.toString());
    }
    finally
    {
      rfcClose();
    }

    return retCode;
  } // end method

  /**
   * Gibt den ermittelten Kontosaldo zurück
   * @return balance Kontosaldo
   */
  public double getBalanceRetval()
  {
    return this.balance;
  }


  /**
   * test connection to sap system and return sap system info if the
   * connection works fine.<p>
   *
   * note: the rfm.toStringExport() method returns a string with to lines.<br>
   * example:<br>
   *    "ECHOTEXT = SapInbound Client - ProgramId: 00" <br>
   *    "RESPTEXT = SAP R/3 Rel. 46C   Sysid: E01 ..."<br>
   * this text will be filterd and only the sap response text in the form:<br>
   *    "SAP R/3 Rel. 46C   Sysid: E01 ..."<br>
   * will be returned to the caller.<p>
   *
   * @param echoText in:  text which will be sent to the sap system
   * @param retMsg   out: return message (sap response or error message)
   * @return rfc return code
   */
  public int testConnection(String echoText,StringBuffer retMsg)
  {
    RFM_STFC_CONNECTION rfm     = null;
    int                 retCode = RFC.RET_CODE_SUCCESS;
    String              match   = "RESPTEXT = ";
    String              buf     = null;
    int                 maxLen  = 70;


    try
    {
      // init
      rfm = new RFM_STFC_CONNECTION(this.fJCoRepository);
      rfcOpen2();

      // set params & execute call
      rfm.add(echoText);
      rfcExec2(rfm.fJCoFunc);

      // filter response (skip carrige return at end of string)
      buf = rfm.toStringExport();
      retMsg.append(buf.substring(buf.lastIndexOf(match)+match.length(),buf.length()-1));
    }

    catch(Exception ex)
    {
      retCode = RFC.RET_CODE_EXCEPTION;

      // remove carrige return from exception text and cut length
      buf = ex.getMessage().replace('\n','#');
      if (buf.length() < maxLen)
      {
        retMsg.append( buf.substring(0,buf.length()) );
      }
      else
      {
        retMsg.append( buf.substring(0,maxLen) + "..." );
      }

    }

    finally
    {
      rfcClose2();
    }

    return retCode;

  } // end method


} // end class


