package ch.ige.ses.client.sapinbound;

import ch.ige.ses.vo.bagis.*;
import java.io.*;
import java.util.*;

import ch.ige.ses.BuildInfo;
import ch.ige.ses.client.saprequest.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.client.sapinbound.test.*;
import ch.ige.ses.exception.SapCallException;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.server.sapctrl.*;
import ch.ige.ses.vo.ses.*;
import ch.ige.util.vo.*;


/**
 * Diese Klasse wird für den manuellen Modus verwendet.
 * Sie schreibt direkt auf den Standard - Output.
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: ManualMode.java,v 1.81 2007/12/18 16:47:59 iten Exp $
 */
public class ManualMode
{
  //
  // fields
  //
  private ManualRFC   fRFC;               // sap rfc calls


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public ManualMode()
  {
    this.fRFC = new ManualRFC();
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void run() throws Exception
  {
    boolean  go      = true;
    int      menuIdx = 0;
    int      cmd;

    //
    // init
    //
    printBuildInfo();
    printParams();
    printProperties("SAP Inbound Client - Properties",SapInbound.getProperties());
    printProperties("System - Properties",System.getProperties());

    //
    // run
    //
    while (go)
    {

      try
      {

        cmd = menuShow(menuIdx);

        if (cmd == 99)
        {
          go = false;
          System.out.println("Exit Sap Inbound Client, good by...");
          System.out.println("");
        }
        else if (cmd < 99)
        {
          menuIdx = cmd;
        }
        else
        {
          menuExecute(cmd);
        }

      }

      catch (Exception ex)
      {
        System.out.println("");
        System.out.println("### Achtung folgende Exception ist aufgetreten:");
        System.out.println(ex.getMessage());
        System.out.println("Stack Trace:");
        ex.printStackTrace();
      }

    } // end while

  } // end run


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private int menuShow(int menu)
  {
    String      buf;
    int         cmd   = 0;
    boolean     valid = false;


    System.out.println();

    switch (menu)
    {
      case 0:
        System.out.println("SAP Inbound Client (Manual Mode) - Hauptmenu          ");
        System.out.println("------------------------------------------------------");
        System.out.println("    1 : Systeminformationen                           ");
        System.out.println("    2 : Rechnungen                                    ");
        System.out.println("    3 : Jahresgebuehren  / Markenverlaengerung        ");
        System.out.println("    4 : Adressen                                      ");
        System.out.println("    5 : Deponentenkonto                               ");
        System.out.println("    6 : Artikel                                       ");
        System.out.println("    7 : Controlling                                   ");
        System.out.println("    8 : Divers                                        ");
        System.out.println("                                                      ");
        System.out.println("   99 : EXIT                                          ");
        break;

      case 1:
        System.out.println("SAP Inbound Client (Manual Mode) - Systeminformationen");
        System.out.println("------------------------------------------------------");
        System.out.println("  100 : Programm Info,Parameter & Properies anzeigen  ");
        System.out.println("  110 : STFC_CONNECTION                               ");
        System.out.println("  111 : RFC_SYSTEM_INFO                               ");
        System.out.println("  120 : ping app server (receive-ping)                ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 2:
        System.out.println("SAP Inbound Client (Manual Mode) - Rechnungen         ");
        System.out.println("------------------------------------------------------");
        System.out.println("  210 : create rechnung - enter bfakt_id              ");
        System.out.println("  211 : create rechnung - enter all                   ");
        System.out.println("  212 : create rechnung - load batch commit           ");
        System.out.println("  213 : create rechnung - load single commit          ");
        System.out.println("  214 : reject rechnung - enter all                   ");
        System.out.println("  216 : delete rechnung - enter bfakt_id              ");
        System.out.println("                                                      ");
        System.out.println("  220 : bean send rechnung - enter bfakt_id           ");
        System.out.println("  221 : bean send rechnung - enter all                ");
        System.out.println("  222 : bean send rechnung - load                     ");
        System.out.println("  224 : bean send annulation - enter all              ");
        System.out.println("  225 : bean send annulation list - enter all         ");
        System.out.println("  226 : bean send del/crea rechnung - enter all       ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 3:
        System.out.println("SAP Inbound Client (Manual Mode) - JGB / AMV          ");
        System.out.println("------------------------------------------------------");
        System.out.println("  310 : jgb - jgb job (inkl. block size)              ");
        System.out.println("  311 : jgb - jgb force finish                        ");
        System.out.println("  320 : jgb - bean send jgb job (inkl. block size)    ");
        System.out.println("  321 : jgb - bean send jgb force finish              ");
        System.out.println("  330 : jgb - insert jgb data - default params        ");
        System.out.println("  331 : jgb - insert jgb data - enter params          ");
        System.out.println("                                                      ");
        System.out.println("  340 : amv - create erinnerung 1                     ");
        System.out.println("  341 : amv - create erinnerung 2 (e1 must exist !)   ");
        System.out.println("  342 : amv - create erinnerung full                  ");
        System.out.println("  343 : amv - commit tm extension                     ");
        System.out.println("  350 : amv - bean send create erinnerung 1           ");
        System.out.println("  351 : amv - bean send create erinnerung 2 (e1...)   ");
        System.out.println("  352 : amv - bean send create erinnerung full        ");
        System.out.println("  353 : amv - bean send commit tm extension           ");
        System.out.println("  360 : amv - insert amv data                         ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 4:
        System.out.println("SAP Inbound Client (Manual Mode) - Adressen           ");
        System.out.println("------------------------------------------------------");
        System.out.println("  410 : adresse create / update                       ");
        System.out.println("  411 : adresse delete / undelete                     ");
        System.out.println("  412 : adresse block / unblock                       ");
        System.out.println("  413 : adresse check existence                       ");
        System.out.println("  414 : adresse check existence by file               ");
        System.out.println("  415 : adresse check open order                      ");
        System.out.println("                                                      ");
        System.out.println("  420 : bean send adresse create / update             ");
        System.out.println("  421 : bean send adresse delete / undelete           ");
        System.out.println("  422 : bean send adresse block / unblock             ");
        System.out.println("                                                      ");
        System.out.println("  430 : client request adresse create / update        ");
        System.out.println("  431 : client request adresse delete / undelete      ");
        System.out.println("  432 : client request adresse block / unblock        ");
        System.out.println("  435 : client request adresse check open order       ");
        System.out.println("                                                      ");
        System.out.println("  440 : client request adresse create multiple threads");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 5:
        System.out.println("SAP Inbound Client (Manual Mode) - Deponentenkonto    ");
        System.out.println("------------------------------------------------------");
        System.out.println("  510 : dep-kto get cust balance (z.B. 500442)        ");
        System.out.println("                                                      ");
        System.out.println("  530 : client request dep-kto get cust balance       ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 6:
        System.out.println("SAP Inbound Client (Manual Mode) - Artikel            ");
        System.out.println("------------------------------------------------------");
        System.out.println("  610 : artikel existenceCheck (z.B. 18500)           ");
        System.out.println("                                                      ");
        System.out.println("  630 : client request artikel existenceCheck         ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 7:
        System.out.println("SAP Inbound Client (Manual Mode) - Controlling        ");
        System.out.println("------------------------------------------------------");
        System.out.println("  710 : getArtikel                                    ");
        System.out.println("  711 : insertStako     (test VO)                     ");
        System.out.println("                                                      ");
        System.out.println("  720 : bean update tabelle artikel auf bagis         ");
        System.out.println("  721 : bean start complete stako (incl update Adressen");
        System.out.println("  722 : bean start stako (excl update Adressen)       ");
        System.out.println("  723 : bean restart stako                            ");
        System.out.println("  724 : bean start stako (enter bisDatum)             ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      case 8:
        System.out.println("SAP Inbound Client (Manual Mode) - Divers             ");
        System.out.println("------------------------------------------------------");
        System.out.println("  800 : bean ctrl - stop sap inbound client           ");
        System.out.println("  880 : convert bagis text to sap format              ");
        System.out.println("  888 : playground                                    ");
        System.out.println("                                                      ");
        System.out.println("    0 : HAUPTMENU                  99 : EXIT          ");
        break;

      default:
        System.out.println("Falsche Eingabe, bitte geben Sie eine gültige Zahl ein.");
        break;


    } // end switch

    System.out.println();

    while (! valid)
    {
      try
      {
        buf = readLine("Eingabe = ");
        cmd = Integer.parseInt(buf);
        valid = true;
      }
      catch (Exception ex)
      {
        System.out.println("Falsche Eingabe, bitte geben Sie eine Zahl ein.");
      }
    }

    return cmd;

  } // end method


  private void menuExecute(int cmd) throws Exception
  {
    switch (cmd)
    {

      //
      // Systeminformationen
      //
      case 100:
      {
        printBuildInfo();
        printParams();
        printProperties("SAP Inbound Client - Properties",SapInbound.getProperties());
        printProperties("System - Properties",System.getProperties());
        break;
      }

      case 110:
      {
        String buf = readLine("Enter Input Parameter : REQUTEXT = ");
        fRFC.testStfcConnection(buf);
        break;
      }

      case 111:
      {
        fRFC.testRfcSystemInfo();
        break;
      }

      case 120:
      {
        SapDelegateOutbound d = SapDelegateOutbound.getInstance();
        System.out.println("send message: <Hello AppServer>");
        System.out.println(d.receivePing("Hello AppServer"));
        break;
      }

      //
      // Rechnungen
      //
      case 210:
      {
        int  id;
        SAPRechnungVO vo;
        id  = Integer.parseInt( readLine("Enter bfakt_id (4'000'000 - 19'999'999) = ") );
        vo = TestData.createRechnung(id);
        fRFC.testCreateBillingReq(vo);
        break;
      }

      case 211:
      {
        int  id, re, pos;
        SAPRechnungVO vo;
        id  = Integer.parseInt( readLine("Enter bfakt_id    (4'000'000 - 19'999'999) = ") );
        re  = Integer.parseInt( readLine("Enter Anzahl zusätzl. Rechnungsempfänger   = ") );
        pos = Integer.parseInt( readLine("Enter Anzahl Rechnungspositionen           = ") );
        vo  = TestData.createRechnung(id,re,pos);
        fRFC.testCreateBillingReq(vo);
        break;
      }

      case 212:
      {
        this.runLoadTestBatchCommit();
        break;
      }

      case 213:
      {
        this.runLoadTestSingleCommit();
        break;
      }

      case 214:
      {
        SAPRechnungAnnullationVO  vo = new SAPRechnungAnnullationVO();
        vo.bfakt_id    = Integer.parseInt(
                         readLine("Enter bfakt_id (4'000'000 - 19'999'999)   = ") );
        vo.absageGrund = readLine("Enter absageGrund (9=annul/Z1 A=frist/Z2) = ");
        fRFC.testRejectBillingReq(vo);
        break;
      }

      case 216:
      {
        SAPRechnungVO vo = new SAPRechnungVO();
        vo.bfakt_id = Integer.parseInt(
                      readLine("Enter bfakt_id (4'000'000 - 19'999'999)   = ") );
        fRFC.testDeleteBillingReq(vo);
        break;
      }

      case 220:
      {
        int                id;
        SAPRechnungVO      vo;
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        id  = Integer.parseInt( readLine("Enter bfakt_id (4'000'000 - 19'999'999) = ") );
        vo = TestData.createRechnung(id);
        sd.sendRechnung(vo);
        break;
      }

      case 221:
      {
        int                id, re, pos;
        SAPRechnungVO      vo;
        SapDelegateInbound sd  = SapDelegateInbound.getInstance();
        id  = Integer.parseInt( readLine("Enter bfakt_id    (4'000'000 - 19'999'999) = ") );
        re  = Integer.parseInt( readLine("Enter Anzahl zusätzl. Rechnungsempfänger   = ") );
        pos = Integer.parseInt( readLine("Enter Anzahl Rechnungspositionen           = ") );
        vo  = TestData.createRechnung(id,re,pos);
        sd.sendRechnung(vo);
        break;
      }

      case 222:
      {
        int                 id, re, pos, cnt, delay;
        SAPRechnungVO        vo;
        SapDelegateInbound sd    = SapDelegateInbound.getInstance();
        id    = Integer.parseInt( readLine("Start bfakt_id    (4'000'000 - 19'999'999) = ") );
        re    = Integer.parseInt( readLine("Enter Anzahl zusätzl. Rechnungsempfänger   = ") );
        pos   = Integer.parseInt( readLine("Enter Anzahl Rechnungspositionen           = ") );
        cnt   = Integer.parseInt( readLine("Number of rechnung vo's to send            = ") );
        delay = Integer.parseInt( readLine("Delay between Messages [ms]                = ") );
        for (int i=0; i<cnt ; i++ )
        {
          vo  = TestData.createRechnung(id + i,re,pos);
          System.out.println("send vo #" + i);
          sd.sendRechnung(vo);
          try
          {
            Thread.sleep(delay);
          }
          catch (InterruptedException ex)
          {
            // ignore
          }
        }
        break;
      }

      case 224:
      {
        SAPRechnungAnnullationVO  vo = new SAPRechnungAnnullationVO();
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        vo.bfakt_id    = Integer.parseInt(
                         readLine("Enter bfakt_id (4'000'000 - 19'999'999)   = ") );
        vo.absageGrund = readLine("Enter absageGrund (9=annul/Z1 A=frist/Z2) = ");
        sd.sendAnnulation(vo);
        break;
      }

      case 225:
      {
        int                       cnt  = 0;
        ArrayList                 list = new ArrayList();

        System.out.println("Send Annulation List :");
        cnt = Integer.parseInt( readLine("Enter number of List entries = ") );

        for (int i=0; i < cnt; i++)
        {
          SAPRechnungAnnullationVO  vo = new SAPRechnungAnnullationVO();

          System.out.println("Enter data for entry #" + (i + 1) + " : ");
          vo.bfakt_id    = Integer.parseInt(
                           readLine("  bfakt_id (4'000'000 - 19'999'999)   = ") );
          vo.absageGrund = readLine("  absageGrund (9=annul/Z1 A=frist/Z2) = ");

          list.add(vo);
        }

        SapDelegateInbound.getInstance().sendAnnulationList(list);
        break;
      }

      case 226:
      {
        int                id, re, pos;
        SAPRechnungVO      vo;
        SapDelegateInbound sd  = SapDelegateInbound.getInstance();
        id  = Integer.parseInt( readLine("Enter bfakt_id    (4'000'000 - 19'999'999) = ") );
        re  = Integer.parseInt( readLine("Enter Anzahl zusätzl. Rechnungsempfänger   = ") );
        pos = Integer.parseInt( readLine("Enter Anzahl Rechnungspositionen           = ") );
        vo  = TestData.createRechnung(id,re,pos);
        vo.hasChanged = SAPRechnungVO.BOOLEAN_TYPES[1];
        sd.sendRechnung(vo);
        break;
      }


      //
      // Jahresgebühren
      //
      case 310:
      {
        int         blockSize = 0;
        SAPJGBJobVO vo        = null;
        JGB         jgb       = null;

        // get params
        blockSize = Integer.parseInt
               ( readLine("Enter block size ses update     = ") );

        vo = getJgbJobParams();

        // init & start process
        jgb = new JGB(fRFC);
        jgb.setBlockSizeSesUpdate(blockSize);
        jgb.process(vo);

        if (jgb.isErrorOccured())
        {
          System.out.println(jgb.getErrorMessage());
        }
        else
        {
          System.out.println("process end successfully, " + jgb.getStatistic());
        }

        break;
      }

      case 311:
      {
        int jgbLaufId = 0;
        JGB jgb       = null;

        // get params
        jgbLaufId = Integer.parseInt
               ( readLine("Enter jgbLaufId = ") );

        // init & force finish
        jgb = new JGB(fRFC);
        jgb.forceFinish(jgbLaufId);

        if (jgb.isErrorOccured())
        {
          System.out.println(jgb.getErrorMessage());
        }
        else
        {
          System.out.println("process finish successfully done");
        }

        break;
      }

      case 320:
      {
        SAPJGBJobVO vo        = null;

        // get params
        vo = getJgbJobParams();

        // send message
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        sd.sendJGBJob(vo);
        break;
      }

      case 321:
      {
        int jgbLaufId = 0;

        // get params
        jgbLaufId = Integer.parseInt
               ( readLine("Enter jgbLaufId = ") );

        // send message
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        sd.sendCtrlJgbFinish(jgbLaufId);
        break;
      }

      case 330:
      {
        insertJGB(true);
        break;
      }

      case 331:
      {
        insertJGB(false);
        break;
      }

      //
      // Automatische Markenverlängerung (AMV)
      //
      case 340:
      {
        int     amvLaufId = Integer.parseInt( readLine("Enter amvLaufId                       = ") );
        String  amvLaufNr =                   readLine("Enter amvLaufNr (YYYYMMDDNN)          = ");
        int     bfakt_id  = Integer.parseInt( readLine("Enter bfakt_id (4'000'000-19'999'999) = ") );

        SAPRechnungVO vo = TestData.createAMV(amvLaufId,amvLaufNr,1,bfakt_id);
        fRFC.testCreateBillingReq(vo);
        break;
      }

      case 341:
      {
        int     amvLaufId = Integer.parseInt( readLine("Enter amvLaufId                       = ") );
        String  amvLaufNr =                   readLine("Enter amvLaufNr (YYYYMMDDNN)          = ");
        int     bfakt_id  = Integer.parseInt( readLine("Enter bfakt_id (4'000'000-19'999'999) = ") );

        SAPRechnungVO vo = TestData.createAMV(amvLaufId,amvLaufNr,2,bfakt_id);
        fRFC.testChangeBillingReq(vo);
        break;
      }

      case 342:
      {
        int     amvLaufId = Integer.parseInt( readLine("Enter amvLaufId                             = ") );
        String  amvLaufNr =                   readLine("Enter amvLaufNr (YYYYMMDDNN)                = ");
        int     amvReCd   = Integer.parseInt( readLine("Enter Erinnerungstyp (1 oder 2)             = ") );
        int     anzahl    = Integer.parseInt( readLine("Enter Anzahl Rechnungen (1..n)              = ") );
        int     bfakt_id  = Integer.parseInt( readLine("Enter start bfakt_id (4'000'000-19'999'999) = ") );
        int     re        = Integer.parseInt( readLine("Enter Anzahl zusätzl. Rechnungsempfänger    = ") );
        int     pos       = Integer.parseInt( readLine("Enter Anzahl Rechnungspositionen            = ") );

        for (int i=0; i<anzahl; i++)
        {
          SAPRechnungVO vo = TestData.createAMV ( amvLaufId, amvLaufNr, amvReCd,
                                                  bfakt_id + i, re, pos );
          if (amvReCd == 1)
          {
            fRFC.testCreateBillingReq(vo);
          }
          else if (amvReCd == 2)
          {
            fRFC.testChangeBillingReq(vo);
          }
        }
        break;
      }

      case 343:
      {
        String        amvLaufNr = readLine("Enter amvLaufNr (YYYYMMDDNN) = ");
        StringBuffer  retMsg    = new StringBuffer();

        fRFC.testCommitTmExtension(amvLaufNr,retMsg);
        break;
      }


      case 350:
      {
        int     amvLaufId = Integer.parseInt( readLine("Enter amvLaufId (1 - n)               = ") );
        String  amvLaufNr =                   readLine("Enter amvLaufNr (YYYYMMDDNN)          = ");
        int     bfakt_id  = Integer.parseInt( readLine("Enter bfakt_id (4'000'000-19'999'999) = ") );

        SAPRechnungVO vo = TestData.createAMV(amvLaufId,amvLaufNr,1,bfakt_id);
        SapDelegateInbound.getInstance().sendRechnung(vo);
        break;
      }

      case 351:
      {
        int     amvLaufId = Integer.parseInt( readLine("Enter amvLaufId (1 - n)               = ") );
        String  amvLaufNr =                   readLine("Enter amvLaufNr (YYYYMMDDNN)          = ");
        int     bfakt_id  = Integer.parseInt( readLine("Enter bfakt_id (4'000'000-19'999'999) = ") );

        SAPRechnungVO vo = TestData.createAMV(amvLaufId,amvLaufNr,2,bfakt_id);
        SapDelegateInbound.getInstance().sendRechnung(vo);
        break;
      }

      case 352:
      {
        int     amvLaufId = Integer.parseInt( readLine("Enter amvLaufId                             = ") );
        String  amvLaufNr =                   readLine("Enter amvLaufNr (YYYYMMDDNN)                = ");
        int     amvReCd   = Integer.parseInt( readLine("Enter Erinnerungstyp (1 oder 2)             = ") );
        int     anzahl    = Integer.parseInt( readLine("Enter Anzahl Rechnungen (1..n)              = ") );
        int     bfakt_id  = Integer.parseInt( readLine("Enter start bfakt_id (4'000'000-19'999'999) = ") );
        int     re        = Integer.parseInt( readLine("Enter Anzahl zusätzl. Rechnungsempfänger    = ") );
        int     pos       = Integer.parseInt( readLine("Enter Anzahl Rechnungspositionen            = ") );

        for (int i=0; i<anzahl; i++)
        {
          SAPRechnungVO vo = TestData.createAMV ( amvLaufId, amvLaufNr, amvReCd,
                                                  bfakt_id + i, re, pos );
          SapDelegateInbound.getInstance().sendRechnung(vo);
        }
        break;
      }

      case 353:
      {
        AmvLaufVO vo = new AmvLaufVO();
        vo.amvLaufId = Integer.parseInt( readLine("Enter amvLaufId (1 - n)   = ") );
        vo.laufNr    =                   readLine("Enter LaufNr (YYYYMMDDNN) = ");

        SapDelegateInbound.getInstance().sendCtrlAmvFinish(vo);
        break;
      }


      case 360:
      {
        String  laufNr  =                   readLine("Enter LaufNr (YYYYMMDDNN)         = ");
        int     e1Count = Integer.parseInt( readLine("Enter Erinnerung 1 Anzahl (0 - n) = ") );
        int     e1Start = Integer.parseInt( readLine("Enter Erinnerung 1 Start bfakt_id = ") );
        int     e2Count = Integer.parseInt( readLine("Enter Erinnerung 2 Anzahl (0 - n) = ") );
        int     e2Start = Integer.parseInt( readLine("Enter Erinnerung 2 Start bfakt_id = ") );

        insertAMV(laufNr,e1Count,e1Start,e2Count,e2Start);
        break;
      }


      //
      // Adressen
      //
      case 410:
      {
        int         id, flag;
        VersadrVO   vo;
        id   = Integer.parseInt( readLine("Enter adr_nr (500'000 - 9'999'999) = "));
        flag = Integer.parseInt( readLine("Enter modus   (1=create, 0=update) = "));
        vo = TestData.createAdresse(id);
        vo.setIsNew(flag == 1 ? true : false);

        // vo.telefon          = readLine("Enter telefon         = ");
        // vo.fax              = readLine("Enter fax             = ");
        // vo.emailNameDomain  = readLine("Enter emailNameDomain = ");

        fRFC.testCustomerCreate(vo);
        break;
      }

      case 411:
      {
        int             id, flag;
        SAPAdrStatusVO  vo;
        id   = Integer.parseInt( readLine("Enter adr_nr    (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter Löschvormerkung (1=set/0=unset) = " ));
        vo   = new SAPAdrStatusVO(id,true,(flag == 1 ? "X" : " "));
        fRFC.testCustomerDelete(vo);
        break;
      }

      case 412:
      {
        int             id, flag;
        SAPAdrStatusVO  vo;
        id   = Integer.parseInt( readLine("Enter adr_nr   (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter Buchungssperre (1=set/0=unset) = " ));
        vo   = new SAPAdrStatusVO(id,false,(flag == 1 ? "X" : " "));
        fRFC.testCustomerBlock(vo);
        break;
      }

      case 413:
      {
        int adrNr;
        adrNr = Integer.parseInt( readLine("Enter adr_nr (500'000 - 9'999'999) = ") );
        fRFC.testCustomerExistence(adrNr);
        break;
      }

      case 414:
      {
        String pfadIn = "";
        String pfadOut = "";
        pfadIn = readLine("Enter Input-File Path (Default: c:\\temp\\adressen.txt) = ");
        pfadOut = readLine("Enter Outuput-File Path (Default: c:\\temp\\fehlende_adressen.txt) = ");
        if (pfadIn.equals(""))
        {
          pfadIn = "c:\\temp\\adressen.txt";
        }
        if (pfadOut.equals(""))
        {
          pfadOut = "c:\\temp\\fehlende_adressen.txt";
        }
        fRFC.testCustomerExistenceFile(pfadIn, pfadOut);
        break;
      }


      case 415:
      {
        ArrayList   al    = new ArrayList();
        int         adrNr = 0;

        System.out.println("Geben Sie die gewünschten Adress-Nummern ein (Abbruch mit Eingabe=0):");
        do {
          adrNr = Integer.parseInt( readLine("Enter adr_nr (500'000 - 9'999'999) = ") );
          if (adrNr > 0)
            al.add( new VersadrCheckVO(adrNr) );
        } while(adrNr > 0);

        fRFC.testCustomerCheck(al);
        break;
      }


      case 420:
      {
        SapDelegateAdresse  adr = SapDelegateAdresse.getInstance();
        int                 id, flag;
        VersadrVO           vo;

        id   = Integer.parseInt( readLine("Enter adr_nr (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter mode    (1=create, 0=update) = " ));
        vo   = TestData.createAdresse(id);

        try
        {
          if (flag == 1)
          {
            adr.create(vo);
          }
          else
          {
            adr.update(vo);
          }
          System.out.println("send successfully executed");
        }
        catch (SapCallException ex)
        {
          System.out.println("send failed, error message is:");
          System.out.println(ex.toString());
        }
        break;
      }

      case 421:
      {
        SapDelegateAdresse  adr = SapDelegateAdresse.getInstance();
        int                 id, flag;

        id   = Integer.parseInt( readLine("Enter adr_nr    (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter Löschvormerkung (1=set/0=unset) = " ));

        try
        {
          if (flag == 1)
          {
            adr.delete(id);
          }
          else
          {
            adr.undelete(id);
          }
          System.out.println("send successfully executed");
        }
        catch (SapCallException ex)
        {
          System.out.println("send failed, error message is:");
          System.out.println(ex.toString());
        }
        break;
      }

      case 422:
      {
        SapDelegateAdresse  adr = SapDelegateAdresse.getInstance();
        int                 id, flag;

        id   = Integer.parseInt( readLine("Enter adr_nr   (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter Buchungssperre (1=set/0=unset) = " ));

        try
        {
          if (flag == 1)
          {
            adr.block(id);
          }
          else
          {
            adr.unblock(id);
          }
          System.out.println("send successfully executed");
        }
        catch (SapCallException ex)
        {
          System.out.println("send failed, error message is:");
          System.out.println(ex.toString());
        }
        break;
      }

      case 430:
      {
        StopWatch   sw    = new StopWatch();
        SapAdresse  adr   = new SapAdresse();
        int         id, flag;
        VersadrVO   vo;

        id   = Integer.parseInt( readLine("Enter adr_nr (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter mode       (1=neu, 0=update) = " ));
        vo   = TestData.createAdresse(id);

        System.out.println("start time: " + sw.start());
        try
        {
          if (flag == 1)
          {
            adr.create(vo);
          }
          else
          {
            adr.update(vo);
          }
          System.out.println("request successfully executed");
        }
        catch (SapCallException ex)
        {
          System.out.println("request failed, error message is:");
          System.out.println(ex.toString());
        }

        System.out.println("end time  : " + sw.stop());
        System.out.println("time used : " + sw.toStringDeltaTimeShort());
        break;
      }

      case 431:
      {
        StopWatch   sw  = new StopWatch();
        SapAdresse  adr = new SapAdresse();
        int         id, flag;

        id   = Integer.parseInt( readLine("Enter adr_nr    (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter Löschvormerkung (1=set/0=unset) = " ));

        System.out.println("start time: " + sw.start());
        try
        {
          if (flag == 1)
          {
            adr.delete(id);
          }
          else
          {
            adr.undelete(id);
          }
          System.out.println("request successfully executed");
        }
        catch (SapCallException ex)
        {
          System.out.println("request failed, error message is:");
          System.out.println(ex.toString());
        }
        System.out.println("end time  : " + sw.stop());
        System.out.println("time used : " + sw.toStringDeltaTimeShort());
        break;
      }

      case 432:
      {
        StopWatch   sw  = new StopWatch();
        SapAdresse  adr = new SapAdresse();
        int         id, flag;

        id   = Integer.parseInt( readLine("Enter adr_nr   (500'000 - 9'999'999) = ") );
        flag = Integer.parseInt( readLine("Enter Buchungssperre (1=set/0=unset) = " ));

        System.out.println("start time: " + sw.start());
        try
        {
          if (flag == 1)
          {
            adr.block(id);
          }
          else
          {
            adr.unblock(id);
          }
          System.out.println("request successfully executed");
        }
        catch (SapCallException ex)
        {
          System.out.println("request failed, error message is:");
          System.out.println(ex.toString());
        }
        System.out.println("end time  : " + sw.stop());
        System.out.println("time used : " + sw.toStringDeltaTimeShort());
        break;
      }

      case 435:
      {
        StopWatch   sw = new StopWatch();
        ArrayList   al = new ArrayList();
        int         adrNrStart, adrNrCnt;

        adrNrStart = Integer.parseInt( readLine("Enter adr_nr start (500'000 - 9'999'999) = ") );
        adrNrCnt   = Integer.parseInt( readLine("Enter adr_nr count (maximal 1000)        = ") );

        for (int i=0; i<adrNrCnt; i++)
        {
          al.add( new VersadrCheckVO(adrNrStart + i) );
        }

        System.out.println("start time: " + sw.start());
        try
        {
          al = new SapAdresse().check(al);
          System.out.println("request successfully executed:");
          for (Iterator iter = al.iterator(); iter.hasNext(); )
          {
            VersadrCheckVO item = (VersadrCheckVO)iter.next();
            System.out.println(item.toString());
          }
        }
        catch (SapCallException ex)
        {
          System.out.println("request failed, error message is:");
          System.out.println(ex.toString());
        }
        System.out.println("end time  : " + sw.stop());
        System.out.println("time used : " + sw.toStringDeltaTimeShort());
        break;
      }

      case 440:
      {
        SapAdrLoadTestSuite suite = new SapAdrLoadTestSuite();
        int threads = Integer.parseInt(readLine("Enter number of threads            = "));
        int repeats = Integer.parseInt(readLine("Enter number of repetitions        = "));
        int delay   = Integer.parseInt(readLine("Enter delay in [ms]                = "));
        int startAdr= Integer.parseInt(readLine("Enter start adr_nr ( >= 9'000'000) = "));
        if(startAdr < 9000000)
        {
          System.out.println("adr_nr muss >= 9000000 sein !");
        }
        else
        {
          suite.testRFMThreads(threads, delay, repeats, startAdr);
        }
        break;
      }

      //
      // Deponentenkonti
      //
      case 510:
      {
        int ktoNr = Integer.parseInt( readLine("Enter adr_nr (ktoNr)  = ") );
        fRFC.testCustomerBalance(ktoNr);
        break;
      }

      case 530:
      {
        try
        {
          SapDepKto kto   = new SapDepKto();
          int       ktoNr = Integer.parseInt( readLine("Enter adr_nr (ktoNr) =   ") );
          double    saldo = kto.getSaldo(ktoNr);

          System.out.println("Saldo = " + saldo);
        }
        catch (SapCallException ex)
        {
          System.out.println("request failed, error message is:");
          System.out.println(ex.toString());
        }
        break;
      }

      //
      //  Artikel - Schnittstelle
      //
      case 610:
      {
        String art_nr = readLine("Enter art_nr (Artikelnummer)  = ");
        fRFC.testExistenceCheck(art_nr);
        break;
      }

      case 630:
      {
        try
        {
          SapArtikel art  = new SapArtikel();
          String art_nr = readLine("Enter art_nr (Artikelnummer) = ");
          boolean exists = art.existenceCheck(art_nr);

          System.out.println("Artikel existiert = " + exists);
        }
        catch (SapCallException ex)
        {
          System.out.println("request failed, error message is:");
          System.out.println(ex.toString());
        }

        break;
      }

      //
      // Controlling - Schnittstelle
      //
      case 710:
      {
        ArrayList artikel = fRFC.testGetArtikel();
        System.out.println("### Artikel: ");

        for (int i=0; i < artikel.size(); i++)
        {
          System.out.println(artikel.get(i).toString());
        }

        break;
      }

      case 711:
      {
        int retCode = fRFC.testInsertStako(TestData.createStako());
        if (retCode == fRFC.RET_CODE_SUCCESS)
        {
          System.out.println("#######");
          System.out.println("SUCCESS");
          System.out.println("#######");
        }
        else
        {
          System.out.println("#######");
          System.out.println("FAILURE");
          System.out.println("#######");
        }
        break;
      }

      case 720:
      {
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        SAPStakoJobVO sapStakoJobVO = new SAPStakoJobVO();
        sapStakoJobVO.modus = sapStakoJobVO.STAKO_MODUS_UPDATE;
        sd.sendStakoJob(sapStakoJobVO);
        break;
      }
      case 721:
      {
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        SAPStakoJobVO sapStakoJobVO = new SAPStakoJobVO();
        sapStakoJobVO.modus = sapStakoJobVO.STAKO_MODUS_START;
        sd.sendStakoJob(sapStakoJobVO);
        break;
      }
      case 722:
      {
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        SAPStakoJobVO sapStakoJobVO = new SAPStakoJobVO();
        sapStakoJobVO.modus = sapStakoJobVO.STAKO_MODUS_NOUPDATE;
        sd.sendStakoJob(sapStakoJobVO);
        break;
      }
      case 723:
      {
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        SAPStakoJobVO sapStakoJobVO = new SAPStakoJobVO();
        sapStakoJobVO.modus = SAPStakoJobVO.STAKO_MODUS_RESTART;
        sd.sendStakoJob(sapStakoJobVO);
        break;
      }
      case 724:
      {
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        SAPStakoJobVO sapStakoJobVO = new SAPStakoJobVO();
        sapStakoJobVO.modus = SAPStakoJobVO.STAKO_MODUS_NOUPDATE;
        String datum = readLine("Enter bisDatum (format: DD.MM.YYYY) = ");
        StringTokenizer st = new StringTokenizer(datum, ".");
        int day = Integer.parseInt(st.nextToken());
        int month = Integer.parseInt(st.nextToken()) - 1;
        int year = Integer.parseInt(st.nextToken());
        GregorianCalendar cal = new GregorianCalendar(year, month, day);
        sapStakoJobVO.bisDatum = new java.sql.Date(cal.getTime().getTime());
        System.out.println("sapStakoJobVO = \n" + sapStakoJobVO.toString());
        sd.sendStakoJob(sapStakoJobVO);
        break;
      }

      //
      // Divers
      //
      case 800:
      {
        SapDelegateInbound sd = SapDelegateInbound.getInstance();
        sd.sendCtrlStop();
        break;
      }

      case 880:
      {
        //String buf = readLine("Enter BAGIS Text = ");
        // Originaltext String buf = "Ausserdem erinnern wir Sie daran, dass unter bestimmten Voraussetzungen (Art. 62 und 62a PatV), insbesondere wenn für die gleiche Erfindung eine europäische Patentanmeldung hängig ist, die Aussetzung der Prüfung beantragt werden kann. Falls die Gebühr nicht durch Belastung Ihres Kontokorrents beim Institut beglichen werden kann, bitten wir Sie, den Einzahlungsschein in der Anlage zu verwenden.";

        String buf = readLine("Enter BAGIS Text = ");
        SapText st = new SapText(buf);

        System.out.println("");
        System.out.println("Text Input:");
        System.out.println(buf);

        System.out.println("");
        System.out.println("Full Text Output:");
        System.out.println("\t  123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890");
        System.out.println(st.toStringText());

        System.out.println("");
        System.out.println("Split Text Output:");
        System.out.println("\t  123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890");
        System.out.println(st.toStringSplitText());
        break;
      }

      case 888:
      {
        this.playground();
        break;
      }

    default:
    {
        System.out.println("Funktion nicht verfügbar!");
        break;
    }

    } // end switch

  } // end menuExecute

  //---------------------------------------------------------------------------
  // private methods - load test
  //---------------------------------------------------------------------------

  private void runLoadTestBatchCommit()
  {
    int           cnt, id, re, pos;               // input params
    ArrayList     list   = new ArrayList();       // list of bills
    String        client = SapInbound.getProperties().getProperty("jco.clientpool.client");
    StopWatch     sw     = new StopWatch();
    SAPRechnungVO vo     = null;

    //
    // init
    //
    System.out.println("");
    System.out.println("### SAP Test RFC - Z_BAPI_ZVL_CREATE_BILLING_REQ - load test batch commit");

    //
    // check client
    //
    if (! client.equals("510"))
    {
      System.out.println("Fehler:");
      System.out.println("  Dieser Test läuft nur mit dem Sanbox Client 510.");
      System.out.println("  Bitte überprüfen sie Ihre Property Datei.");
      return;
    }

    //
    // get input params & check max. count
    //
    cnt  = Integer.parseInt( readLine("  Enter Anzahl Rechnungen (max. 1000)           = ") );
    id   = Integer.parseInt( readLine("  Enter Start bfakt_id (4'000'000 - 19'999'999) = ") );
    re   = Integer.parseInt( readLine("  Enter Anzahl zusätzl. Rechnungsempfänger      = ") );
    pos  = Integer.parseInt( readLine("  Enter Anzahl Rechnungspositionen              = ") );

    if (cnt > 1000)
    {
      System.out.println("Fehler:");
      System.out.println("  Max. Anzahl Rechnungen überschritten !");
      System.out.println("  Bitte überprüfen sie Ihre Eingabewerte.");
      return;
    }

    //
    // start processing
    //
    System.out.println("### start processing: " + sw.start());
    System.out.println("  " + (new Date()) + " : create records : " + id + " to " + (id + cnt-1) );
    for (int i=0; i < cnt; i++)
    {
      vo = TestData.createRechnung(id+i,re,pos);
      vo.druck_art = vo.DRUCK_ART_TYPES[1];           // batch print (important)
      list.add(vo);
    }

    System.out.println("  " + (new Date()) + " : start loading records");
    fRFC.testCreateBillingReqLoadBatchCommit(list);
    System.out.println("### end processing: " + sw.stop());

    //
    // statistics
    //
    System.out.println("");
    System.out.println("### statistics - load test batch commit: ");
    System.out.println("  number of bills       = " + cnt);
    System.out.println("  bfakt_id range        = " + id + " to " + (id + cnt-1) );
    System.out.println("  additional partners   = " + re);
    System.out.println("  number of positions   = " + pos);
    System.out.println("");
    System.out.println("  time start processing = " + sw.toStringStartTime());
    System.out.println("  time end processing   = " + sw.toStringStopTime());
    System.out.println("  time elapsed          = " + sw.toStringDeltaTimeLong());
  }


  private void runLoadTestSingleCommit()
  {
    int           cnt, id, re, pos;               // input params
    int           progress = 0;                   // progress count
    String        client   = SapInbound.getProperties().getProperty("jco.clientpool.client");
    StopWatch     sw       = new StopWatch();
    SAPRechnungVO vo       = null;

    //
    // init
    //
    System.out.println("");
    System.out.println("### SAP Test RFC - Z_BAPI_ZVL_CREATE_BILLING_REQ - load test single commits");

    //
    // check client
    //
    if (! client.equals("510"))
    {
      System.out.println("Fehler:");
      System.out.println("  Dieser Test läuft nur mit dem Sanbox Client 510.");
      System.out.println("  Bitte überprüfen sie Ihre Property Datei.");
      return;
    }

    //
    // get input params & check max. count
    //
    cnt  = Integer.parseInt( readLine("  Enter Anzahl Rechnungen (max. 1000)           = ") );
    id   = Integer.parseInt( readLine("  Enter Start bfakt_id (4'000'000 - 19'999'999) = ") );
    re   = Integer.parseInt( readLine("  Enter Anzahl zusätzl. Rechnungsempfänger      = ") );
    pos  = Integer.parseInt( readLine("  Enter Anzahl Rechnungspositionen              = ") );

    if (cnt > 1000)
    {
      System.out.println("Fehler:");
      System.out.println("  Max. Anzahl Rechnungen überschritten !");
      System.out.println("  Bitte überprüfen sie Ihre Eingabewerte.");
      return;
    }

    //
    // start processing
    //
    System.out.println("### start processing: " + sw.start());
    System.out.println("  " + (new Date()) + " : loading records : " + id + " to " + (id + cnt-1) );
    for (int i=0; i < cnt; i++)
    {
      vo = TestData.createRechnung(id+i,re,pos);
      vo.druck_art = vo.DRUCK_ART_TYPES[1];           // batch print (important)
      fRFC.testCreateBillingReqLoadSingleCommit(vo);

      progress++;
      if (progress == 100)
      {
        System.out.println("  " + (new Date()) + " : total loaded records : " + i);
        progress = 0;
      }
    }
    System.out.println("### end processing: " + sw.stop());

    //
    // statistics
    //
    System.out.println("");
    System.out.println("### statistics - load test single commits:");
    System.out.println("  number of bills       = " + cnt);
    System.out.println("  bfakt_id range        = " + id + " to " + (id + cnt-1) );
    System.out.println("  additional partners   = " + re);
    System.out.println("  number of positions   = " + pos);
    System.out.println("");
    System.out.println("  time start processing = " + sw.toStringStartTime());
    System.out.println("  time end processing   = " + sw.toStringStopTime());
    System.out.println("  time elapsed          = " + sw.toStringDeltaTimeLong());
  }

  //---------------------------------------------------------------------------
  // private methods - jgb
  //---------------------------------------------------------------------------

  private SAPJGBJobVO getJgbJobParams()
  {
    SAPJGBJobVO vo = new SAPJGBJobVO();
    int         lj = 0;

    vo.jgbLaufId = Integer.parseInt
           ( readLine("Enter SAPJGBJobVO jgbLaufId     = ") );
    vo.bfaktIdFrom = Integer.parseInt
           ( readLine("Enter SAPJGBJobVO bfakt_id from = ") );
    vo.bfaktIdTo = Integer.parseInt
           ( readLine("Enter SAPJGBJobVO bfakt_id to   = ") );

    lj = Integer.parseInt
           ( readLine("Enter last job (0=false/1=true) = ") );

    if (lj == 1)
      vo.lastJob = true;
    else
      vo.lastJob = false;

    return vo;
  }

  private void insertJGB(boolean defaultValues) throws Exception
  {
    String          laufNr          = "2003011500";   // default values
    int             startBfaktId    = 4000000;
    int             anzRechnungen   = 100;
    int             anzPositionen   = 2;
    int             zusatzRechEmpf  = 5;

    int             laufId          = 0;
    SAPRechnungVO   vo              = null;


    // get test data params
    if (! defaultValues)
    {
      System.out.println("Enter jgb test data parameter (use approx. 2 min / 1000 records):");
      laufNr         =                   readLine("  Lauf Nummer              (jjjjmmttnn) = ");
      startBfaktId   = Integer.parseInt( readLine("  Start bfakt_id (4'000'000-19'999'999) = ") );
      anzRechnungen  = Integer.parseInt( readLine("  Anzahl Rechnungen                     = ") );
      anzPositionen  = Integer.parseInt( readLine("  Anzahl Rechnungspositionen (1 oder 2) = ") );
      zusatzRechEmpf = Integer.parseInt( readLine("  Zusätzl. Rechnungsempfänger   (0..20) = ") );
    }
    else
    {
      System.out.println("Use the following default jgb test data (use approx. 2 min / 1000 records):");
      System.out.println("  Lauf Nummer              (jjjjmmttnn) = " + laufNr);
      System.out.println("  Start bfakt_id (4'000'000-19'999'999) = " + startBfaktId);
      System.out.println("  Anzahl Rechnungen                     = " + anzRechnungen);
      System.out.println("  Anzahl Rechnungspositionen (1 oder 2) = " + anzPositionen);
      System.out.println("  Zusätzl. Rechnungsempfänger   (0..20) = " + zusatzRechEmpf);
    }


    // insert lauf
    System.out.println("Insert LAUF: " + laufNr);
    laufId = insertJGBLauf(laufNr,anzRechnungen,anzRechnungen * (1 + zusatzRechEmpf));


    // insert rechnungen
    for (int i=0; i<anzRechnungen; i++ )
    {
      vo  = TestData.createJGB(laufId,laufNr,startBfaktId+i,zusatzRechEmpf,anzPositionen);
      System.out.println("  bfakt_id: " + vo.bfakt_id);
      insertJGBRechnung(vo);
    }

    // all done fine
    System.out.println("Insert jgb test data with lauf id = " + laufId + " done !");

  } // end method


  // return jgbLaufId
  private int insertJGBLauf ( String laufNr,
                              int anzahlBfakturGelesen,
                              int anzahlBfaktjgbGeneriert ) throws Exception
  {
    JGBLaufDAO  dao = new JGBLaufDAO();
    JGBLaufVO   vo  = new JGBLaufVO();


    // init & insert lauf
    vo.laufNr                   = laufNr;
    vo.sesStat                  = 3;
    vo.anzahlBfakturGelesen     = anzahlBfakturGelesen;
    vo.anzahlBfaktjgbGeneriert  = anzahlBfaktjgbGeneriert;
    vo.sapStat                  = JGBLaufVO.SAP_STAT_NOT_STARTED;
    vo.anzahlBfakturAnSap       = 0;
    vo.anzahlBfaktjgbAnSap      = 0;
    vo.minBfakturToSap          = 0;
    vo.countBfakturToSap        = 0;

    dao.insert(vo);

    // select lauf
    vo = dao.select(laufNr);

    // return jgb_lauf_id
    return vo.jgbLaufId;
  }


  private void insertJGBRechnung(SAPRechnungVO vo) throws Exception
  {
    BfakturSESDAO  bfakturSESDAO  = new BfakturSESDAO();
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    BfaktJGBSESDAO bfaktJGBSESDAO = new BfaktJGBSESDAO();


    // insert rechnung
    bfakturSESDAO.insert(vo);

    // insert position
    for (Iterator i = vo.posList.iterator(); i.hasNext(); )
    {
      SAPRechnungsPositionVO item = (SAPRechnungsPositionVO)i.next();
      bfaktposSESDAO.insert(item);
    }

    // insert jgb params
    bfaktJGBSESDAO.insert(vo.jgbHauptRechEmpfParam);

    for (Iterator i = vo.jgbZusatzRechEmpfParam.iterator(); i.hasNext(); )
    {
      SAPJGBParamVO item = (SAPJGBParamVO)i.next();
      bfaktJGBSESDAO.insert(item);
    }

  } // end method

  //---------------------------------------------------------------------------
  // private methods - amv
  //---------------------------------------------------------------------------

  private void insertAMV ( String laufNr,
                           int e1Count, int e1Start,
                           int e2Count, int e2Start ) throws Exception
  {
    AmvLaufVO       laufVO  = new AmvLaufVO(laufNr);
    AmvLaufDAO      laufDAO = new AmvLaufDAO();
    AmvRechnungDAO  reDAO   = new AmvRechnungDAO();


    // create lauf
    laufDAO.insert(laufVO);
    laufVO = laufDAO.select(laufNr);

    // create rechnung e1
    for (int i=0; i < e1Count; i++)
    {
      AmvRechnungVO vo = new AmvRechnungVO();

      vo.amvLaufId  = laufVO.amvLaufId;
      vo.rechnungCd = AmvRechnungVO.RE_CD_ERINNERUNG_1;
      vo.bfaktId    = e1Start + i;

      reDAO.insert(vo);
    }

    // create rechnung e2
    for (int i=0; i < e2Count; i++)
    {
      AmvRechnungVO vo = new AmvRechnungVO();

      vo.amvLaufId  = laufVO.amvLaufId;
      vo.rechnungCd = AmvRechnungVO.RE_CD_ERINNERUNG_2;
      vo.bfaktId    = e2Start + i;

      reDAO.insert(vo);
    }

    // display summary
    System.out.println("amv testdata successfully generated:");
    System.out.println("  Lauf Id / Nr = " + laufVO.amvLaufId + " / " + laufVO.laufNr);
    if (e1Count > 0)
    {
      System.out.println("  Erinnerung 1 = " + e1Start + " ... " + (e1Start + e1Count - 1));
    }
    else
    {
        System.out.println("  Erinnerung 1 = none");
    }

    if (e2Count > 0)
    {
      System.out.println("  Erinnerung 2 = " + e2Start + " ... " + (e2Start + e1Count - 1));
    }
    else
    {
        System.out.println("  Erinnerung 2 = none");
    }
  }

  //---------------------------------------------------------------------------
  // private methods - misc
  //---------------------------------------------------------------------------

  private void playground() throws Exception
  {
    System.out.println("not used at the moment ...");
  } // end method


  private void printBuildInfo()
  {
    System.out.println();
    System.out.println("SAP Inbound Client - BuildInfo");
    System.out.println("   CVS Tag  : " + BuildInfo.CVS_TAG  );
    System.out.println("   Build Id : " + BuildInfo.BUILD_ID );
    System.out.println("   Default Provider Url used, when java.naming.provider.url is set to <default>:" );
    System.out.println("   Provider Url : " + ch.ige.util.BuildInfo.PROVIDER_URL );
  } // end method

  private void printParams()
  {
    System.out.println();
    System.out.println("SAP Inbound Client - Params");
    System.out.println("   Program ID          : " + SapInbound.getProgramId()  );
    System.out.println("   Program Action      : " + SapInbound.getProgramAction() );
    System.out.println("   Program Environment : " + SapInbound.getProgramEnv() );
    System.out.println("   Properties File     : " + SapInbound.getPropertiesFile() );
  } // end method

  private void printProperties(String title, Properties prop)
  {
    Enumeration eNames = prop.propertyNames();
    TreeSet     set  = new TreeSet();
    String      key;

    System.out.println();
    System.out.println(title);
    while ( eNames.hasMoreElements() )
    {
      key = eNames.nextElement().toString();
      set.add(key);
    }

    for (Iterator i = set.iterator(); i.hasNext(); )
    {
      key = (String) i.next();
      System.out.println("  " + Convert.toSapStr(key,40) + ": <" + prop.getProperty(key) + ">");
    }

  } // end method


  private String readLine(String prompt)
  {
    try
    {
      BufferedReader stdin  = new BufferedReader(new InputStreamReader(System.in));

      System.out.print (prompt);
      return(stdin.readLine());
    }
    catch (Exception ex)
    {
      return ("");
    }
  } // end method

} // end class

