package ch.ige.ses.client.sapinbound;

import java.util.*;
import java.lang.*;
import com.sap.mw.jco.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.vo.bagis.*;

/**
 * SAP Remote Function Module Z_BAPI_STAKO_INSERT
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum
 * @author Patrick Bolzli
 * @version $Id: RFM_Z_BAPI_STAKO_INSERT.java,v 1.7 2006/12/11 12:43:32 iten Exp $
 */

class RFM_Z_BAPI_STAKO_INSERT extends RFM
{

  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(RFM_Z_BAPI_STAKO_INSERT.class);
  private static final String IS_TESTRUN    = " "; // Testlauf
  private static final String IS_COMMITED   = "X"; // COMMIT
  private boolean fImportParamIsSet  = false;     // set by setImportParam

  //
  // fields
  //
  public  String  fNextStep;      // next step in process (trace info)

  //
  // constructor
  //
  public RFM_Z_BAPI_STAKO_INSERT (IRepository repository)
  {
    super("Z_BAPI_STAKO_INSERT",repository);
  }


  //
  // public methods
  //

  /**
   * Schickt STAKO - Daten an SAP
   * @param stako Value Object mit StakoVO's
   */
  public void addStako(StakoVO stako)
  {
    try
    {
      if (! this.fImportParamIsSet)                 // set only once per call
      {
        this.setImportParam();
      }
       this.setItem(stako);
    }

    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }


  /**
   * Gibt die SAP - Parameter als String zurück
   * @return SAP - Parameter als String
   */
  public String toStringStako()
  {
    JCO.ParameterList in  = this.fJCoFunc.getImportParameterList();
    StringBuffer      buf = new StringBuffer();

    // import params
    buf.append(in.toString());

    // table ITEM
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("ITEM");
    buf.append(tbl.toString());

    return buf.toString();
  }

  //
  // private methods
  //

  private void setImportParam()
  {
    // TESTRUN
    fNextStep = "setImportParam - TESTRUN";
    JCO.Field testrun = fJCoFunc.getImportParameterList().getField("TESTRUN");
    testrun.setValue(this.IS_TESTRUN);

    // COMMIT
    fNextStep = "setImportParam - COMMIT";
    JCO.Field commit = fJCoFunc.getImportParameterList().getField("COMMIT");
    commit.setValue(this.IS_COMMITED);

    this.fImportParamIsSet = true;
  }

  /**
   * Setzt die Werte der Tabelle ITEM
   * @param stako ArrayList mit StakoVO's
   */
  private void setItem(StakoVO stako)
  {
    JCO.Table tbl = fJCoFunc.getTableParameterList().getTable("ITEM");

    fNextStep = "setImportParam - appendRow()";
    tbl.appendRow();  // neue Row in SAP - Tabelle

    fNextStep = "setItem - kontrollId";
    tbl.setValue(Convert.toSapKontrollId(stako.kontrollId), "KONTROLL_ID");

    fNextStep = "setItem - bfaktId";
    tbl.setValue(Convert.toSapDocNumber(stako.bfaktId), "BFAKT_ID");

    fNextStep = "setItem - bfaktPosNr";
    tbl.setValue(Convert.toSapItmNumber(stako.bfaktPosNr), "BFAKT_POSNR");

    fNextStep = "setItem - employeeId";
    tbl.setValue(stako.employeeId , "EMPLOYEE_ID");

    fNextStep = "setItem - schutitNr";
    tbl.setValue(stako.schutitNr , "SCHUTIT_NR");

    fNextStep = "setItem - artNr";
    tbl.setValue(Convert.toSapMaterial(stako.artNr) , "ART_NR");

    fNextStep = "setItem - datum";
    tbl.setValue(stako.datum, "DATUM");

    fNextStep = "setItem - anzahl";
    tbl.setValue(stako.anzahl, "ANZAHL");

    fNextStep = "setItem - herkunft";
    tbl.setValue(stako.herkunft, "HERKUNFT");
  }

} // end class

