package ch.ige.ses.client.sapinbound;

import java.util.*;
import org.apache.log4j.*;
import ch.ige.ses.client.saputil.*;
import ch.ige.ses.vo.bagis.*;
import com.sap.mw.jco.*;

/**
 * SAP Remote Function Module Z_BAPI_ZVL_CHANGE_BILLING_REQ
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ.java,v 1.7 2007/11/13 12:35:02 nydegger Exp $
 */
public class RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ extends RFM
{
  //
  // const
  //
  private static final Logger LOG = Logger.getLogger(RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ.class);

  //
  // fields
  //
  private int     fPrinterSetting   = RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ.DEF_PRINTER_SETTING;
  private String  fPrinterDefName   = RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ.DEF_PRINTER_NAME;
  private boolean fImmediatePrint   = true;

  private boolean fImportParamIsSet = false;     // set by setImportParam

  public  String        fNextStep   = "";        // next step in process (trace info)
  private StringBuffer  fLogDetail  = null;      // log details

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  public RFM_Z_BAPI_ZVL_CHANGE_BILLING_REQ (IRepository repository)
  {
    super("Z_BAPI_ZVL_CHANGE_BILLING_REQ",repository);

    // get property options
    SapProperties sp   = new SapProperties();
    Properties    prop = SapInbound.getProperties();

    // printer settings
    this.fPrinterSetting = sp.getInt ( prop, "sap.rechnung.printer.setting",
                                       RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ.DEF_PRINTER_SETTING );
    if (this.fPrinterSetting == 2)
      fPrinterDefName = prop.getProperty ( "sap.rechnung.printer.defaultname",
                                            RFM_Z_BAPI_ZVL_CREATE_BILLING_REQ.DEF_PRINTER_NAME );

    // immediate print behavior
    if ( prop.getProperty("sap.rechnung.immediate_print").equals("1") )
      this.fImmediatePrint = true;
    else
      this.fImmediatePrint = false;
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public void addRechnung(SAPRechnungVO re)
  {
    try
    {
      // init
      fLogDetail = new StringBuffer();              // log details
      fLogDetail.append("Details: bfakt_id=" + re.bfakt_id + " ");

      // set vaules
      if (! this.fImportParamIsSet)                 // set only once per call
        this.setImportParam(re);

      this.setHeader(re);
      this.setHeaderX(re);
      this.setMessageNew(re);

      // log details
      LOG.debug(fLogDetail.toString());

    }
    catch (Exception ex)
    {
      LOG.error("exception: " + ex.toString());
      LOG.error("hint (next step): " + fNextStep);
      throw new RfmException(ex.toString());
    }
  }


  public String toStringRechnung()
  {
    String[]          tn  = {"HEADER","HEADERX", "MESSAGENEW"};
    JCO.ParameterList in  = this.fJCoFunc.getImportParameterList();
    StringBuffer      buf = new StringBuffer();

    // import params
    buf.append(in.toString());

    // tables
    for (int i = 0; i < tn.length; i++)
    {
      JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable(tn[i]);
      buf.append(tbl.toString());
    }

    return buf.toString();
  }

  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void setImportParam(SAPRechnungVO re)
  {
    JCO.ParameterList input = this.fJCoFunc.getImportParameterList();

    fNextStep = "setImportParam - DOC_TYPE";
    input.setValue("ZVL","DOC_TYPE");

    fNextStep = "setImportParam - CREATED_BY / re.employeeId";
    if (re.employeeId != null)
      input.setValue(re.employeeId,"CREATED_BY");

    fNextStep = "setImportParam - TESTRUN";
    input.setValue(" ","TESTRUN");

    fNextStep = "setImportParam - COMMIT";
    input.setValue("X","COMMIT");                   // enable db commit

    this.fImportParamIsSet = true;
  }


  private void setHeader(SAPRechnungVO re)
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("HEADER");

    tbl.appendRow();

    fNextStep = "setHeader - DOC_NUMBER / re.bfakt_id";
    tbl.setValue(Convert.toSapDocNumber(re.bfakt_id),"DOC_NUMBER");

    if ( re.isAMV() )
    {
      fNextStep = "setHeader - TM_EXT_SEQ_NO / re.amvLaufNr";
      tbl.setValue(re.amvLaufNr,"TM_EXT_SEQ_NO");
    }
  }


  private void setHeaderX(SAPRechnungVO re)
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("HEADERX");

    tbl.appendRow();

    fNextStep = "setHeaderX - UPDATEFLAG / U";
    tbl.setValue("U","UPDATEFLAG");

    if ( re.isAMV() )
    {
      fNextStep = "setHeaderX - TM_EXT_SEQ_NO / X";
      tbl.setValue("X","TM_EXT_SEQ_NO");
    }
  }


  private void setMessageNew(SAPRechnungVO re)
  {
    Integer       zre = null;

    // AG - Auftraggeber
    // wird nicht mehr gesetzt

    //
    // RE - Hauptrechnungsempfänger
    //
    fNextStep = "setPartnerNew - RE / re.hauptRechEmpf";
    setMessageNewEntry (re,"RE",re.hauptRechEmpf);


    // RG - Regulierer
    // wird nicht mehr gesetzt


    //
    // R0..R9 & Z0..Z9 - Zusatzrechnungsempfänger
    //
    if (re.zusatzRechEmpf != null)
    {
      fNextStep = "setPartnerNew - R0..R9 & Z0..Z9 / re.zusatzRechEmpf";

      // check maximal allowed number of partners
      if (re.zusatzRechEmpf.size() > SAPRechnungVO.MAX_RECH_EMPF)
      {
        throw new RfmException("setPartnerNew - re.zusatzRechEmpf überschreitet max. Anzahl zulässige Elemente !");
      }

      // set partner
      for (int i = 0; i < re.zusatzRechEmpf.size(); i++)
      {
        zre = (Integer) re.zusatzRechEmpf.get(i);

        setMessageNewEntry (re,Convert.toSapPartnerRole(i), zre.intValue());
      }
    } // end if
  }


  private void setMessageNewEntry ( SAPRechnungVO re,
                                    String partnerRole, int partnerNumb )
  {
    JCO.Table tbl = this.fJCoFunc.getTableParameterList().getTable("MESSAGENEW");
    tbl.appendRow();
    fLogDetail.append(partnerRole + "=" + partnerNumb + " ");

    //
    // Mandantory Fields
    //
    fNextStep = "setPartnerNewEntry - DOC_NUMBER / re.bfakt_id";
    tbl.setValue(Convert.toSapDocNumber(re.bfakt_id),"DOC_NUMBER");

    fNextStep = "setPartnerNewEntry - ITM_NUMBER";
    tbl.setValue(Convert.toSapItmNumber(0),"ITM_NUMBER");

    fNextStep = "setPartnerNewEntry - PARTN_ROLE / " + partnerRole;
    tbl.setValue(partnerRole,"PARTN_ROLE");

    fNextStep = "setPartnerNewEntry - PARTN_NUMB";
    tbl.setValue(Convert.toSapNr(partnerNumb,10),"PARTN_NUMB");


    //
    // Optional Fields
    //
    fNextStep = "setPartnerNewEntry - MESSAGE_TYPE";
    tbl.setValue(Convert.toSapMessageType(re,null),"MESSAGE_TYPE");

    fNextStep = "setPartnerNewEntry - LANGU";
    tbl.setValue(Convert.toSapLangu(re.sprach_cd),"LANGU");

    fNextStep = "setPartnerNewEntry - MEDIUM";
    tbl.setValue("1","MEDIUM");

    fNextStep = "setPartnerNewEntry - COPIES";
    tbl.setValue(1,"COPIES");

    fNextStep = "setPartnerNewEntry - DISPATCH_TIME";
    if (re.druck_art != null)
      tbl.setValue(Convert.toSapDispatchTime(re.druck_art),"DISPATCH_TIME");

    fNextStep = "setPartnerNewEntry - PRINTER";
    switch (fPrinterSetting)
    {
      case 0:                                     // do not set printername
        tbl.setValue("","PRINTER");
        break;
      case 2:                                     // use default printername
        tbl.setValue(this.fPrinterDefName,"PRINTER");
        break;
    }

    fNextStep = "setPartnerNewEntry - IMMEDIATE_PRINT";
    if ( this.fImmediatePrint )                   // standart behavior
    {
      if (re.isRechnung())
      {
        tbl.setValue("X","IMMEDIATE_PRINT");
      }
      else
      {
        tbl.setValue(" ","IMMEDIATE_PRINT");
      }
    }
    else                                          // switch off printing
    {                                             // (for test purpose)
      tbl.setValue(" ","IMMEDIATE_PRINT");
    }

    // set amv params
    if (re.isAMV())
    {
      fNextStep = "setPartnerNewEntry - PROT_TITLE / re.schutit_nr";
      tbl.setValue(re.schutit_nr,"PROT_TITLE");
    }
  }

} // end class

