package ch.ige.ses.server.bagisoutbound;

import java.util.ArrayList;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.ses.BfaktJGBSESDAO;
import ch.ige.ses.server.dao.ses.BfaktposSESDAO;
import ch.ige.ses.server.dao.ses.BfakturSESDAO;
import ch.ige.ses.server.dao.ses.JGBLaufDAO;
import ch.ige.ses.server.dao.ses.JoinSESRechnungDAO;
import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.ses.server.error.SESErrorProcessLocal;
import ch.ige.ses.server.error.SESErrorProcessLocalHome;
import ch.ige.ses.vo.bagis.SAPJGBParamVO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.bagis.SAPRechnungsPositionVO;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.JGBLaufVO;

/**
 * SES: Bean zur Verarbeitung der Jahresgebühren BAGIS-Rechnungen<br>
 * Speichert die übergebenen bfakturen in den SES-Tabellen ab
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author   Stefan Nydegger
 * @version  $Id: SESJgbWorkBean.java,v 1.58 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SESJgbWork"  
 *  display-name="SESJgbWork"
 *  view-type="local"
 *  jndi-name="SESJgbWork"
 *  local-jndi-name="SESJgbWorkLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="RequiresNew"
 * @ejb.ejb-ref
 *   ejb-name="SESErrorProcess"
 *   view-type="local"
 */
public class SESJgbWorkBean implements SessionBean
{
  //
  // const
  //
  private static final Logger LOG         = Logger.getLogger(SESJgbWorkBean.class);
  private static final String SOURCE_NAME = "SESJgbWorkBean";

  //
  // fields
  //
  SessionContext            sessionContext;

  SESErrorProcessLocalHome  sesErrorProcessLocalHome  = null;
  SESErrorProcessLocal      sesErrorProcessLocal      = null;


  //---------------------------------------------------------------------------
  // ejb methods
  //---------------------------------------------------------------------------

  
  /**
   * create ejb
   */
  public void ejbCreate()
  {
    try
    {
      Context ctx = new InitialContext();
      Object  ref = ctx.lookup(sesErrorProcessLocalHome.COMP_NAME);

      sesErrorProcessLocalHome  = (SESErrorProcessLocalHome)ref;
      sesErrorProcessLocal      = sesErrorProcessLocalHome.create();
    }
    catch (CreateException ex)
    {
      LOG.error("ejbCreate - CreateException : " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("ejbCreate - NamingException: " + ex.getMessage());
    }
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }


  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Methode requestStop schreibt einen Stop Request in die Tabelle
   * JGB_LAUF (SES_STAT = SES_STAT_REQUEST_STOP).
   *
   * @throws SESException Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void requestStop() throws SESException {

    JGBLaufVO  jgbLaufVO  = new JGBLaufVO();
    JGBLaufDAO jgbLaufDAO = new JGBLaufDAO();

    try
    {
      // aktuelle Laufeigenschaften holen
      LOG.info("requestStop - selectMaxLaufID");
      jgbLaufVO = jgbLaufDAO.selectMaxLaufID();
      LOG.info("requestStop - selectMaxLaufID=" + jgbLaufVO.jgbLaufId);

      // Lauf darf nur abgebrochen werden, wenn ses_stat auf status 0
      // (Verarbeitung gestartet) ist
      switch (jgbLaufVO.sesStat)
      {
        case JGBLaufVO.SES_STAT_STARTED:
          jgbLaufUpdSesStat ( jgbLaufVO.jgbLaufId,
                              JGBLaufVO.SES_STAT_REQUEST_STOP );
          break;

        case JGBLaufVO.SES_STAT_REQUEST_STOP:
          throw new SESException ("Abbruch Request bereits in Verarbeitung");

        case JGBLaufVO.SES_STAT_STOPPED:
          throw new SESException ("Verarbeitung bereits abgebrochen");

        case JGBLaufVO.SES_STAT_FINISHED:
          throw new SESException ("Verarbeitung bereits abgeschlossen");
      }

    }
    catch (SESException sx)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( SOURCE_NAME + " requestStop - " + sx.getMessage() + "\n" +
                   "Der Stop Request wird abgebrochen \n" );

    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( SOURCE_NAME + " requestStop - Es ist ein unerwarteter Fehler aufgetreten \n" +
                   "Der Stop Request wird abgebrochen \n" +
                   "Exception : \n" + ex.getMessage() );
    }
  }


  /**
   * Speichert einzelne Rechnung in die SES Tabellen
   *
   * @param  sapRechnungVO  ValueObject mit Rechnungsdaten
   * @return                ErrorVO im Fehlerfall
   *
   * @ejb.interface-method
   */
  public ErrorVO saveSingleVO(SAPRechnungVO sapRechnungVO)
  {
    BfakturSESDAO  bfakturSESDAO  = new BfakturSESDAO();
    BfaktposSESDAO bfaktPosSESDAO = new BfaktposSESDAO();
    BfaktJGBSESDAO bfaktJGBSESDAO = new BfaktJGBSESDAO();

    try
    {
      // bfakturen in Tabelle bfaktur_ses speichern
      // LOG.info("saveSingleVO - bfakturen in Tabelle bfaktur_ses speichern");
      bfakturSESDAO.insert(sapRechnungVO);

      // Rechnungspositionen in Tabelle bfaktpos_ses speichern
      // LOG.info("saveSingleVO - Rechnungspositionen in Tabelle bfaktpos_ses speichern");
      for (int n = 0; n < sapRechnungVO.posList.size(); n++)
      {
        bfaktPosSESDAO.insert((SAPRechnungsPositionVO)sapRechnungVO.posList.get(n));
      }

      // Hauptrechnungsempfänger in Tabelle bfaktjgb_ses speichern
      // LOG.info("saveSingleVO - Hauptrechnungsempfänger in Tabelle bfaktjgb_ses speichern");
      bfaktJGBSESDAO.insert(sapRechnungVO.jgbHauptRechEmpfParam);

      // Zusatz-Rechnungsempfänger in Tabelle bfaktjgb_ses speichern
      // LOG.info("saveSingleVO - Zusatz-Rechnungsempfänger in Tabelle bfaktjgb_ses speichern");
      if(sapRechnungVO.jgbZusatzRechEmpfParam.size() > 0)
      {
        for (int m = 0; m < sapRechnungVO.jgbZusatzRechEmpfParam.size(); m++)
        {
          bfaktJGBSESDAO.insert((SAPJGBParamVO)sapRechnungVO.jgbZusatzRechEmpfParam.get(m));
        }
      }


      // Falls mehr als MAX_RECH_EMPF Zusatzempfänger existieren,
      // zusätzlich SapRechnungVO in ERROR Tabelle speichern
      if ( sapRechnungVO.zusatzRechEmpfNichtVerarbeitet != null &&
           sapRechnungVO.zusatzRechEmpfNichtVerarbeitet.size() > 0 )
      {
        LOG.debug ( "saveSingleVO - Nicht verarbeitete zusätzliche Rechnungsempfänger bei bfakt_id " +
                    sapRechnungVO.bfakt_id + " in Error Tabelle speichern" );

        ErrorVO errorVO           = new ErrorVO();
        errorVO.source            = SOURCE_NAME;
        errorVO.employeeNotify    = ErrorVO.EMPL_NOTIFY_PENDENT;
        errorVO.employeeMsgType   = ErrorVO.EMPL_MSG_TYPE_MAX_EMPF;
        errorVO.description       = "Nicht verarbeitete zusätzliche Rechnungsempfänger";
        errorVO.dataTypeCd        = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_VO;
        errorVO.errorStat         = ErrorVO.ERROR_STAT_PENDENT;
        errorVO.data              = sapRechnungVO;

        errorSave(errorVO);
      }

      // vo erfolgreich in ses tabellen gespeichert
      // LOG.info("saveSingleVO - vo erfolgreich in ses tabellen gespeichert");
      return null;

    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();

      ErrorVO errorVO = new ErrorVO();
      errorVO.source          = SOURCE_NAME;
      errorVO.employeeNotify  = ErrorVO.EMPL_NOTIFY_KEINE;    // keine Benachrichtigung für einzelne Rechnungen
      errorVO.employeeMsgType = ErrorVO.EMPL_MSG_TYPE_JGB;
      errorVO.description     = "Rechnung mit bfakt_id " + sapRechnungVO.bfakt_id + " konnte nicht auf SES gespeichert werden \n" +
                                "Exception : \n" + ex.getMessage() ;
      errorVO.dataTypeCd      = ErrorVO.DATA_TYPE_CD_SAP_RECHNUNG_VO;
      errorVO.errorStat       = ErrorVO.ERROR_STAT_PENDENT;
      errorVO.data            = sapRechnungVO;

      // vo NICHT erfolgreich in ses tabellen gespeichert
      LOG.error(errorVO.description);
      return errorVO;
    }
  }


  //---------------------------------------------------------------------------
  // methods for table jgb lauf
  //---------------------------------------------------------------------------

  /**
   * jgb lauf select
   * 
   * @param laufNr
   * @return result
   * @throws SESException
   * 
   * @ejb.interface-method
   */  
  public JGBLaufVO jgbLaufSel(String laufNr) throws SESException {
    try
    {
      LOG.info("jgbLaufSel - insert laufNr=" + laufNr);
      return (new JGBLaufDAO().select(laufNr));
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( "jgbLaufSel - Es ist ein unerwarteter Fehler aufgetreten \n" +
                   "Die Verarbeitung der Jahresgebühren wird abgebrochen \n" +
                   "laufNr    : \n" + laufNr + "\n" +
                   "Exception : \n" + ex.getMessage() );
    }
  }

  
  /**
   * jgb lauf insert
   * 
   * @param vo
   * @throws SESException
   * 
   * @ejb.interface-method
   */
  public void jgbLaufIns (JGBLaufVO vo) throws SESException {
    try
    {
      LOG.info("jgbLaufIns - insert laufNr=" + vo.laufNr);
      new JGBLaufDAO().insert(vo);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( "jgbLaufIns - Es ist ein unerwarteter Fehler aufgetreten \n" +
                   "Die Verarbeitung der Jahresgebühren wird abgebrochen \n" +
                   "JGBLaufVO : \n" + vo.toString() + "\n" +
                   "Exception : \n" + ex.getMessage() );
    }
  }

  
  /**
   * jgb lauf update ses count
   * 
   * @param vo
   * @throws SESException
   * 
   * @ejb.interface-method
   */
  public void jgbLaufUpdSesCount (JGBLaufVO vo) throws SESException {
    JGBLaufDAO dao = null;
    int        id  = vo.jgbLaufId;

    try
    {
      LOG.info("jgbLaufUpdSesCount");
      dao = new JGBLaufDAO();

      dao.updateAnzahlBfakturGelesen    (id, vo.anzahlBfakturGelesen );
      dao.updateCountBfakturToSap       (id, vo.countBfakturToSap);
      dao.updateAnzahlBfaktjgbGeneriert (id, vo.anzahlBfaktjgbGeneriert);
      dao.updateMinBfakturToSap         (id, vo.minBfakturToSap);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( "jgbLaufUpdSesCount - Es ist ein unerwarteter Fehler aufgetreten \n" +
                   "Die Verarbeitung der Jahresgebühren wird abgebrochen \n" +
                   "JGBLaufVO : \n" + vo.toString() + "\n" +
                   "Exception : \n" + ex.getMessage() );
    }
  }

  /**
   * jgb lauf update ses stat
   * 
   * @param id
   * @param stat
   * @throws SESException
   * 
   * @ejb.interface-method
   */
  public void jgbLaufUpdSesStat (int id, int stat) throws SESException {
    try
    {
      LOG.info("jgbLaufUpdSesStat - id=" + id + ", stat=" + stat);
      new JGBLaufDAO().updateSesStat(id,stat);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( "jgbLaufUpdSesStat - Es ist ein unerwarteter Fehler aufgetreten \n" +
                   "Die Verarbeitung der Jahresgebühren wird abgebrochen \n" +
                   "id / stat : \n" + id + " / " + stat + "\n" +
                   "Exception : \n" + ex.getMessage() );
    }
  }

  
  /**
   * jgb lauf update ses stat
   * 
   * @param id
   * @param statOld
   * @param stat
   * @return result
   * @throws SESException
   * 
   * @ejb.interface-method
   */
  public int jgbLaufUpdSesStat (int id, int statOld, int stat) throws SESException
  {
    try
    {
      LOG.info("jgbLaufUpdSesStat - id=" + id + ", statOld=" + statOld + ", stat=" + stat);
      return ( new JGBLaufDAO().updateSesStat(id,statOld,stat) );
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      throw new SESException
                 ( "jgbLaufUpdSesStat - Es ist ein unerwarteter Fehler aufgetreten \n" +
                   "Die Verarbeitung der Jahresgebühren wird abgebrochen \n" +
                   "id / statOld / stat : \n" + id + " / " + statOld + " / " + stat + "\n" +
                   "Exception : \n" + ex.getMessage() );
    }
  }


  //---------------------------------------------------------------------------
  // methods for table error
  //---------------------------------------------------------------------------

  /**
   * save error
   * 
   * @param errorVO
   *  
   * @ejb.interface-method
   */
  public void errorSave (ErrorVO errorVO)
  {
    try
    {
      sesErrorProcessLocal.processError(errorVO);
    }
    catch (Exception ex)
    {
      // do NOT set setRollbackOnly()
      LOG.fatal ("errorSave - Ein unerwarteter Fehler ist aufgetreten " +
                 "ErrorVO   : \n" + errorVO.toString() + "\n" +
                 "Exception : \n" + ex.getMessage() );
    }
  }

  
  /**
   * save error and description
   * 
   * @param source
   * @param description
   * 
   * @ejb.interface-method
   */
  public void errorSave (String source, String description)
  {
    ErrorVO vo = new ErrorVO();

    LOG.info("errorSave - save vo to ses error db");

    vo.source           = source;
    vo.description      = description;
    vo.employeeNotify   = ErrorVO.EMPL_NOTIFY_PENDENT;
    vo.employeeMsgType  = ErrorVO.EMPL_MSG_TYPE_JGB;
    vo.dataTypeCd       = ErrorVO.DATA_TYPE_CD_NO_DATA;
    vo.errorStat        = ErrorVO.ERROR_STAT_PENDENT;

    errorSave(vo);
  }

  //---------------------------------------------------------------------------
  // methods for table property
  //---------------------------------------------------------------------------

  /**
   * select properties
   * 
   * @param key 
   * @return result
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public String propertySel (String key) throws Exception
  {
    try
    {
      return new PropertyDAO().select(key).value;
    }
    catch (Exception ex)
    {
      // do NOT set setRollbackOnly()

      LOG.error ( "propertySel - Error reading key <" + key + "> " +
                  "Exception : " + ex.getMessage() );

      throw new Exception(ex.getMessage());
    }
  }

  //---------------------------------------------------------------------------
  // methods for JGB - Errorlist
  //---------------------------------------------------------------------------

  /**
   * Liefert eine ArrayList mit fehlerhaften Bfakturen eines JGB-Laufes
   *
   * @param jgbLaufId Lauf-Id des Jahresgebührenlaufes
   * @return ArrayList mit SAPRechnungVO's
   * @throws Exception
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public ArrayList getJgbErrors(int jgbLaufId) throws Exception
  {
    try
    {
      return new BfakturSESDAO().select(jgbLaufId, "E");
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getJgbErrors - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method


  /**
   * Selektiert einen Jahresgebühren-Lauf anhand der Lauf-ID
   * 
   * @param jgbLaufId Lauf-ID
   * @return JGBLaufVO VO des Jahresgebühren-Laufes
   * @throws Exception
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public JGBLaufVO getJgbLauf(int jgbLaufId) throws Exception
  {
    try
    {
      return new JGBLaufDAO().select(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getJgbLauf - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method

  //---------------------------------------------------------------------------
  // methods for JGB - statistics
  //---------------------------------------------------------------------------

  /**
   * Gibt die höchste Bfakt_id eines Jahresgebühren Laufes zurück
   * 
   * @param jgbLaufId Id des Jahresgebühren Laufes
   * @return Die höchste Bfakt_id des Laufes
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public int selectMaxBfaktID(int jgbLaufId) throws Exception
  {
    try
    {
      return new BfakturSESDAO().selectMaxBfaktID(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("selectMaxBfaktId - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method

  
  /**
   * Gibt die kleinest Bfakt_id eines Jahresgebühren Laufes zurück
   * 
   * @param jgbLaufId Id des Jahresgebühren Laufes
   * @return Die kleinste Bfakt_id des Laufes
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public int selectMinBfaktID(int jgbLaufId) throws Exception
  {
    try
    {
      return new BfakturSESDAO().selectMinBfaktID(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("selectMinBfaktId - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method

  
  /**
   * Gibt die Anzahl Bfakturen eines JGB-Laufes zurück
   * 
   * @param jgbLaufId Lauf-ID
   * @return Anzahl Bfakturen eines JGB-Laufes
   * @throws Exception
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public int countBfaktur(int jgbLaufId) throws Exception
  {
    try
    {
      return new BfakturSESDAO().count(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("countBfaktur - Exception" +ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method


  /**
   * Gibt die Anzahl Empfänger eines JGB-Laufes zurück
   * 
   * @param jgbLaufId Lauf-ID
   * @return Anzahl Empfänger eines JGB-Laufes
   * @throws Exception
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public int countEmpfaenger(int jgbLaufId) throws Exception
  {
    try
    {
      return new JoinSESRechnungDAO().countEmpfaenger(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("countEmpfaenger - Exception" + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method

  
  /**
   * Gibt die Anzahl Druckperioden zurück
   * 
   * @param jgbLaufId Lauf-ID
   * @return Anzahl Druckperioden
   * @throws Exception
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public int getAnzPerioden(int jgbLaufId) throws Exception
  {
    try
    {
      return new JoinSESRechnungDAO().getAnzPerioden(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getAnzPerioden - Exception " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  } // end method

  
  /**
   * Gibt die Anzahl Rechnungsempfänger mit einem bestimmten Verarbeitungsstatus
   * eines Laufes zurück
   * 
   * @param jgbLaufId Lauf-ID
   * @param status Verarbeitungsstatus der BFAKTUR
   * @return Anzahl Rechnungsempfänger
   * @throws Exception
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public int countByStatus(int jgbLaufId, String status) throws Exception
  {
    try
    {
      return new JoinSESRechnungDAO().countByStatus(jgbLaufId, status);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("countByStatus - Exception " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  }


  /**
   * Liste mit allen Rechnungsempfängern, welche bereits erfolgreich nach
   * SAP übertragen wurden, d.h. den Status "verarbeitet" oder "gedruckt"
   * aufweisen.
   *
   * @param   jgbLaufId    Lauf Id
   * @throws  Exception    Unerwarteter Fehler
   * @return  ArrayList    Liste mit Rechnungsempfängern
   * 
   * @ejb.interface-method
   */
  public ArrayList getStatistikCalcReVOList (int jgbLaufId) throws Exception
  {
    try
    {
      return new JoinSESRechnungDAO().getStatistikCalcReVOList(jgbLaufId);
    }
    catch (Exception ex)
    {
      sessionContext.setRollbackOnly();
      LOG.error("getStatistikCalcReVOList - Exception " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
  }

} // end class




