package ch.ige.ses.server.batchctrl;

import java.util.*;
import ch.ige.util.ServiceLocator;
import ch.ige.ses.vo.ses.*;

/**
 * Delegate für die Batch Jobs.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: BatchDelegate.java,v 1.30 2006/12/13 14:48:40 iten Exp $
 */
public class BatchDelegate
{
  //
  // fields
  //
  private static BatchDelegate   fDelegate  = null;
  private        BatchJob        fBatchJob  = null;


  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------

  private BatchDelegate() throws Exception
  {
    BatchJobHome home;

    home = (BatchJobHome) ServiceLocator.getInstance().getHome
                            (BatchJobHome.JNDI_NAME, BatchJobHome.class);

    fBatchJob = home.create();
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  public static BatchDelegate getInstance() throws Exception
  {
    if (fDelegate == null)
    {
      fDelegate = new BatchDelegate();
    }
    return fDelegate;
  }

  //
  // batch job - common stuff
  //
  public void batchNotifyEmail (String jobName, String subject, String body) throws Exception
  {
    fBatchJob.batchNotifyEmail(jobName,subject,body);
  }

  public boolean batchJobEnabled (String jobName) throws Exception
  {
    return fBatchJob.batchJobEnabled(jobName);
  }

  public String getProperty (String key) throws Exception
  {
    return fBatchJob.getProperty(key);
  }

  public String getProperty (String key, String defaultKey) throws Exception
  {
    return fBatchJob.getProperty(key,defaultKey);
  }

  //
  // batch job - ses frist ablauf
  //
  public List fristSelectBfaktIdList() throws Exception
  {
    return fBatchJob.fristSelectBfaktIdList();
  }


  //
  // batch job - ses jgb frist ablauf
  //
  public List jgbfristSelectBfaktIdList() throws Exception
  {
    return fBatchJob.jgbfristSelectBfaktIdList();
  }


  //
  // batch job - ses jgb start
  //
  public int jgbStart (String modus, int bfaktIdVon, int bfaktIdBis) throws Exception
  {
    return fBatchJob.jgbStart(modus,bfaktIdVon,bfaktIdBis);
  }

  public int jgbGetNewLaufId (int lastLaufId) throws Exception
  {
    return fBatchJob.jgbGetNewLaufId(lastLaufId);
  }

  public int jgbCtrlJobState (int newLaufId) throws Exception
  {
    return fBatchJob.jgbCtrlJobState(newLaufId);
  }


  //
  // batch job - ses amv start
  //
  public ArrayList amvBagisGetRechnungNrList(int amvRechnungCd) throws Exception
  {
    return fBatchJob.amvBagisGetRechnungNrList(amvRechnungCd);
  }

  public void amvBagisCreateRechnung ( int amvLaufId, String amvLaufNr,
                                       int amvRechnungCd, int bfaktId ) throws Exception
  {
    fBatchJob.amvBagisCreateRechnung(amvLaufId,amvLaufNr,amvRechnungCd,bfaktId);
  }

  public void amvBagisSetRechnungStatusDruck (ArrayList list, String status) throws Exception
  {
    fBatchJob.amvBagisSetRechnungStatusDruck(list,status);
  }

  public boolean amvSesIsLastLaufFinished() throws Exception
  {
    return fBatchJob.amvSesIsLastLaufFinished();
  }

  public AmvLaufVO amvSesCreateLauf() throws Exception
  {
    return fBatchJob.amvSesCreateLauf();
  }

  public AmvLaufVO amvSesGetLauf(int amvLaufId) throws Exception
  {
    return fBatchJob.amvSesGetLauf(amvLaufId);
  }

  public void amvSesInsertRechnung(int amvLaufId,
                                   ArrayList e1List,ArrayList e2List) throws Exception
  {
    fBatchJob.amvSesInsertRechnung(amvLaufId,e1List,e2List);
  }

  public boolean amvSesIsErinnerung2Enabled() throws Exception
  {
    return fBatchJob.amvSesIsErinnerung2Enabled();
  }

  public boolean amvSesIsCheckLastStateEnabled() throws Exception
  {
    return fBatchJob.amvSesIsCheckLastStateEnabled();
  }

  public void amvSesSetRechnungStatus(AmvRechnungVO vo) throws Exception
  {
    fBatchJob.amvSesSetRechnungStatus(vo);
  }

  public void amvSesSetLaufStatus(int amvLaufId, int laufStat) throws Exception
  {
    fBatchJob.amvSesSetLaufStatus(amvLaufId,laufStat);
  }

  public int[] amvSesGetRechnungCount (int amvLaufId) throws Exception
  {
    return fBatchJob.amvSesGetRechnungCount(amvLaufId);
  }

  public int[] amvSesGetRechnungEmpfCount (int amvLaufId, int rechnungCd) throws Exception
  {
    return fBatchJob.amvSesGetRechnungEmpfCount(amvLaufId,rechnungCd);
  }

  public boolean amvSesIsRunning (int amvLaufId) throws Exception
  {
    return fBatchJob.amvSesIsRunning(amvLaufId);
  }

  public boolean amvSesHasErrors (int amvLaufId) throws Exception
  {
    return fBatchJob.amvSesHasErrors(amvLaufId);
  }

  public int amvSesResetErrors (int amvLaufId) throws Exception
  {
    return fBatchJob.amvSesResetErrors(amvLaufId);
  }

  public ArrayList amvSesGetRechnungNrList(int amvLaufId, int amvRechnungCd) throws Exception
  {
    return fBatchJob.amvSesGetRechnungNrList(amvLaufId,amvRechnungCd);
  }

  public ArrayList amvSesGetRechnungListByStat(int amvLaufId, int rechnungStat)
    throws Exception
  {
    return fBatchJob.amvSesGetRechnungListByStat(amvLaufId, rechnungStat);
  }


  //
  // batch job - ses controlling
  //
  public void controllingStart(int modus, java.sql.Date bisDate) throws Exception
  {
    fBatchJob.controllingStart(modus, bisDate);
  }

  public int controllingGetNewId() throws Exception
  {
    return fBatchJob.controllingGetNewId();
  }

  public int controllingCtrlJobState (int kontrollID) throws Exception
  {
    return fBatchJob.controllingCtrlJobState(kontrollID);
  }


  //
  // batch job - ses reorganisation
  //
  public String reorgDatabase (String table, int month) throws Exception
  {
    return fBatchJob.reorgDatabase(table,month);
  }

} // end class





