package ch.ige.ses.server.batchctrl.test;

import junit.framework.*;
import ch.ige.ses.server.batchctrl.BatchDelegate;

/**
 * JUnit Test Class
 * <p>
 * Copyright (c) 2004, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: BatchDelegateTest.java,v 1.6 2006/12/13 14:58:40 iten Exp $
 */
public class BatchDelegateTest extends TestCase
{
  //
  // fields
  //
  BatchDelegate delegate = null;


  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public BatchDelegateTest(String s) throws Exception
  {
    super(s);
    delegate = BatchDelegate.getInstance();
  }


  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(BatchDelegateTest.class);
  }

  protected void setUp()
  {
    // lazy bone
  }


  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void testBatchJobEnabled()
  {
    String  jobName = "SesJgbStart";
    boolean enabled = false;

    try
    {
      enabled = delegate.batchJobEnabled(jobName);
      assertTrue(enabled);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void testBatchJobEnabledException()
  {
    String  jobName = "UnkownJob";

    try
    {
      delegate.batchJobEnabled(jobName);
      fail("test should throw an exception");
    }
    catch (Exception ex)
    {
      assertTrue(true);
    }
  }


  public void testGetProperty()
  {
    String  key   = "batch.SesJgbStart.job.enabled";
    String  value = null;

    try
    {
      value = delegate.getProperty(key);
      assertNotNull(value);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void testGetPropertyException()
  {
    String  key   = "unkown.property";

    try
    {
      delegate.getProperty(key);
      fail("test should throw an exception");
    }
    catch (Exception ex)
    {
      assertTrue(true);
    }
  }


  public void testGetPropertyWithDefaultByDefKey()
  {
    String  key    = "unkown.property";
    String  defKey = "batch.default.mail.enable";
    String  value  = null;

    try
    {
      value = delegate.getProperty(key,defKey);
      assertNotNull(value);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void testGetPropertyWithDefaultByKey()
  {
    String  key     = "batch.default.mail.enable";
    String  defKey  = "unkown.property";
    String  value   = null;

    try
    {
      value = delegate.getProperty(key,defKey);
      assertNotNull(value);
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
      fail(ex.toString());
    }
  }


  public void testGetPropertyWithDefaultException()
  {
    String  key     = "unkown.property";
    String  defKey  = "unkown.property";

    try
    {
      delegate.getProperty(key,defKey);
      fail("test should throw an exception");
    }
    catch (Exception ex)
    {
      assertTrue(true);
    }
  }


} // end class

