package ch.ige.ses.server.dao.bagis.test;

import java.util.*;
import java.sql.*;
import ch.ige.ses.server.dao.bagis.*;
import ch.ige.ses.vo.bagis.*;
import junit.framework.*;
import ch.ige.util.server.DBHelper;
import java.math.BigDecimal;

/**
 * junit test class
 * <p>
 * Copyright (c) 2005, Swiss Federal Institute of Intellectual Property
 * @author T. Iten
 * @version $Id: BfakturDAOTest.java,v 1.22 2007/12/28 09:23:57 iten Exp $
 */
public class BfakturDAOTest extends BagisTestCase
{
  //---------------------------------------------------------------------------
  // consturctor
  //---------------------------------------------------------------------------

  public BfakturDAOTest(String s)
  {
    super(s);
  }


  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(BfakturDAOTest.class);
  }


  protected void setUp()
  {
    // lazy bone
  }


  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests - select
  //---------------------------------------------------------------------------

  public void testSelectBfaktId()
  {
    BfakturDAO  dao     = new BfakturDAO();
    List        result  = null;

    String      fromDate    = "03.01.2000";           // test data
    String      toDate      = "04.01.2000";
    int         resultSize  = 165;

    try
    {
      System.out.println();
      System.out.println("testSelectBfaktId:");

      result = dao.selectBfaktId(fromDate,toDate);
      System.out.println("  query = " + dao.getQuery());

      if (result == null)
      {
        System.out.println("  result = null");
        assertTrue(false);
      }
      else if (result.size() == 0)
      {
        System.out.println("  result size = 0");
        assertTrue(false);
      }
      else
      {
        System.out.println("  result size expected = " + resultSize);
        System.out.println("  result size effectiv = " + result.size());
        assertEquals(resultSize,result.size());
      }
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectBfaktIdRangeForJGB()
  {
    BfakturDAO  dao     = new BfakturDAO();
    HashMap     result  = null;

    try
    {
      System.out.println();
      System.out.println("testSelectBfaktIdRangeForJGB:");

      result = dao.selectBfaktIdRangeForJGB();
      System.out.println("  query = " + dao.getQuery());

      System.out.println("  minValue expected = unknown");
      System.out.println("  minValue effectiv = " + result.get("minValue"));

      System.out.println("  maxValue expected = unknown");
      System.out.println("  maxValue effectiv = " + result.get("maxValue"));

      System.out.println("  count expected    = unknown");
      System.out.println("  count effectiv    = " + result.get("count"));

      // assertTrue( (Integer)result.get("minValue") == null );
      // assertTrue( (Integer)result.get("maxValue") == null );
      // assertTrue(((Integer)result.get("count")).intValue() == 0);

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectBfaktIdForFristAblauf()
  {
    BfakturDAO  dao     = new BfakturDAO();
    List        result  = null;

    try
    {
      System.out.println();
      System.out.println("testSelectBfaktIdForFristAblauf:");

      result = dao.selectBfaktIdForFristAblauf();

      System.out.println("  result size expected = > 0");
      System.out.println("  result size effectiv = " + result.size());
      assertTrue(result.size() > 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectBfaktIdForFristAblaufJGB()
  {
    BfakturDAO  dao     = new BfakturDAO();
    List        result  = null;

    try
    {
      System.out.println();
      System.out.println("testSelectBfaktIdForFristAblaufJGB:");

      result = dao.selectBfaktIdForFristAblaufJGB();

      System.out.println("  result size expected = unknown");
      System.out.println("  result size effectiv = " + result.size());
      // assertTrue(result.size() == 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectBfaktIdForAMV()
  {
    BfakturDAO  dao           = new BfakturDAO();
    String      stateList[]   = {"1","2"};
    ArrayList   result        = null;

    try
    {
      System.out.println();
      System.out.println("testSelectBfaktIdForAMV:");

      for (int i = 0; i < stateList.length; i++)
      {
        System.out.println("  Erinnerung " + stateList[i]);

        result = dao.selectBfaktIdForAMV(stateList[i]);
        System.out.println("    query = " + dao.getQuery());

        System.out.println("    result size expected = 0");
        System.out.println("    result size effectiv = " + result.size());
        
        // assert is only possible if testdata is checked before execution  
        // assertTrue(result.size() == 0);
      }

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectKzFristAbgelaufen()
  {
    BfakturDAO  dao     = new BfakturDAO();
    int         result  = -1;

    int         id[]    = {5000000, 5000010};         // test data
    int         kz[]    = {0, 3};

    try
    {
      System.out.println();
      System.out.println("testSelectKzFristAbgelaufen:");

      for (int i = 0; i < id.length; i++)
      {
        result = dao.selectKzFristAbgelaufen(id[i]);

        System.out.println("  Test Case #" + i + " select kz_frist_abgelaufen for bfakt_id " + id[i]);
        System.out.println("    result expected = " + kz[i]);
        System.out.println("    result effectiv = " + result);
        assertTrue(result == kz[i]);
      }

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectBfakturForTestStatusBezahlt()
  {
    BfakturDAO                dao     = new BfakturDAO();
    SAPRechnungStatusListVO   result  = null;

    try
    {
      System.out.println();
      System.out.println("testSelectBfakturForTestStatusBezahlt:");

      result = dao.selectBfaktIdForTestStatusBezahlt();

      System.out.println("  result size expected = 0");
      System.out.println("  result size effectiv = " + result.statusList.size());
      assertTrue(result.statusList.size() == 0);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectBfakturForTestStatusDruck()
  {
    BfakturDAO                dao     = new BfakturDAO();
    SAPRechnungStatusListVO   result  = null;

    int   id = 5000000;         // test data

    try
    {
      System.out.println();
      System.out.println("testSelectBfakturForTestStatusDruck:");

      System.out.println("  Test Case - Single");
      result = dao.selectBfaktIdForTestStatusDruck(id);
      System.out.println("    result size expected = 1");
      System.out.println("    result size effectiv = " + result.statusList.size());
      assertTrue(result.statusList.size() == 1);

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  //---------------------------------------------------------------------------
  // tests - update
  //---------------------------------------------------------------------------

  public void testUpdateBfakturFristAbgelaufen()
  {
    BfakturDAO  dao = new BfakturDAO();
    int         id  = 5000003;                              // test data

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturFristAbgelaufen:");

      // set up
      update( "UPDATE bfaktur SET status_zahlg='B',version=version+1 WHERE bfakt_id=" + id);
      assertEquals("B",selectStatusZahlg(id));

      update
        ( "update ereignis " +
          "set    publ_stat=0, pmmbl_seite=null, pmmbl_jahr=null, version=version+1 " +
          "where  (schutit_key, ereig_idat) in  " +
          "       (select  schutit_key, ereig_idat " +
          "        from    bfaktpos " +
          "        where   bfakt_id =  " + id +  ")");

      // test
      dao.updateBfakturFristAbgelaufen(id);
      assertEquals("A",selectStatusZahlg(id));

      // tear down
      update( "UPDATE bfaktur SET status_zahlg='4',version=version+1 WHERE bfakt_id=" + id);
      assertEquals("4",selectStatusZahlg(id));

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturAnnullationInArbeit()
  {
    BfakturDAO  dao = new BfakturDAO();
    int         id  = 5000003;                              // test data

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturAnnullationInArbeit:");

      // set up
      update( "UPDATE bfaktur SET status_zahlg='4',version=version+1 WHERE bfakt_id=" + id);
      assertEquals("4",selectStatusZahlg(id));

      // test
      dao.updateBfakturAnnullationInArbeit(id);
      assertEquals("B",selectStatusZahlg(id));

      // tear down
      update( "UPDATE bfaktur SET status_zahlg='4',version=version+1 WHERE bfakt_id=" + id);
      assertEquals("4",selectStatusZahlg(id));

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturFristablaufInArbeitJGB()
  {
    BfakturDAO  dao = new BfakturDAO();
    int         id  = 5000010;                              // test data

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturFristablaufInArbeitJGB:");

      // set up
      update( "UPDATE bfaktur SET kz_frist_abgelaufen=0,version=version+1 WHERE bfakt_id=" + id);
      assertEquals(0,selectKzFristAbgelaufen(id));

      // test
      dao.updateBfakturFristablaufInArbeitJGB(id);
      assertEquals(2,selectKzFristAbgelaufen(id));

      // tear down
      update( "UPDATE bfaktur SET kz_frist_abgelaufen=3,version=version+1 WHERE bfakt_id=" + id);
      assertEquals(3,selectKzFristAbgelaufen(id));

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturBezahlt()
  {
    BfakturDAO          dao     = new BfakturDAO();
    SAPRechnungStatusVO voOrig  = null;
    SAPRechnungStatusVO voTest  = null;              // test data
    SAPRechnungStatusVO voCheck = null;
    int                 id1     = 5000003;           // - ohne Buchdatum
    int                 id2     = 5000007;           // - mit Buchdatum

    try
    {
      //
      // test 1
      //
      System.out.println();
      System.out.println("testUpdateBfakturBezahlt: Test 1");

      // set up
      voOrig = selectSAPRechnungStatusVO(id1);

      voTest = new SAPRechnungStatusVO();
      voTest.bfakt_id       = id1;
      voTest.status_zahlg   = "4";
      voTest.fakt_betrag    = new BigDecimal(100.00);
      voTest.status_datum   = new java.util.Date();

      // test
      dao.updateBfakturBezahlt(voTest);

      voCheck = selectSAPRechnungStatusVO(id1);
      assertEquals(voTest.bfakt_id,voCheck.bfakt_id);
      assertEquals(voTest.status_zahlg,voCheck.status_zahlg);
      assertEquals(voTest.fakt_betrag.intValue(),voCheck.fakt_betrag.intValue());
      assertNotNull(voCheck.status_datum);
      assertNull(voCheck.buch_datum);

      // tear down
      dao.updateBfakturBezahlt(voOrig);

      //
      // test 2
      //
      System.out.println();
      System.out.println("testUpdateBfakturBezahlt: Test 2");

      // set up
      voOrig = selectSAPRechnungStatusVO(id2);

      voTest = new SAPRechnungStatusVO();
      voTest.bfakt_id       = id2;
      voTest.status_zahlg   = "4";
      voTest.fakt_betrag    = new BigDecimal(200.00);
      voTest.status_datum   = new java.util.Date();
      voTest.buch_datum     = new java.util.Date();

      // test
      dao.updateBfakturBezahlt(voTest);

      voCheck = selectSAPRechnungStatusVO(id2);
      assertEquals(voTest.bfakt_id,voCheck.bfakt_id);
      assertEquals(voTest.status_zahlg,voCheck.status_zahlg);
      assertEquals(voTest.fakt_betrag.intValue(),voCheck.fakt_betrag.intValue());
      assertNotNull(voCheck.status_datum);
      assertNotNull(voCheck.buch_datum);

      // tear down
      dao.updateBfakturBezahlt(voOrig);

    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturAnnuliert()
  {
    BfakturDAO            dao = new BfakturDAO();
    int                   id  = 5000014;                        // test data
    SAPRechnungStatusVO   vo  = new SAPRechnungStatusVO();

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturAnnuliert:");

      // set up
      update( "UPDATE bfaktur SET status_zahlg='B',version=version+1 WHERE bfakt_id=" + id);
      assertEquals("B",selectStatusZahlg(id));

      vo.bfakt_id = id;
      vo.status_zahlg = "9";

      // test
      dao.updateBfakturAnnuliert(vo);
      assertEquals("9",selectStatusZahlg(id));

      // tear down
      // nicht nötig, da test mit einer annulierten Rechnung (zahlg_stat = 9)
      // gemacht werden muss. d.h. nach dem test hat die Rechnung wieder den
      // korrekten Status
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturAnnuliertJGB()
  {
    BfakturDAO            dao = new BfakturDAO();
    int                   id  = 5000133;                        // test data
    SAPRechnungStatusVO   vo  = new SAPRechnungStatusVO();

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturAnnuliertJGB:");

      // set up
      update( "UPDATE bfaktur SET kz_frist_abgelaufen=2,version=version+1 WHERE bfakt_id=" + id);
      assertEquals(2,selectKzFristAbgelaufen(id));

      vo.bfakt_id = id;

      // test
      dao.updateBfakturAnnuliertJGB(vo);
      assertEquals(3,selectKzFristAbgelaufen(id));

      // tear down
      // nicht nötig, da test mit einer annulierten JGB Rechnung  gemacht
      // werden muss. d.h. nach dem test hat die Rechnung wieder den korrekten Status
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturStorniert()
  {
    BfakturDAO          dao     = new BfakturDAO();
    SAPRechnungStatusVO voOrig  = null;
    SAPRechnungStatusVO voTest  = null;              // test data
    int                 id      = 5000016;

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturStorniert:");

      // set up
      voOrig = selectSAPRechnungStatusVO(id);

      voTest = new SAPRechnungStatusVO();
      voTest.bfakt_id       = id;
      voTest.status_zahlg   = "2";
      voTest.status_datum   = new java.util.Date();

      // test
      dao.updateBfakturStorniert(voTest);
      assertEquals(voTest.status_zahlg, selectStatusZahlg(id));
      assertNotNull(selectStornoDatum(id));

      // tear down
      dao.updateBfakturBezahlt(voOrig);
      selectStatusZahlg(id);
      selectStornoDatum(id);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturGedruckt()
  {
    BfakturDAO          dao     = new BfakturDAO();
    SAPRechnungStatusVO voTest  = null;              // test data
    SAPRechnungStatusVO voCheck = null;              // test data
    int                 id      = 5000017;

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturStorniert:");

      // set up
      update( "UPDATE bfaktur SET status_druck='1',bfakt_dat=null,version=version+1 WHERE bfakt_id=" + id);
      voTest = new SAPRechnungStatusVO();
      voTest.bfakt_id       = id;
      voTest.status_druck   = "9";
      voTest.bfakt_dat      = new java.util.Date();

      // test
      dao.updateBfakturGedruckt(voTest);

      voCheck = selectSAPRechnungStatusVO(id);
      assertEquals(voTest.status_druck,voCheck.status_druck);
      assertNotNull(voCheck.bfakt_dat);

      // tear down
      // test hat wieder den original druckstatus gesetzt
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturGedrucktJGB()
  {
    BfakturDAO          dao     = new BfakturDAO();
    SAPRechnungStatusVO voTest  = null;              // test data
    SAPRechnungStatusVO voCheck = null;              // test data
    int                 id      = 5000017;

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturGedrucktJGB:");

      // set up
      update( "UPDATE bfaktur SET status_druck='1',version=version+1 WHERE bfakt_id=" + id);

      voTest = new SAPRechnungStatusVO();
      voTest.bfakt_id       = id;
      voTest.status_druck   = "9";

      // test
      dao.updateBfakturGedrucktJGB(voTest);

      voCheck = selectSAPRechnungStatusVO(id);
      assertEquals(voTest.status_druck,voCheck.status_druck);

      // tear down
      // test hat wieder den original druckstatus gesetzt
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateBfakturDruckstatus()
  {
    BfakturDAO  dao           = new BfakturDAO();

    int         id            = 5000007;                      // test data
    String      stateTests[]  = {"0","1","2","3","9"};
    String      stateOrig     = null;

    try
    {
      System.out.println();
      System.out.println("testUpdateBfakturDruckstatus:");

      // set up
      stateOrig = selectStatusDruck(id);

      // test
      for (int i = 0; i < stateTests.length; i++)
      {
        dao.updateBfakturDruckstatus(id,stateTests[i]);
        assertEquals(selectStatusDruck(id),stateTests[i]);
      }

      // tear down
      dao.updateBfakturDruckstatus(id,stateOrig);
      assertEquals(selectStatusDruck(id),stateOrig);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  //---------------------------------------------------------------------------
  // private methods - helper
  //---------------------------------------------------------------------------

  private String selectStatusDruck(int bfaktId) throws SQLException
  {
    Connection  con     = null;
    Statement   stmt    = null;
    ResultSet   rs      = null;
    String      result  = null;

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select * from bfaktur where bfakt_id=" + bfaktId);

      if (rs.next())
      {
        result = rs.getString("status_druck");
      }

      System.out.println("  status druck (bfakt_id=" + bfaktId + ") = " + result);
      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  private String selectStatusZahlg(int bfaktId) throws SQLException
  {
    Connection  con     = null;
    Statement   stmt    = null;
    ResultSet   rs      = null;
    String      result  = null;

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select * from bfaktur where bfakt_id=" + bfaktId);

      if (rs.next())
      {
        result = rs.getString("status_zahlg");
      }

      System.out.println("  status zahlg (bfakt_id=" + bfaktId + ") = " + result);
      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  private int selectKzFristAbgelaufen(int bfaktId) throws SQLException
  {
    Connection  con     = null;
    Statement   stmt    = null;
    ResultSet   rs      = null;
    int         result  = -1;

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select * from bfaktur where bfakt_id=" + bfaktId);

      if (rs.next())
      {
        result = rs.getInt("kz_frist_abgelaufen");
      }

      System.out.println("  kz_frist_abgelaufen (bfakt_id=" + bfaktId + ") = " + result);
      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  private java.util.Date selectStornoDatum(int bfaktId) throws SQLException
  {
    Connection      con     = null;
    Statement       stmt    = null;
    ResultSet       rs      = null;
    java.util.Date  result  = null;

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select * from bfaktur where bfakt_id=" + bfaktId);

      if (rs.next())
      {
        result = rs.getDate("storno_datum");
      }

      System.out.println("  storno_datum (bfakt_id=" + bfaktId + ") = " + result);
      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  private SAPRechnungStatusVO selectSAPRechnungStatusVO(int bfaktId) throws SQLException
  {
    Connection          con     = null;
    Statement           stmt    = null;
    ResultSet           rs      = null;
    SAPRechnungStatusVO result  = null;

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select * from bfaktur where bfakt_id=" + bfaktId);

      if (rs.next())
      {
        result = new SAPRechnungStatusVO();

        result.bfakt_id     = bfaktId;
        // status druck
        result.status_druck = rs.getString ("status_druck");
        result.bfakt_dat    = rs.getDate   ("bfakt_dat");
        // status zahlg
        result.status_zahlg = rs.getString ("status_zahlg");
        result.status_datum = rs.getDate   ("zahl_dat");
        result.buch_datum   = rs.getDate   ("buch_dat");
        result.fakt_betrag  = rs.getBigDecimal ("fakt_betrag");
        result.waehr_cd     = rs.getString ("waehr_cd");
        result.kz_kalk      = rs.getString ("kz_kalk");
      }

      System.out.println("SAPRechnungStatusVO (bfakt_id=" + bfaktId + "):\n" + result.toStringValidDataOnly());
      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  private int update(String stmt) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = new BagisDAO().getConnection();

      ps = con.prepareStatement(stmt);
      return ps.executeUpdate();

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


} // end class
