package ch.ige.ses.server.dao.bagis.test;

import java.util.ArrayList;
import junit.framework.*;

import ch.ige.ses.server.dao.bagis.BformtxtDruckDAO;
import ch.ige.ses.vo.bagis.BformtxtDruckVO;
import java.util.*;


/**
 * Test der Klasse BformtxtDruckDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BformtxtDruckDAOTest.java,v 1.9 2007/01/04 15:58:08 iten Exp $
 */
public class BformtxtDruckDAOTest extends BagisTestCase
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public BformtxtDruckDAOTest(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(BformtxtDruckDAOTest.class);
  }

  protected void setUp()
  {
    // lazy bone
  }


  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------


  public void testSelectByFormartCdTextartSprachCd()
  {
    BformtxtDruckDAO  dao = new BformtxtDruckDAO();
    BformtxtDruckVO   res = null;

    try
    {
      System.out.println();
      System.out.println("testSelect by formartCd, textart, sprachCd:");

      // test 1
      res = dao.select("105", "0", "D");
      System.out.println("  test #1 - dao.select(\"105\", \"0\", \"D\")");
      System.out.println("  query = " + dao.getQuery());
      System.out.println(res.toString());
      assertEquals(res.text,"Abteilung Patente");

      // test 2
      res = dao.select("xxx", "y", "z");
      System.out.println("  test #2 - dao.select(\"xxx\", \"y\", \"z\")");
      System.out.println("  query = " + dao.getQuery());
      assertEquals(res,null);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectByFormartCdSprachCd()
  {
    BformtxtDruckDAO  dao = new BformtxtDruckDAO();
    BformtxtDruckVO   vo  = null;
    ArrayList         res = null;
    int               i   = 0;

    try
    {
      System.out.println();
      System.out.println("testSelect by formartCd, sprachCd:");

      res = dao.select("105", "D");

      System.out.println("Element Size = " + res.size());
      for (Iterator iter = res.iterator(); iter.hasNext(); )
      {
        vo = (BformtxtDruckVO)iter.next();
        System.out.println("Element #" + i + ":\n" + vo.toString());
        i++;
      }
      assertEquals(res.size(),3);

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectByFormartCd()
  {
    BformtxtDruckDAO  dao = new BformtxtDruckDAO();
    BformtxtDruckVO   vo  = null;
    ArrayList         res = null;
    int               i   = 0;

    try
    {
      System.out.println();
      System.out.println("testSelect by formartCd:");

      res = dao.select("105");

      System.out.println("Element Size = " + res.size());
      for (Iterator iter = res.iterator(); iter.hasNext(); )
      {
        vo = (BformtxtDruckVO)iter.next();
        System.out.println("Element #" + i + ":\n" + vo.toString());
        i++;
      }
      assertEquals(res.size(),9);

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testSelectForJGB()
  {
    BformtxtDruckDAO  dao = new BformtxtDruckDAO();
    ArrayList         res = null;

    try
    {
      System.out.println();
      System.out.println("testSelectForJGB:");

      res = dao.selectForJGB();
      System.out.println("  query = " + dao.getQuery());
      System.out.println("  element size = " + res.size());
      assertTrue(res.size() > 200);

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testInsertDelete()
  {
    BformtxtDruckDAO  dao = new BformtxtDruckDAO();
    BformtxtDruckVO   vo  = new BformtxtDruckVO();
    BformtxtDruckVO   res = null;

    try
    {
      System.out.println();
      System.out.println("testInsertDelete:");

      // set up
      vo.textart    = "y";
      vo.formartCd  = "TST";
      vo.sprachCd   = "D";
      vo.text       = "JUnit test data record";

      // test insert
      dao.insert(vo);
      res = dao.select(vo.formartCd,vo.textart,vo.sprachCd);
      System.out.println("  result of insert:");
      System.out.println(res.toString());

      assertTrue(vo.textart.equals  (res.textart));
      assertTrue(vo.formartCd.equals(res.formartCd));
      assertTrue(vo.sprachCd.equals (res.sprachCd));
      assertTrue(vo.text.equals     (res.text));



      // test delete (tear down insert)
      dao.delete(vo.formartCd,vo.textart,vo.sprachCd);
      res = dao.select(vo.formartCd,vo.textart,vo.sprachCd);
      System.out.println("  result of delete:");
      System.out.println(res == null ? "  null" : res.toString() );
      assertNull(res);

    }
    catch(Exception e)
    {
      System.out.println("  query = " + dao.getQuery());
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void testUpdateDelete()
  {
    BformtxtDruckDAO  dao = new BformtxtDruckDAO();
    BformtxtDruckVO   vo  = new BformtxtDruckVO();
    BformtxtDruckVO   res = null;
    String            t1  = "JUnit test data record";
    String            t2  = "JUnit test data record updated";


    try
    {
      System.out.println();
      System.out.println("testUpdateDelete:");

      // set up
      vo.textart    = "y";
      vo.formartCd  = "TST";
      vo.sprachCd   = "D";
      vo.text       = t1;

      if (dao.select(vo.formartCd,vo.textart,vo.sprachCd) == null)
      {
        dao.insert(vo);
      }
      vo = dao.select(vo.formartCd,vo.textart,vo.sprachCd);

      System.out.println(vo.toString());

      // test
      vo.text = t2;
      vo.version++;
      dao.update(vo);

      res = dao.select(vo.formartCd,vo.textart,vo.sprachCd);
      System.out.println(res.toString());
      assertEquals(vo.formartCd,res.formartCd);
      assertEquals(vo.textart,res.textart);
      assertEquals(vo.sprachCd,res.sprachCd);
      assertEquals(vo.text,res.text);

      // tear down
      dao.delete(vo.formartCd,vo.textart,vo.sprachCd);
      res = dao.select(vo.formartCd,vo.textart,vo.sprachCd);
      assertNull(res);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


} // end class
