package ch.ige.ses.server.dao.bagis.test;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import junit.framework.Test;
import junit.framework.TestSuite;
import ch.ige.ses.server.dao.bagis.BagisDAO;
import ch.ige.ses.server.dao.bagis.JoinSAPRechnungDAO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.util.server.DBHelper;

/**
 * Generate test data for class JoinSAPRechnungDAOTest
 * <p>
 * Copyright (c) 2005, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Thomas Iten
 * @version $Id: JoinSAPRechnungDAOTestHelper.java,v 1.16 2008/04/07 06:45:25 iten Exp $
 */

public class JoinSAPRechnungDAOTestHelper extends BagisTestCase
{

  //---------------------------------------------------------------------------
  // test data
  //---------------------------------------------------------------------------

  /**
   * list with test bfakt_id's. this list is used to generate the bfaktStrings
   * test data definition
   */
  public static int[] bfaktIds = {
      4000006, 4646663, 4705891,
      5000003, 5000112, 5213386
  };


  /**
   * jgb test data range A
   */
  public static int[] bfaktJgbIdRangeA = { 5818918, 5818920};


  /**
   * jgb test data range B
   */
  public static int[] bfaktJgbIdRangeB = { 5818943, 5818945 };


  //---------------------------------------------------------------------------
  // generated test data
  // please copy the generated output into this class, if some definitions
  // within the SAPRechnungVO.toString() method changes.
  //---------------------------------------------------------------------------

  public static String[] bfaktStrings = {
    // bfakt_id : 4000006
    "#KOPFDATEN #  bfakt_id         = <4000006>#  bfakt_nr         = <0>#  bfakt_dat        = <2001-01-31" + 
    ">#  schutit_nr       = <00504823>#  auftraggeber     = <4705891>#  regulierer       = <0>#  hauptRec" + 
    "hEmpf    = <500444>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <0> #  " + 
    "sprach_cd        = <D> #  druck_art        = <3> #  waehrung         = <CHF> #  verfall_dat      = <" + 
    "2001-06-30>#  unsere_ref       = <PVerf ****/00504823> #  tel_nr           = <+41 31 322 49 09> #  i" + 
    "nhaber          = <YOSHITOMI PHARMACEUTICAL INDUSTRIES, LTD., Osaka-shi Osaka> #  kz_einschr       =" + 
    " <0> #  schutit_typ      = <2> #  schutit_typ_bez  = <EPP> #  ihr_zeichen      = <Hz/bmr 7155> #  va" + 
    "r_text         = <18.03.1992> #  diversText       = <> #  abteilung        = <Abteilung Patente> #  " + 
    "kopfText         = <<b>Jahresgebührenrechnung zum Patent/Patentgesuch Nr.: %%$$Schutit<br>lautend au" + 
    "f: %%$$Inhaber<br>Anmeldedatum: %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch wird am" + 
    " <b>%%$$EreigDat</b> folgende Gebühr fällig:> #  beilagenText     = <Wir bitten Sie, den obenerwähnt" + 
    "en Betrag bis zum <b>%%$$Verfall</b> mit beiliegendem Einzahlungsschein an uns zu überweisen, sofern" + 
    " er nicht einem Kontokorrent belastet werden soll.<br>Die Zahlungsfrist ist gewahrt, wenn der Betrag" + 
    " rechtzeitig zu Gunsten des Instituts der Schweizerischen Post übergeben oder einem Post- oder Bankk" + 
    "onto in der Schweiz belastet worden ist. Bei Nichtbezahlung innerhalb der obengenannten Frist kann d" + 
    "ie Jahresgebühr gegen Entrichtung eines Zuschlags noch während weiteren drei Monaten entrichtet werd" + 
    "en. Zu diesem Zweck erfolgt nach Ablauf der unbenützten Zahlungsfrist die Zustellung einer Mahnung f" + 
    "ür die Jahres- und Zuschlagsgebühr.<br><br>Mit freundlichen Grüssen<br>Abteilung Patente> #  Belehru" + 
    "ng        = <> #RECHNUNGSPOSITIONEN #  bfakt_id      = <4000006> #  bfakt_posnr   = <10> #  art_nr  " + 
    "      = <28570> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <420> #  artKurzt" + 
    "ext   = <10.Jahresgeb.EP m.Wirk.CH/LI> #  artText       = <> #  artZusatzText = <> #  schutit_key   " + 
    "= <2000504823> #  INTERNE FELDER #    wert        = <420> #    dat_ereignis= <2001-03-31> #    frist" + 
    "       = <2001-06-30> #    schutit_nr  = <00504823> #    createdDat  = null #    changedDat  = null " + 
    "##AMV ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZU" + 
    "NGEN#  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #    null #  jgbZusatzRechEmpfPa" + 
    "ram: #    keine #INTERNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faellig_d" + 
    "at         = <2001-03-31>#  titel               = <Verfahren zur Herstellung von menschlichem Serum " + 
    "Albumin> #  formart_cd          = <290> #  schutz_beginn       = <1992-03-18>#  status_verarbeitung " + 
    "= <0> #  zahl_dat            = null #  anz_kopie           = <0> #  fakt_betrag         = null #  ha" + 
    "sChanged          = <0> #  created_dat         = null #  changed_dat         = null #  error_source " + 
    "       = <> #  error_description   = <> #  zusatzRechEmpfNichtVerarbeitet = keine#",
    // bfakt_id : 4646663
    "#KOPFDATEN #  bfakt_id         = <4646663>#  bfakt_nr         = <0>#  bfakt_dat        = <2003-01-14" + 
    ">#  schutit_nr       = <08815/2002>#  auftraggeber     = <4742681>#  regulierer       = <0>#  hauptR" + 
    "echEmpf    = <4742683>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <123" + 
    "0> #  sprach_cd        = <F> #  druck_art        = <1> #  waehrung         = <CHF> #  verfall_dat   " + 
    "   = <2003-03-17>#  unsere_ref       = <MA-Prüf4 lbj/08815/2002> #  tel_nr           = <+41 31 322 4" + 
    "9 29> #  inhaber          = <INFORMATIQUE BANCAIRE ISYS SA, 1227 Carouge GE> #  kz_einschr       = <" + 
    "0> #  schutit_typ      = <4> #  schutit_typ_bez  = <MCH> #  ihr_zeichen      = <André Malek-Asghar> " + 
    "#  var_text         = <BEST Retail> #  diversText       = <> #  abteilung        = <Keine TEXTART = " + 
    "0 in BFORMTXT_DRUCK für Formular gefunden!> #  kopfText         = <Keine TEXTART 1 od. 2 in BFORMTXT" + 
    "_DRUCK für Formular gefunden!> #  beilagenText     = <> #  Belehrung        = <> #RECHNUNGSPOSITIONE" + 
    "N #  bfakt_id      = <4646663> #  bfakt_posnr   = <10> #  art_nr        = <40570> #  lageinh       =" + 
    " <ANZ> #  anzahl        = <1> #  preis         = <200> #  artKurztext   = <Verf. Hinterlegungsgebühr" + 
    ", verspätet> #  artText       = <> #  artZusatzText = <> #  schutit_key   = <1200783643> #  INTERNE " + 
    "FELDER #    wert        = <200> #    dat_ereignis= <2003-01-14> #    frist       = <2003-03-17> #   " + 
    " schutit_nr  = <08815/2002> #    createdDat  = null #    changedDat  = null ##AMV ERGAENZUNGEN#  amv" + 
    "LaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#  jgbLaufNr = <> # " + 
    " jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #    null #  jgbZusatzRechEmpfParam: #    keine #INTERNE" + 
    " FELDER #  kto_nr              = <> #  unt_name            = <Jolanda Leuenberger-Binggeli> #  faell" + 
    "ig_dat         = <2003-01-14>#  titel               = <ISYS BEST Retail> #  formart_cd          = <4" + 
    "05> #  schutz_beginn       = <2003-05-13>#  status_verarbeitung = <0> #  zahl_dat            = null " + 
    "#  anz_kopie           = <0> #  fakt_betrag         = null #  hasChanged          = <0> #  created_d" + 
    "at         = null #  changed_dat         = null #  error_source        = <> #  error_description   =" + 
    " <> #  zusatzRechEmpfNichtVerarbeitet = keine#",
    // bfakt_id : 4705891
    "#KOPFDATEN #  bfakt_id         = <4705891>#  bfakt_nr         = <0>#  bfakt_dat        = <2003-03-31" + 
    ">#  schutit_nr       = <00877190>#  auftraggeber     = <4589321>#  regulierer       = <0>#  hauptRec" + 
    "hEmpf    = <500902>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <0> #  " + 
    "sprach_cd        = <D> #  druck_art        = <3> #  waehrung         = <CHF> #  verfall_dat      = <" + 
    "2003-08-31>#  unsere_ref       = <PAdmin ****/00877190> #  tel_nr           = <+41 31 322 49 46> #  " + 
    "inhaber          = <ALIA Engineering AG, 6300 Zug> #  kz_einschr       = <0> #  schutit_typ      = <" + 
    "2> #  schutit_typ_bez  = <EPP> #  ihr_zeichen      = <282S/TR/CH> #  var_text         = <02.05.1998>" + 
    " #  diversText       = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <<b>Jahres" + 
    "gebührenrechnung zum Patent/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$Inhaber<br>Anmeldedatu" + 
    "m: %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch wird am <b>%%$$EreigDat</b> folgende" + 
    " Gebühr fällig:> #  beilagenText     = <Wir bitten Sie, den obenerwähnten Betrag bis zum <b>%%$$Verf" + 
    "all</b> mit beiliegendem Einzahlungsschein an uns zu überweisen, sofern er nicht einem Kontokorrent " + 
    "belastet werden soll.<br>Die Zahlungsfrist ist gewahrt, wenn der Betrag rechtzeitig zu Gunsten des I" + 
    "nstituts der Schweizerischen Post übergeben oder einem Post- oder Bankkonto in der Schweiz belastet " + 
    "worden ist. Bei Nichtbezahlung innerhalb der obengenannten Frist kann die Jahresgebühr gegen Entrich" + 
    "tung eines Zuschlags noch während weiteren drei Monaten entrichtet werden. Zu diesem Zweck erfolgt n" + 
    "ach Ablauf der unbenützten Zahlungsfrist die Zustellung einer Mahnung für die Jahres- und Zuschlagsg" + 
    "ebühr.<br><br>Mit freundlichen Grüssen<br>Abteilung Patente> #  Belehrung        = <> #RECHNUNGSPOSI" + 
    "TIONEN #  bfakt_id      = <4705891> #  bfakt_posnr   = <10> #  art_nr        = <28530> #  lageinh   " + 
    "    = <ANZ> #  anzahl        = <1> #  preis         = <420> #  artKurztext   = <6.Jahresgeb.EP m.Wir" + 
    "k.CH/LI> #  artText       = <> #  artZusatzText = <> #  schutit_key   = <2000877190> #  INTERNE FELD" + 
    "ER #    wert        = <420> #    dat_ereignis= <2003-05-31> #    frist       = <2003-08-31> #    sch" + 
    "utit_nr  = <00877190> #    createdDat  = null #    changedDat  = null ##AMV ERGAENZUNGEN#  amvLaufId" + 
    "     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#  jgbLaufNr = <> #  jgbLa" + 
    "ufID = <-1> #  jgbHauptRechEmpfParam: #    null #  jgbZusatzRechEmpfParam: #    keine #INTERNE FELDE" + 
    "R #  kto_nr              = <> #  unt_name            = <> #  faellig_dat         = <2003-05-31>#  ti" + 
    "tel               = <Rohrkupplung> #  formart_cd          = <290> #  schutz_beginn       = <1998-05-" + 
    "02>#  status_verarbeitung = <0> #  zahl_dat            = null #  anz_kopie           = <0> #  fakt_b" + 
    "etrag         = null #  hasChanged          = <0> #  created_dat         = null #  changed_dat      " + 
    "   = null #  error_source        = <> #  error_description   = <> #  zusatzRechEmpfNichtVerarbeitet " + 
    "= keine#",
    // bfakt_id : 5000003
    "#KOPFDATEN #  bfakt_id         = <5000003>#  bfakt_nr         = <0>#  bfakt_dat        = <2003-07-03" + 
    ">#  schutit_nr       = <2003-00488>#  auftraggeber     = <500520>#  regulierer       = <500520>#  ha" + 
    "uptRechEmpf    = <500520>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <" + 
    "1023> #  sprach_cd        = <D> #  druck_art        = <1> #  waehrung         = <CHF> #  verfall_dat" + 
    "      = <2003-08-04>#  unsere_ref       = <Des/Top mg/2003-00488> #  tel_nr           = <+41 31 322 " + 
    "49 65> #  inhaber          = <Urs Villiger, 8309 Nürensdorf> #  kz_einschr       = <0> #  schutit_ty" + 
    "p      = <6> #  schutit_typ_bez  = <DES> #  ihr_zeichen      = <V67-M3-CH> #  var_text         = <> " + 
    "#  diversText       = <> #  abteilung        = <Designs> #  kopfText         = <<b>Rechnung zu Eintr" + 
    "agungsgesuch<br>von: %%$$Inhaber<br>Bez: %%$$Titel</b><br><br>Für Ihr Eintragungsgesuch sind folgend" + 
    "e Kosten entstanden:> #  beilagenText     = <Auftragsgemäss werden wir den Rechnungsbetrag Ihrem Kon" + 
    "tokorrent belasten.<br><br>Mit freundlichen Grüssen<br>%%$$Untersch> #  Belehrung        = <> #RECHN" + 
    "UNGSPOSITIONEN #  bfakt_id      = <5000003> #  bfakt_posnr   = <10> #  art_nr        = <60210> #  la" + 
    "geinh       = <ANZ> #  anzahl        = <1> #  preis         = <200> #  artKurztext   = <Grundgebühr " + 
    "1 Gegenstand> #  artText       = <> #  artZusatzText = <Grundgebühr für ein Design<br>> #  schutit_k" + 
    "ey   = <1200907843> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-07-03> #    fr" + 
    "ist       = <2003-08-04> #    schutit_nr  = <2003-00488> #    createdDat  = null #    changedDat  = " + 
    "null ##  bfakt_id      = <5000003> #  bfakt_posnr   = <20> #  art_nr        = <60300> #  lageinh    " + 
    "   = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Publikationsgrundgebüh" + 
    "r s/w> #  artText       = <> #  artZusatzText = <Publikationsgebühr<br>für die ersten drei Abbildung" + 
    "en (schwarzweiss) des Designs<br>> #  schutit_key   = <1200907843> #  INTERNE FELDER #    wert      " + 
    "  = <0> #    dat_ereignis= <2003-07-03> #    frist       = <2003-08-04> #    schutit_nr  = <2003-004" + 
    "88> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5000003> #  bfakt_posnr   =" + 
    " <30> #  art_nr        = <60320> #  lageinh       = <ANZ> #  anzahl        = <3> #  preis         = " + 
    "<20> #  artKurztext   = <Publ.grundgebühr pro w. Bild s/w> #  artText       = <> #  artZusatzText = " + 
    "<Publikationsgebühr à<br>für jede weitere Abbildung (schwarzweiss)<br>> #  schutit_key   = <12009078" + 
    "43> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-07-03> #    frist       = <200" + 
    "3-08-04> #    schutit_nr  = <2003-00488> #    createdDat  = null #    changedDat  = null ##AMV ERGAE" + 
    "NZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#  jgbL" + 
    "aufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #    null #  jgbZusatzRechEmpfParam: #    k" + 
    "eine #INTERNE FELDER #  kto_nr              = <500520> #  unt_name            = <Marianne Bekasi> # " + 
    " faellig_dat         = <2003-07-03>#  titel               = <Brotkiste> #  formart_cd          = <65" + 
    "0> #  schutz_beginn       = <2003-06-27>#  status_verarbeitung = <0> #  zahl_dat            = null #" + 
    "  anz_kopie           = <0> #  fakt_betrag         = null #  hasChanged          = <0> #  created_da" + 
    "t         = null #  changed_dat         = null #  error_source        = <> #  error_description   = " + 
    "<> #  zusatzRechEmpfNichtVerarbeitet = keine#",
    // bfakt_id : 5000112
    "#KOPFDATEN #  bfakt_id         = <5000112>#  bfakt_nr         = <0>#  bfakt_dat        = <2003-07-03" + 
    ">#  schutit_nr       = <P-317403>#  auftraggeber     = <4650179>#  regulierer       = <4650179>#  ha" + 
    "uptRechEmpf    = <4650179>#  zusatzRechEmpf   = keine#  kontaktPerson    = <Christine Repond> #  emp" + 
    "loyeeID       = <1365> #  sprach_cd        = <F> #  druck_art        = <1> #  waehrung         = <CH" + 
    "F> #  verfall_dat      = <2003-09-04>#  unsere_ref       = <MAREG xs/ P-317403> #  tel_nr           " + 
    "= <+41 31 322 48 68> #  inhaber          = <Worldwide Brands, Inc., Wilmington (DE)> #  kz_einschr  " + 
    "     = <0> #  schutit_typ      = <4> #  schutit_typ_bez  = <MCH> #  ihr_zeichen      = <> #  var_tex" + 
    "t         = <CAMEL ((dromadaire))> #  diversText       = <Titres de protection concernés:  317403, 3" + 
    "33701, 354052, 355280, 358505, 358799, 358831, 367286, 367437, 367438, 368727, 369899, 373215, 37327" + 
    "9, 374180, 380245, 393234, 393235, 397587, 413498, 417306, 424681<br><br>> #  abteilung        = <Di" + 
    "vision des marques> #  kopfText         = <<b>Facture pour modification des inscriptions sur le regi" + 
    "stre des marques<br>marque n° %%$$Schutit</b><br><br>> #  beilagenText     = <%%$$Divers<br>> #  Bel" + 
    "ehrung        = <<b>MISE EN GARDE</b><br><br>Le paiement est réputé effectué lorsqu'il est porté au " + 
    "crédit d'un compte de l'Institut Fédéral de la Propriété Intellectuelle. Le délai de paiement est ob" + 
    "servé si, avant son échéance, la somme due est versée à La Poste Suisse ou débitée en Suisse d'un co" + 
    "mpte postal ou bancaire en faveur de l'Institut (art. 6 IPI-RT).> #RECHNUNGSPOSITIONEN #  bfakt_id  " + 
    "    = <5000112> #  bfakt_posnr   = <10> #  art_nr        = <44273> #  lageinh       = <ANZ> #  anzah" + 
    "l        = <1> #  preis         = <100> #  artKurztext   = <Vertreteränderung Rng> #  artText       " + 
    "= <> #  artZusatzText = <Mise à jour du registre des marques<br><b>Changement de mandataire (nouveau" + 
    " mandataire):</b><br>WORLDWIDE BRANDS, INC.<br>12-14, Chemin Rieu<br>1211 Genève 17<br>> #  schutit_" + 
    "key   = <1200050408> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    f" + 
    "rist       = <2003-09-04> #    schutit_nr  = <P-317403> #    createdDat  = null #    changedDat  = n" + 
    "ull ##  bfakt_id      = <5000112> #  bfakt_posnr   = <20> #  art_nr        = <44273> #  lageinh     " + 
    "  = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> " + 
    "#  artText       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit" + 
    "_key   = <1200023298> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    " + 
    "frist       = <2003-09-04> #    schutit_nr  = <333701> #    createdDat  = null #    changedDat  = nu" + 
    "ll ##  bfakt_id      = <5000112> #  bfakt_posnr   = <30> #  art_nr        = <44273> #  lageinh      " + 
    " = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #" + 
    "  artText       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_" + 
    "key   = <1100354052> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    f" + 
    "rist       = <2003-09-04> #    schutit_nr  = <354052> #    createdDat  = null #    changedDat  = nul" + 
    "l ##  bfakt_id      = <5000112> #  bfakt_posnr   = <40> #  art_nr        = <44273> #  lageinh       " + 
    "= <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> # " + 
    " artText       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_k" + 
    "ey   = <1100355280> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    fr" + 
    "ist       = <2003-09-04> #    schutit_nr  = <355280> #    createdDat  = null #    changedDat  = null" + 
    " ##  bfakt_id      = <5000112> #  bfakt_posnr   = <50> #  art_nr        = <44273> #  lageinh       =" + 
    " <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  " + 
    "artText       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_ke" + 
    "y   = <1100358505> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    fri" + 
    "st       = <2003-09-04> #    schutit_nr  = <358505> #    createdDat  = null #    changedDat  = null " + 
    "##  bfakt_id      = <5000112> #  bfakt_posnr   = <60> #  art_nr        = <44273> #  lageinh       = " + 
    "<ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  a" + 
    "rtText       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key" + 
    "   = <1100358799> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    fris" + 
    "t       = <2003-09-04> #    schutit_nr  = <358799> #    createdDat  = null #    changedDat  = null #" + 
    "#  bfakt_id      = <5000112> #  bfakt_posnr   = <70> #  art_nr        = <44273> #  lageinh       = <" + 
    "ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  ar" + 
    "tText       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key " + 
    "  = <1100358831> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist" + 
    "       = <2003-09-04> #    schutit_nr  = <358831> #    createdDat  = null #    changedDat  = null ##" + 
    "  bfakt_id      = <5000112> #  bfakt_posnr   = <80> #  art_nr        = <44273> #  lageinh       = <A" + 
    "NZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  art" + 
    "Text       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key  " + 
    " = <1100367286> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist " + 
    "      = <2003-09-04> #    schutit_nr  = <367286> #    createdDat  = null #    changedDat  = null ## " + 
    " bfakt_id      = <5000112> #  bfakt_posnr   = <90> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100367437> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <367437> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <100> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100367438> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <367438> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <110> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100368727> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <368727> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <120> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100369899> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <369899> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <130> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100373215> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <373215> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <140> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100373279> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <373279> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <150> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100374180> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <374180> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <160> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100380245> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <380245> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <170> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100393234> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <393234> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <180> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100393235> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <393235> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <190> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100397587> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <397587> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <200> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100413498> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <413498> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <210> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <1100417306> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist  " + 
    "     = <2003-09-04> #    schutit_nr  = <417306> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5000112> #  bfakt_posnr   = <220> #  art_nr        = <44273> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Vertreteränderung Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Même(s) modification(s) au prix réduit<br><br>> #  schutit_key   " + 
    "= <112371995> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2003-05-23> #    frist   " + 
    "    = <2003-09-04> #    schutit_nr  = <424681> #    createdDat  = null #    changedDat  = null ##AMV" + 
    " ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#" + 
    "  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #    null #  jgbZusatzRechEmpfParam: " + 
    "#    keine #INTERNE FELDER #  kto_nr              = <4650179> #  unt_name            = <Xenia Abdin-" + 
    "Stoller> #  faellig_dat         = <2003-05-23>#  titel               = <CAMEL ((fig.))> #  formart_c" + 
    "d          = <436> #  schutz_beginn       = <1980-11-27>#  status_verarbeitung = <0> #  zahl_dat    " + 
    "        = null #  anz_kopie           = <0> #  fakt_betrag         = null #  hasChanged          = <" + 
    "0> #  created_dat         = null #  changed_dat         = null #  error_source        = <> #  error_" + 
    "description   = <> #  zusatzRechEmpfNichtVerarbeitet = keine#",
    // bfakt_id : 5213386
    "#KOPFDATEN #  bfakt_id         = <5213386>#  bfakt_nr         = <0>#  bfakt_dat        = <2004-03-03" + 
    ">#  schutit_nr       = <2P-266091>#  auftraggeber     = <6022843>#  regulierer       = <0>#  hauptRe" + 
    "chEmpf    = <4627720>#  zusatzRechEmpf   = <6022843> #  kontaktPerson    = <Alexandra Salicunaj> #  " + 
    "employeeID       = <1365> #  sprach_cd        = <D> #  druck_art        = <1> #  waehrung         = " + 
    "<CHF> #  verfall_dat      = <2004-05-04>#  unsere_ref       = <MAREG xs/2P-266091> #  tel_nr        " + 
    "   = <+41 31 322 48 68> #  inhaber          = <Medika AG, 4147 Aesch BL> #  kz_einschr       = <0> #" + 
    "  schutit_typ      = <4> #  schutit_typ_bez  = <MCH> #  ihr_zeichen      = <as> #  var_text         " + 
    "= <MALISTREN> #  diversText       = <Betroffene Schutztitel:  266091, 268192, 269761, 272525, 273071" + 
    ", 273104, 274171, 275496, 275497, 276421, 276786, 277886, 277887, 278315, 296091, 298516, 298517, 29" + 
    "9137, 299138, 303775, 305597, 307155, 307156, 307462, 307790, 308353, 308791, 308895, 309176, 310525" + 
    ", 312568, 312569, 312570, 312571, 313813, 314317, 314973, 315490, 315838, 315839, 317901, 318597, 31" + 
    "8598, 318599, 318600, 318602, 320310, 320540, 320541, 322569, 323593, 323594, 323595, 323596, 323608" + 
    ", 327520, 327572, 329111, 332006, 334496, 334497, 336415, 336416, 338821, 339778, 339779, 343285, 34" + 
    "3680, 343681, 344224, 345000, 345001, 346546, 346547, 347940, 348578, 348579, 348580, 348581, 348582" + 
    ", 351361, 352303, 352304, 354922, 360108, 360109, 360655, 361542, 367309, 368816, 368817, 368818, 37" + 
    "3174, 373175, 373176, 377363, 377364, 377365, 380654, 382624, 382932, 382933, 382934, 382935, 384508" + 
    ", 387958, 387959, 388990, 391088, 472715<br><br>> #  abteilung        = <Markenabteilung> #  kopfTex" + 
    "t         = <<b>Rechnung Aktualisierung Markenregister<br>Marke Nr. %%$$Schutit</b><br><br>> #  beil" + 
    "agenText     = <%%$$Divers<br><br>Wir bitten Sie, den Rechnungsbetrag bis zum <b>%%$$Verfall</b> mit" + 
    " beiliegendem Einzahlungsschein auf unser Konto einzubezahlen.> #  Belehrung        = <<b>ACHTUNG</b" + 
    "><br><br>Als Zahlungseingang gilt die Gutschrift auf einem Konto des Instituts. Die Zahlungsfrist is" + 
    "t gewahrt, wenn der Betrag rechtzeitig zu Gunsten des Instituts der Schweizerischen Post übergeben o" + 
    "der einem Post- oder Bankkonto in der Schweiz belastet worden ist (Art. 6 IGE-GebO).> #RECHNUNGSPOSI" + 
    "TIONEN #  bfakt_id      = <5213386> #  bfakt_posnr   = <10> #  art_nr        = <44223> #  lageinh   " + 
    "    = <ANZ> #  anzahl        = <1> #  preis         = <100> #  artKurztext   = <Adressänderung/Sitz " + 
    "Rng> #  artText       = <> #  artZusatzText = <Aktualisierung Markenregister<br><br><b>Adressänderun" + 
    "g</b><br><br>Medika AG<br>Industriestrasse 111<br>4147 Aesch BL<br>> #  schutit_key   = <1200099497>" + 
    " #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-0" + 
    "5-04> #    schutit_nr  = <2P-266091> #    createdDat  = null #    changedDat  = null ##  bfakt_id   " + 
    "   = <5213386> #  bfakt_posnr   = <20> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl" + 
    "        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       " + 
    "= <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <12000" + 
    "98632> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <" + 
    "2004-05-04> #    schutit_nr  = <2P-268192> #    createdDat  = null #    changedDat  = null ##  bfakt" + 
    "_id      = <5213386> #  bfakt_posnr   = <30> #  art_nr        = <44223> #  lageinh       = <ANZ> #  " + 
    "anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText " + 
    "      = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = " + 
    "<1200097492> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist    " + 
    "   = <2004-05-04> #    schutit_nr  = <P-269761> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5213386> #  bfakt_posnr   = <40> #  art_nr        = <44223> #  lageinh       = <ANZ" + 
    "> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  art" + 
    "Text       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key" + 
    "   = <1200096925> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    fris" + 
    "t       = <2004-05-04> #    schutit_nr  = <P-272525> #    createdDat  = null #    changedDat  = null" + 
    " ##  bfakt_id      = <5213386> #  bfakt_posnr   = <50> #  art_nr        = <44223> #  lageinh       =" + 
    " <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #" + 
    "  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schuti" + 
    "t_key   = <1200095702> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #   " + 
    " frist       = <2004-05-04> #    schutit_nr  = <P-273071> #    createdDat  = null #    changedDat  =" + 
    " null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <60> #  art_nr        = <44223> #  lageinh   " + 
    "    = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz R" + 
    "ng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  s" + 
    "chutit_key   = <1200095709> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16>" + 
    " #    frist       = <2004-05-04> #    schutit_nr  = <P-273104> #    createdDat  = null #    changedD" + 
    "at  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <70> #  art_nr        = <44223> #  lagei" + 
    "nh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/S" + 
    "itz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>>" + 
    " #  schutit_key   = <1200094547> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-0" + 
    "2-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-274171> #    createdDat  = null #    cha" + 
    "ngedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <80> #  art_nr        = <44223> #  " + 
    "lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänder" + 
    "ung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br>" + 
    "<br>> #  schutit_key   = <1200093674> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2" + 
    "004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-275496> #    createdDat  = null #  " + 
    "  changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <90> #  art_nr        = <44223" + 
    "> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adress" + 
    "änderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Prei" + 
    "s<br><br>> #  schutit_key   = <1200093675> #  INTERNE FELDER #    wert        = <0> #    dat_ereigni" + 
    "s= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-275497> #    createdDat  = nul" + 
    "l #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <100> #  art_nr        = " + 
    "<44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <" + 
    "Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduzierte" + 
    "m Preis<br><br>> #  schutit_key   = <1200083082> #  INTERNE FELDER #    wert        = <0> #    dat_e" + 
    "reignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-276421> #    createdDat " + 
    " = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <110> #  art_nr    " + 
    "    = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext" + 
    "   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu redu" + 
    "ziertem Preis<br><br>> #  schutit_key   = <1200083084> #  INTERNE FELDER #    wert        = <0> #   " + 
    " dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-276786> #    creat" + 
    "edDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <120> #  art_" + 
    "nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKu" + 
    "rztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en z" + 
    "u reduziertem Preis<br><br>> #  schutit_key   = <1200094385> #  INTERNE FELDER #    wert        = <0" + 
    "> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-277886> #   " + 
    " createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <130> #" + 
    "  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> # " + 
    " artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderun" + 
    "g/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200094386> #  INTERNE FELDER #    wert      " + 
    "  = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-277887" + 
    "> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <" + 
    "140> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <" + 
    "50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Ä" + 
    "nderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200083086> #  INTERNE FELDER #    wert" + 
    "        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-" + 
    "278315> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr" + 
    "   = <150> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis      " + 
    "   = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gle" + 
    "iche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200084634> #  INTERNE FELDER #  " + 
    "  wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr " + 
    " = <P-296091> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt" + 
    "_posnr   = <160> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis" + 
    "         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText " + 
    "= <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200081597> #  INTERNE FELD" + 
    "ER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schut" + 
    "it_nr  = <P-298516> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> # " + 
    " bfakt_posnr   = <170> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> # " + 
    " preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusat" + 
    "zText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200081595> #  INTERN" + 
    "E FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #   " + 
    " schutit_nr  = <P-298517> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <52133" + 
    "86> #  bfakt_posnr   = <180> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = " + 
    "<1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  ar" + 
    "tZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200080259> #  " + 
    "INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04" + 
    "> #    schutit_nr  = <P-299137> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = " + 
    "<5213386> #  bfakt_posnr   = <190> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl    " + 
    "    = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <>" + 
    " #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <120008025" + 
    "8> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004" + 
    "-05-04> #    schutit_nr  = <P-299138> #    createdDat  = null #    changedDat  = null ##  bfakt_id  " + 
    "    = <5213386> #  bfakt_posnr   = <200> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anza" + 
    "hl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText     " + 
    "  = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <120" + 
    "0074335> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       =" + 
    " <2004-05-04> #    schutit_nr  = <P-303775> #    createdDat  = null #    changedDat  = null ##  bfak" + 
    "t_id      = <5213386> #  bfakt_posnr   = <210> #  art_nr        = <44223> #  lageinh       = <ANZ> #" + 
    "  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artTex" + 
    "t       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   " + 
    "= <1200070937> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist  " + 
    "     = <2004-05-04> #    schutit_nr  = <P-305597> #    createdDat  = null #    changedDat  = null ##" + 
    "  bfakt_id      = <5213386> #  bfakt_posnr   = <220> #  art_nr        = <44223> #  lageinh       = <" + 
    "ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  " + 
    "artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_" + 
    "key   = <1200069258> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    f" + 
    "rist       = <2004-05-04> #    schutit_nr  = <P-307155> #    createdDat  = null #    changedDat  = n" + 
    "ull ##  bfakt_id      = <5213386> #  bfakt_posnr   = <230> #  art_nr        = <44223> #  lageinh    " + 
    "   = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rn" + 
    "g> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  sc" + 
    "hutit_key   = <1200069253> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> " + 
    "#    frist       = <2004-05-04> #    schutit_nr  = <P-307156> #    createdDat  = null #    changedDa" + 
    "t  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <240> #  art_nr        = <44223> #  lagei" + 
    "nh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/S" + 
    "itz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>>" + 
    " #  schutit_key   = <1200069646> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-0" + 
    "2-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-307462> #    createdDat  = null #    cha" + 
    "ngedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <250> #  art_nr        = <44223> # " + 
    " lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressände" + 
    "rung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br" + 
    "><br>> #  schutit_key   = <1200068996> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <" + 
    "2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-307790> #    createdDat  = null # " + 
    "   changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <260> #  art_nr        = <442" + 
    "23> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adre" + 
    "ssänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Pr" + 
    "eis<br><br>> #  schutit_key   = <1200065327> #  INTERNE FELDER #    wert        = <0> #    dat_ereig" + 
    "nis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-308353> #    createdDat  = n" + 
    "ull #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <270> #  art_nr        " + 
    "= <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   =" + 
    " <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduzier" + 
    "tem Preis<br><br>> #  schutit_key   = <1200065113> #  INTERNE FELDER #    wert        = <0> #    dat" + 
    "_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-308791> #    createdDa" + 
    "t  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <280> #  art_nr  " + 
    "      = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurzte" + 
    "xt   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu re" + 
    "duziertem Preis<br><br>> #  schutit_key   = <1200064905> #  INTERNE FELDER #    wert        = <0> # " + 
    "   dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-308895> #    cre" + 
    "atedDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <290> #  ar" + 
    "t_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  art" + 
    "Kurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en" + 
    " zu reduziertem Preis<br><br>> #  schutit_key   = <1200062091> #  INTERNE FELDER #    wert        = " + 
    "<0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-309176> # " + 
    "   createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <300>" + 
    " #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> " + 
    "#  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änder" + 
    "ung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200060289> #  INTERNE FELDER #    wert    " + 
    "    = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-3105" + 
    "25> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   =" + 
    " <310> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         =" + 
    " <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche" + 
    " Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200057394> #  INTERNE FELDER #    we" + 
    "rt        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <" + 
    "P-312568> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_pos" + 
    "nr   = <320> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis    " + 
    "     = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <G" + 
    "leiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200057390> #  INTERNE FELDER #" + 
    "    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_n" + 
    "r  = <P-312569> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfa" + 
    "kt_posnr   = <330> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  pre" + 
    "is         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzTex" + 
    "t = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200057389> #  INTERNE FE" + 
    "LDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    sch" + 
    "utit_nr  = <P-312570> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> " + 
    "#  bfakt_posnr   = <340> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> " + 
    "#  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZus" + 
    "atzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200057402> #  INTE" + 
    "RNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> # " + 
    "   schutit_nr  = <P-312571> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <521" + 
    "3386> #  bfakt_posnr   = <350> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        " + 
    "= <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  " + 
    "artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200055942> #" + 
    "  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-" + 
    "04> #    schutit_nr  = <P-313813> #    createdDat  = null #    changedDat  = null ##  bfakt_id      " + 
    "= <5213386> #  bfakt_posnr   = <360> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl  " + 
    "      = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = " + 
    "<> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200054" + 
    "677> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <20" + 
    "04-05-04> #    schutit_nr  = <P-314317> #    createdDat  = null #    changedDat  = null ##  bfakt_id" + 
    "      = <5213386> #  bfakt_posnr   = <370> #  art_nr        = <44223> #  lageinh       = <ANZ> #  an" + 
    "zahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText   " + 
    "    = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1" + 
    "200054738> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist      " + 
    " = <2004-05-04> #    schutit_nr  = <P-314973> #    createdDat  = null #    changedDat  = null ##  bf" + 
    "akt_id      = <5213386> #  bfakt_posnr   = <380> #  art_nr        = <44223> #  lageinh       = <ANZ>" + 
    " #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artT" + 
    "ext       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key " + 
    "  = <1200052877> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist" + 
    "       = <2004-05-04> #    schutit_nr  = <P-315490> #    createdDat  = null #    changedDat  = null " + 
    "##  bfakt_id      = <5213386> #  bfakt_posnr   = <390> #  art_nr        = <44223> #  lageinh       =" + 
    " <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #" + 
    "  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schuti" + 
    "t_key   = <1200054132> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #   " + 
    " frist       = <2004-05-04> #    schutit_nr  = <P-315838> #    createdDat  = null #    changedDat  =" + 
    " null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <400> #  art_nr        = <44223> #  lageinh  " + 
    "     = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz " + 
    "Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  " + 
    "schutit_key   = <1200054035> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16" + 
    "> #    frist       = <2004-05-04> #    schutit_nr  = <P-315839> #    createdDat  = null #    changed" + 
    "Dat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <410> #  art_nr        = <44223> #  lag" + 
    "einh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung" + 
    "/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br" + 
    ">> #  schutit_key   = <1200049771> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004" + 
    "-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-317901> #    createdDat  = null #    c" + 
    "hangedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <420> #  art_nr        = <44223> " + 
    "#  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressän" + 
    "derung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<" + 
    "br><br>> #  schutit_key   = <1200048701> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis=" + 
    " <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-318597> #    createdDat  = null " + 
    "#    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <430> #  art_nr        = <4" + 
    "4223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Ad" + 
    "ressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem " + 
    "Preis<br><br>> #  schutit_key   = <1200048702> #  INTERNE FELDER #    wert        = <0> #    dat_ere" + 
    "ignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-318598> #    createdDat  =" + 
    " null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <440> #  art_nr      " + 
    "  = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext  " + 
    " = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduzi" + 
    "ertem Preis<br><br>> #  schutit_key   = <1200048704> #  INTERNE FELDER #    wert        = <0> #    d" + 
    "at_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-318599> #    created" + 
    "Dat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <450> #  art_nr" + 
    "        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurz" + 
    "text   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu " + 
    "reduziertem Preis<br><br>> #  schutit_key   = <1200048705> #  INTERNE FELDER #    wert        = <0> " + 
    "#    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-318600> #    c" + 
    "reatedDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <460> #  " + 
    "art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  a" + 
    "rtKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/" + 
    "en zu reduziertem Preis<br><br>> #  schutit_key   = <1200048707> #  INTERNE FELDER #    wert        " + 
    "= <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-318602> " + 
    "#    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <47" + 
    "0> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50" + 
    "> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änd" + 
    "erung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200046316> #  INTERNE FELDER #    wert  " + 
    "      = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-32" + 
    "0310> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr  " + 
    " = <480> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis        " + 
    " = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleic" + 
    "he Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200045691> #  INTERNE FELDER #    " + 
    "wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  =" + 
    " <P-320540> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_p" + 
    "osnr   = <490> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis  " + 
    "       = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = " + 
    "<Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200045690> #  INTERNE FELDER" + 
    " #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit" + 
    "_nr  = <P-320541> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  b" + 
    "fakt_posnr   = <500> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  p" + 
    "reis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzT" + 
    "ext = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200040816> #  INTERNE " + 
    "FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    s" + 
    "chutit_nr  = <P-322569> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386" + 
    "> #  bfakt_posnr   = <510> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1" + 
    "> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZ" + 
    "usatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200039276> #  IN" + 
    "TERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> " + 
    "#    schutit_nr  = <P-323593> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5" + 
    "213386> #  bfakt_posnr   = <520> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl      " + 
    "  = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #" + 
    "  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200039275>" + 
    " #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-0" + 
    "5-04> #    schutit_nr  = <P-323594> #    createdDat  = null #    changedDat  = null ##  bfakt_id    " + 
    "  = <5213386> #  bfakt_posnr   = <530> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl" + 
    "        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       " + 
    "= <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <12000" + 
    "39274> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <" + 
    "2004-05-04> #    schutit_nr  = <P-323595> #    createdDat  = null #    changedDat  = null ##  bfakt_" + 
    "id      = <5213386> #  bfakt_posnr   = <540> #  art_nr        = <44223> #  lageinh       = <ANZ> #  " + 
    "anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText " + 
    "      = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = " + 
    "<1200039273> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist    " + 
    "   = <2004-05-04> #    schutit_nr  = <P-323596> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5213386> #  bfakt_posnr   = <550> #  art_nr        = <44223> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  ar" + 
    "tText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_ke" + 
    "y   = <1200039263> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    fri" + 
    "st       = <2004-05-04> #    schutit_nr  = <P-323608> #    createdDat  = null #    changedDat  = nul" + 
    "l ##  bfakt_id      = <5213386> #  bfakt_posnr   = <560> #  art_nr        = <44223> #  lageinh      " + 
    " = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng>" + 
    " #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schu" + 
    "tit_key   = <1200032941> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> # " + 
    "   frist       = <2004-05-04> #    schutit_nr  = <P-327520> #    createdDat  = null #    changedDat " + 
    " = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <570> #  art_nr        = <44223> #  lageinh" + 
    "       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sit" + 
    "z Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #" + 
    "  schutit_key   = <1200032901> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-" + 
    "16> #    frist       = <2004-05-04> #    schutit_nr  = <P-327572> #    createdDat  = null #    chang" + 
    "edDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <580> #  art_nr        = <44223> #  l" + 
    "ageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderu" + 
    "ng/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><" + 
    "br>> #  schutit_key   = <1200030682> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <20" + 
    "04-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <P-329111> #    createdDat  = null #   " + 
    " changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <590> #  art_nr        = <44223" + 
    "> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adress" + 
    "änderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Prei" + 
    "s<br><br>> #  schutit_key   = <1200025950> #  INTERNE FELDER #    wert        = <0> #    dat_ereigni" + 
    "s= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <332006> #    createdDat  = null " + 
    "#    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <600> #  art_nr        = <4" + 
    "4223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Ad" + 
    "ressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem " + 
    "Preis<br><br>> #  schutit_key   = <1200021428> #  INTERNE FELDER #    wert        = <0> #    dat_ere" + 
    "ignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <334496> #    createdDat  = n" + 
    "ull #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <610> #  art_nr        " + 
    "= <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   =" + 
    " <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduzier" + 
    "tem Preis<br><br>> #  schutit_key   = <1200021426> #  INTERNE FELDER #    wert        = <0> #    dat" + 
    "_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <334497> #    createdDat " + 
    " = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <620> #  art_nr    " + 
    "    = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext" + 
    "   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu redu" + 
    "ziertem Preis<br><br>> #  schutit_key   = <1200018506> #  INTERNE FELDER #    wert        = <0> #   " + 
    " dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <336415> #    created" + 
    "Dat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <630> #  art_nr" + 
    "        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurz" + 
    "text   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu " + 
    "reduziertem Preis<br><br>> #  schutit_key   = <1200018504> #  INTERNE FELDER #    wert        = <0> " + 
    "#    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <336416> #    cre" + 
    "atedDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <640> #  ar" + 
    "t_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  art" + 
    "Kurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en" + 
    " zu reduziertem Preis<br><br>> #  schutit_key   = <1200011900> #  INTERNE FELDER #    wert        = " + 
    "<0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <338821> #   " + 
    " createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <650> #" + 
    "  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> # " + 
    " artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderun" + 
    "g/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200008780> #  INTERNE FELDER #    wert      " + 
    "  = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <339778> " + 
    "#    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <66" + 
    "0> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50" + 
    "> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änd" + 
    "erung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1200008778> #  INTERNE FELDER #    wert  " + 
    "      = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <3397" + 
    "79> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   =" + 
    " <670> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         =" + 
    " <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche" + 
    " Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100343285> #  INTERNE FELDER #    we" + 
    "rt        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <" + 
    "343285> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr" + 
    "   = <680> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis      " + 
    "   = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gle" + 
    "iche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100343680> #  INTERNE FELDER #  " + 
    "  wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr " + 
    " = <343680> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_p" + 
    "osnr   = <690> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis  " + 
    "       = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = " + 
    "<Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100343681> #  INTERNE FELDER" + 
    " #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit" + 
    "_nr  = <343681> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfa" + 
    "kt_posnr   = <700> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  pre" + 
    "is         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzTex" + 
    "t = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100344224> #  INTERNE FE" + 
    "LDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    sch" + 
    "utit_nr  = <344224> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> # " + 
    " bfakt_posnr   = <710> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> # " + 
    " preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusat" + 
    "zText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100345000> #  INTERN" + 
    "E FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #   " + 
    " schutit_nr  = <345000> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386" + 
    "> #  bfakt_posnr   = <720> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1" + 
    "> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZ" + 
    "usatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100345001> #  IN" + 
    "TERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> " + 
    "#    schutit_nr  = <345001> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <521" + 
    "3386> #  bfakt_posnr   = <730> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        " + 
    "= <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  " + 
    "artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100346546> #" + 
    "  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-" + 
    "04> #    schutit_nr  = <346546> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = " + 
    "<5213386> #  bfakt_posnr   = <740> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl    " + 
    "    = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <>" + 
    " #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <110034654" + 
    "7> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004" + 
    "-05-04> #    schutit_nr  = <346547> #    createdDat  = null #    changedDat  = null ##  bfakt_id    " + 
    "  = <5213386> #  bfakt_posnr   = <750> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl" + 
    "        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       " + 
    "= <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <11003" + 
    "47940> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <" + 
    "2004-05-04> #    schutit_nr  = <347940> #    createdDat  = null #    changedDat  = null ##  bfakt_id" + 
    "      = <5213386> #  bfakt_posnr   = <760> #  art_nr        = <44223> #  lageinh       = <ANZ> #  an" + 
    "zahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText   " + 
    "    = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1" + 
    "100348578> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist      " + 
    " = <2004-05-04> #    schutit_nr  = <348578> #    createdDat  = null #    changedDat  = null ##  bfak" + 
    "t_id      = <5213386> #  bfakt_posnr   = <770> #  art_nr        = <44223> #  lageinh       = <ANZ> #" + 
    "  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artTex" + 
    "t       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   " + 
    "= <1100348579> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist  " + 
    "     = <2004-05-04> #    schutit_nr  = <348579> #    createdDat  = null #    changedDat  = null ##  " + 
    "bfakt_id      = <5213386> #  bfakt_posnr   = <780> #  art_nr        = <44223> #  lageinh       = <AN" + 
    "Z> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  ar" + 
    "tText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_ke" + 
    "y   = <1100348580> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    fri" + 
    "st       = <2004-05-04> #    schutit_nr  = <348580> #    createdDat  = null #    changedDat  = null " + 
    "##  bfakt_id      = <5213386> #  bfakt_posnr   = <790> #  art_nr        = <44223> #  lageinh       =" + 
    " <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #" + 
    "  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schuti" + 
    "t_key   = <1100348581> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #   " + 
    " frist       = <2004-05-04> #    schutit_nr  = <348581> #    createdDat  = null #    changedDat  = n" + 
    "ull ##  bfakt_id      = <5213386> #  bfakt_posnr   = <800> #  art_nr        = <44223> #  lageinh    " + 
    "   = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rn" + 
    "g> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  sc" + 
    "hutit_key   = <1100348582> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> " + 
    "#    frist       = <2004-05-04> #    schutit_nr  = <348582> #    createdDat  = null #    changedDat " + 
    " = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <810> #  art_nr        = <44223> #  lageinh" + 
    "       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sit" + 
    "z Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #" + 
    "  schutit_key   = <1100351361> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-" + 
    "16> #    frist       = <2004-05-04> #    schutit_nr  = <351361> #    createdDat  = null #    changed" + 
    "Dat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <820> #  art_nr        = <44223> #  lag" + 
    "einh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung" + 
    "/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br" + 
    ">> #  schutit_key   = <1100352303> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004" + 
    "-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <352303> #    createdDat  = null #    cha" + 
    "ngedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <830> #  art_nr        = <44223> # " + 
    " lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressände" + 
    "rung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br" + 
    "><br>> #  schutit_key   = <1100352304> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <" + 
    "2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <352304> #    createdDat  = null #   " + 
    " changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <840> #  art_nr        = <44223" + 
    "> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adress" + 
    "änderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Prei" + 
    "s<br><br>> #  schutit_key   = <1100354922> #  INTERNE FELDER #    wert        = <0> #    dat_ereigni" + 
    "s= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <354922> #    createdDat  = null " + 
    "#    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <850> #  art_nr        = <4" + 
    "4223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Ad" + 
    "ressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem " + 
    "Preis<br><br>> #  schutit_key   = <1100360108> #  INTERNE FELDER #    wert        = <0> #    dat_ere" + 
    "ignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <360108> #    createdDat  = n" + 
    "ull #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <860> #  art_nr        " + 
    "= <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   =" + 
    " <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduzier" + 
    "tem Preis<br><br>> #  schutit_key   = <1100360109> #  INTERNE FELDER #    wert        = <0> #    dat" + 
    "_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <360109> #    createdDat " + 
    " = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <870> #  art_nr    " + 
    "    = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext" + 
    "   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu redu" + 
    "ziertem Preis<br><br>> #  schutit_key   = <1100361542> #  INTERNE FELDER #    wert        = <0> #   " + 
    " dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <361542> #    created" + 
    "Dat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <880> #  art_nr" + 
    "        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurz" + 
    "text   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu " + 
    "reduziertem Preis<br><br>> #  schutit_key   = <1100367309> #  INTERNE FELDER #    wert        = <0> " + 
    "#    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <367309> #    cre" + 
    "atedDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <890> #  ar" + 
    "t_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  art" + 
    "Kurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en" + 
    " zu reduziertem Preis<br><br>> #  schutit_key   = <1100368816> #  INTERNE FELDER #    wert        = " + 
    "<0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <368816> #   " + 
    " createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <900> #" + 
    "  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> # " + 
    " artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderun" + 
    "g/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100368817> #  INTERNE FELDER #    wert      " + 
    "  = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <368817> " + 
    "#    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <91" + 
    "0> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50" + 
    "> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änd" + 
    "erung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100368818> #  INTERNE FELDER #    wert  " + 
    "      = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <3688" + 
    "18> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   =" + 
    " <920> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         =" + 
    " <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche" + 
    " Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100373174> #  INTERNE FELDER #    we" + 
    "rt        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <" + 
    "373174> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr" + 
    "   = <930> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis      " + 
    "   = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gle" + 
    "iche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100373175> #  INTERNE FELDER #  " + 
    "  wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr " + 
    " = <373175> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_p" + 
    "osnr   = <940> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis  " + 
    "       = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = " + 
    "<Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100373176> #  INTERNE FELDER" + 
    " #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit" + 
    "_nr  = <373176> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfa" + 
    "kt_posnr   = <950> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  pre" + 
    "is         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzTex" + 
    "t = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100377363> #  INTERNE FE" + 
    "LDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    sch" + 
    "utit_nr  = <377363> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386> # " + 
    " bfakt_posnr   = <960> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> # " + 
    " preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusat" + 
    "zText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100377364> #  INTERN" + 
    "E FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #   " + 
    " schutit_nr  = <377364> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <5213386" + 
    "> #  bfakt_posnr   = <970> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        = <1" + 
    "> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZ" + 
    "usatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100377365> #  IN" + 
    "TERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> " + 
    "#    schutit_nr  = <377365> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = <521" + 
    "3386> #  bfakt_posnr   = <980> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl        " + 
    "= <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <> #  " + 
    "artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100380654> #" + 
    "  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004-05-" + 
    "04> #    schutit_nr  = <380654> #    createdDat  = null #    changedDat  = null ##  bfakt_id      = " + 
    "<5213386> #  bfakt_posnr   = <990> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzahl    " + 
    "    = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText       = <>" + 
    " #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <110036065" + 
    "5> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = <2004" + 
    "-05-04> #    schutit_nr  = <360655> #    createdDat  = null #    changedDat  = null ##  bfakt_id    " + 
    "  = <5213386> #  bfakt_posnr   = <1000> #  art_nr        = <44223> #  lageinh       = <ANZ> #  anzah" + 
    "l        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText      " + 
    " = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = <1100" + 
    "382624> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist       = " + 
    "<2004-05-04> #    schutit_nr  = <382624> #    createdDat  = null #    changedDat  = null ##  bfakt_i" + 
    "d      = <5213386> #  bfakt_posnr   = <1010> #  art_nr        = <44223> #  lageinh       = <ANZ> #  " + 
    "anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  artText " + 
    "      = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key   = " + 
    "<1100382932> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    frist    " + 
    "   = <2004-05-04> #    schutit_nr  = <382932> #    createdDat  = null #    changedDat  = null ##  bf" + 
    "akt_id      = <5213386> #  bfakt_posnr   = <1020> #  art_nr        = <44223> #  lageinh       = <ANZ" + 
    "> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #  art" + 
    "Text       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schutit_key" + 
    "   = <1100382933> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #    fris" + 
    "t       = <2004-05-04> #    schutit_nr  = <382933> #    createdDat  = null #    changedDat  = null #" + 
    "#  bfakt_id      = <5213386> #  bfakt_posnr   = <1030> #  art_nr        = <44223> #  lageinh       =" + 
    " <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz Rng> #" + 
    "  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  schuti" + 
    "t_key   = <1100382934> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16> #   " + 
    " frist       = <2004-05-04> #    schutit_nr  = <382934> #    createdDat  = null #    changedDat  = n" + 
    "ull ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1040> #  art_nr        = <44223> #  lageinh   " + 
    "    = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/Sitz R" + 
    "ng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>> #  s" + 
    "chutit_key   = <1100382935> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-02-16>" + 
    " #    frist       = <2004-05-04> #    schutit_nr  = <382935> #    createdDat  = null #    changedDat" + 
    "  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1050> #  art_nr        = <44223> #  lagei" + 
    "nh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänderung/S" + 
    "itz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br><br>>" + 
    " #  schutit_key   = <1100384508> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2004-0" + 
    "2-16> #    frist       = <2004-05-04> #    schutit_nr  = <384508> #    createdDat  = null #    chang" + 
    "edDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1060> #  art_nr        = <44223> #  " + 
    "lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adressänder" + 
    "ung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Preis<br>" + 
    "<br>> #  schutit_key   = <1100387958> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2" + 
    "004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <387958> #    createdDat  = null #    " + 
    "changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1070> #  art_nr        = <44223" + 
    "> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <Adress" + 
    "änderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem Prei" + 
    "s<br><br>> #  schutit_key   = <1100387959> #  INTERNE FELDER #    wert        = <0> #    dat_ereigni" + 
    "s= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <387959> #    createdDat  = null " + 
    "#    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1080> #  art_nr        = <" + 
    "44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext   = <A" + 
    "dressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduziertem" + 
    " Preis<br><br>> #  schutit_key   = <1100388990> #  INTERNE FELDER #    wert        = <0> #    dat_er" + 
    "eignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <388990> #    createdDat  = " + 
    "null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1090> #  art_nr      " + 
    "  = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurztext  " + 
    " = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu reduzi" + 
    "ertem Preis<br><br>> #  schutit_key   = <1100391088> #  INTERNE FELDER #    wert        = <0> #    d" + 
    "at_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <391088> #    createdDa" + 
    "t  = null #    changedDat  = null ##  bfakt_id      = <5213386> #  bfakt_posnr   = <1100> #  art_nr " + 
    "       = <44223> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <50> #  artKurzt" + 
    "ext   = <Adressänderung/Sitz Rng> #  artText       = <> #  artZusatzText = <Gleiche Änderung/en zu r" + 
    "eduziertem Preis<br><br>> #  schutit_key   = <1200585509> #  INTERNE FELDER #    wert        = <0> #" + 
    "    dat_ereignis= <2004-02-16> #    frist       = <2004-05-04> #    schutit_nr  = <472715> #    crea" + 
    "tedDat  = null #    changedDat  = null ##AMV ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = " + 
    "<> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpf" + 
    "Param: #    null #  jgbZusatzRechEmpfParam: #    keine #INTERNE FELDER #  kto_nr              = <> #" + 
    "  unt_name            = <Xenia Abdin-Stoller> #  faellig_dat         = <2004-02-16>#  titel         " + 
    "      = <MALISTREN> #  formart_cd          = <436> #  schutz_beginn       = <1973-06-27>#  status_ve" + 
    "rarbeitung = <0> #  zahl_dat            = null #  anz_kopie           = <0> #  fakt_betrag         =" + 
    " null #  hasChanged          = <0> #  created_dat         = null #  changed_dat         = null #  er" + 
    "ror_source        = <> #  error_description   = <> #  zusatzRechEmpfNichtVerarbeitet = keine#"
  };
    
    //********************************************************************************************************

    public static String[] bfaktJgbStringsRangeA = {
      // bfakt_id : 5818918
      "#KOPFDATEN #  bfakt_id         = <5818918>#  bfakt_nr         = <0>#  bfakt_dat        = <2005-11-30" + 
      ">#  schutit_nr       = <01289872>#  auftraggeber     = <4657967>#  regulierer       = <0>#  hauptRec" + 
      "hEmpf    = <4657967>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <1443>" + 
      " #  sprach_cd        = <D> #  druck_art        = <3> #  waehrung         = <CHF> #  verfall_dat     " + 
      " = <2006-04-30>#  unsere_ref       = <PAdmin jgb/01289872> #  tel_nr           = <+41 31 322 49 46> " + 
      "#  inhaber          = <Trimo d.d., 8310 Trebnje> #  kz_einschr       = <0> #  schutit_typ      = <2>" + 
      " #  schutit_typ_bez  = <EPP> #  ihr_zeichen      = <> #  var_text         = <08.01.2001> #  diversTe" + 
      "xt       = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <<b>Jahresgebührenrech" + 
      "nung zum Patent/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$Inhaber<br>Anmeldedatum: %%$$Text<" + 
      "/b><br><br>Für das obenerwähnte Patent/Patentgesuch wird am <b>%%$$EreigDat</b> folgende Gebühr fäll" + 
      "ig:> #  beilagenText     = <Wir bitten Sie, den obenerwähnten Betrag bis zum <b>%%$$Verfall</b> mit " + 
      "beiliegendem Einzahlungsschein an uns zu überweisen, sofern er nicht einem Kontokorrent belastet wer" + 
      "den soll.<br>Die Zahlungsfrist ist gewahrt, wenn der Betrag rechtzeitig zu Gunsten des Instituts der" + 
      " Schweizerischen Post übergeben oder einem Post- oder Bankkonto in der Schweiz belastet worden ist. " + 
      "Bei Nichtbezahlung innerhalb der obengenannten Frist kann die Jahresgebühr gegen Entrichtung eines Z" + 
      "uschlags von CHF 100.00 noch während weiteren drei Monaten entrichtet werden. Zu diesem Zweck erfolg" + 
      "t nach Ablauf der unbenützten Zahlungsfrist die Zustellung einer Mahnung für die Jahres- und Zuschla" + 
      "gsgebühr.<br><br>Mit freundlichen Grüssen<br>Abteilung Patente> #  Belehrung        = <> #RECHNUNGSP" + 
      "OSITIONEN #  bfakt_id      = <5818918> #  bfakt_posnr   = <10> #  art_nr        = <28530> #  lageinh" + 
      "       = <ANZ> #  anzahl        = <1> #  preis         = <310> #  artKurztext   = <6.Jahresgeb.EP m." + 
      "Wirk.CH/LI> #  artText       = <> #  artZusatzText = <6. Jahresgebühr à> #  schutit_key   = <2001289" + 
      "872> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2006-01-31> #    frist       = <20" + 
      "06-04-30> #    schutit_nr  = <01289872> #    createdDat  = null #    changedDat  = null ##AMV ERGAEN" + 
      "ZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#  jgbLa" + 
      "ufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #  bfakt_id          = <5818918> #  adr_nr  " + 
      "          = <4657967> #  druck_periode     = <0> #  listen_art        = <0> #  fach_nr           = <" + 
      "0> #  druck_art         = <3> #  adr_nr_vtr        = <0> #  sort_text         = <> #  INTERNE FELDER" + 
      " #    created_dat     = null #    changed_dat     = null #  jgbZusatzRechEmpfParam: #    keine #INTE" + 
      "RNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faellig_dat         = <2006-01" + 
      "-31>#  titel               = <VORRICHTUNG UND VERFAHREN ZUM TRANSPORTIEREN VON LEICHTEN BAUPLATTEN> " + 
      "#  formart_cd          = <290> #  schutz_beginn       = <2001-01-08>#  status_verarbeitung = <0> #  " + 
      "zahl_dat            = null #  anz_kopie           = <0> #  fakt_betrag         = null #  hasChanged " + 
      "         = <0> #  created_dat         = null #  changed_dat         = null #  error_source        = " + 
      "<> #  error_description   = <> #  zusatzRechEmpfNichtVerarbeitet = keine#",
      // bfakt_id : 5818919
      "#KOPFDATEN #  bfakt_id         = <5818919>#  bfakt_nr         = <0>#  bfakt_dat        = <2005-11-30" + 
      ">#  schutit_nr       = <01290944>#  auftraggeber     = <4707106>#  regulierer       = <0>#  hauptRec" + 
      "hEmpf    = <4707106>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <1443>" + 
      " #  sprach_cd        = <D> #  druck_art        = <3> #  waehrung         = <CHF> #  verfall_dat     " + 
      " = <2006-04-30>#  unsere_ref       = <PAdmin jgb/01290944> #  tel_nr           = <+41 31 322 49 46> " + 
      "#  inhaber          = <Syngenta Participations AG, 4058 Basel> #  kz_einschr       = <0> #  schutit_" + 
      "typ      = <2> #  schutit_typ_bez  = <EPP> #  ihr_zeichen      = <> #  var_text         = <19.01.199" + 
      "9> #  diversText       = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <<b>Jahr" + 
      "esgebührenrechnung zum Patent/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$Inhaber<br>Anmeldeda" + 
      "tum: %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch wird am <b>%%$$EreigDat</b> folgen" + 
      "de Gebühr fällig:> #  beilagenText     = <Wir bitten Sie, den obenerwähnten Betrag bis zum <b>%%$$Ve" + 
      "rfall</b> mit beiliegendem Einzahlungsschein an uns zu überweisen, sofern er nicht einem Kontokorren" + 
      "t belastet werden soll.<br>Die Zahlungsfrist ist gewahrt, wenn der Betrag rechtzeitig zu Gunsten des" + 
      " Instituts der Schweizerischen Post übergeben oder einem Post- oder Bankkonto in der Schweiz belaste" + 
      "t worden ist. Bei Nichtbezahlung innerhalb der obengenannten Frist kann die Jahresgebühr gegen Entri" + 
      "chtung eines Zuschlags von CHF 100.00 noch während weiteren drei Monaten entrichtet werden. Zu diese" + 
      "m Zweck erfolgt nach Ablauf der unbenützten Zahlungsfrist die Zustellung einer Mahnung für die Jahre" + 
      "s- und Zuschlagsgebühr.<br><br>Mit freundlichen Grüssen<br>Abteilung Patente> #  Belehrung        = " + 
      "<> #RECHNUNGSPOSITIONEN #  bfakt_id      = <5818919> #  bfakt_posnr   = <10> #  art_nr        = <285" + 
      "50> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <310> #  artKurztext   = <8.J" + 
      "ahresgeb.EP m.Wirk.CH/LI> #  artText       = <> #  artZusatzText = <8. Jahresgebühr à> #  schutit_ke" + 
      "y   = <2001290944> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2006-01-31> #    fri" + 
      "st       = <2006-04-30> #    schutit_nr  = <01290944> #    createdDat  = null #    changedDat  = nul" + 
      "l ##AMV ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAEN" + 
      "ZUNGEN#  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #  bfakt_id          = <581891" + 
      "9> #  adr_nr            = <4707106> #  druck_periode     = <0> #  listen_art        = <0> #  fach_nr" + 
      "           = <0> #  druck_art         = <3> #  adr_nr_vtr        = <0> #  sort_text         = <> #  " + 
      "INTERNE FELDER #    created_dat     = null #    changed_dat     = null #  jgbZusatzRechEmpfParam: # " + 
      "   keine #INTERNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faellig_dat     " + 
      "    = <2006-01-31>#  titel               = <Herbizides Mittel> #  formart_cd          = <290> #  sch" + 
      "utz_beginn       = <1999-01-19>#  status_verarbeitung = <0> #  zahl_dat            = null #  anz_kop" + 
      "ie           = <0> #  fakt_betrag         = null #  hasChanged          = <0> #  created_dat        " + 
      " = null #  changed_dat         = null #  error_source        = <> #  error_description   = <> #  zus" + 
      "atzRechEmpfNichtVerarbeitet = keine#",
      // bfakt_id : 5818920
      "#KOPFDATEN #  bfakt_id         = <5818920>#  bfakt_nr         = <0>#  bfakt_dat        = <2005-11-30" + 
      ">#  schutit_nr       = <01290945>#  auftraggeber     = <4707106>#  regulierer       = <0>#  hauptRec" + 
      "hEmpf    = <4707106>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <1443>" + 
      " #  sprach_cd        = <D> #  druck_art        = <3> #  waehrung         = <CHF> #  verfall_dat     " + 
      " = <2006-04-30>#  unsere_ref       = <PAdmin jgb/01290945> #  tel_nr           = <+41 31 322 49 46> " + 
      "#  inhaber          = <Syngenta Participations AG, 4058 Basel> #  kz_einschr       = <0> #  schutit_" + 
      "typ      = <2> #  schutit_typ_bez  = <EPP> #  ihr_zeichen      = <> #  var_text         = <19.01.199" + 
      "9> #  diversText       = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <<b>Jahr" + 
      "esgebührenrechnung zum Patent/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$Inhaber<br>Anmeldeda" + 
      "tum: %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch wird am <b>%%$$EreigDat</b> folgen" + 
      "de Gebühr fällig:> #  beilagenText     = <Wir bitten Sie, den obenerwähnten Betrag bis zum <b>%%$$Ve" + 
      "rfall</b> mit beiliegendem Einzahlungsschein an uns zu überweisen, sofern er nicht einem Kontokorren" + 
      "t belastet werden soll.<br>Die Zahlungsfrist ist gewahrt, wenn der Betrag rechtzeitig zu Gunsten des" + 
      " Instituts der Schweizerischen Post übergeben oder einem Post- oder Bankkonto in der Schweiz belaste" + 
      "t worden ist. Bei Nichtbezahlung innerhalb der obengenannten Frist kann die Jahresgebühr gegen Entri" + 
      "chtung eines Zuschlags von CHF 100.00 noch während weiteren drei Monaten entrichtet werden. Zu diese" + 
      "m Zweck erfolgt nach Ablauf der unbenützten Zahlungsfrist die Zustellung einer Mahnung für die Jahre" + 
      "s- und Zuschlagsgebühr.<br><br>Mit freundlichen Grüssen<br>Abteilung Patente> #  Belehrung        = " + 
      "<> #RECHNUNGSPOSITIONEN #  bfakt_id      = <5818920> #  bfakt_posnr   = <10> #  art_nr        = <285" + 
      "50> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <310> #  artKurztext   = <8.J" + 
      "ahresgeb.EP m.Wirk.CH/LI> #  artText       = <> #  artZusatzText = <8. Jahresgebühr à> #  schutit_ke" + 
      "y   = <2001290945> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2006-01-31> #    fri" + 
      "st       = <2006-04-30> #    schutit_nr  = <01290945> #    createdDat  = null #    changedDat  = nul" + 
      "l ##AMV ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAEN" + 
      "ZUNGEN#  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #  bfakt_id          = <581892" + 
      "0> #  adr_nr            = <4707106> #  druck_periode     = <0> #  listen_art        = <0> #  fach_nr" + 
      "           = <0> #  druck_art         = <3> #  adr_nr_vtr        = <0> #  sort_text         = <> #  " + 
      "INTERNE FELDER #    created_dat     = null #    changed_dat     = null #  jgbZusatzRechEmpfParam: # " + 
      "   keine #INTERNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faellig_dat     " + 
      "    = <2006-01-31>#  titel               = <Herbizides Mittel> #  formart_cd          = <290> #  sch" + 
      "utz_beginn       = <1999-01-19>#  status_verarbeitung = <0> #  zahl_dat            = null #  anz_kop" + 
      "ie           = <0> #  fakt_betrag         = null #  hasChanged          = <0> #  created_dat        " + 
      " = null #  changed_dat         = null #  error_source        = <> #  error_description   = <> #  zus" + 
      "atzRechEmpfNichtVerarbeitet = keine#"
    };

    
    public static String[] bfaktJgbStringsRangeB = {
      // bfakt_id : 5818943
      "#KOPFDATEN #  bfakt_id         = <5818943>#  bfakt_nr         = <0>#  bfakt_dat        = <2005-11-30" + 
      ">#  schutit_nr       = <01282280>#  auftraggeber     = <4168831>#  regulierer       = <0>#  hauptRec" + 
      "hEmpf    = <4595648>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <1443>" + 
      " #  sprach_cd        = <D> #  druck_art        = <4> #  waehrung         = <CHF> #  verfall_dat     " + 
      " = <2006-01-31>#  unsere_ref       = <PAdmin jgb/01282280> #  tel_nr           = <+41 31 322 49 46> " + 
      "#  inhaber          = <ALCATEL, 75008 Paris> #  kz_einschr       = <0> #  schutit_typ      = <2> #  " + 
      "schutit_typ_bez  = <EPP> #  ihr_zeichen      = <> #  var_text         = <30.07.2001> #  diversText  " + 
      "     = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <M A H N U N G <b>Jahresge" + 
      "bührenrechnung zum Patent/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$Inhaber<br>Anmeldedatum:" + 
      " %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch sind folgende Gebühren fällig:> #  bei" + 
      "lagenText     = <Nachdem die Zahlung in den ersten drei Monaten nach Fälligkeit nicht erfolgt ist, k" + 
      "ann das Patent/Patentgesuch nur mit der Zahlung des Zuschlags zur Jahresgebühr aufrecht erhalten wer" + 
      "den. Das Patentgesuch/Patent wird zurückgewiesen bzw. gelöscht, wenn der obengenannte Totalbetrag ni" + 
      "cht fristgerecht einbezahlt wird. Wir bitten Sie, den obenerwähnten Betrag bis zum <b>%%$$Verfall</b" + 
      "> mit beiliegendem Einzahlungsschein an uns zu überweisen. Die Zahlungsfrist ist gewahrt, wenn der B" + 
      "etrag rechtzeitig zu Gunsten des Instituts der Schweizerischen Post übergeben oder einem Post- oder " + 
      "Bankkonto in der Schweiz belastet worden ist.<br><br>Mit freundlichen Grüssen<br>Abteilung Patente> " + 
      "#  Belehrung        = <> #RECHNUNGSPOSITIONEN #  bfakt_id      = <5818943> #  bfakt_posnr   = <10> #" + 
      "  art_nr        = <28520> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <310> #" + 
      "  artKurztext   = <5.Jahresgeb.EP m.Wirk.CH/LI> #  artText       = <> #  artZusatzText = <5. Jahresg" + 
      "ebühr à> #  schutit_key   = <2001282280> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis=" + 
      " <2005-07-31> #    frist       = <2006-01-31> #    schutit_nr  = <01282280> #    createdDat  = null " + 
      "#    changedDat  = null ##  bfakt_id      = <5818943> #  bfakt_posnr   = <20> #  art_nr        = <28" + 
      "700> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <200> #  artKurztext   = <Zu" + 
      "schlag JG EP m.Wirk.CH/LI> #  artText       = <> #  artZusatzText = <Zuschlag zur Jahresgebühr à> # " + 
      " schutit_key   = <2001282280> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2005-10-3" + 
      "1> #    frist       = <2006-01-31> #    schutit_nr  = <01282280> #    createdDat  = null #    change" + 
      "dDat  = null ##AMV ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> " + 
      "#JGB ERGAENZUNGEN#  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #  bfakt_id        " + 
      "  = <5818943> #  adr_nr            = <4595648> #  druck_periode     = <0> #  listen_art        = <0>" + 
      " #  fach_nr           = <0> #  druck_art         = <4> #  adr_nr_vtr        = <0> #  sort_text      " + 
      "   = <> #  INTERNE FELDER #    created_dat     = null #    changed_dat     = null #  jgbZusatzRechEm" + 
      "pfParam: #    keine #INTERNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faell" + 
      "ig_dat         = <2005-07-31>#  titel               = <Verfahren, Steuereinrichtung und Programmmodu" + 
      "l zur Steuerung und Lenkung von Datenströmen einer Kommunikationsverbindung zwischen Teilnehmern ein" + 
      "es Paketdatennetzes> #  formart_cd          = <291> #  schutz_beginn       = <2001-07-30>#  status_v" + 
      "erarbeitung = <0> #  zahl_dat            = null #  anz_kopie           = <0> #  fakt_betrag         " + 
      "= null #  hasChanged          = <0> #  created_dat         = null #  changed_dat         = null #  e" + 
      "rror_source        = <> #  error_description   = <> #  zusatzRechEmpfNichtVerarbeitet = keine#",
      // bfakt_id : 5818944
      "#KOPFDATEN #  bfakt_id         = <5818944>#  bfakt_nr         = <0>#  bfakt_dat        = <2005-11-30" + 
      ">#  schutit_nr       = <685203>#  auftraggeber     = <1155093>#  regulierer       = <0>#  hauptRechE" + 
      "mpf    = <500449>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <1443> # " + 
      " sprach_cd        = <D> #  druck_art        = <4> #  waehrung         = <CHF> #  verfall_dat      = " + 
      "<2006-01-31>#  unsere_ref       = <PAdmin jgb/685203> #  tel_nr           = <+41 31 322 49 46> #  in" + 
      "haber          = <Gifu Prefecture, Gifu-shi/Gifu-ken> #  kz_einschr       = <0> #  schutit_typ      " + 
      "= <1> #  schutit_typ_bez  = <CHP> #  ihr_zeichen      = <A 15606 WA/mf> #  var_text         = <16.07" + 
      ".1990> #  diversText       = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <M A" + 
      " H N U N G <b>Jahresgebührenrechnung zum Patent-/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$I" + 
      "nhaber<br>Anmeldedatum: %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch sind folgende G" + 
      "ebühren fällig:> #  beilagenText     = <Nachdem die Zahlung in den ersten drei Monaten nach Fälligke" + 
      "it nicht erfolgt ist, kann das Patent/Patentgesuch nur mit der Zahlung des Zuschlags zur Jahresgebüh" + 
      "r aufrecht erhalten werden. Das Patent/Patentgesuch wird zurückgewiesen bzw. gelöscht, wenn der oben" + 
      "genannte Totalbetrag nicht fristgerecht einbezahlt wird. Wir bitten Sie, den obenerwähnten Betrag bi" + 
      "s zum <b>%%$$Verfall</b> mit beiliegendem Einzahlungsschein an uns zu überweisen. Die Zahlungsfrist " + 
      "ist gewahrt, wenn der Betrag rechtzeitig zu Gunsten des Instituts der Schweizerischen Post übergeben" + 
      " oder einem Post- oder Bankkonto in der Schweiz belastet worden ist.<br><br>Mit freundlichen Grüssen" + 
      "<br>Abteilung Patente> #  Belehrung        = <> #RECHNUNGSPOSITIONEN #  bfakt_id      = <5818944> # " + 
      " bfakt_posnr   = <10> #  art_nr        = <18630> #  lageinh       = <ANZ> #  anzahl        = <1> #  " + 
      "preis         = <310> #  artKurztext   = <16. Jahresgebühr> #  artText       = <> #  artZusatzText =" + 
      " <16. Jahresgebühr à> #  schutit_key   = <30685203> #  INTERNE FELDER #    wert        = <0> #    da" + 
      "t_ereignis= <2005-07-31> #    frist       = <2006-01-31> #    schutit_nr  = <685203> #    createdDat" + 
      "  = null #    changedDat  = null ##  bfakt_id      = <5818944> #  bfakt_posnr   = <20> #  art_nr    " + 
      "    = <18700> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <200> #  artKurztex" + 
      "t   = <Zuschlag> #  artText       = <> #  artZusatzText = <Zuschlag zur Jahresgebühr à> #  schutit_k" + 
      "ey   = <30685203> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2005-10-31> #    fris" + 
      "t       = <2006-01-31> #    schutit_nr  = <685203> #    createdDat  = null #    changedDat  = null #" + 
      "#AMV ERGAENZUNGEN#  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUN" + 
      "GEN#  jgbLaufNr = <> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #  bfakt_id          = <5818944> " + 
      "#  adr_nr            = <500449> #  druck_periode     = <2> #  listen_art        = <2> #  fach_nr    " + 
      "       = <0> #  druck_art         = <4> #  adr_nr_vtr        = <0> #  sort_text         = <685203> #" + 
      "  INTERNE FELDER #    created_dat     = null #    changed_dat     = null #  jgbZusatzRechEmpfParam: " + 
      "#    keine #INTERNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faellig_dat   " + 
      "      = <2005-07-31>#  titel               = <Fadeneinzugsgerät.> #  formart_cd          = <191> #  " + 
      "schutz_beginn       = <1990-07-16>#  status_verarbeitung = <0> #  zahl_dat            = null #  anz_" + 
      "kopie           = <0> #  fakt_betrag         = null #  hasChanged          = <0> #  created_dat     " + 
      "    = null #  changed_dat         = null #  error_source        = <> #  error_description   = <> #  " + 
      "zusatzRechEmpfNichtVerarbeitet = keine#",
      // bfakt_id : 5818945
      "#KOPFDATEN #  bfakt_id         = <5818945>#  bfakt_nr         = <0>#  bfakt_dat        = <2005-11-30" + 
      ">#  schutit_nr       = <682693>#  auftraggeber     = <1162146>#  regulierer       = <0>#  hauptRechE" + 
      "mpf    = <1162146>#  zusatzRechEmpf   = keine#  kontaktPerson    = <> #  employeeID       = <1443> #" + 
      "  sprach_cd        = <D> #  druck_art        = <4> #  waehrung         = <CHF> #  verfall_dat      =" + 
      " <2006-01-31>#  unsere_ref       = <PAdmin jgb/682693> #  tel_nr           = <+41 31 322 49 46> #  i" + 
      "nhaber          = <Sonitel AG, 6300 Zug> #  kz_einschr       = <0> #  schutit_typ      = <1> #  schu" + 
      "tit_typ_bez  = <CHP> #  ihr_zeichen      = <02235/92> #  var_text         = <16.07.1992> #  diversTe" + 
      "xt       = <> #  abteilung        = <Abteilung Patente> #  kopfText         = <M A H N U N G <b>Jahr" + 
      "esgebührenrechnung zum Patent-/Patentgesuch Nr.: %%$$Schutit<br>lautend auf: %%$$Inhaber<br>Anmelded" + 
      "atum: %%$$Text</b><br><br>Für das obenerwähnte Patent/Patentgesuch sind folgende Gebühren fällig:> #" + 
      "  beilagenText     = <Nachdem die Zahlung in den ersten drei Monaten nach Fälligkeit nicht erfolgt i" + 
      "st, kann das Patent/Patentgesuch nur mit der Zahlung des Zuschlags zur Jahresgebühr aufrecht erhalte" + 
      "n werden. Das Patent/Patentgesuch wird zurückgewiesen bzw. gelöscht, wenn der obengenannte Totalbetr" + 
      "ag nicht fristgerecht einbezahlt wird. Wir bitten Sie, den obenerwähnten Betrag bis zum <b>%%$$Verfa" + 
      "ll</b> mit beiliegendem Einzahlungsschein an uns zu überweisen. Die Zahlungsfrist ist gewahrt, wenn " + 
      "der Betrag rechtzeitig zu Gunsten des Instituts der Schweizerischen Post übergeben oder einem Post- " + 
      "oder Bankkonto in der Schweiz belastet worden ist.<br><br>Mit freundlichen Grüssen<br>Abteilung Pate" + 
      "nte> #  Belehrung        = <> #RECHNUNGSPOSITIONEN #  bfakt_id      = <5818945> #  bfakt_posnr   = <" + 
      "10> #  art_nr        = <18610> #  lageinh       = <ANZ> #  anzahl        = <1> #  preis         = <3" + 
      "10> #  artKurztext   = <14. Jahresgebühr> #  artText       = <> #  artZusatzText = <14. Jahresgebühr" + 
      " à> #  schutit_key   = <30682693> #  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2005-" + 
      "07-31> #    frist       = <2006-01-31> #    schutit_nr  = <682693> #    createdDat  = null #    chan" + 
      "gedDat  = null ##  bfakt_id      = <5818945> #  bfakt_posnr   = <20> #  art_nr        = <18700> #  l" + 
      "ageinh       = <ANZ> #  anzahl        = <1> #  preis         = <200> #  artKurztext   = <Zuschlag> #" + 
      "  artText       = <> #  artZusatzText = <Zuschlag zur Jahresgebühr à> #  schutit_key   = <30682693> " + 
      "#  INTERNE FELDER #    wert        = <0> #    dat_ereignis= <2005-10-31> #    frist       = <2006-01" + 
      "-31> #    schutit_nr  = <682693> #    createdDat  = null #    changedDat  = null ##AMV ERGAENZUNGEN#" + 
      "  amvLaufId     = <-1> #  amvLaufNr     = <> #  amvRechnungCd = <0> #JGB ERGAENZUNGEN#  jgbLaufNr = " + 
      "<> #  jgbLaufID = <-1> #  jgbHauptRechEmpfParam: #  bfakt_id          = <5818945> #  adr_nr         " + 
      "   = <1162146> #  druck_periode     = <0> #  listen_art        = <0> #  fach_nr           = <0> #  d" + 
      "ruck_art         = <4> #  adr_nr_vtr        = <0> #  sort_text         = <02235/92> #  INTERNE FELDE" + 
      "R #    created_dat     = null #    changed_dat     = null #  jgbZusatzRechEmpfParam: #    keine #INT" + 
      "ERNE FELDER #  kto_nr              = <> #  unt_name            = <> #  faellig_dat         = <2005-0" + 
      "7-31>#  titel               = <Luftreinigungsgerät mit kombinierter Luftbefeuchtungsfunktion.> #  fo" + 
      "rmart_cd          = <191> #  schutz_beginn       = <1992-07-16>#  status_verarbeitung = <0> #  zahl_" + 
      "dat            = null #  anz_kopie           = <0> #  fakt_betrag         = null #  hasChanged      " + 
      "    = <0> #  created_dat         = null #  changed_dat         = null #  error_source        = <> # " + 
      " error_description   = <> #  zusatzRechEmpfNichtVerarbeitet = keine#"
    };

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public JoinSAPRechnungDAOTestHelper(String s)
  {
    super(s);
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(JoinSAPRechnungDAOTestHelper.class);
  }

  protected void setUp()
  {
    // not used
  }

  protected void tearDown()
  {
    // not used
  }

  //---------------------------------------------------------------------------
  // methods - genrate test data
  //---------------------------------------------------------------------------

  public void testGenerateBfaktStrings()
  {
    JoinSAPRechnungDAO  dao   = new JoinSAPRechnungDAO();
    boolean             first = true;

    try
    {
      System.out.println();
      System.out.println("  public static String[] bfaktStrings = {");

      for (int i = 0; i < bfaktIds.length; i++)
      {
        if (first)
        {
          first = false;
          System.out.println("    // bfakt_id : " + bfaktIds[i]);
        }
        else
        {
          System.out.println(",\n    // bfakt_id : " + bfaktIds[i]);
        }
        printStrDef( (dao.readRechnung(bfaktIds[i])).toString() );
      }

      System.out.println("\n  };");

      assertTrue(true);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  public void _testGenerateBfaktJgbStrings()
  {
    generateBfaktJgbRange("bfaktJgbStringsRangeA",bfaktJgbIdRangeA[0],bfaktJgbIdRangeA[1]);
    System.out.println();
    generateBfaktJgbRange("bfaktJgbStringsRangeB",bfaktJgbIdRangeB[0],bfaktJgbIdRangeB[1]);
  }


  //---------------------------------------------------------------------------
  // methods - helpers
  //---------------------------------------------------------------------------


  private void generateBfaktJgbRange(String name, int bfaktIdVon, int bfaktIdBis)
  {
    JoinSAPRechnungDAO  dao   = new JoinSAPRechnungDAO();
    ArrayList           list  = null;
    boolean             first = true;

    List    origStateList = null;
    Map     newState      = null;

    try
    {
      // init
      newState = new HashMap();
      newState.put("status_druck","3");
      newState.put("status_zahlg","1");


      // generate range
      origStateList = getJGBStateList(bfaktIdVon,bfaktIdBis);
      setJGBStateList(bfaktIdVon,bfaktIdBis,newState);

      list = dao.readJGB(bfaktIdVon,bfaktIdBis);

      System.out.println();
      System.out.println("  public static String[] " + name + " = {");
      for (Iterator iter = list.iterator(); iter.hasNext(); )
      {
        SAPRechnungVO vo = (SAPRechnungVO)iter.next();
        if (first)
        {
          first = false;
          System.out.println("    // bfakt_id : " + vo.bfakt_id);
        }
        else
        {
          System.out.println(",\n    // bfakt_id : " + vo.bfakt_id);
        }
        printStrDef( vo.toString() );
      }
      System.out.println("\n  };");

      setJGBStateList(bfaktIdVon,origStateList);
    }
    catch (Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }

  }


  public static String flatten(String s)
  {
    int     start   = 0;
    int     idx     = s.indexOf("\n");
    String  flatStr = "";

    while (idx > -1)
    {
      flatStr = flatStr + "#" + s.substring(start,idx);
      start = idx+1;
      idx   = s.indexOf("\n",start);
    }
    flatStr = flatStr + "#" + s.substring(start,s.length());

    return flatStr;
  }



  public static void printStrDef(String value)
  {
    String  flatStr = flatten(value);
    int     cnt     = 0;
    String  nose    = "    \"";
    String  tail    = "\" + \n";

    System.out.print(nose);
    for (int i=0; i<flatStr.length(); i++)
    {
      System.out.print(flatStr.charAt(i));
      cnt++;

      if (cnt==100)
      {
        cnt = 0;
        System.out.print(tail);
        System.out.print(nose);
      }
    }
    System.out.print("\"");
  }


  public static List getJGBStateList(int bfaktIdVon, int bfaktIdBis) throws SQLException
  {
    List  result  = new ArrayList();
    int   id      = bfaktIdVon;

    while (id <= bfaktIdBis)
    {
      result.add( getJGBState(id) );
      id++;
    }

    return result;
  }


  public static Map getJGBState(int bfaktId) throws SQLException
  {
    Connection  con     = null;
    Statement   stmt    = null;
    ResultSet   rs      = null;

    Map         result  = new HashMap();

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();
      rs    = stmt.executeQuery("select * from bfaktur where bfakt_id=" + Integer.toString(bfaktId));

      if (rs.next())
      {
        result.put("status_druck",rs.getString("status_druck"));
        result.put("status_zahlg",rs.getString("status_zahlg"));
      }

      return result;
    }
    finally
    {
      DBHelper.close(rs);
      DBHelper.close(stmt, con);
    }
  }


  public static void setJGBStateList(int bfaktIdVon, List list) throws SQLException
  {
    int   id  = bfaktIdVon;
    Map   map = null;

    for (Iterator iter = list.iterator(); iter.hasNext(); )
    {
      map = (Map)iter.next();
      setJGBState(id,map);
      id++;
    }
  }


  public static void setJGBStateList(int bfaktIdVon, int bfaktIdBis, Map map) throws SQLException
  {
    int   id  = bfaktIdVon;

    while (id <= bfaktIdBis)
    {
      setJGBState(id,map);
      id++;
    }
  }


  public static void setJGBState(int bfaktId, Map map) throws SQLException
  {
    Connection  con     = null;
    Statement   stmt    = null;
    String      query   = null;

    try
    {
      con = new BagisDAO().getConnection();

      stmt  = con.createStatement();

      query = "UPDATE bfaktur " +
              "SET    status_druck='" + (String)map.get("status_druck") + "', " +
              "       status_zahlg='" + (String)map.get("status_zahlg") + "', " +
              "       version = version+1 " +
              "WHERE  bfakt_id=" + Integer.toString(bfaktId);

      stmt.executeQuery(query);
    }
    finally
    {
      DBHelper.close(stmt, con);
    }
  }


} // end class
