package ch.ige.ses.server.dao.ses;

import java.sql.*;
import java.sql.Date;
import java.util.ArrayList;

import ch.ige.ses.vo.ses.JGBLaufVO;
import ch.ige.util.server.DBHelper;

/**
 * Data Access Objekt für die Tabelle JGB_LAUF
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rueegsegger
 * @version $Id: JGBLaufDAO.java,v 1.6 2006/12/13 14:54:56 iten Exp $
 */
public class JGBLaufDAO
{

  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  public JGBLaufDAO()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Selektierte alle Einträge der Tablle JGB_LAUF
   *
   * @return ArrayList      ArrayList mit Value Objects JGBLaufVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList selectAll() throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    ArrayList         al  = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "ORDER BY  jgb_lauf_id DESC" );

      rs = ps.executeQuery();

      while(rs.next())
      {
        al.add(fillJGBLaufVO(rs));
      }

      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


 /**
  * Selektiert die höchste jgb_lauf_id der Tabelle jgb_lauf
  *
  * @return   JGBLaufVO     grösste jgb_lauf_id
  * @throws   SQLException  Unerwarteter SQL-Fehler
  */
 public JGBLaufVO selectMaxLaufID() throws SQLException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    JGBLaufVO           vo  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
          ( getSelectStatement() +
            "WHERE jgb_lauf_id in (SELECT MAX(jgb_lauf_id) FROM jgb_lauf)" );

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillJGBLaufVO(rs);
      }

      return vo;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektiert alle LAUF_NR Einträge die älter sind als das angegebene
   * Erstellungsdatum.
   *
   * @param  created        Erstellungsdatum
   * @return ArrayList      ArrayList mit JGBLaufVO's
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public ArrayList select(java.util.Date created) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    ResultSet         rs  = null;
    ArrayList         al  = new ArrayList();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "WHERE CREATED_DAT < ? " +
              "ORDER BY LAUF_NR" );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );

      rs = ps.executeQuery();

      while(rs.next())
      {
        al.add(fillJGBLaufVO(rs));
      }

      return al;
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Selektierte eine Row der Tablle JGB_LAUF via LAUF_NR
   *
   * @param laufNr          Lauf Nummer
   * @return JGBLuafVO      Value Object JGBLaufVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public JGBLaufVO select(String laufNr) throws SQLException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    JGBLaufVO           vo  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "WHERE lauf_nr = ?" );

      ps.setString(1, laufNr);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillJGBLaufVO(rs);
      }

      return vo;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Selektierte eine Row der Tablle JGB_LAUF via JGB_LAUF_ID
   *
   * @param jgbLaufID       JGB LaufID
   * @return JGBLuafVO      Value Object JGBLaufVO
   * @throws SQLException   Unerwarteter SQL-Fehler
   */
  public JGBLaufVO select(int jgbLaufID) throws SQLException
  {
    Connection          con = null;
    PreparedStatement   ps  = null;
    ResultSet           rs  = null;
    JGBLaufVO           vo  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
            ( getSelectStatement() +
              "WHERE jgb_lauf_id = ?" );

      ps.setInt(1, jgbLaufID);

      rs = ps.executeQuery();

      if( rs.next() )
      {
        vo = fillJGBLaufVO(rs);
      }

      return vo;

    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Fügt einen Datensatz in die Tabelle JGB_LAUF ein
   *
   * @param     jgbLaufVO     ValueObject JGBLaufVO
   * @exception SQLException  Unerwarteter SQL Fehler
   */
  public void insert(JGBLaufVO jgbLaufVO) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "INSERT INTO ses.jgb_lauf(" +
            "jgb_lauf_id, " +
            "lauf_nr, " +
            "verarb_start, " +
            "verarb_ende, " +
            "ses_stat, " +
            "anzahl_bfaktur_gelesen, " +
            "anzahl_bfaktjgb_generiert, " +
            "sap_stat, " +
            "anzahl_bfaktur_an_sap, " +
            "anzahl_bfaktur_an_sap_ok, " +
            "anzahl_bfaktur_an_sap_nok, " +
            "anzahl_bfaktjgb_an_sap, " +
            "min_bfaktur_to_sap, " +
            "count_bfaktur_to_sap) " +

          "VALUES( SES.SEQ_JGB_LAUF_ID.nextval,?,sysdate,?,?,?,?,?,?,?,?,?,?,?)"
          );

      ps.setString ( 1, jgbLaufVO.laufNr);
      ps.setDate   ( 2, jgbLaufVO.verarbEnde);
      ps.setInt    ( 3, jgbLaufVO.sesStat);
      ps.setInt    ( 4, jgbLaufVO.anzahlBfakturGelesen);
      ps.setInt    ( 5, jgbLaufVO.anzahlBfaktjgbGeneriert);
      ps.setInt    ( 6, jgbLaufVO.sapStat);
      ps.setInt    ( 7, jgbLaufVO.anzahlBfakturAnSap);
      ps.setInt    ( 8, jgbLaufVO.anzahlBfakturAnSapOk);
      ps.setInt    ( 9, jgbLaufVO.anzahlBfakturAnSapNok);
      ps.setInt    (10, jgbLaufVO.anzahlBfaktjgbAnSap);
      ps.setInt    (11, jgbLaufVO.minBfakturToSap);
      ps.setInt    (12, jgbLaufVO.countBfakturToSap);

      ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut SES_STAT der Tabelle JGB_LAUF
   *
   * @param jgbLaufId PK
   * @param sesStat   Verarbeitungsstatus Bagis -> Ses
   *                  0 = neue Verarbeitung gestartet
   *                  1 = Einlesen ab Bagis
   *                  2 = Erstellen Ses Tabelle
   *                  3 = Ende Erstellung Ses Tabelle
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateSesStat(int jgbLaufId, int sesStat) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "ses_stat = ?, " +
            "changed_dat = sysdate " +
          "WHERE jgb_lauf_id = ? "
          );

      ps.setInt(1, sesStat);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut SES_STAT der Tabelle JGB_LAUF<br>
   * Das Attribut wird nur auf den Wert von sesStatNew aktualisiert,
   * wenn SES_STAT den Wert von sesStatOld hat
   *
   * @param jgbLaufId    PK
   * @param sesStatOld  jetziger Verarbeitungsstatus Bagis -> Ses
   * @param sesStat     Verarbeitungsstatus Bagis -> Ses
   *                    0 = neue Verarbeitung gestartet
   *                    1 = Einlesen ab Bagis
   *                    2 = Erstellen Ses Tabelle
   *                    3 = Ende Erstellung Ses Tabelle
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateSesStat(int jgbLaufId, int sesStatOld, int sesStat) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "ses_stat = ?, " +
            "changed_dat = sysdate " +
          "WHERE jgb_lauf_id = ? " +
          "AND ses_stat = ? "
          );

      ps.setInt(1, sesStat);
      ps.setInt(2, jgbLaufId);
      ps.setInt(3, sesStatOld);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut SAP_STAT der Tabelle JGB_LAUF.
   *
   * Beachte: Falls der Verarbeitungsstatus auf SAP_STAT_STOPPED oder
   *          SAP_STAT_FINISHED gesetzt werden soll, so wird automatisch
   *          auch das Verarbeitungsende - Datum gesetzt.
   *
   * @param jgbLaufId PK
   * @param sapStat   Verarbeitungsstatus Ses -> Sap (siehe JGBLaufVO.SAP_STAT_*)
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateSapStat(int jgbLaufId, int sapStat) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;
    StringBuffer select = new StringBuffer();

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      select.append("UPDATE ses.jgb_lauf SET " +
                      "sap_stat = ?, " +
                      "changed_dat = sysdate");

      if ( (sapStat == JGBLaufVO.SAP_STAT_STOPPED) ||
           (sapStat == JGBLaufVO.SAP_STAT_FINISHED) )
      {
        select.append(", verarb_ende = sysdate");
      }
      select.append(" WHERE jgb_lauf_id = ? ");

      ps = con.prepareStatement(select.toString());
      ps.setInt(1, sapStat);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut ANZAHL_BFAKTUR_GELESEN der Tabelle JGB_LAUF
   *
   * @param jgbLaufId               PK
   * @param anzahlBfakturGelesen    Anzahl von Bagis gelesene Rechnungen
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateAnzahlBfakturGelesen(int jgbLaufId, int anzahlBfakturGelesen) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "anzahl_bfaktur_gelesen = ?, " +
            "changed_dat = sysdate " +
          "WHERE jgb_lauf_id = ? "
          );

      ps.setInt(1, anzahlBfakturGelesen);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut ANZAHL_BFAKTJGB_GENERIERT der Tabelle JGB_LAUF
   *
   * @param jgbLaufId                 PK
   * @param anzahlBfaktjgbGeneriert   Anzahl generierter Rechnungen
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateAnzahlBfaktjgbGeneriert(int jgbLaufId, int anzahlBfaktjgbGeneriert) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "anzahl_bfaktjgb_generiert = ?, " +
            "changed_dat = sysdate " +
          "WHERE jgb_lauf_id = ? "
          );

      ps.setInt(1, anzahlBfaktjgbGeneriert);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * increment the number of rows transfered to the sap system.
   * the following fields are INCREMENTED with the delivered values:
   *  - ANZAHL_BFAKTUR_AN_SAP
   *  - ANZAHL_BFAKTUR_AN_SAP_OK
   *  - ANZAHL_BFAKTUR_AN_SAP_NOK
   *  - ANZAHL_BFAKTJGB_AN_SAP
   *
   * @param jgbLaufId             primary key
   * @param anzahlBfakturAnSap    number of rows from bfaktur_ses transfered to sap
   * @param anzahlBfakturAnSapOk  number of rows from bfaktur_ses transfered to sap successfully
   * @param anzahlBfakturAnSapNok number of rows from bfaktur_ses transfered to sap not successfully
   * @param anzahlBfaktjgbAnSap   number of rows from bfaktjgb_ses transfered to sap
   * @return                      number of affected rows
   * @throws SQLException         unexpected sql error
   */
  public int incrementAnzahlAnSap ( int jgbLaufId,
                                    int anzahlBfakturAnSap,
                                    int anzahlBfakturAnSapOk,
                                    int anzahlBfakturAnSapNok,
                                    int anzahlBfaktjgbAnSap ) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement
            ( "UPDATE ses.jgb_lauf " +
              "SET    anzahl_bfaktur_an_sap  = " +
                      "(select anzahl_bfaktur_an_sap + ?  from ses.jgb_lauf where jgb_lauf_id=?), " +
              "       anzahl_bfaktur_an_sap_ok = " +
                      "(select anzahl_bfaktur_an_sap_ok + ?  from ses.jgb_lauf where jgb_lauf_id=?), " +
              "       anzahl_bfaktur_an_sap_nok = " +
                      "(select anzahl_bfaktur_an_sap_nok + ?  from ses.jgb_lauf where jgb_lauf_id=?), " +
              "       anzahl_bfaktjgb_an_sap = " +
                      "(select anzahl_bfaktjgb_an_sap + ? from ses.jgb_lauf where jgb_lauf_id=?), " +
              "       changed_dat            = sysdate " +
              "WHERE  jgb_lauf_id            = ? " );

      ps.setInt(1,anzahlBfakturAnSap);
      ps.setInt(2,jgbLaufId);
      ps.setInt(3,anzahlBfakturAnSapOk);
      ps.setInt(4,jgbLaufId);
      ps.setInt(5,anzahlBfakturAnSapNok);
      ps.setInt(6,jgbLaufId);
      ps.setInt(7,anzahlBfaktjgbAnSap);
      ps.setInt(8,jgbLaufId);
      ps.setInt(9,jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert Attribut MIN_BFAKTUR_TO_SAP der Tabelle JGB_LAUF
   *
   * @param jgbLaufId         PK
   * @param minBfakturToSap   Rechnung mit der kleinsten bfakt_id, die noch an Sap übermittelt werden muss
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateMinBfakturToSap(int jgbLaufId, int minBfakturToSap) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "min_bfaktur_to_sap = ?, " +
            "changed_dat = sysdate " +
          "WHERE jgb_lauf_id = ? "
          );

      ps.setInt(1, minBfakturToSap);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }

  /**
   * Aktualisiert Attribut COUNT_BFAKTUR_TO_SAP der Tabelle JGB_LAUF
   *
   * @param jgbLaufId             PK
   * @param countBfakturToSap     Anzahl Rechnungen, welche noch an SAP übermittelt werden müssen
   * @return Anzahl aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int updateCountBfakturToSap(int jgbLaufId, int countBfakturToSap) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "count_bfaktur_to_sap = ?, " +
            "changed_dat = sysdate " +
          "WHERE jgb_lauf_id = ? "
          );

      ps.setInt(1, countBfakturToSap);
      ps.setInt(2, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert das Feld created_dat. Die Funktion wird im Zusammenhang
   * mit der Datenbank Reorganisatin zu Testzwecken verwendet.
   *
   * @param   jgbLaufId     eindeutige Lauf Id
   * @param   created       neues created_dat
   * @throws SQLException   unerwartete Fehlermeldung
   * @return int            Anzahl aktualisierte Datensätze
   */
  public int updateCreatedDat (int jgbLaufId, java.util.Date created) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con = DBHelper.getConnection(DBHelper.SES);
      ps  = con.prepareStatement
              ( "update ses.jgb_lauf " +
                "set    created_dat = ?, " +
                "       changed_dat = ? " +
                "WHERE  jgb_lauf_id = ? "
              );

      ps.setTimestamp( 1, new java.sql.Timestamp(created.getTime()) );
      ps.setTimestamp( 2, new java.sql.Timestamp(created.getTime()) );
      ps.setInt(3, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Aktualisiert einen Datensatz der Tabelle JGB_LAUF
   *
   * @param jgbLaufVO       Value Object
   * @return Anzahl         aktualisierter Datensätze
   * @throws SQLException   Unerwarteter SQL Fehler
   */
  public int update(JGBLaufVO jgbLaufVO) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);
      ps = con.prepareStatement(
          "UPDATE ses.jgb_lauf SET " +
            "lauf_nr = ?, " +
            "verarb_start = ?, " +
            "verarb_ende = ?, " +
            "ses_stat = ?, " +
            "anzahl_bfaktur_gelesen = ?, " +
            "anzahl_bfaktjgb_generiert = ?, " +
            "sap_stat = ?, " +
            "anzahl_bfaktur_an_sap = ?, " +
            "anzahl_bfaktur_an_sap_ok = ?, " +
            "anzahl_bfaktur_an_sap_nok = ?, " +
            "anzahl_bfaktjgb_an_sap = ?, " +
            "changed_dat = SYSDATE, " +
            "min_bfaktur_to_sap = ?, " +
            "count_bfaktur_to_sap = ? " +
          "WHERE jgb_lauf_id = ? "
          );

      ps.setString(1, jgbLaufVO.laufNr);
      if(jgbLaufVO.verarbStart == null)
      {
        ps.setNull(2, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(2, new java.sql.Date(jgbLaufVO.verarbStart.getTime()));
			}
      if(jgbLaufVO.verarbEnde == null)
      {
        ps.setNull(3, java.sql.Types.DATE);
      }
      else
      {
        ps.setDate(3, new java.sql.Date(jgbLaufVO.verarbEnde.getTime()));
			}
      ps.setInt ( 4, jgbLaufVO.sesStat);
      ps.setInt ( 5, jgbLaufVO.anzahlBfakturGelesen);
      ps.setInt ( 6, jgbLaufVO.anzahlBfaktjgbGeneriert);
      ps.setInt ( 7, jgbLaufVO.sapStat);
      ps.setInt ( 8, jgbLaufVO.anzahlBfakturAnSap);
      ps.setInt ( 9, jgbLaufVO.anzahlBfakturAnSapOk);
      ps.setInt (10, jgbLaufVO.anzahlBfakturAnSapNok);
      ps.setInt (11, jgbLaufVO.anzahlBfaktjgbAnSap);
      ps.setInt (12, jgbLaufVO.minBfakturToSap);
      ps.setInt (13, jgbLaufVO.countBfakturToSap);
      ps.setInt (14, jgbLaufVO.jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle JGB_LAUF
   *
   * @param     jgbLaufId      PK
   * @return    int            Anzahl gelöschter Datensätze
   * @exception SQLException Unerwarteter SQL Fehler
   */
  public int delete(int jgbLaufId) throws SQLException
  {
    Connection        con = null;
    PreparedStatement ps  = null;

    try
    {
      con  = DBHelper.getConnection(DBHelper.SES);

      ps = con.prepareStatement
      ( "DELETE FROM ses.jgb_lauf WHERE jgb_lauf_id = ? ");
      ps.setInt(1, jgbLaufId);

      return ps.executeUpdate();
    }
    finally
    {
      DBHelper.close(ps, con);
    }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
    * delivers sql statement for the select methods WITHOUT the where part
    * note: the field must correspondent with the fields used in the
    *       method fillJGBLaufVO !
    *
    * @return select statement
    */
  private String getSelectStatement()
  {
    return
      ( "SELECT jgb_lauf_id, lauf_nr, verarb_start, verarb_ende, ses_stat, " +
        "anzahl_bfaktur_gelesen, anzahl_bfaktjgb_generiert, sap_stat, " +
        "anzahl_bfaktur_an_sap, anzahl_bfaktur_an_sap_ok, anzahl_bfaktur_an_sap_nok, " +
        "anzahl_bfaktjgb_an_sap, created_dat, changed_dat, " +
        "min_bfaktur_to_sap, count_bfaktur_to_sap " +
        "FROM ses.jgb_lauf " );
  } // end method


  /**
    * fill fillJGBLaufVO with data from a valid result set
    *
    * @param     rs            valid result set
    * @return    fillJGBLaufVO value object
    * @exception SQLException  unexpected sql error
    */
  private JGBLaufVO fillJGBLaufVO (ResultSet rs) throws SQLException
  {
    JGBLaufVO vo = new JGBLaufVO();

    vo.jgbLaufId               = rs.getInt("jgb_lauf_id");
    vo.laufNr                  = rs.getString("lauf_nr");
    vo.verarbStart             = rs.getTimestamp("verarb_start") == null ?
                                   null : new Date(rs.getTimestamp("verarb_start").getTime());
    vo.verarbEnde              = rs.getTimestamp("verarb_ende") == null ?
                                   null : new Date(rs.getTimestamp("verarb_ende").getTime());
    vo.sesStat                 = rs.getInt("ses_stat");
    vo.anzahlBfakturGelesen    = rs.getInt("anzahl_bfaktur_gelesen");
    vo.anzahlBfaktjgbGeneriert = rs.getInt("anzahl_bfaktjgb_generiert");
    vo.sapStat                 = rs.getInt("sap_stat");
    vo.anzahlBfakturAnSap      = rs.getInt("anzahl_bfaktur_an_sap");
    vo.anzahlBfakturAnSapOk    = rs.getInt("anzahl_bfaktur_an_sap_ok");
    vo.anzahlBfakturAnSapNok   = rs.getInt("anzahl_bfaktur_an_sap_nok");
    vo.anzahlBfaktjgbAnSap     = rs.getInt("anzahl_bfaktjgb_an_sap");
    vo.createdDat              = rs.getTimestamp("created_dat") == null ?
                                   null : new Date(rs.getTimestamp("created_dat").getTime());
    vo.changedDat              = rs.getTimestamp("changed_dat") == null ?
                                   null : new Date(rs.getTimestamp("changed_dat").getTime());
    vo.minBfakturToSap         = rs.getInt("min_bfaktur_to_sap");
    vo.countBfakturToSap       = rs.getInt("count_bfaktur_to_sap");

    return vo;
  } // end method

} // end class


