package ch.ige.ses.server.dao.ses.test;

import java.math.*;
import java.sql.*;
import java.util.*;
import ch.ige.ses.server.dao.ses.*;
import ch.ige.ses.vo.bagis.*;
import ch.ige.ses.vo.ses.JGBLaufVO;
import junit.framework.*;


/**
 * Test der Klasse BfaktposSESDAO
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: BfakturSESDAOTest.java,v 1.12 2007/11/13 12:35:02 nydegger Exp $
 */

public class BfakturSESDAOTest extends SesTestCase
{
  private JGBLaufDAO jgbLaufDAO   = null;
  private static  Connection conJDBC;
  private final   String URL = "jdbc:oracle:thin:@ipiesa1030.ipie.ch:1521:EIGE";
  public static final String LAUF_NR = "2003011000";
  
  
  //---------------------------------------------------------------------------
  // constructor
  //---------------------------------------------------------------------------

  /**
   * Konstruktor
   * Treiberklasse laden und DB-Verbindung aufbauen
   * Zugriff auf Oracle-Datenbank via Jdbc-Treiber
   */
  public BfakturSESDAOTest(String s)
  {
    super(s);

    try
    {
      if (conJDBC == null)
      {
        //Treiberklasse laden
        Class.forName("oracle.jdbc.driver.OracleDriver");
        System.out.println("Treiber erfolgreich geladen");

        // DB-Verbindung aufbauen
        conJDBC = DriverManager.getConnection(URL, "ses_user01", "schnittstelle01");
        System.out.println("Db-Connection erfolgreich durchgeführt");
      }
    }
    catch (SQLException sqlex)
    {
      System.out.println("Db-Connection kann nicht durchgeführt werden: " + sqlex.getMessage() );
    }
    catch (Exception ex)
    {
      System.out.println("Es kann keine Datenbankverbindung hergestellt werden. Message: " + ex.getMessage() );
    }
  }

  //---------------------------------------------------------------------------
  // init / clean-up
  //---------------------------------------------------------------------------

  public static Test suite()
  {
     return new TestSuite(BfakturSESDAOTest.class);
  }

  protected void setUp() throws Exception
  {
    // lazy bone
  }

  protected void tearDown()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // tests
  //---------------------------------------------------------------------------

  public void test00Dummy()
  {
    /** TODO junit - testfälle junit tauglich machen */
    assertEquals(true,true);
  }



  /**
   *  Fügt einen Datensatz in die  Tabelle  bfaktur_ses ein
   */
  public void test01Insert()
  {
    try
    {
      jgbLaufDAO   = new JGBLaufDAO();
      JGBLaufVO jgbLaufVO     = new JGBLaufVO();
      jgbLaufVO.verarbEnde = null;
      jgbLaufVO.laufNr                  = LAUF_NR;
      jgbLaufVO.sesStat                 = 1;
      jgbLaufVO.anzahlBfakturGelesen    = 500;
      jgbLaufVO.anzahlBfaktjgbGeneriert = 700;
      jgbLaufVO.sapStat                 = 1;
      jgbLaufVO.anzahlBfakturAnSap      = 100;
      jgbLaufVO.anzahlBfakturAnSapOk    = 90;
      jgbLaufVO.anzahlBfakturAnSapNok   = 10;
      jgbLaufVO.anzahlBfaktjgbAnSap     = 150;
      jgbLaufDAO.insert(jgbLaufVO);
      
      // Testen ob eingefügt
      int laufID = new JGBLaufDAO().select(LAUF_NR).jgbLaufId;
      
      System.out.println("Test INSERT");
      BfakturSESDAO bfakturSESDAO = new BfakturSESDAO();
      SAPRechnungVO sapRechnungVO = new SAPRechnungVO();

      sapRechnungVO.bfakt_id = 1000;
      sapRechnungVO.jgbLaufId = laufID;
      sapRechnungVO.hauptRechEmpf = 1;
      sapRechnungVO.regulierer = 1;
      sapRechnungVO.auftraggeber = 1;
      sapRechnungVO.bfakt_dat = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
      sapRechnungVO.schutit_nr = "1000";
      sapRechnungVO.schutit_typ = 1000;
      sapRechnungVO.schutit_typ_bez = "1000";
      sapRechnungVO.inhaber = "test_insert";
      sapRechnungVO.ihr_zeichen = "test_insert";
      sapRechnungVO.tel_nr = "99999";
      sapRechnungVO.sprach_cd = "I";
      sapRechnungVO.druck_art = "I";
      sapRechnungVO.faellig_dat = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
      sapRechnungVO.anz_kopie = 1;
      sapRechnungVO.fakt_betrag = new BigDecimal(1000.00);
      sapRechnungVO.kz_einschr = "I";
      sapRechnungVO.status_verarbeitung = "I";
      sapRechnungVO.unsere_ref = "I";
      sapRechnungVO.unt_name = "I";
      sapRechnungVO.var_text = "test_insert";
      sapRechnungVO.zahl_dat = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());
      sapRechnungVO.waehrung = "I";
      sapRechnungVO.employeeId = "test_insert";
      sapRechnungVO.kontaktPerson = "test_insert";
      sapRechnungVO.abteilung = "test_insert";
      sapRechnungVO.kopfText = "test_insert";
      sapRechnungVO.beilagenText = "test_insert";
      sapRechnungVO.belehrungText = "test_insert";
      sapRechnungVO.error_source = "BfakturSESDAOTest, Methode testInsert";
      sapRechnungVO.error_description = "Genaue Beschreibung des Auftretenden Fehlers, kann auch ein bisschen länger sein " +
                                        "da 1000 Zeichen Platz haben";
      sapRechnungVO.verfall_dat = new java.sql.Date(new GregorianCalendar(2002,0,1,0,0,0).getTime().getTime());


      bfakturSESDAO.insert(sapRechnungVO);
      // testen ob Datensatz eingefügt wurde
      sapRechnungVO = bfakturSESDAO.select(1000, laufID);
      assertEquals(sapRechnungVO.bfakt_id, 1000);
      assertEquals(sapRechnungVO.jgbLaufId, laufID);
      assertEquals(sapRechnungVO.hauptRechEmpf, 1);
      assertEquals(sapRechnungVO.regulierer, 1);
      assertEquals(sapRechnungVO.auftraggeber, 1);
      assertEquals(sapRechnungVO.bfakt_dat.toString(), "2002-01-01");
      assertEquals(sapRechnungVO.schutit_nr, "1000");
      assertEquals(sapRechnungVO.schutit_typ, 1000);
      assertEquals(sapRechnungVO.schutit_typ_bez, "1000");
      assertEquals(sapRechnungVO.inhaber, "test_insert");
      assertEquals(sapRechnungVO.ihr_zeichen, "test_insert");
      assertEquals(sapRechnungVO.tel_nr, "99999");
      assertEquals(sapRechnungVO.sprach_cd, "I");
      assertEquals(sapRechnungVO.druck_art, "I");
      assertEquals(sapRechnungVO.faellig_dat.toString(), "2002-01-01");
      assertEquals(sapRechnungVO.anz_kopie, 1);
      assertEquals(sapRechnungVO.fakt_betrag, new BigDecimal(1000.00));
      assertEquals(sapRechnungVO.kz_einschr, "I");
      assertEquals(sapRechnungVO.status_verarbeitung, "I");
      assertEquals(sapRechnungVO.unsere_ref, "I");
      assertEquals(sapRechnungVO.unt_name, "I");
      assertEquals(sapRechnungVO.var_text, "test_insert");
      assertEquals(sapRechnungVO.zahl_dat.toString(), "2002-01-01");
      assertEquals(sapRechnungVO.waehrung, "I");
      assertEquals(sapRechnungVO.employeeId, "test_insert");
      assertEquals(sapRechnungVO.kontaktPerson, "test_insert");
      assertEquals(sapRechnungVO.abteilung, "test_insert");
      assertEquals(sapRechnungVO.kopfText, "test_insert");
      assertEquals(sapRechnungVO.beilagenText, "test_insert");
      assertEquals(sapRechnungVO.belehrungText, "test_insert");
      assertEquals(sapRechnungVO.error_source, "BfakturSESDAOTest, Methode testInsert");
      assertEquals(sapRechnungVO.error_description, "Genaue Beschreibung des Auftretenden Fehlers, kann auch ein bisschen länger sein " +
                                        "da 1000 Zeichen Platz haben");

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   *  Aktualisiert einen Datensatz in der Tabelle bfaktur_ses
   */
  public void test02Update()
  {
    System.out.println("Test UPDATE ");
    BfakturSESDAO bfakturSESDAO = new BfakturSESDAO();
    SAPRechnungVO sapRechnungVO = new SAPRechnungVO();

    try
    {
      //lauf_id von jgb_lauf holen
      int laufID = new JGBLaufDAO().select(LAUF_NR).jgbLaufId;

      sapRechnungVO = bfakturSESDAO.select(1000, laufID);
      if (sapRechnungVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        // Datensatz ändern
        sapRechnungVO.ihr_zeichen = "test_update";
        sapRechnungVO.inhaber = "test_update";
        sapRechnungVO.employeeId = "test_update";
        sapRechnungVO.kontaktPerson = "test_update";
        sapRechnungVO.abteilung = "test_update";
        sapRechnungVO.kopfText = "test_update";
        sapRechnungVO.beilagenText = "test_update";
        sapRechnungVO.belehrungText = "test_update";
        sapRechnungVO.var_text = "test_update";
        sapRechnungVO.kz_einschr = "U";
        sapRechnungVO.status_verarbeitung = "U";
        sapRechnungVO.unsere_ref = "U";
        sapRechnungVO.unt_name = "U";
        sapRechnungVO.waehrung = "U";
        sapRechnungVO.error_source = "BfakturSESDAOTest, Methode testUpdate";
        sapRechnungVO.error_description = "Die Fehlermeldung wurde in der Methode testUpdate aktualisiert";
        // Datensatz updaten
        int retVal = bfakturSESDAO.update(sapRechnungVO);
        assertEquals("Anzahl geänderter Datensätze: ", retVal, 1);
        //Datensatz wieder selektieren und testen, ob Änderungen übernommen
        sapRechnungVO = bfakturSESDAO.select(1000, laufID);
        assertEquals(sapRechnungVO.status_verarbeitung, "U");
        assertEquals(sapRechnungVO.inhaber, "test_update");
        sapRechnungVO = bfakturSESDAO.select(1000, laufID);
        assertEquals(sapRechnungVO.bfakt_id, 1000);
        assertEquals(sapRechnungVO.jgbLaufId, laufID);
        assertEquals(sapRechnungVO.hauptRechEmpf, 1);
        assertEquals(sapRechnungVO.regulierer, 1);
        assertEquals(sapRechnungVO.auftraggeber, 1);
        assertEquals(sapRechnungVO.bfakt_dat.toString() , "2002-01-01");
        assertEquals(sapRechnungVO.schutit_nr, "1000");
        assertEquals(sapRechnungVO.schutit_typ, 1000);
        assertEquals(sapRechnungVO.schutit_typ_bez, "1000");
        assertEquals(sapRechnungVO.inhaber, "test_update");
        assertEquals(sapRechnungVO.ihr_zeichen, "test_update");
        assertEquals(sapRechnungVO.tel_nr, "99999");
        assertEquals(sapRechnungVO.sprach_cd, "I");
        assertEquals(sapRechnungVO.druck_art, "I");
        assertEquals(sapRechnungVO.faellig_dat.toString(), "2002-01-01");
        assertEquals(sapRechnungVO.anz_kopie, 1);
        assertEquals(sapRechnungVO.fakt_betrag, new BigDecimal(1000.00));
        assertEquals(sapRechnungVO.kz_einschr, "U");
        assertEquals(sapRechnungVO.status_verarbeitung, "U");
        assertEquals(sapRechnungVO.unsere_ref, "U");
        assertEquals(sapRechnungVO.unt_name, "U");
        assertEquals(sapRechnungVO.var_text, "test_update");
        assertEquals(sapRechnungVO.zahl_dat.toString(), "2002-01-01");
        assertEquals(sapRechnungVO.waehrung, "U");
        assertEquals(sapRechnungVO.employeeId, "test_update");
        assertEquals(sapRechnungVO.kontaktPerson, "test_update");
        assertEquals(sapRechnungVO.abteilung, "test_update");
        assertEquals(sapRechnungVO.kopfText, "test_update");
        assertEquals(sapRechnungVO.beilagenText, "test_update");
        assertEquals(sapRechnungVO.belehrungText, "test_update");
        assertEquals(sapRechnungVO.error_source, "BfakturSESDAOTest, Methode testUpdate");
        assertEquals(sapRechnungVO.error_description, "Die Fehlermeldung wurde in der Methode testUpdate aktualisiert");
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   * Selektiert die grösste bfakt_id aus der Tabelle bfaktur_ses
   */
  public void test03SelectMaxBfaktID()
  {
    System.out.println("Test SELECT MaxBfaktID ");

    try
    {
      BfakturSESDAO bfakturSESDAO = new BfakturSESDAO();
      int           laufID        = new JGBLaufDAO().select(LAUF_NR).jgbLaufId;
      int           bfaktID       = bfakturSESDAO.selectMaxBfaktID(laufID);

      System.out.println("  bfakt_id = " + bfaktID);
      assertTrue("bfaktID > 0", bfaktID > 0);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle bfaktur_ses
   */
  public void test04Select()
  {
    System.out.println("Test SELECT ");
    BfakturSESDAO bfakturSESDAO = new BfakturSESDAO();
    SAPRechnungVO sapRechnungVO = new SAPRechnungVO();
    try
    {
      //Lauf ID von jgb_lauf holen
      int laufID = new JGBLaufDAO().select(LAUF_NR).jgbLaufId;

      sapRechnungVO = bfakturSESDAO.select(1000, laufID);

      System.out.println(sapRechnungVO.toString());

      assertEquals("bfakid: ", sapRechnungVO.bfakt_id, 1000);
      assertEquals("jgb_lauf_id: ", sapRechnungVO.jgbLaufId, laufID);
      assertEquals("inhaber: ", sapRechnungVO.inhaber, "test_update");
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   * Selektiert folgende Attribute der Tabelle BFAKTUR_SES
   * die den Selektionskriterien entsprechen
   *  bfakt_id
   *  jgb_lauf_id
   *  hauptrechempf
   *  regulierer
   *  auftraggeber
   *  schutit_nr
   *  schutit_typ
   *  status_druck
   *  created_dat
   *  changed_dat
   */
  public void test05SelectBfakturen()
  {
    System.out.println("Test SELECT BFAKTUREN");

    BfakturSESDAO   bfakturSESDAO   = new BfakturSESDAO();
    SAPRechnungVO   selKritVO       = new SAPRechnungVO();
    ArrayList       alSapRechnungVO = new ArrayList();
    SAPRechnungVO[] sapRechnungVO;

    selKritVO.jgbLaufId = 255;

    try
    {
      alSapRechnungVO = bfakturSESDAO.select(selKritVO);
      //sapRechnungVO = (SAPRechnungVO[])alSapRechnungVO.toArray(new SAPRechnungVO[0]);

      //assertEquals("jgb_lauf_id: ", sapRechnungVO[0].jgbLaufId, 255);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  /**
   * Selektiert alle Datensatze eines JGB-Laufes aus der Tabelle bfaktur_ses
   * mit einem Verarbeitungsstatus = 'E'
   */
  public void test06SelectErrorList()
  {
    PreparedStatement ps   = null;
    ResultSet         rs   = null;
    BfakturSESDAO     dao = new BfakturSESDAO();
    ArrayList         al = new ArrayList();

    int               jgbLaufId = selectMaxLaufId(); // zugriff via jdbc
    int               anzahl = 0;

    try
    {
      System.out.println("TEST SELECTERRORLIST");

      ps = conJDBC.prepareStatement
      ( "select count(*) as anzahl from bfaktur_ses where jgb_lauf_id = " + jgbLaufId +
        " AND status_verarbeitung = 'E'");

      rs = ps.executeQuery();

      if( rs.next() )
      {
        anzahl = rs.getInt("anzahl");
      }

      // Test selectErrorList
      al = dao.selectStatusVerarbList(jgbLaufId, "E");

      System.out.println("jgb_lauf_id: " + jgbLaufId);
      System.out.println("Anzahl gefundene Datensätze: " + al.size());

      assertEquals(al.size(), anzahl);

    }
    catch (Exception ex)
    {
      fail(ex.getMessage());
    }
    finally
    {
      try
      {
        if (ps != null) ps.close();
      }
      catch (Exception ex)
      {
        fail(ex.getMessage());
      }
    }
  }


  /**
   * Löscht einen Datensatz aus der Tabelle bfaktur_ses
   */
  public void test07Delete()
  {
    System.out.println("Test DELETE");
    jgbLaufDAO = new JGBLaufDAO();
    BfakturSESDAO bfakturSESDAO = new BfakturSESDAO();
    try
    {
      //LaufID von jgb_lauf holen
      int laufID = jgbLaufDAO.select(LAUF_NR).jgbLaufId;
      int retVal = bfakturSESDAO.delete(1000, laufID);
      assertEquals("1 gelöschter Datensatz",retVal, 1);
      retVal = jgbLaufDAO.delete( laufID );
      assertEquals("1 gelöschter Datensatz",retVal, 1);

    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }


  //---------------------------------------------------------------------------
  // public methods - manuel helpers
  //---------------------------------------------------------------------------

  /**
   * Manueller Test mit Ausgabe auf stdout:
   * Selektiert einen Datensatz aus der Tabelle bfaktur_ses und gibt diesen
   * auf den standard output aus.
   * Vorbereitung:
   *  - key's bfaktId und jgbLaufId initialisieren
   */
  public void ManuelSelect()
  {
    int           bfaktId   = 3769262;            // init with valid value !
    int           jgbLaufId = 434;                // init with valid value !
    BfakturSESDAO dao       = new BfakturSESDAO();
    SAPRechnungVO vo        = null;


    try
    {
      System.out.println("### Test ManuelSelect ###");
      System.out.println("  jgbLaufId = " + jgbLaufId);
      System.out.println("  bfaktId   = " + bfaktId);

      vo = dao.select(bfaktId,jgbLaufId);

      if (vo != null)
      {
        System.out.println("selected data:");
        System.out.println(vo.toString());
      }
      else
      {
        System.out.println("no data selected !");
      }

    }
    catch(Exception ex)
    {
      System.out.println("Exception: " + ex.toString());
      ex.printStackTrace();
    }
  }


  /**
   * Manueller Test mit Ausgabe auf stdout:
   * Selektiert einen Datensatz Range aus der Tabelle bfaktur_ses und gibt
   * diesen auf den standard output aus.
   * Vorbereitung:
   *  - key's initialisieren
   */
  public void ManuelSelectRange()
  {
    int           bfaktIdFrom = 3012337;      // init with valid value !
    int           bfaktIdTo   = 3013083;      // init with valid value !
    int           jgbLaufId   = 437;          // init with valid value !
    String        status      = null;         // init with valid value or null !

    BfakturSESDAO dao         = new BfakturSESDAO();
    ArrayList     al          = null;
    SAPRechnungVO vo          = null;

    try
    {
      System.out.println("### Test ManuelSelectRange ###");
      System.out.println("  jgbLaufId   = " + jgbLaufId);
      System.out.println("  bfaktIdFrom = " + bfaktIdFrom);
      System.out.println("  bfaktIdTo   = " + bfaktIdTo);
      System.out.println("  status      = " + status);

      al = dao.select(bfaktIdFrom,bfaktIdTo,jgbLaufId,status);

      if (al.size() > 0)
      {
        System.out.println("selected list of bfakt id's is:");
        System.out.println("  BFAKT_ID  VERARBEITUNGS_STATUS");

        for (Iterator i = al.iterator(); i.hasNext(); )
        {
          vo = (SAPRechnungVO) i.next();
          System.out.println("  " + vo.bfakt_id + "   " + vo.status_verarbeitung);
        }

        System.out.println("total records: " + al.size());
      }
      else
      {
        System.out.println("no data selected !");
      }

    }
    catch(Exception ex)
    {
      System.out.println("Exception: " + ex.toString());
      ex.printStackTrace();
    }
  }



  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * Selektiert die grösste Lauf-Id der Tabelle jgb_lauf
   * Der Zugriff erfolgt via jdbc-Treiber direkt auf die Oracle-Db
   * @return maxLaufId die grösste Lauf-Id, 0 falls keine Einträge vorhanden sind
   */
  private int selectMaxLaufId()
  {
     PreparedStatement ps   = null;
     ResultSet         rs   = null;
     int               maxLaufId = 0;

     try
     {
       ps = conJDBC.prepareStatement
           ( "select MAX (jgb_lauf_id) as jgb_lauf_id from jgb_lauf" );

       rs = ps.executeQuery();

       if( rs.next() )
       {
         maxLaufId = rs.getInt("jgb_lauf_id");
       }
     }
     catch (Exception ex)
     {
       System.out.println("Message: " + ex.getMessage() );
     }
     finally
     {
       try
       {
         if (ps != null) ps.close();
       }
       catch (Exception ex)
       {
         System.out.println("Message: " + ex.getMessage() );
       }
     }
    return maxLaufId;
  }
}


