package ch.ige.ses.server.dao.ses.test;

import junit.framework.TestCase;
import java.util.ArrayList;

import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.ses.vo.ses.PropertyVO;


/**
 * <p>Test der Klasse PropertyDAO</p>
 * <p>Copyright: Copyright (c) Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author A.Rueegsegger
 * @version $Id: PropertyDAOTest.java,v 1.6 2007/01/04 15:58:08 iten Exp $
 */

public class PropertyDAOTest extends SesTestCase
{

  public PropertyDAOTest(String s)
  {
    super(s);
  }

  /**
   *  Fügt einen Datensatz in die  Tabelle PROPERTY ein
   */
  public void testInsert()
  {
    System.out.println("PROPERTY: Test INSERT");
    PropertyDAO propertyDAO = new PropertyDAO();
    PropertyVO propertyVO = new PropertyVO();

    propertyVO.key = "test.bagisinbound";
    propertyVO.value = "1";
    propertyVO.description = "Einheit in Sekunden";

    try
    {
      propertyDAO.insert(propertyVO);
      propertyVO = propertyDAO.select("test.bagisinbound");
      if (propertyVO == null)
      {
        System.out.println("VO wurde nicht eingefügt");
      }
      assertTrue("PropertyVO eingefügt ", propertyVO != null);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Selektiert einen Datensatz aus der Tabelle PROPERTY
   */
  public void testSelect()
  {
    System.out.println("PROPERTY: Test SELECT ");
    PropertyDAO propertyDAO = new PropertyDAO();
    PropertyVO propertyVO = new PropertyVO();
    try
    {
      propertyVO = propertyDAO.select("test.bagisinbound");
      assertEquals("key: ", propertyVO.key, "test.bagisinbound");
      assertEquals("value: ", propertyVO.value, "1");
      assertEquals("descrption: ", propertyVO.description, "Einheit in Sekunden");
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Selektiert alle Datensätze aus der Tabelle PROPERTY
   */
  public void testSelectAll()
  {
    System.out.println("PROPERTY: Test SELECT ALL");
    PropertyDAO propertyDAO = new PropertyDAO();
    ArrayList al = new ArrayList();
    try
    {
      al = propertyDAO.selectAll();
      assertTrue("Select All: ", al.size() > 0);
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   *  Aktualisiert Attribut VALUE der Tabelle PROPERTY
   */
  public void testUpdateValue()
  {
    System.out.println("PROPERTY: Test UPDATE");
    PropertyDAO propertyDAO = new PropertyDAO();
    PropertyVO propertyVO = new PropertyVO();

    try
    {
      propertyVO = propertyDAO.select("test.bagisinbound");
      if (propertyVO == null)
      {
        System.out.println("UPDATE: Datensatz existiert nicht");
      }
      else
      {
        propertyVO.value = "10";
        propertyVO.description = "blabla";
        int retVal = propertyDAO.update(propertyVO);
        assertEquals("Anzahl geänderter Datensätze = 1", retVal, 1);
        propertyVO = propertyDAO.select("test.bagisinbound");
        assertEquals("Value: ", propertyVO.value, "10");
        assertEquals("Description: ", propertyVO.description, "blabla");
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }

  /**
   * Löscht einen Datensatz aus der Tabelle PROPERTY
   */
  public void testDelete()
  {
    System.out.println("PROPERTY: Test DELETE");
    PropertyDAO propertyDAO = new PropertyDAO();
    PropertyVO propertyVO = new PropertyVO();

    try
    {
      propertyVO = propertyDAO.select("test.bagisinbound");
      if (propertyVO == null)
      {
        System.out.println("DELETE: Datensatz existiert nicht");
      }
      else
      {
        int retVal = propertyDAO.delete(propertyVO.key);
        assertEquals("Anzahl gelöschte Datensätze = 1", retVal, 1);
      }
    }
    catch(Exception e)
    {
      e.printStackTrace();
      fail(e.toString());
    }
  }
}

