package ch.ige.ses.server.error;

import java.io.ByteArrayInputStream;
import java.io.ObjectInputStream;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.MessageDrivenBean;
import javax.ejb.MessageDrivenContext;
import javax.jms.BytesMessage;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.vo.ses.ErrorVO;

/**
 * MDB welches die SES - Errors aus JMS empfängt und via DAO in die 
 * Oracle - DB schreibt
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author Patrick Bolzli
 * @version $Id: SESErrorListenerBean.java,v 1.18 2006/12/28 07:55:44 iten Exp $
 * 
 * @ejb.bean name="SESErrorListener" 
 *  destination-type="javax.jms.Queue"
 *  transaction-type="Container"
 *  destination-jndi-name="jms/SES.errorQueue"
 *  connection-factory-jndi-name="jms/IGEQueueConnectionFactory" 
 * @jboss.destination-jndi-name name="jms/SES.errorQueue" 
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESErrorProcess"
 *   view-type="local"
 */
public class SESErrorListenerBean implements MessageDrivenBean, MessageListener
{
  private static final Logger LOG = Logger.getLogger(SESErrorListenerBean.class);
  MessageDrivenContext messageDrivenContext;

  SESErrorProcessLocalHome sesErrorProcessLocalHome = null;
  SESErrorProcessLocal sesErrorProcessLocal         = null;


  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb 
   */
  public void ejbCreate() 
  {
    try
    {
      //Naming-Kontext ermitteln
      Context ctx = new InitialContext();

      Object ref = ctx.lookup(sesErrorProcessLocalHome.COMP_NAME);
      sesErrorProcessLocalHome = (SESErrorProcessLocalHome)ref;
      sesErrorProcessLocal = sesErrorProcessLocalHome.create();
    }
    catch (CreateException ex)
    {
      LOG.error("SESErrorListenerBean CreateException: " + ex.getMessage());
    }
    catch (NamingException ex)
    {
      LOG.error("SESErrorListenerBean NamingException: " + ex.getMessage());
    }
  }
  
  
  public void ejbRemove()
  {
    // lazy bone
  }
  
  
  public void setMessageDrivenContext(MessageDrivenContext messageDrivenContext)
  {
    this.messageDrivenContext = messageDrivenContext;
  }
  
  
  //---------------------------------------------------------------------------
  // onMessage
  //---------------------------------------------------------------------------
  
  /**
   * Methode onMessage empängt eine BytesMessage. Aus dem byte[] wird das ErrorVO
   * deserialisiert. Dieser Umweg war nötig, da das versenden einer ObjectMessage
   * innerhalb des selben EAR's zu einer ClassCastException führte
   * 
   * @param msg Message
   */
  public void onMessage(Message msg)
  {
    try
    {
      if (msg instanceof javax.jms.BytesMessage)
      {
        // extract byte[] of Message
        BytesMessage bm = (BytesMessage)msg;
        byte[] barr = new byte[100000];
        bm.readBytes(barr);

        // byte[] --> ErrorVO
        ByteArrayInputStream bais = new ByteArrayInputStream(barr);
        ObjectInputStream ois = new ObjectInputStream(bais);
        Object obj = ois.readObject();
        ErrorVO eVO = (ErrorVO)obj;

        // forward the ErrorVO to sesErrorProcess
        sesErrorProcessLocal.processError(eVO);
      }
      else
      {
        LOG.error("serial://jms/SES.errorQueue: MessageTyp: BytesMessage wird erwartet");
      }
    }
    catch (Exception ex)
    {
      LOG.error("SESErrorListenerBean Methode on Message Fehler: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }
    
} // end of class

