package ch.ige.ses.server.error;

import javax.ejb.SessionBean;
import javax.ejb.SessionContext;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.ses.ErrorDAO;
import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.ses.vo.ses.ErrorVO;


/**
 * Schreibt ErrorVO in SES-Tabelle ERROR und liest Mailadresse anhand der 
 * EmployeeId aus dem Active Directory und schickt Emails an die betroffenen 
 * Benutzer.
 * <p>
 * Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author  Stefan Nydegger
 * @version $Id: SESErrorProcessBean.java,v 1.35 2007/01/10 14:46:37 iten Exp $
 * 
 * @ejb.bean 
 *  name="SESErrorProcess"  
 *  display-name="SESErrorProcess"
 *  view-type="local"
 *  jndi-name="SESErrorProcess"
 *  local-jndi-name="SESErrorProcessLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 */
public class SESErrorProcessBean implements SessionBean
{
  private static final Logger LOG = Logger.getLogger(SESErrorProcessBean.class);

  SessionContext sessionContext;

  
  //---------------------------------------------------------------------------
  //  bean methods
  //---------------------------------------------------------------------------
  
  /**
   * create ejb
   */
  public void ejbCreate()
  {
    // lazy bone
  }

  public void ejbRemove()
  {
    // lazy bone
  }

  public void ejbActivate()
  {
    // lazy bone
  }

  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * Methode processError speichert errorVO in der SES-Tabelle ERROR
   * und versendet eine Mailbenachrichtigung (falls aktiviert).
   * <p>
   * Wird ein Mail an den Benutzer verschickt, wird immer auch ein CC an
   * den Userdesk geschickt. Kann der Benutzer nicht eruiert werden,
   * wird nur ein Mail an den Userdesk geschickt.
   *
   * @param errorVO ValueObject
   * 
   * @ejb.interface-method
   */
  public void processError(ErrorVO errorVO)
  {
    processEmail(errorVO);      // vor saveErrorVo aufrufen, damit Status
                                // errorVO.employeeNotify korrekt gesetzt wird

    processErrorVo(errorVO);
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  /**
   * ErrorVO in SES-Tabelle ERROR speichern
   *
   * @param errorVO ValueObject
   */
  private void processErrorVo (ErrorVO errorVO)
  {
    try
    {
      LOG.debug("processErrorVo - start saving errorVO...");

      new ErrorDAO().insert(errorVO);

      LOG.debug("processErrorVo - ErrorVO wurde erfogreich gespeichert");
    }
    catch (Exception ex)
    {
      LOG.error ( "processErrorVo - Ein unerwarteter Fehler ist aufgetreten " +
                  "Exception : " + ex.getMessage() );
    }
  }


  /**
   * Mailbenachrichtigung versenden (falls aktiviert)
   *
   * @param errorVO ValueObject
   */
  private void processEmail (ErrorVO errorVO)
  {
    PropertyDAO         propertyDAO = new PropertyDAO();
    SESErrorPrepareMail prepareMail = new SESErrorPrepareMail();
    boolean             isAdmin     = false;

    try
    {
      //
      // Überprüfung ob Mailbenachrichtigung aktiviert
      //
      if ( errorVO.employeeNotify == errorVO.EMPL_NOTIFY_KEINE )
      {
        LOG.debug ( "processEmail - Mailbenachrichtigung deaktiviert " +
                    "(employeeNotify == EMPL_NOTIFY_KEINE)" );

        return;
      }
      else if ( errorVO.employeeNotify == errorVO.EMPL_NOTIFY_PENDENT )
      {
        LOG.debug ( "processEmail - Mailbenachrichtigung aktiviert ");
      }
      else
      {
        LOG.error ( "processEmail - Aufruf mit ungültigem Status " +
                    "(employeeNotify == EMPL_NOTIFY_BENACHRICHTIGT)" );
        return;
      }


      //
      // Setzen Empfänger - Optionen
      //
      if  ( propertyDAO.select("error.mail.mailbenutzer").value.equals("1") &&
            errorVO.employeeId != null )
      {
        isAdmin = false;
      }
      else if ( propertyDAO.select("error.mail.mailuserdesk").value.equals("1") )
      {
        isAdmin = true;
      }
      else
      {
        LOG.debug ( "processEmail - Mailbenachrichtigung an Benutzer " +
                    "und Userdesk auf SES Property DB ausgeschaltet !" );
        return;
      }


      //
      // Meldung aufbereiten und Mail versenden
      //
      if ( prepareMail.prepareMail(isAdmin,errorVO) )
      {
        errorVO.employeeNotify = ErrorVO.EMPL_NOTIFY_BENACHRICHTIGT;

        LOG.debug("processEmail - Mail wurde erfogreich versendet");
      }
      else
      {
        LOG.error("processEmail - Mail konnte nicht versendet werden");
      }

    }
    catch (Exception ex)
    {
      LOG.error ( "processEmail - Ein unerwarteter Fehler ist aufgetreten " +
                  "Exception : " + ex.getMessage() );
    }
  }

} // end class


