package ch.ige.ses.server.monitor;

import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.EJBException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.bagisoutbound.SESJgbBagisBO;
import ch.ige.ses.server.bagisoutbound.SESJgbBagisReadLocal;
import ch.ige.ses.server.bagisoutbound.SESJgbBagisReadLocalHome;
import ch.ige.ses.server.bagisoutbound.SESJgbWorkLocal;
import ch.ige.ses.server.bagisoutbound.SESJgbWorkLocalHome;
import ch.ige.ses.server.dao.ses.AmvLaufDAO;
import ch.ige.ses.server.dao.ses.AmvRechnungDAO;
import ch.ige.ses.server.dao.ses.BfaktJGBSESDAO;
import ch.ige.ses.server.dao.ses.BfaktposSESDAO;
import ch.ige.ses.server.dao.ses.BfakturSESDAO;
import ch.ige.ses.server.dao.ses.ErrorDAO;
import ch.ige.ses.server.dao.ses.InfoDAO;
import ch.ige.ses.server.dao.ses.JGBLaufDAO;
import ch.ige.ses.server.dao.ses.JoinSESRechnungDAO;
import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.ses.server.dao.ses.StakoKontrolleDAO;
import ch.ige.ses.server.dao.ses.StatistikDAO;
import ch.ige.ses.server.dao.ses.TraceDAO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.ses.AmvLaufVO;
import ch.ige.ses.vo.ses.AmvRechnungSelKritVO;
import ch.ige.ses.vo.ses.AmvRechnungVO;
import ch.ige.ses.vo.ses.ErrorSelKritVO;
import ch.ige.ses.vo.ses.ErrorVO;
import ch.ige.ses.vo.ses.JGBLaufVO;
import ch.ige.ses.vo.ses.PropertyVO;
import ch.ige.ses.vo.ses.StakoKontrolleVO;
import ch.ige.ses.vo.ses.StatistikVO;
import ch.ige.ses.vo.ses.TraceSelKritVO;

/**
 * Bean für SES-MONITOR
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author  A.Rüegsegger
 * @version $Id: MonitorCtrlBean.java,v 1.42 2007/01/10 18:21:18 iten Exp $
 * 
 * @ejb.bean 
 *  name="MonitorCtrl"  
 *  display-name="MonitorCtrl"
 *  view-type="remote"
 *  jndi-name="MonitorCtrl"
 *  local-jndi-name="MonitorCtrlLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 * @ejb.ejb-ref
 *   ejb-name="SESJgbBagisRead"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SESJgbWork"
 *   view-type="local"
 * @ejb.ejb-ref
 *   ejb-name="SapInboundCtrl"
 *   view-type="local"
 */
public class MonitorCtrlBean implements SessionBean
{
  //constants
  private final static Logger LOG = Logger.getLogger(MonitorCtrlBean.class);

  // fields
  private SESJgbBagisReadLocalHome home;
  private SESJgbBagisReadLocal ejbBagis;
  private SESJgbWorkLocalHome home2;
  private SESJgbWorkLocal ejbSes;

  SessionContext sessionContext;

  //---------------------------------------------------------------------------
  //  bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   */
  public void ejbCreate()
  {
    // lazy bone
  }


  public void ejbRemove()
  {
    // lazy bone
  }

  
  public void ejbActivate()
  {
    // lazy bone
  }


  public void ejbPassivate()
  {
    // lazy bone
  }

  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  
  //-----------------------------------------------------------------------------
  //  SES-URL, SES-VERSION
  //-----------------------------------------------------------------------------

  /**
   * Gibt die Meta-Daten von der SES-DB zurück.
   * 
   * @return String URL
   * 
   * @ejb.interface-method
   */
  public String getSESDbUrl()
  {
    try
    {
      return new InfoDAO().getSESDbURL();
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getSESDbUrl: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Gibt die Version zurück (muss von Hand nachgetragen werden)
   * 
   * @return String Version
   * 
   * @ejb.interface-method
   */
  public String getSESVersion()
  {
    try
    {
      return new InfoDAO().getSESVersion();
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getSESVersion: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  BFAKTUR_SES, BFAKTJGB_SES, BFAKTPOS_SES
  //-----------------------------------------------------------------------------

  /**
   * Gibt das Value Object der Rechnung zurück.
   *
   * @param   bfaktId    Rechnungsnummer
   * @param   jgbLaufId  Id des Jahresgebührenlaufs
   * @return  ValueObject der Rechnung
   * 
   * @ejb.interface-method
   */
  public SAPRechnungVO getRechnung(int bfaktId, int jgbLaufId)
  {
    SAPRechnungVO sapRechnungVO = new SAPRechnungVO();
    BfakturSESDAO bfakturSESDAO = new BfakturSESDAO();
    BfaktposSESDAO bfaktposSESDAO = new BfaktposSESDAO();
    BfaktJGBSESDAO bfaktJGBSESDAO = new BfaktJGBSESDAO();

    try
    {
      sapRechnungVO = bfakturSESDAO.select(bfaktId, jgbLaufId);
      LOG.debug("MonitorBean getRechnung: SelectBFaktur durchgeführt.");
      sapRechnungVO.posList = bfaktposSESDAO.select(bfaktId);
      LOG.debug("MonitorBean getRechnung: SelectBFaktpos durchgeführt.");
      sapRechnungVO.jgbZusatzRechEmpfParam = bfaktJGBSESDAO.select(bfaktId);
      LOG.debug("MonitorBean getRechnung: SelectBFaktJGB durchgeführt.");
    }
    catch (SQLException ex)
    {
      LOG.error("MonitorBean: Methode getRechnung SQLException:" +
                ex.getMessage());
      throw new EJBException(ex);
    }
    catch (IOException ioex)
    {
      LOG.error("MonitorBean: Methode getRechnung IOException:" +
                ioex.getMessage());
      throw new EJBException(ioex);
    }

    return sapRechnungVO;
  }

  
  /**
   * Selektiert diverse Attribute der Rechnungen die den Selektionskriterien entsprechen.
   * Rechnuungen werden in der Sortierreihenfolge der Jahresgebühren zurückgegeben
   * 
   * @param sapRechnungVO Value Object mit Selektionskriterien Rechnungs-Daten & Rechnungsempfänger
   * @return ArrayList von SAPRechnungVOs mit integrierten Rechnungempfänger(SAPJGBParamVO)
   * 
   * @ejb.interface-method
   */
  public ArrayList findSESRechnungen(SAPRechnungVO sapRechnungVO)
  {
    try
    {
      return new JoinSESRechnungDAO().selectJGBSort(sapRechnungVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findSESRechnungen: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  BFAKTUR SES
  //-----------------------------------------------------------------------------

  /**
   * Selektiert alle Rechnungen aus der Tabelle bfaktur_ses
   * die den Selektionskriterien entsprechen
   * 
   * @param sapRechnungVO Value Object mit Selektionskriterien
   * @return ArrayList von SAPRechnungVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findBfakturen(SAPRechnungVO sapRechnungVO)
  {
    try
    {
      return new BfakturSESDAO().select(sapRechnungVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findBfakturen: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  /**
   * Selektiert die grösste bfakt_id des gewünschtes Laufes aus der Tabelle bfaktur_ses
   * 
   * @param     jgbLaufId lauf_id eines Jahresgebühren Laufes
   * @return    grösste bfakt_id des ensprechenden Laufes
   * 
   * @ejb.interface-method
   */
  public int getMaxBfaktId(int jgbLaufId)
  {
    try
    {
      return new BfakturSESDAO().selectMaxBfaktID(jgbLaufId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getMaxBfaktId: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Selektiert die kleinste bfakt_id des gewünschten Laufes aus der Tabelle bfaktur_ses
   * 
   * @param     jgbLaufId lauf_id eines Jahresgebühren Laufes
   * @return    kleinste bfakt_id des entsprechenden Laufes
   * 
   * @ejb.interface-method
   */
  public int getMinBfaktId(int jgbLaufId)
  {
    try
    {
      return new BfakturSESDAO().selectMinBfaktID(jgbLaufId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getMinBfaktId: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Selektiert alle Datensätze eines JGB-Laufes aus der Tabelle bfaktur_ses
   * mit einem bestimmten Verarbeitungsstatus
   *
   * @param jgbLaufId Lauf-Id des JGB-Laufes
   * @param statusVerarbeitung Verarbeitungsstatus
   * @return Liste von SapRechnungV0s
   * 
   * @ejb.interface-method
   */
  public ArrayList findStatusVerarbList(int jgbLaufId,
                                        String statusVerarbeitung)
  {
    try
    {
      return new BfakturSESDAO().selectStatusVerarbList(jgbLaufId,
          statusVerarbeitung);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findStatusVerarbList: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Gibt die Anzahl Bfakturen pro Verarbeitungsstatus eines JGB - Laufes zurück
   * 
   * @param jgbLaufId Jahresgebühren - Lauf
   * @return Anzahl Datensätze pro Verarbeitungsstatus
   * 
   * @ejb.interface-method
   */
  public HashMap countStatusVerarbeitung(int jgbLaufId)
  {
    try
    {
      return new BfakturSESDAO().countStatusVerarbeitung(jgbLaufId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method countStatusVerarbeitung: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Gibt eine ArrayList von BfaktIds mit einem bestimmten Verarbeitungsstatus
   * eines Jahresgebühren - Laufes zurück
   *
   * @param jgbLaufId  JGB-Lauf
   * @param statusVerarbeitung Verarbeitungsstatus
   * @return ArrayList von bfaktIds
   * 
   * @ejb.interface-method
   */
  public ArrayList findBfaktIds(int jgbLaufId, String statusVerarbeitung)
  {
    try
    {
      return new BfakturSESDAO().selectBfaktIds(jgbLaufId, statusVerarbeitung);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findBfaktIds: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Verarbeitungstatus wird zurückgesetzt von 'E' auf '0'
   * Die Zähler des JGB-Laufes werden pro Update wie folgt aktualisiert
   *    anzahl_bfaktur_an_sap:      -1
   *    anzahl_bfaktur_an_sap_nok:  -1
   *    anzahl_bfaktur_an_sap_ok:   nichts
   *    anzahl_bfaktjgb_an_sap_ok:  -anzahl Rechnungsempfänger
   *
   * @param sapRechnungVO Array von SAPREechnungVO's
   * @return Anzahl upgedatete Datensätze
   * 
   * @ejb.interface-method
   */
  public int updateStatusVerarbeitung(SAPRechnungVO[] sapRechnungVO)
  {
    BfakturSESDAO bfakturDAO = new BfakturSESDAO();
    BfaktJGBSESDAO bfaktJGBDAO = new BfaktJGBSESDAO();

    int anzRechEmpf = 0; // Zähler Rechnungsempfänger(Haupt-, und Zusatz-)
    int bfaktId = 0; // Zähler Anzahl bfaktIds

    if (sapRechnungVO != null)
    {
      try
      {
        for (int i = 0; i < sapRechnungVO.length; i++)
        {
          int result = bfakturDAO.updateStatusVerarbeitung(sapRechnungVO[i].
              jgbLaufId, sapRechnungVO[i].bfakt_id, "0");
          if (result == 1)
          {
            anzRechEmpf += bfaktJGBDAO.selectCount(sapRechnungVO[i].bfakt_id); // Zusatzrechnungsempfänger
            anzRechEmpf++; // Hauptrechnungsempfänger
            bfaktId++;
          }
        }
        new JGBLaufDAO().incrementAnzahlAnSap(sapRechnungVO[0].jgbLaufId,
                                              -bfaktId, 0, -bfaktId,
                                              -anzRechEmpf);
      }
      catch (Exception ex)
      {
        LOG.error(getClass().toString() + ": Method updateStatusVerarbeitung: " +
                  ex.getMessage());
        sessionContext.setRollbackOnly();
        throw new EJBException(ex);
      }
    }
    return bfaktId; //Anzahl Updates auf bfaktur_ses
  }

  //-----------------------------------------------------------------------------
  //  PROPERTY
  //-----------------------------------------------------------------------------

  /**
   * Neuen Datensatz in Tabelle Property einfügen
   * 
   * @param propertyVO Value Object mit den Property-Daten
   *
   * @ejb.interface-method
   */
  public void createProperty(PropertyVO propertyVO)
  {
    try
    {
      new PropertyDAO().insert(propertyVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method createProperty: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Datensatz der Tabelle Property via key selektieren
   * 
   * @param key PK
   * @return Value Object mit den Property-Daten
   * 
   * @ejb.interface-method
   */
  public PropertyVO getProperty(String key)
  {
    try
    {
      return new PropertyDAO().select(key);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getProperty: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Alle Datensätze der Tabelle Property selektieren
   * 
   * @return ArrayList von PropertyVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findProperties()
  {
    try
    {
      return new PropertyDAO().selectAll();
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findProperties: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Datensatz der Tabelle Property updaten
   * 
   * @param propertyVO Value Object mit den Property-Daten
   * 
   * @ejb.interface-method
   */
  public void setProperty(PropertyVO propertyVO)
  {
    try
    {
      new PropertyDAO().update(propertyVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method setProperty: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Datensatz der Tabelle Property löschen
   * 
   * @param key PK
   * 
   * @ejb.interface-method
   */
  public void deleteProperty(String key)
  {
    try
    {
      new PropertyDAO().delete(key);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method deleteProperty: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  ERROR
  //-----------------------------------------------------------------------------

  /**
   * Alle Datensätze der Tabelle Error selektieren die den Selektionskriterien 
   * entsprechen
   * 
   * @param errorSelKritVO enthält Selektionskriterien
   * @return ArrayList von ErrorVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findErrors(ErrorSelKritVO errorSelKritVO)
  {
    try
    {
      return new ErrorDAO().select(errorSelKritVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findErrors: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Datensatz der Tabelle Error via key selektieren
   * 
   * @param errorId PK
   * @return Value Object mit den Error-Daten
   * 
   * @ejb.interface-method
   */
  public ErrorVO getError(int errorId)
  {
    try
    {
      return new ErrorDAO().select(errorId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getError: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Datensatz der Tabelle Error updaten
   * 
   * @param errorVO Value Object mit den Property-Daten
   * 
   * @ejb.interface-method
   */
  public void setError(ErrorVO errorVO)
  {
    try
    {
      new ErrorDAO().update(errorVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method setError: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  //-----------------------------------------------------------------------------
  //  JGB_LAUF
  //-----------------------------------------------------------------------------


  /**
   * Alle Datensätze der Tabelle JGB_Lauf selektieren
   * 
   * @return ArrayList von JGBLaufVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findJGBLauf()
  {
    try
    {
      return new JGBLaufDAO().selectAll();
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findJGBLauf: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Einen Datensätze der Tabelle JGB_Lauf selektieren
   * 
   * @param jgbLaufNr Lauf-Nummer
   * @return JGBLaufVO
   * 
   * @ejb.interface-method
   */
  public JGBLaufVO getJGBLauf(String jgbLaufNr)
  {
    try
    {
      return new JGBLaufDAO().select(jgbLaufNr);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getJGBLauf: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Einen Datensätze der Tabelle JGB_Lauf updaten
   * 
   * @param jgbLaufVO Daten des Laufes
   * @return Anzahl veränderter Datensätze
   * 
   * @ejb.interface-method
   */
  public int setJGBLauf(JGBLaufVO jgbLaufVO)
  {
    try
    {
      return new JGBLaufDAO().update(jgbLaufVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method setJGBLauf: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  CONTROLLING
  //-----------------------------------------------------------------------------

  /**
   * Alle Datensätze der Tabelle STAKO_KONTROLLE selektieren
   * 
   * @return ArrayList von StakoKontrolleVO's
   * 
   * @ejb.interface-method
   */
  public ArrayList findStakoKontrolle()
  {
    try
    {
      return new StakoKontrolleDAO().selectAll();
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findStakoKontrolle: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Einen Datensätze der Tabelle STAKO_KONTROLLE selektieren
   * 
   * @param stakoKontrolleId Verarbeitungsnummer
   * @return StakoKontrolleVO
   * 
   * @ejb.interface-method
   */
  public StakoKontrolleVO getStakoKontrolle(int stakoKontrolleId)
  {
    try
    {
      return new StakoKontrolleDAO().select(stakoKontrolleId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getStakoKontrolle: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Einen Datensätze der Tabelle STAKO_KONTROLLE updaten
   * 
   * @param stakoKontrolleVO Daten des Laufes
   * @return Anzahl veränderter Datensätze
   * 
   * @ejb.interface-method
   */
  public int setStakoKontrolle(StakoKontrolleVO stakoKontrolleVO)
  {
    try
    {
      return new StakoKontrolleDAO().update(stakoKontrolleVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method setStakoKontrolle: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  TRACE
  //-----------------------------------------------------------------------------

  /**
   * Alle Datensätze der Tabelle Trace selektieren die den Selektionskriterien 
   * entsprechen
   * 
   * @param traceSelKritVO enthält Selektionskriterien
   * @return ArrayList von TraceVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findTraces(TraceSelKritVO traceSelKritVO)
  {
    try
    {
      return new TraceDAO().select(traceSelKritVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findTraces: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Löscht alle Datensaetze die aelter als die übergebene Anzahl Tage sind
   * 
   * @param days Anzahl Tage
   * @return Anzahl gelöschte Rows
   * 
   * @ejb.interface-method
   */
  public int deleteTraces(int days)
  {
    try
    {
      return new TraceDAO().deleteRows(days);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method deleteTraces: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  Jahresgebühren - Auswertungen
  //-----------------------------------------------------------------------------

  /**
   * Gibt die Fehlerliste eines Jahresgebühren Laufes als String zurück
   * 
   * @param jgbLaufId Lauf-Id eines Jahresgebühren-Laufes
   * @return Fehlerliste als String
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public String getProtokoll(int jgbLaufId)
  {
    SESJgbBagisBO bo;
    String errorList;

    try
    {
      getLocalInterfaces(); // interfaces holen

      // getProtokoll auf dem BO aufrufen
      bo = new SESJgbBagisBO(this.ejbBagis, this.ejbSes);
      errorList = bo.getProtokoll(jgbLaufId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getProtokoll: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
    return errorList;
  } // end method

  
  /**
   * Gibt die Statistik eines Jahresgebühren Laufes als String zurück
   * Falls eine Neuberechnung gewünscht wird oder noch keine Statistikberechnung
   * für diesen Lauf durchgeführt wurde, wird eine Berechnung ausgelöst und diese
   * in der SES-DB gespeichert.
   * Ansonsten wird die letzte Berechnung zurückgegeben
   *
   * @param jgbLaufId Lauf-Id eines Jahresgebühren-Laufes
   * @param newcalc true -> Neuberechnung durchführen
   * @return Statistik als String
   * @author Patrick Bolzli
   * 
   * @ejb.interface-method
   */
  public String getStatistik(int jgbLaufId, boolean newcalc)
  {
    StatistikDAO statistikDAO = new StatistikDAO();
    StatistikVO statistikVO = new StatistikVO();
    SESJgbBagisBO bo;
    String statistik;

    try
    {
      if (!newcalc)
      {
        statistikVO = statistikDAO.select(jgbLaufId);
        if (statistikVO != null)
        {
          return statistikVO.data; // Aktuellste Berechnung
        }
      }

      getLocalInterfaces(); // interfaces holen

      // getStatistik auf dem BO aufrufen
      bo = new SESJgbBagisBO(this.ejbBagis, this.ejbSes);
      statistik = bo.getStatistik(jgbLaufId);

      // Statistik in SES-DB speichern
      statistikVO = new StatistikVO();
      statistikVO.jgbLaufId = jgbLaufId;
      statistikVO.data = statistik;
      statistikDAO.insert(statistikVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getStatistik: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
    return statistik;
  } // end method

  
  //-----------------------------------------------------------------------------
  //  JGB Bagis Druckstatus
  //-----------------------------------------------------------------------------

  /**
   * Aktualisiert bei der JGB Verarbeitung den Status druck auf BAGIS von
   * 3 (bereit zum Druck JGB) auf 1 (bereit zum Druck), sobald der Datensatz
   * von BAGIS nach SES übertragen wurde.
   *
   * @param   vo          Rechnungsdaten
   * @throws  Exception   Unerwarteter Fehler
   * 
   * @ejb.interface-method
   */
  public void setBfakturStatusDruck(SAPRechnungVO vo) throws Exception
  {
    SESJgbBagisBO bo = null;

    // init
    getLocalInterfaces(); // interfaces ejbBagis, ejbSes
    bo = new SESJgbBagisBO(this.ejbBagis, this.ejbSes);

    // set bfaktur status druck
    bo.setBfakturStatusDruck(vo);

  } // end method

  //-----------------------------------------------------------------------------
  //  AMV-LAUF
  //-----------------------------------------------------------------------------

  /**
   * Alle Datensätze der Tabelle AMV_Lauf selektieren
   * 
   * @return ArrayList von AmvLaufVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findAmvLauf()
  {
    try
    {
      return new AmvLaufDAO().select();
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findAmvLauf: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Einen Datensätze der Tabelle AMV_Lauf selektieren
   * 
   * @param amvLaufNr Lauf-Nummer
   * @return AmvLaufVO
   * 
   * @ejb.interface-method
   */
  public AmvLaufVO getAmvLauf(String amvLaufNr)
  {
    try
    {
      return new AmvLaufDAO().select(amvLaufNr);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getAmvLauf: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Einen Datensätze der Tabelle AMV_Lauf selektieren
   * 
   * @param amvLaufId Lauf-Nummer
   * @return AmvLaufVO
   * 
   * @ejb.interface-method
   */
  public AmvLaufVO getAmvLauf(int amvLaufId)
  {
    try
    {
      return new AmvLaufDAO().select(amvLaufId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getAmvLauf: " + ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Selektiert alle Einträge der Tabelle AMV_RECHNUNG
   * die den Selektionskriterien entsprechen
   * 
   * @param amvRechnungSelKritVO ValueObject mit Selektionskriterien
   * @return ArrayList von AmvRechnungVOs
   * 
   * @ejb.interface-method
   */
  public ArrayList findAmvRechnungen(AmvRechnungSelKritVO amvRechnungSelKritVO)
  {
    try
    {
      return new AmvRechnungDAO().select(amvRechnungSelKritVO);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method findAmvRechnungen: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  
  /**
   * Holt alle Daten einer Amv-Rechnung aus der Tabllen AMV_RECHNUNG
   * 
   * @param amvLaufId    PK AmvLauf
   * @param rechnungCd   PK Rechnungstyp
   * @param bfaktId      PK Rechnungsnummer
   * @return AmvRechnungVO mit Rechnungs-Informationen
   * 
   * @ejb.interface-method
   */
  public AmvRechnungVO getAmvRechnung(int amvLaufId, int rechnungCd, int bfaktId)
  {
    try
    {
      return new AmvRechnungDAO().select(amvLaufId, rechnungCd, bfaktId);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getAmvRechnung: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }


  /**
   * Selektiert pro Verarbeitungsstatus die summierte Anzahl Rechnungsempfänger
   * 
   * @param   amvLaufId     Lauf Id
   * @param   rechnungCd    Erinnerungstyp
   * @return  Map           beinhaltet summierten Rechnungsempfänger pro Verarbeitungsstatus
   * 
   * @ejb.interface-method
   */
  public Map getStatCounters(int amvLaufId, int rechnungCd)
  {
    try
    {
      return new AmvRechnungDAO().selectGroupByStat(amvLaufId, rechnungCd);
    }
    catch (Exception ex)
    {
      LOG.error(getClass().toString() + ": Method getStatCounters: " +
                ex.getMessage());
      throw new EJBException(ex);
    }
  }

  //-----------------------------------------------------------------------------
  //  private methods
  //-----------------------------------------------------------------------------

  /**
   * Erzeugt die lokalen Interfaces zu den Beans SESJgbBagisRead & SESJgbWork
   * 
   * @throws CreateException
   * @throws NamingException
   * @author Patrick Bolzli
   */
  private void getLocalInterfaces() throws CreateException, NamingException
  {
    // Für das BO lokale Referenzen zu den Beans holen
    Context ctx = new InitialContext();
    Object ref = ctx.lookup(SESJgbBagisReadLocalHome.COMP_NAME);
    this.home = (SESJgbBagisReadLocalHome) ref;
    this.ejbBagis = home.create();

    ref = ctx.lookup(SESJgbWorkLocalHome.COMP_NAME);
    this.home2 = (SESJgbWorkLocalHome) ref;
    this.ejbSes = home2.create();
  } // end method

  
} // end class

