package ch.ige.ses.server.sapctrl;

import java.util.ArrayList;

import javax.ejb.CreateException;
import javax.ejb.SessionBean;
import javax.ejb.SessionContext;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.ObjectMessage;
import javax.jms.QueueSender;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.naming.NamingException;

import org.apache.log4j.Logger;

import ch.ige.ses.util.Jms;
import ch.ige.ses.util.JmsQueue;
import ch.ige.ses.vo.bagis.SAPRechnungAnnullationVO;
import ch.ige.ses.vo.bagis.SAPRechnungVO;
import ch.ige.ses.vo.bagis.StakoVO;
import ch.ige.ses.vo.ses.AmvLaufVO;
import ch.ige.ses.vo.ses.SAPAdrStatusVO;
import ch.ige.ses.vo.ses.SAPJGBJobVO;
import ch.ige.ses.vo.ses.SAPStakoJobVO;
import ch.ige.util.vo.VersadrVO;

/**
 * Versendet JMS Meldungen für den SAP Inbound Service
 * <p>
 * Copyright (c) 2002, Eidgenössisches Institut für Geistiges Eigentum
 * @author T. Iten
 * @version $Id: SapSendBean.java,v 1.24 2007/12/24 10:13:23 iten Exp $
 * 
 * @ejb.bean 
 *  name="SapSend"  
 *  display-name="SapSend"
 *  view-type="local"
 *  jndi-name="SapSend"
 *  local-jndi-name="SapSendLocal"
 *  type="Stateless"
 *  transaction-type="Container"
 * @ejb.transaction 
 *  type="Required"
 */
public class SapSendBean implements SessionBean
{
  //TODO maintenance (optional) - jms handling umbauen
  
  //
  // const
  //
  private static final Logger   LOG = Logger.getLogger(SapSendBean.class);
  private static final boolean  JMS_SESSION_TRANSACTED        = false;
  private static final int      JMS_SESSION_ACKNOWLEDGE_MODE  = Session.AUTO_ACKNOWLEDGE;

  private static final int      JMS_DELIVERY_MODE = javax.jms.DeliveryMode.PERSISTENT;
  private static final int      JMS_PRIORITY      = javax.jms.Message.DEFAULT_PRIORITY;
  private static final long     JMS_TIME_TO_LIVE  = javax.jms.Message.DEFAULT_TIME_TO_LIVE;

  //
  // fields
  //
  SessionContext  sessionContext;

  private JmsQueue        fJmsQueue = null;         // set by ejbCreate()

  private QueueSession    fSession  = null;         // set by connect()
  private QueueSender     fSender   = null;
  private ObjectMessage   fObjMsg   = null;



  //---------------------------------------------------------------------------
  // bean methods
  //---------------------------------------------------------------------------

  /**
   * create ejb
   * 
   * @throws CreateException 
   */
  public void ejbCreate()
  {
      fJmsQueue = new JmsQueue ( 
          JmsQueue.RUN_ON_APPSRV,
          Jms.JNDI_JMS_QUEUE_FACTORY_NAME,
          Jms.JMS_INBOUND_QUEUE_NAME,
          JMS_SESSION_TRANSACTED,
          JMS_SESSION_ACKNOWLEDGE_MODE );

  } // end method


  public void ejbRemove()
  {
    // lazy bone    
  }


  public void ejbActivate()
  {
    // lazy bone
  }


  public void ejbPassivate()
  {
    // lazy bone
  }


  public void setSessionContext(SessionContext sessionContext)
  {
    this.sessionContext = sessionContext;
  }

  //---------------------------------------------------------------------------
  // public methods
  //---------------------------------------------------------------------------

  /**
   * @param vo 
   * @throws Exception 
   * 
   * @ejb.interface-method
   */
  public void sendRechnung (SAPRechnungVO vo) throws Exception
  {
    try
    {
      connect();
      
      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_SAP_RECHNUNG,
                                 Jms.MSG_DESC_SAP_RECHNUNG );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );
    }
    catch (JMSException ex)
    {
      LOG.error("sendRechnung - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendJGBJob (SAPJGBJobVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_SAP_JGB_JOB,
                                 Jms.MSG_DESC_SAP_JGB_JOB );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendJGBJob - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendAnnulation (SAPRechnungAnnullationVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_SAP_ANNULLATION,
                                 Jms.MSG_DESC_SAP_ANNULLATION );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendAnnulation - JmsException occured: " + ex.toString());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param list
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendAnnulationList (ArrayList list) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(list);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_SAP_ANNULLATION_LIST,
                                 Jms.MSG_DESC_SAP_ANNULLATION_LIST );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendAnnulationList - JmsException occured: " + ex.toString());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendCtrlStop() throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(null);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_CTRL_STOP,
                                 Jms.MSG_DESC_CTRL_STOP );

      fSender.send(fObjMsg);                 // send NOT in persistent delivery mode

    }
    catch (JMSException ex)
    {
      LOG.error("sendCtrlStop - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param jgbLaudId
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendCtrlJgbFinish (int jgbLaudId) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(new Integer(jgbLaudId));

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_CTRL_JGB_FINISH,
                                 Jms.MSG_DESC_CTRL_JGB_FINISH );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendCtrlJgbFinish - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendCtrlAmvFinish (AmvLaufVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_CTRL_AMV_FINISH,
                                 Jms.MSG_DESC_CTRL_AMV_FINISH );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendCtrlAmvFinish - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendAdressCreate (VersadrVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties ( fObjMsg, Jms.MSG_TYPE_SAP_CUST_CREATE,
                                 Jms.MSG_DESC_SAP_CUST_CREATE );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendAdressCreate - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendAdressStatus (SAPAdrStatusVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties (fObjMsg , Jms.MSG_TYPE_SAP_CUST_STATUS,
                                 Jms.MSG_DESC_SAP_CUST_STATUS );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendAdressStatus - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendStakoJob (SAPStakoJobVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties ( fObjMsg,  Jms.MSG_TYPE_SAP_STAKO_JOB,
                                  Jms.MSG_DESC_SAP_STAKO_JOB );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                         JMS_PRIORITY,
                         JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendStakoJob - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }

  /**
   * @param vo
   * @throws Exception
   * 
   * @ejb.interface-method
   */
  public void sendStako (StakoVO vo) throws Exception
  {
    try
    {
      connect();

      fObjMsg = fSession.createObjectMessage(vo);

      setMessageProperties ( fObjMsg,  Jms.MSG_TYPE_SAP_STAKO,
                             Jms.MSG_DESC_SAP_STAKO );

      fSender.send ( fObjMsg, JMS_DELIVERY_MODE,
                     JMS_PRIORITY,
                     JMS_TIME_TO_LIVE );

    }
    catch (JMSException ex)
    {
      LOG.error("sendStako - JmsException occured: " + ex.getMessage());
      throw new Exception(ex.getMessage());
    }
    finally
    {
      disconnect();
    }
  }


  //---------------------------------------------------------------------------
  // private methods
  //---------------------------------------------------------------------------

  private void setMessageProperties (Message m, int type, String description)
    throws JMSException
  {
    m.setIntProperty    ( Jms.JMS_PROP_NAME_MSG_TYPE, type );
    m.setStringProperty ( Jms.JMS_PROP_NAME_MSG_DESC, description );
  }

  private void connect() throws NamingException, JMSException 
  {
    fJmsQueue.connectSender();

    fSession = fJmsQueue.getSession();
    fSender  = fJmsQueue.getSender();
  }
  
  private void disconnect()
  {
    fJmsQueue.disconnect();
    fSession = null;        
    fSender  = null;
    fObjMsg  = null;
  }
  
} // end class


