package ch.ige.ses.util;

import java.util.*;
import javax.mail.*;
import javax.mail.internet.*;

import org.apache.log4j.*;

/**
 * <p>Überschrift: MailSender</p>
 * <p>Beschreibung: Klasse zum Verschicken von Emails</p>
 * <p>Copyright: Copyright (c) 2001-2003, Eidgenössisches Institut für Geistiges Eigentum</p>
 * <p>Organisation: Eidgenössisches Institut für Geistiges Eigentum</p>
 * @author Stefan Nydegger
 * @version $Id: MailSender.java,v 1.11 2007/01/10 12:10:14 iten Exp $
 */
public class MailSender
{
  private static final Logger LOG = Logger.getLogger(MailSender.class);

  public MailSender()
  {
    // lazy bone
  }

  /**
   * Methode sendMail() schickt ein Email über den angegebenen SMTP-Server<br>
   * Mehrere Empfänger und CC-Empfänger können durch "," getrennt angegeben werden.   *
   * @param smtpServer  Mailserver
   * @param to          Empfänger
   * @param from        Absender
   * @param subject     Titel
   * @param body        Nachricht
   * @return true on success otherwise false
   */
  public static boolean sendMail(String smtpServer, String to, String from, String subject, String body)
  {
    return sendMail(smtpServer, to, null, from, subject, body);
  }

  /**
   * Methode sendMail() schickt ein Email über den angegebenen SMTP-Server<br>
   * Mehrere Empfänger und CC-Empfänger können durch "," getrennt angegeben werden.
   * @param smtpServer  Mailserver (Name oder IP-Adresse)
   * @param to          Empfänger
   * @param cc          Carbon Copy (CC) Empfänger
   * @param from        Absender
   * @param subject     Titel
   * @param body        Nachricht
   * @return true on success otherwise false
   */
  public static boolean sendMail(String smtpServer, String to, String cc, String from, String subject, String body)
  {
    try
    {
      Properties props = System.getProperties();
      props.put("mail.smtp.host", smtpServer);
      Session session = Session.getDefaultInstance(props, null);

      // Neue Message erstellen
      Message msg = new MimeMessage(session);

      // From und To Felder setzen
      msg.setFrom(new InternetAddress(from));
      msg.setRecipients(Message.RecipientType.TO, InternetAddress.parse(to, false));
      // allfällige CC Empfänger setzen
      if (cc != null)
      {
        msg.setRecipients(Message.RecipientType.CC, InternetAddress.parse(cc, false));
      }
      // Titel und Nachricht setzen

      msg.setSubject(javax.mail.internet.MimeUtility.encodeText(subject, "ISO-8859-1", null));
      msg.setText( body );
      // zusätzliche Headerinformationen setzen
      msg.setSentDate(new Date());
      // Message senden
      Transport.send(msg);
      
      return true;
    }
    catch (Exception ex)
    {
      LOG.error("MailSender Methode sendMail(), Message konnte nicht gesendet werden. " +
                "Fehler " + ex.getMessage());
      
      return false;
    }
  }
}
