package ch.ige.ses.util;

import java.sql.SQLException;

import org.apache.log4j.Logger;

import ch.ige.ses.server.dao.ses.PropertyDAO;
import ch.ige.util.ADSHelper;

/**
 * Spezialisierung der Util-Klasse ADSHelper<br>
 * Liest anhand der EmployeeID die entsprechende Email-Adresse aus dem ADS.
 * Die Verbindungsinformationen zum LDAP-Server werden aus der SES PROPERTY
 * Tabelle gelesen.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author  Stefan Nydegger
 * @version $Id: SESADSHelper.java,v 1.10 2009/03/12 16:55:44 locher Exp $
 */
public class SESADSHelper extends ADSHelper
{
  private static final Logger LOG = Logger.getLogger( SESADSHelper.class );

  /**
   * Liest die Anmeldeinformationen für den LDAP-Server aus der SES-Tabelle PROPERTY aus und erstellt einen AD Helper.
   * @return the configured AD helper
   */
  public static SESADSHelper createSESADSHelper()
  {
    PropertyDAO propertyDAO = new PropertyDAO();
    try
    {
      String providerURL = propertyDAO.select( "adshelper.providerurl" ).value;
      String context = propertyDAO.select( "adshelper.context" ).value;
      String username = propertyDAO.select( "adshelper.username" ).value;
      String password = propertyDAO.select( "adshelper.password" ).value;
      String refFollwing = propertyDAO.select( "adshelper.ref_following" ).value;
      return new SESADSHelper( providerURL, context, username, password, refFollwing );
    }
    catch ( SQLException ex )
    {
      LOG.error( "ADSHelper Methode: getSESLogin(), Holen der Verbindungsinformationen " + "von SES-Datenbank fehlgeschlagen, Fehler: " + ex.getMessage() );
      throw new RuntimeException( ex );
    }
  }

  /**
   * Constructor
   */
  protected SESADSHelper( final String providerURL, final String context, final String username, final String password, final String referralFollowing )
  {
    super( providerURL, context, username, password, referralFollowing );
  }

  /**
   * Die Methode getEmailFromEmployeeID liest anhand der Employee-ID die
   * Emailadresse des entsprechenden Users aus dem Active Directory
   * @param employeeID  EmployeeID des Users
   * @return Email-Adresse des Users
   */
  public String getEmailFromEmployeeID( String employeeID )
  {
    try
    {

      String filter = "employeeID=" + employeeID;
      return getAttribute( filter, "mail" );
    }
    catch ( Exception ex )
    {
      LOG.error( "SESADSHelper Method getEmailFromEmployeeID: Auslesen der Email " + "Adresse von Active Directory fehlgeschlagen" );
      return null;
    }
  }

}
