package ch.ige.ses.vo.bagis;

import java.io.*;
import java.util.*;
import java.math.BigDecimal;
import ch.ige.ses.vo.ses.AmvRechnungVO;

/**
 * SES Value Object für eine Rechnung.
 * <p>
 * Copyright (c) 2001, Eidgenössisches Institut für Geistiges Eigentum
 * @author    Pirmin Suter, Thomas Iten
 * @version   $Id: SAPRechnungVO.java,v 1.7 2007/11/13 12:35:02 nydegger Exp $
 */

public class SAPRechnungVO implements Serializable
{
  //
  // const
  //
  public static final int      MAX_RECH_EMPF   = 20;
  public static final String[] SPRACH_CD_TYPES = {"D","F","E","I","USA"};
  public static final String[] DRUCK_ART_TYPES = {"1",    // sofort drucken
                                                  "2",    // batch druck
                                                  "3",    // druck patent jgb rechnungen
                                                  "4",    // druck patent jgb mahnungen
                                                  "5"};   // löschungsanzeige
  public static final String[] WAEHRUNG_TYPES  = {"CHF"};
  public static final String[] BOOLEAN_TYPES   = {"0","1"};

  public static final String STATUS_VERARBEITUNG_NEW     = "0";
  public static final String STATUS_VERARBEITUNG_SAP     = "1";
  public static final String STATUS_VERARBEITUNG_PRINTED = "9";
  public static final String STATUS_VERARBEITUNG_ERROR   = "E";


  //
  // fields
  //

  // Kopfdaten
  public int    bfakt_id          = 0;
  public int    bfakt_nr          = 0;                    // ACHTUNG: Wird nur für Datenübernahme verwendet !
  public Date   bfakt_dat         = null;                 // %%$$EreigDat  BFAKTUR.BFAKT_DAT
  public String schutit_nr        = "";

  public int    auftraggeber      = 0;
  public int    hauptRechEmpf     = 0;
  public int    regulierer        = 0;
  public ArrayList zusatzRechEmpf = new ArrayList();      // adr_nr Liste der zusätzlichen Empfänger (Integer)

  public String kontaktPerson     = "";                   // Kontaktperson Hauptrechnungsempfänger
  public String employeeId        = "";
  public String sprach_cd         = "";                   // SPRACH_CD_TYPES
  public String druck_art         = "";                   // DRUCK_ART_TYPES
  public String waehrung          = WAEHRUNG_TYPES[0];    // CHF
  public Date   verfall_dat       = null;                 // %%$$Frist	    BFAKTUR.VERFALL_DAT
  public String unsere_ref        = "";                   // BFAKTUR.UNSERE_REF ==> OUR_REF
  public String tel_nr            = "";                   // BFAKTUR.TEL_NR ==> TELEPHONE


  public String inhaber           = "";                   // BFAKTUR.INHABER ==> PROT_HOLDER
  public String kz_einschr        = BOOLEAN_TYPES[0];     // BARTIKEL.KZ_EINSCH ==> REG_MAIL

  public int    schutit_typ       = 0;
  public String schutit_typ_bez   = "";
  public String ihr_zeichen       = "";
  public String var_text          = "";                   // %%$$Text	    BFAKTUR.VAR_TEXT
  public String diversText        = "";                   // Text für Platzhalter %%$$Divers

  public String abteilung         = "";                   // BFORMTXT_DRUCK.TEXT_ART = 0 ==> DEPARTMENT
  public String kopfText          = "";                   // BFORMTXT_DRUCK.TEXT_ART = 1 & 2 ==> Z003
  public String beilagenText      = "";                   // BFORMTXT_DRUCK.TEXT_ART = 3 od. 4 ==> Z004
  public String belehrungText     = "";                   // BFORMTXT_DRUCK.TEXT_ART = 8 ==> Z005

  // Rechnungspositionen
  public ArrayList      posList   = new ArrayList();      // Liste mit RechnungsPositionVO

  // AMV Ergänzungen
  public int            amvLaufId               = -1;     // Id der AMV Verarbeitung
  public String         amvLaufNr               = "";     // Nr der AMV Verarbeitung
  public int            amvRechnungCd           = 0;      // Typ der AMV Rechnung (Erinnerung 1 oder 2)

  // JGB Ergänzungen
  public int            jgbLaufId               = -1;     // LaufId eines JGB-Laufs
  public String         jgbLaufNr               = "";     // LaufNr im Format "YYYYMMDD"
  public SAPJGBParamVO  jgbHauptRechEmpfParam   = null;
  public ArrayList      jgbZusatzRechEmpfParam  = new ArrayList();  // Liste mit SAPJGBParamVO

  // Interne Felder (keine Uebertragung an SAP)
  public String kto_nr              = "";                 // Info betreffend Dep-Kto-Belastung
  public String unt_name            = "";                 // %%$$Unterschr   BFAKTUR.UNT_NAME
  public Date   faellig_dat         = null;               // BFAKTUR.FAELLIG_DAT ==> DUE_DATE
  public String titel               = "";                 // %%$$Titel	    SCHUTIT.BESCHREIBUNG
  public String formart_cd          = "";                 // Formularcode für Formulartextaufbereitung
  public Date   schutz_beginn       = null;               // Feld für Platzhalter %%$$Schutzb

  public String status_verarbeitung = STATUS_VERARBEITUNG_NEW;
  public Date   zahl_dat            = null;
  public int    anz_kopie           = 0;
  public BigDecimal fakt_betrag     = null;
  public String hasChanged          = BOOLEAN_TYPES[0];   // "0" = Neue Rechnung anlegen
                                                          // "1" = Bestehende Rechnung mutieren

  public Date   created_dat         = null;               // Erstellungsdatum in SES-Tabelle
  public Date   changed_dat         = null;               // Datum Änderung in SES-Tabelle

  public String error_source        = "";                 // Quellenname(Prozess/EJB) der Fehlermeldung
  public String error_description   = "";                 // Fehlermeldung

                                                          // adr_nr Liste der nicht verarbeitete zusatzl. Rechn.-Empf.
  public ArrayList zusatzRechEmpfNichtVerarbeitet = new ArrayList();

  //---------------------------------------------------------------------------
  // constructors
  //---------------------------------------------------------------------------
  public SAPRechnungVO()
  {
    // lazy bone
  }

  //---------------------------------------------------------------------------
  // methods
  //---------------------------------------------------------------------------

  public boolean isRechnung()
  {
    if ((jgbLaufId == -1) && (amvLaufId == -1 ))
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public boolean isJGB()
  {
    if (jgbLaufId > -1)
    {
      return true;
    }
    else
    {
      return false;
    }
  }

  public boolean isAMV()
  {
    if (amvLaufId > -1)
    {
      return true;
    }
    else
    {
      return false;
    }
  }


  public String toString()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append ("KOPFDATEN \n");
    buf.append (this.toStringKopfdaten());

    buf.append ("RECHNUNGSPOSITIONEN \n");
    buf.append (this.toStringRechnungspositionen());

    buf.append ("AMV ERGAENZUNGEN\n");
    buf.append (this.toStringAmv());

    buf.append ("JGB ERGAENZUNGEN\n");
    buf.append (this.toStringJgb());

    buf.append ("INTERNE FELDER \n");
    buf.append (this.toStringIntern());


    return buf.toString();
  } // end toString


  public String toStringKopfdaten()
  {
    StringBuffer  buf = new StringBuffer();
    StringBuffer  tmp = new StringBuffer();


    buf.append ("  bfakt_id         = <"  + this.bfakt_id      + ">\n");
    buf.append ("  bfakt_nr         = <"  + this.bfakt_nr      + ">\n");

    if (bfakt_dat == null)
      buf.append ("  bfakt_dat        = null \n");
    else
      buf.append ("  bfakt_dat        = <"  + this.bfakt_dat     + ">\n");

    buf.append ("  schutit_nr       = <"  + this.schutit_nr    + ">\n");
    buf.append ("  auftraggeber     = <"  + this.auftraggeber  + ">\n");
    buf.append ("  regulierer       = <"  + this.regulierer    + ">\n");
    buf.append ("  hauptRechEmpf    = <"  + this.hauptRechEmpf + ">\n");

    if (zusatzRechEmpf.size() == 0)
    {
      tmp.append ("keine");
    }
    else
    {
      for (int i = 0; i < zusatzRechEmpf.size(); i++)
      {
        tmp.append("<" + zusatzRechEmpf.get(i) + "> ");
      }
    }
    buf.append ("  zusatzRechEmpf   = " + tmp.toString() + "\n");

    buf.append ("  kontaktPerson    = <"  + this.kontaktPerson + "> \n");
    buf.append ("  employeeID       = <"  + this.employeeId    + "> \n");
    buf.append ("  sprach_cd        = <"  + this.sprach_cd     + "> \n");
    buf.append ("  druck_art        = <"  + this.druck_art     + "> \n");
    buf.append ("  waehrung         = <"  + this.waehrung      + "> \n");

    if (verfall_dat == null)
      buf.append ("  verfall_dat      = null \n");
    else
      buf.append ("  verfall_dat      = <"  + this.verfall_dat   + ">\n");

    buf.append ("  unsere_ref       = <"  + this.unsere_ref      + "> \n");
    buf.append ("  tel_nr           = <"  + this.tel_nr          + "> \n");
    buf.append ("  inhaber          = <"  + this.inhaber         + "> \n");
    buf.append ("  kz_einschr       = <"  + this.kz_einschr      + "> \n");
    buf.append ("  schutit_typ      = <"  + this.schutit_typ     + "> \n");
    buf.append ("  schutit_typ_bez  = <"  + this.schutit_typ_bez + "> \n");
    buf.append ("  ihr_zeichen      = <"  + this.ihr_zeichen     + "> \n");
    buf.append ("  var_text         = <"  + this.var_text        + "> \n");
    buf.append ("  diversText       = <"  + this.diversText      + "> \n");
    buf.append ("  abteilung        = <"  + this.abteilung       + "> \n");
    buf.append ("  kopfText         = <"  + this.kopfText        + "> \n");
    buf.append ("  beilagenText     = <"  + this.beilagenText    + "> \n");
    buf.append ("  Belehrung        = <"  + this.belehrungText   + "> \n");

    return buf.toString();
  } // end


  public String toStringRechnungspositionen()
  {
    StringBuffer           buf = new StringBuffer();
    Iterator               it  = this.posList.iterator();
    SAPRechnungsPositionVO vo;

    if ( ! it.hasNext())
      buf.append ("  posList enthält keine Elemente ! \n");

    while ( it.hasNext() )
    {
      vo = (SAPRechnungsPositionVO) it.next();
      buf.append (vo.toString() + "\n");
    }

    return buf.toString();
  } // end


  public String toStringAmv()
  {
    StringBuffer  buf = new StringBuffer();

    buf.append ("  amvLaufId     = <"  + this.amvLaufId     + "> \n");
    buf.append ("  amvLaufNr     = <"  + this.amvLaufNr     + "> \n");
    buf.append ("  amvRechnungCd = <"  + this.amvRechnungCd + "> \n");

    return buf.toString();
  } // end


  public String toStringJgb()
  {
    StringBuffer  buf = new StringBuffer();
    Iterator      it  = this.jgbZusatzRechEmpfParam.iterator();
    SAPJGBParamVO vo;


    buf.append ("  jgbLaufNr = <"  + this.jgbLaufNr + "> \n");
		buf.append ("  jgbLaufID = <"  + this.jgbLaufId + "> \n");

    buf.append ("  jgbHauptRechEmpfParam: \n");
    if (jgbHauptRechEmpfParam == null)
    {
      buf.append ("    null \n");
    }
    else
    {
      buf.append (jgbHauptRechEmpfParam.toString());
    }

    buf.append ("  jgbZusatzRechEmpfParam: \n");
    if ( ! it.hasNext())
      buf.append ("    keine \n");

    while ( it.hasNext() )
    {
      vo = (SAPJGBParamVO) it.next();
      buf.append (vo.toString() + "\n");
    }

    return buf.toString();
  } // end


  public String toStringIntern()
  {
    StringBuffer  buf = new StringBuffer();
    StringBuffer  tmp = new StringBuffer();

    buf.append ("  kto_nr              = <" + this.kto_nr     + "> \n");
    buf.append ("  unt_name            = <" + this.unt_name   + "> \n");

    if (faellig_dat == null)
      buf.append ("  faellig_dat         = null \n");
    else
      buf.append ("  faellig_dat         = <"  + this.faellig_dat   + ">\n");

    buf.append ("  titel               = <" + this.titel      + "> \n");
    buf.append ("  formart_cd          = <" + this.formart_cd + "> \n");
    if (schutz_beginn == null)
      buf.append ("  schutz_beginn       = null \n");
    else
      buf.append ("  schutz_beginn       = <"  + this.schutz_beginn   + ">\n");

    buf.append ("  status_verarbeitung = <"  + this.status_verarbeitung + "> \n");

    if (zahl_dat == null)
      buf.append ("  zahl_dat            = null \n");
    else
      buf.append ("  zahl_dat            = <"  + this.zahl_dat   + ">\n");

    buf.append ("  anz_kopie           = <"  + this.anz_kopie     + "> \n");

    if (fakt_betrag == null)
      buf.append ("  fakt_betrag         = null \n");
    else
      buf.append ("  fakt_betrag         = <"  + this.fakt_betrag   + ">\n");

    buf.append ("  hasChanged          = <"  + this.hasChanged + "> \n");

    if (created_dat == null)
      buf.append ("  created_dat         = null \n");
    else
      buf.append ("  created_dat         = <"  + this.created_dat   + ">\n");

    if (changed_dat == null)
      buf.append ("  changed_dat         = null \n");
    else
      buf.append ("  changed_dat         = <"  + this.changed_dat   + ">\n");

    buf.append ("  error_source        = <"  + this.error_source       + "> \n");
    buf.append ("  error_description   = <"  + this.error_description  + "> \n");

    if (zusatzRechEmpfNichtVerarbeitet.size() == 0)
    {
      tmp.append ("keine");
    }
    else
    {
      for (int i = 0; i < zusatzRechEmpfNichtVerarbeitet.size(); i++)
      {
        tmp.append("<" + zusatzRechEmpfNichtVerarbeitet.get(i) + "> ");
      }
    }
    buf.append ("  zusatzRechEmpfNichtVerarbeitet = " + tmp.toString() + "\n");

    return buf.toString();
  } // end

} // end Class

