package ch.ige.ses.vo.ses;

import java.util.*;
import java.text.SimpleDateFormat;

/**
 * Statistik Calculations
 * Das Value Object beinhaltet die für die Satistik relevanten Daten,
 * welche bei der JGB Statistik ausgegeben werden.
 * <p>
 * Copyright (c) 2003, Eidgenössisches Institut für Geistiges Eigentum
 * @author Thomas Iten
 * @version $Id: StatistikCalcVO.java,v 1.5 2006/12/13 14:58:39 iten Exp $
 */
public class StatistikCalcVO
{
  //
  // fields
  //
  private JGBLaufVO   lauf     = null;          // Daten der Tabelle SES.JGB_LAUF
  private String      laufzeit = "";            // Berechnete Laufzeit

  // BAGIS Daten
  public  int     bagFakturen     = 0;          // Anzahl Fakturaanforderungen
  public  int     bagRe           = 0;          // Anzahl Rechnungsempfänger

  // SES Daten
  public  int     sesFakturen     = 0;          // Anzahl Fakturaanforderungen
  public  int     sesRe           = 0;          // Anzahl Rechnungsempfänger
  public  int     sesDruckperi    = 0;          // Anzahl Druckperioden
  public  int     sesMinBfaktId   = 0;          // Minimale bfakt_id
  public  int     sesMaxBfaktId   = 0;          // Maximale bfakt_id
  private String  sesStatSES      = null;       // Status BAGIS > SES(SES_STAT)
  private String  sesStatSAP      = null;       // Status SES > SAP (SAP_STAT)
  public  int     sesReAnSapOK    = 0;          // Anzahl Rechnungsempfänger an SAP OK
  public  int     sesReAnSapNOK   = 0;          // Anzahl Rechnungsempfänger an SAP NOK

  // Statistik Vergleich SES <> SAP
  public  int     vglFakturen     = 0;          // Anzahl Fakturaanforderungen
  public  int     vglBagisStatus  = 0;          // - mit BAGIS-Status
  public  int     vglReFa         = 0;          // Anzahl Rechnungsempfänger (FA)
  public  int     vglInland       = 0;          // - davon Inland
  public  int     vglAusland      = 0;          // - davon Ausland
  public  int     vglLA7          = 0;          // - davon mit Listenart 7
  public  int     vglReJgb        = 0;          // Anzahl Rechnungsempfänger (JGB)

  //
  // internal fields
  //
  private Date             date = new Date();
  private SimpleDateFormat sdf  = new SimpleDateFormat("dd.MM.yyyy HH:mm:ss");

  //----------------------------------------------------------------------------
  // constructors
  //----------------------------------------------------------------------------

  public StatistikCalcVO()
  {
    // lazy bone
  }

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------

  /**
   * Daten des JGB Lauf setzen. Gleichzeitig wird die Laufzeit berechnet
   * und der Ses und Sap Status als String Representation gesetzt.
   *
   * @param lauf JGBLaufVO  Lauf Daten
   */
  public void setLauf(JGBLaufVO lauf)
  {
    // set lauf data
    this.lauf = lauf;

    // berechne laufzeit
    if (lauf.verarbStart != null && lauf.verarbEnde != null)
    {
      laufzeit = calcDelta(lauf.verarbStart, lauf.verarbEnde);
    }

    // stati setzen
    setStatusSES(lauf.sesStat);
    setStatusSAP(lauf.sapStat);
  }


  /**
   * Statistikdaten aufbereiten und Rückgabe als String.
   *
   * @return  Statistik
   */
  public String toString()
  {
    StringBuffer buf = new StringBuffer();

    // Kopfdaten aufbereiten
    buf.append("SES JGB Statistik - Lauf Nr: " + lauf.laufNr + " - Lauf Id: " + lauf.jgbLaufId +" \n");
    buf.append("erstellt am " + sdf.format(date) + "\n");
    buf.append(getFillers("=",50) + "\n");
    buf.append("Verarbeitung Start \t: " + (lauf.verarbStart == null ? "null" : sdf.format(lauf.verarbStart)) + "\n");
    buf.append("Verarbeitung Ende  \t: " + (lauf.verarbEnde  == null ? "null" : sdf.format(lauf.verarbEnde))  + "\n");
    buf.append("Laufzeit           \t: " + laufzeit + "\n");
    buf.append(getFillers("=",50) + "\n");

    // BAGIS
    buf.append("\n*** BAGIS *** \n");
    buf.append("Anzahl Fakturaanforderungen    \t\t: " + bagFakturen    + "\n");
    buf.append("Anzahl Rechnungsempfänger (FA) \t\t: " + bagRe          + "\n");

    // SES - Daten aufbereiten
    buf.append("\n*** SES *** \n");
    buf.append("Anzahl Fakturaanforderungen    \t\t: " + sesFakturen    + "\n");
    buf.append("Anzahl Rechnungsempfänger (FA) \t\t: " + sesRe          + "\n");
    buf.append("Anzahl Druckperioden           \t\t: " + sesDruckperi   + "\n");
    buf.append("Minimale bfakt_id            \t\t\t: " + sesMinBfaktId  + "\n");
    buf.append("Maximale bfakt_id              \t\t: " + sesMaxBfaktId  + "\n");

    // SES Detail 1 aufbereiten
    buf.append("\n*** SES Detail 1 - BAGIS nach SES *** \n");
    buf.append("Status (SES_STAT)                             \t\t: " + sesStatSES                    + "\n");
    buf.append("Anzahl Fakturaanforderungen von BAGIS gelesen   \t: " + lauf.anzahlBfakturGelesen     + "\n");
    buf.append("Anzahl Fakturaanforderungen auf SES gespeichert \t: " + lauf.countBfakturToSap        + "\n");
    buf.append("Anzahl Rechnungsempf. (FA)  auf SES gespeichert \t: " + lauf.anzahlBfaktjgbGeneriert  + "\n");
    buf.append("Letzte bfakt_id (an SapInbound gesendet)        \t: " + lauf.minBfakturToSap          + "\n");

    // Ses Details 2 aufbereiten
    buf.append("\n*** SES Detail 2 - SES nach SAP *** \n");
    buf.append("Status (SAP_STAT)                             \t\t: " + sesStatSAP                    + "\n");
    buf.append("Anzahl Fakturaanforderungen an SAP              \t: " + lauf.anzahlBfakturAnSap       + "\n");
    buf.append(" - davon erfolgreich verarbeitet (OK)           \t: " + lauf.anzahlBfakturAnSapOk     + "\n");
    buf.append(" - davon nicht verarbeitet (NOK)                \t: " + lauf.anzahlBfakturAnSapNok    + "\n");
    buf.append("Anzahl Rechnungsempfänger (FA) an SAP           \t: " + lauf.anzahlBfaktjgbAnSap      + "\n");
    buf.append(" - davon erfolgreich verarbeitet (OK)           \t: " + sesReAnSapOK                  + "\n");
    buf.append(" - davon nicht verarbeitet (NOK)                \t: " + sesReAnSapNOK                 + "\n");

    // Statistik Vergleich mit SAP
    buf.append("\n*** Statistik Vergleich SES <> SAP *** \n");
    buf.append("Anzahl Fakturaanforderungen    \t\t: " + vglFakturen    + "\n");
    buf.append(" - mit BAGIS-Status            \t\t: " + vglBagisStatus + "  (Inland ohne LA7) \n");
    buf.append("Anzahl Rechnungsempfänger (FA) \t\t: " + vglReFa        + "\n");
    buf.append(" - davon Inland (CH/LI)        \t\t: " + vglInland      + "\n");
    buf.append(" - davon Ausland             \t\t\t: " + vglAusland     + "\n");
    buf.append(" - davon mit Listenart 7       \t\t: " + vglLA7         + "\n");
    buf.append("Anzahl Rechnungsempfänger (JGB)\t\t: " + vglReJgb       + "  (Inland ohne LA7) \n");

    return buf.toString();
  }

  //----------------------------------------------------------------------------
  // methods
  //----------------------------------------------------------------------------

  /**
   * Ses Status (feld se1Status) als Textmeldung setzten.
   *
   * @param   status  Statuswert
   */
  private void setStatusSES (int status)
  {
    switch (status)
    {
      case JGBLaufVO.SES_STAT_STARTED:
        sesStatSES = status + " - " + "STARTED";
        break;
      case JGBLaufVO.SES_STAT_REQUEST_STOP:
        sesStatSES = status + " - " + "REQUEST_STOP";
        break;
      case JGBLaufVO.SES_STAT_STOPPED:
        sesStatSES = status + " - " + "STOPPED";
        break;
      case JGBLaufVO.SES_STAT_FINISHED:
        sesStatSES = status + " - " + "FINISHED";
        break;
    }
  }


  /**
   * Sap Status (feld se2Status) als Textmeldung setzten.
   *
   * @param   status  Statuswert
   */
  private void setStatusSAP (int status)
  {
    switch (status)
    {
      case JGBLaufVO.SAP_STAT_NOT_STARTED:
        sesStatSAP = status + " - " + "NOT_STARTED";
        break;
      case JGBLaufVO.SAP_STAT_STARTED:
        sesStatSAP = status + " - " + "STARTED";
        break;
      case JGBLaufVO.SAP_STAT_STARTED_LAST:
        sesStatSAP = status + " - " + "STARTED_LAST";
        break;
      case JGBLaufVO.SAP_STAT_STOPPED:
        sesStatSAP = status + " - " + "STOPPED";
        break;
      case JGBLaufVO.SAP_STAT_FINISHED:
        sesStatSAP = status + " - " + "FINISHED";
        break;
    }
  }


  /**
   * Gibt die Differenz zwischen zwei Daten zurück
   *
   * @param   von   Von - Datum
   * @param   bis   Bis - Datum
   * @return        Differenz in Stunden und Minuten
   */
  private String calcDelta(Date von, Date bis)
  {
    String retVal;
    int fDay = 0;
    int fHour = 0;
    int fMin = 0;
    int fSec = 0;
    int fMs = 0;

    long delta = bis.getTime() - von.getTime();
    long rest  = 0;

    fDay  = (int) (delta / (1000 * 60 * 60 *24));
    rest = fDay * 24 * 60 * 60 * 1000;
    fHour = (int) ((delta - rest)  / (1000 * 60 * 60));
    rest = rest + fHour * 60 * 60 * 1000;
    fMin  = (int) ((delta - rest ) / (1000 * 60));
    rest = rest + fMin * 60 * 1000;
    fSec  = (int) ((delta - rest ) / 1000);
    rest = rest + fSec * 1000;
    fMs   = (int) (delta - rest);

    if (fDay == 0)
    {
      retVal = fHour + " Std, " + fMin + " Min";
    }
    else
    {
      retVal = fDay + " Tag(e), " + fHour + " Std, " + fMin + " Min";
    }

    return retVal;
  }


  /**
   * Wiederholt ein Zeichen n mal und gibt diese Filler als String zurück.
   *
   * @param   zeichen   Zeichen, welches wiederholt werden soll
   * @param   anzahl    wie viele Male soll das Zeichen wiederholt werden
   * @return            ein String mit Fillern
   */
  private String getFillers(String zeichen, int anzahl)
  {
    StringBuffer buf = new StringBuffer();

    for (int i = 0; i < anzahl; i++)
    {
      buf.append(zeichen);
    }

    return buf.toString();
  }

} // end class


