package ch.ige.wdltool.checker;

import junit.framework.TestCase;

import org.apache.log4j.BasicConfigurator;

import ch.ige.dbutils.util.ClientInitializer;
import ch.ige.ipobjects.ServicesFactory;
import ch.ige.util.date.DateConverter;

/**
 * Tests {@link ch.ige.wdltool.checker.BatchChecker}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: BatchCheckerTest.java,v 1.3 2007/10/30 09:57:26 lewis_lee Exp $
 */
public class BatchCheckerTest extends TestCase
{
  static
  {
    BasicConfigurator.configure();
    ClientInitializer.initStandaloneClient();
  }

  // @see junit.framework.TestCase#setUp()
  @Override
  protected void setUp() throws Exception
  {
    super.setUp();
    ServicesFactory.getStandaloneServices();
  }

  /**
   * The only constructor.
   * @param name The test's name
   */
  public BatchCheckerTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.BatchChecker#checkWdlForApplication(java.util.Date, java.util.Date)}.
   * @throws Exception if the test fails
   */
  public void testCheckWdlForApplication() throws Exception
  {
    String[] results = BatchChecker.checkWdlForApplication( DateConverter.stringToDate( "01.02.2006" ), DateConverter.stringToDate( "01.02.2006" ) );
    assertEquals( "Checking number of files", 2, results.length );
    assertTrue( "Checking for summary title", results[1].contains( "ZUSAMMENFASSUNG FÜR MARKEN MIT ANMELDUNG VON 01.02.2006 BIS 01.02.2006" ) );
    System.out.println( results[0] );
    System.out.println( results[1] );
  }

  /**
   * Tests {@link ch.ige.wdltool.checker.BatchChecker#checkWdlForRegistration(java.util.Date, java.util.Date)}.
   * @throws Exception if the test fails
   */
  public void testCheckWdlForRegistration() throws Exception
  {
    String[] results = BatchChecker.checkWdlForRegistration( DateConverter.stringToDate( "31.01.2005" ), DateConverter.stringToDate( "07.02.2005" ) );
    assertEquals( "Checking number of files", 2, results.length );
    assertTrue( "Checking for summary title", results[1].contains( "ZUSAMMENFASSUNG FÜR MARKEN MIT EINTRAGUNG VON 31.01.2005 BIS 07.02.2005" ) );
    System.out.println( results[0] );
    System.out.println( results[1] );
  }
}
