package ch.ige.wdltool.dao;

import java.sql.SQLException;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.wdltool.vo.AnalyzedVO;

/**
 * Tests {@link ch.ige.wdltool.dao.AnalyzedDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Christian Heinzer
 * @version  $Id: AnalyzedDAOTest.java,v 1.1 2008/04/18 09:40:21 heinzer Exp $
 */
public class AnalyzedDAOTest extends BaseDAOWithCRUDTestTemplate<AnalyzedDAO, AnalyzedVO, Integer>
{
  /**
   * A schutit key that must not exist in the list of analyzed trademarks.
   */
  private static final int NON_EXISTING_TM_KEY = 1202186498; //is a patent key - so it can never appear

  /**
   * Public constructor
   * @param name The test's name
   */
  public AnalyzedDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.AnalyzedDAO#contains(int)} as well as {@link ch.ige.wdltool.dao.AnalyzedDAO#insert(AnalyzedVO)}.
   * @throws Exception if the test fails
   */
  public void testContains() throws Exception
  {
    boolean hasBeedAnalyzed = getDAO().contains( NON_EXISTING_TM_KEY );
    assertTrue( "Non existing trademark is not in the list of analyzed key", !hasBeedAnalyzed );

    AnalyzedVO dummyAnalyzedVO = new AnalyzedVO( NON_EXISTING_TM_KEY );

    Integer insertedID = getDAO().insert( dummyAnalyzedVO );
    setSavedId( insertedID );

    hasBeedAnalyzed = getDAO().contains( NON_EXISTING_TM_KEY );
    assertTrue( "Newly inserted trademark is in the list of analyzed key", hasBeedAnalyzed );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<AnalyzedDAO> getDAOClass()
  {
    return AnalyzedDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @SuppressWarnings( "unused" )
  @Override
  protected void delete( Integer stopwordId ) throws SQLException
  {
    getDAO().delete( stopwordId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @SuppressWarnings( "unused" )
  @Override
  protected AnalyzedVO getSerializableVO() throws SQLException
  {
    return new AnalyzedVO( 123 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected boolean assertVOEqual( AnalyzedVO originalVO, AnalyzedVO serializedVO )
  {
    return originalVO.getIPKey().equals( serializedVO.getIPKey() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @SuppressWarnings( "unused" )
  @Override
  protected void compareVOs( AnalyzedVO originalVO, AnalyzedVO loadedVO )
  {
    throw new IllegalStateException( "no implemented" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @SuppressWarnings( "unused" )
  @Override
  protected Integer insert( AnalyzedVO vo ) throws SQLException
  {
    throw new IllegalStateException( "no implemented" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @SuppressWarnings( "unused" )
  @Override
  protected void modifyVOBeforeUpdate( AnalyzedVO vo )
  {
    throw new IllegalStateException( "no implemented" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @SuppressWarnings( "unused" )
  @Override
  protected AnalyzedVO read( Integer id ) throws SQLException
  {
    throw new IllegalStateException( "no implemented" );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @SuppressWarnings( "unused" )
  @Override
  protected void update( AnalyzedVO vo ) throws SQLException
  {
    throw new IllegalStateException( "no implemented" );
  }
}