package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.apache.commons.lang.ObjectUtils;

import ch.ige.dbutils.dao.DAOPool;
import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.vo.AbstractTermVO;
import ch.ige.wdltool.vo.NiceTermGroupVO;
import ch.ige.wdltool.vo.NiceTermVO;
import ch.ige.wdltool.vo.TermReplicationVO;

/**
 * Tests {@link ch.ige.wdltool.dao.NiceTermDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: NiceTermDAOTest.java,v 1.19 2009/04/17 09:41:17 bandschapp Exp $
 */
public class NiceTermDAOTest extends BaseDAOWithCRUDTestTemplate<NiceTermDAO, NiceTermVO, Integer>
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public NiceTermDAOTest( String name )
  {
    super( name );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<NiceTermDAO> getDAOClass()
  {
    return NiceTermDAO.class;
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    NiceTermVO niceTermVO = createAndInsert( true );
    NiceTermVO expectedVO = getSerializableVO();
    expectedVO.setTermGroupId( null );
    expectedVO.setIdentifier( niceTermVO.getIdentifier() );
    defaultTestRead( expectedVO, -1, 999999999, niceTermVO.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    List<NiceTermVO> terms = getDAO().findAll();
    assertNotNull( "Checking List", terms );
    assertTrue( "Checking for content", !terms.isEmpty() );
    assertTrue( "Checking number of terms", terms.size() > 3000 ); // This may change!
    for ( int i = 0; i < terms.size(); i++ )
    {
      // This may look stupid, but it didn't work...
      assertEquals( "Checking class", NiceTermVO.class, terms.get( i ).getClass() );
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#findGenericTerms()}.
   * @throws Exception if the test fails
   */
  public void testFindGenericTerms() throws Exception
  {
    List<NiceTermVO> terms = getDAO().findGenericTerms();
    assertNotNull( "Checking List of Generic Terms", terms );
    assertTrue( "Checking for content", !terms.isEmpty() );
    assertTrue( "Checking minimum number of terms", terms.size() > 10 ); // This will change over time!
    assertTrue( "Checking maximum number of terms", terms.size() < 20 ); // This will change over time!

    for ( int i = 0; i < terms.size(); i++ )
    {
      // This may look stupid, but it didn't work...
      assertEquals( "Checking class", NiceTermVO.class, terms.get( i ).getClass() );
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#insert(ch.ige.wdltool.vo.NiceTermVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#update(ch.ige.wdltool.vo.NiceTermVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected NiceTermVO read( Integer termId ) throws SQLException
  {
    return getDAO().find( termId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  protected Collection<NiceTermVO> read( Object... searchParameters ) throws SQLException
  {
    Map<Integer, List<AbstractTermVO>> termMap = DAOPool.get( GeneralTermDAO.class ).findPositiveTerms(
        (Locale) searchParameters[0], (Integer) searchParameters[1] );
    List<NiceTermVO> terms = new ArrayList<NiceTermVO>( termMap.size() );
    for ( Integer classNumber: termMap.keySet() )
    {
      List<AbstractTermVO> allTerms = termMap.get( classNumber );
      for ( AbstractTermVO oneTerm: allTerms )
      {
        if ( oneTerm instanceof NiceTermVO )
        {
          terms.add( (NiceTermVO) oneTerm );
        }
      }
    }
    return terms;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( NiceTermVO originalVO, NiceTermVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#validateVO(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void validateVO( NiceTermVO loadedVO )
  {
    assertTrue( "Checking change date", loadedVO.getChangeDate().after( GeneralTermDAOTest.getLatestMigration() ) );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#assertVOEqual(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public boolean assertVOEqual( NiceTermVO originalVO, NiceTermVO serializedVO )
  {
    return super.assertVOEqual( originalVO, serializedVO )
        && ObjectUtils.equals( originalVO.getChangeDate(), serializedVO.getChangeDate() )
        && ObjectUtils.equals( originalVO.getNiceTermGroupNumber(), serializedVO.getNiceTermGroupNumber() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( NiceTermVO originalVO, NiceTermVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking class number", originalVO.getClassNumber(), loadedVO.getClassNumber() );
    assertEquals( "Checking language", originalVO.getLanguage(), loadedVO.getLanguage() );
    assertEquals( "Checking term", originalVO.getTerm(), loadedVO.getTerm() );
    assertEquals( "Checking private comment", originalVO.getPrivateComment(), loadedVO.getPrivateComment() );
    assertEquals( "Checking public comment", originalVO.getPublicComment(), loadedVO.getPublicComment() );
    assertEquals( "Checking creation user", originalVO.getCreationUser(), loadedVO.getCreationUser() );
    assertEquals( "Checking change user", originalVO.getChangeUser(), loadedVO.getChangeUser() );
    assertEquals( "Checking data source", originalVO.getDataSource(), loadedVO.getDataSource() );
    assertEquals( "Checking search term", originalVO.getSearchTerm(), loadedVO.getSearchTerm() );
    assertEquals( "Checking term group ID", originalVO.getTermGroupId(), loadedVO.getTermGroupId() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeInsert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeInsert( NiceTermVO niceTermVO )
  {
    // The term group must exist otherwise
    niceTermVO.setTermGroupId( null );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( NiceTermVO niceTermVO )
  {
    return getDAO().insert( niceTermVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void modifyVOBeforeUpdate( NiceTermVO niceTermVO )
  {
    niceTermVO.setClassNumber( 33 );
    niceTermVO.setLanguage( Locale.ITALIAN );
    niceTermVO.setTerm( "Use me" );
    niceTermVO.setSearchTerm( "use me" );
    niceTermVO.setPrivateComment( "The private comment, modified" );
    niceTermVO.setPublicComment( "The public comment, modified" );
    try
    {
      List<NiceTermGroupVO> niceTermGroups = DAOPool.get( NiceTermGroupDAO.class ).findAll();
      for ( NiceTermGroupVO niceTermGroup: niceTermGroups )
      {
        // The Nice term group number must match the class number.
        if ( niceTermGroup.getNiceTermNumber() / 10000 == niceTermVO.getClassNumber().intValue() )
        {
          niceTermVO.setTermGroupId( niceTermGroup.getIdentifier() );
          niceTermVO.setNiceTermGroupNumber( niceTermGroup.getNiceTermNumber() );
          break;
        }
      }
    }
    catch ( SQLException e )
    {
      // Too bad...
    }
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( NiceTermVO niceTermVO )
  {
    getDAO().update( niceTermVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer termId )
  {
    getDAO().delete( termId );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermDAO#getReplicationTerms(String)}.
   * @throws Exception if the test fails
   */
  public void testGetReplicationTerms() throws Exception
  {
    TermReplicationVO niceTermsToReplicate = getDAO().getReplicationTerms( "wdldmz" );
    assertNotNull( "Checking terms to replicate", niceTermsToReplicate );
    assertNotNull( "Checking SQL statement", niceTermsToReplicate.getInsertStatement() );
    assertNotNull( "Checking terms", niceTermsToReplicate.getTermParameters() );
    assertTrue( "Checking number of terms", niceTermsToReplicate.getTermParameters().size() > 0 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  public NiceTermVO getExpectedVO() throws ParseException
  {
    NiceTermVO term = new NiceTermVO();
    term.setIdentifier( 38490 ); // This may change!
    term.setClassNumber( 45 );
    term.setLanguage( Locale.GERMAN );
    term.setTerm( "Hüten von Haustieren" );
    term.setPrivateComment( null );
    term.setPublicComment( null );
    term.setChangeDate( DateConverter.stringToTimestamp( "07.11.2007 14:37:31.000" ) );
    term.setCreationUser( "benninger" );
    term.setChangeUser( "migration script" );
    term.setGeneric( false );
    term.setSearchTerm( "huten von haustiren" );
    term.setTermGroupId( 15678 );
    return term;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  public NiceTermVO getSerializableVO() throws SQLException
  {
    try
    {
      NiceTermVO term = new NiceTermVO();
      term.setIdentifier( 33 );
      term.setClassNumber( 42 );
      term.setLanguage( Locale.FRENCH );
      term.setTerm( "Dont use me" );
      term.setPrivateComment( "The private comment" );
      term.setPublicComment( "The public comment" );
      term.setCreationUser( "dummy" );
      term.setChangeDate( DateConverter.stringToTimestamp( "03.02.2008 18:44:55.000" ) );
      term.setChangeUser( "billg" );
      term.setGeneric( false );
      term.setSearchTerm( "dont use me" );
      term.setTermGroupId( 15 );
      return term;
    }
    catch ( ParseException e )
    {
      throw new SQLException( e.getMessage() );
    }
  }
}
