package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.Collection;

import ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate;
import ch.ige.wdltool.vo.NiceTermGroupVO;

/**
 * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO}.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Esther Bandschapp
 * @version  $Id: NiceTermGroupDAOTest.java,v 1.11 2008/04/29 16:38:12 heinzer Exp $
 */
public class NiceTermGroupDAOTest extends BaseDAOWithCRUDTestTemplate<NiceTermGroupDAO, NiceTermGroupVO, Integer>
{
  /**
   * The only constructor
   * @param name The test's name
   */
  public NiceTermGroupDAOTest( String name )
  {
    super( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO#insert(NiceTermGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testInsert() throws Exception
  {
    defaultTestCreate( true );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO#find(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testFind() throws Exception
  {
    NiceTermGroupVO termGroupVO = createAndInsert( true );
    defaultTestRead( termGroupVO, -1, 999999999, termGroupVO.getIdentifier() );
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO#findByNumber(Integer)}.
   * @throws Exception if the test fails
   */
  public void testFindByNumber() throws Exception
  {
    try
    {
      getDAO().findByNumber( null );
      fail( "Null number must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    try
    {
      getDAO().findByNumber( -1 );
      fail( "Negative number must fail" );
    }
    catch ( IllegalArgumentException e )
    {
      // This is expected
    }

    NiceTermGroupVO niceTermGroup = getDAO().findByNumber( 999999999 );
    assertNull( "Checking for unknown term group number", niceTermGroup );

    niceTermGroup = getDAO().findByNumber( 90634 ); // This may change!
    assertNotNull( "Checking for known term group number", niceTermGroup );
    assertEquals( "Checking ID for known term group number", 10712, niceTermGroup.getIdentifier().intValue() ); // This may change!
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO#findAll()}.
   * @throws Exception if the test fails
   */
  public void testFindAll() throws Exception
  {
    NiceTermGroupVO termGroupVO = createAndInsert( true );
    defaultTestReadCollection( termGroupVO, 4000 );

    Collection<NiceTermGroupVO> termGroups = read( new Object[] {} );
    int previousTermGroupNumber = 0;
    for ( NiceTermGroupVO termGroup: termGroups )
    {
      assertTrue( "Checking that the term groups are ordered by term group number", termGroup.getNiceTermNumber() > previousTermGroupNumber );
      previousTermGroupNumber = termGroup.getNiceTermNumber().intValue();
    }
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO#update(NiceTermGroupVO)}.
   * @throws Exception if the test fails
   */
  public void testUpdate() throws Exception
  {
    defaultTestUpdate();
  }

  /**
   * Tests {@link ch.ige.wdltool.dao.NiceTermGroupDAO#delete(java.lang.Integer)}.
   * @throws Exception if the test fails
   */
  public void testDelete() throws Exception
  {
    defaultTestDelete( -1, 999999999 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getDAOClass()
  @Override
  protected Class<NiceTermGroupDAO> getDAOClass()
  {
    return NiceTermGroupDAO.class;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#insert(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Integer insert( NiceTermGroupVO termGroupVO )
  {
    return getDAO().insert( termGroupVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOs(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOs( NiceTermGroupVO originalVO, NiceTermGroupVO loadedVO )
  {
    assertEquals( "Checking ID", originalVO.getIdentifier(), loadedVO.getIdentifier() );
    assertEquals( "Checking Nice term number", originalVO.getNiceTermNumber(), loadedVO.getNiceTermNumber() );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  public void compareVOsAndValidateLoaded( NiceTermGroupVO originalVO, NiceTermGroupVO loadedVO )
  {
    super.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getSerializableVO()
  @Override
  protected NiceTermGroupVO getSerializableVO()
  {
    NiceTermGroupVO termGroup = new NiceTermGroupVO();
    termGroup.setIdentifier( 44 );
    termGroup.setNiceTermNumber( 12345 );
    return termGroup;
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#modifyVOBeforeUpdate(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void modifyVOBeforeUpdate( NiceTermGroupVO termGroupVO )
  {
    termGroupVO.setNiceTermNumber( 387654 );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.io.Serializable)
  @Override
  protected NiceTermGroupVO read( Integer termGroupId ) throws SQLException
  {
    return getDAO().find( termGroupId );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#read(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  protected Collection<NiceTermGroupVO> read( Object... searchParameters ) throws SQLException
  {
    return getDAO().findAll();
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#update(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void update( NiceTermGroupVO termGroupVO )
  {
    getDAO().update( termGroupVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#delete(java.io.Serializable)
  @Override
  protected void delete( Integer termGroupId )
  {
    getDAO().delete( termGroupId );
  }
}
