package ch.ige.wdltool.dao;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import ch.ige.util.validation.Check;
import ch.ige.wdltool.vo.StandardWdlTermGroupVO;
import ch.ige.wdltool.vo.TermGroupVO;

/**
 * Accesses the table <code>TERM_GROUP</code> and deals with the standard WDL term group data.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   bandschapp
 * @version  $Id: StandardWdlTermGroupDAO.java,v 1.11 2008/03/31 09:58:12 lewis_lee Exp $
 */
public class StandardWdlTermGroupDAO extends BaseTermGroupDAO<StandardWdlTermGroupVO>
{
  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getValues(ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected Object[] getValues( StandardWdlTermGroupVO vo )
  {
    return new Object[] { vo.getName(), null, null };
  }

  // @see ch.ige.dbutils.dao.AbstractWdlDAO#getVOType()
  @Override
  protected Class<StandardWdlTermGroupVO> getVOType()
  {
    return StandardWdlTermGroupVO.class;
  }

  /**
   * Inserts a standard WDL term group.
   * @param standardWdlTermGroup The standard WDL term group to insert
   * @return The ID of the inserted standard WDL term group
   */
  public Integer insert( final StandardWdlTermGroupVO standardWdlTermGroup )
  {
    Integer id = super.insert( standardWdlTermGroup );
    clearCache();
    return id;
  }

  /**
   * Finds one standard WDL term group.
   * @param standardWdlTermGroupId The ID of this standard WDL term group
   * @return The standard WDL term group
   * @throws SQLException in case the database access is not successful
   */
  public StandardWdlTermGroupVO find( final Integer standardWdlTermGroupId ) throws SQLException
  {
    Check.notNegative( standardWdlTermGroupId, "Standard WDL term group ID" );
    return (StandardWdlTermGroupVO) getTermGroups( StandardWdlTermGroupVO.class ).get( standardWdlTermGroupId );
  }

  /**
   * Finds all the standard WDL term groups.
   * @return The standard WDL term groups ordered by ID
   * @throws SQLException in case the database access is not successful
   */
  public List<StandardWdlTermGroupVO> findAll() throws SQLException
  {
    Collection<TermGroupVO> rawTermGroups = getTermGroups( StandardWdlTermGroupVO.class ).values();
    List<StandardWdlTermGroupVO> termGroups = new ArrayList<StandardWdlTermGroupVO>( rawTermGroups.size() );
    for ( TermGroupVO rawTermGroup: rawTermGroups )
    {
      termGroups.add( (StandardWdlTermGroupVO) rawTermGroup );
    }
    return termGroups;
  }

  /**
   * Updates a standard WDL term group.
   * @param standardWdlTermGroup The standard WDL term group to be updated
   */
  public void update( final StandardWdlTermGroupVO standardWdlTermGroup )
  {
    super.update( standardWdlTermGroup );
    clearCache();
  }

  /**
   * Deletes a standard WDL term group.
   * @param standardWdlTermGroupId The ID of the standard WDL term group to be deleted
   */
  public void delete( final Integer standardWdlTermGroupId )
  {
    super.delete( standardWdlTermGroupId );
    clearCache();
  }
}
