package ch.ige.wdltool.ejb;

import java.text.ParseException;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import ch.ige.util.date.DateConverter;
import ch.ige.wdltool.dao.DataSourceDAO;
import ch.ige.wdltool.dao.DataSourceDAOTest;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.vo.DataSourceVO;

/**
 * Tests {@link ch.ige.wdltool.ejb.WdlServices} for data sources.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlServicesForDataSourceTest.java,v 1.8 2008/04/29 16:38:12 heinzer Exp $
 */
public class WdlServicesForDataSourceTest extends BaseServiceTestTemplate<DataSourceDAO, DataSourceVO, DataSource, WdlServices>
{
  /**
   * The unit under test
   */
  private WdlServices wdlServices;

  /**
   * Used for utility stuff
   */
  private DataSourceDAOTest dataSourceDAOTest;

  /**
   * The only constructor.
   * @param name The test's name
   */
  public WdlServicesForDataSourceTest( String name )
  {
    super( name );
    this.wdlServices = WdlServicesFactory.getStandaloneWdlServices();
    this.dataSourceDAOTest = new DataSourceDAOTest( name );
  }

  /**
   * Tests {@link ch.ige.wdltool.ejb.WdlServices#findAllDataSources()}.
   * @throws Exception if the test fails
   */
  public void testFindAllDataSources() throws Exception
  {
    defaultTestReadCollection( getExpectedVO(), 3, (Object[]) null );

    Map<DataSource, DataSourceVO> dataSources = findAllDataSources();
    // There's not much to do, really...
    assertEquals( "Check number of data sources", 3, dataSources.size() );
    assertTrue( "Checking IPI", dataSources.containsKey( DataSource.IPI ) );
    assertTrue( "Checking Nice", dataSources.containsKey( DataSource.NICE ) );
    assertTrue( "Checking WIPO", dataSources.containsKey( DataSource.WIPO ) );
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.lang.Object[])
  @Override
  @SuppressWarnings( "unused" )
  public Collection<DataSourceVO> readWithService( Object... searchParameters ) throws Exception
  {
    return getServices().findAllDataSources();
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#readWithService(java.io.Serializable)
  @Override
  public DataSourceVO readWithService( DataSource dataSourceType ) throws Exception
  {
    List<DataSourceVO> allDataSources = getServices().findAllDataSources();
    for ( DataSourceVO dataSource: allDataSources )
    {
      if ( dataSource.getIdentifier().equals( dataSourceType ) )
      {
        return dataSource;
      }
    }
    return null;
  }

  /**
   * Finds all data sources and puts them into a Map.
   * @return The data sources in a Map
   * @throws Exception if the query fails
   */
  private Map<DataSource, DataSourceVO> findAllDataSources() throws Exception
  {
    List<DataSourceVO> dataSources = (List<DataSourceVO>) readWithService( (Object[]) null );
    Map<DataSource, DataSourceVO> dataSourceMap = new HashMap<DataSource, DataSourceVO>( dataSources.size() );
    for ( DataSourceVO dataSource: dataSources )
    {
      dataSourceMap.put( dataSource.getIdentifier(), dataSource );
    }
    return dataSourceMap;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#compareVOsAndValidateLoaded(ch.ige.dbutils.vo.AbstractVO, ch.ige.dbutils.vo.AbstractVO)
  @Override
  protected void compareVOsAndValidateLoaded( DataSourceVO originalVO, DataSourceVO loadedVO )
  {
    this.dataSourceDAOTest.compareVOsAndValidateLoaded( originalVO, loadedVO );
  }

  // @see ch.ige.dbutils.dao.test.BaseDAOWithCRUDTestTemplate#getExpectedVO()
  @Override
  protected DataSourceVO getExpectedVO() throws ParseException
  {
    DataSourceVO dataSource = new DataSourceVO();
    dataSource.setIdentifier( DataSource.NICE );
    dataSource.setTermValidityPeriod( null );
    dataSource.setTermConfirmationPeriod( null );
    dataSource.setName( "Nizza" );
    dataSource.setCleanerActive( false );
    dataSource.setChangeDate( DateConverter.stringToTimestamp( "09.08.2007 09:32:49.000" ) );
    return dataSource;
  }

  // @see ch.ige.wdltool.ejb.BaseServiceTestTemplate#getServices()
  @Override
  protected WdlServices getServices()
  {
    return this.wdlServices;
  }
}
