package ch.ige.wdltool.ejb.impl;

import java.sql.Date;
import java.util.ArrayList;
import java.util.List;

import ch.ige.ipobjects.ServicesFactory;
import ch.ige.ipobjects.key.vo.IPKeyVO;
import ch.ige.ipobjects.time.PointInTime;
import ch.ige.ipobjects.time.vo.EreigIDat;
import ch.ige.ipobjects.trademark.Trademark;
import ch.ige.ipobjects.trademark.vo.MarWarklVO;

/**
 * Utility methods for work with {@link Trademark}.
 *
 * <p>Copyright (c) 2008, Swiss Federal Institute of Intellectual Property</p>
 * @author   heinzer
 * @version  $Id: TrademarkImpl.java,v 1.2 2008/05/08 10:42:29 lewis_lee Exp $
 */
public class TrademarkImpl
{
  /**
   * Gets the current date as PointInTime.
   * @return The highest possible PointInTime for the current date
   */
  public static PointInTime getCurrentIDat()
  {
    return EreigIDat.max( new Date( System.currentTimeMillis() ) );
  }

  /**
   * Gets the trademark for a key, using the IPobjects services.
   * @param ipKey The key to get the trademark for
   * @return The trademark for the key
   */
  public static Trademark getTrademarkForKey( Integer ipKey )
  {
    return ServicesFactory.getInstance().findTrademark( IPKeyVO.create( ipKey ) );
  }

  /**
   * Gets the current WDL (goods and services) text of a trademark.
   * @param trademark The trademark containing the WDL text
   * @return The current WDL text of the given trademark
   */
  public static String getCurrentWdlText( Trademark trademark )
  {
    return getWdlText( trademark, getCurrentIDat() );
  }

  /**
   * Gets the WDL (goods and services) text of a trademark for a certain point in time.
   * @param trademark The trademark containing the WDL text
   * @param currentState The point in time to get the WDL text for
   * @return The WDL text of the given trademark for the given point in time
   */
  public static String getWdlText( Trademark trademark, PointInTime currentState )
  {
    return trademark.getGoodsAndServicesText().getItemContaining( currentState ).getValue();
  }

  /**
   * Gets the current WDL (goods and services) classes of a trademark.
   * @param trademark The trademark containing the WDL classes
   * @return The current WDL classes of the given trademark, which may be empty (but really shouldn't!)
   */
  public static List<Integer> getCurrentWdlClasses( Trademark trademark )
  {
    return getWdlClasses( trademark, getCurrentIDat() );
  }

  /**
   * Gets the current WDL (goods and services) classes of a trademark.
   * @param trademark The trademark containing the WDL classes
   * @param currentState The point in time to get the WDL classes for
   * @return The current WDL classes of the given trademark, which may be empty (but really shouldn't!)
   */
  public static List<Integer> getWdlClasses( Trademark trademark, PointInTime currentState )
  {
    List<MarWarklVO> classObjects = trademark.getGoodsAndServicesClasses().getPeriodsContainingTime( currentState );
    List<Integer> classNumbers = new ArrayList<Integer>();
    for ( MarWarklVO niceClass: classObjects )
    {
      classNumbers.add( niceClass.getWarkl_nr() );
    }
    return classNumbers;
  }
}