package ch.ige.wdltool.text;

/**
 * Replaces characters that are officially valid and may exist in a WDL, but are never found within terms.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: WdlCharacterReplacer.java,v 1.2 2007/12/10 10:43:34 lewis_lee Exp $
 */
public class WdlCharacterReplacer
{
  /**
   * The special characters to replace
   */
  private static final char[] SOURCE_CHARACTERS = new char[] { 'Œ', 'œ' };

  /**
   * The special characters are replace with these
   */
  private static final String[] TARGET = new String[] { "Oe", "oe" };

  /**
   * Replaces special characters in the WDL to characters that are also found
   * in terms. This is used for the {@link ch.ige.wdltool.checker.WdlChecker},
   * because otherwise a word within the WDL like <code>Straßenbahn</code> isn't
   * found by them term <code>Strassenbahn</code>.
   * @param wdlText The original WDL text
   * @return The same WDL text, but with the special characters replaced by normal characters,
   *         ready for WDL checking
   */
  public static String replaceSpecialCharacters( final String wdlText )
  {
    if ( SOURCE_CHARACTERS.length != TARGET.length )
    {
      throw new IllegalStateException( "Source and target characters don't match!" );
    }
    String escapedWdl = new String( wdlText );
    for ( int i = 0; i < SOURCE_CHARACTERS.length; i++ )
    {
      char sourceCharacter = SOURCE_CHARACTERS[i];
      String target = TARGET[i];
      if ( target.length() == 1 )
      {
        escapedWdl = escapedWdl.replace( sourceCharacter, target.charAt( 0 ) );
      }
      else
      {
        escapedWdl = escapedWdl.replace( String.valueOf( sourceCharacter ), target );
      }
    }
    return escapedWdl;
  }
}
