package ch.ige.wdltool.validator;

import java.util.Locale;

import org.hibernate.validator.Validator;

/**
 * Validates a language (Locale).
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: LanguageValidator.java,v 1.1 2007/08/08 12:52:59 lewis_lee Exp $
 */
public class LanguageValidator implements Validator<Language>
{
  /**
   * The allowed ISO codes for the language
   */
  private String[] allowedIsoCodes;

  // @see org.hibernate.validator.Validator#initialize(java.lang.annotation.Annotation)
  public void initialize( Language parameters )
  {
    this.allowedIsoCodes = parameters.isoCodes().split( ",\\s*" );
  }

  //part of the property constraint contract
  public boolean isValid( Object value )
  {
    if ( value == null )
    {
      return true;
    }
    if ( !( value instanceof Locale ) )
    {
      return false;
    }
    Locale language = (Locale) value;
    for ( String allowedIsoCode: this.allowedIsoCodes )
    {
      if ( language.getLanguage().equals( allowedIsoCode ) )
      {
        return true;
      }
    }
    return false;
  }
}
