package ch.ige.wdltool.vo;

import java.io.Serializable;
import java.util.Date;
import java.util.Locale;
import java.util.Set;

import ch.ige.wdltool.enums.DataSource;

/**
 * Contains an extended search with a lot of search criteria.
 * <p>
 * Copyright (c) 2007, Swiss Federal Institute of Intellectual Property
 * </p>
 * @author Christian Heinzer
 * @version $Id: ExtendedSearchVO.java,v 1.16 2008/12/08 15:57:14 heinzer Exp $
 */
public class ExtendedSearchVO extends SimpleSearchVO implements Serializable
{
  /**
   * Defines how to sort the search results.
   * Since {@link #language} also filters by language (as well as sorting by language), this
   * overrides the sorting language. So the search results can be sorted by language without only having
   * terms in that language.
   */
  public Locale sortLanguage;

  /**
   * The value to choose if a term must not contain a class number.
   */
  public static final Integer NO_CLASS = 0;

  /**
   * The classNumber to search for, if it's <code>null</code>, all terms are shown, regardless of class number; if it
   * contains {@link #NO_CLASS}, it filters on terms without class.
   */
  public Set<Integer> classNumbers;

  /**
   * This is only relevant if {@link #splitWords} is <code>true</code>: If this is <code>true</code>, all of the
   * split words coming from {@link #searchText} must be found (<code>AND</code>), otherwise only one word from the
   * split words has to be found (<code>OR</code>).
   */
  public boolean compositionAnd = false;

  /**
   * If this is <code>true</code>, then the {@link #searchText} is only searched in the terms, as is; if this is
   * <code>false</code>, the {@link #searchText} is also simplified and compared to the search term in the term
   * table.
   */
  public boolean exactSearch = false;

  /**
   * If this is <code>true</code>, then the {@link #searchText} is searched only as a whole word.
   * There are two possibilities for this:
   * <ul>
   *  <li>If {@link #splitWords} is <code>false</code>, the whole {@link #searchText} is searched as one text block.</li>
   *  <li>If {@link #splitWords} is <code>true</code>, the {@link #searchText} is split into words, and each of them is searched as one word.
   *   Of course, this case then also depends on {@link #compositionAnd}.
   *  </li>
   * </ul>
   */
  public boolean wholeWordOnly = false;

  /**
   * If this is <code>true</code>, the {@link #searchText} is split up into blank-separated words; otherwise it's
   * used as exact phrase.
   */
  public boolean splitWords = false;

  /**
   * This determines whether the search should be limited to the comment or to the term only. If this is
   * <code>null</code> both comment and term are searched. If this is <code>true</code> only comment is searched. If
   * this is <code>false</code> only term is searched.
   */
  public Boolean searchComment = null;

  /**
   * This determines whether the search should be limited to negative or positive terms. If this is <code>null</code>
   * both negative and positive terms are searched. If this is <code>true</code> only negative terms are searched. If
   * this is <code>false</code> only positive terms are searched.
   */
  public Boolean isNegative = null;

  /**
   * This determines whether the search should be limited to private or public terms. If this is <code>null</code>
   * both private and public terms are searched. If this is <code>true</code> only public terms are searched. If this
   * is <code>false</code> only private terms are searched.
   */
  public Boolean isPublic = null;

  /**
   * This determines whether the search should be limited to checker only or 'search and checker' terms. If this is <code>null</code>
   * both checker only and 'search and checker' terms are searched. If this is <code>true</code> only checker only terms are searched. If this
   * is <code>false</code> only 'search and checker' terms are searched.
   */
  public Boolean isCheckerOnly = false;

  /**
   * The data source to search for; if it's <code>null</code>, all data sources are searched.
   */
  public DataSource dataSource = null;

  /**
   * The term group number to search for; if it's <code>null</code>, it's ignored.
   */
  public Integer termGroupNumber = null;

  /**
   * For Nice terms, this determines whether the search should be limited to generic or non-generic terms. If this is
   * <code>null</code> both generic and non-generic terms are searched, which must be the case for all non-Nice terms.
   * If this is <code>true</code> only generic Nice terms are searched. If this is <code>false</code> only
   * non-generic Nice terms are searched.
   */
  public Boolean isGeneric = null;

  /**
   * If this is <code>null</code> the start of last changes is unlimited. Otherwise this limits the terms to all whose
   * last change is later than this or at the same date.
   */
  public Date changedFromDate = null;

  /**
   * If this is <code>null</code> the end of last changes is unlimited. Otherwise this limits the terms to all whose
   * last change is earlier than this or at the same date.
   */
  public Date changedToDate = null;

  /**
   * If this is <code>null</code>, the user that created a term is ignored. Otherwise it must match this.
   */
  public String createdUser = null;

  /**
   * If this is <code>null</code>, the user that changed a term is ignored. Otherwise it must match this.
   */
  public String changedUser = null;

  /**
   * If this is <code>null</code> the start of last confirmation is unlimited. Otherwise this limits the terms to all
   * whose last confirmation is later than this or at the same date.
   */
  public Date renewedFromDate = null;

  /**
   * If this is <code>null</code> the end of last confirmation is unlimited. Otherwise this limits the terms to all
   * whose last confirmation is earlier than this or at the same date.
   */
  public Date renewedToDate = null;
}