package ch.ige.wdltool.vo;

import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.hibernate.validator.NotNull;

import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.wdltool.enums.DataSource;
import ch.ige.wdltool.enums.TermType;
import ch.ige.wdltool.validator.SearchTerm;
import ch.ige.wdltool.validator.Term;

/**
 * Contains a Nice term. 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: NiceTermVO.java,v 1.7 2007/11/15 10:49:49 lewis_lee Exp $
 */
public class NiceTermVO extends AbstractTermVO implements IdentifiableVO<Integer>
{
  /**
   * The empty constructor
   */
  public NiceTermVO()
  {
    super();
    // Nice terms are always public
    this.isPublic = true;
  }

  /**
   * Creates this from an abstract VO.
   * @param vo The VO whose values are copied
   */
  public NiceTermVO( AbstractTermVO vo )
  {
    super( vo );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#initTermTypeAndDataSource()
  @Override
  protected void initTermTypeAndDataSource()
  {
    this.termType = TermType.NICE_TERM;
    this.dataSource = DataSource.NICE;
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getClassNumber()
  @Override
  @NotNull
  public Integer getClassNumber()
  {
    return super.getClassNumber();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setClassNumber(java.lang.Integer)
  @Override
  public void setClassNumber( Integer classNumber )
  {
    super.setClassNumber( classNumber );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTermGroupId()
  @Override
  public Integer getTermGroupId()
  {
    return super.getTermGroupId();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setTermGroupId(java.lang.Integer)
  @Override
  public void setTermGroupId( Integer termGroupId )
  {
    super.setTermGroupId( termGroupId );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTerm()
  @Override
  @Term
  public String getTerm()
  {
    return super.getTerm();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getSearchTerm()
  @Override
  @SearchTerm
  public String getSearchTerm()
  {
    return super.getSearchTerm();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#isGeneric()
  @Override
  public boolean isGeneric()
  {
    return super.isGeneric();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setGeneric(boolean)
  @Override
  public void setGeneric( boolean isGeneric )
  {
    super.setGeneric( isGeneric );
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#getTermGroupTerms()
  @Override
  public Map<Locale, List<AbstractTermVO>> getTermGroupTerms()
  {
    return super.getTermGroupTerms();
  }

  // @see ch.ige.wdltool.vo.AbstractTermVO#setTermGroupTerms(java.util.Map)
  @Override
  public void setTermGroupTerms( Map<Locale, List<AbstractTermVO>> termGroupTerms )
  {
    super.setTermGroupTerms( termGroupTerms );
  }

  /**
   * Gets the Nice term group's number.
   * <strong>Note: This is not always set, check the documentation of the method that creates this!</strong>
   * @return The Nice term group's number
   */
  public Integer getNiceTermGroupNumber()
  {
    return super.getTermGroupNumber();
  }

  /**
   * Sets the Nice term group's number.
   * @param niceTermGroupNumber the Nice term group's number
   */
  public void setNiceTermGroupNumber( Integer niceTermGroupNumber )
  {
    super.setTermGroupNumber( niceTermGroupNumber );
  }
}
