package ch.ige.wdltool.vo;

import java.util.Locale;

import org.hibernate.validator.Length;
import org.hibernate.validator.Min;
import org.hibernate.validator.NotEmpty;
import org.hibernate.validator.NotNull;

import ch.ige.dbutils.vo.AbstractVO;
import ch.ige.dbutils.vo.IdentifiableVO;
import ch.ige.wdltool.validator.Language;
import ch.ige.wdltool.validator.Term;

/**
 * Contains a stopword, that is a word that must not be part of a WDL.
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Eric Lewis
 * @version  $Id: StopwordVO.java,v 1.12 2008/02/14 16:49:14 lewis_lee Exp $
 */
public class StopwordVO extends AbstractVO implements IdentifiableVO<Integer>
{
  /**
   * The unique ID
   */
  private Integer stopwordId;

  /**
   * The language of the stopword
   */
  private Locale language;

  /**
   * The actual stopword
   */
  private String word;

  /**
   * The empty constructor.
   */
  public StopwordVO()
  {
    super();
  }

  /**
   * Constructs this, but without the identifier.
   * @param language The language
   * @param word The actual stopword
   */
  public StopwordVO( Locale language, String word )
  {
    this.language = language;
    this.word = word;
  }

  // @see ch.ige.dbutils.vo.IdentifiableVO#getIdentifier()
  @NotNull
  // See the Validator's checkIdentifierExists flag
  @Min( value = 1 )
  public Integer getIdentifier()
  {
    return this.stopwordId;
  }

  // @see ch.ige.dbutils.vo.IdentifiableVO#setIdentifier(java.io.Serializable)
  public void setIdentifier( Integer id )
  {
    this.stopwordId = id;
  }

  /**
   * Gets the language.
   * @return the language
   */
  @NotNull
  @Language( isoCodes = "de,fr,it" )
  public Locale getLanguage()
  {
    return this.language;
  }

  /**
   * Sets the language.
   * @param language The language
   */
  public void setLanguage( Locale language )
  {
    this.language = language;
  }

  /**
   * Gets the actual stopword.
   * @return the actual stopword
   */
  @NotEmpty
  @Length( min = 1, max = 25 )
  @Term
  // Must be treated like a term, since it's also used like a term in the Checker
  public String getWord()
  {
    return this.word;
  }

  /**
   * Sets the actual stopword.
   * @param word The actual stopword
   */
  public void setWord( String word )
  {
    this.word = word;
  }

  // @see java.lang.Object#clone()
  @Override
  public Object clone()
  {
    StopwordVO newStopword = new StopwordVO();
    newStopword.language = this.language;
    newStopword.stopwordId = this.stopwordId;
    newStopword.word = this.word;
    return newStopword;
  }
}
