package ch.ige.wdltool.vo;

import java.io.Serializable;

/**
 * Contains the information an update or insert operation on an active term produces.
 * That is the new term and a flag determining whether a candidate has been deleted because of the operation. 
 *
 * <p>Copyright (c) 2007, Swiss Federal Institute of Intellectual Property</p>
 * @author   Christian Heinzer
 * @version  $Id: TermCleanupCandidateStatusVO.java,v 1.3 2007/12/04 06:48:54 heinzer Exp $
 * @param <T> The Term VO Type - either <code>NiceTermVO</code> or <code>IpiTermVO</code>
 */
public class TermCleanupCandidateStatusVO<T extends AbstractTermVO> implements Serializable
{
  /**
   * The new term.
   */
  private T newTerm;

  /**
   * If this is <code>true</code>, then a candidate has been deleted because of the new/updated term
   * if this is <code>false</code>, then no corresponding candidate existed 
   */
  private boolean candidateRemoved;

  /**
   * Public constructor.
   * @param newTerm The new term
   * @param candidateRemoved The flag must be <code>true</code> when a candidate has been deleted because of the new/updated term and <code>false</code> otherwise
   */
  public TermCleanupCandidateStatusVO( T newTerm, boolean candidateRemoved )
  {
    this.newTerm = newTerm;
    this.candidateRemoved = candidateRemoved;
  }

  /**
   * Gets the new term.
   * @return the term
   */
  public T getNewTerm()
  {
    return this.newTerm;
  }

  /**
   * Gets the flag showing whether a candidate has been deleted because of the new/updated term
   * @return the flag 
   * If this is <code>true</code>, then a candidate has been deleted because of the new/updated term
   * if this is <code>false</code>, then no corresponding candidate existed 
   */
  public boolean isCandidateRemoved()
  {
    return this.candidateRemoved;
  }
}
